#!/usr/bin/env node
/**
 * CLI Interface for Aptos Function Indexer
 *
 * Usage: node cli.js <project_path> <function_name>
 *
 * Example: node cli.js ./test/caas-framework grant_read_authorization
 */
export {};
//# sourceMappingURL=cli.d.ts.map