/* Automatically generated by tree-sitter v0.25.3 (2a835ee029dca1c325e6f1c01dbce40396f6123e) */

#include "tree_sitter/parser.h"

#if defined(__GNUC__) || defined(__clang__)
#pragma GCC diagnostic ignored "-Wmissing-field-initializers"
#endif

#define LANGUAGE_VERSION 14
#define STATE_COUNT 2464
#define LARGE_STATE_COUNT 957
#define SYMBOL_COUNT 323
#define ALIAS_COUNT 6
#define TOKEN_COUNT 138
#define EXTERNAL_TOKEN_COUNT 4
#define FIELD_COUNT 72
#define MAX_ALIAS_SEQUENCE_LENGTH 11
#define MAX_RESERVED_WORD_SET_SIZE 0
#define PRODUCTION_ID_COUNT 224
#define SUPERTYPE_COUNT 0

enum ts_symbol_identifiers {
  sym_identifier = 1,
  anon_sym_u8 = 2,
  anon_sym_u16 = 3,
  anon_sym_u32 = 4,
  anon_sym_u64 = 5,
  anon_sym_u128 = 6,
  anon_sym_u256 = 7,
  anon_sym_exists = 8,
  anon_sym_forall = 9,
  anon_sym_choose = 10,
  anon_sym_min = 11,
  anon_sym_bool = 12,
  anon_sym_address = 13,
  anon_sym_vector = 14,
  anon_sym_for = 15,
  anon_sym_while = 16,
  anon_sym_friend = 17,
  anon_sym_match = 18,
  aux_sym_number_token1 = 19,
  aux_sym_number_token2 = 20,
  aux_sym_number_token3 = 21,
  aux_sym_number_token4 = 22,
  aux_sym_number_token5 = 23,
  aux_sym_number_token6 = 24,
  aux_sym_number_token7 = 25,
  anon_sym_true = 26,
  anon_sym_false = 27,
  aux_sym_byte_string_token1 = 28,
  aux_sym_byte_string_token2 = 29,
  anon_sym_STAR = 30,
  anon_sym_COLON_COLON = 31,
  anon_sym_LPAREN = 32,
  anon_sym_COMMA = 33,
  anon_sym_RPAREN = 34,
  anon_sym_PIPE = 35,
  anon_sym_AMP = 36,
  anon_sym_AMPmut = 37,
  anon_sym_EQ = 38,
  anon_sym_where = 39,
  anon_sym_COLON = 40,
  anon_sym_in = 41,
  anon_sym_LBRACE = 42,
  anon_sym_RBRACE = 43,
  anon_sym_EQ_EQ_GT = 44,
  anon_sym_LT_EQ_EQ_GT = 45,
  anon_sym_PIPE_PIPE = 46,
  anon_sym_AMP_AMP = 47,
  anon_sym_EQ_EQ = 48,
  anon_sym_BANG_EQ = 49,
  anon_sym_LT = 50,
  anon_sym_GT = 51,
  anon_sym_LT_EQ = 52,
  anon_sym_GT_EQ = 53,
  anon_sym_DOT_DOT = 54,
  anon_sym_CARET = 55,
  anon_sym_LT_LT = 56,
  anon_sym_GT_GT = 57,
  anon_sym_PLUS = 58,
  anon_sym_DASH = 59,
  anon_sym_SLASH = 60,
  anon_sym_PERCENT = 61,
  anon_sym_BANG = 62,
  anon_sym_move = 63,
  anon_sym_copy = 64,
  anon_sym_DOT = 65,
  anon_sym_LBRACK = 66,
  anon_sym_RBRACK = 67,
  sym_anon_field_index = 68,
  anon_sym_break = 69,
  anon_sym_continue = 70,
  anon_sym_as = 71,
  anon_sym_if = 72,
  anon_sym_EQ_GT = 73,
  anon_sym_else = 74,
  anon_sym_loop = 75,
  anon_sym_return = 76,
  anon_sym_abort = 77,
  anon_sym_LT2 = 78,
  anon_sym_AT = 79,
  anon_sym_POUND = 80,
  anon_sym_module = 81,
  anon_sym_spec = 82,
  anon_sym_schema = 83,
  anon_sym_invariant = 84,
  anon_sym_update = 85,
  anon_sym_SEMI = 86,
  anon_sym_assert = 87,
  anon_sym_assume = 88,
  anon_sym_ensures = 89,
  anon_sym_requires = 90,
  anon_sym_aborts_if = 91,
  anon_sym_with = 92,
  anon_sym_aborts_with = 93,
  anon_sym_modifies = 94,
  anon_sym_emits = 95,
  anon_sym_to = 96,
  anon_sym_global = 97,
  anon_sym_local = 98,
  anon_sym_fun = 99,
  anon_sym_native = 100,
  anon_sym_include = 101,
  anon_sym_apply = 102,
  anon_sym_except = 103,
  anon_sym_public = 104,
  anon_sym_internal = 105,
  anon_sym_pragma = 106,
  anon_sym_let = 107,
  anon_sym_post = 108,
  anon_sym_axiom = 109,
  anon_sym_script = 110,
  anon_sym_package = 111,
  anon_sym_entry = 112,
  anon_sym_use = 113,
  anon_sym_const = 114,
  anon_sym_inline = 115,
  anon_sym_pure = 116,
  anon_sym_acquires = 117,
  anon_sym_reads = 118,
  anon_sym_writes = 119,
  anon_sym_struct = 120,
  sym_phantom = 121,
  anon_sym_enum = 122,
  anon_sym_drop = 123,
  anon_sym_store = 124,
  anon_sym_key = 125,
  anon_sym_has = 126,
  anon_sym_SLASH_SLASH = 127,
  anon_sym_SLASH_SLASH2 = 128,
  aux_sym_line_comment_token1 = 129,
  anon_sym_SLASH2 = 130,
  aux_sym_line_comment_token2 = 131,
  anon_sym_SLASH_STAR = 132,
  anon_sym_STAR_SLASH = 133,
  sym__block_doc_comment_marker = 134,
  sym__block_comment_content = 135,
  sym__doc_line_comment = 136,
  sym__error_sentinel = 137,
  sym_source_file = 138,
  sym_number_type = 139,
  sym__quantifier_directive = 140,
  sym_primitive_type = 141,
  sym_var_name = 142,
  sym_discouraged_name = 143,
  sym__reuseable_keywords = 144,
  sym_number = 145,
  sym_numerical_addr = 146,
  sym_bool_literal = 147,
  sym_typed_number = 148,
  sym_byte_string = 149,
  sym__leading_name_access = 150,
  sym__leading_name_access_wildcard = 151,
  sym_name_access_chain = 152,
  sym_name_access_chain_wildcard = 153,
  sym_type = 154,
  sym__type = 155,
  sym_tuple_type = 156,
  sym_closure_type = 157,
  sym__ref_type = 158,
  sym__expr = 159,
  sym_assignment = 160,
  sym_lambda_bind_list = 161,
  sym_quantifier = 162,
  sym_quantifier_bind = 163,
  sym_triggers = 164,
  sym__op_expr = 165,
  sym_bin_op_expr = 166,
  sym__unary_expr = 167,
  sym_not_expr = 168,
  sym_ref_expr = 169,
  sym_ref_mut_expr = 170,
  sym_deref_expr = 171,
  sym_move_expr = 172,
  sym_copy_expr = 173,
  sym_receiver_call = 174,
  sym_mem_access = 175,
  sym_access_field = 176,
  sym__identifier_or_anon_field = 177,
  sym_term = 178,
  sym_vector_value_expr = 179,
  sym_tuple_expr = 180,
  sym_type_hint_expr = 181,
  sym_cast_expr = 182,
  sym_parenthesized_expr = 183,
  sym_match_expr = 184,
  sym_match_arm = 185,
  sym_if_expr = 186,
  sym_while_expr = 187,
  sym_loop_expr = 188,
  sym_return_expr = 189,
  sym_abort_expr = 190,
  sym_for_loop_expr = 191,
  sym__control_body = 192,
  sym_var = 193,
  sym_call_expr = 194,
  sym_pack_expr = 195,
  sym_macro_call_expr = 196,
  sym_call_args = 197,
  sym_type_args = 198,
  sym_expr_field = 199,
  sym_value = 200,
  sym_attributes = 201,
  sym_attribute = 202,
  sym__attribute_name = 203,
  sym__attribute_val = 204,
  sym_address_block = 205,
  sym__address_member = 206,
  sym__module_keyword = 207,
  sym__module_path = 208,
  sym_module = 209,
  sym_declaration = 210,
  sym_spec_block = 211,
  sym_spec_block_target = 212,
  sym__spec_target_signature_opt = 213,
  sym_spec_loop_invariant = 214,
  sym__spec_block_member = 215,
  sym_spec_invariant = 216,
  sym_spec_condition = 217,
  sym_asserts = 218,
  sym_aborts_if = 219,
  sym_aborts_with_or_modifies = 220,
  sym_emits = 221,
  sym_condition_props = 222,
  sym_spec_variable = 223,
  sym_spec_func = 224,
  sym__spec_func_signatures = 225,
  sym_spec_include = 226,
  sym_spec_apply = 227,
  sym__spec_apply_pattern = 228,
  sym__spec_apply_fragment = 229,
  sym_spec_pragma = 230,
  sym__spec_pragma_prop = 231,
  sym_spec_let = 232,
  sym_spec_update = 233,
  sym_spec_axiom = 234,
  sym_visibility = 235,
  sym_module_member_modifier = 236,
  sym_module_ident = 237,
  sym_use_decl = 238,
  sym__use_alias = 239,
  sym__use_member = 240,
  sym_friend_decl = 241,
  sym_constant_decl = 242,
  sym_function_decl = 243,
  sym__function_signature = 244,
  sym__specifier = 245,
  sym_block = 246,
  sym_parameters = 247,
  sym_type_param = 248,
  sym_type_params = 249,
  sym_constraints = 250,
  sym_parameter = 251,
  sym_access_specifier_list = 252,
  sym_access_specifier = 253,
  sym__address_specifier = 254,
  sym__struct_or_enum_decl = 255,
  sym_struct_decl = 256,
  sym__struct_signature = 257,
  sym_struct_body = 258,
  sym_anon_fields = 259,
  sym__struct_def_name = 260,
  sym__struct_type_params = 261,
  sym__struct_type_parameter = 262,
  sym_field_annot = 263,
  sym_enum_decl = 264,
  sym__enum_signature = 265,
  sym_enum_body = 266,
  sym__variant = 267,
  sym__variant_last = 268,
  sym_enum_variant_struct = 269,
  sym_enum_variant_posit = 270,
  sym_enum_variant = 271,
  sym_ability = 272,
  sym_abilities = 273,
  sym__sequence_item = 274,
  sym_let_expr = 275,
  sym_bind_list = 276,
  sym__bind = 277,
  sym__bind_tuple = 278,
  sym__bind_fields = 279,
  sym_bind_field = 280,
  sym_script = 281,
  sym__script_use_decl = 282,
  sym__script_constant_decl = 283,
  sym__script_func_decl = 284,
  sym__script_spec_block = 285,
  sym_line_comment = 286,
  sym_block_comment = 287,
  aux_sym_source_file_repeat1 = 288,
  aux_sym_tuple_type_repeat1 = 289,
  aux_sym_lambda_bind_list_repeat1 = 290,
  aux_sym_quantifier_repeat1 = 291,
  aux_sym_triggers_repeat1 = 292,
  aux_sym_triggers_repeat2 = 293,
  aux_sym_vector_value_expr_repeat1 = 294,
  aux_sym_match_expr_repeat1 = 295,
  aux_sym_pack_expr_repeat1 = 296,
  aux_sym_attributes_repeat1 = 297,
  aux_sym_attributes_repeat2 = 298,
  aux_sym__attribute_name_repeat1 = 299,
  aux_sym_address_block_repeat1 = 300,
  aux_sym_module_repeat1 = 301,
  aux_sym_declaration_repeat1 = 302,
  aux_sym_spec_block_repeat1 = 303,
  aux_sym_spec_block_repeat2 = 304,
  aux_sym_condition_props_repeat1 = 305,
  aux_sym_spec_apply_repeat1 = 306,
  aux_sym__spec_apply_pattern_repeat1 = 307,
  aux_sym_use_decl_repeat1 = 308,
  aux_sym__specifier_repeat1 = 309,
  aux_sym_block_repeat1 = 310,
  aux_sym_parameters_repeat1 = 311,
  aux_sym_type_params_repeat1 = 312,
  aux_sym_constraints_repeat1 = 313,
  aux_sym_access_specifier_list_repeat1 = 314,
  aux_sym_struct_body_repeat1 = 315,
  aux_sym__struct_type_params_repeat1 = 316,
  aux_sym_enum_body_repeat1 = 317,
  aux_sym_abilities_repeat1 = 318,
  aux_sym__bind_fields_repeat1 = 319,
  aux_sym_script_repeat1 = 320,
  aux_sym_script_repeat2 = 321,
  aux_sym_script_repeat3 = 322,
  alias_sym_alias = 323,
  alias_sym_body = 324,
  alias_sym_condition = 325,
  alias_sym_pattern = 326,
  alias_sym_result = 327,
  alias_sym_shorthand_field_identifier = 328,
};

static const char * const ts_symbol_names[] = {
  [ts_builtin_sym_end] = "end",
  [sym_identifier] = "identifier",
  [anon_sym_u8] = "u8",
  [anon_sym_u16] = "u16",
  [anon_sym_u32] = "u32",
  [anon_sym_u64] = "u64",
  [anon_sym_u128] = "u128",
  [anon_sym_u256] = "u256",
  [anon_sym_exists] = "exists",
  [anon_sym_forall] = "forall",
  [anon_sym_choose] = "choose",
  [anon_sym_min] = "min",
  [anon_sym_bool] = "bool",
  [anon_sym_address] = "address",
  [anon_sym_vector] = "vector",
  [anon_sym_for] = "for",
  [anon_sym_while] = "while",
  [anon_sym_friend] = "friend",
  [anon_sym_match] = "match",
  [aux_sym_number_token1] = "number_token1",
  [aux_sym_number_token2] = "number_token2",
  [aux_sym_number_token3] = "number_token3",
  [aux_sym_number_token4] = "number_token4",
  [aux_sym_number_token5] = "number_token5",
  [aux_sym_number_token6] = "number_token6",
  [aux_sym_number_token7] = "number_token7",
  [anon_sym_true] = "true",
  [anon_sym_false] = "false",
  [aux_sym_byte_string_token1] = "byte_string_token1",
  [aux_sym_byte_string_token2] = "byte_string_token2",
  [anon_sym_STAR] = "*",
  [anon_sym_COLON_COLON] = "::",
  [anon_sym_LPAREN] = "(",
  [anon_sym_COMMA] = ",",
  [anon_sym_RPAREN] = ")",
  [anon_sym_PIPE] = "|",
  [anon_sym_AMP] = "&",
  [anon_sym_AMPmut] = "&mut",
  [anon_sym_EQ] = "=",
  [anon_sym_where] = "where",
  [anon_sym_COLON] = ":",
  [anon_sym_in] = "in",
  [anon_sym_LBRACE] = "{",
  [anon_sym_RBRACE] = "}",
  [anon_sym_EQ_EQ_GT] = "binary_operator",
  [anon_sym_LT_EQ_EQ_GT] = "binary_operator",
  [anon_sym_PIPE_PIPE] = "binary_operator",
  [anon_sym_AMP_AMP] = "binary_operator",
  [anon_sym_EQ_EQ] = "binary_operator",
  [anon_sym_BANG_EQ] = "binary_operator",
  [anon_sym_LT] = "<",
  [anon_sym_GT] = ">",
  [anon_sym_LT_EQ] = "binary_operator",
  [anon_sym_GT_EQ] = "binary_operator",
  [anon_sym_DOT_DOT] = "..",
  [anon_sym_CARET] = "binary_operator",
  [anon_sym_LT_LT] = "binary_operator",
  [anon_sym_GT_GT] = "binary_operator",
  [anon_sym_PLUS] = "+",
  [anon_sym_DASH] = "binary_operator",
  [anon_sym_SLASH] = "binary_operator",
  [anon_sym_PERCENT] = "binary_operator",
  [anon_sym_BANG] = "!",
  [anon_sym_move] = "move",
  [anon_sym_copy] = "copy",
  [anon_sym_DOT] = ".",
  [anon_sym_LBRACK] = "[",
  [anon_sym_RBRACK] = "]",
  [sym_anon_field_index] = "anon_field_index",
  [anon_sym_break] = "break_expr",
  [anon_sym_continue] = "continue_expr",
  [anon_sym_as] = "as",
  [anon_sym_if] = "if",
  [anon_sym_EQ_GT] = "=>",
  [anon_sym_else] = "else",
  [anon_sym_loop] = "loop",
  [anon_sym_return] = "return",
  [anon_sym_abort] = "abort",
  [anon_sym_LT2] = "<",
  [anon_sym_AT] = "@",
  [anon_sym_POUND] = "#",
  [anon_sym_module] = "module",
  [anon_sym_spec] = "spec",
  [anon_sym_schema] = "schema",
  [anon_sym_invariant] = "invariant",
  [anon_sym_update] = "update",
  [anon_sym_SEMI] = ";",
  [anon_sym_assert] = "assert",
  [anon_sym_assume] = "assume",
  [anon_sym_ensures] = "ensures",
  [anon_sym_requires] = "requires",
  [anon_sym_aborts_if] = "aborts_if",
  [anon_sym_with] = "with",
  [anon_sym_aborts_with] = "aborts_with",
  [anon_sym_modifies] = "modifies",
  [anon_sym_emits] = "emits",
  [anon_sym_to] = "to",
  [anon_sym_global] = "global",
  [anon_sym_local] = "local",
  [anon_sym_fun] = "fun",
  [anon_sym_native] = "native",
  [anon_sym_include] = "include",
  [anon_sym_apply] = "apply",
  [anon_sym_except] = "except",
  [anon_sym_public] = "public",
  [anon_sym_internal] = "internal",
  [anon_sym_pragma] = "pragma",
  [anon_sym_let] = "let",
  [anon_sym_post] = "post",
  [anon_sym_axiom] = "axiom",
  [anon_sym_script] = "script",
  [anon_sym_package] = "package",
  [anon_sym_entry] = "entry",
  [anon_sym_use] = "use",
  [anon_sym_const] = "const",
  [anon_sym_inline] = "inline",
  [anon_sym_pure] = "pure",
  [anon_sym_acquires] = "acquires",
  [anon_sym_reads] = "reads",
  [anon_sym_writes] = "writes",
  [anon_sym_struct] = "struct",
  [sym_phantom] = "phantom",
  [anon_sym_enum] = "enum",
  [anon_sym_drop] = "drop",
  [anon_sym_store] = "store",
  [anon_sym_key] = "key",
  [anon_sym_has] = "has",
  [anon_sym_SLASH_SLASH] = "//",
  [anon_sym_SLASH_SLASH2] = "//",
  [aux_sym_line_comment_token1] = "line_comment_token1",
  [anon_sym_SLASH2] = "/",
  [aux_sym_line_comment_token2] = "line_comment_token2",
  [anon_sym_SLASH_STAR] = "/*",
  [anon_sym_STAR_SLASH] = "*/",
  [sym__block_doc_comment_marker] = "_block_doc_comment_marker",
  [sym__block_comment_content] = "_block_comment_content",
  [sym__doc_line_comment] = "doc_comment",
  [sym__error_sentinel] = "_error_sentinel",
  [sym_source_file] = "source_file",
  [sym_number_type] = "number_type",
  [sym__quantifier_directive] = "_quantifier_directive",
  [sym_primitive_type] = "primitive_type",
  [sym_var_name] = "var_name",
  [sym_discouraged_name] = "discouraged_name",
  [sym__reuseable_keywords] = "_reuseable_keywords",
  [sym_number] = "number",
  [sym_numerical_addr] = "numerical_addr",
  [sym_bool_literal] = "bool_literal",
  [sym_typed_number] = "typed_number",
  [sym_byte_string] = "byte_string",
  [sym__leading_name_access] = "_leading_name_access",
  [sym__leading_name_access_wildcard] = "_leading_name_access_wildcard",
  [sym_name_access_chain] = "name_access_chain",
  [sym_name_access_chain_wildcard] = "name_access_chain_wildcard",
  [sym_type] = "type",
  [sym__type] = "_type",
  [sym_tuple_type] = "tuple_type",
  [sym_closure_type] = "closure_type",
  [sym__ref_type] = "_ref_type",
  [sym__expr] = "_expr",
  [sym_assignment] = "assignment",
  [sym_lambda_bind_list] = "lambda",
  [sym_quantifier] = "quantifier",
  [sym_quantifier_bind] = "quantifier_bind",
  [sym_triggers] = "triggers",
  [sym__op_expr] = "_op_expr",
  [sym_bin_op_expr] = "bin_op_expr",
  [sym__unary_expr] = "_unary_expr",
  [sym_not_expr] = "not_expr",
  [sym_ref_expr] = "ref_expr",
  [sym_ref_mut_expr] = "ref_mut_expr",
  [sym_deref_expr] = "deref_expr",
  [sym_move_expr] = "move_expr",
  [sym_copy_expr] = "copy_expr",
  [sym_receiver_call] = "receiver_call",
  [sym_mem_access] = "mem_access",
  [sym_access_field] = "access_field",
  [sym__identifier_or_anon_field] = "_identifier_or_anon_field",
  [sym_term] = "expr_term",
  [sym_vector_value_expr] = "vector_value_expr",
  [sym_tuple_expr] = "tuple_expr",
  [sym_type_hint_expr] = "type_hint_expr",
  [sym_cast_expr] = "cast_expr",
  [sym_parenthesized_expr] = "parenthesized_expr",
  [sym_match_expr] = "match_expr",
  [sym_match_arm] = "match_arm",
  [sym_if_expr] = "if_expr",
  [sym_while_expr] = "while_expr",
  [sym_loop_expr] = "loop_expr",
  [sym_return_expr] = "return_expr",
  [sym_abort_expr] = "abort_expr",
  [sym_for_loop_expr] = "for_loop_expr",
  [sym__control_body] = "_control_body",
  [sym_var] = "var",
  [sym_call_expr] = "call_expr",
  [sym_pack_expr] = "pack_expr",
  [sym_macro_call_expr] = "macro_call_expr",
  [sym_call_args] = "call_args",
  [sym_type_args] = "type_args",
  [sym_expr_field] = "expr_field",
  [sym_value] = "value",
  [sym_attributes] = "attributes",
  [sym_attribute] = "attribute",
  [sym__attribute_name] = "_attribute_name",
  [sym__attribute_val] = "_attribute_val",
  [sym_address_block] = "address_block",
  [sym__address_member] = "_address_member",
  [sym__module_keyword] = "_module_keyword",
  [sym__module_path] = "_module_path",
  [sym_module] = "module",
  [sym_declaration] = "declaration",
  [sym_spec_block] = "spec_block",
  [sym_spec_block_target] = "spec_block_target",
  [sym__spec_target_signature_opt] = "_spec_target_signature_opt",
  [sym_spec_loop_invariant] = "spec_loop_invariant",
  [sym__spec_block_member] = "_spec_block_member",
  [sym_spec_invariant] = "spec_invariant",
  [sym_spec_condition] = "spec_condition",
  [sym_asserts] = "asserts",
  [sym_aborts_if] = "aborts_if",
  [sym_aborts_with_or_modifies] = "aborts_with_or_modifies",
  [sym_emits] = "emits",
  [sym_condition_props] = "condition_props",
  [sym_spec_variable] = "spec_variable",
  [sym_spec_func] = "spec_func",
  [sym__spec_func_signatures] = "_spec_func_signatures",
  [sym_spec_include] = "spec_include",
  [sym_spec_apply] = "spec_apply",
  [sym__spec_apply_pattern] = "_spec_apply_pattern",
  [sym__spec_apply_fragment] = "_spec_apply_fragment",
  [sym_spec_pragma] = "spec_pragma",
  [sym__spec_pragma_prop] = "_spec_pragma_prop",
  [sym_spec_let] = "spec_let",
  [sym_spec_update] = "spec_update",
  [sym_spec_axiom] = "spec_axiom",
  [sym_visibility] = "visibility",
  [sym_module_member_modifier] = "module_member_modifier",
  [sym_module_ident] = "module_ident",
  [sym_use_decl] = "use_decl",
  [sym__use_alias] = "_use_alias",
  [sym__use_member] = "member",
  [sym_friend_decl] = "friend_decl",
  [sym_constant_decl] = "constant_decl",
  [sym_function_decl] = "function_decl",
  [sym__function_signature] = "_function_signature",
  [sym__specifier] = "_specifier",
  [sym_block] = "block",
  [sym_parameters] = "parameters",
  [sym_type_param] = "type_param",
  [sym_type_params] = "type_params",
  [sym_constraints] = "constraints",
  [sym_parameter] = "parameter",
  [sym_access_specifier_list] = "access_specifier_list",
  [sym_access_specifier] = "access_specifier",
  [sym__address_specifier] = "_address_specifier",
  [sym__struct_or_enum_decl] = "_struct_or_enum_decl",
  [sym_struct_decl] = "struct_decl",
  [sym__struct_signature] = "_struct_signature",
  [sym_struct_body] = "struct_body",
  [sym_anon_fields] = "anon_fields",
  [sym__struct_def_name] = "_struct_def_name",
  [sym__struct_type_params] = "type_params",
  [sym__struct_type_parameter] = "type_param",
  [sym_field_annot] = "field_annot",
  [sym_enum_decl] = "enum_decl",
  [sym__enum_signature] = "_enum_signature",
  [sym_enum_body] = "enum_body",
  [sym__variant] = "_variant",
  [sym__variant_last] = "_variant_last",
  [sym_enum_variant_struct] = "enum_variant_struct",
  [sym_enum_variant_posit] = "enum_variant_posit",
  [sym_enum_variant] = "enum_variant",
  [sym_ability] = "ability",
  [sym_abilities] = "abilities",
  [sym__sequence_item] = "_sequence_item",
  [sym_let_expr] = "let_expr",
  [sym_bind_list] = "bind_list",
  [sym__bind] = "_bind",
  [sym__bind_tuple] = "tuple",
  [sym__bind_fields] = "fields",
  [sym_bind_field] = "bind_field",
  [sym_script] = "script",
  [sym__script_use_decl] = "declaration",
  [sym__script_constant_decl] = "declaration",
  [sym__script_func_decl] = "declaration",
  [sym__script_spec_block] = "declaration",
  [sym_line_comment] = "line_comment",
  [sym_block_comment] = "block_comment",
  [aux_sym_source_file_repeat1] = "source_file_repeat1",
  [aux_sym_tuple_type_repeat1] = "tuple_type_repeat1",
  [aux_sym_lambda_bind_list_repeat1] = "lambda_bind_list_repeat1",
  [aux_sym_quantifier_repeat1] = "quantifier_repeat1",
  [aux_sym_triggers_repeat1] = "triggers_repeat1",
  [aux_sym_triggers_repeat2] = "triggers_repeat2",
  [aux_sym_vector_value_expr_repeat1] = "vector_value_expr_repeat1",
  [aux_sym_match_expr_repeat1] = "match_expr_repeat1",
  [aux_sym_pack_expr_repeat1] = "pack_expr_repeat1",
  [aux_sym_attributes_repeat1] = "attributes_repeat1",
  [aux_sym_attributes_repeat2] = "attributes_repeat2",
  [aux_sym__attribute_name_repeat1] = "_attribute_name_repeat1",
  [aux_sym_address_block_repeat1] = "address_block_repeat1",
  [aux_sym_module_repeat1] = "module_repeat1",
  [aux_sym_declaration_repeat1] = "declaration_repeat1",
  [aux_sym_spec_block_repeat1] = "spec_block_repeat1",
  [aux_sym_spec_block_repeat2] = "spec_block_repeat2",
  [aux_sym_condition_props_repeat1] = "condition_props_repeat1",
  [aux_sym_spec_apply_repeat1] = "spec_apply_repeat1",
  [aux_sym__spec_apply_pattern_repeat1] = "_spec_apply_pattern_repeat1",
  [aux_sym_use_decl_repeat1] = "use_decl_repeat1",
  [aux_sym__specifier_repeat1] = "_specifier_repeat1",
  [aux_sym_block_repeat1] = "block_repeat1",
  [aux_sym_parameters_repeat1] = "parameters_repeat1",
  [aux_sym_type_params_repeat1] = "type_params_repeat1",
  [aux_sym_constraints_repeat1] = "constraints_repeat1",
  [aux_sym_access_specifier_list_repeat1] = "access_specifier_list_repeat1",
  [aux_sym_struct_body_repeat1] = "struct_body_repeat1",
  [aux_sym__struct_type_params_repeat1] = "_struct_type_params_repeat1",
  [aux_sym_enum_body_repeat1] = "enum_body_repeat1",
  [aux_sym_abilities_repeat1] = "abilities_repeat1",
  [aux_sym__bind_fields_repeat1] = "_bind_fields_repeat1",
  [aux_sym_script_repeat1] = "script_repeat1",
  [aux_sym_script_repeat2] = "script_repeat2",
  [aux_sym_script_repeat3] = "script_repeat3",
  [alias_sym_alias] = "alias",
  [alias_sym_body] = "body",
  [alias_sym_condition] = "condition",
  [alias_sym_pattern] = "pattern",
  [alias_sym_result] = "result",
  [alias_sym_shorthand_field_identifier] = "shorthand_field_identifier",
};

static const TSSymbol ts_symbol_map[] = {
  [ts_builtin_sym_end] = ts_builtin_sym_end,
  [sym_identifier] = sym_identifier,
  [anon_sym_u8] = anon_sym_u8,
  [anon_sym_u16] = anon_sym_u16,
  [anon_sym_u32] = anon_sym_u32,
  [anon_sym_u64] = anon_sym_u64,
  [anon_sym_u128] = anon_sym_u128,
  [anon_sym_u256] = anon_sym_u256,
  [anon_sym_exists] = anon_sym_exists,
  [anon_sym_forall] = anon_sym_forall,
  [anon_sym_choose] = anon_sym_choose,
  [anon_sym_min] = anon_sym_min,
  [anon_sym_bool] = anon_sym_bool,
  [anon_sym_address] = anon_sym_address,
  [anon_sym_vector] = anon_sym_vector,
  [anon_sym_for] = anon_sym_for,
  [anon_sym_while] = anon_sym_while,
  [anon_sym_friend] = anon_sym_friend,
  [anon_sym_match] = anon_sym_match,
  [aux_sym_number_token1] = aux_sym_number_token1,
  [aux_sym_number_token2] = aux_sym_number_token2,
  [aux_sym_number_token3] = aux_sym_number_token3,
  [aux_sym_number_token4] = aux_sym_number_token4,
  [aux_sym_number_token5] = aux_sym_number_token5,
  [aux_sym_number_token6] = aux_sym_number_token6,
  [aux_sym_number_token7] = aux_sym_number_token7,
  [anon_sym_true] = anon_sym_true,
  [anon_sym_false] = anon_sym_false,
  [aux_sym_byte_string_token1] = aux_sym_byte_string_token1,
  [aux_sym_byte_string_token2] = aux_sym_byte_string_token2,
  [anon_sym_STAR] = anon_sym_STAR,
  [anon_sym_COLON_COLON] = anon_sym_COLON_COLON,
  [anon_sym_LPAREN] = anon_sym_LPAREN,
  [anon_sym_COMMA] = anon_sym_COMMA,
  [anon_sym_RPAREN] = anon_sym_RPAREN,
  [anon_sym_PIPE] = anon_sym_PIPE,
  [anon_sym_AMP] = anon_sym_AMP,
  [anon_sym_AMPmut] = anon_sym_AMPmut,
  [anon_sym_EQ] = anon_sym_EQ,
  [anon_sym_where] = anon_sym_where,
  [anon_sym_COLON] = anon_sym_COLON,
  [anon_sym_in] = anon_sym_in,
  [anon_sym_LBRACE] = anon_sym_LBRACE,
  [anon_sym_RBRACE] = anon_sym_RBRACE,
  [anon_sym_EQ_EQ_GT] = anon_sym_EQ_EQ_GT,
  [anon_sym_LT_EQ_EQ_GT] = anon_sym_EQ_EQ_GT,
  [anon_sym_PIPE_PIPE] = anon_sym_EQ_EQ_GT,
  [anon_sym_AMP_AMP] = anon_sym_EQ_EQ_GT,
  [anon_sym_EQ_EQ] = anon_sym_EQ_EQ_GT,
  [anon_sym_BANG_EQ] = anon_sym_EQ_EQ_GT,
  [anon_sym_LT] = anon_sym_LT,
  [anon_sym_GT] = anon_sym_GT,
  [anon_sym_LT_EQ] = anon_sym_EQ_EQ_GT,
  [anon_sym_GT_EQ] = anon_sym_EQ_EQ_GT,
  [anon_sym_DOT_DOT] = anon_sym_DOT_DOT,
  [anon_sym_CARET] = anon_sym_EQ_EQ_GT,
  [anon_sym_LT_LT] = anon_sym_EQ_EQ_GT,
  [anon_sym_GT_GT] = anon_sym_EQ_EQ_GT,
  [anon_sym_PLUS] = anon_sym_PLUS,
  [anon_sym_DASH] = anon_sym_EQ_EQ_GT,
  [anon_sym_SLASH] = anon_sym_EQ_EQ_GT,
  [anon_sym_PERCENT] = anon_sym_EQ_EQ_GT,
  [anon_sym_BANG] = anon_sym_BANG,
  [anon_sym_move] = anon_sym_move,
  [anon_sym_copy] = anon_sym_copy,
  [anon_sym_DOT] = anon_sym_DOT,
  [anon_sym_LBRACK] = anon_sym_LBRACK,
  [anon_sym_RBRACK] = anon_sym_RBRACK,
  [sym_anon_field_index] = sym_anon_field_index,
  [anon_sym_break] = anon_sym_break,
  [anon_sym_continue] = anon_sym_continue,
  [anon_sym_as] = anon_sym_as,
  [anon_sym_if] = anon_sym_if,
  [anon_sym_EQ_GT] = anon_sym_EQ_GT,
  [anon_sym_else] = anon_sym_else,
  [anon_sym_loop] = anon_sym_loop,
  [anon_sym_return] = anon_sym_return,
  [anon_sym_abort] = anon_sym_abort,
  [anon_sym_LT2] = anon_sym_LT,
  [anon_sym_AT] = anon_sym_AT,
  [anon_sym_POUND] = anon_sym_POUND,
  [anon_sym_module] = anon_sym_module,
  [anon_sym_spec] = anon_sym_spec,
  [anon_sym_schema] = anon_sym_schema,
  [anon_sym_invariant] = anon_sym_invariant,
  [anon_sym_update] = anon_sym_update,
  [anon_sym_SEMI] = anon_sym_SEMI,
  [anon_sym_assert] = anon_sym_assert,
  [anon_sym_assume] = anon_sym_assume,
  [anon_sym_ensures] = anon_sym_ensures,
  [anon_sym_requires] = anon_sym_requires,
  [anon_sym_aborts_if] = anon_sym_aborts_if,
  [anon_sym_with] = anon_sym_with,
  [anon_sym_aborts_with] = anon_sym_aborts_with,
  [anon_sym_modifies] = anon_sym_modifies,
  [anon_sym_emits] = anon_sym_emits,
  [anon_sym_to] = anon_sym_to,
  [anon_sym_global] = anon_sym_global,
  [anon_sym_local] = anon_sym_local,
  [anon_sym_fun] = anon_sym_fun,
  [anon_sym_native] = anon_sym_native,
  [anon_sym_include] = anon_sym_include,
  [anon_sym_apply] = anon_sym_apply,
  [anon_sym_except] = anon_sym_except,
  [anon_sym_public] = anon_sym_public,
  [anon_sym_internal] = anon_sym_internal,
  [anon_sym_pragma] = anon_sym_pragma,
  [anon_sym_let] = anon_sym_let,
  [anon_sym_post] = anon_sym_post,
  [anon_sym_axiom] = anon_sym_axiom,
  [anon_sym_script] = anon_sym_script,
  [anon_sym_package] = anon_sym_package,
  [anon_sym_entry] = anon_sym_entry,
  [anon_sym_use] = anon_sym_use,
  [anon_sym_const] = anon_sym_const,
  [anon_sym_inline] = anon_sym_inline,
  [anon_sym_pure] = anon_sym_pure,
  [anon_sym_acquires] = anon_sym_acquires,
  [anon_sym_reads] = anon_sym_reads,
  [anon_sym_writes] = anon_sym_writes,
  [anon_sym_struct] = anon_sym_struct,
  [sym_phantom] = sym_phantom,
  [anon_sym_enum] = anon_sym_enum,
  [anon_sym_drop] = anon_sym_drop,
  [anon_sym_store] = anon_sym_store,
  [anon_sym_key] = anon_sym_key,
  [anon_sym_has] = anon_sym_has,
  [anon_sym_SLASH_SLASH] = anon_sym_SLASH_SLASH,
  [anon_sym_SLASH_SLASH2] = anon_sym_SLASH_SLASH,
  [aux_sym_line_comment_token1] = aux_sym_line_comment_token1,
  [anon_sym_SLASH2] = anon_sym_SLASH,
  [aux_sym_line_comment_token2] = aux_sym_line_comment_token2,
  [anon_sym_SLASH_STAR] = anon_sym_SLASH_STAR,
  [anon_sym_STAR_SLASH] = anon_sym_STAR_SLASH,
  [sym__block_doc_comment_marker] = sym__block_doc_comment_marker,
  [sym__block_comment_content] = sym__block_comment_content,
  [sym__doc_line_comment] = sym__doc_line_comment,
  [sym__error_sentinel] = sym__error_sentinel,
  [sym_source_file] = sym_source_file,
  [sym_number_type] = sym_number_type,
  [sym__quantifier_directive] = sym__quantifier_directive,
  [sym_primitive_type] = sym_primitive_type,
  [sym_var_name] = sym_var_name,
  [sym_discouraged_name] = sym_discouraged_name,
  [sym__reuseable_keywords] = sym__reuseable_keywords,
  [sym_number] = sym_number,
  [sym_numerical_addr] = sym_numerical_addr,
  [sym_bool_literal] = sym_bool_literal,
  [sym_typed_number] = sym_typed_number,
  [sym_byte_string] = sym_byte_string,
  [sym__leading_name_access] = sym__leading_name_access,
  [sym__leading_name_access_wildcard] = sym__leading_name_access_wildcard,
  [sym_name_access_chain] = sym_name_access_chain,
  [sym_name_access_chain_wildcard] = sym_name_access_chain_wildcard,
  [sym_type] = sym_type,
  [sym__type] = sym__type,
  [sym_tuple_type] = sym_tuple_type,
  [sym_closure_type] = sym_closure_type,
  [sym__ref_type] = sym__ref_type,
  [sym__expr] = sym__expr,
  [sym_assignment] = sym_assignment,
  [sym_lambda_bind_list] = sym_lambda_bind_list,
  [sym_quantifier] = sym_quantifier,
  [sym_quantifier_bind] = sym_quantifier_bind,
  [sym_triggers] = sym_triggers,
  [sym__op_expr] = sym__op_expr,
  [sym_bin_op_expr] = sym_bin_op_expr,
  [sym__unary_expr] = sym__unary_expr,
  [sym_not_expr] = sym_not_expr,
  [sym_ref_expr] = sym_ref_expr,
  [sym_ref_mut_expr] = sym_ref_mut_expr,
  [sym_deref_expr] = sym_deref_expr,
  [sym_move_expr] = sym_move_expr,
  [sym_copy_expr] = sym_copy_expr,
  [sym_receiver_call] = sym_receiver_call,
  [sym_mem_access] = sym_mem_access,
  [sym_access_field] = sym_access_field,
  [sym__identifier_or_anon_field] = sym__identifier_or_anon_field,
  [sym_term] = sym_term,
  [sym_vector_value_expr] = sym_vector_value_expr,
  [sym_tuple_expr] = sym_tuple_expr,
  [sym_type_hint_expr] = sym_type_hint_expr,
  [sym_cast_expr] = sym_cast_expr,
  [sym_parenthesized_expr] = sym_parenthesized_expr,
  [sym_match_expr] = sym_match_expr,
  [sym_match_arm] = sym_match_arm,
  [sym_if_expr] = sym_if_expr,
  [sym_while_expr] = sym_while_expr,
  [sym_loop_expr] = sym_loop_expr,
  [sym_return_expr] = sym_return_expr,
  [sym_abort_expr] = sym_abort_expr,
  [sym_for_loop_expr] = sym_for_loop_expr,
  [sym__control_body] = sym__control_body,
  [sym_var] = sym_var,
  [sym_call_expr] = sym_call_expr,
  [sym_pack_expr] = sym_pack_expr,
  [sym_macro_call_expr] = sym_macro_call_expr,
  [sym_call_args] = sym_call_args,
  [sym_type_args] = sym_type_args,
  [sym_expr_field] = sym_expr_field,
  [sym_value] = sym_value,
  [sym_attributes] = sym_attributes,
  [sym_attribute] = sym_attribute,
  [sym__attribute_name] = sym__attribute_name,
  [sym__attribute_val] = sym__attribute_val,
  [sym_address_block] = sym_address_block,
  [sym__address_member] = sym__address_member,
  [sym__module_keyword] = sym__module_keyword,
  [sym__module_path] = sym__module_path,
  [sym_module] = sym_module,
  [sym_declaration] = sym_declaration,
  [sym_spec_block] = sym_spec_block,
  [sym_spec_block_target] = sym_spec_block_target,
  [sym__spec_target_signature_opt] = sym__spec_target_signature_opt,
  [sym_spec_loop_invariant] = sym_spec_loop_invariant,
  [sym__spec_block_member] = sym__spec_block_member,
  [sym_spec_invariant] = sym_spec_invariant,
  [sym_spec_condition] = sym_spec_condition,
  [sym_asserts] = sym_asserts,
  [sym_aborts_if] = sym_aborts_if,
  [sym_aborts_with_or_modifies] = sym_aborts_with_or_modifies,
  [sym_emits] = sym_emits,
  [sym_condition_props] = sym_condition_props,
  [sym_spec_variable] = sym_spec_variable,
  [sym_spec_func] = sym_spec_func,
  [sym__spec_func_signatures] = sym__spec_func_signatures,
  [sym_spec_include] = sym_spec_include,
  [sym_spec_apply] = sym_spec_apply,
  [sym__spec_apply_pattern] = sym__spec_apply_pattern,
  [sym__spec_apply_fragment] = sym__spec_apply_fragment,
  [sym_spec_pragma] = sym_spec_pragma,
  [sym__spec_pragma_prop] = sym__spec_pragma_prop,
  [sym_spec_let] = sym_spec_let,
  [sym_spec_update] = sym_spec_update,
  [sym_spec_axiom] = sym_spec_axiom,
  [sym_visibility] = sym_visibility,
  [sym_module_member_modifier] = sym_module_member_modifier,
  [sym_module_ident] = sym_module_ident,
  [sym_use_decl] = sym_use_decl,
  [sym__use_alias] = sym__use_alias,
  [sym__use_member] = sym__use_member,
  [sym_friend_decl] = sym_friend_decl,
  [sym_constant_decl] = sym_constant_decl,
  [sym_function_decl] = sym_function_decl,
  [sym__function_signature] = sym__function_signature,
  [sym__specifier] = sym__specifier,
  [sym_block] = sym_block,
  [sym_parameters] = sym_parameters,
  [sym_type_param] = sym_type_param,
  [sym_type_params] = sym_type_params,
  [sym_constraints] = sym_constraints,
  [sym_parameter] = sym_parameter,
  [sym_access_specifier_list] = sym_access_specifier_list,
  [sym_access_specifier] = sym_access_specifier,
  [sym__address_specifier] = sym__address_specifier,
  [sym__struct_or_enum_decl] = sym__struct_or_enum_decl,
  [sym_struct_decl] = sym_struct_decl,
  [sym__struct_signature] = sym__struct_signature,
  [sym_struct_body] = sym_struct_body,
  [sym_anon_fields] = sym_anon_fields,
  [sym__struct_def_name] = sym__struct_def_name,
  [sym__struct_type_params] = sym_type_params,
  [sym__struct_type_parameter] = sym_type_param,
  [sym_field_annot] = sym_field_annot,
  [sym_enum_decl] = sym_enum_decl,
  [sym__enum_signature] = sym__enum_signature,
  [sym_enum_body] = sym_enum_body,
  [sym__variant] = sym__variant,
  [sym__variant_last] = sym__variant_last,
  [sym_enum_variant_struct] = sym_enum_variant_struct,
  [sym_enum_variant_posit] = sym_enum_variant_posit,
  [sym_enum_variant] = sym_enum_variant,
  [sym_ability] = sym_ability,
  [sym_abilities] = sym_abilities,
  [sym__sequence_item] = sym__sequence_item,
  [sym_let_expr] = sym_let_expr,
  [sym_bind_list] = sym_bind_list,
  [sym__bind] = sym__bind,
  [sym__bind_tuple] = sym__bind_tuple,
  [sym__bind_fields] = sym__bind_fields,
  [sym_bind_field] = sym_bind_field,
  [sym_script] = sym_script,
  [sym__script_use_decl] = sym_declaration,
  [sym__script_constant_decl] = sym_declaration,
  [sym__script_func_decl] = sym_declaration,
  [sym__script_spec_block] = sym_declaration,
  [sym_line_comment] = sym_line_comment,
  [sym_block_comment] = sym_block_comment,
  [aux_sym_source_file_repeat1] = aux_sym_source_file_repeat1,
  [aux_sym_tuple_type_repeat1] = aux_sym_tuple_type_repeat1,
  [aux_sym_lambda_bind_list_repeat1] = aux_sym_lambda_bind_list_repeat1,
  [aux_sym_quantifier_repeat1] = aux_sym_quantifier_repeat1,
  [aux_sym_triggers_repeat1] = aux_sym_triggers_repeat1,
  [aux_sym_triggers_repeat2] = aux_sym_triggers_repeat2,
  [aux_sym_vector_value_expr_repeat1] = aux_sym_vector_value_expr_repeat1,
  [aux_sym_match_expr_repeat1] = aux_sym_match_expr_repeat1,
  [aux_sym_pack_expr_repeat1] = aux_sym_pack_expr_repeat1,
  [aux_sym_attributes_repeat1] = aux_sym_attributes_repeat1,
  [aux_sym_attributes_repeat2] = aux_sym_attributes_repeat2,
  [aux_sym__attribute_name_repeat1] = aux_sym__attribute_name_repeat1,
  [aux_sym_address_block_repeat1] = aux_sym_address_block_repeat1,
  [aux_sym_module_repeat1] = aux_sym_module_repeat1,
  [aux_sym_declaration_repeat1] = aux_sym_declaration_repeat1,
  [aux_sym_spec_block_repeat1] = aux_sym_spec_block_repeat1,
  [aux_sym_spec_block_repeat2] = aux_sym_spec_block_repeat2,
  [aux_sym_condition_props_repeat1] = aux_sym_condition_props_repeat1,
  [aux_sym_spec_apply_repeat1] = aux_sym_spec_apply_repeat1,
  [aux_sym__spec_apply_pattern_repeat1] = aux_sym__spec_apply_pattern_repeat1,
  [aux_sym_use_decl_repeat1] = aux_sym_use_decl_repeat1,
  [aux_sym__specifier_repeat1] = aux_sym__specifier_repeat1,
  [aux_sym_block_repeat1] = aux_sym_block_repeat1,
  [aux_sym_parameters_repeat1] = aux_sym_parameters_repeat1,
  [aux_sym_type_params_repeat1] = aux_sym_type_params_repeat1,
  [aux_sym_constraints_repeat1] = aux_sym_constraints_repeat1,
  [aux_sym_access_specifier_list_repeat1] = aux_sym_access_specifier_list_repeat1,
  [aux_sym_struct_body_repeat1] = aux_sym_struct_body_repeat1,
  [aux_sym__struct_type_params_repeat1] = aux_sym__struct_type_params_repeat1,
  [aux_sym_enum_body_repeat1] = aux_sym_enum_body_repeat1,
  [aux_sym_abilities_repeat1] = aux_sym_abilities_repeat1,
  [aux_sym__bind_fields_repeat1] = aux_sym__bind_fields_repeat1,
  [aux_sym_script_repeat1] = aux_sym_script_repeat1,
  [aux_sym_script_repeat2] = aux_sym_script_repeat2,
  [aux_sym_script_repeat3] = aux_sym_script_repeat3,
  [alias_sym_alias] = alias_sym_alias,
  [alias_sym_body] = alias_sym_body,
  [alias_sym_condition] = alias_sym_condition,
  [alias_sym_pattern] = alias_sym_pattern,
  [alias_sym_result] = alias_sym_result,
  [alias_sym_shorthand_field_identifier] = alias_sym_shorthand_field_identifier,
};

static const TSSymbolMetadata ts_symbol_metadata[] = {
  [ts_builtin_sym_end] = {
    .visible = false,
    .named = true,
  },
  [sym_identifier] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_u8] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_u16] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_u32] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_u64] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_u128] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_u256] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_exists] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_forall] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_choose] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_min] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_bool] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_address] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_vector] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_for] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_while] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_friend] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_match] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_number_token1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_number_token2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_number_token3] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_number_token4] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_number_token5] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_number_token6] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_number_token7] = {
    .visible = false,
    .named = false,
  },
  [anon_sym_true] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_false] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_byte_string_token1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_byte_string_token2] = {
    .visible = false,
    .named = false,
  },
  [anon_sym_STAR] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COLON_COLON] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LPAREN] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COMMA] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RPAREN] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMPmut] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_where] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COLON] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_in] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LBRACE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ_EQ_GT] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_LT_EQ_EQ_GT] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_PIPE_PIPE] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_AMP_AMP] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_EQ_EQ] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_BANG_EQ] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_LT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_EQ] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_GT_EQ] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_DOT_DOT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_CARET] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_LT_LT] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_GT_GT] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_PLUS] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_SLASH] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_PERCENT] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_BANG] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_move] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_copy] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACK] = {
    .visible = true,
    .named = false,
  },
  [sym_anon_field_index] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_break] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_continue] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_as] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_if] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_else] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_loop] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_return] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_abort] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POUND] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_module] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_spec] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_schema] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_invariant] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_update] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SEMI] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_assert] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_assume] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ensures] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_requires] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_aborts_if] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_with] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_aborts_with] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_modifies] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_emits] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_to] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_global] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_local] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_fun] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_native] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_include] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_apply] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_except] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_public] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_internal] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_pragma] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_let] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_post] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_axiom] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_script] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_package] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_entry] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_use] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_const] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_inline] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_pure] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_acquires] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_reads] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_writes] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_struct] = {
    .visible = true,
    .named = false,
  },
  [sym_phantom] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_enum] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_drop] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_store] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_key] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_has] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH_SLASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH_SLASH2] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_line_comment_token1] = {
    .visible = false,
    .named = false,
  },
  [anon_sym_SLASH2] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_line_comment_token2] = {
    .visible = false,
    .named = false,
  },
  [anon_sym_SLASH_STAR] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR_SLASH] = {
    .visible = true,
    .named = false,
  },
  [sym__block_doc_comment_marker] = {
    .visible = false,
    .named = true,
  },
  [sym__block_comment_content] = {
    .visible = false,
    .named = true,
  },
  [sym__doc_line_comment] = {
    .visible = true,
    .named = true,
  },
  [sym__error_sentinel] = {
    .visible = false,
    .named = true,
  },
  [sym_source_file] = {
    .visible = true,
    .named = true,
  },
  [sym_number_type] = {
    .visible = true,
    .named = true,
  },
  [sym__quantifier_directive] = {
    .visible = false,
    .named = true,
  },
  [sym_primitive_type] = {
    .visible = true,
    .named = true,
  },
  [sym_var_name] = {
    .visible = true,
    .named = true,
  },
  [sym_discouraged_name] = {
    .visible = true,
    .named = true,
  },
  [sym__reuseable_keywords] = {
    .visible = false,
    .named = true,
  },
  [sym_number] = {
    .visible = true,
    .named = true,
  },
  [sym_numerical_addr] = {
    .visible = true,
    .named = true,
  },
  [sym_bool_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_typed_number] = {
    .visible = true,
    .named = true,
  },
  [sym_byte_string] = {
    .visible = true,
    .named = true,
  },
  [sym__leading_name_access] = {
    .visible = false,
    .named = true,
  },
  [sym__leading_name_access_wildcard] = {
    .visible = false,
    .named = true,
  },
  [sym_name_access_chain] = {
    .visible = true,
    .named = true,
  },
  [sym_name_access_chain_wildcard] = {
    .visible = true,
    .named = true,
  },
  [sym_type] = {
    .visible = true,
    .named = true,
  },
  [sym__type] = {
    .visible = false,
    .named = true,
  },
  [sym_tuple_type] = {
    .visible = true,
    .named = true,
  },
  [sym_closure_type] = {
    .visible = true,
    .named = true,
  },
  [sym__ref_type] = {
    .visible = false,
    .named = true,
  },
  [sym__expr] = {
    .visible = false,
    .named = true,
  },
  [sym_assignment] = {
    .visible = true,
    .named = true,
  },
  [sym_lambda_bind_list] = {
    .visible = true,
    .named = true,
  },
  [sym_quantifier] = {
    .visible = true,
    .named = true,
  },
  [sym_quantifier_bind] = {
    .visible = true,
    .named = true,
  },
  [sym_triggers] = {
    .visible = true,
    .named = true,
  },
  [sym__op_expr] = {
    .visible = false,
    .named = true,
  },
  [sym_bin_op_expr] = {
    .visible = true,
    .named = true,
  },
  [sym__unary_expr] = {
    .visible = false,
    .named = true,
  },
  [sym_not_expr] = {
    .visible = true,
    .named = true,
  },
  [sym_ref_expr] = {
    .visible = true,
    .named = true,
  },
  [sym_ref_mut_expr] = {
    .visible = true,
    .named = true,
  },
  [sym_deref_expr] = {
    .visible = true,
    .named = true,
  },
  [sym_move_expr] = {
    .visible = true,
    .named = true,
  },
  [sym_copy_expr] = {
    .visible = true,
    .named = true,
  },
  [sym_receiver_call] = {
    .visible = true,
    .named = true,
  },
  [sym_mem_access] = {
    .visible = true,
    .named = true,
  },
  [sym_access_field] = {
    .visible = true,
    .named = true,
  },
  [sym__identifier_or_anon_field] = {
    .visible = false,
    .named = true,
  },
  [sym_term] = {
    .visible = true,
    .named = false,
  },
  [sym_vector_value_expr] = {
    .visible = true,
    .named = true,
  },
  [sym_tuple_expr] = {
    .visible = true,
    .named = true,
  },
  [sym_type_hint_expr] = {
    .visible = true,
    .named = true,
  },
  [sym_cast_expr] = {
    .visible = true,
    .named = true,
  },
  [sym_parenthesized_expr] = {
    .visible = true,
    .named = true,
  },
  [sym_match_expr] = {
    .visible = true,
    .named = true,
  },
  [sym_match_arm] = {
    .visible = true,
    .named = true,
  },
  [sym_if_expr] = {
    .visible = true,
    .named = true,
  },
  [sym_while_expr] = {
    .visible = true,
    .named = true,
  },
  [sym_loop_expr] = {
    .visible = true,
    .named = true,
  },
  [sym_return_expr] = {
    .visible = true,
    .named = true,
  },
  [sym_abort_expr] = {
    .visible = true,
    .named = true,
  },
  [sym_for_loop_expr] = {
    .visible = true,
    .named = true,
  },
  [sym__control_body] = {
    .visible = false,
    .named = true,
  },
  [sym_var] = {
    .visible = true,
    .named = true,
  },
  [sym_call_expr] = {
    .visible = true,
    .named = true,
  },
  [sym_pack_expr] = {
    .visible = true,
    .named = true,
  },
  [sym_macro_call_expr] = {
    .visible = true,
    .named = true,
  },
  [sym_call_args] = {
    .visible = true,
    .named = true,
  },
  [sym_type_args] = {
    .visible = true,
    .named = true,
  },
  [sym_expr_field] = {
    .visible = true,
    .named = true,
  },
  [sym_value] = {
    .visible = true,
    .named = true,
  },
  [sym_attributes] = {
    .visible = true,
    .named = true,
  },
  [sym_attribute] = {
    .visible = true,
    .named = true,
  },
  [sym__attribute_name] = {
    .visible = false,
    .named = true,
  },
  [sym__attribute_val] = {
    .visible = false,
    .named = true,
  },
  [sym_address_block] = {
    .visible = true,
    .named = true,
  },
  [sym__address_member] = {
    .visible = false,
    .named = true,
  },
  [sym__module_keyword] = {
    .visible = false,
    .named = true,
  },
  [sym__module_path] = {
    .visible = false,
    .named = true,
  },
  [sym_module] = {
    .visible = true,
    .named = true,
  },
  [sym_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_spec_block] = {
    .visible = true,
    .named = true,
  },
  [sym_spec_block_target] = {
    .visible = true,
    .named = true,
  },
  [sym__spec_target_signature_opt] = {
    .visible = false,
    .named = true,
  },
  [sym_spec_loop_invariant] = {
    .visible = true,
    .named = true,
  },
  [sym__spec_block_member] = {
    .visible = false,
    .named = true,
  },
  [sym_spec_invariant] = {
    .visible = true,
    .named = true,
  },
  [sym_spec_condition] = {
    .visible = true,
    .named = true,
  },
  [sym_asserts] = {
    .visible = true,
    .named = true,
  },
  [sym_aborts_if] = {
    .visible = true,
    .named = true,
  },
  [sym_aborts_with_or_modifies] = {
    .visible = true,
    .named = true,
  },
  [sym_emits] = {
    .visible = true,
    .named = true,
  },
  [sym_condition_props] = {
    .visible = true,
    .named = true,
  },
  [sym_spec_variable] = {
    .visible = true,
    .named = true,
  },
  [sym_spec_func] = {
    .visible = true,
    .named = true,
  },
  [sym__spec_func_signatures] = {
    .visible = false,
    .named = true,
  },
  [sym_spec_include] = {
    .visible = true,
    .named = true,
  },
  [sym_spec_apply] = {
    .visible = true,
    .named = true,
  },
  [sym__spec_apply_pattern] = {
    .visible = false,
    .named = true,
  },
  [sym__spec_apply_fragment] = {
    .visible = false,
    .named = true,
  },
  [sym_spec_pragma] = {
    .visible = true,
    .named = true,
  },
  [sym__spec_pragma_prop] = {
    .visible = false,
    .named = true,
  },
  [sym_spec_let] = {
    .visible = true,
    .named = true,
  },
  [sym_spec_update] = {
    .visible = true,
    .named = true,
  },
  [sym_spec_axiom] = {
    .visible = true,
    .named = true,
  },
  [sym_visibility] = {
    .visible = true,
    .named = true,
  },
  [sym_module_member_modifier] = {
    .visible = true,
    .named = true,
  },
  [sym_module_ident] = {
    .visible = true,
    .named = true,
  },
  [sym_use_decl] = {
    .visible = true,
    .named = true,
  },
  [sym__use_alias] = {
    .visible = false,
    .named = true,
  },
  [sym__use_member] = {
    .visible = true,
    .named = true,
  },
  [sym_friend_decl] = {
    .visible = true,
    .named = true,
  },
  [sym_constant_decl] = {
    .visible = true,
    .named = true,
  },
  [sym_function_decl] = {
    .visible = true,
    .named = true,
  },
  [sym__function_signature] = {
    .visible = false,
    .named = true,
  },
  [sym__specifier] = {
    .visible = false,
    .named = true,
  },
  [sym_block] = {
    .visible = true,
    .named = true,
  },
  [sym_parameters] = {
    .visible = true,
    .named = true,
  },
  [sym_type_param] = {
    .visible = true,
    .named = true,
  },
  [sym_type_params] = {
    .visible = true,
    .named = true,
  },
  [sym_constraints] = {
    .visible = true,
    .named = true,
  },
  [sym_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_access_specifier_list] = {
    .visible = true,
    .named = true,
  },
  [sym_access_specifier] = {
    .visible = true,
    .named = true,
  },
  [sym__address_specifier] = {
    .visible = false,
    .named = true,
  },
  [sym__struct_or_enum_decl] = {
    .visible = false,
    .named = true,
  },
  [sym_struct_decl] = {
    .visible = true,
    .named = true,
  },
  [sym__struct_signature] = {
    .visible = false,
    .named = true,
  },
  [sym_struct_body] = {
    .visible = true,
    .named = true,
  },
  [sym_anon_fields] = {
    .visible = true,
    .named = true,
  },
  [sym__struct_def_name] = {
    .visible = false,
    .named = true,
  },
  [sym__struct_type_params] = {
    .visible = true,
    .named = true,
  },
  [sym__struct_type_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_field_annot] = {
    .visible = true,
    .named = true,
  },
  [sym_enum_decl] = {
    .visible = true,
    .named = true,
  },
  [sym__enum_signature] = {
    .visible = false,
    .named = true,
  },
  [sym_enum_body] = {
    .visible = true,
    .named = true,
  },
  [sym__variant] = {
    .visible = false,
    .named = true,
  },
  [sym__variant_last] = {
    .visible = false,
    .named = true,
  },
  [sym_enum_variant_struct] = {
    .visible = true,
    .named = true,
  },
  [sym_enum_variant_posit] = {
    .visible = true,
    .named = true,
  },
  [sym_enum_variant] = {
    .visible = true,
    .named = true,
  },
  [sym_ability] = {
    .visible = true,
    .named = true,
  },
  [sym_abilities] = {
    .visible = true,
    .named = true,
  },
  [sym__sequence_item] = {
    .visible = false,
    .named = true,
  },
  [sym_let_expr] = {
    .visible = true,
    .named = true,
  },
  [sym_bind_list] = {
    .visible = true,
    .named = true,
  },
  [sym__bind] = {
    .visible = false,
    .named = true,
  },
  [sym__bind_tuple] = {
    .visible = true,
    .named = true,
  },
  [sym__bind_fields] = {
    .visible = true,
    .named = true,
  },
  [sym_bind_field] = {
    .visible = true,
    .named = true,
  },
  [sym_script] = {
    .visible = true,
    .named = true,
  },
  [sym__script_use_decl] = {
    .visible = true,
    .named = true,
  },
  [sym__script_constant_decl] = {
    .visible = true,
    .named = true,
  },
  [sym__script_func_decl] = {
    .visible = true,
    .named = true,
  },
  [sym__script_spec_block] = {
    .visible = true,
    .named = true,
  },
  [sym_line_comment] = {
    .visible = true,
    .named = true,
  },
  [sym_block_comment] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_source_file_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_tuple_type_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_lambda_bind_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_quantifier_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_triggers_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_triggers_repeat2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_vector_value_expr_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_match_expr_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_pack_expr_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_attributes_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_attributes_repeat2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__attribute_name_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_address_block_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_module_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_declaration_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_spec_block_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_spec_block_repeat2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_condition_props_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_spec_apply_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__spec_apply_pattern_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_use_decl_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__specifier_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_block_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_parameters_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_type_params_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_constraints_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_access_specifier_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_struct_body_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__struct_type_params_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_enum_body_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_abilities_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__bind_fields_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_script_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_script_repeat2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_script_repeat3] = {
    .visible = false,
    .named = false,
  },
  [alias_sym_alias] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_body] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_condition] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_pattern] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_result] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_shorthand_field_identifier] = {
    .visible = true,
    .named = true,
  },
};

enum ts_field_identifiers {
  field_access_four = 1,
  field_access_three = 2,
  field_access_two = 3,
  field_arg = 4,
  field_arguments = 5,
  field_assertion = 6,
  field_attr_path = 7,
  field_attribute = 8,
  field_begin = 9,
  field_bind = 10,
  field_body = 11,
  field_condition = 12,
  field_else = 13,
  field_emission = 14,
  field_end = 15,
  field_exclusions = 16,
  field_expression = 17,
  field_field = 18,
  field_func = 19,
  field_func_name = 20,
  field_index = 21,
  field_kind = 22,
  field_lhs = 23,
  field_literal_address = 24,
  field_macro_name = 25,
  field_member = 26,
  field_module_name = 27,
  field_name = 28,
  field_name_pattern = 29,
  field_negated = 30,
  field_object = 31,
  field_param_types = 32,
  field_parameters = 33,
  field_path = 34,
  field_post_state = 35,
  field_prop_name = 36,
  field_properties = 37,
  field_property = 38,
  field_pure = 39,
  field_receiver = 40,
  field_ref = 41,
  field_ref_mut = 42,
  field_return_type = 43,
  field_rhs = 44,
  field_schema = 45,
  field_schema_name = 46,
  field_scope = 47,
  field_signature = 48,
  field_spec_func_name = 49,
  field_specifier = 50,
  field_struct = 51,
  field_struct_name = 52,
  field_target = 53,
  field_targets = 54,
  field_then = 55,
  field_trigger = 56,
  field_triggers = 57,
  field_type = 58,
  field_type_arguments = 59,
  field_type_bind = 60,
  field_type_generics = 61,
  field_type_parameters = 62,
  field_type_params = 63,
  field_update = 64,
  field_value = 65,
  field_value_bind = 66,
  field_var = 67,
  field_variable = 68,
  field_variant = 69,
  field_visibility = 70,
  field_where = 71,
  field_with = 72,
};

static const char * const ts_field_names[] = {
  [0] = NULL,
  [field_access_four] = "access_four",
  [field_access_three] = "access_three",
  [field_access_two] = "access_two",
  [field_arg] = "arg",
  [field_arguments] = "arguments",
  [field_assertion] = "assertion",
  [field_attr_path] = "attr_path",
  [field_attribute] = "attribute",
  [field_begin] = "begin",
  [field_bind] = "bind",
  [field_body] = "body",
  [field_condition] = "condition",
  [field_else] = "else",
  [field_emission] = "emission",
  [field_end] = "end",
  [field_exclusions] = "exclusions",
  [field_expression] = "expression",
  [field_field] = "field",
  [field_func] = "func",
  [field_func_name] = "func_name",
  [field_index] = "index",
  [field_kind] = "kind",
  [field_lhs] = "lhs",
  [field_literal_address] = "literal_address",
  [field_macro_name] = "macro_name",
  [field_member] = "member",
  [field_module_name] = "module_name",
  [field_name] = "name",
  [field_name_pattern] = "name_pattern",
  [field_negated] = "negated",
  [field_object] = "object",
  [field_param_types] = "param_types",
  [field_parameters] = "parameters",
  [field_path] = "path",
  [field_post_state] = "post_state",
  [field_prop_name] = "prop_name",
  [field_properties] = "properties",
  [field_property] = "property",
  [field_pure] = "pure",
  [field_receiver] = "receiver",
  [field_ref] = "ref",
  [field_ref_mut] = "ref_mut",
  [field_return_type] = "return_type",
  [field_rhs] = "rhs",
  [field_schema] = "schema",
  [field_schema_name] = "schema_name",
  [field_scope] = "scope",
  [field_signature] = "signature",
  [field_spec_func_name] = "spec_func_name",
  [field_specifier] = "specifier",
  [field_struct] = "struct",
  [field_struct_name] = "struct_name",
  [field_target] = "target",
  [field_targets] = "targets",
  [field_then] = "then",
  [field_trigger] = "trigger",
  [field_triggers] = "triggers",
  [field_type] = "type",
  [field_type_arguments] = "type_arguments",
  [field_type_bind] = "type_bind",
  [field_type_generics] = "type_generics",
  [field_type_parameters] = "type_parameters",
  [field_type_params] = "type_params",
  [field_update] = "update",
  [field_value] = "value",
  [field_value_bind] = "value_bind",
  [field_var] = "var",
  [field_variable] = "variable",
  [field_variant] = "variant",
  [field_visibility] = "visibility",
  [field_where] = "where",
  [field_with] = "with",
};

static const TSMapSlice ts_field_map_slices[PRODUCTION_ID_COUNT] = {
  [1] = {.index = 0, .length = 1},
  [2] = {.index = 1, .length = 1},
  [3] = {.index = 2, .length = 2},
  [5] = {.index = 4, .length = 2},
  [6] = {.index = 6, .length = 2},
  [7] = {.index = 8, .length = 2},
  [8] = {.index = 10, .length = 7},
  [9] = {.index = 17, .length = 8},
  [10] = {.index = 25, .length = 1},
  [11] = {.index = 26, .length = 2},
  [12] = {.index = 28, .length = 3},
  [13] = {.index = 31, .length = 1},
  [14] = {.index = 32, .length = 1},
  [15] = {.index = 33, .length = 1},
  [16] = {.index = 33, .length = 1},
  [17] = {.index = 34, .length = 2},
  [18] = {.index = 36, .length = 1},
  [19] = {.index = 37, .length = 1},
  [20] = {.index = 38, .length = 4},
  [21] = {.index = 42, .length = 1},
  [22] = {.index = 43, .length = 2},
  [23] = {.index = 45, .length = 2},
  [24] = {.index = 47, .length = 1},
  [25] = {.index = 48, .length = 1},
  [26] = {.index = 49, .length = 1},
  [27] = {.index = 50, .length = 1},
  [28] = {.index = 51, .length = 1},
  [29] = {.index = 52, .length = 1},
  [30] = {.index = 53, .length = 1},
  [31] = {.index = 54, .length = 1},
  [32] = {.index = 55, .length = 2},
  [33] = {.index = 57, .length = 1},
  [35] = {.index = 58, .length = 1},
  [36] = {.index = 59, .length = 5},
  [37] = {.index = 64, .length = 1},
  [38] = {.index = 65, .length = 3},
  [39] = {.index = 68, .length = 1},
  [41] = {.index = 69, .length = 2},
  [42] = {.index = 71, .length = 2},
  [43] = {.index = 73, .length = 1},
  [44] = {.index = 74, .length = 2},
  [45] = {.index = 76, .length = 1},
  [46] = {.index = 77, .length = 1},
  [47] = {.index = 78, .length = 2},
  [48] = {.index = 80, .length = 1},
  [49] = {.index = 81, .length = 3},
  [50] = {.index = 84, .length = 3},
  [51] = {.index = 87, .length = 3},
  [52] = {.index = 90, .length = 4},
  [53] = {.index = 94, .length = 5},
  [54] = {.index = 99, .length = 2},
  [55] = {.index = 101, .length = 1},
  [56] = {.index = 102, .length = 2},
  [57] = {.index = 104, .length = 3},
  [58] = {.index = 107, .length = 2},
  [59] = {.index = 109, .length = 1},
  [60] = {.index = 110, .length = 3},
  [61] = {.index = 113, .length = 2},
  [62] = {.index = 115, .length = 1},
  [63] = {.index = 116, .length = 1},
  [64] = {.index = 117, .length = 1},
  [65] = {.index = 118, .length = 2},
  [66] = {.index = 120, .length = 3},
  [67] = {.index = 123, .length = 2},
  [68] = {.index = 125, .length = 2},
  [69] = {.index = 127, .length = 2},
  [70] = {.index = 129, .length = 1},
  [71] = {.index = 130, .length = 1},
  [72] = {.index = 131, .length = 3},
  [73] = {.index = 134, .length = 2},
  [74] = {.index = 136, .length = 6},
  [75] = {.index = 142, .length = 2},
  [76] = {.index = 144, .length = 1},
  [77] = {.index = 145, .length = 1},
  [78] = {.index = 146, .length = 5},
  [79] = {.index = 151, .length = 3},
  [80] = {.index = 154, .length = 2},
  [81] = {.index = 156, .length = 2},
  [82] = {.index = 158, .length = 1},
  [83] = {.index = 159, .length = 1},
  [84] = {.index = 160, .length = 3},
  [85] = {.index = 163, .length = 1},
  [86] = {.index = 164, .length = 4},
  [87] = {.index = 168, .length = 4},
  [88] = {.index = 172, .length = 3},
  [89] = {.index = 175, .length = 3},
  [90] = {.index = 178, .length = 3},
  [91] = {.index = 181, .length = 3},
  [92] = {.index = 184, .length = 2},
  [93] = {.index = 186, .length = 2},
  [94] = {.index = 188, .length = 1},
  [96] = {.index = 189, .length = 3},
  [97] = {.index = 192, .length = 4},
  [98] = {.index = 196, .length = 4},
  [99] = {.index = 200, .length = 2},
  [100] = {.index = 202, .length = 4},
  [101] = {.index = 206, .length = 2},
  [102] = {.index = 208, .length = 6},
  [103] = {.index = 214, .length = 4},
  [104] = {.index = 218, .length = 2},
  [105] = {.index = 220, .length = 3},
  [106] = {.index = 223, .length = 1},
  [107] = {.index = 224, .length = 2},
  [108] = {.index = 116, .length = 1},
  [109] = {.index = 226, .length = 6},
  [110] = {.index = 232, .length = 1},
  [111] = {.index = 233, .length = 4},
  [112] = {.index = 237, .length = 1},
  [113] = {.index = 238, .length = 1},
  [114] = {.index = 239, .length = 3},
  [115] = {.index = 242, .length = 6},
  [116] = {.index = 248, .length = 2},
  [117] = {.index = 250, .length = 2},
  [118] = {.index = 252, .length = 2},
  [119] = {.index = 254, .length = 2},
  [120] = {.index = 256, .length = 2},
  [121] = {.index = 258, .length = 2},
  [122] = {.index = 260, .length = 1},
  [123] = {.index = 261, .length = 4},
  [124] = {.index = 265, .length = 6},
  [125] = {.index = 271, .length = 1},
  [126] = {.index = 272, .length = 2},
  [127] = {.index = 274, .length = 3},
  [128] = {.index = 277, .length = 4},
  [129] = {.index = 281, .length = 1},
  [130] = {.index = 282, .length = 3},
  [131] = {.index = 285, .length = 2},
  [132] = {.index = 287, .length = 1},
  [133] = {.index = 288, .length = 7},
  [134] = {.index = 295, .length = 3},
  [135] = {.index = 298, .length = 2},
  [136] = {.index = 300, .length = 1},
  [137] = {.index = 301, .length = 1},
  [138] = {.index = 302, .length = 7},
  [139] = {.index = 309, .length = 2},
  [140] = {.index = 311, .length = 2},
  [141] = {.index = 313, .length = 2},
  [142] = {.index = 315, .length = 3},
  [143] = {.index = 318, .length = 6},
  [144] = {.index = 324, .length = 4},
  [145] = {.index = 328, .length = 2},
  [146] = {.index = 330, .length = 2},
  [147] = {.index = 332, .length = 2},
  [148] = {.index = 334, .length = 2},
  [149] = {.index = 336, .length = 2},
  [150] = {.index = 338, .length = 5},
  [151] = {.index = 343, .length = 2},
  [152] = {.index = 345, .length = 7},
  [153] = {.index = 352, .length = 2},
  [154] = {.index = 354, .length = 2},
  [155] = {.index = 356, .length = 3},
  [156] = {.index = 359, .length = 3},
  [157] = {.index = 362, .length = 2},
  [158] = {.index = 364, .length = 3},
  [159] = {.index = 367, .length = 1},
  [160] = {.index = 368, .length = 4},
  [161] = {.index = 372, .length = 5},
  [162] = {.index = 377, .length = 3},
  [163] = {.index = 380, .length = 3},
  [164] = {.index = 383, .length = 3},
  [165] = {.index = 386, .length = 3},
  [166] = {.index = 389, .length = 7},
  [167] = {.index = 396, .length = 3},
  [168] = {.index = 399, .length = 3},
  [169] = {.index = 402, .length = 2},
  [170] = {.index = 404, .length = 3},
  [171] = {.index = 407, .length = 2},
  [172] = {.index = 409, .length = 5},
  [173] = {.index = 414, .length = 3},
  [174] = {.index = 417, .length = 6},
  [175] = {.index = 423, .length = 3},
  [176] = {.index = 426, .length = 9},
  [177] = {.index = 435, .length = 6},
  [178] = {.index = 441, .length = 3},
  [179] = {.index = 444, .length = 4},
  [180] = {.index = 448, .length = 3},
  [181] = {.index = 451, .length = 3},
  [182] = {.index = 454, .length = 4},
  [183] = {.index = 458, .length = 4},
  [184] = {.index = 462, .length = 2},
  [185] = {.index = 464, .length = 3},
  [186] = {.index = 467, .length = 3},
  [187] = {.index = 470, .length = 3},
  [188] = {.index = 473, .length = 6},
  [189] = {.index = 479, .length = 6},
  [190] = {.index = 485, .length = 9},
  [191] = {.index = 494, .length = 6},
  [192] = {.index = 500, .length = 9},
  [193] = {.index = 509, .length = 10},
  [194] = {.index = 519, .length = 4},
  [196] = {.index = 523, .length = 2},
  [197] = {.index = 525, .length = 3},
  [198] = {.index = 528, .length = 7},
  [199] = {.index = 535, .length = 10},
  [200] = {.index = 545, .length = 10},
  [201] = {.index = 555, .length = 7},
  [202] = {.index = 562, .length = 10},
  [203] = {.index = 572, .length = 10},
  [204] = {.index = 582, .length = 13},
  [205] = {.index = 595, .length = 10},
  [206] = {.index = 605, .length = 13},
  [207] = {.index = 618, .length = 4},
  [208] = {.index = 622, .length = 2},
  [209] = {.index = 624, .length = 11},
  [210] = {.index = 635, .length = 11},
  [211] = {.index = 646, .length = 14},
  [212] = {.index = 660, .length = 14},
  [213] = {.index = 674, .length = 11},
  [214] = {.index = 685, .length = 14},
  [215] = {.index = 699, .length = 14},
  [216] = {.index = 713, .length = 17},
  [217] = {.index = 730, .length = 4},
  [219] = {.index = 734, .length = 15},
  [220] = {.index = 749, .length = 15},
  [221] = {.index = 764, .length = 18},
  [222] = {.index = 782, .length = 18},
  [223] = {.index = 800, .length = 19},
};

static const TSFieldMapEntry ts_field_map_entries[] = {
  [0] =
    {field_name, 0},
  [1] =
    {field_attr_path, 0},
  [2] =
    {field_attr_path, 0, .inherited = true},
    {field_attribute, 0},
  [4] =
    {field_attr_path, 0},
    {field_attr_path, 1, .inherited = true},
  [6] =
    {field_name, 2},
    {field_path, 0},
  [8] =
    {field_name, 1, .inherited = true},
    {field_path, 1, .inherited = true},
  [10] =
    {field_name, 0, .inherited = true},
    {field_negated, 0, .inherited = true},
    {field_parameters, 0, .inherited = true},
    {field_pure, 0, .inherited = true},
    {field_return_type, 0, .inherited = true},
    {field_specifier, 0, .inherited = true},
    {field_type_parameters, 0, .inherited = true},
  [17] =
    {field_body, 1},
    {field_name, 0, .inherited = true},
    {field_negated, 0, .inherited = true},
    {field_parameters, 0, .inherited = true},
    {field_pure, 0, .inherited = true},
    {field_return_type, 0, .inherited = true},
    {field_specifier, 0, .inherited = true},
    {field_type_parameters, 0, .inherited = true},
  [25] =
    {field_attr_path, 1},
  [26] =
    {field_attr_path, 0, .inherited = true},
    {field_attr_path, 1, .inherited = true},
  [28] =
    {field_attr_path, 0, .inherited = true},
    {field_attribute, 0},
    {field_value, 2},
  [31] =
    {field_func_name, 0},
  [32] =
    {field_name, 1, .inherited = true},
  [33] =
    {field_name, 0, .inherited = true},
  [34] =
    {field_name, 1},
    {field_parameters, 2},
  [36] =
    {field_path, 1},
  [37] =
    {field_name, 1},
  [38] =
    {field_func_name, 0},
    {field_parameters, 1, .inherited = true},
    {field_return_type, 1, .inherited = true},
    {field_signature, 1},
  [42] =
    {field_parameters, 0},
  [43] =
    {field_body, 1},
    {field_body, 2},
  [45] =
    {field_schema, 0},
    {field_schema_name, 1},
  [47] =
    {field_variable, 0},
  [48] =
    {field_struct, 0},
  [49] =
    {field_type, 0},
  [50] =
    {field_variable, 1},
  [51] =
    {field_prop_name, 0},
  [52] =
    {field_body, 1},
  [53] =
    {field_value, 1},
  [54] =
    {field_condition, 1},
  [55] =
    {field_arguments, 1},
    {field_func_name, 0},
  [57] =
    {field_type_arguments, 1},
  [58] =
    {field_pure, 0},
  [59] =
    {field_name, 1},
    {field_negated, 3, .inherited = true},
    {field_parameters, 2},
    {field_pure, 3, .inherited = true},
    {field_specifier, 3},
  [64] =
    {field_negated, 0, .inherited = true},
  [65] =
    {field_name, 1},
    {field_parameters, 3},
    {field_type_parameters, 2},
  [68] =
    {field_module_name, 2},
  [69] =
    {field_ref, 0, .inherited = true},
    {field_ref_mut, 0, .inherited = true},
  [71] =
    {field_name, 2},
    {field_parameters, 3},
  [73] =
    {field_bind, 1},
  [74] =
    {field_struct, 0, .inherited = true},
    {field_variable, 0, .inherited = true},
  [76] =
    {field_access_two, 2},
  [77] =
    {field_parameters, 1},
  [78] =
    {field_condition, 1},
    {field_kind, 0},
  [80] =
    {field_kind, 0},
  [81] =
    {field_body, 1},
    {field_body, 2},
    {field_body, 3},
  [84] =
    {field_body, 1},
    {field_body, 3},
    {field_member, 2},
  [87] =
    {field_schema, 0},
    {field_schema, 2},
    {field_schema_name, 1},
  [90] =
    {field_parameters, 1, .inherited = true},
    {field_return_type, 1, .inherited = true},
    {field_signature, 1},
    {field_spec_func_name, 1, .inherited = true},
  [94] =
    {field_body, 2},
    {field_parameters, 1, .inherited = true},
    {field_return_type, 1, .inherited = true},
    {field_signature, 1},
    {field_spec_func_name, 1, .inherited = true},
  [99] =
    {field_body, 2},
    {field_body, 3},
  [101] =
    {field_trigger, 0, .inherited = true},
  [102] =
    {field_body, 2},
    {field_condition, 1},
  [104] =
    {field_struct, 0},
    {field_struct, 1, .inherited = true},
    {field_variable, 1, .inherited = true},
  [107] =
    {field_struct, 1, .inherited = true},
    {field_variable, 1, .inherited = true},
  [109] =
    {field_property, 1},
  [110] =
    {field_prop_name, 1, .inherited = true},
    {field_property, 1},
    {field_value, 1, .inherited = true},
  [113] =
    {field_condition, 1},
    {field_then, 2},
  [115] =
    {field_update, 1},
  [116] =
    {field_field, 0},
  [117] =
    {field_struct_name, 0},
  [118] =
    {field_arguments, 2},
    {field_macro_name, 0},
  [120] =
    {field_arguments, 2},
    {field_func_name, 0},
    {field_type_arguments, 1},
  [123] =
    {field_lhs, 0},
    {field_rhs, 2},
  [125] =
    {field_target, 0},
    {field_value, 2},
  [127] =
    {field_field, 2},
    {field_object, 0},
  [129] =
    {field_type_params, 1},
  [130] =
    {field_variant, 0},
  [131] =
    {field_name, 1},
    {field_parameters, 2},
    {field_return_type, 4},
  [134] =
    {field_negated, 0, .inherited = true},
    {field_negated, 1, .inherited = true},
  [136] =
    {field_name, 1},
    {field_negated, 4, .inherited = true},
    {field_parameters, 3},
    {field_pure, 4, .inherited = true},
    {field_specifier, 4},
    {field_type_parameters, 2},
  [142] =
    {field_member, 3},
    {field_path, 1},
  [144] =
    {field_ref, 1},
  [145] =
    {field_ref_mut, 1},
  [146] =
    {field_name, 2},
    {field_negated, 4, .inherited = true},
    {field_parameters, 3},
    {field_pure, 4, .inherited = true},
    {field_specifier, 4},
  [151] =
    {field_name, 2},
    {field_parameters, 4},
    {field_type_parameters, 3},
  [154] =
    {field_parameters, 0},
    {field_return_type, 2},
  [156] =
    {field_condition, 2},
    {field_kind, 0},
  [158] =
    {field_condition, 2},
  [159] =
    {field_properties, 1},
  [160] =
    {field_prop_name, 1, .inherited = true},
    {field_properties, 1},
    {field_value, 1, .inherited = true},
  [163] =
    {field_expression, 1},
  [164] =
    {field_body, 1},
    {field_body, 2},
    {field_body, 4},
    {field_member, 3},
  [168] =
    {field_parameters, 2, .inherited = true},
    {field_return_type, 2, .inherited = true},
    {field_signature, 2},
    {field_spec_func_name, 2, .inherited = true},
  [172] =
    {field_body, 2},
    {field_body, 3},
    {field_body, 4},
  [175] =
    {field_body, 2},
    {field_body, 4},
    {field_member, 3},
  [178] =
    {field_type_bind, 1},
    {field_type_bind, 2},
    {field_var, 0},
  [181] =
    {field_scope, 2},
    {field_value_bind, 1},
    {field_var, 0},
  [184] =
    {field_assertion, 3},
    {field_scope, 0},
  [186] =
    {field_trigger, 0, .inherited = true},
    {field_trigger, 1, .inherited = true},
  [188] =
    {field_condition, 3},
  [189] =
    {field_struct, 0},
    {field_struct, 2, .inherited = true},
    {field_variable, 2, .inherited = true},
  [192] =
    {field_struct, 1, .inherited = true},
    {field_struct, 2, .inherited = true},
    {field_variable, 1, .inherited = true},
    {field_variable, 2, .inherited = true},
  [196] =
    {field_struct, 0, .inherited = true},
    {field_struct, 1, .inherited = true},
    {field_variable, 0, .inherited = true},
    {field_variable, 1, .inherited = true},
  [200] =
    {field_prop_name, 0},
    {field_value, 2},
  [202] =
    {field_prop_name, 1, .inherited = true},
    {field_property, 1},
    {field_property, 2},
    {field_value, 1, .inherited = true},
  [206] =
    {field_prop_name, 1, .inherited = true},
    {field_value, 1, .inherited = true},
  [208] =
    {field_prop_name, 1, .inherited = true},
    {field_prop_name, 2, .inherited = true},
    {field_property, 1},
    {field_property, 2},
    {field_value, 1, .inherited = true},
    {field_value, 2, .inherited = true},
  [214] =
    {field_prop_name, 0, .inherited = true},
    {field_prop_name, 1, .inherited = true},
    {field_value, 0, .inherited = true},
    {field_value, 1, .inherited = true},
  [218] =
    {field_struct_name, 0},
    {field_type_arguments, 1},
  [220] =
    {field_arguments, 3},
    {field_func, 2},
    {field_receiver, 0},
  [223] =
    {field_index, 2},
  [224] =
    {field_type_params, 1},
    {field_update, 2},
  [226] =
    {field_name, 1},
    {field_negated, 5, .inherited = true},
    {field_parameters, 2},
    {field_pure, 5, .inherited = true},
    {field_return_type, 4},
    {field_specifier, 5},
  [232] =
    {field_negated, 0},
  [233] =
    {field_name, 1},
    {field_parameters, 3},
    {field_return_type, 5},
    {field_type_parameters, 2},
  [237] =
    {field_param_types, 1},
  [238] =
    {field_return_type, 2},
  [239] =
    {field_name, 2},
    {field_parameters, 3},
    {field_return_type, 5},
  [242] =
    {field_name, 2},
    {field_negated, 5, .inherited = true},
    {field_parameters, 4},
    {field_pure, 5, .inherited = true},
    {field_specifier, 5},
    {field_type_parameters, 3},
  [248] =
    {field_bind, 1},
    {field_value, 3},
  [250] =
    {field_bind, 1},
    {field_type, 3},
  [252] =
    {field_access_three, 4},
    {field_access_two, 2},
  [254] =
    {field_parameters, 1},
    {field_return_type, 3},
  [256] =
    {field_condition, 1},
    {field_with, 3},
  [258] =
    {field_emission, 1},
    {field_target, 3},
  [260] =
    {field_name_pattern, 0},
  [261] =
    {field_prop_name, 1, .inherited = true},
    {field_properties, 1},
    {field_properties, 2},
    {field_value, 1, .inherited = true},
  [265] =
    {field_prop_name, 1, .inherited = true},
    {field_prop_name, 2, .inherited = true},
    {field_properties, 1},
    {field_properties, 2},
    {field_value, 1, .inherited = true},
    {field_value, 2, .inherited = true},
  [271] =
    {field_expression, 2},
  [272] =
    {field_expression, 2},
    {field_kind, 1},
  [274] =
    {field_parameters, 1},
    {field_return_type, 3},
    {field_spec_func_name, 0},
  [277] =
    {field_body, 2},
    {field_body, 3},
    {field_body, 5},
    {field_member, 4},
  [281] =
    {field_trigger, 1},
  [282] =
    {field_assertion, 4},
    {field_scope, 0},
    {field_triggers, 2},
  [285] =
    {field_assertion, 4},
    {field_scope, 0},
  [287] =
    {field_condition, 4},
  [288] =
    {field_prop_name, 1, .inherited = true},
    {field_prop_name, 2, .inherited = true},
    {field_property, 1},
    {field_property, 2},
    {field_property, 3},
    {field_value, 1, .inherited = true},
    {field_value, 2, .inherited = true},
  [295] =
    {field_condition, 1},
    {field_else, 4},
    {field_then, 2},
  [298] =
    {field_field, 0},
    {field_value, 2},
  [300] =
    {field_literal_address, 0},
  [301] =
    {field_func, 0},
  [302] =
    {field_name, 1},
    {field_negated, 6, .inherited = true},
    {field_parameters, 3},
    {field_pure, 6, .inherited = true},
    {field_return_type, 5},
    {field_specifier, 6},
    {field_type_parameters, 2},
  [309] =
    {field_member, 4},
    {field_path, 1},
  [311] =
    {field_param_types, 1},
    {field_return_type, 3},
  [313] =
    {field_param_types, 1},
    {field_param_types, 2},
  [315] =
    {field_name, 1},
    {field_type, 3},
    {field_value, 5},
  [318] =
    {field_name, 2},
    {field_negated, 6, .inherited = true},
    {field_parameters, 3},
    {field_pure, 6, .inherited = true},
    {field_return_type, 5},
    {field_specifier, 6},
  [324] =
    {field_name, 2},
    {field_parameters, 4},
    {field_return_type, 6},
    {field_type_parameters, 3},
  [328] =
    {field_condition, 2},
    {field_with, 4},
  [330] =
    {field_emission, 2},
    {field_target, 4},
  [332] =
    {field_scope, 0},
    {field_variable, 1},
  [334] =
    {field_expression, 1},
    {field_targets, 3},
  [336] =
    {field_name_pattern, 1},
    {field_visibility, 0},
  [338] =
    {field_expression, 1},
    {field_name_pattern, 3, .inherited = true},
    {field_targets, 3},
    {field_type_params, 3, .inherited = true},
    {field_visibility, 3, .inherited = true},
  [343] =
    {field_name_pattern, 0},
    {field_type_params, 1},
  [345] =
    {field_prop_name, 1, .inherited = true},
    {field_prop_name, 2, .inherited = true},
    {field_properties, 1},
    {field_properties, 2},
    {field_properties, 3},
    {field_value, 1, .inherited = true},
    {field_value, 2, .inherited = true},
  [352] =
    {field_value, 3},
    {field_variable, 1},
  [354] =
    {field_expression, 3},
    {field_kind, 1},
  [356] =
    {field_parameters, 2},
    {field_return_type, 4},
    {field_spec_func_name, 0},
  [359] =
    {field_assertion, 5},
    {field_scope, 0},
    {field_where, 3},
  [362] =
    {field_trigger, 1},
    {field_trigger, 2, .inherited = true},
  [364] =
    {field_assertion, 5},
    {field_scope, 0},
    {field_triggers, 3},
  [367] =
    {field_value, 2},
  [368] =
    {field_bind, 2},
    {field_field, 0},
    {field_struct, 2, .inherited = true},
    {field_variable, 2, .inherited = true},
  [372] =
    {field_arguments, 5},
    {field_func, 2},
    {field_receiver, 0},
    {field_type_generics, 3},
    {field_type_generics, 4},
  [377] =
    {field_arg, 2, .inherited = true},
    {field_func, 2, .inherited = true},
    {field_literal_address, 2, .inherited = true},
  [380] =
    {field_member, 4},
    {field_member, 5},
    {field_path, 1},
  [383] =
    {field_param_types, 1},
    {field_param_types, 2},
    {field_return_type, 4},
  [386] =
    {field_param_types, 1},
    {field_param_types, 2},
    {field_param_types, 3},
  [389] =
    {field_name, 2},
    {field_negated, 7, .inherited = true},
    {field_parameters, 4},
    {field_pure, 7, .inherited = true},
    {field_return_type, 6},
    {field_specifier, 7},
    {field_type_parameters, 3},
  [396] =
    {field_bind, 1},
    {field_type, 3},
    {field_value, 5},
  [399] =
    {field_access_four, 6},
    {field_access_three, 4},
    {field_access_two, 2},
  [402] =
    {field_value, 4},
    {field_variable, 0},
  [404] =
    {field_condition, 5},
    {field_emission, 1},
    {field_target, 3},
  [407] =
    {field_exclusions, 4},
    {field_expression, 1},
  [409] =
    {field_exclusions, 4},
    {field_expression, 1},
    {field_name_pattern, 4, .inherited = true},
    {field_type_params, 4, .inherited = true},
    {field_visibility, 4, .inherited = true},
  [414] =
    {field_name_pattern, 1},
    {field_type_params, 2},
    {field_visibility, 0},
  [417] =
    {field_expression, 1},
    {field_name_pattern, 3, .inherited = true},
    {field_targets, 3},
    {field_targets, 4},
    {field_type_params, 3, .inherited = true},
    {field_visibility, 3, .inherited = true},
  [423] =
    {field_name_pattern, 1, .inherited = true},
    {field_type_params, 1, .inherited = true},
    {field_visibility, 1, .inherited = true},
  [426] =
    {field_expression, 1},
    {field_name_pattern, 3, .inherited = true},
    {field_name_pattern, 4, .inherited = true},
    {field_targets, 3},
    {field_targets, 4},
    {field_type_params, 3, .inherited = true},
    {field_type_params, 4, .inherited = true},
    {field_visibility, 3, .inherited = true},
    {field_visibility, 4, .inherited = true},
  [435] =
    {field_name_pattern, 0, .inherited = true},
    {field_name_pattern, 1, .inherited = true},
    {field_type_params, 0, .inherited = true},
    {field_type_params, 1, .inherited = true},
    {field_visibility, 0, .inherited = true},
    {field_visibility, 1, .inherited = true},
  [441] =
    {field_post_state, 1},
    {field_value, 4},
    {field_variable, 2},
  [444] =
    {field_assertion, 6},
    {field_scope, 0},
    {field_triggers, 2},
    {field_where, 4},
  [448] =
    {field_assertion, 6},
    {field_scope, 0},
    {field_where, 4},
  [451] =
    {field_arg, 3, .inherited = true},
    {field_func, 3, .inherited = true},
    {field_literal_address, 3, .inherited = true},
  [454] =
    {field_member, 4},
    {field_member, 5},
    {field_member, 6},
    {field_path, 1},
  [458] =
    {field_param_types, 1},
    {field_param_types, 2},
    {field_param_types, 3},
    {field_return_type, 5},
  [462] =
    {field_value, 5},
    {field_variable, 0},
  [464] =
    {field_condition, 6},
    {field_emission, 2},
    {field_target, 4},
  [467] =
    {field_scope, 0},
    {field_value, 5},
    {field_variable, 1},
  [470] =
    {field_exclusions, 5},
    {field_expression, 1},
    {field_targets, 3},
  [473] =
    {field_exclusions, 5},
    {field_expression, 1},
    {field_name_pattern, 5, .inherited = true},
    {field_targets, 3},
    {field_type_params, 5, .inherited = true},
    {field_visibility, 5, .inherited = true},
  [479] =
    {field_exclusions, 4},
    {field_exclusions, 5},
    {field_expression, 1},
    {field_name_pattern, 4, .inherited = true},
    {field_type_params, 4, .inherited = true},
    {field_visibility, 4, .inherited = true},
  [485] =
    {field_exclusions, 4},
    {field_exclusions, 5},
    {field_expression, 1},
    {field_name_pattern, 4, .inherited = true},
    {field_name_pattern, 5, .inherited = true},
    {field_type_params, 4, .inherited = true},
    {field_type_params, 5, .inherited = true},
    {field_visibility, 4, .inherited = true},
    {field_visibility, 5, .inherited = true},
  [494] =
    {field_exclusions, 5},
    {field_expression, 1},
    {field_name_pattern, 3, .inherited = true},
    {field_targets, 3},
    {field_type_params, 3, .inherited = true},
    {field_visibility, 3, .inherited = true},
  [500] =
    {field_exclusions, 5},
    {field_expression, 1},
    {field_name_pattern, 3, .inherited = true},
    {field_name_pattern, 5, .inherited = true},
    {field_targets, 3},
    {field_type_params, 3, .inherited = true},
    {field_type_params, 5, .inherited = true},
    {field_visibility, 3, .inherited = true},
    {field_visibility, 5, .inherited = true},
  [509] =
    {field_expression, 1},
    {field_name_pattern, 3, .inherited = true},
    {field_name_pattern, 4, .inherited = true},
    {field_targets, 3},
    {field_targets, 4},
    {field_targets, 5},
    {field_type_params, 3, .inherited = true},
    {field_type_params, 4, .inherited = true},
    {field_visibility, 3, .inherited = true},
    {field_visibility, 4, .inherited = true},
  [519] =
    {field_assertion, 7},
    {field_scope, 0},
    {field_triggers, 3},
    {field_where, 5},
  [523] =
    {field_arg, 2},
    {field_func, 0},
  [525] =
    {field_scope, 0},
    {field_value, 6},
    {field_variable, 1},
  [528] =
    {field_exclusions, 5},
    {field_exclusions, 6},
    {field_expression, 1},
    {field_name_pattern, 5, .inherited = true},
    {field_targets, 3},
    {field_type_params, 5, .inherited = true},
    {field_visibility, 5, .inherited = true},
  [535] =
    {field_exclusions, 5},
    {field_exclusions, 6},
    {field_expression, 1},
    {field_name_pattern, 5, .inherited = true},
    {field_name_pattern, 6, .inherited = true},
    {field_targets, 3},
    {field_type_params, 5, .inherited = true},
    {field_type_params, 6, .inherited = true},
    {field_visibility, 5, .inherited = true},
    {field_visibility, 6, .inherited = true},
  [545] =
    {field_exclusions, 4},
    {field_exclusions, 5},
    {field_exclusions, 6},
    {field_expression, 1},
    {field_name_pattern, 4, .inherited = true},
    {field_name_pattern, 5, .inherited = true},
    {field_type_params, 4, .inherited = true},
    {field_type_params, 5, .inherited = true},
    {field_visibility, 4, .inherited = true},
    {field_visibility, 5, .inherited = true},
  [555] =
    {field_exclusions, 6},
    {field_expression, 1},
    {field_name_pattern, 3, .inherited = true},
    {field_targets, 3},
    {field_targets, 4},
    {field_type_params, 3, .inherited = true},
    {field_visibility, 3, .inherited = true},
  [562] =
    {field_exclusions, 6},
    {field_expression, 1},
    {field_name_pattern, 3, .inherited = true},
    {field_name_pattern, 6, .inherited = true},
    {field_targets, 3},
    {field_targets, 4},
    {field_type_params, 3, .inherited = true},
    {field_type_params, 6, .inherited = true},
    {field_visibility, 3, .inherited = true},
    {field_visibility, 6, .inherited = true},
  [572] =
    {field_exclusions, 5},
    {field_exclusions, 6},
    {field_expression, 1},
    {field_name_pattern, 3, .inherited = true},
    {field_name_pattern, 5, .inherited = true},
    {field_targets, 3},
    {field_type_params, 3, .inherited = true},
    {field_type_params, 5, .inherited = true},
    {field_visibility, 3, .inherited = true},
    {field_visibility, 5, .inherited = true},
  [582] =
    {field_exclusions, 5},
    {field_exclusions, 6},
    {field_expression, 1},
    {field_name_pattern, 3, .inherited = true},
    {field_name_pattern, 5, .inherited = true},
    {field_name_pattern, 6, .inherited = true},
    {field_targets, 3},
    {field_type_params, 3, .inherited = true},
    {field_type_params, 5, .inherited = true},
    {field_type_params, 6, .inherited = true},
    {field_visibility, 3, .inherited = true},
    {field_visibility, 5, .inherited = true},
    {field_visibility, 6, .inherited = true},
  [595] =
    {field_exclusions, 6},
    {field_expression, 1},
    {field_name_pattern, 3, .inherited = true},
    {field_name_pattern, 4, .inherited = true},
    {field_targets, 3},
    {field_targets, 4},
    {field_type_params, 3, .inherited = true},
    {field_type_params, 4, .inherited = true},
    {field_visibility, 3, .inherited = true},
    {field_visibility, 4, .inherited = true},
  [605] =
    {field_exclusions, 6},
    {field_expression, 1},
    {field_name_pattern, 3, .inherited = true},
    {field_name_pattern, 4, .inherited = true},
    {field_name_pattern, 6, .inherited = true},
    {field_targets, 3},
    {field_targets, 4},
    {field_type_params, 3, .inherited = true},
    {field_type_params, 4, .inherited = true},
    {field_type_params, 6, .inherited = true},
    {field_visibility, 3, .inherited = true},
    {field_visibility, 4, .inherited = true},
    {field_visibility, 6, .inherited = true},
  [618] =
    {field_begin, 4},
    {field_body, 8},
    {field_end, 6},
    {field_var, 2},
  [622] =
    {field_arg, 3},
    {field_func, 0},
  [624] =
    {field_exclusions, 5},
    {field_exclusions, 6},
    {field_exclusions, 7},
    {field_expression, 1},
    {field_name_pattern, 5, .inherited = true},
    {field_name_pattern, 6, .inherited = true},
    {field_targets, 3},
    {field_type_params, 5, .inherited = true},
    {field_type_params, 6, .inherited = true},
    {field_visibility, 5, .inherited = true},
    {field_visibility, 6, .inherited = true},
  [635] =
    {field_exclusions, 6},
    {field_exclusions, 7},
    {field_expression, 1},
    {field_name_pattern, 3, .inherited = true},
    {field_name_pattern, 6, .inherited = true},
    {field_targets, 3},
    {field_targets, 4},
    {field_type_params, 3, .inherited = true},
    {field_type_params, 6, .inherited = true},
    {field_visibility, 3, .inherited = true},
    {field_visibility, 6, .inherited = true},
  [646] =
    {field_exclusions, 6},
    {field_exclusions, 7},
    {field_expression, 1},
    {field_name_pattern, 3, .inherited = true},
    {field_name_pattern, 6, .inherited = true},
    {field_name_pattern, 7, .inherited = true},
    {field_targets, 3},
    {field_targets, 4},
    {field_type_params, 3, .inherited = true},
    {field_type_params, 6, .inherited = true},
    {field_type_params, 7, .inherited = true},
    {field_visibility, 3, .inherited = true},
    {field_visibility, 6, .inherited = true},
    {field_visibility, 7, .inherited = true},
  [660] =
    {field_exclusions, 5},
    {field_exclusions, 6},
    {field_exclusions, 7},
    {field_expression, 1},
    {field_name_pattern, 3, .inherited = true},
    {field_name_pattern, 5, .inherited = true},
    {field_name_pattern, 6, .inherited = true},
    {field_targets, 3},
    {field_type_params, 3, .inherited = true},
    {field_type_params, 5, .inherited = true},
    {field_type_params, 6, .inherited = true},
    {field_visibility, 3, .inherited = true},
    {field_visibility, 5, .inherited = true},
    {field_visibility, 6, .inherited = true},
  [674] =
    {field_exclusions, 7},
    {field_expression, 1},
    {field_name_pattern, 3, .inherited = true},
    {field_name_pattern, 4, .inherited = true},
    {field_targets, 3},
    {field_targets, 4},
    {field_targets, 5},
    {field_type_params, 3, .inherited = true},
    {field_type_params, 4, .inherited = true},
    {field_visibility, 3, .inherited = true},
    {field_visibility, 4, .inherited = true},
  [685] =
    {field_exclusions, 7},
    {field_expression, 1},
    {field_name_pattern, 3, .inherited = true},
    {field_name_pattern, 4, .inherited = true},
    {field_name_pattern, 7, .inherited = true},
    {field_targets, 3},
    {field_targets, 4},
    {field_targets, 5},
    {field_type_params, 3, .inherited = true},
    {field_type_params, 4, .inherited = true},
    {field_type_params, 7, .inherited = true},
    {field_visibility, 3, .inherited = true},
    {field_visibility, 4, .inherited = true},
    {field_visibility, 7, .inherited = true},
  [699] =
    {field_exclusions, 6},
    {field_exclusions, 7},
    {field_expression, 1},
    {field_name_pattern, 3, .inherited = true},
    {field_name_pattern, 4, .inherited = true},
    {field_name_pattern, 6, .inherited = true},
    {field_targets, 3},
    {field_targets, 4},
    {field_type_params, 3, .inherited = true},
    {field_type_params, 4, .inherited = true},
    {field_type_params, 6, .inherited = true},
    {field_visibility, 3, .inherited = true},
    {field_visibility, 4, .inherited = true},
    {field_visibility, 6, .inherited = true},
  [713] =
    {field_exclusions, 6},
    {field_exclusions, 7},
    {field_expression, 1},
    {field_name_pattern, 3, .inherited = true},
    {field_name_pattern, 4, .inherited = true},
    {field_name_pattern, 6, .inherited = true},
    {field_name_pattern, 7, .inherited = true},
    {field_targets, 3},
    {field_targets, 4},
    {field_type_params, 3, .inherited = true},
    {field_type_params, 4, .inherited = true},
    {field_type_params, 6, .inherited = true},
    {field_type_params, 7, .inherited = true},
    {field_visibility, 3, .inherited = true},
    {field_visibility, 4, .inherited = true},
    {field_visibility, 6, .inherited = true},
    {field_visibility, 7, .inherited = true},
  [730] =
    {field_begin, 4},
    {field_body, 9},
    {field_end, 6},
    {field_var, 2},
  [734] =
    {field_exclusions, 6},
    {field_exclusions, 7},
    {field_exclusions, 8},
    {field_expression, 1},
    {field_name_pattern, 3, .inherited = true},
    {field_name_pattern, 6, .inherited = true},
    {field_name_pattern, 7, .inherited = true},
    {field_targets, 3},
    {field_targets, 4},
    {field_type_params, 3, .inherited = true},
    {field_type_params, 6, .inherited = true},
    {field_type_params, 7, .inherited = true},
    {field_visibility, 3, .inherited = true},
    {field_visibility, 6, .inherited = true},
    {field_visibility, 7, .inherited = true},
  [749] =
    {field_exclusions, 7},
    {field_exclusions, 8},
    {field_expression, 1},
    {field_name_pattern, 3, .inherited = true},
    {field_name_pattern, 4, .inherited = true},
    {field_name_pattern, 7, .inherited = true},
    {field_targets, 3},
    {field_targets, 4},
    {field_targets, 5},
    {field_type_params, 3, .inherited = true},
    {field_type_params, 4, .inherited = true},
    {field_type_params, 7, .inherited = true},
    {field_visibility, 3, .inherited = true},
    {field_visibility, 4, .inherited = true},
    {field_visibility, 7, .inherited = true},
  [764] =
    {field_exclusions, 7},
    {field_exclusions, 8},
    {field_expression, 1},
    {field_name_pattern, 3, .inherited = true},
    {field_name_pattern, 4, .inherited = true},
    {field_name_pattern, 7, .inherited = true},
    {field_name_pattern, 8, .inherited = true},
    {field_targets, 3},
    {field_targets, 4},
    {field_targets, 5},
    {field_type_params, 3, .inherited = true},
    {field_type_params, 4, .inherited = true},
    {field_type_params, 7, .inherited = true},
    {field_type_params, 8, .inherited = true},
    {field_visibility, 3, .inherited = true},
    {field_visibility, 4, .inherited = true},
    {field_visibility, 7, .inherited = true},
    {field_visibility, 8, .inherited = true},
  [782] =
    {field_exclusions, 6},
    {field_exclusions, 7},
    {field_exclusions, 8},
    {field_expression, 1},
    {field_name_pattern, 3, .inherited = true},
    {field_name_pattern, 4, .inherited = true},
    {field_name_pattern, 6, .inherited = true},
    {field_name_pattern, 7, .inherited = true},
    {field_targets, 3},
    {field_targets, 4},
    {field_type_params, 3, .inherited = true},
    {field_type_params, 4, .inherited = true},
    {field_type_params, 6, .inherited = true},
    {field_type_params, 7, .inherited = true},
    {field_visibility, 3, .inherited = true},
    {field_visibility, 4, .inherited = true},
    {field_visibility, 6, .inherited = true},
    {field_visibility, 7, .inherited = true},
  [800] =
    {field_exclusions, 7},
    {field_exclusions, 8},
    {field_exclusions, 9},
    {field_expression, 1},
    {field_name_pattern, 3, .inherited = true},
    {field_name_pattern, 4, .inherited = true},
    {field_name_pattern, 7, .inherited = true},
    {field_name_pattern, 8, .inherited = true},
    {field_targets, 3},
    {field_targets, 4},
    {field_targets, 5},
    {field_type_params, 3, .inherited = true},
    {field_type_params, 4, .inherited = true},
    {field_type_params, 7, .inherited = true},
    {field_type_params, 8, .inherited = true},
    {field_visibility, 3, .inherited = true},
    {field_visibility, 4, .inherited = true},
    {field_visibility, 7, .inherited = true},
    {field_visibility, 8, .inherited = true},
};

static const TSSymbol ts_alias_sequences[PRODUCTION_ID_COUNT][MAX_ALIAS_SEQUENCE_LENGTH] = {
  [0] = {0},
  [4] = {
    [2] = sym__doc_line_comment,
  },
  [16] = {
    [1] = alias_sym_body,
  },
  [34] = {
    [1] = sym_lambda_bind_list,
  },
  [40] = {
    [1] = alias_sym_alias,
  },
  [63] = {
    [0] = alias_sym_shorthand_field_identifier,
  },
  [67] = {
    [1] = anon_sym_EQ_EQ_GT,
  },
  [95] = {
    [0] = sym_spec_loop_invariant,
  },
  [195] = {
    [0] = alias_sym_pattern,
    [2] = alias_sym_result,
  },
  [218] = {
    [0] = alias_sym_pattern,
    [2] = alias_sym_condition,
    [4] = alias_sym_result,
  },
};

static const uint16_t ts_non_terminal_alias_map[] = {
  sym_var_name, 2,
    sym_var_name,
    alias_sym_shorthand_field_identifier,
  sym__expr, 3,
    sym__expr,
    alias_sym_condition,
    sym_lambda_bind_list,
  sym__control_body, 2,
    sym__control_body,
    alias_sym_result,
  sym_spec_block, 2,
    sym_spec_block,
    sym_spec_loop_invariant,
  sym_struct_body, 2,
    sym_struct_body,
    alias_sym_body,
  sym_bind_list, 2,
    sym_bind_list,
    alias_sym_pattern,
  0,
};

static const TSStateId ts_primary_state_ids[STATE_COUNT] = {
  [0] = 0,
  [1] = 1,
  [2] = 2,
  [3] = 2,
  [4] = 2,
  [5] = 2,
  [6] = 2,
  [7] = 2,
  [8] = 2,
  [9] = 2,
  [10] = 2,
  [11] = 2,
  [12] = 2,
  [13] = 2,
  [14] = 2,
  [15] = 2,
  [16] = 2,
  [17] = 2,
  [18] = 2,
  [19] = 2,
  [20] = 2,
  [21] = 2,
  [22] = 22,
  [23] = 22,
  [24] = 24,
  [25] = 24,
  [26] = 26,
  [27] = 27,
  [28] = 26,
  [29] = 29,
  [30] = 30,
  [31] = 29,
  [32] = 32,
  [33] = 2,
  [34] = 34,
  [35] = 2,
  [36] = 2,
  [37] = 37,
  [38] = 38,
  [39] = 39,
  [40] = 40,
  [41] = 41,
  [42] = 42,
  [43] = 43,
  [44] = 44,
  [45] = 45,
  [46] = 46,
  [47] = 47,
  [48] = 48,
  [49] = 49,
  [50] = 50,
  [51] = 51,
  [52] = 52,
  [53] = 53,
  [54] = 54,
  [55] = 50,
  [56] = 56,
  [57] = 57,
  [58] = 58,
  [59] = 52,
  [60] = 50,
  [61] = 56,
  [62] = 58,
  [63] = 56,
  [64] = 58,
  [65] = 52,
  [66] = 50,
  [67] = 56,
  [68] = 58,
  [69] = 52,
  [70] = 50,
  [71] = 56,
  [72] = 58,
  [73] = 56,
  [74] = 58,
  [75] = 52,
  [76] = 50,
  [77] = 56,
  [78] = 58,
  [79] = 52,
  [80] = 50,
  [81] = 56,
  [82] = 58,
  [83] = 56,
  [84] = 58,
  [85] = 52,
  [86] = 50,
  [87] = 56,
  [88] = 58,
  [89] = 52,
  [90] = 50,
  [91] = 56,
  [92] = 58,
  [93] = 52,
  [94] = 50,
  [95] = 56,
  [96] = 58,
  [97] = 52,
  [98] = 50,
  [99] = 56,
  [100] = 58,
  [101] = 52,
  [102] = 102,
  [103] = 56,
  [104] = 58,
  [105] = 52,
  [106] = 50,
  [107] = 56,
  [108] = 58,
  [109] = 52,
  [110] = 50,
  [111] = 56,
  [112] = 58,
  [113] = 52,
  [114] = 50,
  [115] = 115,
  [116] = 52,
  [117] = 50,
  [118] = 52,
  [119] = 50,
  [120] = 52,
  [121] = 50,
  [122] = 122,
  [123] = 52,
  [124] = 50,
  [125] = 52,
  [126] = 50,
  [127] = 50,
  [128] = 56,
  [129] = 58,
  [130] = 56,
  [131] = 58,
  [132] = 56,
  [133] = 58,
  [134] = 56,
  [135] = 56,
  [136] = 58,
  [137] = 52,
  [138] = 50,
  [139] = 56,
  [140] = 58,
  [141] = 52,
  [142] = 50,
  [143] = 56,
  [144] = 58,
  [145] = 52,
  [146] = 52,
  [147] = 147,
  [148] = 148,
  [149] = 149,
  [150] = 58,
  [151] = 151,
  [152] = 50,
  [153] = 153,
  [154] = 154,
  [155] = 155,
  [156] = 156,
  [157] = 157,
  [158] = 158,
  [159] = 159,
  [160] = 160,
  [161] = 161,
  [162] = 162,
  [163] = 163,
  [164] = 164,
  [165] = 165,
  [166] = 166,
  [167] = 167,
  [168] = 153,
  [169] = 169,
  [170] = 170,
  [171] = 171,
  [172] = 172,
  [173] = 173,
  [174] = 171,
  [175] = 175,
  [176] = 176,
  [177] = 177,
  [178] = 178,
  [179] = 153,
  [180] = 170,
  [181] = 181,
  [182] = 182,
  [183] = 154,
  [184] = 184,
  [185] = 156,
  [186] = 157,
  [187] = 158,
  [188] = 159,
  [189] = 160,
  [190] = 171,
  [191] = 175,
  [192] = 192,
  [193] = 178,
  [194] = 153,
  [195] = 170,
  [196] = 181,
  [197] = 182,
  [198] = 154,
  [199] = 199,
  [200] = 156,
  [201] = 157,
  [202] = 158,
  [203] = 159,
  [204] = 160,
  [205] = 205,
  [206] = 206,
  [207] = 207,
  [208] = 181,
  [209] = 171,
  [210] = 175,
  [211] = 211,
  [212] = 182,
  [213] = 178,
  [214] = 153,
  [215] = 170,
  [216] = 181,
  [217] = 182,
  [218] = 154,
  [219] = 154,
  [220] = 156,
  [221] = 157,
  [222] = 158,
  [223] = 159,
  [224] = 160,
  [225] = 225,
  [226] = 226,
  [227] = 227,
  [228] = 228,
  [229] = 171,
  [230] = 175,
  [231] = 231,
  [232] = 232,
  [233] = 178,
  [234] = 153,
  [235] = 170,
  [236] = 181,
  [237] = 182,
  [238] = 154,
  [239] = 156,
  [240] = 156,
  [241] = 157,
  [242] = 158,
  [243] = 159,
  [244] = 160,
  [245] = 245,
  [246] = 157,
  [247] = 171,
  [248] = 175,
  [249] = 178,
  [250] = 153,
  [251] = 170,
  [252] = 181,
  [253] = 182,
  [254] = 154,
  [255] = 255,
  [256] = 156,
  [257] = 157,
  [258] = 158,
  [259] = 159,
  [260] = 160,
  [261] = 261,
  [262] = 262,
  [263] = 263,
  [264] = 158,
  [265] = 159,
  [266] = 266,
  [267] = 267,
  [268] = 268,
  [269] = 160,
  [270] = 270,
  [271] = 178,
  [272] = 171,
  [273] = 175,
  [274] = 274,
  [275] = 175,
  [276] = 178,
  [277] = 166,
  [278] = 178,
  [279] = 167,
  [280] = 153,
  [281] = 170,
  [282] = 207,
  [283] = 181,
  [284] = 211,
  [285] = 182,
  [286] = 154,
  [287] = 156,
  [288] = 245,
  [289] = 157,
  [290] = 158,
  [291] = 159,
  [292] = 160,
  [293] = 153,
  [294] = 171,
  [295] = 175,
  [296] = 178,
  [297] = 153,
  [298] = 170,
  [299] = 181,
  [300] = 182,
  [301] = 154,
  [302] = 156,
  [303] = 157,
  [304] = 158,
  [305] = 159,
  [306] = 160,
  [307] = 170,
  [308] = 181,
  [309] = 171,
  [310] = 175,
  [311] = 178,
  [312] = 153,
  [313] = 170,
  [314] = 181,
  [315] = 182,
  [316] = 154,
  [317] = 182,
  [318] = 157,
  [319] = 158,
  [320] = 159,
  [321] = 160,
  [322] = 167,
  [323] = 207,
  [324] = 211,
  [325] = 245,
  [326] = 167,
  [327] = 207,
  [328] = 211,
  [329] = 245,
  [330] = 167,
  [331] = 207,
  [332] = 211,
  [333] = 245,
  [334] = 167,
  [335] = 207,
  [336] = 211,
  [337] = 245,
  [338] = 167,
  [339] = 207,
  [340] = 211,
  [341] = 245,
  [342] = 167,
  [343] = 207,
  [344] = 211,
  [345] = 245,
  [346] = 167,
  [347] = 207,
  [348] = 211,
  [349] = 245,
  [350] = 167,
  [351] = 207,
  [352] = 211,
  [353] = 245,
  [354] = 182,
  [355] = 154,
  [356] = 171,
  [357] = 175,
  [358] = 358,
  [359] = 156,
  [360] = 178,
  [361] = 153,
  [362] = 170,
  [363] = 181,
  [364] = 182,
  [365] = 154,
  [366] = 157,
  [367] = 156,
  [368] = 157,
  [369] = 158,
  [370] = 159,
  [371] = 160,
  [372] = 171,
  [373] = 175,
  [374] = 158,
  [375] = 178,
  [376] = 153,
  [377] = 170,
  [378] = 181,
  [379] = 182,
  [380] = 154,
  [381] = 159,
  [382] = 156,
  [383] = 157,
  [384] = 158,
  [385] = 159,
  [386] = 160,
  [387] = 160,
  [388] = 388,
  [389] = 171,
  [390] = 175,
  [391] = 391,
  [392] = 171,
  [393] = 175,
  [394] = 178,
  [395] = 153,
  [396] = 170,
  [397] = 181,
  [398] = 182,
  [399] = 154,
  [400] = 156,
  [401] = 157,
  [402] = 158,
  [403] = 159,
  [404] = 160,
  [405] = 171,
  [406] = 175,
  [407] = 178,
  [408] = 153,
  [409] = 170,
  [410] = 181,
  [411] = 182,
  [412] = 154,
  [413] = 156,
  [414] = 157,
  [415] = 158,
  [416] = 159,
  [417] = 160,
  [418] = 171,
  [419] = 175,
  [420] = 178,
  [421] = 153,
  [422] = 170,
  [423] = 181,
  [424] = 182,
  [425] = 154,
  [426] = 156,
  [427] = 157,
  [428] = 158,
  [429] = 159,
  [430] = 160,
  [431] = 171,
  [432] = 175,
  [433] = 178,
  [434] = 153,
  [435] = 170,
  [436] = 181,
  [437] = 182,
  [438] = 154,
  [439] = 156,
  [440] = 157,
  [441] = 158,
  [442] = 159,
  [443] = 160,
  [444] = 178,
  [445] = 167,
  [446] = 153,
  [447] = 170,
  [448] = 207,
  [449] = 181,
  [450] = 211,
  [451] = 182,
  [452] = 154,
  [453] = 156,
  [454] = 245,
  [455] = 157,
  [456] = 158,
  [457] = 159,
  [458] = 160,
  [459] = 167,
  [460] = 207,
  [461] = 211,
  [462] = 245,
  [463] = 167,
  [464] = 207,
  [465] = 211,
  [466] = 245,
  [467] = 167,
  [468] = 207,
  [469] = 211,
  [470] = 245,
  [471] = 167,
  [472] = 207,
  [473] = 211,
  [474] = 245,
  [475] = 178,
  [476] = 171,
  [477] = 175,
  [478] = 171,
  [479] = 175,
  [480] = 178,
  [481] = 245,
  [482] = 170,
  [483] = 181,
  [484] = 182,
  [485] = 154,
  [486] = 156,
  [487] = 157,
  [488] = 158,
  [489] = 159,
  [490] = 160,
  [491] = 153,
  [492] = 167,
  [493] = 207,
  [494] = 211,
  [495] = 245,
  [496] = 170,
  [497] = 171,
  [498] = 175,
  [499] = 171,
  [500] = 175,
  [501] = 178,
  [502] = 153,
  [503] = 170,
  [504] = 181,
  [505] = 182,
  [506] = 154,
  [507] = 156,
  [508] = 157,
  [509] = 158,
  [510] = 159,
  [511] = 160,
  [512] = 181,
  [513] = 167,
  [514] = 207,
  [515] = 211,
  [516] = 245,
  [517] = 167,
  [518] = 207,
  [519] = 211,
  [520] = 245,
  [521] = 167,
  [522] = 207,
  [523] = 211,
  [524] = 245,
  [525] = 167,
  [526] = 207,
  [527] = 211,
  [528] = 156,
  [529] = 529,
  [530] = 530,
  [531] = 531,
  [532] = 532,
  [533] = 533,
  [534] = 534,
  [535] = 532,
  [536] = 536,
  [537] = 537,
  [538] = 538,
  [539] = 539,
  [540] = 540,
  [541] = 541,
  [542] = 532,
  [543] = 543,
  [544] = 544,
  [545] = 536,
  [546] = 544,
  [547] = 540,
  [548] = 541,
  [549] = 543,
  [550] = 533,
  [551] = 537,
  [552] = 538,
  [553] = 539,
  [554] = 534,
  [555] = 544,
  [556] = 540,
  [557] = 541,
  [558] = 543,
  [559] = 533,
  [560] = 534,
  [561] = 532,
  [562] = 536,
  [563] = 537,
  [564] = 538,
  [565] = 539,
  [566] = 532,
  [567] = 536,
  [568] = 544,
  [569] = 540,
  [570] = 544,
  [571] = 540,
  [572] = 541,
  [573] = 543,
  [574] = 533,
  [575] = 534,
  [576] = 532,
  [577] = 536,
  [578] = 537,
  [579] = 538,
  [580] = 539,
  [581] = 541,
  [582] = 543,
  [583] = 544,
  [584] = 540,
  [585] = 541,
  [586] = 543,
  [587] = 533,
  [588] = 534,
  [589] = 532,
  [590] = 536,
  [591] = 537,
  [592] = 538,
  [593] = 539,
  [594] = 533,
  [595] = 534,
  [596] = 532,
  [597] = 536,
  [598] = 537,
  [599] = 538,
  [600] = 539,
  [601] = 544,
  [602] = 540,
  [603] = 541,
  [604] = 537,
  [605] = 538,
  [606] = 539,
  [607] = 544,
  [608] = 540,
  [609] = 541,
  [610] = 543,
  [611] = 540,
  [612] = 541,
  [613] = 543,
  [614] = 544,
  [615] = 540,
  [616] = 541,
  [617] = 543,
  [618] = 533,
  [619] = 534,
  [620] = 532,
  [621] = 536,
  [622] = 537,
  [623] = 538,
  [624] = 533,
  [625] = 534,
  [626] = 539,
  [627] = 533,
  [628] = 544,
  [629] = 540,
  [630] = 541,
  [631] = 543,
  [632] = 533,
  [633] = 534,
  [634] = 532,
  [635] = 536,
  [636] = 537,
  [637] = 538,
  [638] = 539,
  [639] = 534,
  [640] = 532,
  [641] = 536,
  [642] = 544,
  [643] = 540,
  [644] = 541,
  [645] = 543,
  [646] = 533,
  [647] = 534,
  [648] = 532,
  [649] = 536,
  [650] = 537,
  [651] = 538,
  [652] = 544,
  [653] = 536,
  [654] = 537,
  [655] = 538,
  [656] = 539,
  [657] = 537,
  [658] = 538,
  [659] = 539,
  [660] = 533,
  [661] = 534,
  [662] = 532,
  [663] = 536,
  [664] = 540,
  [665] = 537,
  [666] = 543,
  [667] = 544,
  [668] = 540,
  [669] = 541,
  [670] = 544,
  [671] = 544,
  [672] = 540,
  [673] = 541,
  [674] = 543,
  [675] = 533,
  [676] = 534,
  [677] = 532,
  [678] = 536,
  [679] = 537,
  [680] = 538,
  [681] = 539,
  [682] = 540,
  [683] = 541,
  [684] = 543,
  [685] = 533,
  [686] = 539,
  [687] = 534,
  [688] = 532,
  [689] = 536,
  [690] = 537,
  [691] = 538,
  [692] = 539,
  [693] = 544,
  [694] = 540,
  [695] = 541,
  [696] = 543,
  [697] = 533,
  [698] = 534,
  [699] = 532,
  [700] = 536,
  [701] = 537,
  [702] = 538,
  [703] = 539,
  [704] = 544,
  [705] = 540,
  [706] = 541,
  [707] = 543,
  [708] = 533,
  [709] = 534,
  [710] = 532,
  [711] = 536,
  [712] = 537,
  [713] = 538,
  [714] = 539,
  [715] = 543,
  [716] = 533,
  [717] = 534,
  [718] = 532,
  [719] = 536,
  [720] = 537,
  [721] = 538,
  [722] = 539,
  [723] = 538,
  [724] = 539,
  [725] = 533,
  [726] = 541,
  [727] = 534,
  [728] = 544,
  [729] = 540,
  [730] = 541,
  [731] = 543,
  [732] = 533,
  [733] = 534,
  [734] = 532,
  [735] = 536,
  [736] = 537,
  [737] = 538,
  [738] = 539,
  [739] = 543,
  [740] = 544,
  [741] = 540,
  [742] = 541,
  [743] = 543,
  [744] = 533,
  [745] = 534,
  [746] = 532,
  [747] = 536,
  [748] = 537,
  [749] = 538,
  [750] = 539,
  [751] = 544,
  [752] = 752,
  [753] = 753,
  [754] = 754,
  [755] = 755,
  [756] = 756,
  [757] = 753,
  [758] = 754,
  [759] = 755,
  [760] = 756,
  [761] = 753,
  [762] = 754,
  [763] = 755,
  [764] = 756,
  [765] = 753,
  [766] = 754,
  [767] = 755,
  [768] = 756,
  [769] = 753,
  [770] = 754,
  [771] = 771,
  [772] = 754,
  [773] = 755,
  [774] = 756,
  [775] = 753,
  [776] = 754,
  [777] = 755,
  [778] = 756,
  [779] = 753,
  [780] = 755,
  [781] = 756,
  [782] = 753,
  [783] = 783,
  [784] = 754,
  [785] = 755,
  [786] = 756,
  [787] = 753,
  [788] = 754,
  [789] = 755,
  [790] = 756,
  [791] = 753,
  [792] = 754,
  [793] = 755,
  [794] = 756,
  [795] = 753,
  [796] = 754,
  [797] = 753,
  [798] = 756,
  [799] = 753,
  [800] = 754,
  [801] = 755,
  [802] = 756,
  [803] = 753,
  [804] = 754,
  [805] = 755,
  [806] = 756,
  [807] = 753,
  [808] = 754,
  [809] = 755,
  [810] = 756,
  [811] = 753,
  [812] = 754,
  [813] = 755,
  [814] = 756,
  [815] = 753,
  [816] = 754,
  [817] = 755,
  [818] = 756,
  [819] = 753,
  [820] = 754,
  [821] = 755,
  [822] = 756,
  [823] = 753,
  [824] = 754,
  [825] = 755,
  [826] = 756,
  [827] = 753,
  [828] = 754,
  [829] = 755,
  [830] = 756,
  [831] = 753,
  [832] = 754,
  [833] = 755,
  [834] = 756,
  [835] = 754,
  [836] = 755,
  [837] = 756,
  [838] = 753,
  [839] = 754,
  [840] = 755,
  [841] = 756,
  [842] = 753,
  [843] = 754,
  [844] = 755,
  [845] = 756,
  [846] = 755,
  [847] = 847,
  [848] = 848,
  [849] = 849,
  [850] = 850,
  [851] = 851,
  [852] = 852,
  [853] = 853,
  [854] = 854,
  [855] = 855,
  [856] = 856,
  [857] = 857,
  [858] = 858,
  [859] = 859,
  [860] = 860,
  [861] = 861,
  [862] = 862,
  [863] = 863,
  [864] = 864,
  [865] = 865,
  [866] = 866,
  [867] = 867,
  [868] = 868,
  [869] = 869,
  [870] = 870,
  [871] = 871,
  [872] = 872,
  [873] = 873,
  [874] = 874,
  [875] = 875,
  [876] = 876,
  [877] = 877,
  [878] = 878,
  [879] = 876,
  [880] = 880,
  [881] = 881,
  [882] = 882,
  [883] = 883,
  [884] = 884,
  [885] = 885,
  [886] = 886,
  [887] = 887,
  [888] = 888,
  [889] = 889,
  [890] = 890,
  [891] = 891,
  [892] = 892,
  [893] = 893,
  [894] = 894,
  [895] = 895,
  [896] = 896,
  [897] = 897,
  [898] = 898,
  [899] = 899,
  [900] = 900,
  [901] = 901,
  [902] = 902,
  [903] = 903,
  [904] = 904,
  [905] = 905,
  [906] = 906,
  [907] = 907,
  [908] = 908,
  [909] = 909,
  [910] = 910,
  [911] = 911,
  [912] = 912,
  [913] = 913,
  [914] = 914,
  [915] = 915,
  [916] = 916,
  [917] = 917,
  [918] = 918,
  [919] = 919,
  [920] = 920,
  [921] = 921,
  [922] = 922,
  [923] = 923,
  [924] = 924,
  [925] = 925,
  [926] = 926,
  [927] = 927,
  [928] = 862,
  [929] = 929,
  [930] = 930,
  [931] = 931,
  [932] = 932,
  [933] = 933,
  [934] = 891,
  [935] = 935,
  [936] = 936,
  [937] = 937,
  [938] = 938,
  [939] = 939,
  [940] = 940,
  [941] = 941,
  [942] = 942,
  [943] = 943,
  [944] = 944,
  [945] = 945,
  [946] = 946,
  [947] = 947,
  [948] = 948,
  [949] = 949,
  [950] = 950,
  [951] = 951,
  [952] = 952,
  [953] = 953,
  [954] = 954,
  [955] = 955,
  [956] = 956,
  [957] = 957,
  [958] = 958,
  [959] = 959,
  [960] = 960,
  [961] = 961,
  [962] = 962,
  [963] = 962,
  [964] = 958,
  [965] = 965,
  [966] = 966,
  [967] = 967,
  [968] = 968,
  [969] = 969,
  [970] = 962,
  [971] = 961,
  [972] = 961,
  [973] = 958,
  [974] = 958,
  [975] = 962,
  [976] = 976,
  [977] = 977,
  [978] = 961,
  [979] = 979,
  [980] = 980,
  [981] = 981,
  [982] = 982,
  [983] = 977,
  [984] = 984,
  [985] = 976,
  [986] = 986,
  [987] = 987,
  [988] = 987,
  [989] = 989,
  [990] = 990,
  [991] = 991,
  [992] = 992,
  [993] = 993,
  [994] = 994,
  [995] = 995,
  [996] = 996,
  [997] = 997,
  [998] = 998,
  [999] = 999,
  [1000] = 992,
  [1001] = 987,
  [1002] = 997,
  [1003] = 990,
  [1004] = 990,
  [1005] = 991,
  [1006] = 986,
  [1007] = 996,
  [1008] = 998,
  [1009] = 994,
  [1010] = 989,
  [1011] = 995,
  [1012] = 999,
  [1013] = 992,
  [1014] = 997,
  [1015] = 991,
  [1016] = 986,
  [1017] = 993,
  [1018] = 996,
  [1019] = 993,
  [1020] = 998,
  [1021] = 994,
  [1022] = 989,
  [1023] = 987,
  [1024] = 995,
  [1025] = 999,
  [1026] = 1026,
  [1027] = 990,
  [1028] = 991,
  [1029] = 986,
  [1030] = 998,
  [1031] = 996,
  [1032] = 891,
  [1033] = 997,
  [1034] = 1034,
  [1035] = 1035,
  [1036] = 994,
  [1037] = 989,
  [1038] = 993,
  [1039] = 992,
  [1040] = 995,
  [1041] = 999,
  [1042] = 1042,
  [1043] = 1043,
  [1044] = 984,
  [1045] = 1045,
  [1046] = 1046,
  [1047] = 1047,
  [1048] = 958,
  [1049] = 958,
  [1050] = 959,
  [1051] = 1051,
  [1052] = 958,
  [1053] = 1053,
  [1054] = 1054,
  [1055] = 962,
  [1056] = 1056,
  [1057] = 1057,
  [1058] = 1058,
  [1059] = 962,
  [1060] = 1060,
  [1061] = 958,
  [1062] = 1062,
  [1063] = 1063,
  [1064] = 1064,
  [1065] = 1065,
  [1066] = 1066,
  [1067] = 1067,
  [1068] = 1068,
  [1069] = 1069,
  [1070] = 962,
  [1071] = 1069,
  [1072] = 1072,
  [1073] = 962,
  [1074] = 961,
  [1075] = 1075,
  [1076] = 1076,
  [1077] = 1077,
  [1078] = 1078,
  [1079] = 1079,
  [1080] = 1080,
  [1081] = 982,
  [1082] = 961,
  [1083] = 1079,
  [1084] = 1080,
  [1085] = 961,
  [1086] = 1076,
  [1087] = 1087,
  [1088] = 1088,
  [1089] = 1089,
  [1090] = 1090,
  [1091] = 1091,
  [1092] = 1092,
  [1093] = 1093,
  [1094] = 1094,
  [1095] = 1095,
  [1096] = 1096,
  [1097] = 1097,
  [1098] = 1098,
  [1099] = 1099,
  [1100] = 1100,
  [1101] = 1101,
  [1102] = 1102,
  [1103] = 1103,
  [1104] = 1104,
  [1105] = 1105,
  [1106] = 961,
  [1107] = 1107,
  [1108] = 1108,
  [1109] = 1109,
  [1110] = 1110,
  [1111] = 1111,
  [1112] = 1112,
  [1113] = 1113,
  [1114] = 1091,
  [1115] = 1115,
  [1116] = 1116,
  [1117] = 1117,
  [1118] = 1118,
  [1119] = 1119,
  [1120] = 1120,
  [1121] = 1121,
  [1122] = 958,
  [1123] = 958,
  [1124] = 1124,
  [1125] = 1125,
  [1126] = 962,
  [1127] = 962,
  [1128] = 1128,
  [1129] = 1129,
  [1130] = 1130,
  [1131] = 958,
  [1132] = 1132,
  [1133] = 958,
  [1134] = 958,
  [1135] = 958,
  [1136] = 998,
  [1137] = 1137,
  [1138] = 990,
  [1139] = 991,
  [1140] = 986,
  [1141] = 996,
  [1142] = 994,
  [1143] = 989,
  [1144] = 995,
  [1145] = 997,
  [1146] = 987,
  [1147] = 993,
  [1148] = 1148,
  [1149] = 999,
  [1150] = 987,
  [1151] = 992,
  [1152] = 962,
  [1153] = 958,
  [1154] = 1137,
  [1155] = 1148,
  [1156] = 958,
  [1157] = 958,
  [1158] = 958,
  [1159] = 961,
  [1160] = 962,
  [1161] = 962,
  [1162] = 962,
  [1163] = 996,
  [1164] = 992,
  [1165] = 997,
  [1166] = 986,
  [1167] = 959,
  [1168] = 990,
  [1169] = 991,
  [1170] = 991,
  [1171] = 986,
  [1172] = 994,
  [1173] = 992,
  [1174] = 958,
  [1175] = 1175,
  [1176] = 1176,
  [1177] = 958,
  [1178] = 962,
  [1179] = 999,
  [1180] = 987,
  [1181] = 993,
  [1182] = 994,
  [1183] = 962,
  [1184] = 993,
  [1185] = 961,
  [1186] = 989,
  [1187] = 1187,
  [1188] = 997,
  [1189] = 998,
  [1190] = 996,
  [1191] = 989,
  [1192] = 995,
  [1193] = 1193,
  [1194] = 987,
  [1195] = 990,
  [1196] = 962,
  [1197] = 961,
  [1198] = 995,
  [1199] = 999,
  [1200] = 961,
  [1201] = 962,
  [1202] = 998,
  [1203] = 962,
  [1204] = 1204,
  [1205] = 990,
  [1206] = 991,
  [1207] = 986,
  [1208] = 996,
  [1209] = 998,
  [1210] = 994,
  [1211] = 989,
  [1212] = 995,
  [1213] = 999,
  [1214] = 992,
  [1215] = 1215,
  [1216] = 997,
  [1217] = 1217,
  [1218] = 1218,
  [1219] = 1219,
  [1220] = 1220,
  [1221] = 993,
  [1222] = 962,
  [1223] = 961,
  [1224] = 961,
  [1225] = 961,
  [1226] = 1226,
  [1227] = 961,
  [1228] = 1228,
  [1229] = 1229,
  [1230] = 1230,
  [1231] = 1231,
  [1232] = 1232,
  [1233] = 1233,
  [1234] = 1234,
  [1235] = 961,
  [1236] = 1236,
  [1237] = 961,
  [1238] = 1238,
  [1239] = 961,
  [1240] = 1234,
  [1241] = 876,
  [1242] = 961,
  [1243] = 982,
  [1244] = 1244,
  [1245] = 1245,
  [1246] = 1246,
  [1247] = 1247,
  [1248] = 1248,
  [1249] = 1249,
  [1250] = 1250,
  [1251] = 1251,
  [1252] = 1252,
  [1253] = 987,
  [1254] = 993,
  [1255] = 994,
  [1256] = 990,
  [1257] = 989,
  [1258] = 991,
  [1259] = 986,
  [1260] = 996,
  [1261] = 990,
  [1262] = 991,
  [1263] = 986,
  [1264] = 996,
  [1265] = 998,
  [1266] = 994,
  [1267] = 989,
  [1268] = 995,
  [1269] = 999,
  [1270] = 992,
  [1271] = 997,
  [1272] = 992,
  [1273] = 993,
  [1274] = 987,
  [1275] = 987,
  [1276] = 997,
  [1277] = 987,
  [1278] = 995,
  [1279] = 529,
  [1280] = 531,
  [1281] = 530,
  [1282] = 998,
  [1283] = 999,
  [1284] = 996,
  [1285] = 993,
  [1286] = 990,
  [1287] = 991,
  [1288] = 986,
  [1289] = 996,
  [1290] = 998,
  [1291] = 994,
  [1292] = 989,
  [1293] = 995,
  [1294] = 999,
  [1295] = 992,
  [1296] = 997,
  [1297] = 990,
  [1298] = 991,
  [1299] = 986,
  [1300] = 993,
  [1301] = 998,
  [1302] = 994,
  [1303] = 989,
  [1304] = 995,
  [1305] = 999,
  [1306] = 992,
  [1307] = 997,
  [1308] = 987,
  [1309] = 990,
  [1310] = 991,
  [1311] = 986,
  [1312] = 996,
  [1313] = 998,
  [1314] = 994,
  [1315] = 989,
  [1316] = 995,
  [1317] = 999,
  [1318] = 992,
  [1319] = 987,
  [1320] = 993,
  [1321] = 997,
  [1322] = 990,
  [1323] = 991,
  [1324] = 986,
  [1325] = 996,
  [1326] = 998,
  [1327] = 994,
  [1328] = 989,
  [1329] = 995,
  [1330] = 999,
  [1331] = 992,
  [1332] = 997,
  [1333] = 993,
  [1334] = 987,
  [1335] = 987,
  [1336] = 991,
  [1337] = 990,
  [1338] = 991,
  [1339] = 986,
  [1340] = 996,
  [1341] = 998,
  [1342] = 994,
  [1343] = 989,
  [1344] = 995,
  [1345] = 990,
  [1346] = 986,
  [1347] = 996,
  [1348] = 998,
  [1349] = 994,
  [1350] = 989,
  [1351] = 995,
  [1352] = 997,
  [1353] = 992,
  [1354] = 999,
  [1355] = 992,
  [1356] = 997,
  [1357] = 990,
  [1358] = 991,
  [1359] = 986,
  [1360] = 996,
  [1361] = 998,
  [1362] = 994,
  [1363] = 989,
  [1364] = 995,
  [1365] = 987,
  [1366] = 999,
  [1367] = 992,
  [1368] = 993,
  [1369] = 997,
  [1370] = 987,
  [1371] = 993,
  [1372] = 993,
  [1373] = 999,
  [1374] = 996,
  [1375] = 1375,
  [1376] = 891,
  [1377] = 998,
  [1378] = 994,
  [1379] = 989,
  [1380] = 1380,
  [1381] = 995,
  [1382] = 999,
  [1383] = 992,
  [1384] = 990,
  [1385] = 991,
  [1386] = 986,
  [1387] = 996,
  [1388] = 998,
  [1389] = 994,
  [1390] = 989,
  [1391] = 995,
  [1392] = 987,
  [1393] = 997,
  [1394] = 999,
  [1395] = 992,
  [1396] = 987,
  [1397] = 997,
  [1398] = 993,
  [1399] = 1399,
  [1400] = 993,
  [1401] = 1401,
  [1402] = 991,
  [1403] = 986,
  [1404] = 990,
  [1405] = 997,
  [1406] = 991,
  [1407] = 986,
  [1408] = 995,
  [1409] = 993,
  [1410] = 891,
  [1411] = 999,
  [1412] = 996,
  [1413] = 1413,
  [1414] = 998,
  [1415] = 1415,
  [1416] = 992,
  [1417] = 1417,
  [1418] = 891,
  [1419] = 990,
  [1420] = 994,
  [1421] = 989,
  [1422] = 1422,
  [1423] = 1423,
  [1424] = 1424,
  [1425] = 1425,
  [1426] = 1426,
  [1427] = 1427,
  [1428] = 1428,
  [1429] = 1429,
  [1430] = 1430,
  [1431] = 1431,
  [1432] = 1432,
  [1433] = 1433,
  [1434] = 1043,
  [1435] = 960,
  [1436] = 965,
  [1437] = 1437,
  [1438] = 968,
  [1439] = 957,
  [1440] = 1045,
  [1441] = 1441,
  [1442] = 967,
  [1443] = 966,
  [1444] = 969,
  [1445] = 1445,
  [1446] = 1446,
  [1447] = 1447,
  [1448] = 1448,
  [1449] = 1449,
  [1450] = 1450,
  [1451] = 1451,
  [1452] = 1452,
  [1453] = 1453,
  [1454] = 1454,
  [1455] = 1455,
  [1456] = 1456,
  [1457] = 1457,
  [1458] = 1458,
  [1459] = 1459,
  [1460] = 1460,
  [1461] = 1461,
  [1462] = 1462,
  [1463] = 1463,
  [1464] = 1464,
  [1465] = 1465,
  [1466] = 1466,
  [1467] = 1467,
  [1468] = 1468,
  [1469] = 1469,
  [1470] = 1470,
  [1471] = 1471,
  [1472] = 1472,
  [1473] = 1473,
  [1474] = 1474,
  [1475] = 1475,
  [1476] = 1476,
  [1477] = 1477,
  [1478] = 1478,
  [1479] = 1479,
  [1480] = 1480,
  [1481] = 1481,
  [1482] = 1482,
  [1483] = 1483,
  [1484] = 1484,
  [1485] = 1485,
  [1486] = 1486,
  [1487] = 1487,
  [1488] = 1488,
  [1489] = 1489,
  [1490] = 1490,
  [1491] = 1491,
  [1492] = 1492,
  [1493] = 1493,
  [1494] = 1494,
  [1495] = 1495,
  [1496] = 1496,
  [1497] = 1497,
  [1498] = 1498,
  [1499] = 1499,
  [1500] = 1500,
  [1501] = 1501,
  [1502] = 1502,
  [1503] = 1503,
  [1504] = 850,
  [1505] = 1505,
  [1506] = 1506,
  [1507] = 1507,
  [1508] = 1508,
  [1509] = 1509,
  [1510] = 848,
  [1511] = 1511,
  [1512] = 852,
  [1513] = 1513,
  [1514] = 854,
  [1515] = 1515,
  [1516] = 1516,
  [1517] = 1517,
  [1518] = 1518,
  [1519] = 1519,
  [1520] = 1520,
  [1521] = 1521,
  [1522] = 1522,
  [1523] = 1523,
  [1524] = 862,
  [1525] = 863,
  [1526] = 859,
  [1527] = 860,
  [1528] = 861,
  [1529] = 867,
  [1530] = 1530,
  [1531] = 865,
  [1532] = 1532,
  [1533] = 1533,
  [1534] = 984,
  [1535] = 1535,
  [1536] = 1536,
  [1537] = 864,
  [1538] = 868,
  [1539] = 866,
  [1540] = 1540,
  [1541] = 1541,
  [1542] = 1542,
  [1543] = 1543,
  [1544] = 1544,
  [1545] = 1545,
  [1546] = 872,
  [1547] = 1547,
  [1548] = 1548,
  [1549] = 869,
  [1550] = 871,
  [1551] = 870,
  [1552] = 1552,
  [1553] = 1553,
  [1554] = 1521,
  [1555] = 1555,
  [1556] = 1556,
  [1557] = 1557,
  [1558] = 1558,
  [1559] = 1559,
  [1560] = 1560,
  [1561] = 1561,
  [1562] = 1562,
  [1563] = 1563,
  [1564] = 1533,
  [1565] = 1565,
  [1566] = 1566,
  [1567] = 1567,
  [1568] = 1568,
  [1569] = 1569,
  [1570] = 1570,
  [1571] = 1571,
  [1572] = 1572,
  [1573] = 1573,
  [1574] = 1574,
  [1575] = 1575,
  [1576] = 1576,
  [1577] = 1577,
  [1578] = 1578,
  [1579] = 1579,
  [1580] = 1580,
  [1581] = 1581,
  [1582] = 1582,
  [1583] = 1583,
  [1584] = 1584,
  [1585] = 1585,
  [1586] = 1586,
  [1587] = 1587,
  [1588] = 1588,
  [1589] = 1589,
  [1590] = 1590,
  [1591] = 1591,
  [1592] = 1592,
  [1593] = 1593,
  [1594] = 1594,
  [1595] = 1595,
  [1596] = 1596,
  [1597] = 1597,
  [1598] = 1598,
  [1599] = 961,
  [1600] = 1600,
  [1601] = 1601,
  [1602] = 1602,
  [1603] = 1603,
  [1604] = 1604,
  [1605] = 1605,
  [1606] = 1606,
  [1607] = 1607,
  [1608] = 1608,
  [1609] = 1609,
  [1610] = 1610,
  [1611] = 1611,
  [1612] = 1612,
  [1613] = 1613,
  [1614] = 1614,
  [1615] = 1615,
  [1616] = 1616,
  [1617] = 1617,
  [1618] = 1618,
  [1619] = 1618,
  [1620] = 1620,
  [1621] = 1621,
  [1622] = 1622,
  [1623] = 1623,
  [1624] = 1624,
  [1625] = 1625,
  [1626] = 961,
  [1627] = 1627,
  [1628] = 1628,
  [1629] = 1629,
  [1630] = 961,
  [1631] = 1631,
  [1632] = 1632,
  [1633] = 1633,
  [1634] = 1634,
  [1635] = 1635,
  [1636] = 1636,
  [1637] = 1637,
  [1638] = 1638,
  [1639] = 1639,
  [1640] = 1640,
  [1641] = 1641,
  [1642] = 1642,
  [1643] = 1643,
  [1644] = 1644,
  [1645] = 1645,
  [1646] = 1646,
  [1647] = 1647,
  [1648] = 1648,
  [1649] = 1649,
  [1650] = 1650,
  [1651] = 1651,
  [1652] = 1652,
  [1653] = 1653,
  [1654] = 1654,
  [1655] = 1655,
  [1656] = 1656,
  [1657] = 1657,
  [1658] = 1658,
  [1659] = 1659,
  [1660] = 1660,
  [1661] = 1661,
  [1662] = 1662,
  [1663] = 1663,
  [1664] = 1664,
  [1665] = 1665,
  [1666] = 1666,
  [1667] = 1667,
  [1668] = 1668,
  [1669] = 1579,
  [1670] = 1670,
  [1671] = 1671,
  [1672] = 1672,
  [1673] = 1673,
  [1674] = 1674,
  [1675] = 1675,
  [1676] = 1676,
  [1677] = 1677,
  [1678] = 1678,
  [1679] = 1679,
  [1680] = 1680,
  [1681] = 1681,
  [1682] = 1682,
  [1683] = 1683,
  [1684] = 1684,
  [1685] = 1685,
  [1686] = 1686,
  [1687] = 1687,
  [1688] = 1670,
  [1689] = 1689,
  [1690] = 1690,
  [1691] = 1691,
  [1692] = 1692,
  [1693] = 1693,
  [1694] = 1694,
  [1695] = 1695,
  [1696] = 1696,
  [1697] = 1697,
  [1698] = 1695,
  [1699] = 1697,
  [1700] = 1695,
  [1701] = 1701,
  [1702] = 1702,
  [1703] = 1703,
  [1704] = 1697,
  [1705] = 1705,
  [1706] = 1695,
  [1707] = 1695,
  [1708] = 1708,
  [1709] = 1697,
  [1710] = 1695,
  [1711] = 1711,
  [1712] = 1712,
  [1713] = 1713,
  [1714] = 1697,
  [1715] = 1695,
  [1716] = 1716,
  [1717] = 1717,
  [1718] = 1718,
  [1719] = 1697,
  [1720] = 1697,
  [1721] = 1695,
  [1722] = 1722,
  [1723] = 1697,
  [1724] = 1697,
  [1725] = 1695,
  [1726] = 1697,
  [1727] = 1697,
  [1728] = 1695,
  [1729] = 1695,
  [1730] = 1697,
  [1731] = 1695,
  [1732] = 1697,
  [1733] = 1695,
  [1734] = 1697,
  [1735] = 1735,
  [1736] = 1736,
  [1737] = 1697,
  [1738] = 1697,
  [1739] = 1695,
  [1740] = 1697,
  [1741] = 1695,
  [1742] = 1695,
  [1743] = 1697,
  [1744] = 1744,
  [1745] = 1697,
  [1746] = 1695,
  [1747] = 1695,
  [1748] = 1697,
  [1749] = 1695,
  [1750] = 1695,
  [1751] = 1751,
  [1752] = 1752,
  [1753] = 1753,
  [1754] = 1754,
  [1755] = 1755,
  [1756] = 1756,
  [1757] = 1757,
  [1758] = 1758,
  [1759] = 1595,
  [1760] = 1760,
  [1761] = 1761,
  [1762] = 1762,
  [1763] = 1763,
  [1764] = 1764,
  [1765] = 1765,
  [1766] = 1766,
  [1767] = 1767,
  [1768] = 1768,
  [1769] = 1769,
  [1770] = 1770,
  [1771] = 1771,
  [1772] = 1772,
  [1773] = 1773,
  [1774] = 1774,
  [1775] = 1775,
  [1776] = 1776,
  [1777] = 1777,
  [1778] = 1778,
  [1779] = 1779,
  [1780] = 1780,
  [1781] = 1781,
  [1782] = 1782,
  [1783] = 1783,
  [1784] = 1784,
  [1785] = 1785,
  [1786] = 1786,
  [1787] = 993,
  [1788] = 1788,
  [1789] = 1789,
  [1790] = 1790,
  [1791] = 1791,
  [1792] = 1792,
  [1793] = 1793,
  [1794] = 1794,
  [1795] = 1795,
  [1796] = 993,
  [1797] = 1797,
  [1798] = 1798,
  [1799] = 993,
  [1800] = 1800,
  [1801] = 1801,
  [1802] = 1802,
  [1803] = 1803,
  [1804] = 1804,
  [1805] = 1805,
  [1806] = 1806,
  [1807] = 1807,
  [1808] = 1808,
  [1809] = 1809,
  [1810] = 1810,
  [1811] = 1811,
  [1812] = 1812,
  [1813] = 1813,
  [1814] = 1814,
  [1815] = 1815,
  [1816] = 1816,
  [1817] = 1817,
  [1818] = 1818,
  [1819] = 1819,
  [1820] = 1820,
  [1821] = 1821,
  [1822] = 1822,
  [1823] = 1823,
  [1824] = 1824,
  [1825] = 1825,
  [1826] = 1826,
  [1827] = 1827,
  [1828] = 1828,
  [1829] = 1829,
  [1830] = 1830,
  [1831] = 1831,
  [1832] = 1832,
  [1833] = 1833,
  [1834] = 1834,
  [1835] = 1835,
  [1836] = 1836,
  [1837] = 1834,
  [1838] = 1838,
  [1839] = 1839,
  [1840] = 1840,
  [1841] = 1841,
  [1842] = 1842,
  [1843] = 1843,
  [1844] = 1844,
  [1845] = 1845,
  [1846] = 1846,
  [1847] = 1847,
  [1848] = 1795,
  [1849] = 1849,
  [1850] = 1850,
  [1851] = 1851,
  [1852] = 1852,
  [1853] = 1853,
  [1854] = 1854,
  [1855] = 1855,
  [1856] = 1856,
  [1857] = 1857,
  [1858] = 1858,
  [1859] = 1859,
  [1860] = 1860,
  [1861] = 1861,
  [1862] = 1862,
  [1863] = 1863,
  [1864] = 1864,
  [1865] = 1865,
  [1866] = 1866,
  [1867] = 1867,
  [1868] = 1868,
  [1869] = 1811,
  [1870] = 1870,
  [1871] = 1871,
  [1872] = 1872,
  [1873] = 1873,
  [1874] = 1874,
  [1875] = 1875,
  [1876] = 1876,
  [1877] = 1877,
  [1878] = 1878,
  [1879] = 1879,
  [1880] = 1880,
  [1881] = 1881,
  [1882] = 1882,
  [1883] = 1883,
  [1884] = 1884,
  [1885] = 1885,
  [1886] = 1886,
  [1887] = 1887,
  [1888] = 1888,
  [1889] = 1889,
  [1890] = 1890,
  [1891] = 1891,
  [1892] = 1892,
  [1893] = 1893,
  [1894] = 1894,
  [1895] = 1895,
  [1896] = 1896,
  [1897] = 1866,
  [1898] = 1898,
  [1899] = 1899,
  [1900] = 1900,
  [1901] = 1901,
  [1902] = 1902,
  [1903] = 1903,
  [1904] = 1904,
  [1905] = 1898,
  [1906] = 1906,
  [1907] = 1907,
  [1908] = 1908,
  [1909] = 1909,
  [1910] = 1910,
  [1911] = 1911,
  [1912] = 1912,
  [1913] = 1913,
  [1914] = 1914,
  [1915] = 1883,
  [1916] = 1916,
  [1917] = 1917,
  [1918] = 1918,
  [1919] = 1919,
  [1920] = 1920,
  [1921] = 1862,
  [1922] = 1922,
  [1923] = 1887,
  [1924] = 1924,
  [1925] = 1925,
  [1926] = 1926,
  [1927] = 1927,
  [1928] = 1928,
  [1929] = 1898,
  [1930] = 1919,
  [1931] = 1931,
  [1932] = 1932,
  [1933] = 1933,
  [1934] = 1934,
  [1935] = 1935,
  [1936] = 1936,
  [1937] = 1937,
  [1938] = 1938,
  [1939] = 1939,
  [1940] = 1940,
  [1941] = 1941,
  [1942] = 1942,
  [1943] = 1943,
  [1944] = 1944,
  [1945] = 1945,
  [1946] = 1946,
  [1947] = 1947,
  [1948] = 1948,
  [1949] = 1949,
  [1950] = 1950,
  [1951] = 1951,
  [1952] = 1952,
  [1953] = 1953,
  [1954] = 1954,
  [1955] = 1863,
  [1956] = 1956,
  [1957] = 1957,
  [1958] = 1958,
  [1959] = 1959,
  [1960] = 1960,
  [1961] = 1961,
  [1962] = 1962,
  [1963] = 1963,
  [1964] = 1928,
  [1965] = 1965,
  [1966] = 1966,
  [1967] = 1967,
  [1968] = 1968,
  [1969] = 1969,
  [1970] = 1970,
  [1971] = 1971,
  [1972] = 1972,
  [1973] = 1973,
  [1974] = 1974,
  [1975] = 1867,
  [1976] = 1976,
  [1977] = 1977,
  [1978] = 1978,
  [1979] = 1979,
  [1980] = 1980,
  [1981] = 1981,
  [1982] = 1982,
  [1983] = 1983,
  [1984] = 1984,
  [1985] = 1985,
  [1986] = 1986,
  [1987] = 1987,
  [1988] = 1988,
  [1989] = 1989,
  [1990] = 1990,
  [1991] = 1895,
  [1992] = 1992,
  [1993] = 1993,
  [1994] = 1994,
  [1995] = 1995,
  [1996] = 1996,
  [1997] = 1997,
  [1998] = 1998,
  [1999] = 1864,
  [2000] = 2000,
  [2001] = 2001,
  [2002] = 2002,
  [2003] = 2003,
  [2004] = 2004,
  [2005] = 2005,
  [2006] = 2005,
  [2007] = 2007,
  [2008] = 2008,
  [2009] = 2009,
  [2010] = 2010,
  [2011] = 2005,
  [2012] = 2012,
  [2013] = 2013,
  [2014] = 2014,
  [2015] = 2015,
  [2016] = 2016,
  [2017] = 2017,
  [2018] = 2018,
  [2019] = 2019,
  [2020] = 2020,
  [2021] = 2021,
  [2022] = 2016,
  [2023] = 2023,
  [2024] = 2024,
  [2025] = 2005,
  [2026] = 2026,
  [2027] = 2016,
  [2028] = 2028,
  [2029] = 2029,
  [2030] = 2030,
  [2031] = 2029,
  [2032] = 2029,
  [2033] = 2033,
  [2034] = 2034,
  [2035] = 2035,
  [2036] = 2005,
  [2037] = 2016,
  [2038] = 2038,
  [2039] = 2039,
  [2040] = 2040,
  [2041] = 2041,
  [2042] = 2042,
  [2043] = 2018,
  [2044] = 2004,
  [2045] = 2034,
  [2046] = 2046,
  [2047] = 2047,
  [2048] = 2005,
  [2049] = 2049,
  [2050] = 2050,
  [2051] = 2029,
  [2052] = 2052,
  [2053] = 2053,
  [2054] = 2054,
  [2055] = 2055,
  [2056] = 2005,
  [2057] = 2057,
  [2058] = 2058,
  [2059] = 2016,
  [2060] = 2029,
  [2061] = 2029,
  [2062] = 2016,
  [2063] = 2063,
  [2064] = 2064,
  [2065] = 2065,
  [2066] = 2004,
  [2067] = 2005,
  [2068] = 2068,
  [2069] = 2016,
  [2070] = 2029,
  [2071] = 2071,
  [2072] = 2005,
  [2073] = 2073,
  [2074] = 2074,
  [2075] = 2075,
  [2076] = 2076,
  [2077] = 2029,
  [2078] = 2078,
  [2079] = 2016,
  [2080] = 2029,
  [2081] = 2081,
  [2082] = 2082,
  [2083] = 2083,
  [2084] = 2005,
  [2085] = 2085,
  [2086] = 2016,
  [2087] = 2029,
  [2088] = 2088,
  [2089] = 2016,
  [2090] = 2090,
  [2091] = 2091,
  [2092] = 2092,
  [2093] = 2093,
  [2094] = 2093,
  [2095] = 2016,
  [2096] = 2029,
  [2097] = 2097,
  [2098] = 2005,
  [2099] = 2029,
  [2100] = 2016,
  [2101] = 2038,
  [2102] = 2102,
  [2103] = 2004,
  [2104] = 2104,
  [2105] = 2105,
  [2106] = 2016,
  [2107] = 2005,
  [2108] = 2029,
  [2109] = 2109,
  [2110] = 2110,
  [2111] = 2016,
  [2112] = 2112,
  [2113] = 2029,
  [2114] = 2005,
  [2115] = 2004,
  [2116] = 2017,
  [2117] = 2078,
  [2118] = 2004,
  [2119] = 2119,
  [2120] = 2005,
  [2121] = 2005,
  [2122] = 2004,
  [2123] = 2004,
  [2124] = 2124,
  [2125] = 2004,
  [2126] = 2126,
  [2127] = 2127,
  [2128] = 2016,
  [2129] = 2005,
  [2130] = 2016,
  [2131] = 2004,
  [2132] = 2029,
  [2133] = 2133,
  [2134] = 2029,
  [2135] = 2135,
  [2136] = 2136,
  [2137] = 2016,
  [2138] = 2138,
  [2139] = 2029,
  [2140] = 2004,
  [2141] = 2141,
  [2142] = 2004,
  [2143] = 2005,
  [2144] = 2005,
  [2145] = 2005,
  [2146] = 2005,
  [2147] = 2147,
  [2148] = 2148,
  [2149] = 2004,
  [2150] = 2150,
  [2151] = 2016,
  [2152] = 2029,
  [2153] = 2153,
  [2154] = 2004,
  [2155] = 2004,
  [2156] = 2156,
  [2157] = 2016,
  [2158] = 2005,
  [2159] = 2005,
  [2160] = 2004,
  [2161] = 2029,
  [2162] = 2016,
  [2163] = 2029,
  [2164] = 2005,
  [2165] = 2004,
  [2166] = 2004,
  [2167] = 2167,
  [2168] = 2004,
  [2169] = 2004,
  [2170] = 2017,
  [2171] = 2171,
  [2172] = 2172,
  [2173] = 2172,
  [2174] = 2174,
  [2175] = 2175,
  [2176] = 2176,
  [2177] = 2177,
  [2178] = 2178,
  [2179] = 2172,
  [2180] = 2180,
  [2181] = 2181,
  [2182] = 2182,
  [2183] = 2180,
  [2184] = 2184,
  [2185] = 2185,
  [2186] = 2186,
  [2187] = 2187,
  [2188] = 2188,
  [2189] = 2189,
  [2190] = 2190,
  [2191] = 2191,
  [2192] = 2192,
  [2193] = 2193,
  [2194] = 2194,
  [2195] = 2195,
  [2196] = 2196,
  [2197] = 2197,
  [2198] = 2198,
  [2199] = 2199,
  [2200] = 2172,
  [2201] = 2201,
  [2202] = 2202,
  [2203] = 2172,
  [2204] = 2177,
  [2205] = 2178,
  [2206] = 2206,
  [2207] = 2181,
  [2208] = 2182,
  [2209] = 2180,
  [2210] = 2210,
  [2211] = 2211,
  [2212] = 2212,
  [2213] = 2213,
  [2214] = 2214,
  [2215] = 2215,
  [2216] = 2216,
  [2217] = 2217,
  [2218] = 2218,
  [2219] = 2219,
  [2220] = 2220,
  [2221] = 2221,
  [2222] = 2222,
  [2223] = 2223,
  [2224] = 2172,
  [2225] = 2225,
  [2226] = 2226,
  [2227] = 2177,
  [2228] = 2178,
  [2229] = 2229,
  [2230] = 2181,
  [2231] = 2182,
  [2232] = 2180,
  [2233] = 2233,
  [2234] = 2172,
  [2235] = 2235,
  [2236] = 2236,
  [2237] = 2237,
  [2238] = 2238,
  [2239] = 2193,
  [2240] = 2240,
  [2241] = 2241,
  [2242] = 2242,
  [2243] = 2243,
  [2244] = 2244,
  [2245] = 2177,
  [2246] = 2177,
  [2247] = 2247,
  [2248] = 2172,
  [2249] = 2249,
  [2250] = 2250,
  [2251] = 2177,
  [2252] = 2178,
  [2253] = 2178,
  [2254] = 2181,
  [2255] = 2182,
  [2256] = 2180,
  [2257] = 2178,
  [2258] = 2181,
  [2259] = 2172,
  [2260] = 2260,
  [2261] = 2181,
  [2262] = 2182,
  [2263] = 2180,
  [2264] = 2222,
  [2265] = 2265,
  [2266] = 2177,
  [2267] = 2267,
  [2268] = 2177,
  [2269] = 2269,
  [2270] = 2270,
  [2271] = 2271,
  [2272] = 2272,
  [2273] = 2273,
  [2274] = 2274,
  [2275] = 2275,
  [2276] = 2276,
  [2277] = 2277,
  [2278] = 2278,
  [2279] = 2279,
  [2280] = 2280,
  [2281] = 2281,
  [2282] = 2282,
  [2283] = 2283,
  [2284] = 2212,
  [2285] = 2172,
  [2286] = 2185,
  [2287] = 2178,
  [2288] = 2288,
  [2289] = 2289,
  [2290] = 2290,
  [2291] = 2177,
  [2292] = 2292,
  [2293] = 2293,
  [2294] = 2178,
  [2295] = 2182,
  [2296] = 2181,
  [2297] = 2182,
  [2298] = 2180,
  [2299] = 2299,
  [2300] = 2177,
  [2301] = 2301,
  [2302] = 2302,
  [2303] = 2172,
  [2304] = 2177,
  [2305] = 2305,
  [2306] = 2306,
  [2307] = 2307,
  [2308] = 2308,
  [2309] = 2309,
  [2310] = 2177,
  [2311] = 2178,
  [2312] = 2172,
  [2313] = 2181,
  [2314] = 2314,
  [2315] = 2177,
  [2316] = 2316,
  [2317] = 2317,
  [2318] = 2182,
  [2319] = 2180,
  [2320] = 2178,
  [2321] = 2321,
  [2322] = 2322,
  [2323] = 2323,
  [2324] = 2180,
  [2325] = 2325,
  [2326] = 2178,
  [2327] = 2327,
  [2328] = 2328,
  [2329] = 2172,
  [2330] = 2330,
  [2331] = 2189,
  [2332] = 2332,
  [2333] = 2333,
  [2334] = 2334,
  [2335] = 2181,
  [2336] = 2336,
  [2337] = 2181,
  [2338] = 2172,
  [2339] = 2182,
  [2340] = 2340,
  [2341] = 2341,
  [2342] = 2175,
  [2343] = 2343,
  [2344] = 2344,
  [2345] = 2182,
  [2346] = 2346,
  [2347] = 2347,
  [2348] = 2348,
  [2349] = 2349,
  [2350] = 2172,
  [2351] = 2351,
  [2352] = 2176,
  [2353] = 2177,
  [2354] = 2178,
  [2355] = 2178,
  [2356] = 2181,
  [2357] = 2182,
  [2358] = 2180,
  [2359] = 2177,
  [2360] = 2360,
  [2361] = 2361,
  [2362] = 2362,
  [2363] = 2363,
  [2364] = 2364,
  [2365] = 2178,
  [2366] = 2177,
  [2367] = 2181,
  [2368] = 2182,
  [2369] = 2281,
  [2370] = 2370,
  [2371] = 2371,
  [2372] = 2180,
  [2373] = 2373,
  [2374] = 2172,
  [2375] = 2375,
  [2376] = 2181,
  [2377] = 2177,
  [2378] = 2178,
  [2379] = 2180,
  [2380] = 2181,
  [2381] = 2182,
  [2382] = 2180,
  [2383] = 2383,
  [2384] = 2384,
  [2385] = 2385,
  [2386] = 2386,
  [2387] = 2341,
  [2388] = 2388,
  [2389] = 2389,
  [2390] = 2289,
  [2391] = 2182,
  [2392] = 2392,
  [2393] = 2172,
  [2394] = 2394,
  [2395] = 2180,
  [2396] = 2363,
  [2397] = 2397,
  [2398] = 2398,
  [2399] = 2178,
  [2400] = 2400,
  [2401] = 2401,
  [2402] = 2402,
  [2403] = 2181,
  [2404] = 2404,
  [2405] = 2172,
  [2406] = 2182,
  [2407] = 2407,
  [2408] = 2180,
  [2409] = 2172,
  [2410] = 2410,
  [2411] = 2411,
  [2412] = 2177,
  [2413] = 2178,
  [2414] = 2414,
  [2415] = 2181,
  [2416] = 2182,
  [2417] = 2180,
  [2418] = 2418,
  [2419] = 2419,
  [2420] = 2181,
  [2421] = 2226,
  [2422] = 2182,
  [2423] = 2423,
  [2424] = 2424,
  [2425] = 2177,
  [2426] = 2275,
  [2427] = 2427,
  [2428] = 2178,
  [2429] = 2429,
  [2430] = 2278,
  [2431] = 2172,
  [2432] = 2432,
  [2433] = 2433,
  [2434] = 2434,
  [2435] = 2177,
  [2436] = 2181,
  [2437] = 2178,
  [2438] = 2182,
  [2439] = 2180,
  [2440] = 2440,
  [2441] = 2181,
  [2442] = 2182,
  [2443] = 2443,
  [2444] = 2180,
  [2445] = 2445,
  [2446] = 2446,
  [2447] = 2447,
  [2448] = 2181,
  [2449] = 2316,
  [2450] = 2450,
  [2451] = 2180,
  [2452] = 2452,
  [2453] = 2453,
  [2454] = 2178,
  [2455] = 2182,
  [2456] = 2180,
  [2457] = 2457,
  [2458] = 2458,
  [2459] = 2459,
  [2460] = 2460,
  [2461] = 2461,
  [2462] = 2462,
  [2463] = 2463,
};

static bool ts_lex(TSLexer *lexer, TSStateId state) {
  START_LEXER();
  eof = lexer->eof(lexer);
  switch (state) {
    case 0:
      if (eof) ADVANCE(22);
      ADVANCE_MAP(
        '!', 76,
        '#', 85,
        '%', 74,
        '&', 46,
        '(', 40,
        ')', 42,
        '*', 38,
        '+', 71,
        ',', 41,
        '-', 72,
        '.', 77,
        '/', 93,
        ':', 52,
        ';', 86,
        '<', 83,
        '=', 49,
        '>', 64,
        '@', 84,
        '[', 78,
        ']', 79,
        '^', 68,
        'b', 23,
        'x', 24,
        '{', 53,
        '|', 44,
        '}', 54,
        '0', 27,
        '1', 27,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(20);
      if (('2' <= lookahead && lookahead <= '9')) ADVANCE(28);
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(25);
      END_STATE();
    case 1:
      ADVANCE_MAP(
        '!', 76,
        '#', 85,
        '%', 74,
        '&', 45,
        '(', 40,
        ')', 42,
        '*', 37,
        '+', 71,
        ',', 41,
        '-', 72,
        '.', 77,
        '/', 73,
        '0', 26,
        ':', 52,
        ';', 86,
        '<', 62,
        '=', 49,
        '>', 64,
        '[', 78,
        ']', 79,
        '^', 68,
        '{', 53,
        '|', 44,
        '}', 54,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(1);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(28);
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(25);
      END_STATE();
    case 2:
      ADVANCE_MAP(
        '!', 76,
        '%', 74,
        '&', 45,
        '(', 40,
        ')', 42,
        '*', 37,
        '+', 71,
        ',', 41,
        '-', 72,
        '.', 77,
        '/', 73,
        '0', 26,
        ':', 52,
        ';', 86,
        '<', 83,
        '=', 49,
        '>', 64,
        '[', 78,
        ']', 79,
        '^', 68,
        '{', 53,
        '|', 44,
        '}', 54,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(3);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(28);
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(25);
      END_STATE();
    case 3:
      ADVANCE_MAP(
        '!', 76,
        '%', 74,
        '&', 45,
        '(', 40,
        ')', 42,
        '*', 37,
        '+', 71,
        ',', 41,
        '-', 72,
        '.', 77,
        '/', 73,
        '0', 26,
        ':', 52,
        ';', 86,
        '<', 62,
        '=', 49,
        '>', 64,
        '[', 78,
        ']', 79,
        '^', 68,
        '{', 53,
        '|', 44,
        '}', 54,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(3);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(28);
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(25);
      END_STATE();
    case 4:
      ADVANCE_MAP(
        '!', 75,
        '#', 85,
        '&', 47,
        '(', 40,
        ')', 42,
        '*', 37,
        '+', 71,
        ',', 41,
        '.', 77,
        '/', 9,
        '0', 26,
        ':', 51,
        ';', 86,
        '<', 61,
        '=', 50,
        '>', 63,
        '@', 84,
        '[', 78,
        ']', 79,
        'b', 23,
        'x', 24,
        '{', 53,
        '|', 43,
        '}', 54,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(4);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(28);
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(25);
      END_STATE();
    case 5:
      ADVANCE_MAP(
        '!', 75,
        '(', 40,
        ')', 42,
        '*', 10,
        ',', 41,
        '/', 9,
        ':', 52,
        ';', 86,
        '<', 82,
        '=', 50,
        '>', 63,
        '{', 53,
        '|', 43,
        '}', 54,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(6);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(80);
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(25);
      END_STATE();
    case 6:
      ADVANCE_MAP(
        '!', 75,
        '(', 40,
        ')', 42,
        '*', 10,
        ',', 41,
        '/', 9,
        ':', 52,
        ';', 86,
        '=', 50,
        '>', 63,
        '{', 53,
        '|', 43,
        '}', 54,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(6);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(80);
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(25);
      END_STATE();
    case 7:
      if (lookahead == '"') ADVANCE(36);
      if (lookahead == '\\') ADVANCE(17);
      if (lookahead != 0) ADVANCE(7);
      END_STATE();
    case 8:
      if (lookahead == '"') ADVANCE(35);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(8);
      END_STATE();
    case 9:
      if (lookahead == '*') ADVANCE(98);
      if (lookahead == '/') ADVANCE(87);
      END_STATE();
    case 10:
      if (lookahead == '/') ADVANCE(100);
      END_STATE();
    case 11:
      if (lookahead == '/') ADVANCE(9);
      if (lookahead == '0' ||
          lookahead == '1') ADVANCE(32);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(11);
      END_STATE();
    case 12:
      if (lookahead == '/') ADVANCE(9);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(12);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(30);
      END_STATE();
    case 13:
      if (lookahead == '/') ADVANCE(9);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(13);
      if (('0' <= lookahead && lookahead <= '7')) ADVANCE(34);
      END_STATE();
    case 14:
      if (lookahead == '>') ADVANCE(56);
      END_STATE();
    case 15:
      if (lookahead == 't') ADVANCE(48);
      END_STATE();
    case 16:
      if (lookahead == 'u') ADVANCE(15);
      END_STATE();
    case 17:
      if (lookahead == 'x') ADVANCE(19);
      if (lookahead == '"' ||
          lookahead == '0' ||
          lookahead == '\\' ||
          lookahead == 'n' ||
          lookahead == 'r' ||
          lookahead == 't') ADVANCE(7);
      END_STATE();
    case 18:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(7);
      END_STATE();
    case 19:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(18);
      END_STATE();
    case 20:
      if (eof) ADVANCE(22);
      ADVANCE_MAP(
        '!', 76,
        '#', 85,
        '%', 74,
        '&', 46,
        '(', 40,
        ')', 42,
        '*', 38,
        '+', 71,
        ',', 41,
        '-', 72,
        '.', 77,
        '/', 73,
        ':', 52,
        ';', 86,
        '<', 62,
        '=', 49,
        '>', 64,
        '@', 84,
        '[', 78,
        ']', 79,
        '^', 68,
        'b', 23,
        'x', 24,
        '{', 53,
        '|', 44,
        '}', 54,
        '0', 27,
        '1', 27,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(20);
      if (('2' <= lookahead && lookahead <= '9')) ADVANCE(28);
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(25);
      END_STATE();
    case 21:
      if (eof) ADVANCE(22);
      ADVANCE_MAP(
        '!', 76,
        '#', 85,
        '%', 74,
        '&', 46,
        '(', 40,
        ')', 42,
        '*', 37,
        '+', 71,
        ',', 41,
        '-', 72,
        '.', 77,
        '/', 73,
        '0', 26,
        ':', 52,
        ';', 86,
        '<', 62,
        '=', 49,
        '>', 64,
        '@', 84,
        '[', 78,
        ']', 79,
        '^', 68,
        'b', 23,
        'x', 24,
        '{', 53,
        '|', 44,
        '}', 54,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(21);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(28);
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(25);
      END_STATE();
    case 22:
      ACCEPT_TOKEN(ts_builtin_sym_end);
      END_STATE();
    case 23:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '"') ADVANCE(7);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(25);
      END_STATE();
    case 24:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '"') ADVANCE(8);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(25);
      END_STATE();
    case 25:
      ACCEPT_TOKEN(sym_identifier);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(25);
      END_STATE();
    case 26:
      ACCEPT_TOKEN(aux_sym_number_token1);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(31);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(33);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(29);
      if (('0' <= lookahead && lookahead <= '9') ||
          lookahead == '_') ADVANCE(28);
      END_STATE();
    case 27:
      ACCEPT_TOKEN(aux_sym_number_token1);
      if (lookahead == '0' ||
          lookahead == '1' ||
          lookahead == '_') ADVANCE(27);
      if (('2' <= lookahead && lookahead <= '9')) ADVANCE(28);
      END_STATE();
    case 28:
      ACCEPT_TOKEN(aux_sym_number_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          lookahead == '_') ADVANCE(28);
      END_STATE();
    case 29:
      ACCEPT_TOKEN(aux_sym_number_token2);
      END_STATE();
    case 30:
      ACCEPT_TOKEN(aux_sym_number_token3);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(30);
      END_STATE();
    case 31:
      ACCEPT_TOKEN(aux_sym_number_token4);
      END_STATE();
    case 32:
      ACCEPT_TOKEN(aux_sym_number_token5);
      if (lookahead == '0' ||
          lookahead == '1' ||
          lookahead == '_') ADVANCE(32);
      END_STATE();
    case 33:
      ACCEPT_TOKEN(aux_sym_number_token6);
      END_STATE();
    case 34:
      ACCEPT_TOKEN(aux_sym_number_token7);
      if (('0' <= lookahead && lookahead <= '7') ||
          lookahead == '_') ADVANCE(34);
      END_STATE();
    case 35:
      ACCEPT_TOKEN(aux_sym_byte_string_token1);
      END_STATE();
    case 36:
      ACCEPT_TOKEN(aux_sym_byte_string_token2);
      END_STATE();
    case 37:
      ACCEPT_TOKEN(anon_sym_STAR);
      END_STATE();
    case 38:
      ACCEPT_TOKEN(anon_sym_STAR);
      if (lookahead == '/') ADVANCE(100);
      END_STATE();
    case 39:
      ACCEPT_TOKEN(anon_sym_COLON_COLON);
      END_STATE();
    case 40:
      ACCEPT_TOKEN(anon_sym_LPAREN);
      END_STATE();
    case 41:
      ACCEPT_TOKEN(anon_sym_COMMA);
      END_STATE();
    case 42:
      ACCEPT_TOKEN(anon_sym_RPAREN);
      END_STATE();
    case 43:
      ACCEPT_TOKEN(anon_sym_PIPE);
      END_STATE();
    case 44:
      ACCEPT_TOKEN(anon_sym_PIPE);
      if (lookahead == '|') ADVANCE(57);
      END_STATE();
    case 45:
      ACCEPT_TOKEN(anon_sym_AMP);
      if (lookahead == '&') ADVANCE(58);
      END_STATE();
    case 46:
      ACCEPT_TOKEN(anon_sym_AMP);
      if (lookahead == '&') ADVANCE(58);
      if (lookahead == 'm') ADVANCE(16);
      END_STATE();
    case 47:
      ACCEPT_TOKEN(anon_sym_AMP);
      if (lookahead == 'm') ADVANCE(16);
      END_STATE();
    case 48:
      ACCEPT_TOKEN(anon_sym_AMPmut);
      END_STATE();
    case 49:
      ACCEPT_TOKEN(anon_sym_EQ);
      if (lookahead == '=') ADVANCE(59);
      if (lookahead == '>') ADVANCE(81);
      END_STATE();
    case 50:
      ACCEPT_TOKEN(anon_sym_EQ);
      if (lookahead == '>') ADVANCE(81);
      END_STATE();
    case 51:
      ACCEPT_TOKEN(anon_sym_COLON);
      END_STATE();
    case 52:
      ACCEPT_TOKEN(anon_sym_COLON);
      if (lookahead == ':') ADVANCE(39);
      END_STATE();
    case 53:
      ACCEPT_TOKEN(anon_sym_LBRACE);
      END_STATE();
    case 54:
      ACCEPT_TOKEN(anon_sym_RBRACE);
      END_STATE();
    case 55:
      ACCEPT_TOKEN(anon_sym_EQ_EQ_GT);
      END_STATE();
    case 56:
      ACCEPT_TOKEN(anon_sym_LT_EQ_EQ_GT);
      END_STATE();
    case 57:
      ACCEPT_TOKEN(anon_sym_PIPE_PIPE);
      END_STATE();
    case 58:
      ACCEPT_TOKEN(anon_sym_AMP_AMP);
      END_STATE();
    case 59:
      ACCEPT_TOKEN(anon_sym_EQ_EQ);
      if (lookahead == '>') ADVANCE(55);
      END_STATE();
    case 60:
      ACCEPT_TOKEN(anon_sym_BANG_EQ);
      END_STATE();
    case 61:
      ACCEPT_TOKEN(anon_sym_LT);
      END_STATE();
    case 62:
      ACCEPT_TOKEN(anon_sym_LT);
      if (lookahead == '<') ADVANCE(69);
      if (lookahead == '=') ADVANCE(65);
      END_STATE();
    case 63:
      ACCEPT_TOKEN(anon_sym_GT);
      END_STATE();
    case 64:
      ACCEPT_TOKEN(anon_sym_GT);
      if (lookahead == '=') ADVANCE(66);
      if (lookahead == '>') ADVANCE(70);
      END_STATE();
    case 65:
      ACCEPT_TOKEN(anon_sym_LT_EQ);
      if (lookahead == '=') ADVANCE(14);
      END_STATE();
    case 66:
      ACCEPT_TOKEN(anon_sym_GT_EQ);
      END_STATE();
    case 67:
      ACCEPT_TOKEN(anon_sym_DOT_DOT);
      END_STATE();
    case 68:
      ACCEPT_TOKEN(anon_sym_CARET);
      END_STATE();
    case 69:
      ACCEPT_TOKEN(anon_sym_LT_LT);
      END_STATE();
    case 70:
      ACCEPT_TOKEN(anon_sym_GT_GT);
      END_STATE();
    case 71:
      ACCEPT_TOKEN(anon_sym_PLUS);
      END_STATE();
    case 72:
      ACCEPT_TOKEN(anon_sym_DASH);
      END_STATE();
    case 73:
      ACCEPT_TOKEN(anon_sym_SLASH);
      if (lookahead == '*') ADVANCE(98);
      if (lookahead == '/') ADVANCE(87);
      END_STATE();
    case 74:
      ACCEPT_TOKEN(anon_sym_PERCENT);
      END_STATE();
    case 75:
      ACCEPT_TOKEN(anon_sym_BANG);
      END_STATE();
    case 76:
      ACCEPT_TOKEN(anon_sym_BANG);
      if (lookahead == '=') ADVANCE(60);
      END_STATE();
    case 77:
      ACCEPT_TOKEN(anon_sym_DOT);
      if (lookahead == '.') ADVANCE(67);
      END_STATE();
    case 78:
      ACCEPT_TOKEN(anon_sym_LBRACK);
      END_STATE();
    case 79:
      ACCEPT_TOKEN(anon_sym_RBRACK);
      END_STATE();
    case 80:
      ACCEPT_TOKEN(sym_anon_field_index);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(80);
      END_STATE();
    case 81:
      ACCEPT_TOKEN(anon_sym_EQ_GT);
      END_STATE();
    case 82:
      ACCEPT_TOKEN(anon_sym_LT2);
      END_STATE();
    case 83:
      ACCEPT_TOKEN(anon_sym_LT2);
      if (lookahead == '<') ADVANCE(69);
      if (lookahead == '=') ADVANCE(65);
      END_STATE();
    case 84:
      ACCEPT_TOKEN(anon_sym_AT);
      END_STATE();
    case 85:
      ACCEPT_TOKEN(anon_sym_POUND);
      END_STATE();
    case 86:
      ACCEPT_TOKEN(anon_sym_SEMI);
      END_STATE();
    case 87:
      ACCEPT_TOKEN(anon_sym_SLASH_SLASH);
      END_STATE();
    case 88:
      ACCEPT_TOKEN(anon_sym_SLASH_SLASH);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(92);
      END_STATE();
    case 89:
      ACCEPT_TOKEN(anon_sym_SLASH_SLASH2);
      END_STATE();
    case 90:
      ACCEPT_TOKEN(aux_sym_line_comment_token1);
      if (lookahead == '*') ADVANCE(99);
      if (lookahead == '/') ADVANCE(88);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(92);
      END_STATE();
    case 91:
      ACCEPT_TOKEN(aux_sym_line_comment_token1);
      if (lookahead == '/') ADVANCE(90);
      if (lookahead == '\t' ||
          (0x0b <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(91);
      if (lookahead != 0 &&
          (lookahead < '\t' || '\r' < lookahead)) ADVANCE(92);
      END_STATE();
    case 92:
      ACCEPT_TOKEN(aux_sym_line_comment_token1);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(92);
      END_STATE();
    case 93:
      ACCEPT_TOKEN(anon_sym_SLASH2);
      if (lookahead == '/') ADVANCE(89);
      END_STATE();
    case 94:
      ACCEPT_TOKEN(aux_sym_line_comment_token2);
      if (lookahead == '*') ADVANCE(97);
      if (lookahead == '/') ADVANCE(97);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(97);
      END_STATE();
    case 95:
      ACCEPT_TOKEN(aux_sym_line_comment_token2);
      if (lookahead == '/') ADVANCE(93);
      if (lookahead == '\t' ||
          (0x0b <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(96);
      if (lookahead != 0 &&
          (lookahead < '\t' || '\r' < lookahead)) ADVANCE(97);
      END_STATE();
    case 96:
      ACCEPT_TOKEN(aux_sym_line_comment_token2);
      if (lookahead == '/') ADVANCE(94);
      if (lookahead == '\t' ||
          (0x0b <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(96);
      if (lookahead != 0 &&
          (lookahead < '\t' || '\r' < lookahead)) ADVANCE(97);
      END_STATE();
    case 97:
      ACCEPT_TOKEN(aux_sym_line_comment_token2);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(97);
      END_STATE();
    case 98:
      ACCEPT_TOKEN(anon_sym_SLASH_STAR);
      END_STATE();
    case 99:
      ACCEPT_TOKEN(anon_sym_SLASH_STAR);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(92);
      END_STATE();
    case 100:
      ACCEPT_TOKEN(anon_sym_STAR_SLASH);
      END_STATE();
    default:
      return false;
  }
}

static bool ts_lex_keywords(TSLexer *lexer, TSStateId state) {
  START_LEXER();
  eof = lexer->eof(lexer);
  switch (state) {
    case 0:
      ADVANCE_MAP(
        'a', 1,
        'b', 2,
        'c', 3,
        'd', 4,
        'e', 5,
        'f', 6,
        'g', 7,
        'h', 8,
        'i', 9,
        'k', 10,
        'l', 11,
        'm', 12,
        'n', 13,
        'p', 14,
        'r', 15,
        's', 16,
        't', 17,
        'u', 18,
        'v', 19,
        'w', 20,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(0);
      END_STATE();
    case 1:
      if (lookahead == 'b') ADVANCE(21);
      if (lookahead == 'c') ADVANCE(22);
      if (lookahead == 'd') ADVANCE(23);
      if (lookahead == 'p') ADVANCE(24);
      if (lookahead == 's') ADVANCE(25);
      if (lookahead == 'x') ADVANCE(26);
      END_STATE();
    case 2:
      if (lookahead == 'o') ADVANCE(27);
      if (lookahead == 'r') ADVANCE(28);
      END_STATE();
    case 3:
      if (lookahead == 'h') ADVANCE(29);
      if (lookahead == 'o') ADVANCE(30);
      END_STATE();
    case 4:
      if (lookahead == 'r') ADVANCE(31);
      END_STATE();
    case 5:
      if (lookahead == 'l') ADVANCE(32);
      if (lookahead == 'm') ADVANCE(33);
      if (lookahead == 'n') ADVANCE(34);
      if (lookahead == 'x') ADVANCE(35);
      END_STATE();
    case 6:
      if (lookahead == 'a') ADVANCE(36);
      if (lookahead == 'o') ADVANCE(37);
      if (lookahead == 'r') ADVANCE(38);
      if (lookahead == 'u') ADVANCE(39);
      END_STATE();
    case 7:
      if (lookahead == 'l') ADVANCE(40);
      END_STATE();
    case 8:
      if (lookahead == 'a') ADVANCE(41);
      END_STATE();
    case 9:
      if (lookahead == 'f') ADVANCE(42);
      if (lookahead == 'n') ADVANCE(43);
      END_STATE();
    case 10:
      if (lookahead == 'e') ADVANCE(44);
      END_STATE();
    case 11:
      if (lookahead == 'e') ADVANCE(45);
      if (lookahead == 'o') ADVANCE(46);
      END_STATE();
    case 12:
      if (lookahead == 'a') ADVANCE(47);
      if (lookahead == 'i') ADVANCE(48);
      if (lookahead == 'o') ADVANCE(49);
      END_STATE();
    case 13:
      if (lookahead == 'a') ADVANCE(50);
      END_STATE();
    case 14:
      if (lookahead == 'a') ADVANCE(51);
      if (lookahead == 'h') ADVANCE(52);
      if (lookahead == 'o') ADVANCE(53);
      if (lookahead == 'r') ADVANCE(54);
      if (lookahead == 'u') ADVANCE(55);
      END_STATE();
    case 15:
      if (lookahead == 'e') ADVANCE(56);
      END_STATE();
    case 16:
      if (lookahead == 'c') ADVANCE(57);
      if (lookahead == 'p') ADVANCE(58);
      if (lookahead == 't') ADVANCE(59);
      END_STATE();
    case 17:
      if (lookahead == 'o') ADVANCE(60);
      if (lookahead == 'r') ADVANCE(61);
      END_STATE();
    case 18:
      if (lookahead == '1') ADVANCE(62);
      if (lookahead == '2') ADVANCE(63);
      if (lookahead == '3') ADVANCE(64);
      if (lookahead == '6') ADVANCE(65);
      if (lookahead == '8') ADVANCE(66);
      if (lookahead == 'p') ADVANCE(67);
      if (lookahead == 's') ADVANCE(68);
      END_STATE();
    case 19:
      if (lookahead == 'e') ADVANCE(69);
      END_STATE();
    case 20:
      if (lookahead == 'h') ADVANCE(70);
      if (lookahead == 'i') ADVANCE(71);
      if (lookahead == 'r') ADVANCE(72);
      END_STATE();
    case 21:
      if (lookahead == 'o') ADVANCE(73);
      END_STATE();
    case 22:
      if (lookahead == 'q') ADVANCE(74);
      END_STATE();
    case 23:
      if (lookahead == 'd') ADVANCE(75);
      END_STATE();
    case 24:
      if (lookahead == 'p') ADVANCE(76);
      END_STATE();
    case 25:
      ACCEPT_TOKEN(anon_sym_as);
      if (lookahead == 's') ADVANCE(77);
      END_STATE();
    case 26:
      if (lookahead == 'i') ADVANCE(78);
      END_STATE();
    case 27:
      if (lookahead == 'o') ADVANCE(79);
      END_STATE();
    case 28:
      if (lookahead == 'e') ADVANCE(80);
      END_STATE();
    case 29:
      if (lookahead == 'o') ADVANCE(81);
      END_STATE();
    case 30:
      if (lookahead == 'n') ADVANCE(82);
      if (lookahead == 'p') ADVANCE(83);
      END_STATE();
    case 31:
      if (lookahead == 'o') ADVANCE(84);
      END_STATE();
    case 32:
      if (lookahead == 's') ADVANCE(85);
      END_STATE();
    case 33:
      if (lookahead == 'i') ADVANCE(86);
      END_STATE();
    case 34:
      if (lookahead == 's') ADVANCE(87);
      if (lookahead == 't') ADVANCE(88);
      if (lookahead == 'u') ADVANCE(89);
      END_STATE();
    case 35:
      if (lookahead == 'c') ADVANCE(90);
      if (lookahead == 'i') ADVANCE(91);
      END_STATE();
    case 36:
      if (lookahead == 'l') ADVANCE(92);
      END_STATE();
    case 37:
      if (lookahead == 'r') ADVANCE(93);
      END_STATE();
    case 38:
      if (lookahead == 'i') ADVANCE(94);
      END_STATE();
    case 39:
      if (lookahead == 'n') ADVANCE(95);
      END_STATE();
    case 40:
      if (lookahead == 'o') ADVANCE(96);
      END_STATE();
    case 41:
      if (lookahead == 's') ADVANCE(97);
      END_STATE();
    case 42:
      ACCEPT_TOKEN(anon_sym_if);
      END_STATE();
    case 43:
      ACCEPT_TOKEN(anon_sym_in);
      if (lookahead == 'c') ADVANCE(98);
      if (lookahead == 'l') ADVANCE(99);
      if (lookahead == 't') ADVANCE(100);
      if (lookahead == 'v') ADVANCE(101);
      END_STATE();
    case 44:
      if (lookahead == 'y') ADVANCE(102);
      END_STATE();
    case 45:
      if (lookahead == 't') ADVANCE(103);
      END_STATE();
    case 46:
      if (lookahead == 'c') ADVANCE(104);
      if (lookahead == 'o') ADVANCE(105);
      END_STATE();
    case 47:
      if (lookahead == 't') ADVANCE(106);
      END_STATE();
    case 48:
      if (lookahead == 'n') ADVANCE(107);
      END_STATE();
    case 49:
      if (lookahead == 'd') ADVANCE(108);
      if (lookahead == 'v') ADVANCE(109);
      END_STATE();
    case 50:
      if (lookahead == 't') ADVANCE(110);
      END_STATE();
    case 51:
      if (lookahead == 'c') ADVANCE(111);
      END_STATE();
    case 52:
      if (lookahead == 'a') ADVANCE(112);
      END_STATE();
    case 53:
      if (lookahead == 's') ADVANCE(113);
      END_STATE();
    case 54:
      if (lookahead == 'a') ADVANCE(114);
      END_STATE();
    case 55:
      if (lookahead == 'b') ADVANCE(115);
      if (lookahead == 'r') ADVANCE(116);
      END_STATE();
    case 56:
      if (lookahead == 'a') ADVANCE(117);
      if (lookahead == 'q') ADVANCE(118);
      if (lookahead == 't') ADVANCE(119);
      END_STATE();
    case 57:
      if (lookahead == 'h') ADVANCE(120);
      if (lookahead == 'r') ADVANCE(121);
      END_STATE();
    case 58:
      if (lookahead == 'e') ADVANCE(122);
      END_STATE();
    case 59:
      if (lookahead == 'o') ADVANCE(123);
      if (lookahead == 'r') ADVANCE(124);
      END_STATE();
    case 60:
      ACCEPT_TOKEN(anon_sym_to);
      END_STATE();
    case 61:
      if (lookahead == 'u') ADVANCE(125);
      END_STATE();
    case 62:
      if (lookahead == '2') ADVANCE(126);
      if (lookahead == '6') ADVANCE(127);
      END_STATE();
    case 63:
      if (lookahead == '5') ADVANCE(128);
      END_STATE();
    case 64:
      if (lookahead == '2') ADVANCE(129);
      END_STATE();
    case 65:
      if (lookahead == '4') ADVANCE(130);
      END_STATE();
    case 66:
      ACCEPT_TOKEN(anon_sym_u8);
      END_STATE();
    case 67:
      if (lookahead == 'd') ADVANCE(131);
      END_STATE();
    case 68:
      if (lookahead == 'e') ADVANCE(132);
      END_STATE();
    case 69:
      if (lookahead == 'c') ADVANCE(133);
      END_STATE();
    case 70:
      if (lookahead == 'e') ADVANCE(134);
      if (lookahead == 'i') ADVANCE(135);
      END_STATE();
    case 71:
      if (lookahead == 't') ADVANCE(136);
      END_STATE();
    case 72:
      if (lookahead == 'i') ADVANCE(137);
      END_STATE();
    case 73:
      if (lookahead == 'r') ADVANCE(138);
      END_STATE();
    case 74:
      if (lookahead == 'u') ADVANCE(139);
      END_STATE();
    case 75:
      if (lookahead == 'r') ADVANCE(140);
      END_STATE();
    case 76:
      if (lookahead == 'l') ADVANCE(141);
      END_STATE();
    case 77:
      if (lookahead == 'e') ADVANCE(142);
      if (lookahead == 'u') ADVANCE(143);
      END_STATE();
    case 78:
      if (lookahead == 'o') ADVANCE(144);
      END_STATE();
    case 79:
      if (lookahead == 'l') ADVANCE(145);
      END_STATE();
    case 80:
      if (lookahead == 'a') ADVANCE(146);
      END_STATE();
    case 81:
      if (lookahead == 'o') ADVANCE(147);
      END_STATE();
    case 82:
      if (lookahead == 's') ADVANCE(148);
      if (lookahead == 't') ADVANCE(149);
      END_STATE();
    case 83:
      if (lookahead == 'y') ADVANCE(150);
      END_STATE();
    case 84:
      if (lookahead == 'p') ADVANCE(151);
      END_STATE();
    case 85:
      if (lookahead == 'e') ADVANCE(152);
      END_STATE();
    case 86:
      if (lookahead == 't') ADVANCE(153);
      END_STATE();
    case 87:
      if (lookahead == 'u') ADVANCE(154);
      END_STATE();
    case 88:
      if (lookahead == 'r') ADVANCE(155);
      END_STATE();
    case 89:
      if (lookahead == 'm') ADVANCE(156);
      END_STATE();
    case 90:
      if (lookahead == 'e') ADVANCE(157);
      END_STATE();
    case 91:
      if (lookahead == 's') ADVANCE(158);
      END_STATE();
    case 92:
      if (lookahead == 's') ADVANCE(159);
      END_STATE();
    case 93:
      ACCEPT_TOKEN(anon_sym_for);
      if (lookahead == 'a') ADVANCE(160);
      END_STATE();
    case 94:
      if (lookahead == 'e') ADVANCE(161);
      END_STATE();
    case 95:
      ACCEPT_TOKEN(anon_sym_fun);
      END_STATE();
    case 96:
      if (lookahead == 'b') ADVANCE(162);
      END_STATE();
    case 97:
      ACCEPT_TOKEN(anon_sym_has);
      END_STATE();
    case 98:
      if (lookahead == 'l') ADVANCE(163);
      END_STATE();
    case 99:
      if (lookahead == 'i') ADVANCE(164);
      END_STATE();
    case 100:
      if (lookahead == 'e') ADVANCE(165);
      END_STATE();
    case 101:
      if (lookahead == 'a') ADVANCE(166);
      END_STATE();
    case 102:
      ACCEPT_TOKEN(anon_sym_key);
      END_STATE();
    case 103:
      ACCEPT_TOKEN(anon_sym_let);
      END_STATE();
    case 104:
      if (lookahead == 'a') ADVANCE(167);
      END_STATE();
    case 105:
      if (lookahead == 'p') ADVANCE(168);
      END_STATE();
    case 106:
      if (lookahead == 'c') ADVANCE(169);
      END_STATE();
    case 107:
      ACCEPT_TOKEN(anon_sym_min);
      END_STATE();
    case 108:
      if (lookahead == 'i') ADVANCE(170);
      if (lookahead == 'u') ADVANCE(171);
      END_STATE();
    case 109:
      if (lookahead == 'e') ADVANCE(172);
      END_STATE();
    case 110:
      if (lookahead == 'i') ADVANCE(173);
      END_STATE();
    case 111:
      if (lookahead == 'k') ADVANCE(174);
      END_STATE();
    case 112:
      if (lookahead == 'n') ADVANCE(175);
      END_STATE();
    case 113:
      if (lookahead == 't') ADVANCE(176);
      END_STATE();
    case 114:
      if (lookahead == 'g') ADVANCE(177);
      END_STATE();
    case 115:
      if (lookahead == 'l') ADVANCE(178);
      END_STATE();
    case 116:
      if (lookahead == 'e') ADVANCE(179);
      END_STATE();
    case 117:
      if (lookahead == 'd') ADVANCE(180);
      END_STATE();
    case 118:
      if (lookahead == 'u') ADVANCE(181);
      END_STATE();
    case 119:
      if (lookahead == 'u') ADVANCE(182);
      END_STATE();
    case 120:
      if (lookahead == 'e') ADVANCE(183);
      END_STATE();
    case 121:
      if (lookahead == 'i') ADVANCE(184);
      END_STATE();
    case 122:
      if (lookahead == 'c') ADVANCE(185);
      END_STATE();
    case 123:
      if (lookahead == 'r') ADVANCE(186);
      END_STATE();
    case 124:
      if (lookahead == 'u') ADVANCE(187);
      END_STATE();
    case 125:
      if (lookahead == 'e') ADVANCE(188);
      END_STATE();
    case 126:
      if (lookahead == '8') ADVANCE(189);
      END_STATE();
    case 127:
      ACCEPT_TOKEN(anon_sym_u16);
      END_STATE();
    case 128:
      if (lookahead == '6') ADVANCE(190);
      END_STATE();
    case 129:
      ACCEPT_TOKEN(anon_sym_u32);
      END_STATE();
    case 130:
      ACCEPT_TOKEN(anon_sym_u64);
      END_STATE();
    case 131:
      if (lookahead == 'a') ADVANCE(191);
      END_STATE();
    case 132:
      ACCEPT_TOKEN(anon_sym_use);
      END_STATE();
    case 133:
      if (lookahead == 't') ADVANCE(192);
      END_STATE();
    case 134:
      if (lookahead == 'r') ADVANCE(193);
      END_STATE();
    case 135:
      if (lookahead == 'l') ADVANCE(194);
      END_STATE();
    case 136:
      if (lookahead == 'h') ADVANCE(195);
      END_STATE();
    case 137:
      if (lookahead == 't') ADVANCE(196);
      END_STATE();
    case 138:
      if (lookahead == 't') ADVANCE(197);
      END_STATE();
    case 139:
      if (lookahead == 'i') ADVANCE(198);
      END_STATE();
    case 140:
      if (lookahead == 'e') ADVANCE(199);
      END_STATE();
    case 141:
      if (lookahead == 'y') ADVANCE(200);
      END_STATE();
    case 142:
      if (lookahead == 'r') ADVANCE(201);
      END_STATE();
    case 143:
      if (lookahead == 'm') ADVANCE(202);
      END_STATE();
    case 144:
      if (lookahead == 'm') ADVANCE(203);
      END_STATE();
    case 145:
      ACCEPT_TOKEN(anon_sym_bool);
      END_STATE();
    case 146:
      if (lookahead == 'k') ADVANCE(204);
      END_STATE();
    case 147:
      if (lookahead == 's') ADVANCE(205);
      END_STATE();
    case 148:
      if (lookahead == 't') ADVANCE(206);
      END_STATE();
    case 149:
      if (lookahead == 'i') ADVANCE(207);
      END_STATE();
    case 150:
      ACCEPT_TOKEN(anon_sym_copy);
      END_STATE();
    case 151:
      ACCEPT_TOKEN(anon_sym_drop);
      END_STATE();
    case 152:
      ACCEPT_TOKEN(anon_sym_else);
      END_STATE();
    case 153:
      if (lookahead == 's') ADVANCE(208);
      END_STATE();
    case 154:
      if (lookahead == 'r') ADVANCE(209);
      END_STATE();
    case 155:
      if (lookahead == 'y') ADVANCE(210);
      END_STATE();
    case 156:
      ACCEPT_TOKEN(anon_sym_enum);
      END_STATE();
    case 157:
      if (lookahead == 'p') ADVANCE(211);
      END_STATE();
    case 158:
      if (lookahead == 't') ADVANCE(212);
      END_STATE();
    case 159:
      if (lookahead == 'e') ADVANCE(213);
      END_STATE();
    case 160:
      if (lookahead == 'l') ADVANCE(214);
      END_STATE();
    case 161:
      if (lookahead == 'n') ADVANCE(215);
      END_STATE();
    case 162:
      if (lookahead == 'a') ADVANCE(216);
      END_STATE();
    case 163:
      if (lookahead == 'u') ADVANCE(217);
      END_STATE();
    case 164:
      if (lookahead == 'n') ADVANCE(218);
      END_STATE();
    case 165:
      if (lookahead == 'r') ADVANCE(219);
      END_STATE();
    case 166:
      if (lookahead == 'r') ADVANCE(220);
      END_STATE();
    case 167:
      if (lookahead == 'l') ADVANCE(221);
      END_STATE();
    case 168:
      ACCEPT_TOKEN(anon_sym_loop);
      END_STATE();
    case 169:
      if (lookahead == 'h') ADVANCE(222);
      END_STATE();
    case 170:
      if (lookahead == 'f') ADVANCE(223);
      END_STATE();
    case 171:
      if (lookahead == 'l') ADVANCE(224);
      END_STATE();
    case 172:
      ACCEPT_TOKEN(anon_sym_move);
      END_STATE();
    case 173:
      if (lookahead == 'v') ADVANCE(225);
      END_STATE();
    case 174:
      if (lookahead == 'a') ADVANCE(226);
      END_STATE();
    case 175:
      if (lookahead == 't') ADVANCE(227);
      END_STATE();
    case 176:
      ACCEPT_TOKEN(anon_sym_post);
      END_STATE();
    case 177:
      if (lookahead == 'm') ADVANCE(228);
      END_STATE();
    case 178:
      if (lookahead == 'i') ADVANCE(229);
      END_STATE();
    case 179:
      ACCEPT_TOKEN(anon_sym_pure);
      END_STATE();
    case 180:
      if (lookahead == 's') ADVANCE(230);
      END_STATE();
    case 181:
      if (lookahead == 'i') ADVANCE(231);
      END_STATE();
    case 182:
      if (lookahead == 'r') ADVANCE(232);
      END_STATE();
    case 183:
      if (lookahead == 'm') ADVANCE(233);
      END_STATE();
    case 184:
      if (lookahead == 'p') ADVANCE(234);
      END_STATE();
    case 185:
      ACCEPT_TOKEN(anon_sym_spec);
      END_STATE();
    case 186:
      if (lookahead == 'e') ADVANCE(235);
      END_STATE();
    case 187:
      if (lookahead == 'c') ADVANCE(236);
      END_STATE();
    case 188:
      ACCEPT_TOKEN(anon_sym_true);
      END_STATE();
    case 189:
      ACCEPT_TOKEN(anon_sym_u128);
      END_STATE();
    case 190:
      ACCEPT_TOKEN(anon_sym_u256);
      END_STATE();
    case 191:
      if (lookahead == 't') ADVANCE(237);
      END_STATE();
    case 192:
      if (lookahead == 'o') ADVANCE(238);
      END_STATE();
    case 193:
      if (lookahead == 'e') ADVANCE(239);
      END_STATE();
    case 194:
      if (lookahead == 'e') ADVANCE(240);
      END_STATE();
    case 195:
      ACCEPT_TOKEN(anon_sym_with);
      END_STATE();
    case 196:
      if (lookahead == 'e') ADVANCE(241);
      END_STATE();
    case 197:
      ACCEPT_TOKEN(anon_sym_abort);
      if (lookahead == 's') ADVANCE(242);
      END_STATE();
    case 198:
      if (lookahead == 'r') ADVANCE(243);
      END_STATE();
    case 199:
      if (lookahead == 's') ADVANCE(244);
      END_STATE();
    case 200:
      ACCEPT_TOKEN(anon_sym_apply);
      END_STATE();
    case 201:
      if (lookahead == 't') ADVANCE(245);
      END_STATE();
    case 202:
      if (lookahead == 'e') ADVANCE(246);
      END_STATE();
    case 203:
      ACCEPT_TOKEN(anon_sym_axiom);
      END_STATE();
    case 204:
      ACCEPT_TOKEN(anon_sym_break);
      END_STATE();
    case 205:
      if (lookahead == 'e') ADVANCE(247);
      END_STATE();
    case 206:
      ACCEPT_TOKEN(anon_sym_const);
      END_STATE();
    case 207:
      if (lookahead == 'n') ADVANCE(248);
      END_STATE();
    case 208:
      ACCEPT_TOKEN(anon_sym_emits);
      END_STATE();
    case 209:
      if (lookahead == 'e') ADVANCE(249);
      END_STATE();
    case 210:
      ACCEPT_TOKEN(anon_sym_entry);
      END_STATE();
    case 211:
      if (lookahead == 't') ADVANCE(250);
      END_STATE();
    case 212:
      if (lookahead == 's') ADVANCE(251);
      END_STATE();
    case 213:
      ACCEPT_TOKEN(anon_sym_false);
      END_STATE();
    case 214:
      if (lookahead == 'l') ADVANCE(252);
      END_STATE();
    case 215:
      if (lookahead == 'd') ADVANCE(253);
      END_STATE();
    case 216:
      if (lookahead == 'l') ADVANCE(254);
      END_STATE();
    case 217:
      if (lookahead == 'd') ADVANCE(255);
      END_STATE();
    case 218:
      if (lookahead == 'e') ADVANCE(256);
      END_STATE();
    case 219:
      if (lookahead == 'n') ADVANCE(257);
      END_STATE();
    case 220:
      if (lookahead == 'i') ADVANCE(258);
      END_STATE();
    case 221:
      ACCEPT_TOKEN(anon_sym_local);
      END_STATE();
    case 222:
      ACCEPT_TOKEN(anon_sym_match);
      END_STATE();
    case 223:
      if (lookahead == 'i') ADVANCE(259);
      END_STATE();
    case 224:
      if (lookahead == 'e') ADVANCE(260);
      END_STATE();
    case 225:
      if (lookahead == 'e') ADVANCE(261);
      END_STATE();
    case 226:
      if (lookahead == 'g') ADVANCE(262);
      END_STATE();
    case 227:
      if (lookahead == 'o') ADVANCE(263);
      END_STATE();
    case 228:
      if (lookahead == 'a') ADVANCE(264);
      END_STATE();
    case 229:
      if (lookahead == 'c') ADVANCE(265);
      END_STATE();
    case 230:
      ACCEPT_TOKEN(anon_sym_reads);
      END_STATE();
    case 231:
      if (lookahead == 'r') ADVANCE(266);
      END_STATE();
    case 232:
      if (lookahead == 'n') ADVANCE(267);
      END_STATE();
    case 233:
      if (lookahead == 'a') ADVANCE(268);
      END_STATE();
    case 234:
      if (lookahead == 't') ADVANCE(269);
      END_STATE();
    case 235:
      ACCEPT_TOKEN(anon_sym_store);
      END_STATE();
    case 236:
      if (lookahead == 't') ADVANCE(270);
      END_STATE();
    case 237:
      if (lookahead == 'e') ADVANCE(271);
      END_STATE();
    case 238:
      if (lookahead == 'r') ADVANCE(272);
      END_STATE();
    case 239:
      ACCEPT_TOKEN(anon_sym_where);
      END_STATE();
    case 240:
      ACCEPT_TOKEN(anon_sym_while);
      END_STATE();
    case 241:
      if (lookahead == 's') ADVANCE(273);
      END_STATE();
    case 242:
      if (lookahead == '_') ADVANCE(274);
      END_STATE();
    case 243:
      if (lookahead == 'e') ADVANCE(275);
      END_STATE();
    case 244:
      if (lookahead == 's') ADVANCE(276);
      END_STATE();
    case 245:
      ACCEPT_TOKEN(anon_sym_assert);
      END_STATE();
    case 246:
      ACCEPT_TOKEN(anon_sym_assume);
      END_STATE();
    case 247:
      ACCEPT_TOKEN(anon_sym_choose);
      END_STATE();
    case 248:
      if (lookahead == 'u') ADVANCE(277);
      END_STATE();
    case 249:
      if (lookahead == 's') ADVANCE(278);
      END_STATE();
    case 250:
      ACCEPT_TOKEN(anon_sym_except);
      END_STATE();
    case 251:
      ACCEPT_TOKEN(anon_sym_exists);
      END_STATE();
    case 252:
      ACCEPT_TOKEN(anon_sym_forall);
      END_STATE();
    case 253:
      ACCEPT_TOKEN(anon_sym_friend);
      END_STATE();
    case 254:
      ACCEPT_TOKEN(anon_sym_global);
      END_STATE();
    case 255:
      if (lookahead == 'e') ADVANCE(279);
      END_STATE();
    case 256:
      ACCEPT_TOKEN(anon_sym_inline);
      END_STATE();
    case 257:
      if (lookahead == 'a') ADVANCE(280);
      END_STATE();
    case 258:
      if (lookahead == 'a') ADVANCE(281);
      END_STATE();
    case 259:
      if (lookahead == 'e') ADVANCE(282);
      END_STATE();
    case 260:
      ACCEPT_TOKEN(anon_sym_module);
      END_STATE();
    case 261:
      ACCEPT_TOKEN(anon_sym_native);
      END_STATE();
    case 262:
      if (lookahead == 'e') ADVANCE(283);
      END_STATE();
    case 263:
      if (lookahead == 'm') ADVANCE(284);
      END_STATE();
    case 264:
      ACCEPT_TOKEN(anon_sym_pragma);
      END_STATE();
    case 265:
      ACCEPT_TOKEN(anon_sym_public);
      END_STATE();
    case 266:
      if (lookahead == 'e') ADVANCE(285);
      END_STATE();
    case 267:
      ACCEPT_TOKEN(anon_sym_return);
      END_STATE();
    case 268:
      ACCEPT_TOKEN(anon_sym_schema);
      END_STATE();
    case 269:
      ACCEPT_TOKEN(anon_sym_script);
      END_STATE();
    case 270:
      ACCEPT_TOKEN(anon_sym_struct);
      END_STATE();
    case 271:
      ACCEPT_TOKEN(anon_sym_update);
      END_STATE();
    case 272:
      ACCEPT_TOKEN(anon_sym_vector);
      END_STATE();
    case 273:
      ACCEPT_TOKEN(anon_sym_writes);
      END_STATE();
    case 274:
      if (lookahead == 'i') ADVANCE(286);
      if (lookahead == 'w') ADVANCE(287);
      END_STATE();
    case 275:
      if (lookahead == 's') ADVANCE(288);
      END_STATE();
    case 276:
      ACCEPT_TOKEN(anon_sym_address);
      END_STATE();
    case 277:
      if (lookahead == 'e') ADVANCE(289);
      END_STATE();
    case 278:
      ACCEPT_TOKEN(anon_sym_ensures);
      END_STATE();
    case 279:
      ACCEPT_TOKEN(anon_sym_include);
      END_STATE();
    case 280:
      if (lookahead == 'l') ADVANCE(290);
      END_STATE();
    case 281:
      if (lookahead == 'n') ADVANCE(291);
      END_STATE();
    case 282:
      if (lookahead == 's') ADVANCE(292);
      END_STATE();
    case 283:
      ACCEPT_TOKEN(anon_sym_package);
      END_STATE();
    case 284:
      ACCEPT_TOKEN(sym_phantom);
      END_STATE();
    case 285:
      if (lookahead == 's') ADVANCE(293);
      END_STATE();
    case 286:
      if (lookahead == 'f') ADVANCE(294);
      END_STATE();
    case 287:
      if (lookahead == 'i') ADVANCE(295);
      END_STATE();
    case 288:
      ACCEPT_TOKEN(anon_sym_acquires);
      END_STATE();
    case 289:
      ACCEPT_TOKEN(anon_sym_continue);
      END_STATE();
    case 290:
      ACCEPT_TOKEN(anon_sym_internal);
      END_STATE();
    case 291:
      if (lookahead == 't') ADVANCE(296);
      END_STATE();
    case 292:
      ACCEPT_TOKEN(anon_sym_modifies);
      END_STATE();
    case 293:
      ACCEPT_TOKEN(anon_sym_requires);
      END_STATE();
    case 294:
      ACCEPT_TOKEN(anon_sym_aborts_if);
      END_STATE();
    case 295:
      if (lookahead == 't') ADVANCE(297);
      END_STATE();
    case 296:
      ACCEPT_TOKEN(anon_sym_invariant);
      END_STATE();
    case 297:
      if (lookahead == 'h') ADVANCE(298);
      END_STATE();
    case 298:
      ACCEPT_TOKEN(anon_sym_aborts_with);
      END_STATE();
    default:
      return false;
  }
}

static const TSLexMode ts_lex_modes[STATE_COUNT] = {
  [0] = {.lex_state = 0, .external_lex_state = 1},
  [1] = {.lex_state = 21},
  [2] = {.lex_state = 21},
  [3] = {.lex_state = 21},
  [4] = {.lex_state = 21},
  [5] = {.lex_state = 21},
  [6] = {.lex_state = 21},
  [7] = {.lex_state = 21},
  [8] = {.lex_state = 21},
  [9] = {.lex_state = 21},
  [10] = {.lex_state = 21},
  [11] = {.lex_state = 21},
  [12] = {.lex_state = 21},
  [13] = {.lex_state = 21},
  [14] = {.lex_state = 21},
  [15] = {.lex_state = 21},
  [16] = {.lex_state = 21},
  [17] = {.lex_state = 21},
  [18] = {.lex_state = 21},
  [19] = {.lex_state = 21},
  [20] = {.lex_state = 21},
  [21] = {.lex_state = 21},
  [22] = {.lex_state = 4},
  [23] = {.lex_state = 4},
  [24] = {.lex_state = 4},
  [25] = {.lex_state = 4},
  [26] = {.lex_state = 4},
  [27] = {.lex_state = 4},
  [28] = {.lex_state = 4},
  [29] = {.lex_state = 4},
  [30] = {.lex_state = 4},
  [31] = {.lex_state = 4},
  [32] = {.lex_state = 4},
  [33] = {.lex_state = 4},
  [34] = {.lex_state = 4},
  [35] = {.lex_state = 4},
  [36] = {.lex_state = 4},
  [37] = {.lex_state = 4},
  [38] = {.lex_state = 4},
  [39] = {.lex_state = 4},
  [40] = {.lex_state = 4},
  [41] = {.lex_state = 4},
  [42] = {.lex_state = 4},
  [43] = {.lex_state = 4},
  [44] = {.lex_state = 4},
  [45] = {.lex_state = 4},
  [46] = {.lex_state = 4},
  [47] = {.lex_state = 4},
  [48] = {.lex_state = 4},
  [49] = {.lex_state = 4},
  [50] = {.lex_state = 4},
  [51] = {.lex_state = 4},
  [52] = {.lex_state = 4},
  [53] = {.lex_state = 4},
  [54] = {.lex_state = 4},
  [55] = {.lex_state = 4},
  [56] = {.lex_state = 4},
  [57] = {.lex_state = 4},
  [58] = {.lex_state = 4},
  [59] = {.lex_state = 4},
  [60] = {.lex_state = 4},
  [61] = {.lex_state = 4},
  [62] = {.lex_state = 4},
  [63] = {.lex_state = 4},
  [64] = {.lex_state = 4},
  [65] = {.lex_state = 4},
  [66] = {.lex_state = 4},
  [67] = {.lex_state = 4},
  [68] = {.lex_state = 4},
  [69] = {.lex_state = 4},
  [70] = {.lex_state = 4},
  [71] = {.lex_state = 4},
  [72] = {.lex_state = 4},
  [73] = {.lex_state = 4},
  [74] = {.lex_state = 4},
  [75] = {.lex_state = 4},
  [76] = {.lex_state = 4},
  [77] = {.lex_state = 4},
  [78] = {.lex_state = 4},
  [79] = {.lex_state = 4},
  [80] = {.lex_state = 4},
  [81] = {.lex_state = 4},
  [82] = {.lex_state = 4},
  [83] = {.lex_state = 4},
  [84] = {.lex_state = 4},
  [85] = {.lex_state = 4},
  [86] = {.lex_state = 4},
  [87] = {.lex_state = 4},
  [88] = {.lex_state = 4},
  [89] = {.lex_state = 4},
  [90] = {.lex_state = 4},
  [91] = {.lex_state = 4},
  [92] = {.lex_state = 4},
  [93] = {.lex_state = 4},
  [94] = {.lex_state = 4},
  [95] = {.lex_state = 4},
  [96] = {.lex_state = 4},
  [97] = {.lex_state = 4},
  [98] = {.lex_state = 4},
  [99] = {.lex_state = 4},
  [100] = {.lex_state = 4},
  [101] = {.lex_state = 4},
  [102] = {.lex_state = 4},
  [103] = {.lex_state = 4},
  [104] = {.lex_state = 4},
  [105] = {.lex_state = 4},
  [106] = {.lex_state = 4},
  [107] = {.lex_state = 4},
  [108] = {.lex_state = 4},
  [109] = {.lex_state = 4},
  [110] = {.lex_state = 4},
  [111] = {.lex_state = 4},
  [112] = {.lex_state = 4},
  [113] = {.lex_state = 4},
  [114] = {.lex_state = 4},
  [115] = {.lex_state = 4},
  [116] = {.lex_state = 4},
  [117] = {.lex_state = 4},
  [118] = {.lex_state = 4},
  [119] = {.lex_state = 4},
  [120] = {.lex_state = 4},
  [121] = {.lex_state = 4},
  [122] = {.lex_state = 4},
  [123] = {.lex_state = 4},
  [124] = {.lex_state = 4},
  [125] = {.lex_state = 4},
  [126] = {.lex_state = 4},
  [127] = {.lex_state = 4},
  [128] = {.lex_state = 4},
  [129] = {.lex_state = 4},
  [130] = {.lex_state = 4},
  [131] = {.lex_state = 4},
  [132] = {.lex_state = 4},
  [133] = {.lex_state = 4},
  [134] = {.lex_state = 4},
  [135] = {.lex_state = 4},
  [136] = {.lex_state = 4},
  [137] = {.lex_state = 4},
  [138] = {.lex_state = 4},
  [139] = {.lex_state = 4},
  [140] = {.lex_state = 4},
  [141] = {.lex_state = 4},
  [142] = {.lex_state = 4},
  [143] = {.lex_state = 4},
  [144] = {.lex_state = 4},
  [145] = {.lex_state = 4},
  [146] = {.lex_state = 4},
  [147] = {.lex_state = 4},
  [148] = {.lex_state = 4},
  [149] = {.lex_state = 4},
  [150] = {.lex_state = 4},
  [151] = {.lex_state = 4},
  [152] = {.lex_state = 4},
  [153] = {.lex_state = 4},
  [154] = {.lex_state = 4},
  [155] = {.lex_state = 4},
  [156] = {.lex_state = 4},
  [157] = {.lex_state = 4},
  [158] = {.lex_state = 4},
  [159] = {.lex_state = 4},
  [160] = {.lex_state = 4},
  [161] = {.lex_state = 4},
  [162] = {.lex_state = 4},
  [163] = {.lex_state = 4},
  [164] = {.lex_state = 4},
  [165] = {.lex_state = 4},
  [166] = {.lex_state = 4},
  [167] = {.lex_state = 4},
  [168] = {.lex_state = 4},
  [169] = {.lex_state = 4},
  [170] = {.lex_state = 4},
  [171] = {.lex_state = 4},
  [172] = {.lex_state = 4},
  [173] = {.lex_state = 4},
  [174] = {.lex_state = 4},
  [175] = {.lex_state = 4},
  [176] = {.lex_state = 4},
  [177] = {.lex_state = 4},
  [178] = {.lex_state = 4},
  [179] = {.lex_state = 4},
  [180] = {.lex_state = 4},
  [181] = {.lex_state = 4},
  [182] = {.lex_state = 4},
  [183] = {.lex_state = 4},
  [184] = {.lex_state = 4},
  [185] = {.lex_state = 4},
  [186] = {.lex_state = 4},
  [187] = {.lex_state = 4},
  [188] = {.lex_state = 4},
  [189] = {.lex_state = 4},
  [190] = {.lex_state = 4},
  [191] = {.lex_state = 4},
  [192] = {.lex_state = 4},
  [193] = {.lex_state = 4},
  [194] = {.lex_state = 4},
  [195] = {.lex_state = 4},
  [196] = {.lex_state = 4},
  [197] = {.lex_state = 4},
  [198] = {.lex_state = 4},
  [199] = {.lex_state = 4},
  [200] = {.lex_state = 4},
  [201] = {.lex_state = 4},
  [202] = {.lex_state = 4},
  [203] = {.lex_state = 4},
  [204] = {.lex_state = 4},
  [205] = {.lex_state = 4},
  [206] = {.lex_state = 4},
  [207] = {.lex_state = 4},
  [208] = {.lex_state = 4},
  [209] = {.lex_state = 4},
  [210] = {.lex_state = 4},
  [211] = {.lex_state = 4},
  [212] = {.lex_state = 4},
  [213] = {.lex_state = 4},
  [214] = {.lex_state = 4},
  [215] = {.lex_state = 4},
  [216] = {.lex_state = 4},
  [217] = {.lex_state = 4},
  [218] = {.lex_state = 4},
  [219] = {.lex_state = 4},
  [220] = {.lex_state = 4},
  [221] = {.lex_state = 4},
  [222] = {.lex_state = 4},
  [223] = {.lex_state = 4},
  [224] = {.lex_state = 4},
  [225] = {.lex_state = 4},
  [226] = {.lex_state = 4},
  [227] = {.lex_state = 4},
  [228] = {.lex_state = 4},
  [229] = {.lex_state = 4},
  [230] = {.lex_state = 4},
  [231] = {.lex_state = 4},
  [232] = {.lex_state = 4},
  [233] = {.lex_state = 4},
  [234] = {.lex_state = 4},
  [235] = {.lex_state = 4},
  [236] = {.lex_state = 4},
  [237] = {.lex_state = 4},
  [238] = {.lex_state = 4},
  [239] = {.lex_state = 4},
  [240] = {.lex_state = 4},
  [241] = {.lex_state = 4},
  [242] = {.lex_state = 4},
  [243] = {.lex_state = 4},
  [244] = {.lex_state = 4},
  [245] = {.lex_state = 4},
  [246] = {.lex_state = 4},
  [247] = {.lex_state = 4},
  [248] = {.lex_state = 4},
  [249] = {.lex_state = 4},
  [250] = {.lex_state = 4},
  [251] = {.lex_state = 4},
  [252] = {.lex_state = 4},
  [253] = {.lex_state = 4},
  [254] = {.lex_state = 4},
  [255] = {.lex_state = 4},
  [256] = {.lex_state = 4},
  [257] = {.lex_state = 4},
  [258] = {.lex_state = 4},
  [259] = {.lex_state = 4},
  [260] = {.lex_state = 4},
  [261] = {.lex_state = 4},
  [262] = {.lex_state = 4},
  [263] = {.lex_state = 4},
  [264] = {.lex_state = 4},
  [265] = {.lex_state = 4},
  [266] = {.lex_state = 4},
  [267] = {.lex_state = 4},
  [268] = {.lex_state = 4},
  [269] = {.lex_state = 4},
  [270] = {.lex_state = 4},
  [271] = {.lex_state = 4},
  [272] = {.lex_state = 4},
  [273] = {.lex_state = 4},
  [274] = {.lex_state = 4},
  [275] = {.lex_state = 4},
  [276] = {.lex_state = 4},
  [277] = {.lex_state = 4},
  [278] = {.lex_state = 4},
  [279] = {.lex_state = 4},
  [280] = {.lex_state = 4},
  [281] = {.lex_state = 4},
  [282] = {.lex_state = 4},
  [283] = {.lex_state = 4},
  [284] = {.lex_state = 4},
  [285] = {.lex_state = 4},
  [286] = {.lex_state = 4},
  [287] = {.lex_state = 4},
  [288] = {.lex_state = 4},
  [289] = {.lex_state = 4},
  [290] = {.lex_state = 4},
  [291] = {.lex_state = 4},
  [292] = {.lex_state = 4},
  [293] = {.lex_state = 4},
  [294] = {.lex_state = 4},
  [295] = {.lex_state = 4},
  [296] = {.lex_state = 4},
  [297] = {.lex_state = 4},
  [298] = {.lex_state = 4},
  [299] = {.lex_state = 4},
  [300] = {.lex_state = 4},
  [301] = {.lex_state = 4},
  [302] = {.lex_state = 4},
  [303] = {.lex_state = 4},
  [304] = {.lex_state = 4},
  [305] = {.lex_state = 4},
  [306] = {.lex_state = 4},
  [307] = {.lex_state = 4},
  [308] = {.lex_state = 4},
  [309] = {.lex_state = 4},
  [310] = {.lex_state = 4},
  [311] = {.lex_state = 4},
  [312] = {.lex_state = 4},
  [313] = {.lex_state = 4},
  [314] = {.lex_state = 4},
  [315] = {.lex_state = 4},
  [316] = {.lex_state = 4},
  [317] = {.lex_state = 4},
  [318] = {.lex_state = 4},
  [319] = {.lex_state = 4},
  [320] = {.lex_state = 4},
  [321] = {.lex_state = 4},
  [322] = {.lex_state = 4},
  [323] = {.lex_state = 4},
  [324] = {.lex_state = 4},
  [325] = {.lex_state = 4},
  [326] = {.lex_state = 4},
  [327] = {.lex_state = 4},
  [328] = {.lex_state = 4},
  [329] = {.lex_state = 4},
  [330] = {.lex_state = 4},
  [331] = {.lex_state = 4},
  [332] = {.lex_state = 4},
  [333] = {.lex_state = 4},
  [334] = {.lex_state = 4},
  [335] = {.lex_state = 4},
  [336] = {.lex_state = 4},
  [337] = {.lex_state = 4},
  [338] = {.lex_state = 4},
  [339] = {.lex_state = 4},
  [340] = {.lex_state = 4},
  [341] = {.lex_state = 4},
  [342] = {.lex_state = 4},
  [343] = {.lex_state = 4},
  [344] = {.lex_state = 4},
  [345] = {.lex_state = 4},
  [346] = {.lex_state = 4},
  [347] = {.lex_state = 4},
  [348] = {.lex_state = 4},
  [349] = {.lex_state = 4},
  [350] = {.lex_state = 4},
  [351] = {.lex_state = 4},
  [352] = {.lex_state = 4},
  [353] = {.lex_state = 4},
  [354] = {.lex_state = 4},
  [355] = {.lex_state = 4},
  [356] = {.lex_state = 4},
  [357] = {.lex_state = 4},
  [358] = {.lex_state = 4},
  [359] = {.lex_state = 4},
  [360] = {.lex_state = 4},
  [361] = {.lex_state = 4},
  [362] = {.lex_state = 4},
  [363] = {.lex_state = 4},
  [364] = {.lex_state = 4},
  [365] = {.lex_state = 4},
  [366] = {.lex_state = 4},
  [367] = {.lex_state = 4},
  [368] = {.lex_state = 4},
  [369] = {.lex_state = 4},
  [370] = {.lex_state = 4},
  [371] = {.lex_state = 4},
  [372] = {.lex_state = 4},
  [373] = {.lex_state = 4},
  [374] = {.lex_state = 4},
  [375] = {.lex_state = 4},
  [376] = {.lex_state = 4},
  [377] = {.lex_state = 4},
  [378] = {.lex_state = 4},
  [379] = {.lex_state = 4},
  [380] = {.lex_state = 4},
  [381] = {.lex_state = 4},
  [382] = {.lex_state = 4},
  [383] = {.lex_state = 4},
  [384] = {.lex_state = 4},
  [385] = {.lex_state = 4},
  [386] = {.lex_state = 4},
  [387] = {.lex_state = 4},
  [388] = {.lex_state = 4},
  [389] = {.lex_state = 4},
  [390] = {.lex_state = 4},
  [391] = {.lex_state = 4},
  [392] = {.lex_state = 4},
  [393] = {.lex_state = 4},
  [394] = {.lex_state = 4},
  [395] = {.lex_state = 4},
  [396] = {.lex_state = 4},
  [397] = {.lex_state = 4},
  [398] = {.lex_state = 4},
  [399] = {.lex_state = 4},
  [400] = {.lex_state = 4},
  [401] = {.lex_state = 4},
  [402] = {.lex_state = 4},
  [403] = {.lex_state = 4},
  [404] = {.lex_state = 4},
  [405] = {.lex_state = 4},
  [406] = {.lex_state = 4},
  [407] = {.lex_state = 4},
  [408] = {.lex_state = 4},
  [409] = {.lex_state = 4},
  [410] = {.lex_state = 4},
  [411] = {.lex_state = 4},
  [412] = {.lex_state = 4},
  [413] = {.lex_state = 4},
  [414] = {.lex_state = 4},
  [415] = {.lex_state = 4},
  [416] = {.lex_state = 4},
  [417] = {.lex_state = 4},
  [418] = {.lex_state = 4},
  [419] = {.lex_state = 4},
  [420] = {.lex_state = 4},
  [421] = {.lex_state = 4},
  [422] = {.lex_state = 4},
  [423] = {.lex_state = 4},
  [424] = {.lex_state = 4},
  [425] = {.lex_state = 4},
  [426] = {.lex_state = 4},
  [427] = {.lex_state = 4},
  [428] = {.lex_state = 4},
  [429] = {.lex_state = 4},
  [430] = {.lex_state = 4},
  [431] = {.lex_state = 4},
  [432] = {.lex_state = 4},
  [433] = {.lex_state = 4},
  [434] = {.lex_state = 4},
  [435] = {.lex_state = 4},
  [436] = {.lex_state = 4},
  [437] = {.lex_state = 4},
  [438] = {.lex_state = 4},
  [439] = {.lex_state = 4},
  [440] = {.lex_state = 4},
  [441] = {.lex_state = 4},
  [442] = {.lex_state = 4},
  [443] = {.lex_state = 4},
  [444] = {.lex_state = 4},
  [445] = {.lex_state = 4},
  [446] = {.lex_state = 4},
  [447] = {.lex_state = 4},
  [448] = {.lex_state = 4},
  [449] = {.lex_state = 4},
  [450] = {.lex_state = 4},
  [451] = {.lex_state = 4},
  [452] = {.lex_state = 4},
  [453] = {.lex_state = 4},
  [454] = {.lex_state = 4},
  [455] = {.lex_state = 4},
  [456] = {.lex_state = 4},
  [457] = {.lex_state = 4},
  [458] = {.lex_state = 4},
  [459] = {.lex_state = 4},
  [460] = {.lex_state = 4},
  [461] = {.lex_state = 4},
  [462] = {.lex_state = 4},
  [463] = {.lex_state = 4},
  [464] = {.lex_state = 4},
  [465] = {.lex_state = 4},
  [466] = {.lex_state = 4},
  [467] = {.lex_state = 4},
  [468] = {.lex_state = 4},
  [469] = {.lex_state = 4},
  [470] = {.lex_state = 4},
  [471] = {.lex_state = 4},
  [472] = {.lex_state = 4},
  [473] = {.lex_state = 4},
  [474] = {.lex_state = 4},
  [475] = {.lex_state = 4},
  [476] = {.lex_state = 4},
  [477] = {.lex_state = 4},
  [478] = {.lex_state = 4},
  [479] = {.lex_state = 4},
  [480] = {.lex_state = 4},
  [481] = {.lex_state = 4},
  [482] = {.lex_state = 4},
  [483] = {.lex_state = 4},
  [484] = {.lex_state = 4},
  [485] = {.lex_state = 4},
  [486] = {.lex_state = 4},
  [487] = {.lex_state = 4},
  [488] = {.lex_state = 4},
  [489] = {.lex_state = 4},
  [490] = {.lex_state = 4},
  [491] = {.lex_state = 4},
  [492] = {.lex_state = 4},
  [493] = {.lex_state = 4},
  [494] = {.lex_state = 4},
  [495] = {.lex_state = 4},
  [496] = {.lex_state = 4},
  [497] = {.lex_state = 4},
  [498] = {.lex_state = 4},
  [499] = {.lex_state = 4},
  [500] = {.lex_state = 4},
  [501] = {.lex_state = 4},
  [502] = {.lex_state = 4},
  [503] = {.lex_state = 4},
  [504] = {.lex_state = 4},
  [505] = {.lex_state = 4},
  [506] = {.lex_state = 4},
  [507] = {.lex_state = 4},
  [508] = {.lex_state = 4},
  [509] = {.lex_state = 4},
  [510] = {.lex_state = 4},
  [511] = {.lex_state = 4},
  [512] = {.lex_state = 4},
  [513] = {.lex_state = 4},
  [514] = {.lex_state = 4},
  [515] = {.lex_state = 4},
  [516] = {.lex_state = 4},
  [517] = {.lex_state = 4},
  [518] = {.lex_state = 4},
  [519] = {.lex_state = 4},
  [520] = {.lex_state = 4},
  [521] = {.lex_state = 4},
  [522] = {.lex_state = 4},
  [523] = {.lex_state = 4},
  [524] = {.lex_state = 4},
  [525] = {.lex_state = 4},
  [526] = {.lex_state = 4},
  [527] = {.lex_state = 4},
  [528] = {.lex_state = 4},
  [529] = {.lex_state = 1},
  [530] = {.lex_state = 1},
  [531] = {.lex_state = 1},
  [532] = {.lex_state = 4},
  [533] = {.lex_state = 4},
  [534] = {.lex_state = 4},
  [535] = {.lex_state = 4},
  [536] = {.lex_state = 4},
  [537] = {.lex_state = 4},
  [538] = {.lex_state = 4},
  [539] = {.lex_state = 4},
  [540] = {.lex_state = 4},
  [541] = {.lex_state = 4},
  [542] = {.lex_state = 4},
  [543] = {.lex_state = 4},
  [544] = {.lex_state = 4},
  [545] = {.lex_state = 4},
  [546] = {.lex_state = 4},
  [547] = {.lex_state = 4},
  [548] = {.lex_state = 4},
  [549] = {.lex_state = 4},
  [550] = {.lex_state = 4},
  [551] = {.lex_state = 4},
  [552] = {.lex_state = 4},
  [553] = {.lex_state = 4},
  [554] = {.lex_state = 4},
  [555] = {.lex_state = 4},
  [556] = {.lex_state = 4},
  [557] = {.lex_state = 4},
  [558] = {.lex_state = 4},
  [559] = {.lex_state = 4},
  [560] = {.lex_state = 4},
  [561] = {.lex_state = 4},
  [562] = {.lex_state = 4},
  [563] = {.lex_state = 4},
  [564] = {.lex_state = 4},
  [565] = {.lex_state = 4},
  [566] = {.lex_state = 4},
  [567] = {.lex_state = 4},
  [568] = {.lex_state = 4},
  [569] = {.lex_state = 4},
  [570] = {.lex_state = 4},
  [571] = {.lex_state = 4},
  [572] = {.lex_state = 4},
  [573] = {.lex_state = 4},
  [574] = {.lex_state = 4},
  [575] = {.lex_state = 4},
  [576] = {.lex_state = 4},
  [577] = {.lex_state = 4},
  [578] = {.lex_state = 4},
  [579] = {.lex_state = 4},
  [580] = {.lex_state = 4},
  [581] = {.lex_state = 4},
  [582] = {.lex_state = 4},
  [583] = {.lex_state = 4},
  [584] = {.lex_state = 4},
  [585] = {.lex_state = 4},
  [586] = {.lex_state = 4},
  [587] = {.lex_state = 4},
  [588] = {.lex_state = 4},
  [589] = {.lex_state = 4},
  [590] = {.lex_state = 4},
  [591] = {.lex_state = 4},
  [592] = {.lex_state = 4},
  [593] = {.lex_state = 4},
  [594] = {.lex_state = 4},
  [595] = {.lex_state = 4},
  [596] = {.lex_state = 4},
  [597] = {.lex_state = 4},
  [598] = {.lex_state = 4},
  [599] = {.lex_state = 4},
  [600] = {.lex_state = 4},
  [601] = {.lex_state = 4},
  [602] = {.lex_state = 4},
  [603] = {.lex_state = 4},
  [604] = {.lex_state = 4},
  [605] = {.lex_state = 4},
  [606] = {.lex_state = 4},
  [607] = {.lex_state = 4},
  [608] = {.lex_state = 4},
  [609] = {.lex_state = 4},
  [610] = {.lex_state = 4},
  [611] = {.lex_state = 4},
  [612] = {.lex_state = 4},
  [613] = {.lex_state = 4},
  [614] = {.lex_state = 4},
  [615] = {.lex_state = 4},
  [616] = {.lex_state = 4},
  [617] = {.lex_state = 4},
  [618] = {.lex_state = 4},
  [619] = {.lex_state = 4},
  [620] = {.lex_state = 4},
  [621] = {.lex_state = 4},
  [622] = {.lex_state = 4},
  [623] = {.lex_state = 4},
  [624] = {.lex_state = 4},
  [625] = {.lex_state = 4},
  [626] = {.lex_state = 4},
  [627] = {.lex_state = 4},
  [628] = {.lex_state = 4},
  [629] = {.lex_state = 4},
  [630] = {.lex_state = 4},
  [631] = {.lex_state = 4},
  [632] = {.lex_state = 4},
  [633] = {.lex_state = 4},
  [634] = {.lex_state = 4},
  [635] = {.lex_state = 4},
  [636] = {.lex_state = 4},
  [637] = {.lex_state = 4},
  [638] = {.lex_state = 4},
  [639] = {.lex_state = 4},
  [640] = {.lex_state = 4},
  [641] = {.lex_state = 4},
  [642] = {.lex_state = 4},
  [643] = {.lex_state = 4},
  [644] = {.lex_state = 4},
  [645] = {.lex_state = 4},
  [646] = {.lex_state = 4},
  [647] = {.lex_state = 4},
  [648] = {.lex_state = 4},
  [649] = {.lex_state = 4},
  [650] = {.lex_state = 4},
  [651] = {.lex_state = 4},
  [652] = {.lex_state = 4},
  [653] = {.lex_state = 4},
  [654] = {.lex_state = 4},
  [655] = {.lex_state = 4},
  [656] = {.lex_state = 4},
  [657] = {.lex_state = 4},
  [658] = {.lex_state = 4},
  [659] = {.lex_state = 4},
  [660] = {.lex_state = 4},
  [661] = {.lex_state = 4},
  [662] = {.lex_state = 4},
  [663] = {.lex_state = 4},
  [664] = {.lex_state = 4},
  [665] = {.lex_state = 4},
  [666] = {.lex_state = 4},
  [667] = {.lex_state = 4},
  [668] = {.lex_state = 4},
  [669] = {.lex_state = 4},
  [670] = {.lex_state = 4},
  [671] = {.lex_state = 4},
  [672] = {.lex_state = 4},
  [673] = {.lex_state = 4},
  [674] = {.lex_state = 4},
  [675] = {.lex_state = 4},
  [676] = {.lex_state = 4},
  [677] = {.lex_state = 4},
  [678] = {.lex_state = 4},
  [679] = {.lex_state = 4},
  [680] = {.lex_state = 4},
  [681] = {.lex_state = 4},
  [682] = {.lex_state = 4},
  [683] = {.lex_state = 4},
  [684] = {.lex_state = 4},
  [685] = {.lex_state = 4},
  [686] = {.lex_state = 4},
  [687] = {.lex_state = 4},
  [688] = {.lex_state = 4},
  [689] = {.lex_state = 4},
  [690] = {.lex_state = 4},
  [691] = {.lex_state = 4},
  [692] = {.lex_state = 4},
  [693] = {.lex_state = 4},
  [694] = {.lex_state = 4},
  [695] = {.lex_state = 4},
  [696] = {.lex_state = 4},
  [697] = {.lex_state = 4},
  [698] = {.lex_state = 4},
  [699] = {.lex_state = 4},
  [700] = {.lex_state = 4},
  [701] = {.lex_state = 4},
  [702] = {.lex_state = 4},
  [703] = {.lex_state = 4},
  [704] = {.lex_state = 4},
  [705] = {.lex_state = 4},
  [706] = {.lex_state = 4},
  [707] = {.lex_state = 4},
  [708] = {.lex_state = 4},
  [709] = {.lex_state = 4},
  [710] = {.lex_state = 4},
  [711] = {.lex_state = 4},
  [712] = {.lex_state = 4},
  [713] = {.lex_state = 4},
  [714] = {.lex_state = 4},
  [715] = {.lex_state = 4},
  [716] = {.lex_state = 4},
  [717] = {.lex_state = 4},
  [718] = {.lex_state = 4},
  [719] = {.lex_state = 4},
  [720] = {.lex_state = 4},
  [721] = {.lex_state = 4},
  [722] = {.lex_state = 4},
  [723] = {.lex_state = 4},
  [724] = {.lex_state = 4},
  [725] = {.lex_state = 4},
  [726] = {.lex_state = 4},
  [727] = {.lex_state = 4},
  [728] = {.lex_state = 4},
  [729] = {.lex_state = 4},
  [730] = {.lex_state = 4},
  [731] = {.lex_state = 4},
  [732] = {.lex_state = 4},
  [733] = {.lex_state = 4},
  [734] = {.lex_state = 4},
  [735] = {.lex_state = 4},
  [736] = {.lex_state = 4},
  [737] = {.lex_state = 4},
  [738] = {.lex_state = 4},
  [739] = {.lex_state = 4},
  [740] = {.lex_state = 4},
  [741] = {.lex_state = 4},
  [742] = {.lex_state = 4},
  [743] = {.lex_state = 4},
  [744] = {.lex_state = 4},
  [745] = {.lex_state = 4},
  [746] = {.lex_state = 4},
  [747] = {.lex_state = 4},
  [748] = {.lex_state = 4},
  [749] = {.lex_state = 4},
  [750] = {.lex_state = 4},
  [751] = {.lex_state = 4},
  [752] = {.lex_state = 4},
  [753] = {.lex_state = 4},
  [754] = {.lex_state = 4},
  [755] = {.lex_state = 4},
  [756] = {.lex_state = 4},
  [757] = {.lex_state = 4},
  [758] = {.lex_state = 4},
  [759] = {.lex_state = 4},
  [760] = {.lex_state = 4},
  [761] = {.lex_state = 4},
  [762] = {.lex_state = 4},
  [763] = {.lex_state = 4},
  [764] = {.lex_state = 4},
  [765] = {.lex_state = 4},
  [766] = {.lex_state = 4},
  [767] = {.lex_state = 4},
  [768] = {.lex_state = 4},
  [769] = {.lex_state = 4},
  [770] = {.lex_state = 4},
  [771] = {.lex_state = 4},
  [772] = {.lex_state = 4},
  [773] = {.lex_state = 4},
  [774] = {.lex_state = 4},
  [775] = {.lex_state = 4},
  [776] = {.lex_state = 4},
  [777] = {.lex_state = 4},
  [778] = {.lex_state = 4},
  [779] = {.lex_state = 4},
  [780] = {.lex_state = 4},
  [781] = {.lex_state = 4},
  [782] = {.lex_state = 4},
  [783] = {.lex_state = 4},
  [784] = {.lex_state = 4},
  [785] = {.lex_state = 4},
  [786] = {.lex_state = 4},
  [787] = {.lex_state = 4},
  [788] = {.lex_state = 4},
  [789] = {.lex_state = 4},
  [790] = {.lex_state = 4},
  [791] = {.lex_state = 4},
  [792] = {.lex_state = 4},
  [793] = {.lex_state = 4},
  [794] = {.lex_state = 4},
  [795] = {.lex_state = 4},
  [796] = {.lex_state = 4},
  [797] = {.lex_state = 4},
  [798] = {.lex_state = 4},
  [799] = {.lex_state = 4},
  [800] = {.lex_state = 4},
  [801] = {.lex_state = 4},
  [802] = {.lex_state = 4},
  [803] = {.lex_state = 4},
  [804] = {.lex_state = 4},
  [805] = {.lex_state = 4},
  [806] = {.lex_state = 4},
  [807] = {.lex_state = 4},
  [808] = {.lex_state = 4},
  [809] = {.lex_state = 4},
  [810] = {.lex_state = 4},
  [811] = {.lex_state = 4},
  [812] = {.lex_state = 4},
  [813] = {.lex_state = 4},
  [814] = {.lex_state = 4},
  [815] = {.lex_state = 4},
  [816] = {.lex_state = 4},
  [817] = {.lex_state = 4},
  [818] = {.lex_state = 4},
  [819] = {.lex_state = 4},
  [820] = {.lex_state = 4},
  [821] = {.lex_state = 4},
  [822] = {.lex_state = 4},
  [823] = {.lex_state = 4},
  [824] = {.lex_state = 4},
  [825] = {.lex_state = 4},
  [826] = {.lex_state = 4},
  [827] = {.lex_state = 4},
  [828] = {.lex_state = 4},
  [829] = {.lex_state = 4},
  [830] = {.lex_state = 4},
  [831] = {.lex_state = 4},
  [832] = {.lex_state = 4},
  [833] = {.lex_state = 4},
  [834] = {.lex_state = 4},
  [835] = {.lex_state = 4},
  [836] = {.lex_state = 4},
  [837] = {.lex_state = 4},
  [838] = {.lex_state = 4},
  [839] = {.lex_state = 4},
  [840] = {.lex_state = 4},
  [841] = {.lex_state = 4},
  [842] = {.lex_state = 4},
  [843] = {.lex_state = 4},
  [844] = {.lex_state = 4},
  [845] = {.lex_state = 4},
  [846] = {.lex_state = 4},
  [847] = {.lex_state = 1},
  [848] = {.lex_state = 1},
  [849] = {.lex_state = 1},
  [850] = {.lex_state = 1},
  [851] = {.lex_state = 1},
  [852] = {.lex_state = 1},
  [853] = {.lex_state = 1},
  [854] = {.lex_state = 1},
  [855] = {.lex_state = 1},
  [856] = {.lex_state = 1},
  [857] = {.lex_state = 1},
  [858] = {.lex_state = 1},
  [859] = {.lex_state = 2},
  [860] = {.lex_state = 2},
  [861] = {.lex_state = 2},
  [862] = {.lex_state = 2},
  [863] = {.lex_state = 2},
  [864] = {.lex_state = 2},
  [865] = {.lex_state = 2},
  [866] = {.lex_state = 2},
  [867] = {.lex_state = 2},
  [868] = {.lex_state = 2},
  [869] = {.lex_state = 1},
  [870] = {.lex_state = 1},
  [871] = {.lex_state = 1},
  [872] = {.lex_state = 1},
  [873] = {.lex_state = 2},
  [874] = {.lex_state = 2},
  [875] = {.lex_state = 2},
  [876] = {.lex_state = 1},
  [877] = {.lex_state = 1},
  [878] = {.lex_state = 1},
  [879] = {.lex_state = 1},
  [880] = {.lex_state = 1},
  [881] = {.lex_state = 1},
  [882] = {.lex_state = 1},
  [883] = {.lex_state = 1},
  [884] = {.lex_state = 1},
  [885] = {.lex_state = 1},
  [886] = {.lex_state = 1},
  [887] = {.lex_state = 1},
  [888] = {.lex_state = 1},
  [889] = {.lex_state = 1},
  [890] = {.lex_state = 1},
  [891] = {.lex_state = 1},
  [892] = {.lex_state = 1},
  [893] = {.lex_state = 1},
  [894] = {.lex_state = 1},
  [895] = {.lex_state = 1},
  [896] = {.lex_state = 1},
  [897] = {.lex_state = 1},
  [898] = {.lex_state = 1},
  [899] = {.lex_state = 1},
  [900] = {.lex_state = 1},
  [901] = {.lex_state = 1},
  [902] = {.lex_state = 1},
  [903] = {.lex_state = 1},
  [904] = {.lex_state = 1},
  [905] = {.lex_state = 1},
  [906] = {.lex_state = 1},
  [907] = {.lex_state = 1},
  [908] = {.lex_state = 1},
  [909] = {.lex_state = 1},
  [910] = {.lex_state = 1},
  [911] = {.lex_state = 1},
  [912] = {.lex_state = 1},
  [913] = {.lex_state = 1},
  [914] = {.lex_state = 1},
  [915] = {.lex_state = 1},
  [916] = {.lex_state = 1},
  [917] = {.lex_state = 1},
  [918] = {.lex_state = 1},
  [919] = {.lex_state = 1},
  [920] = {.lex_state = 1},
  [921] = {.lex_state = 1},
  [922] = {.lex_state = 1},
  [923] = {.lex_state = 1},
  [924] = {.lex_state = 1},
  [925] = {.lex_state = 1},
  [926] = {.lex_state = 1},
  [927] = {.lex_state = 1},
  [928] = {.lex_state = 1},
  [929] = {.lex_state = 1},
  [930] = {.lex_state = 1},
  [931] = {.lex_state = 1},
  [932] = {.lex_state = 1},
  [933] = {.lex_state = 1},
  [934] = {.lex_state = 1},
  [935] = {.lex_state = 1},
  [936] = {.lex_state = 1},
  [937] = {.lex_state = 1},
  [938] = {.lex_state = 1},
  [939] = {.lex_state = 1},
  [940] = {.lex_state = 1},
  [941] = {.lex_state = 1},
  [942] = {.lex_state = 1},
  [943] = {.lex_state = 1},
  [944] = {.lex_state = 1},
  [945] = {.lex_state = 1},
  [946] = {.lex_state = 1},
  [947] = {.lex_state = 1},
  [948] = {.lex_state = 1},
  [949] = {.lex_state = 1},
  [950] = {.lex_state = 1},
  [951] = {.lex_state = 1},
  [952] = {.lex_state = 1},
  [953] = {.lex_state = 1},
  [954] = {.lex_state = 1},
  [955] = {.lex_state = 1},
  [956] = {.lex_state = 1},
  [957] = {.lex_state = 4},
  [958] = {.lex_state = 2},
  [959] = {.lex_state = 2},
  [960] = {.lex_state = 4},
  [961] = {.lex_state = 2},
  [962] = {.lex_state = 2},
  [963] = {.lex_state = 2},
  [964] = {.lex_state = 2},
  [965] = {.lex_state = 4},
  [966] = {.lex_state = 4},
  [967] = {.lex_state = 4},
  [968] = {.lex_state = 4},
  [969] = {.lex_state = 4},
  [970] = {.lex_state = 2},
  [971] = {.lex_state = 2},
  [972] = {.lex_state = 2},
  [973] = {.lex_state = 2},
  [974] = {.lex_state = 2},
  [975] = {.lex_state = 2},
  [976] = {.lex_state = 4},
  [977] = {.lex_state = 4},
  [978] = {.lex_state = 2},
  [979] = {.lex_state = 4},
  [980] = {.lex_state = 4},
  [981] = {.lex_state = 4},
  [982] = {.lex_state = 1},
  [983] = {.lex_state = 4},
  [984] = {.lex_state = 1},
  [985] = {.lex_state = 4},
  [986] = {.lex_state = 1},
  [987] = {.lex_state = 1},
  [988] = {.lex_state = 1},
  [989] = {.lex_state = 1},
  [990] = {.lex_state = 1},
  [991] = {.lex_state = 1},
  [992] = {.lex_state = 1},
  [993] = {.lex_state = 1},
  [994] = {.lex_state = 1},
  [995] = {.lex_state = 1},
  [996] = {.lex_state = 1},
  [997] = {.lex_state = 1},
  [998] = {.lex_state = 1},
  [999] = {.lex_state = 1},
  [1000] = {.lex_state = 1},
  [1001] = {.lex_state = 1},
  [1002] = {.lex_state = 1},
  [1003] = {.lex_state = 1},
  [1004] = {.lex_state = 1},
  [1005] = {.lex_state = 1},
  [1006] = {.lex_state = 1},
  [1007] = {.lex_state = 1},
  [1008] = {.lex_state = 1},
  [1009] = {.lex_state = 1},
  [1010] = {.lex_state = 1},
  [1011] = {.lex_state = 1},
  [1012] = {.lex_state = 1},
  [1013] = {.lex_state = 1},
  [1014] = {.lex_state = 1},
  [1015] = {.lex_state = 1},
  [1016] = {.lex_state = 1},
  [1017] = {.lex_state = 1},
  [1018] = {.lex_state = 1},
  [1019] = {.lex_state = 1},
  [1020] = {.lex_state = 1},
  [1021] = {.lex_state = 1},
  [1022] = {.lex_state = 1},
  [1023] = {.lex_state = 1},
  [1024] = {.lex_state = 1},
  [1025] = {.lex_state = 1},
  [1026] = {.lex_state = 4},
  [1027] = {.lex_state = 1},
  [1028] = {.lex_state = 1},
  [1029] = {.lex_state = 1},
  [1030] = {.lex_state = 1},
  [1031] = {.lex_state = 1},
  [1032] = {.lex_state = 1},
  [1033] = {.lex_state = 1},
  [1034] = {.lex_state = 4},
  [1035] = {.lex_state = 4},
  [1036] = {.lex_state = 1},
  [1037] = {.lex_state = 1},
  [1038] = {.lex_state = 1},
  [1039] = {.lex_state = 1},
  [1040] = {.lex_state = 1},
  [1041] = {.lex_state = 1},
  [1042] = {.lex_state = 4},
  [1043] = {.lex_state = 4},
  [1044] = {.lex_state = 1},
  [1045] = {.lex_state = 4},
  [1046] = {.lex_state = 4},
  [1047] = {.lex_state = 4},
  [1048] = {.lex_state = 2},
  [1049] = {.lex_state = 2},
  [1050] = {.lex_state = 2},
  [1051] = {.lex_state = 4},
  [1052] = {.lex_state = 2},
  [1053] = {.lex_state = 4},
  [1054] = {.lex_state = 4},
  [1055] = {.lex_state = 2},
  [1056] = {.lex_state = 4},
  [1057] = {.lex_state = 4},
  [1058] = {.lex_state = 4},
  [1059] = {.lex_state = 2},
  [1060] = {.lex_state = 4},
  [1061] = {.lex_state = 2},
  [1062] = {.lex_state = 4},
  [1063] = {.lex_state = 4},
  [1064] = {.lex_state = 4},
  [1065] = {.lex_state = 4},
  [1066] = {.lex_state = 4},
  [1067] = {.lex_state = 4},
  [1068] = {.lex_state = 4},
  [1069] = {.lex_state = 4},
  [1070] = {.lex_state = 2},
  [1071] = {.lex_state = 4},
  [1072] = {.lex_state = 4},
  [1073] = {.lex_state = 2},
  [1074] = {.lex_state = 2},
  [1075] = {.lex_state = 21},
  [1076] = {.lex_state = 4},
  [1077] = {.lex_state = 4},
  [1078] = {.lex_state = 4},
  [1079] = {.lex_state = 4},
  [1080] = {.lex_state = 4},
  [1081] = {.lex_state = 1},
  [1082] = {.lex_state = 2},
  [1083] = {.lex_state = 4},
  [1084] = {.lex_state = 4},
  [1085] = {.lex_state = 2},
  [1086] = {.lex_state = 4},
  [1087] = {.lex_state = 4},
  [1088] = {.lex_state = 4},
  [1089] = {.lex_state = 4},
  [1090] = {.lex_state = 4},
  [1091] = {.lex_state = 4},
  [1092] = {.lex_state = 21},
  [1093] = {.lex_state = 4},
  [1094] = {.lex_state = 4},
  [1095] = {.lex_state = 4},
  [1096] = {.lex_state = 4},
  [1097] = {.lex_state = 4},
  [1098] = {.lex_state = 21},
  [1099] = {.lex_state = 4},
  [1100] = {.lex_state = 4},
  [1101] = {.lex_state = 4},
  [1102] = {.lex_state = 4},
  [1103] = {.lex_state = 4},
  [1104] = {.lex_state = 21},
  [1105] = {.lex_state = 4},
  [1106] = {.lex_state = 2},
  [1107] = {.lex_state = 21},
  [1108] = {.lex_state = 4},
  [1109] = {.lex_state = 4},
  [1110] = {.lex_state = 21},
  [1111] = {.lex_state = 4},
  [1112] = {.lex_state = 4},
  [1113] = {.lex_state = 4},
  [1114] = {.lex_state = 4},
  [1115] = {.lex_state = 4},
  [1116] = {.lex_state = 4},
  [1117] = {.lex_state = 4},
  [1118] = {.lex_state = 21},
  [1119] = {.lex_state = 21},
  [1120] = {.lex_state = 21},
  [1121] = {.lex_state = 21},
  [1122] = {.lex_state = 2},
  [1123] = {.lex_state = 2},
  [1124] = {.lex_state = 21},
  [1125] = {.lex_state = 21},
  [1126] = {.lex_state = 2},
  [1127] = {.lex_state = 2},
  [1128] = {.lex_state = 21},
  [1129] = {.lex_state = 21},
  [1130] = {.lex_state = 21},
  [1131] = {.lex_state = 2},
  [1132] = {.lex_state = 21},
  [1133] = {.lex_state = 2},
  [1134] = {.lex_state = 2},
  [1135] = {.lex_state = 2},
  [1136] = {.lex_state = 1},
  [1137] = {.lex_state = 4},
  [1138] = {.lex_state = 1},
  [1139] = {.lex_state = 1},
  [1140] = {.lex_state = 1},
  [1141] = {.lex_state = 1},
  [1142] = {.lex_state = 1},
  [1143] = {.lex_state = 1},
  [1144] = {.lex_state = 1},
  [1145] = {.lex_state = 1},
  [1146] = {.lex_state = 1},
  [1147] = {.lex_state = 1},
  [1148] = {.lex_state = 4},
  [1149] = {.lex_state = 1},
  [1150] = {.lex_state = 1},
  [1151] = {.lex_state = 1},
  [1152] = {.lex_state = 2},
  [1153] = {.lex_state = 2},
  [1154] = {.lex_state = 4},
  [1155] = {.lex_state = 4},
  [1156] = {.lex_state = 2},
  [1157] = {.lex_state = 2},
  [1158] = {.lex_state = 2},
  [1159] = {.lex_state = 2},
  [1160] = {.lex_state = 2},
  [1161] = {.lex_state = 2},
  [1162] = {.lex_state = 2},
  [1163] = {.lex_state = 1},
  [1164] = {.lex_state = 1},
  [1165] = {.lex_state = 1},
  [1166] = {.lex_state = 1},
  [1167] = {.lex_state = 2},
  [1168] = {.lex_state = 1},
  [1169] = {.lex_state = 1},
  [1170] = {.lex_state = 1},
  [1171] = {.lex_state = 1},
  [1172] = {.lex_state = 1},
  [1173] = {.lex_state = 1},
  [1174] = {.lex_state = 2},
  [1175] = {.lex_state = 21},
  [1176] = {.lex_state = 21},
  [1177] = {.lex_state = 2},
  [1178] = {.lex_state = 2},
  [1179] = {.lex_state = 1},
  [1180] = {.lex_state = 1},
  [1181] = {.lex_state = 1},
  [1182] = {.lex_state = 1},
  [1183] = {.lex_state = 2},
  [1184] = {.lex_state = 1},
  [1185] = {.lex_state = 2},
  [1186] = {.lex_state = 1},
  [1187] = {.lex_state = 21},
  [1188] = {.lex_state = 1},
  [1189] = {.lex_state = 1},
  [1190] = {.lex_state = 1},
  [1191] = {.lex_state = 1},
  [1192] = {.lex_state = 1},
  [1193] = {.lex_state = 4},
  [1194] = {.lex_state = 1},
  [1195] = {.lex_state = 1},
  [1196] = {.lex_state = 2},
  [1197] = {.lex_state = 2},
  [1198] = {.lex_state = 1},
  [1199] = {.lex_state = 1},
  [1200] = {.lex_state = 2},
  [1201] = {.lex_state = 2},
  [1202] = {.lex_state = 1},
  [1203] = {.lex_state = 2},
  [1204] = {.lex_state = 4},
  [1205] = {.lex_state = 1},
  [1206] = {.lex_state = 1},
  [1207] = {.lex_state = 1},
  [1208] = {.lex_state = 1},
  [1209] = {.lex_state = 1},
  [1210] = {.lex_state = 1},
  [1211] = {.lex_state = 1},
  [1212] = {.lex_state = 1},
  [1213] = {.lex_state = 1},
  [1214] = {.lex_state = 1},
  [1215] = {.lex_state = 21},
  [1216] = {.lex_state = 1},
  [1217] = {.lex_state = 4},
  [1218] = {.lex_state = 21},
  [1219] = {.lex_state = 21},
  [1220] = {.lex_state = 21},
  [1221] = {.lex_state = 1},
  [1222] = {.lex_state = 2},
  [1223] = {.lex_state = 2},
  [1224] = {.lex_state = 2},
  [1225] = {.lex_state = 2},
  [1226] = {.lex_state = 21},
  [1227] = {.lex_state = 2},
  [1228] = {.lex_state = 21},
  [1229] = {.lex_state = 21},
  [1230] = {.lex_state = 21},
  [1231] = {.lex_state = 21},
  [1232] = {.lex_state = 21},
  [1233] = {.lex_state = 21},
  [1234] = {.lex_state = 21},
  [1235] = {.lex_state = 2},
  [1236] = {.lex_state = 21},
  [1237] = {.lex_state = 2},
  [1238] = {.lex_state = 21},
  [1239] = {.lex_state = 2},
  [1240] = {.lex_state = 21},
  [1241] = {.lex_state = 1},
  [1242] = {.lex_state = 2},
  [1243] = {.lex_state = 1},
  [1244] = {.lex_state = 21},
  [1245] = {.lex_state = 21},
  [1246] = {.lex_state = 21},
  [1247] = {.lex_state = 21},
  [1248] = {.lex_state = 21},
  [1249] = {.lex_state = 21},
  [1250] = {.lex_state = 21},
  [1251] = {.lex_state = 21},
  [1252] = {.lex_state = 21},
  [1253] = {.lex_state = 1},
  [1254] = {.lex_state = 1},
  [1255] = {.lex_state = 1},
  [1256] = {.lex_state = 1},
  [1257] = {.lex_state = 1},
  [1258] = {.lex_state = 1},
  [1259] = {.lex_state = 1},
  [1260] = {.lex_state = 1},
  [1261] = {.lex_state = 1},
  [1262] = {.lex_state = 1},
  [1263] = {.lex_state = 1},
  [1264] = {.lex_state = 1},
  [1265] = {.lex_state = 1},
  [1266] = {.lex_state = 1},
  [1267] = {.lex_state = 1},
  [1268] = {.lex_state = 1},
  [1269] = {.lex_state = 1},
  [1270] = {.lex_state = 1},
  [1271] = {.lex_state = 1},
  [1272] = {.lex_state = 1},
  [1273] = {.lex_state = 1},
  [1274] = {.lex_state = 1},
  [1275] = {.lex_state = 1},
  [1276] = {.lex_state = 1},
  [1277] = {.lex_state = 1},
  [1278] = {.lex_state = 1},
  [1279] = {.lex_state = 1},
  [1280] = {.lex_state = 1},
  [1281] = {.lex_state = 1},
  [1282] = {.lex_state = 1},
  [1283] = {.lex_state = 1},
  [1284] = {.lex_state = 1},
  [1285] = {.lex_state = 1},
  [1286] = {.lex_state = 1},
  [1287] = {.lex_state = 1},
  [1288] = {.lex_state = 1},
  [1289] = {.lex_state = 1},
  [1290] = {.lex_state = 1},
  [1291] = {.lex_state = 1},
  [1292] = {.lex_state = 1},
  [1293] = {.lex_state = 1},
  [1294] = {.lex_state = 1},
  [1295] = {.lex_state = 1},
  [1296] = {.lex_state = 1},
  [1297] = {.lex_state = 1},
  [1298] = {.lex_state = 1},
  [1299] = {.lex_state = 1},
  [1300] = {.lex_state = 1},
  [1301] = {.lex_state = 1},
  [1302] = {.lex_state = 1},
  [1303] = {.lex_state = 1},
  [1304] = {.lex_state = 1},
  [1305] = {.lex_state = 1},
  [1306] = {.lex_state = 1},
  [1307] = {.lex_state = 1},
  [1308] = {.lex_state = 1},
  [1309] = {.lex_state = 1},
  [1310] = {.lex_state = 1},
  [1311] = {.lex_state = 1},
  [1312] = {.lex_state = 1},
  [1313] = {.lex_state = 1},
  [1314] = {.lex_state = 1},
  [1315] = {.lex_state = 1},
  [1316] = {.lex_state = 1},
  [1317] = {.lex_state = 1},
  [1318] = {.lex_state = 1},
  [1319] = {.lex_state = 1},
  [1320] = {.lex_state = 1},
  [1321] = {.lex_state = 1},
  [1322] = {.lex_state = 1},
  [1323] = {.lex_state = 1},
  [1324] = {.lex_state = 1},
  [1325] = {.lex_state = 1},
  [1326] = {.lex_state = 1},
  [1327] = {.lex_state = 1},
  [1328] = {.lex_state = 1},
  [1329] = {.lex_state = 1},
  [1330] = {.lex_state = 1},
  [1331] = {.lex_state = 1},
  [1332] = {.lex_state = 1},
  [1333] = {.lex_state = 1},
  [1334] = {.lex_state = 1},
  [1335] = {.lex_state = 1},
  [1336] = {.lex_state = 1},
  [1337] = {.lex_state = 1},
  [1338] = {.lex_state = 1},
  [1339] = {.lex_state = 1},
  [1340] = {.lex_state = 1},
  [1341] = {.lex_state = 1},
  [1342] = {.lex_state = 1},
  [1343] = {.lex_state = 1},
  [1344] = {.lex_state = 1},
  [1345] = {.lex_state = 1},
  [1346] = {.lex_state = 1},
  [1347] = {.lex_state = 1},
  [1348] = {.lex_state = 1},
  [1349] = {.lex_state = 1},
  [1350] = {.lex_state = 1},
  [1351] = {.lex_state = 1},
  [1352] = {.lex_state = 1},
  [1353] = {.lex_state = 1},
  [1354] = {.lex_state = 1},
  [1355] = {.lex_state = 1},
  [1356] = {.lex_state = 1},
  [1357] = {.lex_state = 1},
  [1358] = {.lex_state = 1},
  [1359] = {.lex_state = 1},
  [1360] = {.lex_state = 1},
  [1361] = {.lex_state = 1},
  [1362] = {.lex_state = 1},
  [1363] = {.lex_state = 1},
  [1364] = {.lex_state = 1},
  [1365] = {.lex_state = 1},
  [1366] = {.lex_state = 1},
  [1367] = {.lex_state = 1},
  [1368] = {.lex_state = 1},
  [1369] = {.lex_state = 1},
  [1370] = {.lex_state = 1},
  [1371] = {.lex_state = 1},
  [1372] = {.lex_state = 1},
  [1373] = {.lex_state = 1},
  [1374] = {.lex_state = 1},
  [1375] = {.lex_state = 21},
  [1376] = {.lex_state = 1},
  [1377] = {.lex_state = 1},
  [1378] = {.lex_state = 1},
  [1379] = {.lex_state = 1},
  [1380] = {.lex_state = 21},
  [1381] = {.lex_state = 1},
  [1382] = {.lex_state = 1},
  [1383] = {.lex_state = 1},
  [1384] = {.lex_state = 1},
  [1385] = {.lex_state = 1},
  [1386] = {.lex_state = 1},
  [1387] = {.lex_state = 1},
  [1388] = {.lex_state = 1},
  [1389] = {.lex_state = 1},
  [1390] = {.lex_state = 1},
  [1391] = {.lex_state = 1},
  [1392] = {.lex_state = 1},
  [1393] = {.lex_state = 1},
  [1394] = {.lex_state = 1},
  [1395] = {.lex_state = 1},
  [1396] = {.lex_state = 1},
  [1397] = {.lex_state = 1},
  [1398] = {.lex_state = 1},
  [1399] = {.lex_state = 21},
  [1400] = {.lex_state = 1},
  [1401] = {.lex_state = 21},
  [1402] = {.lex_state = 1},
  [1403] = {.lex_state = 1},
  [1404] = {.lex_state = 1},
  [1405] = {.lex_state = 1},
  [1406] = {.lex_state = 1},
  [1407] = {.lex_state = 1},
  [1408] = {.lex_state = 1},
  [1409] = {.lex_state = 1},
  [1410] = {.lex_state = 1},
  [1411] = {.lex_state = 1},
  [1412] = {.lex_state = 1},
  [1413] = {.lex_state = 21},
  [1414] = {.lex_state = 1},
  [1415] = {.lex_state = 21},
  [1416] = {.lex_state = 1},
  [1417] = {.lex_state = 21},
  [1418] = {.lex_state = 1},
  [1419] = {.lex_state = 1},
  [1420] = {.lex_state = 1},
  [1421] = {.lex_state = 1},
  [1422] = {.lex_state = 21},
  [1423] = {.lex_state = 21},
  [1424] = {.lex_state = 21},
  [1425] = {.lex_state = 21},
  [1426] = {.lex_state = 21},
  [1427] = {.lex_state = 21},
  [1428] = {.lex_state = 21},
  [1429] = {.lex_state = 21},
  [1430] = {.lex_state = 21},
  [1431] = {.lex_state = 21},
  [1432] = {.lex_state = 21},
  [1433] = {.lex_state = 21},
  [1434] = {.lex_state = 21},
  [1435] = {.lex_state = 21},
  [1436] = {.lex_state = 21},
  [1437] = {.lex_state = 21},
  [1438] = {.lex_state = 21},
  [1439] = {.lex_state = 21},
  [1440] = {.lex_state = 21},
  [1441] = {.lex_state = 21},
  [1442] = {.lex_state = 21},
  [1443] = {.lex_state = 21},
  [1444] = {.lex_state = 21},
  [1445] = {.lex_state = 21},
  [1446] = {.lex_state = 21},
  [1447] = {.lex_state = 21},
  [1448] = {.lex_state = 21},
  [1449] = {.lex_state = 21},
  [1450] = {.lex_state = 21},
  [1451] = {.lex_state = 21},
  [1452] = {.lex_state = 21},
  [1453] = {.lex_state = 21},
  [1454] = {.lex_state = 21},
  [1455] = {.lex_state = 21},
  [1456] = {.lex_state = 21},
  [1457] = {.lex_state = 21},
  [1458] = {.lex_state = 21},
  [1459] = {.lex_state = 21},
  [1460] = {.lex_state = 21},
  [1461] = {.lex_state = 21},
  [1462] = {.lex_state = 21},
  [1463] = {.lex_state = 21},
  [1464] = {.lex_state = 21},
  [1465] = {.lex_state = 21},
  [1466] = {.lex_state = 21},
  [1467] = {.lex_state = 21},
  [1468] = {.lex_state = 21},
  [1469] = {.lex_state = 21},
  [1470] = {.lex_state = 21},
  [1471] = {.lex_state = 21},
  [1472] = {.lex_state = 21},
  [1473] = {.lex_state = 21},
  [1474] = {.lex_state = 21},
  [1475] = {.lex_state = 21},
  [1476] = {.lex_state = 21},
  [1477] = {.lex_state = 21},
  [1478] = {.lex_state = 21},
  [1479] = {.lex_state = 21},
  [1480] = {.lex_state = 21},
  [1481] = {.lex_state = 21},
  [1482] = {.lex_state = 21},
  [1483] = {.lex_state = 21},
  [1484] = {.lex_state = 21},
  [1485] = {.lex_state = 21},
  [1486] = {.lex_state = 21},
  [1487] = {.lex_state = 21},
  [1488] = {.lex_state = 21},
  [1489] = {.lex_state = 21},
  [1490] = {.lex_state = 21},
  [1491] = {.lex_state = 21},
  [1492] = {.lex_state = 21},
  [1493] = {.lex_state = 21},
  [1494] = {.lex_state = 21},
  [1495] = {.lex_state = 21},
  [1496] = {.lex_state = 21},
  [1497] = {.lex_state = 21},
  [1498] = {.lex_state = 21},
  [1499] = {.lex_state = 21},
  [1500] = {.lex_state = 21},
  [1501] = {.lex_state = 21},
  [1502] = {.lex_state = 21},
  [1503] = {.lex_state = 21},
  [1504] = {.lex_state = 21},
  [1505] = {.lex_state = 21},
  [1506] = {.lex_state = 21},
  [1507] = {.lex_state = 21},
  [1508] = {.lex_state = 21},
  [1509] = {.lex_state = 21},
  [1510] = {.lex_state = 21},
  [1511] = {.lex_state = 21},
  [1512] = {.lex_state = 21},
  [1513] = {.lex_state = 21},
  [1514] = {.lex_state = 21},
  [1515] = {.lex_state = 21},
  [1516] = {.lex_state = 21},
  [1517] = {.lex_state = 21},
  [1518] = {.lex_state = 21},
  [1519] = {.lex_state = 21},
  [1520] = {.lex_state = 21},
  [1521] = {.lex_state = 5},
  [1522] = {.lex_state = 21},
  [1523] = {.lex_state = 21},
  [1524] = {.lex_state = 5},
  [1525] = {.lex_state = 5},
  [1526] = {.lex_state = 5},
  [1527] = {.lex_state = 5},
  [1528] = {.lex_state = 5},
  [1529] = {.lex_state = 5},
  [1530] = {.lex_state = 21},
  [1531] = {.lex_state = 5},
  [1532] = {.lex_state = 21},
  [1533] = {.lex_state = 5},
  [1534] = {.lex_state = 21},
  [1535] = {.lex_state = 21},
  [1536] = {.lex_state = 21},
  [1537] = {.lex_state = 5},
  [1538] = {.lex_state = 5},
  [1539] = {.lex_state = 5},
  [1540] = {.lex_state = 21},
  [1541] = {.lex_state = 21},
  [1542] = {.lex_state = 4},
  [1543] = {.lex_state = 4},
  [1544] = {.lex_state = 4},
  [1545] = {.lex_state = 21},
  [1546] = {.lex_state = 4},
  [1547] = {.lex_state = 21},
  [1548] = {.lex_state = 21},
  [1549] = {.lex_state = 4},
  [1550] = {.lex_state = 4},
  [1551] = {.lex_state = 4},
  [1552] = {.lex_state = 21},
  [1553] = {.lex_state = 21},
  [1554] = {.lex_state = 2},
  [1555] = {.lex_state = 4},
  [1556] = {.lex_state = 4},
  [1557] = {.lex_state = 4},
  [1558] = {.lex_state = 4},
  [1559] = {.lex_state = 4},
  [1560] = {.lex_state = 4},
  [1561] = {.lex_state = 4},
  [1562] = {.lex_state = 4},
  [1563] = {.lex_state = 4},
  [1564] = {.lex_state = 2},
  [1565] = {.lex_state = 4},
  [1566] = {.lex_state = 4},
  [1567] = {.lex_state = 4},
  [1568] = {.lex_state = 21},
  [1569] = {.lex_state = 21},
  [1570] = {.lex_state = 21},
  [1571] = {.lex_state = 21},
  [1572] = {.lex_state = 21},
  [1573] = {.lex_state = 21},
  [1574] = {.lex_state = 21},
  [1575] = {.lex_state = 21},
  [1576] = {.lex_state = 21},
  [1577] = {.lex_state = 21},
  [1578] = {.lex_state = 21},
  [1579] = {.lex_state = 2},
  [1580] = {.lex_state = 21},
  [1581] = {.lex_state = 21},
  [1582] = {.lex_state = 21},
  [1583] = {.lex_state = 21},
  [1584] = {.lex_state = 21},
  [1585] = {.lex_state = 21},
  [1586] = {.lex_state = 21},
  [1587] = {.lex_state = 21},
  [1588] = {.lex_state = 21},
  [1589] = {.lex_state = 21},
  [1590] = {.lex_state = 4},
  [1591] = {.lex_state = 21},
  [1592] = {.lex_state = 21},
  [1593] = {.lex_state = 5},
  [1594] = {.lex_state = 21},
  [1595] = {.lex_state = 2},
  [1596] = {.lex_state = 21},
  [1597] = {.lex_state = 21},
  [1598] = {.lex_state = 21},
  [1599] = {.lex_state = 2},
  [1600] = {.lex_state = 21},
  [1601] = {.lex_state = 21},
  [1602] = {.lex_state = 21},
  [1603] = {.lex_state = 2},
  [1604] = {.lex_state = 2},
  [1605] = {.lex_state = 21},
  [1606] = {.lex_state = 2},
  [1607] = {.lex_state = 21},
  [1608] = {.lex_state = 21},
  [1609] = {.lex_state = 21},
  [1610] = {.lex_state = 2},
  [1611] = {.lex_state = 2},
  [1612] = {.lex_state = 21},
  [1613] = {.lex_state = 21},
  [1614] = {.lex_state = 21},
  [1615] = {.lex_state = 21},
  [1616] = {.lex_state = 4},
  [1617] = {.lex_state = 4},
  [1618] = {.lex_state = 21},
  [1619] = {.lex_state = 21},
  [1620] = {.lex_state = 4},
  [1621] = {.lex_state = 21},
  [1622] = {.lex_state = 21},
  [1623] = {.lex_state = 4},
  [1624] = {.lex_state = 4},
  [1625] = {.lex_state = 4},
  [1626] = {.lex_state = 2},
  [1627] = {.lex_state = 4},
  [1628] = {.lex_state = 4},
  [1629] = {.lex_state = 21},
  [1630] = {.lex_state = 2},
  [1631] = {.lex_state = 4},
  [1632] = {.lex_state = 21},
  [1633] = {.lex_state = 21},
  [1634] = {.lex_state = 4},
  [1635] = {.lex_state = 21},
  [1636] = {.lex_state = 21},
  [1637] = {.lex_state = 21},
  [1638] = {.lex_state = 21},
  [1639] = {.lex_state = 21},
  [1640] = {.lex_state = 4},
  [1641] = {.lex_state = 4},
  [1642] = {.lex_state = 21},
  [1643] = {.lex_state = 21},
  [1644] = {.lex_state = 2},
  [1645] = {.lex_state = 21},
  [1646] = {.lex_state = 21},
  [1647] = {.lex_state = 4},
  [1648] = {.lex_state = 21},
  [1649] = {.lex_state = 4},
  [1650] = {.lex_state = 4},
  [1651] = {.lex_state = 21},
  [1652] = {.lex_state = 21},
  [1653] = {.lex_state = 21},
  [1654] = {.lex_state = 21},
  [1655] = {.lex_state = 21},
  [1656] = {.lex_state = 21},
  [1657] = {.lex_state = 21},
  [1658] = {.lex_state = 21},
  [1659] = {.lex_state = 21},
  [1660] = {.lex_state = 21},
  [1661] = {.lex_state = 21},
  [1662] = {.lex_state = 21},
  [1663] = {.lex_state = 21},
  [1664] = {.lex_state = 21},
  [1665] = {.lex_state = 21},
  [1666] = {.lex_state = 21},
  [1667] = {.lex_state = 21},
  [1668] = {.lex_state = 21},
  [1669] = {.lex_state = 5},
  [1670] = {.lex_state = 21},
  [1671] = {.lex_state = 21},
  [1672] = {.lex_state = 21},
  [1673] = {.lex_state = 21},
  [1674] = {.lex_state = 21},
  [1675] = {.lex_state = 21},
  [1676] = {.lex_state = 21},
  [1677] = {.lex_state = 21},
  [1678] = {.lex_state = 21},
  [1679] = {.lex_state = 21},
  [1680] = {.lex_state = 21},
  [1681] = {.lex_state = 21},
  [1682] = {.lex_state = 21},
  [1683] = {.lex_state = 21},
  [1684] = {.lex_state = 21},
  [1685] = {.lex_state = 21},
  [1686] = {.lex_state = 21},
  [1687] = {.lex_state = 21},
  [1688] = {.lex_state = 21},
  [1689] = {.lex_state = 21},
  [1690] = {.lex_state = 21},
  [1691] = {.lex_state = 21},
  [1692] = {.lex_state = 21},
  [1693] = {.lex_state = 21},
  [1694] = {.lex_state = 21},
  [1695] = {.lex_state = 21},
  [1696] = {.lex_state = 21},
  [1697] = {.lex_state = 21},
  [1698] = {.lex_state = 21},
  [1699] = {.lex_state = 21},
  [1700] = {.lex_state = 21},
  [1701] = {.lex_state = 21},
  [1702] = {.lex_state = 21},
  [1703] = {.lex_state = 21},
  [1704] = {.lex_state = 21},
  [1705] = {.lex_state = 21},
  [1706] = {.lex_state = 21},
  [1707] = {.lex_state = 21},
  [1708] = {.lex_state = 21},
  [1709] = {.lex_state = 21},
  [1710] = {.lex_state = 21},
  [1711] = {.lex_state = 21},
  [1712] = {.lex_state = 21},
  [1713] = {.lex_state = 21},
  [1714] = {.lex_state = 21},
  [1715] = {.lex_state = 21},
  [1716] = {.lex_state = 21},
  [1717] = {.lex_state = 21},
  [1718] = {.lex_state = 21},
  [1719] = {.lex_state = 21},
  [1720] = {.lex_state = 21},
  [1721] = {.lex_state = 21},
  [1722] = {.lex_state = 21},
  [1723] = {.lex_state = 21},
  [1724] = {.lex_state = 21},
  [1725] = {.lex_state = 21},
  [1726] = {.lex_state = 21},
  [1727] = {.lex_state = 21},
  [1728] = {.lex_state = 21},
  [1729] = {.lex_state = 21},
  [1730] = {.lex_state = 21},
  [1731] = {.lex_state = 21},
  [1732] = {.lex_state = 21},
  [1733] = {.lex_state = 21},
  [1734] = {.lex_state = 21},
  [1735] = {.lex_state = 21},
  [1736] = {.lex_state = 21},
  [1737] = {.lex_state = 21},
  [1738] = {.lex_state = 21},
  [1739] = {.lex_state = 21},
  [1740] = {.lex_state = 21},
  [1741] = {.lex_state = 21},
  [1742] = {.lex_state = 21},
  [1743] = {.lex_state = 21},
  [1744] = {.lex_state = 21},
  [1745] = {.lex_state = 21},
  [1746] = {.lex_state = 21},
  [1747] = {.lex_state = 21},
  [1748] = {.lex_state = 21},
  [1749] = {.lex_state = 21},
  [1750] = {.lex_state = 21},
  [1751] = {.lex_state = 21},
  [1752] = {.lex_state = 21},
  [1753] = {.lex_state = 21},
  [1754] = {.lex_state = 21},
  [1755] = {.lex_state = 21},
  [1756] = {.lex_state = 21},
  [1757] = {.lex_state = 21},
  [1758] = {.lex_state = 4},
  [1759] = {.lex_state = 5},
  [1760] = {.lex_state = 21},
  [1761] = {.lex_state = 21},
  [1762] = {.lex_state = 21},
  [1763] = {.lex_state = 21},
  [1764] = {.lex_state = 21},
  [1765] = {.lex_state = 21},
  [1766] = {.lex_state = 21},
  [1767] = {.lex_state = 21},
  [1768] = {.lex_state = 21},
  [1769] = {.lex_state = 21},
  [1770] = {.lex_state = 21},
  [1771] = {.lex_state = 21},
  [1772] = {.lex_state = 21},
  [1773] = {.lex_state = 21},
  [1774] = {.lex_state = 21},
  [1775] = {.lex_state = 21},
  [1776] = {.lex_state = 21},
  [1777] = {.lex_state = 4},
  [1778] = {.lex_state = 4},
  [1779] = {.lex_state = 21},
  [1780] = {.lex_state = 21},
  [1781] = {.lex_state = 21},
  [1782] = {.lex_state = 21},
  [1783] = {.lex_state = 21},
  [1784] = {.lex_state = 21},
  [1785] = {.lex_state = 21},
  [1786] = {.lex_state = 21},
  [1787] = {.lex_state = 21},
  [1788] = {.lex_state = 21},
  [1789] = {.lex_state = 4},
  [1790] = {.lex_state = 21},
  [1791] = {.lex_state = 21},
  [1792] = {.lex_state = 21},
  [1793] = {.lex_state = 21},
  [1794] = {.lex_state = 4},
  [1795] = {.lex_state = 21},
  [1796] = {.lex_state = 21},
  [1797] = {.lex_state = 21},
  [1798] = {.lex_state = 21},
  [1799] = {.lex_state = 21},
  [1800] = {.lex_state = 21},
  [1801] = {.lex_state = 21},
  [1802] = {.lex_state = 4},
  [1803] = {.lex_state = 4},
  [1804] = {.lex_state = 21},
  [1805] = {.lex_state = 21},
  [1806] = {.lex_state = 21},
  [1807] = {.lex_state = 21},
  [1808] = {.lex_state = 21},
  [1809] = {.lex_state = 4},
  [1810] = {.lex_state = 21},
  [1811] = {.lex_state = 4},
  [1812] = {.lex_state = 21},
  [1813] = {.lex_state = 21},
  [1814] = {.lex_state = 21},
  [1815] = {.lex_state = 21},
  [1816] = {.lex_state = 21},
  [1817] = {.lex_state = 21},
  [1818] = {.lex_state = 21},
  [1819] = {.lex_state = 4},
  [1820] = {.lex_state = 21},
  [1821] = {.lex_state = 21},
  [1822] = {.lex_state = 21},
  [1823] = {.lex_state = 21},
  [1824] = {.lex_state = 21},
  [1825] = {.lex_state = 21},
  [1826] = {.lex_state = 21},
  [1827] = {.lex_state = 21},
  [1828] = {.lex_state = 21},
  [1829] = {.lex_state = 21},
  [1830] = {.lex_state = 21},
  [1831] = {.lex_state = 21},
  [1832] = {.lex_state = 21},
  [1833] = {.lex_state = 21},
  [1834] = {.lex_state = 21},
  [1835] = {.lex_state = 2},
  [1836] = {.lex_state = 21},
  [1837] = {.lex_state = 21},
  [1838] = {.lex_state = 21},
  [1839] = {.lex_state = 21},
  [1840] = {.lex_state = 21},
  [1841] = {.lex_state = 4},
  [1842] = {.lex_state = 21},
  [1843] = {.lex_state = 21},
  [1844] = {.lex_state = 21},
  [1845] = {.lex_state = 21},
  [1846] = {.lex_state = 21},
  [1847] = {.lex_state = 21},
  [1848] = {.lex_state = 21},
  [1849] = {.lex_state = 21},
  [1850] = {.lex_state = 21},
  [1851] = {.lex_state = 21},
  [1852] = {.lex_state = 21},
  [1853] = {.lex_state = 21},
  [1854] = {.lex_state = 21},
  [1855] = {.lex_state = 21},
  [1856] = {.lex_state = 21},
  [1857] = {.lex_state = 21},
  [1858] = {.lex_state = 21},
  [1859] = {.lex_state = 21},
  [1860] = {.lex_state = 21},
  [1861] = {.lex_state = 21},
  [1862] = {.lex_state = 4},
  [1863] = {.lex_state = 21},
  [1864] = {.lex_state = 4},
  [1865] = {.lex_state = 21},
  [1866] = {.lex_state = 21},
  [1867] = {.lex_state = 4},
  [1868] = {.lex_state = 21},
  [1869] = {.lex_state = 21},
  [1870] = {.lex_state = 21},
  [1871] = {.lex_state = 21},
  [1872] = {.lex_state = 4},
  [1873] = {.lex_state = 21},
  [1874] = {.lex_state = 21},
  [1875] = {.lex_state = 21},
  [1876] = {.lex_state = 4},
  [1877] = {.lex_state = 21},
  [1878] = {.lex_state = 21},
  [1879] = {.lex_state = 21},
  [1880] = {.lex_state = 21},
  [1881] = {.lex_state = 21},
  [1882] = {.lex_state = 21},
  [1883] = {.lex_state = 21},
  [1884] = {.lex_state = 21},
  [1885] = {.lex_state = 21},
  [1886] = {.lex_state = 21},
  [1887] = {.lex_state = 21},
  [1888] = {.lex_state = 21},
  [1889] = {.lex_state = 21},
  [1890] = {.lex_state = 21},
  [1891] = {.lex_state = 21},
  [1892] = {.lex_state = 21},
  [1893] = {.lex_state = 21},
  [1894] = {.lex_state = 21},
  [1895] = {.lex_state = 4},
  [1896] = {.lex_state = 4},
  [1897] = {.lex_state = 21},
  [1898] = {.lex_state = 21},
  [1899] = {.lex_state = 21},
  [1900] = {.lex_state = 21},
  [1901] = {.lex_state = 21},
  [1902] = {.lex_state = 21},
  [1903] = {.lex_state = 21},
  [1904] = {.lex_state = 21},
  [1905] = {.lex_state = 21},
  [1906] = {.lex_state = 21},
  [1907] = {.lex_state = 4},
  [1908] = {.lex_state = 21},
  [1909] = {.lex_state = 21},
  [1910] = {.lex_state = 21},
  [1911] = {.lex_state = 4},
  [1912] = {.lex_state = 21},
  [1913] = {.lex_state = 21},
  [1914] = {.lex_state = 21},
  [1915] = {.lex_state = 21},
  [1916] = {.lex_state = 4},
  [1917] = {.lex_state = 21},
  [1918] = {.lex_state = 21},
  [1919] = {.lex_state = 21},
  [1920] = {.lex_state = 21},
  [1921] = {.lex_state = 4},
  [1922] = {.lex_state = 21},
  [1923] = {.lex_state = 21},
  [1924] = {.lex_state = 21},
  [1925] = {.lex_state = 21},
  [1926] = {.lex_state = 21},
  [1927] = {.lex_state = 21},
  [1928] = {.lex_state = 21},
  [1929] = {.lex_state = 21},
  [1930] = {.lex_state = 21},
  [1931] = {.lex_state = 21},
  [1932] = {.lex_state = 21},
  [1933] = {.lex_state = 21},
  [1934] = {.lex_state = 95},
  [1935] = {.lex_state = 21},
  [1936] = {.lex_state = 21},
  [1937] = {.lex_state = 21},
  [1938] = {.lex_state = 5, .external_lex_state = 2},
  [1939] = {.lex_state = 21},
  [1940] = {.lex_state = 21},
  [1941] = {.lex_state = 21},
  [1942] = {.lex_state = 21},
  [1943] = {.lex_state = 21},
  [1944] = {.lex_state = 21},
  [1945] = {.lex_state = 21},
  [1946] = {.lex_state = 4},
  [1947] = {.lex_state = 21},
  [1948] = {.lex_state = 21},
  [1949] = {.lex_state = 5},
  [1950] = {.lex_state = 21},
  [1951] = {.lex_state = 21},
  [1952] = {.lex_state = 21},
  [1953] = {.lex_state = 21},
  [1954] = {.lex_state = 21},
  [1955] = {.lex_state = 4},
  [1956] = {.lex_state = 21},
  [1957] = {.lex_state = 21},
  [1958] = {.lex_state = 21},
  [1959] = {.lex_state = 21},
  [1960] = {.lex_state = 21},
  [1961] = {.lex_state = 21},
  [1962] = {.lex_state = 21},
  [1963] = {.lex_state = 21},
  [1964] = {.lex_state = 21},
  [1965] = {.lex_state = 21},
  [1966] = {.lex_state = 21},
  [1967] = {.lex_state = 21},
  [1968] = {.lex_state = 21},
  [1969] = {.lex_state = 21},
  [1970] = {.lex_state = 21},
  [1971] = {.lex_state = 21},
  [1972] = {.lex_state = 4},
  [1973] = {.lex_state = 4},
  [1974] = {.lex_state = 21},
  [1975] = {.lex_state = 4},
  [1976] = {.lex_state = 21},
  [1977] = {.lex_state = 21},
  [1978] = {.lex_state = 21},
  [1979] = {.lex_state = 21},
  [1980] = {.lex_state = 21},
  [1981] = {.lex_state = 21},
  [1982] = {.lex_state = 21},
  [1983] = {.lex_state = 21},
  [1984] = {.lex_state = 21},
  [1985] = {.lex_state = 21},
  [1986] = {.lex_state = 21},
  [1987] = {.lex_state = 21},
  [1988] = {.lex_state = 21},
  [1989] = {.lex_state = 21},
  [1990] = {.lex_state = 21},
  [1991] = {.lex_state = 4},
  [1992] = {.lex_state = 21},
  [1993] = {.lex_state = 4},
  [1994] = {.lex_state = 4},
  [1995] = {.lex_state = 21},
  [1996] = {.lex_state = 21},
  [1997] = {.lex_state = 21},
  [1998] = {.lex_state = 4},
  [1999] = {.lex_state = 4},
  [2000] = {.lex_state = 21},
  [2001] = {.lex_state = 21},
  [2002] = {.lex_state = 21},
  [2003] = {.lex_state = 21},
  [2004] = {.lex_state = 21},
  [2005] = {.lex_state = 21},
  [2006] = {.lex_state = 21},
  [2007] = {.lex_state = 21},
  [2008] = {.lex_state = 21},
  [2009] = {.lex_state = 21},
  [2010] = {.lex_state = 21},
  [2011] = {.lex_state = 21},
  [2012] = {.lex_state = 21},
  [2013] = {.lex_state = 21},
  [2014] = {.lex_state = 21},
  [2015] = {.lex_state = 21},
  [2016] = {.lex_state = 21},
  [2017] = {.lex_state = 21},
  [2018] = {.lex_state = 21},
  [2019] = {.lex_state = 21},
  [2020] = {.lex_state = 21},
  [2021] = {.lex_state = 21},
  [2022] = {.lex_state = 21},
  [2023] = {.lex_state = 21},
  [2024] = {.lex_state = 21},
  [2025] = {.lex_state = 21},
  [2026] = {.lex_state = 21},
  [2027] = {.lex_state = 21},
  [2028] = {.lex_state = 21},
  [2029] = {.lex_state = 21},
  [2030] = {.lex_state = 4},
  [2031] = {.lex_state = 21},
  [2032] = {.lex_state = 21},
  [2033] = {.lex_state = 21},
  [2034] = {.lex_state = 21},
  [2035] = {.lex_state = 4},
  [2036] = {.lex_state = 21},
  [2037] = {.lex_state = 21},
  [2038] = {.lex_state = 21},
  [2039] = {.lex_state = 21},
  [2040] = {.lex_state = 21},
  [2041] = {.lex_state = 21},
  [2042] = {.lex_state = 21},
  [2043] = {.lex_state = 21},
  [2044] = {.lex_state = 21},
  [2045] = {.lex_state = 21},
  [2046] = {.lex_state = 4},
  [2047] = {.lex_state = 21},
  [2048] = {.lex_state = 21},
  [2049] = {.lex_state = 21},
  [2050] = {.lex_state = 21},
  [2051] = {.lex_state = 21},
  [2052] = {.lex_state = 21},
  [2053] = {.lex_state = 21},
  [2054] = {.lex_state = 21},
  [2055] = {.lex_state = 21},
  [2056] = {.lex_state = 21},
  [2057] = {.lex_state = 21},
  [2058] = {.lex_state = 21},
  [2059] = {.lex_state = 21},
  [2060] = {.lex_state = 21},
  [2061] = {.lex_state = 21},
  [2062] = {.lex_state = 21},
  [2063] = {.lex_state = 21},
  [2064] = {.lex_state = 21},
  [2065] = {.lex_state = 21},
  [2066] = {.lex_state = 21},
  [2067] = {.lex_state = 21},
  [2068] = {.lex_state = 21},
  [2069] = {.lex_state = 21},
  [2070] = {.lex_state = 21},
  [2071] = {.lex_state = 21},
  [2072] = {.lex_state = 21},
  [2073] = {.lex_state = 21},
  [2074] = {.lex_state = 21},
  [2075] = {.lex_state = 21},
  [2076] = {.lex_state = 21},
  [2077] = {.lex_state = 21},
  [2078] = {.lex_state = 21},
  [2079] = {.lex_state = 21},
  [2080] = {.lex_state = 21},
  [2081] = {.lex_state = 2},
  [2082] = {.lex_state = 21},
  [2083] = {.lex_state = 21},
  [2084] = {.lex_state = 21},
  [2085] = {.lex_state = 21},
  [2086] = {.lex_state = 21},
  [2087] = {.lex_state = 21},
  [2088] = {.lex_state = 21},
  [2089] = {.lex_state = 21},
  [2090] = {.lex_state = 21},
  [2091] = {.lex_state = 21},
  [2092] = {.lex_state = 21},
  [2093] = {.lex_state = 21},
  [2094] = {.lex_state = 21},
  [2095] = {.lex_state = 21},
  [2096] = {.lex_state = 21},
  [2097] = {.lex_state = 5, .external_lex_state = 3},
  [2098] = {.lex_state = 21},
  [2099] = {.lex_state = 21},
  [2100] = {.lex_state = 21},
  [2101] = {.lex_state = 21},
  [2102] = {.lex_state = 21},
  [2103] = {.lex_state = 21},
  [2104] = {.lex_state = 21},
  [2105] = {.lex_state = 21},
  [2106] = {.lex_state = 21},
  [2107] = {.lex_state = 21},
  [2108] = {.lex_state = 21},
  [2109] = {.lex_state = 4},
  [2110] = {.lex_state = 21},
  [2111] = {.lex_state = 21},
  [2112] = {.lex_state = 21},
  [2113] = {.lex_state = 21},
  [2114] = {.lex_state = 21},
  [2115] = {.lex_state = 21},
  [2116] = {.lex_state = 21},
  [2117] = {.lex_state = 21},
  [2118] = {.lex_state = 21},
  [2119] = {.lex_state = 21},
  [2120] = {.lex_state = 21},
  [2121] = {.lex_state = 21},
  [2122] = {.lex_state = 21},
  [2123] = {.lex_state = 21},
  [2124] = {.lex_state = 21},
  [2125] = {.lex_state = 21},
  [2126] = {.lex_state = 21},
  [2127] = {.lex_state = 21},
  [2128] = {.lex_state = 21},
  [2129] = {.lex_state = 21},
  [2130] = {.lex_state = 21},
  [2131] = {.lex_state = 21},
  [2132] = {.lex_state = 21},
  [2133] = {.lex_state = 21},
  [2134] = {.lex_state = 21},
  [2135] = {.lex_state = 21},
  [2136] = {.lex_state = 21},
  [2137] = {.lex_state = 21},
  [2138] = {.lex_state = 21},
  [2139] = {.lex_state = 21},
  [2140] = {.lex_state = 21},
  [2141] = {.lex_state = 4},
  [2142] = {.lex_state = 21},
  [2143] = {.lex_state = 21},
  [2144] = {.lex_state = 21},
  [2145] = {.lex_state = 21},
  [2146] = {.lex_state = 21},
  [2147] = {.lex_state = 21},
  [2148] = {.lex_state = 21},
  [2149] = {.lex_state = 21},
  [2150] = {.lex_state = 21},
  [2151] = {.lex_state = 21},
  [2152] = {.lex_state = 21},
  [2153] = {.lex_state = 21},
  [2154] = {.lex_state = 21},
  [2155] = {.lex_state = 21},
  [2156] = {.lex_state = 21},
  [2157] = {.lex_state = 21},
  [2158] = {.lex_state = 21},
  [2159] = {.lex_state = 21},
  [2160] = {.lex_state = 21},
  [2161] = {.lex_state = 21},
  [2162] = {.lex_state = 21},
  [2163] = {.lex_state = 21},
  [2164] = {.lex_state = 21},
  [2165] = {.lex_state = 21},
  [2166] = {.lex_state = 21},
  [2167] = {.lex_state = 21},
  [2168] = {.lex_state = 21},
  [2169] = {.lex_state = 21},
  [2170] = {.lex_state = 21},
  [2171] = {.lex_state = 11},
  [2172] = {.lex_state = 21},
  [2173] = {.lex_state = 21},
  [2174] = {.lex_state = 21},
  [2175] = {.lex_state = 21},
  [2176] = {.lex_state = 21},
  [2177] = {.lex_state = 21},
  [2178] = {.lex_state = 21},
  [2179] = {.lex_state = 21},
  [2180] = {.lex_state = 21},
  [2181] = {.lex_state = 21},
  [2182] = {.lex_state = 21},
  [2183] = {.lex_state = 21},
  [2184] = {.lex_state = 21},
  [2185] = {.lex_state = 21},
  [2186] = {.lex_state = 21},
  [2187] = {.lex_state = 21},
  [2188] = {.lex_state = 21},
  [2189] = {.lex_state = 21},
  [2190] = {.lex_state = 21},
  [2191] = {.lex_state = 21},
  [2192] = {.lex_state = 21},
  [2193] = {.lex_state = 21},
  [2194] = {.lex_state = 21},
  [2195] = {.lex_state = 21},
  [2196] = {.lex_state = 21},
  [2197] = {.lex_state = 21},
  [2198] = {.lex_state = 21},
  [2199] = {.lex_state = 21},
  [2200] = {.lex_state = 21},
  [2201] = {.lex_state = 21},
  [2202] = {.lex_state = 21},
  [2203] = {.lex_state = 21},
  [2204] = {.lex_state = 21},
  [2205] = {.lex_state = 21},
  [2206] = {.lex_state = 21},
  [2207] = {.lex_state = 21},
  [2208] = {.lex_state = 21},
  [2209] = {.lex_state = 21},
  [2210] = {.lex_state = 21},
  [2211] = {.lex_state = 21},
  [2212] = {.lex_state = 21},
  [2213] = {.lex_state = 21},
  [2214] = {.lex_state = 21},
  [2215] = {.lex_state = 21},
  [2216] = {.lex_state = 21},
  [2217] = {.lex_state = 21},
  [2218] = {.lex_state = 21},
  [2219] = {.lex_state = 21},
  [2220] = {.lex_state = 21},
  [2221] = {.lex_state = 21},
  [2222] = {.lex_state = 21},
  [2223] = {.lex_state = 21},
  [2224] = {.lex_state = 21},
  [2225] = {.lex_state = 21},
  [2226] = {.lex_state = 21},
  [2227] = {.lex_state = 21},
  [2228] = {.lex_state = 21},
  [2229] = {.lex_state = 21},
  [2230] = {.lex_state = 21},
  [2231] = {.lex_state = 21},
  [2232] = {.lex_state = 21},
  [2233] = {.lex_state = 21},
  [2234] = {.lex_state = 21},
  [2235] = {.lex_state = 21},
  [2236] = {.lex_state = 21},
  [2237] = {.lex_state = 21},
  [2238] = {.lex_state = 21},
  [2239] = {.lex_state = 21},
  [2240] = {.lex_state = 21},
  [2241] = {.lex_state = 21},
  [2242] = {.lex_state = 4},
  [2243] = {.lex_state = 21},
  [2244] = {.lex_state = 21},
  [2245] = {.lex_state = 21},
  [2246] = {.lex_state = 21},
  [2247] = {.lex_state = 21},
  [2248] = {.lex_state = 21},
  [2249] = {.lex_state = 21},
  [2250] = {.lex_state = 21},
  [2251] = {.lex_state = 21},
  [2252] = {.lex_state = 21},
  [2253] = {.lex_state = 21},
  [2254] = {.lex_state = 21},
  [2255] = {.lex_state = 21},
  [2256] = {.lex_state = 21},
  [2257] = {.lex_state = 21},
  [2258] = {.lex_state = 21},
  [2259] = {.lex_state = 21},
  [2260] = {.lex_state = 21},
  [2261] = {.lex_state = 21},
  [2262] = {.lex_state = 21},
  [2263] = {.lex_state = 21},
  [2264] = {.lex_state = 21},
  [2265] = {.lex_state = 21},
  [2266] = {.lex_state = 21},
  [2267] = {.lex_state = 21},
  [2268] = {.lex_state = 21},
  [2269] = {.lex_state = 21},
  [2270] = {.lex_state = 21},
  [2271] = {.lex_state = 21},
  [2272] = {.lex_state = 21},
  [2273] = {.lex_state = 21},
  [2274] = {.lex_state = 21},
  [2275] = {.lex_state = 21},
  [2276] = {.lex_state = 21},
  [2277] = {.lex_state = 21},
  [2278] = {.lex_state = 4},
  [2279] = {.lex_state = 21},
  [2280] = {.lex_state = 21},
  [2281] = {.lex_state = 21},
  [2282] = {.lex_state = 21},
  [2283] = {.lex_state = 21},
  [2284] = {.lex_state = 21},
  [2285] = {.lex_state = 21},
  [2286] = {.lex_state = 21},
  [2287] = {.lex_state = 21},
  [2288] = {.lex_state = 21},
  [2289] = {.lex_state = 21},
  [2290] = {.lex_state = 21},
  [2291] = {.lex_state = 21},
  [2292] = {.lex_state = 21},
  [2293] = {.lex_state = 21},
  [2294] = {.lex_state = 21},
  [2295] = {.lex_state = 21},
  [2296] = {.lex_state = 21},
  [2297] = {.lex_state = 21},
  [2298] = {.lex_state = 21},
  [2299] = {.lex_state = 21},
  [2300] = {.lex_state = 21},
  [2301] = {.lex_state = 12},
  [2302] = {.lex_state = 21},
  [2303] = {.lex_state = 21},
  [2304] = {.lex_state = 21},
  [2305] = {.lex_state = 21},
  [2306] = {.lex_state = 21},
  [2307] = {.lex_state = 21},
  [2308] = {.lex_state = 21},
  [2309] = {.lex_state = 21},
  [2310] = {.lex_state = 21},
  [2311] = {.lex_state = 21},
  [2312] = {.lex_state = 21},
  [2313] = {.lex_state = 21},
  [2314] = {.lex_state = 21},
  [2315] = {.lex_state = 21},
  [2316] = {.lex_state = 4},
  [2317] = {.lex_state = 21},
  [2318] = {.lex_state = 21},
  [2319] = {.lex_state = 21},
  [2320] = {.lex_state = 21},
  [2321] = {.lex_state = 21},
  [2322] = {.lex_state = 21},
  [2323] = {.lex_state = 21},
  [2324] = {.lex_state = 21},
  [2325] = {.lex_state = 21},
  [2326] = {.lex_state = 21},
  [2327] = {.lex_state = 21},
  [2328] = {.lex_state = 21},
  [2329] = {.lex_state = 21},
  [2330] = {.lex_state = 21},
  [2331] = {.lex_state = 21},
  [2332] = {.lex_state = 21},
  [2333] = {.lex_state = 5},
  [2334] = {.lex_state = 21},
  [2335] = {.lex_state = 21},
  [2336] = {.lex_state = 21},
  [2337] = {.lex_state = 21},
  [2338] = {.lex_state = 21},
  [2339] = {.lex_state = 21},
  [2340] = {.lex_state = 21},
  [2341] = {.lex_state = 21},
  [2342] = {.lex_state = 21},
  [2343] = {.lex_state = 21},
  [2344] = {.lex_state = 21},
  [2345] = {.lex_state = 21},
  [2346] = {.lex_state = 21},
  [2347] = {.lex_state = 21},
  [2348] = {.lex_state = 21},
  [2349] = {.lex_state = 21},
  [2350] = {.lex_state = 21},
  [2351] = {.lex_state = 21},
  [2352] = {.lex_state = 21},
  [2353] = {.lex_state = 21},
  [2354] = {.lex_state = 21},
  [2355] = {.lex_state = 21},
  [2356] = {.lex_state = 21},
  [2357] = {.lex_state = 21},
  [2358] = {.lex_state = 21},
  [2359] = {.lex_state = 21},
  [2360] = {.lex_state = 21},
  [2361] = {.lex_state = 21},
  [2362] = {.lex_state = 21},
  [2363] = {.lex_state = 21},
  [2364] = {.lex_state = 21},
  [2365] = {.lex_state = 21},
  [2366] = {.lex_state = 21},
  [2367] = {.lex_state = 21},
  [2368] = {.lex_state = 21},
  [2369] = {.lex_state = 21},
  [2370] = {.lex_state = 21},
  [2371] = {.lex_state = 21},
  [2372] = {.lex_state = 21},
  [2373] = {.lex_state = 21},
  [2374] = {.lex_state = 21},
  [2375] = {.lex_state = 21},
  [2376] = {.lex_state = 21},
  [2377] = {.lex_state = 21},
  [2378] = {.lex_state = 21},
  [2379] = {.lex_state = 21},
  [2380] = {.lex_state = 21},
  [2381] = {.lex_state = 21},
  [2382] = {.lex_state = 21},
  [2383] = {.lex_state = 21},
  [2384] = {.lex_state = 21},
  [2385] = {.lex_state = 21},
  [2386] = {.lex_state = 21},
  [2387] = {.lex_state = 21},
  [2388] = {.lex_state = 21},
  [2389] = {.lex_state = 21},
  [2390] = {.lex_state = 21},
  [2391] = {.lex_state = 21},
  [2392] = {.lex_state = 21},
  [2393] = {.lex_state = 21},
  [2394] = {.lex_state = 21},
  [2395] = {.lex_state = 21},
  [2396] = {.lex_state = 21},
  [2397] = {.lex_state = 21},
  [2398] = {.lex_state = 21},
  [2399] = {.lex_state = 21},
  [2400] = {.lex_state = 4},
  [2401] = {.lex_state = 21},
  [2402] = {.lex_state = 5},
  [2403] = {.lex_state = 21},
  [2404] = {.lex_state = 21},
  [2405] = {.lex_state = 21},
  [2406] = {.lex_state = 21},
  [2407] = {.lex_state = 21},
  [2408] = {.lex_state = 21},
  [2409] = {.lex_state = 21},
  [2410] = {.lex_state = 21},
  [2411] = {.lex_state = 21},
  [2412] = {.lex_state = 21},
  [2413] = {.lex_state = 21},
  [2414] = {.lex_state = 21},
  [2415] = {.lex_state = 21},
  [2416] = {.lex_state = 21},
  [2417] = {.lex_state = 21},
  [2418] = {.lex_state = 21},
  [2419] = {.lex_state = 21},
  [2420] = {.lex_state = 21},
  [2421] = {.lex_state = 21},
  [2422] = {.lex_state = 21},
  [2423] = {.lex_state = 21},
  [2424] = {.lex_state = 21},
  [2425] = {.lex_state = 21},
  [2426] = {.lex_state = 21},
  [2427] = {.lex_state = 21},
  [2428] = {.lex_state = 21},
  [2429] = {.lex_state = 21},
  [2430] = {.lex_state = 4},
  [2431] = {.lex_state = 21},
  [2432] = {.lex_state = 21},
  [2433] = {.lex_state = 91},
  [2434] = {.lex_state = 13},
  [2435] = {.lex_state = 21},
  [2436] = {.lex_state = 21},
  [2437] = {.lex_state = 21},
  [2438] = {.lex_state = 21},
  [2439] = {.lex_state = 21},
  [2440] = {.lex_state = 21},
  [2441] = {.lex_state = 21},
  [2442] = {.lex_state = 21},
  [2443] = {.lex_state = 21},
  [2444] = {.lex_state = 21},
  [2445] = {.lex_state = 21},
  [2446] = {.lex_state = 21, .external_lex_state = 4},
  [2447] = {.lex_state = 21},
  [2448] = {.lex_state = 21},
  [2449] = {.lex_state = 4},
  [2450] = {.lex_state = 21},
  [2451] = {.lex_state = 21},
  [2452] = {.lex_state = 4},
  [2453] = {.lex_state = 21},
  [2454] = {.lex_state = 21},
  [2455] = {.lex_state = 21},
  [2456] = {.lex_state = 21},
  [2457] = {.lex_state = 21},
  [2458] = {.lex_state = 21},
  [2459] = {(TSStateId)(-1),},
  [2460] = {(TSStateId)(-1),},
  [2461] = {(TSStateId)(-1),},
  [2462] = {(TSStateId)(-1),},
  [2463] = {(TSStateId)(-1),},
};

static const uint16_t ts_parse_table[LARGE_STATE_COUNT][SYMBOL_COUNT] = {
  [STATE(0)] = {
    [sym_line_comment] = STATE(0),
    [sym_block_comment] = STATE(0),
    [ts_builtin_sym_end] = ACTIONS(1),
    [sym_identifier] = ACTIONS(1),
    [anon_sym_u8] = ACTIONS(1),
    [anon_sym_u16] = ACTIONS(1),
    [anon_sym_u32] = ACTIONS(1),
    [anon_sym_u64] = ACTIONS(1),
    [anon_sym_u128] = ACTIONS(1),
    [anon_sym_u256] = ACTIONS(1),
    [anon_sym_exists] = ACTIONS(1),
    [anon_sym_forall] = ACTIONS(1),
    [anon_sym_choose] = ACTIONS(1),
    [anon_sym_min] = ACTIONS(1),
    [anon_sym_bool] = ACTIONS(1),
    [anon_sym_address] = ACTIONS(1),
    [anon_sym_vector] = ACTIONS(1),
    [anon_sym_for] = ACTIONS(1),
    [anon_sym_while] = ACTIONS(1),
    [anon_sym_friend] = ACTIONS(1),
    [anon_sym_match] = ACTIONS(1),
    [aux_sym_number_token1] = ACTIONS(1),
    [aux_sym_number_token5] = ACTIONS(1),
    [anon_sym_true] = ACTIONS(1),
    [anon_sym_false] = ACTIONS(1),
    [aux_sym_byte_string_token1] = ACTIONS(1),
    [aux_sym_byte_string_token2] = ACTIONS(1),
    [anon_sym_STAR] = ACTIONS(1),
    [anon_sym_COLON_COLON] = ACTIONS(1),
    [anon_sym_LPAREN] = ACTIONS(1),
    [anon_sym_COMMA] = ACTIONS(1),
    [anon_sym_RPAREN] = ACTIONS(1),
    [anon_sym_PIPE] = ACTIONS(1),
    [anon_sym_AMP] = ACTIONS(1),
    [anon_sym_AMPmut] = ACTIONS(1),
    [anon_sym_EQ] = ACTIONS(1),
    [anon_sym_where] = ACTIONS(1),
    [anon_sym_COLON] = ACTIONS(1),
    [anon_sym_in] = ACTIONS(1),
    [anon_sym_LBRACE] = ACTIONS(1),
    [anon_sym_RBRACE] = ACTIONS(1),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1),
    [anon_sym_PIPE_PIPE] = ACTIONS(1),
    [anon_sym_AMP_AMP] = ACTIONS(1),
    [anon_sym_EQ_EQ] = ACTIONS(1),
    [anon_sym_BANG_EQ] = ACTIONS(1),
    [anon_sym_LT] = ACTIONS(1),
    [anon_sym_GT] = ACTIONS(1),
    [anon_sym_LT_EQ] = ACTIONS(1),
    [anon_sym_GT_EQ] = ACTIONS(1),
    [anon_sym_DOT_DOT] = ACTIONS(1),
    [anon_sym_CARET] = ACTIONS(1),
    [anon_sym_LT_LT] = ACTIONS(1),
    [anon_sym_GT_GT] = ACTIONS(1),
    [anon_sym_PLUS] = ACTIONS(1),
    [anon_sym_DASH] = ACTIONS(1),
    [anon_sym_SLASH] = ACTIONS(1),
    [anon_sym_PERCENT] = ACTIONS(1),
    [anon_sym_BANG] = ACTIONS(1),
    [anon_sym_move] = ACTIONS(1),
    [anon_sym_copy] = ACTIONS(1),
    [anon_sym_DOT] = ACTIONS(1),
    [anon_sym_LBRACK] = ACTIONS(1),
    [anon_sym_RBRACK] = ACTIONS(1),
    [anon_sym_break] = ACTIONS(1),
    [anon_sym_continue] = ACTIONS(1),
    [anon_sym_as] = ACTIONS(1),
    [anon_sym_if] = ACTIONS(1),
    [anon_sym_EQ_GT] = ACTIONS(1),
    [anon_sym_else] = ACTIONS(1),
    [anon_sym_loop] = ACTIONS(1),
    [anon_sym_return] = ACTIONS(1),
    [anon_sym_abort] = ACTIONS(1),
    [anon_sym_LT2] = ACTIONS(1),
    [anon_sym_AT] = ACTIONS(1),
    [anon_sym_POUND] = ACTIONS(1),
    [anon_sym_module] = ACTIONS(1),
    [anon_sym_spec] = ACTIONS(1),
    [anon_sym_schema] = ACTIONS(1),
    [anon_sym_invariant] = ACTIONS(1),
    [anon_sym_update] = ACTIONS(1),
    [anon_sym_SEMI] = ACTIONS(1),
    [anon_sym_assert] = ACTIONS(1),
    [anon_sym_assume] = ACTIONS(1),
    [anon_sym_ensures] = ACTIONS(1),
    [anon_sym_requires] = ACTIONS(1),
    [anon_sym_aborts_if] = ACTIONS(1),
    [anon_sym_with] = ACTIONS(1),
    [anon_sym_aborts_with] = ACTIONS(1),
    [anon_sym_modifies] = ACTIONS(1),
    [anon_sym_emits] = ACTIONS(1),
    [anon_sym_to] = ACTIONS(1),
    [anon_sym_global] = ACTIONS(1),
    [anon_sym_local] = ACTIONS(1),
    [anon_sym_fun] = ACTIONS(1),
    [anon_sym_native] = ACTIONS(1),
    [anon_sym_include] = ACTIONS(1),
    [anon_sym_apply] = ACTIONS(1),
    [anon_sym_except] = ACTIONS(1),
    [anon_sym_public] = ACTIONS(1),
    [anon_sym_internal] = ACTIONS(1),
    [anon_sym_pragma] = ACTIONS(1),
    [anon_sym_let] = ACTIONS(1),
    [anon_sym_post] = ACTIONS(1),
    [anon_sym_axiom] = ACTIONS(1),
    [anon_sym_script] = ACTIONS(1),
    [anon_sym_package] = ACTIONS(1),
    [anon_sym_entry] = ACTIONS(1),
    [anon_sym_use] = ACTIONS(1),
    [anon_sym_const] = ACTIONS(1),
    [anon_sym_inline] = ACTIONS(1),
    [anon_sym_pure] = ACTIONS(1),
    [anon_sym_acquires] = ACTIONS(1),
    [anon_sym_reads] = ACTIONS(1),
    [anon_sym_writes] = ACTIONS(1),
    [anon_sym_struct] = ACTIONS(1),
    [sym_phantom] = ACTIONS(1),
    [anon_sym_enum] = ACTIONS(1),
    [anon_sym_drop] = ACTIONS(1),
    [anon_sym_store] = ACTIONS(1),
    [anon_sym_key] = ACTIONS(1),
    [anon_sym_has] = ACTIONS(1),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_SLASH2] = ACTIONS(1),
    [anon_sym_SLASH2] = ACTIONS(1),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [anon_sym_STAR_SLASH] = ACTIONS(1),
    [sym__block_doc_comment_marker] = ACTIONS(1),
    [sym__block_comment_content] = ACTIONS(1),
    [sym__doc_line_comment] = ACTIONS(1),
    [sym__error_sentinel] = ACTIONS(1),
  },
  [STATE(1)] = {
    [sym_source_file] = STATE(2362),
    [sym_attributes] = STATE(1662),
    [sym_address_block] = STATE(1767),
    [sym__module_keyword] = STATE(1646),
    [sym_module] = STATE(1767),
    [sym_script] = STATE(1767),
    [sym_line_comment] = STATE(1),
    [sym_block_comment] = STATE(1),
    [aux_sym_source_file_repeat1] = STATE(1592),
    [aux_sym_attributes_repeat2] = STATE(1536),
    [ts_builtin_sym_end] = ACTIONS(7),
    [anon_sym_address] = ACTIONS(9),
    [anon_sym_POUND] = ACTIONS(11),
    [anon_sym_module] = ACTIONS(13),
    [anon_sym_spec] = ACTIONS(15),
    [anon_sym_script] = ACTIONS(17),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(2)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(894),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(477),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1188),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1194),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(2),
    [sym_block_comment] = STATE(2),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(27),
    [anon_sym_forall] = ACTIONS(27),
    [anon_sym_choose] = ACTIONS(29),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(57),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_COMMA] = ACTIONS(57),
    [anon_sym_RPAREN] = ACTIONS(57),
    [anon_sym_PIPE] = ACTIONS(61),
    [anon_sym_AMP] = ACTIONS(61),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_EQ] = ACTIONS(61),
    [anon_sym_where] = ACTIONS(61),
    [anon_sym_COLON] = ACTIONS(57),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_RBRACE] = ACTIONS(57),
    [anon_sym_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_PIPE_PIPE] = ACTIONS(57),
    [anon_sym_AMP_AMP] = ACTIONS(57),
    [anon_sym_EQ_EQ] = ACTIONS(61),
    [anon_sym_BANG_EQ] = ACTIONS(57),
    [anon_sym_LT] = ACTIONS(61),
    [anon_sym_GT] = ACTIONS(61),
    [anon_sym_LT_EQ] = ACTIONS(61),
    [anon_sym_GT_EQ] = ACTIONS(57),
    [anon_sym_DOT_DOT] = ACTIONS(57),
    [anon_sym_CARET] = ACTIONS(57),
    [anon_sym_LT_LT] = ACTIONS(57),
    [anon_sym_GT_GT] = ACTIONS(57),
    [anon_sym_PLUS] = ACTIONS(57),
    [anon_sym_DASH] = ACTIONS(57),
    [anon_sym_SLASH] = ACTIONS(61),
    [anon_sym_PERCENT] = ACTIONS(57),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_DOT] = ACTIONS(61),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_RBRACK] = ACTIONS(57),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_EQ_GT] = ACTIONS(57),
    [anon_sym_else] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SEMI] = ACTIONS(57),
    [anon_sym_with] = ACTIONS(61),
    [anon_sym_to] = ACTIONS(61),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(3)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(894),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(273),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1145),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1150),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(3),
    [sym_block_comment] = STATE(3),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(87),
    [anon_sym_forall] = ACTIONS(87),
    [anon_sym_choose] = ACTIONS(89),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(57),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_COMMA] = ACTIONS(57),
    [anon_sym_RPAREN] = ACTIONS(57),
    [anon_sym_PIPE] = ACTIONS(61),
    [anon_sym_AMP] = ACTIONS(61),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_EQ] = ACTIONS(61),
    [anon_sym_where] = ACTIONS(61),
    [anon_sym_COLON] = ACTIONS(57),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_RBRACE] = ACTIONS(57),
    [anon_sym_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_PIPE_PIPE] = ACTIONS(57),
    [anon_sym_AMP_AMP] = ACTIONS(57),
    [anon_sym_EQ_EQ] = ACTIONS(61),
    [anon_sym_BANG_EQ] = ACTIONS(57),
    [anon_sym_LT] = ACTIONS(61),
    [anon_sym_GT] = ACTIONS(61),
    [anon_sym_LT_EQ] = ACTIONS(61),
    [anon_sym_GT_EQ] = ACTIONS(57),
    [anon_sym_DOT_DOT] = ACTIONS(57),
    [anon_sym_CARET] = ACTIONS(57),
    [anon_sym_LT_LT] = ACTIONS(57),
    [anon_sym_GT_GT] = ACTIONS(57),
    [anon_sym_PLUS] = ACTIONS(57),
    [anon_sym_DASH] = ACTIONS(57),
    [anon_sym_SLASH] = ACTIONS(61),
    [anon_sym_PERCENT] = ACTIONS(57),
    [anon_sym_BANG] = ACTIONS(95),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_DOT] = ACTIONS(61),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_RBRACK] = ACTIONS(57),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_EQ_GT] = ACTIONS(57),
    [anon_sym_else] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(61),
    [anon_sym_SEMI] = ACTIONS(57),
    [anon_sym_with] = ACTIONS(61),
    [anon_sym_to] = ACTIONS(61),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(4)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(894),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(4),
    [sym_block_comment] = STATE(4),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(57),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_COMMA] = ACTIONS(57),
    [anon_sym_RPAREN] = ACTIONS(57),
    [anon_sym_PIPE] = ACTIONS(61),
    [anon_sym_AMP] = ACTIONS(61),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_where] = ACTIONS(61),
    [anon_sym_COLON] = ACTIONS(57),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_RBRACE] = ACTIONS(57),
    [anon_sym_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_PIPE_PIPE] = ACTIONS(57),
    [anon_sym_AMP_AMP] = ACTIONS(57),
    [anon_sym_EQ_EQ] = ACTIONS(61),
    [anon_sym_BANG_EQ] = ACTIONS(57),
    [anon_sym_LT] = ACTIONS(61),
    [anon_sym_GT] = ACTIONS(61),
    [anon_sym_LT_EQ] = ACTIONS(61),
    [anon_sym_GT_EQ] = ACTIONS(57),
    [anon_sym_DOT_DOT] = ACTIONS(57),
    [anon_sym_CARET] = ACTIONS(57),
    [anon_sym_LT_LT] = ACTIONS(57),
    [anon_sym_GT_GT] = ACTIONS(57),
    [anon_sym_PLUS] = ACTIONS(57),
    [anon_sym_DASH] = ACTIONS(57),
    [anon_sym_SLASH] = ACTIONS(61),
    [anon_sym_PERCENT] = ACTIONS(57),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_DOT] = ACTIONS(61),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_RBRACK] = ACTIONS(57),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_EQ_GT] = ACTIONS(57),
    [anon_sym_else] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SEMI] = ACTIONS(57),
    [anon_sym_with] = ACTIONS(61),
    [anon_sym_to] = ACTIONS(61),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(5)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(894),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(479),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1165),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1146),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(5),
    [sym_block_comment] = STATE(5),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(109),
    [anon_sym_forall] = ACTIONS(109),
    [anon_sym_choose] = ACTIONS(111),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(57),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_COMMA] = ACTIONS(57),
    [anon_sym_RPAREN] = ACTIONS(57),
    [anon_sym_PIPE] = ACTIONS(61),
    [anon_sym_AMP] = ACTIONS(61),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_where] = ACTIONS(61),
    [anon_sym_COLON] = ACTIONS(57),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_RBRACE] = ACTIONS(57),
    [anon_sym_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_PIPE_PIPE] = ACTIONS(57),
    [anon_sym_AMP_AMP] = ACTIONS(57),
    [anon_sym_EQ_EQ] = ACTIONS(61),
    [anon_sym_BANG_EQ] = ACTIONS(57),
    [anon_sym_LT] = ACTIONS(61),
    [anon_sym_GT] = ACTIONS(61),
    [anon_sym_LT_EQ] = ACTIONS(61),
    [anon_sym_GT_EQ] = ACTIONS(57),
    [anon_sym_DOT_DOT] = ACTIONS(57),
    [anon_sym_CARET] = ACTIONS(57),
    [anon_sym_LT_LT] = ACTIONS(57),
    [anon_sym_GT_GT] = ACTIONS(57),
    [anon_sym_PLUS] = ACTIONS(57),
    [anon_sym_DASH] = ACTIONS(57),
    [anon_sym_SLASH] = ACTIONS(61),
    [anon_sym_PERCENT] = ACTIONS(57),
    [anon_sym_BANG] = ACTIONS(95),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_DOT] = ACTIONS(61),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_RBRACK] = ACTIONS(57),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_EQ_GT] = ACTIONS(57),
    [anon_sym_else] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(61),
    [anon_sym_SEMI] = ACTIONS(57),
    [anon_sym_with] = ACTIONS(61),
    [anon_sym_to] = ACTIONS(61),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(6)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(894),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(498),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1321),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1334),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(6),
    [sym_block_comment] = STATE(6),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(113),
    [anon_sym_forall] = ACTIONS(113),
    [anon_sym_choose] = ACTIONS(115),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(57),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_COMMA] = ACTIONS(57),
    [anon_sym_RPAREN] = ACTIONS(57),
    [anon_sym_PIPE] = ACTIONS(61),
    [anon_sym_AMP] = ACTIONS(61),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_EQ] = ACTIONS(61),
    [anon_sym_COLON] = ACTIONS(57),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_PIPE_PIPE] = ACTIONS(57),
    [anon_sym_AMP_AMP] = ACTIONS(57),
    [anon_sym_EQ_EQ] = ACTIONS(61),
    [anon_sym_BANG_EQ] = ACTIONS(57),
    [anon_sym_LT] = ACTIONS(61),
    [anon_sym_GT] = ACTIONS(61),
    [anon_sym_LT_EQ] = ACTIONS(61),
    [anon_sym_GT_EQ] = ACTIONS(57),
    [anon_sym_DOT_DOT] = ACTIONS(57),
    [anon_sym_CARET] = ACTIONS(57),
    [anon_sym_LT_LT] = ACTIONS(57),
    [anon_sym_GT_GT] = ACTIONS(57),
    [anon_sym_PLUS] = ACTIONS(57),
    [anon_sym_DASH] = ACTIONS(57),
    [anon_sym_SLASH] = ACTIONS(61),
    [anon_sym_PERCENT] = ACTIONS(57),
    [anon_sym_BANG] = ACTIONS(121),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_DOT] = ACTIONS(61),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_as] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_else] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(7)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(894),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(373),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1271),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1253),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(7),
    [sym_block_comment] = STATE(7),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(133),
    [anon_sym_forall] = ACTIONS(133),
    [anon_sym_choose] = ACTIONS(135),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(57),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_COMMA] = ACTIONS(57),
    [anon_sym_RPAREN] = ACTIONS(57),
    [anon_sym_PIPE] = ACTIONS(61),
    [anon_sym_AMP] = ACTIONS(61),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_EQ] = ACTIONS(61),
    [anon_sym_COLON] = ACTIONS(57),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_PIPE_PIPE] = ACTIONS(57),
    [anon_sym_AMP_AMP] = ACTIONS(57),
    [anon_sym_EQ_EQ] = ACTIONS(61),
    [anon_sym_BANG_EQ] = ACTIONS(57),
    [anon_sym_LT] = ACTIONS(61),
    [anon_sym_GT] = ACTIONS(61),
    [anon_sym_LT_EQ] = ACTIONS(61),
    [anon_sym_GT_EQ] = ACTIONS(57),
    [anon_sym_DOT_DOT] = ACTIONS(57),
    [anon_sym_CARET] = ACTIONS(57),
    [anon_sym_LT_LT] = ACTIONS(57),
    [anon_sym_GT_GT] = ACTIONS(57),
    [anon_sym_PLUS] = ACTIONS(57),
    [anon_sym_DASH] = ACTIONS(57),
    [anon_sym_SLASH] = ACTIONS(61),
    [anon_sym_PERCENT] = ACTIONS(57),
    [anon_sym_BANG] = ACTIONS(141),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_DOT] = ACTIONS(61),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_as] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_else] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(61),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(8)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(894),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(230),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1276),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1277),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(8),
    [sym_block_comment] = STATE(8),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(151),
    [anon_sym_forall] = ACTIONS(151),
    [anon_sym_choose] = ACTIONS(153),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(57),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_COMMA] = ACTIONS(57),
    [anon_sym_PIPE] = ACTIONS(61),
    [anon_sym_AMP] = ACTIONS(61),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_EQ] = ACTIONS(61),
    [anon_sym_where] = ACTIONS(61),
    [anon_sym_COLON] = ACTIONS(57),
    [anon_sym_LBRACE] = ACTIONS(57),
    [anon_sym_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_PIPE_PIPE] = ACTIONS(57),
    [anon_sym_AMP_AMP] = ACTIONS(57),
    [anon_sym_EQ_EQ] = ACTIONS(61),
    [anon_sym_BANG_EQ] = ACTIONS(57),
    [anon_sym_LT] = ACTIONS(61),
    [anon_sym_GT] = ACTIONS(61),
    [anon_sym_LT_EQ] = ACTIONS(61),
    [anon_sym_GT_EQ] = ACTIONS(57),
    [anon_sym_DOT_DOT] = ACTIONS(57),
    [anon_sym_CARET] = ACTIONS(57),
    [anon_sym_LT_LT] = ACTIONS(57),
    [anon_sym_GT_GT] = ACTIONS(57),
    [anon_sym_PLUS] = ACTIONS(57),
    [anon_sym_DASH] = ACTIONS(57),
    [anon_sym_SLASH] = ACTIONS(61),
    [anon_sym_PERCENT] = ACTIONS(57),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_DOT] = ACTIONS(61),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_else] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(61),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(9)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(894),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(393),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1296),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1319),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(9),
    [sym_block_comment] = STATE(9),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(169),
    [anon_sym_forall] = ACTIONS(169),
    [anon_sym_choose] = ACTIONS(171),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(57),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_COMMA] = ACTIONS(57),
    [anon_sym_PIPE] = ACTIONS(61),
    [anon_sym_AMP] = ACTIONS(61),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_EQ] = ACTIONS(61),
    [anon_sym_where] = ACTIONS(61),
    [anon_sym_COLON] = ACTIONS(57),
    [anon_sym_LBRACE] = ACTIONS(57),
    [anon_sym_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_PIPE_PIPE] = ACTIONS(57),
    [anon_sym_AMP_AMP] = ACTIONS(57),
    [anon_sym_EQ_EQ] = ACTIONS(61),
    [anon_sym_BANG_EQ] = ACTIONS(57),
    [anon_sym_LT] = ACTIONS(61),
    [anon_sym_GT] = ACTIONS(61),
    [anon_sym_LT_EQ] = ACTIONS(61),
    [anon_sym_GT_EQ] = ACTIONS(57),
    [anon_sym_DOT_DOT] = ACTIONS(57),
    [anon_sym_CARET] = ACTIONS(57),
    [anon_sym_LT_LT] = ACTIONS(57),
    [anon_sym_GT_GT] = ACTIONS(57),
    [anon_sym_PLUS] = ACTIONS(57),
    [anon_sym_DASH] = ACTIONS(57),
    [anon_sym_SLASH] = ACTIONS(61),
    [anon_sym_PERCENT] = ACTIONS(57),
    [anon_sym_BANG] = ACTIONS(177),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_DOT] = ACTIONS(61),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_else] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(10)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(894),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(500),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1332),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1274),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(10),
    [sym_block_comment] = STATE(10),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(187),
    [anon_sym_forall] = ACTIONS(187),
    [anon_sym_choose] = ACTIONS(189),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(57),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_COMMA] = ACTIONS(57),
    [anon_sym_RPAREN] = ACTIONS(57),
    [anon_sym_PIPE] = ACTIONS(61),
    [anon_sym_AMP] = ACTIONS(61),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_COLON] = ACTIONS(57),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_PIPE_PIPE] = ACTIONS(57),
    [anon_sym_AMP_AMP] = ACTIONS(57),
    [anon_sym_EQ_EQ] = ACTIONS(61),
    [anon_sym_BANG_EQ] = ACTIONS(57),
    [anon_sym_LT] = ACTIONS(61),
    [anon_sym_GT] = ACTIONS(61),
    [anon_sym_LT_EQ] = ACTIONS(61),
    [anon_sym_GT_EQ] = ACTIONS(57),
    [anon_sym_DOT_DOT] = ACTIONS(57),
    [anon_sym_CARET] = ACTIONS(57),
    [anon_sym_LT_LT] = ACTIONS(57),
    [anon_sym_GT_GT] = ACTIONS(57),
    [anon_sym_PLUS] = ACTIONS(57),
    [anon_sym_DASH] = ACTIONS(57),
    [anon_sym_SLASH] = ACTIONS(61),
    [anon_sym_PERCENT] = ACTIONS(57),
    [anon_sym_BANG] = ACTIONS(141),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_DOT] = ACTIONS(61),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_as] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_else] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(61),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(11)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(894),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(357),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1352),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1308),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(11),
    [sym_block_comment] = STATE(11),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(191),
    [anon_sym_forall] = ACTIONS(191),
    [anon_sym_choose] = ACTIONS(193),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(57),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_COMMA] = ACTIONS(57),
    [anon_sym_RPAREN] = ACTIONS(57),
    [anon_sym_PIPE] = ACTIONS(61),
    [anon_sym_AMP] = ACTIONS(61),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_COLON] = ACTIONS(57),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_PIPE_PIPE] = ACTIONS(57),
    [anon_sym_AMP_AMP] = ACTIONS(57),
    [anon_sym_EQ_EQ] = ACTIONS(61),
    [anon_sym_BANG_EQ] = ACTIONS(57),
    [anon_sym_LT] = ACTIONS(61),
    [anon_sym_GT] = ACTIONS(61),
    [anon_sym_LT_EQ] = ACTIONS(61),
    [anon_sym_GT_EQ] = ACTIONS(57),
    [anon_sym_DOT_DOT] = ACTIONS(57),
    [anon_sym_CARET] = ACTIONS(57),
    [anon_sym_LT_LT] = ACTIONS(57),
    [anon_sym_GT_GT] = ACTIONS(57),
    [anon_sym_PLUS] = ACTIONS(57),
    [anon_sym_DASH] = ACTIONS(57),
    [anon_sym_SLASH] = ACTIONS(61),
    [anon_sym_PERCENT] = ACTIONS(57),
    [anon_sym_BANG] = ACTIONS(121),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_DOT] = ACTIONS(61),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_as] = ACTIONS(61),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_else] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(12)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(894),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(295),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1002),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1001),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(12),
    [sym_block_comment] = STATE(12),
    [sym_identifier] = ACTIONS(61),
    [anon_sym_u8] = ACTIONS(61),
    [anon_sym_u16] = ACTIONS(61),
    [anon_sym_u32] = ACTIONS(61),
    [anon_sym_u64] = ACTIONS(61),
    [anon_sym_u128] = ACTIONS(61),
    [anon_sym_u256] = ACTIONS(61),
    [anon_sym_exists] = ACTIONS(61),
    [anon_sym_forall] = ACTIONS(61),
    [anon_sym_choose] = ACTIONS(61),
    [anon_sym_min] = ACTIONS(61),
    [anon_sym_bool] = ACTIONS(61),
    [anon_sym_address] = ACTIONS(61),
    [anon_sym_vector] = ACTIONS(61),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_while] = ACTIONS(61),
    [anon_sym_friend] = ACTIONS(61),
    [anon_sym_match] = ACTIONS(61),
    [aux_sym_number_token1] = ACTIONS(61),
    [aux_sym_number_token2] = ACTIONS(57),
    [aux_sym_number_token4] = ACTIONS(57),
    [aux_sym_number_token6] = ACTIONS(57),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(57),
    [anon_sym_LPAREN] = ACTIONS(57),
    [anon_sym_COMMA] = ACTIONS(57),
    [anon_sym_PIPE] = ACTIONS(61),
    [anon_sym_AMP] = ACTIONS(61),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_EQ] = ACTIONS(61),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_RBRACE] = ACTIONS(57),
    [anon_sym_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_PIPE_PIPE] = ACTIONS(57),
    [anon_sym_AMP_AMP] = ACTIONS(57),
    [anon_sym_EQ_EQ] = ACTIONS(61),
    [anon_sym_BANG_EQ] = ACTIONS(57),
    [anon_sym_LT] = ACTIONS(61),
    [anon_sym_GT] = ACTIONS(61),
    [anon_sym_LT_EQ] = ACTIONS(61),
    [anon_sym_GT_EQ] = ACTIONS(57),
    [anon_sym_DOT_DOT] = ACTIONS(57),
    [anon_sym_CARET] = ACTIONS(57),
    [anon_sym_LT_LT] = ACTIONS(57),
    [anon_sym_GT_GT] = ACTIONS(57),
    [anon_sym_PLUS] = ACTIONS(57),
    [anon_sym_DASH] = ACTIONS(57),
    [anon_sym_SLASH] = ACTIONS(61),
    [anon_sym_PERCENT] = ACTIONS(57),
    [anon_sym_BANG] = ACTIONS(197),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_DOT] = ACTIONS(61),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_else] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(13)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(894),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(175),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1356),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1335),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(13),
    [sym_block_comment] = STATE(13),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(207),
    [anon_sym_forall] = ACTIONS(207),
    [anon_sym_choose] = ACTIONS(209),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(57),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_COMMA] = ACTIONS(57),
    [anon_sym_PIPE] = ACTIONS(61),
    [anon_sym_AMP] = ACTIONS(61),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_where] = ACTIONS(61),
    [anon_sym_COLON] = ACTIONS(57),
    [anon_sym_LBRACE] = ACTIONS(57),
    [anon_sym_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_PIPE_PIPE] = ACTIONS(57),
    [anon_sym_AMP_AMP] = ACTIONS(57),
    [anon_sym_EQ_EQ] = ACTIONS(61),
    [anon_sym_BANG_EQ] = ACTIONS(57),
    [anon_sym_LT] = ACTIONS(61),
    [anon_sym_GT] = ACTIONS(61),
    [anon_sym_LT_EQ] = ACTIONS(61),
    [anon_sym_GT_EQ] = ACTIONS(57),
    [anon_sym_DOT_DOT] = ACTIONS(57),
    [anon_sym_CARET] = ACTIONS(57),
    [anon_sym_LT_LT] = ACTIONS(57),
    [anon_sym_GT_GT] = ACTIONS(57),
    [anon_sym_PLUS] = ACTIONS(57),
    [anon_sym_DASH] = ACTIONS(57),
    [anon_sym_SLASH] = ACTIONS(61),
    [anon_sym_PERCENT] = ACTIONS(57),
    [anon_sym_BANG] = ACTIONS(177),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_DOT] = ACTIONS(61),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_else] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(14)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(894),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(191),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(997),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(987),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(14),
    [sym_block_comment] = STATE(14),
    [sym_identifier] = ACTIONS(61),
    [anon_sym_u8] = ACTIONS(61),
    [anon_sym_u16] = ACTIONS(61),
    [anon_sym_u32] = ACTIONS(61),
    [anon_sym_u64] = ACTIONS(61),
    [anon_sym_u128] = ACTIONS(61),
    [anon_sym_u256] = ACTIONS(61),
    [anon_sym_exists] = ACTIONS(61),
    [anon_sym_forall] = ACTIONS(61),
    [anon_sym_choose] = ACTIONS(61),
    [anon_sym_min] = ACTIONS(61),
    [anon_sym_bool] = ACTIONS(61),
    [anon_sym_address] = ACTIONS(61),
    [anon_sym_vector] = ACTIONS(61),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_while] = ACTIONS(61),
    [anon_sym_friend] = ACTIONS(61),
    [anon_sym_match] = ACTIONS(61),
    [aux_sym_number_token1] = ACTIONS(61),
    [aux_sym_number_token2] = ACTIONS(57),
    [aux_sym_number_token4] = ACTIONS(57),
    [aux_sym_number_token6] = ACTIONS(57),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(57),
    [anon_sym_LPAREN] = ACTIONS(57),
    [anon_sym_COMMA] = ACTIONS(57),
    [anon_sym_PIPE] = ACTIONS(61),
    [anon_sym_AMP] = ACTIONS(61),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_EQ] = ACTIONS(61),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_RBRACE] = ACTIONS(57),
    [anon_sym_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_PIPE_PIPE] = ACTIONS(57),
    [anon_sym_AMP_AMP] = ACTIONS(57),
    [anon_sym_EQ_EQ] = ACTIONS(61),
    [anon_sym_BANG_EQ] = ACTIONS(57),
    [anon_sym_LT] = ACTIONS(61),
    [anon_sym_GT] = ACTIONS(61),
    [anon_sym_LT_EQ] = ACTIONS(61),
    [anon_sym_GT_EQ] = ACTIONS(57),
    [anon_sym_DOT_DOT] = ACTIONS(57),
    [anon_sym_CARET] = ACTIONS(57),
    [anon_sym_LT_LT] = ACTIONS(57),
    [anon_sym_GT_GT] = ACTIONS(57),
    [anon_sym_PLUS] = ACTIONS(57),
    [anon_sym_DASH] = ACTIONS(57),
    [anon_sym_SLASH] = ACTIONS(61),
    [anon_sym_PERCENT] = ACTIONS(57),
    [anon_sym_BANG] = ACTIONS(213),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_DOT] = ACTIONS(61),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_else] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(61),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(15)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(894),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(419),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1307),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1275),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(15),
    [sym_block_comment] = STATE(15),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(223),
    [anon_sym_forall] = ACTIONS(223),
    [anon_sym_choose] = ACTIONS(225),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(57),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_COMMA] = ACTIONS(57),
    [anon_sym_PIPE] = ACTIONS(61),
    [anon_sym_AMP] = ACTIONS(61),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_where] = ACTIONS(61),
    [anon_sym_COLON] = ACTIONS(57),
    [anon_sym_LBRACE] = ACTIONS(57),
    [anon_sym_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_PIPE_PIPE] = ACTIONS(57),
    [anon_sym_AMP_AMP] = ACTIONS(57),
    [anon_sym_EQ_EQ] = ACTIONS(61),
    [anon_sym_BANG_EQ] = ACTIONS(57),
    [anon_sym_LT] = ACTIONS(61),
    [anon_sym_GT] = ACTIONS(61),
    [anon_sym_LT_EQ] = ACTIONS(61),
    [anon_sym_GT_EQ] = ACTIONS(57),
    [anon_sym_DOT_DOT] = ACTIONS(57),
    [anon_sym_CARET] = ACTIONS(57),
    [anon_sym_LT_LT] = ACTIONS(57),
    [anon_sym_GT_GT] = ACTIONS(57),
    [anon_sym_PLUS] = ACTIONS(57),
    [anon_sym_DASH] = ACTIONS(57),
    [anon_sym_SLASH] = ACTIONS(61),
    [anon_sym_PERCENT] = ACTIONS(57),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_DOT] = ACTIONS(61),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_else] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(61),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(16)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(894),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(406),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1397),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1392),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(16),
    [sym_block_comment] = STATE(16),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(227),
    [anon_sym_forall] = ACTIONS(227),
    [anon_sym_choose] = ACTIONS(229),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(57),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(61),
    [anon_sym_AMP] = ACTIONS(61),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_EQ] = ACTIONS(61),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_PIPE_PIPE] = ACTIONS(57),
    [anon_sym_AMP_AMP] = ACTIONS(57),
    [anon_sym_EQ_EQ] = ACTIONS(61),
    [anon_sym_BANG_EQ] = ACTIONS(57),
    [anon_sym_LT] = ACTIONS(61),
    [anon_sym_GT] = ACTIONS(61),
    [anon_sym_LT_EQ] = ACTIONS(61),
    [anon_sym_GT_EQ] = ACTIONS(57),
    [anon_sym_DOT_DOT] = ACTIONS(57),
    [anon_sym_CARET] = ACTIONS(57),
    [anon_sym_LT_LT] = ACTIONS(57),
    [anon_sym_GT_GT] = ACTIONS(57),
    [anon_sym_PLUS] = ACTIONS(57),
    [anon_sym_DASH] = ACTIONS(57),
    [anon_sym_SLASH] = ACTIONS(61),
    [anon_sym_PERCENT] = ACTIONS(57),
    [anon_sym_BANG] = ACTIONS(235),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_DOT] = ACTIONS(61),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(61),
    [anon_sym_else] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SEMI] = ACTIONS(57),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(17)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(894),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(248),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1369),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1370),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(17),
    [sym_block_comment] = STATE(17),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(243),
    [anon_sym_forall] = ACTIONS(243),
    [anon_sym_choose] = ACTIONS(245),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(57),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(61),
    [anon_sym_AMP] = ACTIONS(61),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_EQ] = ACTIONS(61),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_PIPE_PIPE] = ACTIONS(57),
    [anon_sym_AMP_AMP] = ACTIONS(57),
    [anon_sym_EQ_EQ] = ACTIONS(61),
    [anon_sym_BANG_EQ] = ACTIONS(57),
    [anon_sym_LT] = ACTIONS(61),
    [anon_sym_GT] = ACTIONS(61),
    [anon_sym_LT_EQ] = ACTIONS(61),
    [anon_sym_GT_EQ] = ACTIONS(57),
    [anon_sym_DOT_DOT] = ACTIONS(57),
    [anon_sym_CARET] = ACTIONS(57),
    [anon_sym_LT_LT] = ACTIONS(57),
    [anon_sym_GT_GT] = ACTIONS(57),
    [anon_sym_PLUS] = ACTIONS(57),
    [anon_sym_DASH] = ACTIONS(57),
    [anon_sym_SLASH] = ACTIONS(61),
    [anon_sym_PERCENT] = ACTIONS(57),
    [anon_sym_BANG] = ACTIONS(251),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_DOT] = ACTIONS(61),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(61),
    [anon_sym_else] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(61),
    [anon_sym_SEMI] = ACTIONS(57),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(18)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(894),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(390),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1033),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1023),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(18),
    [sym_block_comment] = STATE(18),
    [sym_identifier] = ACTIONS(61),
    [anon_sym_u8] = ACTIONS(61),
    [anon_sym_u16] = ACTIONS(61),
    [anon_sym_u32] = ACTIONS(61),
    [anon_sym_u64] = ACTIONS(61),
    [anon_sym_u128] = ACTIONS(61),
    [anon_sym_u256] = ACTIONS(61),
    [anon_sym_exists] = ACTIONS(61),
    [anon_sym_forall] = ACTIONS(61),
    [anon_sym_choose] = ACTIONS(61),
    [anon_sym_min] = ACTIONS(61),
    [anon_sym_bool] = ACTIONS(61),
    [anon_sym_address] = ACTIONS(61),
    [anon_sym_vector] = ACTIONS(61),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_while] = ACTIONS(61),
    [anon_sym_friend] = ACTIONS(61),
    [anon_sym_match] = ACTIONS(61),
    [aux_sym_number_token1] = ACTIONS(61),
    [aux_sym_number_token2] = ACTIONS(57),
    [aux_sym_number_token4] = ACTIONS(57),
    [aux_sym_number_token6] = ACTIONS(57),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(57),
    [anon_sym_LPAREN] = ACTIONS(57),
    [anon_sym_COMMA] = ACTIONS(57),
    [anon_sym_PIPE] = ACTIONS(61),
    [anon_sym_AMP] = ACTIONS(61),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_RBRACE] = ACTIONS(57),
    [anon_sym_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_PIPE_PIPE] = ACTIONS(57),
    [anon_sym_AMP_AMP] = ACTIONS(57),
    [anon_sym_EQ_EQ] = ACTIONS(61),
    [anon_sym_BANG_EQ] = ACTIONS(57),
    [anon_sym_LT] = ACTIONS(61),
    [anon_sym_GT] = ACTIONS(61),
    [anon_sym_LT_EQ] = ACTIONS(61),
    [anon_sym_GT_EQ] = ACTIONS(57),
    [anon_sym_DOT_DOT] = ACTIONS(57),
    [anon_sym_CARET] = ACTIONS(57),
    [anon_sym_LT_LT] = ACTIONS(57),
    [anon_sym_GT_GT] = ACTIONS(57),
    [anon_sym_PLUS] = ACTIONS(57),
    [anon_sym_DASH] = ACTIONS(57),
    [anon_sym_SLASH] = ACTIONS(61),
    [anon_sym_PERCENT] = ACTIONS(57),
    [anon_sym_BANG] = ACTIONS(197),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_DOT] = ACTIONS(61),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_else] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(19)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(894),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(310),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1014),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(988),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(19),
    [sym_block_comment] = STATE(19),
    [sym_identifier] = ACTIONS(61),
    [anon_sym_u8] = ACTIONS(61),
    [anon_sym_u16] = ACTIONS(61),
    [anon_sym_u32] = ACTIONS(61),
    [anon_sym_u64] = ACTIONS(61),
    [anon_sym_u128] = ACTIONS(61),
    [anon_sym_u256] = ACTIONS(61),
    [anon_sym_exists] = ACTIONS(61),
    [anon_sym_forall] = ACTIONS(61),
    [anon_sym_choose] = ACTIONS(61),
    [anon_sym_min] = ACTIONS(61),
    [anon_sym_bool] = ACTIONS(61),
    [anon_sym_address] = ACTIONS(61),
    [anon_sym_vector] = ACTIONS(61),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_while] = ACTIONS(61),
    [anon_sym_friend] = ACTIONS(61),
    [anon_sym_match] = ACTIONS(61),
    [aux_sym_number_token1] = ACTIONS(61),
    [aux_sym_number_token2] = ACTIONS(57),
    [aux_sym_number_token4] = ACTIONS(57),
    [aux_sym_number_token6] = ACTIONS(57),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(57),
    [anon_sym_LPAREN] = ACTIONS(57),
    [anon_sym_COMMA] = ACTIONS(57),
    [anon_sym_PIPE] = ACTIONS(61),
    [anon_sym_AMP] = ACTIONS(61),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_RBRACE] = ACTIONS(57),
    [anon_sym_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_PIPE_PIPE] = ACTIONS(57),
    [anon_sym_AMP_AMP] = ACTIONS(57),
    [anon_sym_EQ_EQ] = ACTIONS(61),
    [anon_sym_BANG_EQ] = ACTIONS(57),
    [anon_sym_LT] = ACTIONS(61),
    [anon_sym_GT] = ACTIONS(61),
    [anon_sym_LT_EQ] = ACTIONS(61),
    [anon_sym_GT_EQ] = ACTIONS(57),
    [anon_sym_DOT_DOT] = ACTIONS(57),
    [anon_sym_CARET] = ACTIONS(57),
    [anon_sym_LT_LT] = ACTIONS(57),
    [anon_sym_GT_GT] = ACTIONS(57),
    [anon_sym_PLUS] = ACTIONS(57),
    [anon_sym_DASH] = ACTIONS(57),
    [anon_sym_SLASH] = ACTIONS(61),
    [anon_sym_PERCENT] = ACTIONS(57),
    [anon_sym_BANG] = ACTIONS(213),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_DOT] = ACTIONS(61),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_else] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(61),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(20)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(894),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(432),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1393),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1365),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(20),
    [sym_block_comment] = STATE(20),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(259),
    [anon_sym_forall] = ACTIONS(259),
    [anon_sym_choose] = ACTIONS(261),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(57),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(61),
    [anon_sym_AMP] = ACTIONS(61),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_PIPE_PIPE] = ACTIONS(57),
    [anon_sym_AMP_AMP] = ACTIONS(57),
    [anon_sym_EQ_EQ] = ACTIONS(61),
    [anon_sym_BANG_EQ] = ACTIONS(57),
    [anon_sym_LT] = ACTIONS(61),
    [anon_sym_GT] = ACTIONS(61),
    [anon_sym_LT_EQ] = ACTIONS(61),
    [anon_sym_GT_EQ] = ACTIONS(57),
    [anon_sym_DOT_DOT] = ACTIONS(57),
    [anon_sym_CARET] = ACTIONS(57),
    [anon_sym_LT_LT] = ACTIONS(57),
    [anon_sym_GT_GT] = ACTIONS(57),
    [anon_sym_PLUS] = ACTIONS(57),
    [anon_sym_DASH] = ACTIONS(57),
    [anon_sym_SLASH] = ACTIONS(61),
    [anon_sym_PERCENT] = ACTIONS(57),
    [anon_sym_BANG] = ACTIONS(251),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_DOT] = ACTIONS(61),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(61),
    [anon_sym_else] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(61),
    [anon_sym_SEMI] = ACTIONS(57),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(21)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(894),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(210),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1405),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1396),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(21),
    [sym_block_comment] = STATE(21),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(263),
    [anon_sym_forall] = ACTIONS(263),
    [anon_sym_choose] = ACTIONS(265),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(57),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(61),
    [anon_sym_AMP] = ACTIONS(61),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(57),
    [anon_sym_PIPE_PIPE] = ACTIONS(57),
    [anon_sym_AMP_AMP] = ACTIONS(57),
    [anon_sym_EQ_EQ] = ACTIONS(61),
    [anon_sym_BANG_EQ] = ACTIONS(57),
    [anon_sym_LT] = ACTIONS(61),
    [anon_sym_GT] = ACTIONS(61),
    [anon_sym_LT_EQ] = ACTIONS(61),
    [anon_sym_GT_EQ] = ACTIONS(57),
    [anon_sym_DOT_DOT] = ACTIONS(57),
    [anon_sym_CARET] = ACTIONS(57),
    [anon_sym_LT_LT] = ACTIONS(57),
    [anon_sym_GT_GT] = ACTIONS(57),
    [anon_sym_PLUS] = ACTIONS(57),
    [anon_sym_DASH] = ACTIONS(57),
    [anon_sym_SLASH] = ACTIONS(61),
    [anon_sym_PERCENT] = ACTIONS(57),
    [anon_sym_BANG] = ACTIONS(235),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_DOT] = ACTIONS(61),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(61),
    [anon_sym_else] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SEMI] = ACTIONS(57),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(22)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2078),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_use_decl] = STATE(1045),
    [sym_block] = STATE(908),
    [sym__sequence_item] = STATE(1047),
    [sym_let_expr] = STATE(2401),
    [sym_line_comment] = STATE(22),
    [sym_block_comment] = STATE(22),
    [aux_sym_spec_block_repeat1] = STATE(25),
    [aux_sym_block_repeat1] = STATE(28),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_RBRACE] = ACTIONS(273),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_let] = ACTIONS(277),
    [anon_sym_use] = ACTIONS(279),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(23)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2117),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_use_decl] = STATE(1045),
    [sym_block] = STATE(908),
    [sym__sequence_item] = STATE(1047),
    [sym_let_expr] = STATE(2401),
    [sym_line_comment] = STATE(23),
    [sym_block_comment] = STATE(23),
    [aux_sym_spec_block_repeat1] = STATE(24),
    [aux_sym_block_repeat1] = STATE(26),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_RBRACE] = ACTIONS(281),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_let] = ACTIONS(277),
    [anon_sym_use] = ACTIONS(279),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(24)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2094),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_use_decl] = STATE(1045),
    [sym_block] = STATE(908),
    [sym__sequence_item] = STATE(1047),
    [sym_let_expr] = STATE(2401),
    [sym_line_comment] = STATE(24),
    [sym_block_comment] = STATE(24),
    [aux_sym_spec_block_repeat1] = STATE(1043),
    [aux_sym_block_repeat1] = STATE(31),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_RBRACE] = ACTIONS(283),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_let] = ACTIONS(277),
    [anon_sym_use] = ACTIONS(279),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(25)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2093),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_use_decl] = STATE(1045),
    [sym_block] = STATE(908),
    [sym__sequence_item] = STATE(1047),
    [sym_let_expr] = STATE(2401),
    [sym_line_comment] = STATE(25),
    [sym_block_comment] = STATE(25),
    [aux_sym_spec_block_repeat1] = STATE(1043),
    [aux_sym_block_repeat1] = STATE(29),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_RBRACE] = ACTIONS(285),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_let] = ACTIONS(277),
    [anon_sym_use] = ACTIONS(279),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(26)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2094),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym__sequence_item] = STATE(1047),
    [sym_let_expr] = STATE(2401),
    [sym_line_comment] = STATE(26),
    [sym_block_comment] = STATE(26),
    [aux_sym_block_repeat1] = STATE(27),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_RBRACE] = ACTIONS(283),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_let] = ACTIONS(277),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(27)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2401),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym__sequence_item] = STATE(1047),
    [sym_let_expr] = STATE(2401),
    [sym_line_comment] = STATE(27),
    [sym_block_comment] = STATE(27),
    [aux_sym_block_repeat1] = STATE(27),
    [sym_identifier] = ACTIONS(287),
    [anon_sym_u8] = ACTIONS(290),
    [anon_sym_u16] = ACTIONS(290),
    [anon_sym_u32] = ACTIONS(290),
    [anon_sym_u64] = ACTIONS(290),
    [anon_sym_u128] = ACTIONS(290),
    [anon_sym_u256] = ACTIONS(290),
    [anon_sym_exists] = ACTIONS(293),
    [anon_sym_forall] = ACTIONS(293),
    [anon_sym_choose] = ACTIONS(296),
    [anon_sym_min] = ACTIONS(299),
    [anon_sym_bool] = ACTIONS(302),
    [anon_sym_address] = ACTIONS(302),
    [anon_sym_vector] = ACTIONS(305),
    [anon_sym_for] = ACTIONS(308),
    [anon_sym_while] = ACTIONS(311),
    [anon_sym_friend] = ACTIONS(314),
    [anon_sym_match] = ACTIONS(317),
    [aux_sym_number_token1] = ACTIONS(320),
    [aux_sym_number_token2] = ACTIONS(323),
    [aux_sym_number_token4] = ACTIONS(326),
    [aux_sym_number_token6] = ACTIONS(329),
    [anon_sym_true] = ACTIONS(332),
    [anon_sym_false] = ACTIONS(332),
    [aux_sym_byte_string_token1] = ACTIONS(335),
    [aux_sym_byte_string_token2] = ACTIONS(335),
    [anon_sym_STAR] = ACTIONS(338),
    [anon_sym_LPAREN] = ACTIONS(341),
    [anon_sym_PIPE] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(347),
    [anon_sym_AMPmut] = ACTIONS(350),
    [anon_sym_LBRACE] = ACTIONS(353),
    [anon_sym_RBRACE] = ACTIONS(356),
    [anon_sym_BANG] = ACTIONS(358),
    [anon_sym_move] = ACTIONS(361),
    [anon_sym_copy] = ACTIONS(364),
    [anon_sym_break] = ACTIONS(367),
    [anon_sym_continue] = ACTIONS(367),
    [anon_sym_if] = ACTIONS(370),
    [anon_sym_loop] = ACTIONS(373),
    [anon_sym_return] = ACTIONS(376),
    [anon_sym_abort] = ACTIONS(379),
    [anon_sym_AT] = ACTIONS(382),
    [anon_sym_spec] = ACTIONS(385),
    [anon_sym_let] = ACTIONS(388),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(28)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2093),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym__sequence_item] = STATE(1047),
    [sym_let_expr] = STATE(2401),
    [sym_line_comment] = STATE(28),
    [sym_block_comment] = STATE(28),
    [aux_sym_block_repeat1] = STATE(27),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_RBRACE] = ACTIONS(285),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_let] = ACTIONS(277),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(29)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2045),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym__sequence_item] = STATE(1047),
    [sym_let_expr] = STATE(2401),
    [sym_line_comment] = STATE(29),
    [sym_block_comment] = STATE(29),
    [aux_sym_block_repeat1] = STATE(27),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_RBRACE] = ACTIONS(391),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_let] = ACTIONS(277),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(30)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2241),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_condition_props] = STATE(388),
    [sym_block] = STATE(908),
    [sym_type_params] = STATE(34),
    [sym_line_comment] = STATE(30),
    [sym_block_comment] = STATE(30),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_LT] = ACTIONS(393),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_LBRACK] = ACTIONS(395),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_update] = ACTIONS(397),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(31)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2034),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym__sequence_item] = STATE(1047),
    [sym_let_expr] = STATE(2401),
    [sym_line_comment] = STATE(31),
    [sym_block_comment] = STATE(31),
    [aux_sym_block_repeat1] = STATE(27),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_RBRACE] = ACTIONS(399),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_let] = ACTIONS(277),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(32)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2394),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_condition_props] = STATE(165),
    [sym_block] = STATE(908),
    [sym_type_params] = STATE(38),
    [sym_line_comment] = STATE(32),
    [sym_block_comment] = STATE(32),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_LT] = ACTIONS(393),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_LBRACK] = ACTIONS(395),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(33)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(894),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(479),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1165),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1146),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(33),
    [sym_block_comment] = STATE(33),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(109),
    [anon_sym_forall] = ACTIONS(109),
    [anon_sym_choose] = ACTIONS(111),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_RPAREN] = ACTIONS(57),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_DOT] = ACTIONS(57),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(61),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(34)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2327),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_condition_props] = STATE(358),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(34),
    [sym_block_comment] = STATE(34),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_LBRACK] = ACTIONS(395),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_update] = ACTIONS(407),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(35)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(894),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(477),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1188),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1194),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(35),
    [sym_block_comment] = STATE(35),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(27),
    [anon_sym_forall] = ACTIONS(27),
    [anon_sym_choose] = ACTIONS(29),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_EQ] = ACTIONS(57),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_DOT] = ACTIONS(57),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(36)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(894),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(36),
    [sym_block_comment] = STATE(36),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_DOT_DOT] = ACTIONS(57),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_DOT] = ACTIONS(61),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(37)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(1877),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(37),
    [sym_block_comment] = STATE(37),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_COMMA] = ACTIONS(409),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_RBRACK] = ACTIONS(411),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(38)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2386),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_condition_props] = STATE(206),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(38),
    [sym_block_comment] = STATE(38),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_LBRACK] = ACTIONS(395),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(39)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2317),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_condition_props] = STATE(163),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(39),
    [sym_block_comment] = STATE(39),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_LBRACK] = ACTIONS(395),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(40)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2336),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_condition_props] = STATE(164),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(40),
    [sym_block_comment] = STATE(40),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_LBRACK] = ACTIONS(395),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(41)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2302),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_condition_props] = STATE(155),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(41),
    [sym_block_comment] = STATE(41),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_LBRACK] = ACTIONS(395),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(42)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2110),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_condition_props] = STATE(161),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(42),
    [sym_block_comment] = STATE(42),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_LBRACK] = ACTIONS(395),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(43)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(1922),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(43),
    [sym_block_comment] = STATE(43),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_COMMA] = ACTIONS(413),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_RBRACK] = ACTIONS(415),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(44)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(1891),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(44),
    [sym_block_comment] = STATE(44),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_COMMA] = ACTIONS(417),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_RBRACE] = ACTIONS(419),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(45)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(1771),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(357),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1352),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1308),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(45),
    [sym_block_comment] = STATE(45),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(191),
    [anon_sym_forall] = ACTIONS(191),
    [anon_sym_choose] = ACTIONS(193),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_COMMA] = ACTIONS(423),
    [anon_sym_RPAREN] = ACTIONS(425),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(46)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(1980),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(46),
    [sym_block_comment] = STATE(46),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_COMMA] = ACTIONS(431),
    [anon_sym_RPAREN] = ACTIONS(433),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(47)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2206),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_condition_props] = STATE(173),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(47),
    [sym_block_comment] = STATE(47),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_LBRACK] = ACTIONS(395),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(48)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2351),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_condition_props] = STATE(172),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(48),
    [sym_block_comment] = STATE(48),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_LBRACK] = ACTIONS(395),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(49)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(1941),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_condition_props] = STATE(162),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(49),
    [sym_block_comment] = STATE(49),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_LBRACK] = ACTIONS(395),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(50)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(500),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1332),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1274),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(876),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(50),
    [sym_block_comment] = STATE(50),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(187),
    [anon_sym_forall] = ACTIONS(187),
    [anon_sym_choose] = ACTIONS(189),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(51)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(1831),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(51),
    [sym_block_comment] = STATE(51),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_RBRACK] = ACTIONS(441),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(52)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(477),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1188),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1194),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(890),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(1410),
    [sym_line_comment] = STATE(52),
    [sym_block_comment] = STATE(52),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(27),
    [anon_sym_forall] = ACTIONS(27),
    [anon_sym_choose] = ACTIONS(29),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(53)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(1831),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(53),
    [sym_block_comment] = STATE(53),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_RPAREN] = ACTIONS(443),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(54)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2156),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(54),
    [sym_block_comment] = STATE(54),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_RBRACE] = ACTIONS(445),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(55)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(273),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1145),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1150),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(879),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(1410),
    [sym_line_comment] = STATE(55),
    [sym_block_comment] = STATE(55),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(87),
    [anon_sym_forall] = ACTIONS(87),
    [anon_sym_choose] = ACTIONS(89),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(56)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(477),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1188),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1194),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1799),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(1410),
    [sym_line_comment] = STATE(56),
    [sym_block_comment] = STATE(56),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(27),
    [anon_sym_forall] = ACTIONS(27),
    [anon_sym_choose] = ACTIONS(29),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(57)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(1831),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(57),
    [sym_block_comment] = STATE(57),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_RBRACK] = ACTIONS(447),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(58)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(477),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1188),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1194),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(901),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(1410),
    [sym_line_comment] = STATE(58),
    [sym_block_comment] = STATE(58),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(27),
    [anon_sym_forall] = ACTIONS(27),
    [anon_sym_choose] = ACTIONS(29),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(59)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(191),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(997),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(987),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(890),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(59),
    [sym_block_comment] = STATE(59),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(449),
    [anon_sym_forall] = ACTIONS(449),
    [anon_sym_choose] = ACTIONS(451),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(60)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(191),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(997),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(987),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(876),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(60),
    [sym_block_comment] = STATE(60),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(449),
    [anon_sym_forall] = ACTIONS(449),
    [anon_sym_choose] = ACTIONS(451),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(61)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(393),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1296),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1319),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1333),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(61),
    [sym_block_comment] = STATE(61),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(169),
    [anon_sym_forall] = ACTIONS(169),
    [anon_sym_choose] = ACTIONS(171),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(62)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(393),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1296),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1319),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(901),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(62),
    [sym_block_comment] = STATE(62),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(169),
    [anon_sym_forall] = ACTIONS(169),
    [anon_sym_choose] = ACTIONS(171),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(63)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(191),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(997),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(987),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(993),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(63),
    [sym_block_comment] = STATE(63),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(449),
    [anon_sym_forall] = ACTIONS(449),
    [anon_sym_choose] = ACTIONS(451),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(64)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(191),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(997),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(987),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(901),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(64),
    [sym_block_comment] = STATE(64),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(449),
    [anon_sym_forall] = ACTIONS(449),
    [anon_sym_choose] = ACTIONS(451),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(65)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(273),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1145),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1150),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(890),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(65),
    [sym_block_comment] = STATE(65),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(87),
    [anon_sym_forall] = ACTIONS(87),
    [anon_sym_choose] = ACTIONS(89),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(66)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(273),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1145),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1150),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(876),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(66),
    [sym_block_comment] = STATE(66),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(87),
    [anon_sym_forall] = ACTIONS(87),
    [anon_sym_choose] = ACTIONS(89),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(67)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1221),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(67),
    [sym_block_comment] = STATE(67),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(68)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(901),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(68),
    [sym_block_comment] = STATE(68),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(69)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(390),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1033),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1023),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(890),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(69),
    [sym_block_comment] = STATE(69),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(467),
    [anon_sym_forall] = ACTIONS(467),
    [anon_sym_choose] = ACTIONS(469),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(70)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(310),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1014),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(988),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(879),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(70),
    [sym_block_comment] = STATE(70),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(479),
    [anon_sym_forall] = ACTIONS(479),
    [anon_sym_choose] = ACTIONS(481),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(71)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(406),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1397),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1392),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1400),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(71),
    [sym_block_comment] = STATE(71),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(227),
    [anon_sym_forall] = ACTIONS(227),
    [anon_sym_choose] = ACTIONS(229),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(72)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(406),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1397),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1392),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(901),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(72),
    [sym_block_comment] = STATE(72),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(227),
    [anon_sym_forall] = ACTIONS(227),
    [anon_sym_choose] = ACTIONS(229),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(73)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(390),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1033),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1023),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1038),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(73),
    [sym_block_comment] = STATE(73),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(467),
    [anon_sym_forall] = ACTIONS(467),
    [anon_sym_choose] = ACTIONS(469),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(74)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(390),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1033),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1023),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(901),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(74),
    [sym_block_comment] = STATE(74),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(467),
    [anon_sym_forall] = ACTIONS(467),
    [anon_sym_choose] = ACTIONS(469),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(75)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(890),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(75),
    [sym_block_comment] = STATE(75),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(76)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(479),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1165),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1146),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(879),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(76),
    [sym_block_comment] = STATE(76),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(109),
    [anon_sym_forall] = ACTIONS(109),
    [anon_sym_choose] = ACTIONS(111),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(77)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1796),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(1418),
    [sym_line_comment] = STATE(77),
    [sym_block_comment] = STATE(77),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(78)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(901),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(1418),
    [sym_line_comment] = STATE(78),
    [sym_block_comment] = STATE(78),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(79)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(310),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1014),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(988),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(890),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(79),
    [sym_block_comment] = STATE(79),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(479),
    [anon_sym_forall] = ACTIONS(479),
    [anon_sym_choose] = ACTIONS(481),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(80)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(310),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1014),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(988),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(876),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(80),
    [sym_block_comment] = STATE(80),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(479),
    [anon_sym_forall] = ACTIONS(479),
    [anon_sym_choose] = ACTIONS(481),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(81)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(230),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1276),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1277),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1254),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(81),
    [sym_block_comment] = STATE(81),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(151),
    [anon_sym_forall] = ACTIONS(151),
    [anon_sym_choose] = ACTIONS(153),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(82)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(230),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1276),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1277),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(901),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(82),
    [sym_block_comment] = STATE(82),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(151),
    [anon_sym_forall] = ACTIONS(151),
    [anon_sym_choose] = ACTIONS(153),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(83)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(310),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1014),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(988),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1017),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(83),
    [sym_block_comment] = STATE(83),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(479),
    [anon_sym_forall] = ACTIONS(479),
    [anon_sym_choose] = ACTIONS(481),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(84)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(310),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1014),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(988),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(901),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(84),
    [sym_block_comment] = STATE(84),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(479),
    [anon_sym_forall] = ACTIONS(479),
    [anon_sym_choose] = ACTIONS(481),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(85)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(393),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1296),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1319),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(890),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(85),
    [sym_block_comment] = STATE(85),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(169),
    [anon_sym_forall] = ACTIONS(169),
    [anon_sym_choose] = ACTIONS(171),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(86)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(230),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1276),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1277),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(879),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(86),
    [sym_block_comment] = STATE(86),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(151),
    [anon_sym_forall] = ACTIONS(151),
    [anon_sym_choose] = ACTIONS(153),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(87)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(175),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1356),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1335),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1371),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(87),
    [sym_block_comment] = STATE(87),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(207),
    [anon_sym_forall] = ACTIONS(207),
    [anon_sym_choose] = ACTIONS(209),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(88)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(175),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1356),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1335),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(901),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(88),
    [sym_block_comment] = STATE(88),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(207),
    [anon_sym_forall] = ACTIONS(207),
    [anon_sym_choose] = ACTIONS(209),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(89)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(406),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1397),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1392),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(890),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(89),
    [sym_block_comment] = STATE(89),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(227),
    [anon_sym_forall] = ACTIONS(227),
    [anon_sym_choose] = ACTIONS(229),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(90)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(248),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1369),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1370),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(879),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(90),
    [sym_block_comment] = STATE(90),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(243),
    [anon_sym_forall] = ACTIONS(243),
    [anon_sym_choose] = ACTIONS(245),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(91)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(248),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1369),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1370),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1372),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(91),
    [sym_block_comment] = STATE(91),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(243),
    [anon_sym_forall] = ACTIONS(243),
    [anon_sym_choose] = ACTIONS(245),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(92)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(248),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1369),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1370),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(901),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(92),
    [sym_block_comment] = STATE(92),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(243),
    [anon_sym_forall] = ACTIONS(243),
    [anon_sym_choose] = ACTIONS(245),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(93)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(890),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(1418),
    [sym_line_comment] = STATE(93),
    [sym_block_comment] = STATE(93),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(94)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(479),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1165),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1146),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(879),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(1418),
    [sym_line_comment] = STATE(94),
    [sym_block_comment] = STATE(94),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(109),
    [anon_sym_forall] = ACTIONS(109),
    [anon_sym_choose] = ACTIONS(111),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(95)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(210),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1405),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1396),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1409),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(95),
    [sym_block_comment] = STATE(95),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(263),
    [anon_sym_forall] = ACTIONS(263),
    [anon_sym_choose] = ACTIONS(265),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(96)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(210),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1405),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1396),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(901),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(96),
    [sym_block_comment] = STATE(96),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(263),
    [anon_sym_forall] = ACTIONS(263),
    [anon_sym_choose] = ACTIONS(265),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(97)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(230),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1276),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1277),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(890),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(97),
    [sym_block_comment] = STATE(97),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(151),
    [anon_sym_forall] = ACTIONS(151),
    [anon_sym_choose] = ACTIONS(153),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(98)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(230),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1276),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1277),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(876),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(98),
    [sym_block_comment] = STATE(98),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(151),
    [anon_sym_forall] = ACTIONS(151),
    [anon_sym_choose] = ACTIONS(153),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(99)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(479),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1165),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1146),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1787),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(1376),
    [sym_line_comment] = STATE(99),
    [sym_block_comment] = STATE(99),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(109),
    [anon_sym_forall] = ACTIONS(109),
    [anon_sym_choose] = ACTIONS(111),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(100)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(479),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1165),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1146),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(901),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(1376),
    [sym_line_comment] = STATE(100),
    [sym_block_comment] = STATE(100),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(109),
    [anon_sym_forall] = ACTIONS(109),
    [anon_sym_choose] = ACTIONS(111),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(101)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(175),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1356),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1335),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(890),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(101),
    [sym_block_comment] = STATE(101),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(207),
    [anon_sym_forall] = ACTIONS(207),
    [anon_sym_choose] = ACTIONS(209),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(102)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2156),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(102),
    [sym_block_comment] = STATE(102),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_RBRACE] = ACTIONS(505),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(103)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(479),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1165),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1146),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1181),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(103),
    [sym_block_comment] = STATE(103),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(109),
    [anon_sym_forall] = ACTIONS(109),
    [anon_sym_choose] = ACTIONS(111),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(104)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(479),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1165),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1146),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(901),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(104),
    [sym_block_comment] = STATE(104),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(109),
    [anon_sym_forall] = ACTIONS(109),
    [anon_sym_choose] = ACTIONS(111),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(105)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(248),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1369),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1370),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(890),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(105),
    [sym_block_comment] = STATE(105),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(243),
    [anon_sym_forall] = ACTIONS(243),
    [anon_sym_choose] = ACTIONS(245),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(106)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(248),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1369),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1370),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(876),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(106),
    [sym_block_comment] = STATE(106),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(243),
    [anon_sym_forall] = ACTIONS(243),
    [anon_sym_choose] = ACTIONS(245),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(107)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(419),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1307),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1275),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1285),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(107),
    [sym_block_comment] = STATE(107),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(223),
    [anon_sym_forall] = ACTIONS(223),
    [anon_sym_choose] = ACTIONS(225),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(108)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(419),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1307),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1275),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(901),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(108),
    [sym_block_comment] = STATE(108),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(223),
    [anon_sym_forall] = ACTIONS(223),
    [anon_sym_choose] = ACTIONS(225),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(109)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(210),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1405),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1396),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(890),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(109),
    [sym_block_comment] = STATE(109),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(263),
    [anon_sym_forall] = ACTIONS(263),
    [anon_sym_choose] = ACTIONS(265),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(110)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(432),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1393),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1365),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(879),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(110),
    [sym_block_comment] = STATE(110),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(259),
    [anon_sym_forall] = ACTIONS(259),
    [anon_sym_choose] = ACTIONS(261),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(111)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(432),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1393),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1365),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1398),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(111),
    [sym_block_comment] = STATE(111),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(259),
    [anon_sym_forall] = ACTIONS(259),
    [anon_sym_choose] = ACTIONS(261),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(112)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(432),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1393),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1365),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(901),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(112),
    [sym_block_comment] = STATE(112),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(259),
    [anon_sym_forall] = ACTIONS(259),
    [anon_sym_choose] = ACTIONS(261),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(113)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(479),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1165),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1146),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(890),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(1376),
    [sym_line_comment] = STATE(113),
    [sym_block_comment] = STATE(113),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(109),
    [anon_sym_forall] = ACTIONS(109),
    [anon_sym_choose] = ACTIONS(111),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(114)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(479),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1165),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1146),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(876),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(1376),
    [sym_line_comment] = STATE(114),
    [sym_block_comment] = STATE(114),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(109),
    [anon_sym_forall] = ACTIONS(109),
    [anon_sym_choose] = ACTIONS(111),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(115)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(390),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1033),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1023),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1428),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(1032),
    [sym_line_comment] = STATE(115),
    [sym_block_comment] = STATE(115),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(467),
    [anon_sym_forall] = ACTIONS(467),
    [anon_sym_choose] = ACTIONS(469),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(116)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(479),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1165),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1146),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(890),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(116),
    [sym_block_comment] = STATE(116),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(109),
    [anon_sym_forall] = ACTIONS(109),
    [anon_sym_choose] = ACTIONS(111),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(117)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(479),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1165),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1146),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(876),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(117),
    [sym_block_comment] = STATE(117),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(109),
    [anon_sym_forall] = ACTIONS(109),
    [anon_sym_choose] = ACTIONS(111),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(118)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(419),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1307),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1275),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(890),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(118),
    [sym_block_comment] = STATE(118),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(223),
    [anon_sym_forall] = ACTIONS(223),
    [anon_sym_choose] = ACTIONS(225),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(119)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(419),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1307),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1275),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(876),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(119),
    [sym_block_comment] = STATE(119),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(223),
    [anon_sym_forall] = ACTIONS(223),
    [anon_sym_choose] = ACTIONS(225),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(120)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(432),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1393),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1365),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(890),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(120),
    [sym_block_comment] = STATE(120),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(259),
    [anon_sym_forall] = ACTIONS(259),
    [anon_sym_choose] = ACTIONS(261),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(121)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(432),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1393),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1365),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(876),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(121),
    [sym_block_comment] = STATE(121),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(259),
    [anon_sym_forall] = ACTIONS(259),
    [anon_sym_choose] = ACTIONS(261),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(122)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(390),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1033),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1023),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1426),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(1032),
    [sym_line_comment] = STATE(122),
    [sym_block_comment] = STATE(122),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(467),
    [anon_sym_forall] = ACTIONS(467),
    [anon_sym_choose] = ACTIONS(469),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(123)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(295),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1002),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1001),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(890),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(123),
    [sym_block_comment] = STATE(123),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(507),
    [anon_sym_forall] = ACTIONS(507),
    [anon_sym_choose] = ACTIONS(509),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(124)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(273),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1145),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1150),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(879),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(124),
    [sym_block_comment] = STATE(124),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(87),
    [anon_sym_forall] = ACTIONS(87),
    [anon_sym_choose] = ACTIONS(89),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(125)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(498),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1321),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1334),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(890),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(125),
    [sym_block_comment] = STATE(125),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(113),
    [anon_sym_forall] = ACTIONS(113),
    [anon_sym_choose] = ACTIONS(115),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(126)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(191),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(997),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(987),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(879),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(126),
    [sym_block_comment] = STATE(126),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(449),
    [anon_sym_forall] = ACTIONS(449),
    [anon_sym_choose] = ACTIONS(451),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(127)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(373),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1271),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1253),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1241),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(127),
    [sym_block_comment] = STATE(127),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(133),
    [anon_sym_forall] = ACTIONS(133),
    [anon_sym_choose] = ACTIONS(135),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(128)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(498),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1321),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1334),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1300),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(128),
    [sym_block_comment] = STATE(128),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(113),
    [anon_sym_forall] = ACTIONS(113),
    [anon_sym_choose] = ACTIONS(115),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(129)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(498),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1321),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1334),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(901),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(129),
    [sym_block_comment] = STATE(129),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(113),
    [anon_sym_forall] = ACTIONS(113),
    [anon_sym_choose] = ACTIONS(115),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(130)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(373),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1271),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1253),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1273),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(130),
    [sym_block_comment] = STATE(130),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(133),
    [anon_sym_forall] = ACTIONS(133),
    [anon_sym_choose] = ACTIONS(135),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(131)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(373),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1271),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1253),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(901),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(131),
    [sym_block_comment] = STATE(131),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(133),
    [anon_sym_forall] = ACTIONS(133),
    [anon_sym_choose] = ACTIONS(135),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(132)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(273),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1145),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1150),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1147),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(132),
    [sym_block_comment] = STATE(132),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(87),
    [anon_sym_forall] = ACTIONS(87),
    [anon_sym_choose] = ACTIONS(89),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(133)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(273),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1145),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1150),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(901),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(133),
    [sym_block_comment] = STATE(133),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(87),
    [anon_sym_forall] = ACTIONS(87),
    [anon_sym_choose] = ACTIONS(89),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(134)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(477),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1188),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1194),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1184),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(134),
    [sym_block_comment] = STATE(134),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(27),
    [anon_sym_forall] = ACTIONS(27),
    [anon_sym_choose] = ACTIONS(29),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(135)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(295),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1002),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1001),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1019),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(135),
    [sym_block_comment] = STATE(135),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(507),
    [anon_sym_forall] = ACTIONS(507),
    [anon_sym_choose] = ACTIONS(509),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(136)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(295),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1002),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1001),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(901),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(136),
    [sym_block_comment] = STATE(136),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(507),
    [anon_sym_forall] = ACTIONS(507),
    [anon_sym_choose] = ACTIONS(509),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(137)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(373),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1271),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1253),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(890),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(137),
    [sym_block_comment] = STATE(137),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(133),
    [anon_sym_forall] = ACTIONS(133),
    [anon_sym_choose] = ACTIONS(135),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(138)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(373),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1271),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1253),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(876),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(138),
    [sym_block_comment] = STATE(138),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(133),
    [anon_sym_forall] = ACTIONS(133),
    [anon_sym_choose] = ACTIONS(135),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(139)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(357),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1352),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1308),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1368),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(139),
    [sym_block_comment] = STATE(139),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(191),
    [anon_sym_forall] = ACTIONS(191),
    [anon_sym_choose] = ACTIONS(193),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(140)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(357),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1352),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1308),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(901),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(140),
    [sym_block_comment] = STATE(140),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(191),
    [anon_sym_forall] = ACTIONS(191),
    [anon_sym_choose] = ACTIONS(193),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(141)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(357),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1352),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1308),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(890),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(141),
    [sym_block_comment] = STATE(141),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(191),
    [anon_sym_forall] = ACTIONS(191),
    [anon_sym_choose] = ACTIONS(193),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(142)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(500),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1332),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1274),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1241),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(142),
    [sym_block_comment] = STATE(142),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(187),
    [anon_sym_forall] = ACTIONS(187),
    [anon_sym_choose] = ACTIONS(189),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(143)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(500),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1332),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1274),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(1320),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(143),
    [sym_block_comment] = STATE(143),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(187),
    [anon_sym_forall] = ACTIONS(187),
    [anon_sym_choose] = ACTIONS(189),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(144)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(500),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1332),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1274),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(901),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(144),
    [sym_block_comment] = STATE(144),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(187),
    [anon_sym_forall] = ACTIONS(187),
    [anon_sym_choose] = ACTIONS(189),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(145)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(500),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1332),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1274),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(890),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(145),
    [sym_block_comment] = STATE(145),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(187),
    [anon_sym_forall] = ACTIONS(187),
    [anon_sym_choose] = ACTIONS(189),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(146)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(477),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1188),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1194),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(890),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(146),
    [sym_block_comment] = STATE(146),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(27),
    [anon_sym_forall] = ACTIONS(27),
    [anon_sym_choose] = ACTIONS(29),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(147)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(1831),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(147),
    [sym_block_comment] = STATE(147),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_RPAREN] = ACTIONS(511),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(148)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(1831),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(148),
    [sym_block_comment] = STATE(148),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_RBRACK] = ACTIONS(513),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(149)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(1831),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(149),
    [sym_block_comment] = STATE(149),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_RPAREN] = ACTIONS(515),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(150)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(477),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1188),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1194),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(901),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(891),
    [sym_line_comment] = STATE(150),
    [sym_block_comment] = STATE(150),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(27),
    [anon_sym_forall] = ACTIONS(27),
    [anon_sym_choose] = ACTIONS(29),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(151)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(1831),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(151),
    [sym_block_comment] = STATE(151),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_RPAREN] = ACTIONS(517),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(152)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(889),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(419),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1307),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1275),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym__control_body] = STATE(879),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(934),
    [sym_line_comment] = STATE(152),
    [sym_block_comment] = STATE(152),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(223),
    [anon_sym_forall] = ACTIONS(223),
    [anon_sym_choose] = ACTIONS(225),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(153)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(933),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(479),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1165),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1146),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(153),
    [sym_block_comment] = STATE(153),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(109),
    [anon_sym_forall] = ACTIONS(109),
    [anon_sym_choose] = ACTIONS(111),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(154)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(896),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(390),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1033),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1023),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(154),
    [sym_block_comment] = STATE(154),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(467),
    [anon_sym_forall] = ACTIONS(467),
    [anon_sym_choose] = ACTIONS(469),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(155)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2243),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(155),
    [sym_block_comment] = STATE(155),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(156)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(907),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(390),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1033),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1023),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(156),
    [sym_block_comment] = STATE(156),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(467),
    [anon_sym_forall] = ACTIONS(467),
    [anon_sym_choose] = ACTIONS(469),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(157)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(909),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(390),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1033),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1023),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(157),
    [sym_block_comment] = STATE(157),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(467),
    [anon_sym_forall] = ACTIONS(467),
    [anon_sym_choose] = ACTIONS(469),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(158)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(916),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(390),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1033),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1023),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(158),
    [sym_block_comment] = STATE(158),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(467),
    [anon_sym_forall] = ACTIONS(467),
    [anon_sym_choose] = ACTIONS(469),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(159)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(917),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(390),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1033),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1023),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(159),
    [sym_block_comment] = STATE(159),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(467),
    [anon_sym_forall] = ACTIONS(467),
    [anon_sym_choose] = ACTIONS(469),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(160)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(922),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(390),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1033),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1023),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(160),
    [sym_block_comment] = STATE(160),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(467),
    [anon_sym_forall] = ACTIONS(467),
    [anon_sym_choose] = ACTIONS(469),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(161)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2047),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(161),
    [sym_block_comment] = STATE(161),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(162)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(1855),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(162),
    [sym_block_comment] = STATE(162),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(163)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2265),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(163),
    [sym_block_comment] = STATE(163),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(164)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2344),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(164),
    [sym_block_comment] = STATE(164),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(165)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2385),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(165),
    [sym_block_comment] = STATE(165),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(166)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(1817),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(175),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1356),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1335),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(166),
    [sym_block_comment] = STATE(166),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(207),
    [anon_sym_forall] = ACTIONS(207),
    [anon_sym_choose] = ACTIONS(209),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(167)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2454),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(167),
    [sym_block_comment] = STATE(167),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(168)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(933),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(168),
    [sym_block_comment] = STATE(168),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(169)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2273),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(169),
    [sym_block_comment] = STATE(169),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(170)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(939),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(170),
    [sym_block_comment] = STATE(170),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(171)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(895),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(477),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1188),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1194),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(171),
    [sym_block_comment] = STATE(171),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(27),
    [anon_sym_forall] = ACTIONS(27),
    [anon_sym_choose] = ACTIONS(29),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(172)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2280),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(172),
    [sym_block_comment] = STATE(172),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(173)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2419),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(173),
    [sym_block_comment] = STATE(173),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(174)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(895),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(393),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1296),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1319),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(174),
    [sym_block_comment] = STATE(174),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(169),
    [anon_sym_forall] = ACTIONS(169),
    [anon_sym_choose] = ACTIONS(171),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(175)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(906),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(175),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1356),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1335),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(175),
    [sym_block_comment] = STATE(175),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(207),
    [anon_sym_forall] = ACTIONS(207),
    [anon_sym_choose] = ACTIONS(209),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(176)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2364),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(176),
    [sym_block_comment] = STATE(176),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(177)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2447),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(177),
    [sym_block_comment] = STATE(177),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(178)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(887),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(175),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1356),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1335),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(178),
    [sym_block_comment] = STATE(178),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(207),
    [anon_sym_forall] = ACTIONS(207),
    [anon_sym_choose] = ACTIONS(209),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(179)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(933),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(175),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1356),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1335),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(179),
    [sym_block_comment] = STATE(179),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(207),
    [anon_sym_forall] = ACTIONS(207),
    [anon_sym_choose] = ACTIONS(209),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(180)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(939),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(175),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1356),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1335),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(180),
    [sym_block_comment] = STATE(180),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(207),
    [anon_sym_forall] = ACTIONS(207),
    [anon_sym_choose] = ACTIONS(209),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(181)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(892),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(175),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1356),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1335),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(181),
    [sym_block_comment] = STATE(181),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(207),
    [anon_sym_forall] = ACTIONS(207),
    [anon_sym_choose] = ACTIONS(209),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(182)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(893),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(175),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1356),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1335),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(182),
    [sym_block_comment] = STATE(182),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(207),
    [anon_sym_forall] = ACTIONS(207),
    [anon_sym_choose] = ACTIONS(209),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(183)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(896),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(175),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1356),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1335),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(183),
    [sym_block_comment] = STATE(183),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(207),
    [anon_sym_forall] = ACTIONS(207),
    [anon_sym_choose] = ACTIONS(209),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(184)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2332),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(184),
    [sym_block_comment] = STATE(184),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(185)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(907),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(175),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1356),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1335),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(185),
    [sym_block_comment] = STATE(185),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(207),
    [anon_sym_forall] = ACTIONS(207),
    [anon_sym_choose] = ACTIONS(209),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(186)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(909),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(175),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1356),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1335),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(186),
    [sym_block_comment] = STATE(186),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(207),
    [anon_sym_forall] = ACTIONS(207),
    [anon_sym_choose] = ACTIONS(209),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(187)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(916),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(175),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1356),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1335),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(187),
    [sym_block_comment] = STATE(187),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(207),
    [anon_sym_forall] = ACTIONS(207),
    [anon_sym_choose] = ACTIONS(209),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(188)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(917),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(175),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1356),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1335),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(188),
    [sym_block_comment] = STATE(188),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(207),
    [anon_sym_forall] = ACTIONS(207),
    [anon_sym_choose] = ACTIONS(209),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(189)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(922),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(175),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1356),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1335),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(189),
    [sym_block_comment] = STATE(189),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(207),
    [anon_sym_forall] = ACTIONS(207),
    [anon_sym_choose] = ACTIONS(209),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(190)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(895),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(191),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(997),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(987),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(190),
    [sym_block_comment] = STATE(190),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(449),
    [anon_sym_forall] = ACTIONS(449),
    [anon_sym_choose] = ACTIONS(451),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(191)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(906),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(191),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(997),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(987),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(191),
    [sym_block_comment] = STATE(191),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(449),
    [anon_sym_forall] = ACTIONS(449),
    [anon_sym_choose] = ACTIONS(451),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(192)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(1831),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(192),
    [sym_block_comment] = STATE(192),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(193)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(887),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(191),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(997),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(987),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(193),
    [sym_block_comment] = STATE(193),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(449),
    [anon_sym_forall] = ACTIONS(449),
    [anon_sym_choose] = ACTIONS(451),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(194)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(933),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(191),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(997),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(987),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(194),
    [sym_block_comment] = STATE(194),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(449),
    [anon_sym_forall] = ACTIONS(449),
    [anon_sym_choose] = ACTIONS(451),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(195)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(939),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(191),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(997),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(987),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(195),
    [sym_block_comment] = STATE(195),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(449),
    [anon_sym_forall] = ACTIONS(449),
    [anon_sym_choose] = ACTIONS(451),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(196)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(892),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(191),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(997),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(987),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(196),
    [sym_block_comment] = STATE(196),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(449),
    [anon_sym_forall] = ACTIONS(449),
    [anon_sym_choose] = ACTIONS(451),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(197)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(893),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(191),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(997),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(987),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(197),
    [sym_block_comment] = STATE(197),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(449),
    [anon_sym_forall] = ACTIONS(449),
    [anon_sym_choose] = ACTIONS(451),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(198)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(896),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(191),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(997),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(987),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(198),
    [sym_block_comment] = STATE(198),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(449),
    [anon_sym_forall] = ACTIONS(449),
    [anon_sym_choose] = ACTIONS(451),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(199)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2039),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(210),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1405),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1396),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(199),
    [sym_block_comment] = STATE(199),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(263),
    [anon_sym_forall] = ACTIONS(263),
    [anon_sym_choose] = ACTIONS(265),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(200)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(907),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(191),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(997),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(987),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(200),
    [sym_block_comment] = STATE(200),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(449),
    [anon_sym_forall] = ACTIONS(449),
    [anon_sym_choose] = ACTIONS(451),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(201)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(909),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(191),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(997),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(987),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(201),
    [sym_block_comment] = STATE(201),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(449),
    [anon_sym_forall] = ACTIONS(449),
    [anon_sym_choose] = ACTIONS(451),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(202)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(916),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(191),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(997),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(987),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(202),
    [sym_block_comment] = STATE(202),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(449),
    [anon_sym_forall] = ACTIONS(449),
    [anon_sym_choose] = ACTIONS(451),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(203)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(917),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(191),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(997),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(987),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(203),
    [sym_block_comment] = STATE(203),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(449),
    [anon_sym_forall] = ACTIONS(449),
    [anon_sym_choose] = ACTIONS(451),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(204)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(922),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(191),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(997),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(987),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(204),
    [sym_block_comment] = STATE(204),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(449),
    [anon_sym_forall] = ACTIONS(449),
    [anon_sym_choose] = ACTIONS(451),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(205)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2247),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(205),
    [sym_block_comment] = STATE(205),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(206)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2270),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(206),
    [sym_block_comment] = STATE(206),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(207)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2337),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(207),
    [sym_block_comment] = STATE(207),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(208)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(892),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(208),
    [sym_block_comment] = STATE(208),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(209)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(895),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(406),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1397),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1392),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(209),
    [sym_block_comment] = STATE(209),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(227),
    [anon_sym_forall] = ACTIONS(227),
    [anon_sym_choose] = ACTIONS(229),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(210)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(906),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(210),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1405),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1396),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(210),
    [sym_block_comment] = STATE(210),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(263),
    [anon_sym_forall] = ACTIONS(263),
    [anon_sym_choose] = ACTIONS(265),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(211)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2345),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(211),
    [sym_block_comment] = STATE(211),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(212)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(893),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(212),
    [sym_block_comment] = STATE(212),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(213)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(887),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(210),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1405),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1396),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(213),
    [sym_block_comment] = STATE(213),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(263),
    [anon_sym_forall] = ACTIONS(263),
    [anon_sym_choose] = ACTIONS(265),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(214)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(933),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(210),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1405),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1396),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(214),
    [sym_block_comment] = STATE(214),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(263),
    [anon_sym_forall] = ACTIONS(263),
    [anon_sym_choose] = ACTIONS(265),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(215)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(939),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(210),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1405),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1396),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(215),
    [sym_block_comment] = STATE(215),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(263),
    [anon_sym_forall] = ACTIONS(263),
    [anon_sym_choose] = ACTIONS(265),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(216)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(892),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(210),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1405),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1396),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(216),
    [sym_block_comment] = STATE(216),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(263),
    [anon_sym_forall] = ACTIONS(263),
    [anon_sym_choose] = ACTIONS(265),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(217)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(893),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(210),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1405),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1396),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(217),
    [sym_block_comment] = STATE(217),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(263),
    [anon_sym_forall] = ACTIONS(263),
    [anon_sym_choose] = ACTIONS(265),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(218)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(896),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(210),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1405),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1396),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(218),
    [sym_block_comment] = STATE(218),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(263),
    [anon_sym_forall] = ACTIONS(263),
    [anon_sym_choose] = ACTIONS(265),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(219)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(896),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(219),
    [sym_block_comment] = STATE(219),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(220)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(907),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(210),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1405),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1396),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(220),
    [sym_block_comment] = STATE(220),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(263),
    [anon_sym_forall] = ACTIONS(263),
    [anon_sym_choose] = ACTIONS(265),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(221)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(909),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(210),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1405),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1396),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(221),
    [sym_block_comment] = STATE(221),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(263),
    [anon_sym_forall] = ACTIONS(263),
    [anon_sym_choose] = ACTIONS(265),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(222)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(916),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(210),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1405),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1396),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(222),
    [sym_block_comment] = STATE(222),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(263),
    [anon_sym_forall] = ACTIONS(263),
    [anon_sym_choose] = ACTIONS(265),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(223)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(917),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(210),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1405),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1396),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(223),
    [sym_block_comment] = STATE(223),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(263),
    [anon_sym_forall] = ACTIONS(263),
    [anon_sym_choose] = ACTIONS(265),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(224)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(922),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(210),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1405),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1396),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(224),
    [sym_block_comment] = STATE(224),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(263),
    [anon_sym_forall] = ACTIONS(263),
    [anon_sym_choose] = ACTIONS(265),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(225)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2383),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(225),
    [sym_block_comment] = STATE(225),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(226)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2019),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(226),
    [sym_block_comment] = STATE(226),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(227)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2299),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(227),
    [sym_block_comment] = STATE(227),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(228)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2328),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(228),
    [sym_block_comment] = STATE(228),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(229)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(895),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(230),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1276),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1277),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(229),
    [sym_block_comment] = STATE(229),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(151),
    [anon_sym_forall] = ACTIONS(151),
    [anon_sym_choose] = ACTIONS(153),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(230)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(906),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(230),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1276),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1277),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(230),
    [sym_block_comment] = STATE(230),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(151),
    [anon_sym_forall] = ACTIONS(151),
    [anon_sym_choose] = ACTIONS(153),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(231)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2124),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(210),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1405),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1396),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(231),
    [sym_block_comment] = STATE(231),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(263),
    [anon_sym_forall] = ACTIONS(263),
    [anon_sym_choose] = ACTIONS(265),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(232)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2398),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(232),
    [sym_block_comment] = STATE(232),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(233)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(887),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(230),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1276),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1277),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(233),
    [sym_block_comment] = STATE(233),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(151),
    [anon_sym_forall] = ACTIONS(151),
    [anon_sym_choose] = ACTIONS(153),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(234)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(933),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(230),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1276),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1277),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(234),
    [sym_block_comment] = STATE(234),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(151),
    [anon_sym_forall] = ACTIONS(151),
    [anon_sym_choose] = ACTIONS(153),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(235)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(939),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(230),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1276),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1277),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(235),
    [sym_block_comment] = STATE(235),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(151),
    [anon_sym_forall] = ACTIONS(151),
    [anon_sym_choose] = ACTIONS(153),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(236)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(892),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(230),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1276),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1277),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(236),
    [sym_block_comment] = STATE(236),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(151),
    [anon_sym_forall] = ACTIONS(151),
    [anon_sym_choose] = ACTIONS(153),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(237)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(893),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(230),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1276),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1277),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(237),
    [sym_block_comment] = STATE(237),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(151),
    [anon_sym_forall] = ACTIONS(151),
    [anon_sym_choose] = ACTIONS(153),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(238)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(896),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(230),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1276),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1277),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(238),
    [sym_block_comment] = STATE(238),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(151),
    [anon_sym_forall] = ACTIONS(151),
    [anon_sym_choose] = ACTIONS(153),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(239)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(907),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(239),
    [sym_block_comment] = STATE(239),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(240)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(907),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(230),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1276),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1277),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(240),
    [sym_block_comment] = STATE(240),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(151),
    [anon_sym_forall] = ACTIONS(151),
    [anon_sym_choose] = ACTIONS(153),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(241)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(909),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(230),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1276),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1277),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(241),
    [sym_block_comment] = STATE(241),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(151),
    [anon_sym_forall] = ACTIONS(151),
    [anon_sym_choose] = ACTIONS(153),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(242)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(916),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(230),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1276),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1277),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(242),
    [sym_block_comment] = STATE(242),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(151),
    [anon_sym_forall] = ACTIONS(151),
    [anon_sym_choose] = ACTIONS(153),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(243)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(917),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(230),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1276),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1277),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(243),
    [sym_block_comment] = STATE(243),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(151),
    [anon_sym_forall] = ACTIONS(151),
    [anon_sym_choose] = ACTIONS(153),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(244)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(922),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(230),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1276),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1277),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(244),
    [sym_block_comment] = STATE(244),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(151),
    [anon_sym_forall] = ACTIONS(151),
    [anon_sym_choose] = ACTIONS(153),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(245)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2451),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(245),
    [sym_block_comment] = STATE(245),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(246)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(909),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(246),
    [sym_block_comment] = STATE(246),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(247)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(895),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(248),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1369),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1370),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(247),
    [sym_block_comment] = STATE(247),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(243),
    [anon_sym_forall] = ACTIONS(243),
    [anon_sym_choose] = ACTIONS(245),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(248)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(906),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(248),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1369),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1370),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(248),
    [sym_block_comment] = STATE(248),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(243),
    [anon_sym_forall] = ACTIONS(243),
    [anon_sym_choose] = ACTIONS(245),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(249)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(887),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(248),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1369),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1370),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(249),
    [sym_block_comment] = STATE(249),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(243),
    [anon_sym_forall] = ACTIONS(243),
    [anon_sym_choose] = ACTIONS(245),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(250)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(933),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(248),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1369),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1370),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(250),
    [sym_block_comment] = STATE(250),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(243),
    [anon_sym_forall] = ACTIONS(243),
    [anon_sym_choose] = ACTIONS(245),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(251)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(939),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(248),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1369),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1370),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(251),
    [sym_block_comment] = STATE(251),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(243),
    [anon_sym_forall] = ACTIONS(243),
    [anon_sym_choose] = ACTIONS(245),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(252)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(892),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(248),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1369),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1370),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(252),
    [sym_block_comment] = STATE(252),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(243),
    [anon_sym_forall] = ACTIONS(243),
    [anon_sym_choose] = ACTIONS(245),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(253)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(893),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(248),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1369),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1370),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(253),
    [sym_block_comment] = STATE(253),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(243),
    [anon_sym_forall] = ACTIONS(243),
    [anon_sym_choose] = ACTIONS(245),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(254)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(896),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(248),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1369),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1370),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(254),
    [sym_block_comment] = STATE(254),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(243),
    [anon_sym_forall] = ACTIONS(243),
    [anon_sym_choose] = ACTIONS(245),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(255)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2184),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(255),
    [sym_block_comment] = STATE(255),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(256)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(907),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(248),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1369),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1370),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(256),
    [sym_block_comment] = STATE(256),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(243),
    [anon_sym_forall] = ACTIONS(243),
    [anon_sym_choose] = ACTIONS(245),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(257)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(909),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(248),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1369),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1370),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(257),
    [sym_block_comment] = STATE(257),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(243),
    [anon_sym_forall] = ACTIONS(243),
    [anon_sym_choose] = ACTIONS(245),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(258)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(916),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(248),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1369),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1370),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(258),
    [sym_block_comment] = STATE(258),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(243),
    [anon_sym_forall] = ACTIONS(243),
    [anon_sym_choose] = ACTIONS(245),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(259)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(917),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(248),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1369),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1370),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(259),
    [sym_block_comment] = STATE(259),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(243),
    [anon_sym_forall] = ACTIONS(243),
    [anon_sym_choose] = ACTIONS(245),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(260)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(922),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(248),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1369),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1370),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(260),
    [sym_block_comment] = STATE(260),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(243),
    [anon_sym_forall] = ACTIONS(243),
    [anon_sym_choose] = ACTIONS(245),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(261)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2196),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(261),
    [sym_block_comment] = STATE(261),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(262)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2197),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(262),
    [sym_block_comment] = STATE(262),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(263)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2199),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(263),
    [sym_block_comment] = STATE(263),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(264)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(916),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(264),
    [sym_block_comment] = STATE(264),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(265)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(917),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(265),
    [sym_block_comment] = STATE(265),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(266)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2423),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(266),
    [sym_block_comment] = STATE(266),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(267)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2429),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(267),
    [sym_block_comment] = STATE(267),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(268)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2156),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(268),
    [sym_block_comment] = STATE(268),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(269)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(922),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(269),
    [sym_block_comment] = STATE(269),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(270)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2195),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(270),
    [sym_block_comment] = STATE(270),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(271)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(887),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(271),
    [sym_block_comment] = STATE(271),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(272)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(895),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(273),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1145),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1150),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(272),
    [sym_block_comment] = STATE(272),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(87),
    [anon_sym_forall] = ACTIONS(87),
    [anon_sym_choose] = ACTIONS(89),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(273)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(906),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(273),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1145),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1150),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(273),
    [sym_block_comment] = STATE(273),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(87),
    [anon_sym_forall] = ACTIONS(87),
    [anon_sym_choose] = ACTIONS(89),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(274)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2457),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(274),
    [sym_block_comment] = STATE(274),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(275)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(906),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(275),
    [sym_block_comment] = STATE(275),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(276)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(887),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(273),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1145),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1150),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(276),
    [sym_block_comment] = STATE(276),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(87),
    [anon_sym_forall] = ACTIONS(87),
    [anon_sym_choose] = ACTIONS(89),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(277)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(1817),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(277),
    [sym_block_comment] = STATE(277),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(278)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(887),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(477),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1188),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1194),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(278),
    [sym_block_comment] = STATE(278),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(27),
    [anon_sym_forall] = ACTIONS(27),
    [anon_sym_choose] = ACTIONS(29),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(279)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2287),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(279),
    [sym_block_comment] = STATE(279),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(280)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(933),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(477),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1188),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1194),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(280),
    [sym_block_comment] = STATE(280),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(27),
    [anon_sym_forall] = ACTIONS(27),
    [anon_sym_choose] = ACTIONS(29),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(281)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(939),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(477),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1188),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1194),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(281),
    [sym_block_comment] = STATE(281),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(27),
    [anon_sym_forall] = ACTIONS(27),
    [anon_sym_choose] = ACTIONS(29),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(282)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2335),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(282),
    [sym_block_comment] = STATE(282),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(283)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(892),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(477),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1188),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1194),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(283),
    [sym_block_comment] = STATE(283),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(27),
    [anon_sym_forall] = ACTIONS(27),
    [anon_sym_choose] = ACTIONS(29),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(284)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2339),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(284),
    [sym_block_comment] = STATE(284),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(285)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(893),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(477),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1188),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1194),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(285),
    [sym_block_comment] = STATE(285),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(27),
    [anon_sym_forall] = ACTIONS(27),
    [anon_sym_choose] = ACTIONS(29),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(286)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(896),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(477),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1188),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1194),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(286),
    [sym_block_comment] = STATE(286),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(27),
    [anon_sym_forall] = ACTIONS(27),
    [anon_sym_choose] = ACTIONS(29),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(287)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(907),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(477),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1188),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1194),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(287),
    [sym_block_comment] = STATE(287),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(27),
    [anon_sym_forall] = ACTIONS(27),
    [anon_sym_choose] = ACTIONS(29),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(288)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2379),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(288),
    [sym_block_comment] = STATE(288),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(289)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(909),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(477),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1188),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1194),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(289),
    [sym_block_comment] = STATE(289),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(27),
    [anon_sym_forall] = ACTIONS(27),
    [anon_sym_choose] = ACTIONS(29),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(290)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(916),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(477),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1188),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1194),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(290),
    [sym_block_comment] = STATE(290),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(27),
    [anon_sym_forall] = ACTIONS(27),
    [anon_sym_choose] = ACTIONS(29),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(291)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(917),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(477),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1188),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1194),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(291),
    [sym_block_comment] = STATE(291),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(27),
    [anon_sym_forall] = ACTIONS(27),
    [anon_sym_choose] = ACTIONS(29),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(292)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(922),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(477),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1188),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1194),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(292),
    [sym_block_comment] = STATE(292),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(27),
    [anon_sym_forall] = ACTIONS(27),
    [anon_sym_choose] = ACTIONS(29),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(293)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(933),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(273),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1145),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1150),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(293),
    [sym_block_comment] = STATE(293),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(87),
    [anon_sym_forall] = ACTIONS(87),
    [anon_sym_choose] = ACTIONS(89),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(294)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(895),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(390),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1033),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1023),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(294),
    [sym_block_comment] = STATE(294),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(467),
    [anon_sym_forall] = ACTIONS(467),
    [anon_sym_choose] = ACTIONS(469),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(295)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(906),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(295),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1002),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1001),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(295),
    [sym_block_comment] = STATE(295),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(507),
    [anon_sym_forall] = ACTIONS(507),
    [anon_sym_choose] = ACTIONS(509),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(296)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(887),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(295),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1002),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1001),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(296),
    [sym_block_comment] = STATE(296),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(507),
    [anon_sym_forall] = ACTIONS(507),
    [anon_sym_choose] = ACTIONS(509),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(297)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(933),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(295),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1002),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1001),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(297),
    [sym_block_comment] = STATE(297),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(507),
    [anon_sym_forall] = ACTIONS(507),
    [anon_sym_choose] = ACTIONS(509),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(298)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(939),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(295),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1002),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1001),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(298),
    [sym_block_comment] = STATE(298),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(507),
    [anon_sym_forall] = ACTIONS(507),
    [anon_sym_choose] = ACTIONS(509),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(299)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(892),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(295),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1002),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1001),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(299),
    [sym_block_comment] = STATE(299),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(507),
    [anon_sym_forall] = ACTIONS(507),
    [anon_sym_choose] = ACTIONS(509),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(300)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(893),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(295),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1002),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1001),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(300),
    [sym_block_comment] = STATE(300),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(507),
    [anon_sym_forall] = ACTIONS(507),
    [anon_sym_choose] = ACTIONS(509),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(301)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(896),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(295),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1002),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1001),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(301),
    [sym_block_comment] = STATE(301),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(507),
    [anon_sym_forall] = ACTIONS(507),
    [anon_sym_choose] = ACTIONS(509),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(302)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(907),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(295),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1002),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1001),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(302),
    [sym_block_comment] = STATE(302),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(507),
    [anon_sym_forall] = ACTIONS(507),
    [anon_sym_choose] = ACTIONS(509),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(303)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(909),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(295),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1002),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1001),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(303),
    [sym_block_comment] = STATE(303),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(507),
    [anon_sym_forall] = ACTIONS(507),
    [anon_sym_choose] = ACTIONS(509),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(304)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(916),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(295),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1002),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1001),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(304),
    [sym_block_comment] = STATE(304),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(507),
    [anon_sym_forall] = ACTIONS(507),
    [anon_sym_choose] = ACTIONS(509),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(305)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(917),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(295),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1002),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1001),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(305),
    [sym_block_comment] = STATE(305),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(507),
    [anon_sym_forall] = ACTIONS(507),
    [anon_sym_choose] = ACTIONS(509),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(306)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(922),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(295),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1002),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1001),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(306),
    [sym_block_comment] = STATE(306),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(507),
    [anon_sym_forall] = ACTIONS(507),
    [anon_sym_choose] = ACTIONS(509),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(307)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(939),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(273),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1145),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1150),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(307),
    [sym_block_comment] = STATE(307),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(87),
    [anon_sym_forall] = ACTIONS(87),
    [anon_sym_choose] = ACTIONS(89),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(308)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(892),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(273),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1145),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1150),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(308),
    [sym_block_comment] = STATE(308),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(87),
    [anon_sym_forall] = ACTIONS(87),
    [anon_sym_choose] = ACTIONS(89),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(309)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(895),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(310),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1014),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(988),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(309),
    [sym_block_comment] = STATE(309),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(479),
    [anon_sym_forall] = ACTIONS(479),
    [anon_sym_choose] = ACTIONS(481),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(310)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(906),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(310),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1014),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(988),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(310),
    [sym_block_comment] = STATE(310),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(479),
    [anon_sym_forall] = ACTIONS(479),
    [anon_sym_choose] = ACTIONS(481),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(311)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(887),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(310),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1014),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(988),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(311),
    [sym_block_comment] = STATE(311),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(479),
    [anon_sym_forall] = ACTIONS(479),
    [anon_sym_choose] = ACTIONS(481),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(312)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(933),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(310),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1014),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(988),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(312),
    [sym_block_comment] = STATE(312),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(479),
    [anon_sym_forall] = ACTIONS(479),
    [anon_sym_choose] = ACTIONS(481),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(313)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(939),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(310),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1014),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(988),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(313),
    [sym_block_comment] = STATE(313),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(479),
    [anon_sym_forall] = ACTIONS(479),
    [anon_sym_choose] = ACTIONS(481),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(314)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(892),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(310),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1014),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(988),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(314),
    [sym_block_comment] = STATE(314),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(479),
    [anon_sym_forall] = ACTIONS(479),
    [anon_sym_choose] = ACTIONS(481),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(315)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(893),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(310),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1014),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(988),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(315),
    [sym_block_comment] = STATE(315),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(479),
    [anon_sym_forall] = ACTIONS(479),
    [anon_sym_choose] = ACTIONS(481),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(316)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(896),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(310),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1014),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(988),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(316),
    [sym_block_comment] = STATE(316),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(479),
    [anon_sym_forall] = ACTIONS(479),
    [anon_sym_choose] = ACTIONS(481),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(317)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(893),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(390),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1033),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1023),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(317),
    [sym_block_comment] = STATE(317),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(467),
    [anon_sym_forall] = ACTIONS(467),
    [anon_sym_choose] = ACTIONS(469),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(318)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(909),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(310),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1014),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(988),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(318),
    [sym_block_comment] = STATE(318),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(479),
    [anon_sym_forall] = ACTIONS(479),
    [anon_sym_choose] = ACTIONS(481),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(319)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(916),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(310),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1014),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(988),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(319),
    [sym_block_comment] = STATE(319),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(479),
    [anon_sym_forall] = ACTIONS(479),
    [anon_sym_choose] = ACTIONS(481),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(320)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(917),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(310),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1014),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(988),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(320),
    [sym_block_comment] = STATE(320),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(479),
    [anon_sym_forall] = ACTIONS(479),
    [anon_sym_choose] = ACTIONS(481),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(321)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(922),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(310),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1014),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(988),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(321),
    [sym_block_comment] = STATE(321),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(479),
    [anon_sym_forall] = ACTIONS(479),
    [anon_sym_choose] = ACTIONS(481),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(322)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2326),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(322),
    [sym_block_comment] = STATE(322),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(323)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2376),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(323),
    [sym_block_comment] = STATE(323),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(324)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2391),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(324),
    [sym_block_comment] = STATE(324),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(325)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2395),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(325),
    [sym_block_comment] = STATE(325),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(326)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2320),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(326),
    [sym_block_comment] = STATE(326),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(327)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2420),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(327),
    [sym_block_comment] = STATE(327),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(328)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2422),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(328),
    [sym_block_comment] = STATE(328),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(329)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2456),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(329),
    [sym_block_comment] = STATE(329),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(330)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2257),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(330),
    [sym_block_comment] = STATE(330),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(331)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2261),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(331),
    [sym_block_comment] = STATE(331),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(332)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2262),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(332),
    [sym_block_comment] = STATE(332),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(333)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2263),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(333),
    [sym_block_comment] = STATE(333),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(334)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2365),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(334),
    [sym_block_comment] = STATE(334),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(335)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2367),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(335),
    [sym_block_comment] = STATE(335),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(336)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2368),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(336),
    [sym_block_comment] = STATE(336),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(337)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2372),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(337),
    [sym_block_comment] = STATE(337),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(338)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2428),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(338),
    [sym_block_comment] = STATE(338),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(339)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2436),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(339),
    [sym_block_comment] = STATE(339),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(340)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2438),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(340),
    [sym_block_comment] = STATE(340),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(341)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2439),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(341),
    [sym_block_comment] = STATE(341),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(342)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2294),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(342),
    [sym_block_comment] = STATE(342),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(343)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2296),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(343),
    [sym_block_comment] = STATE(343),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(344)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2297),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(344),
    [sym_block_comment] = STATE(344),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(345)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2298),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(345),
    [sym_block_comment] = STATE(345),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(346)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2253),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(346),
    [sym_block_comment] = STATE(346),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(347)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2258),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(347),
    [sym_block_comment] = STATE(347),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(348)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2295),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(348),
    [sym_block_comment] = STATE(348),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(349)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2324),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(349),
    [sym_block_comment] = STATE(349),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(350)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2311),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(350),
    [sym_block_comment] = STATE(350),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(351)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2313),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(351),
    [sym_block_comment] = STATE(351),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(352)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2318),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(352),
    [sym_block_comment] = STATE(352),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(353)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2319),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(353),
    [sym_block_comment] = STATE(353),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(354)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(893),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(273),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1145),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1150),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(354),
    [sym_block_comment] = STATE(354),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(87),
    [anon_sym_forall] = ACTIONS(87),
    [anon_sym_choose] = ACTIONS(89),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(355)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(896),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(273),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1145),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1150),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(355),
    [sym_block_comment] = STATE(355),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(87),
    [anon_sym_forall] = ACTIONS(87),
    [anon_sym_choose] = ACTIONS(89),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(356)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(895),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(498),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1321),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1334),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(356),
    [sym_block_comment] = STATE(356),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(113),
    [anon_sym_forall] = ACTIONS(113),
    [anon_sym_choose] = ACTIONS(115),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(357)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(906),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(357),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1352),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1308),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(357),
    [sym_block_comment] = STATE(357),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(191),
    [anon_sym_forall] = ACTIONS(191),
    [anon_sym_choose] = ACTIONS(193),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(358)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2240),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(358),
    [sym_block_comment] = STATE(358),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(359)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(907),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(273),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1145),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1150),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(359),
    [sym_block_comment] = STATE(359),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(87),
    [anon_sym_forall] = ACTIONS(87),
    [anon_sym_choose] = ACTIONS(89),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(360)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(887),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(357),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1352),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1308),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(360),
    [sym_block_comment] = STATE(360),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(191),
    [anon_sym_forall] = ACTIONS(191),
    [anon_sym_choose] = ACTIONS(193),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(361)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(933),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(357),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1352),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1308),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(361),
    [sym_block_comment] = STATE(361),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(191),
    [anon_sym_forall] = ACTIONS(191),
    [anon_sym_choose] = ACTIONS(193),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(362)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(939),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(357),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1352),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1308),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(362),
    [sym_block_comment] = STATE(362),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(191),
    [anon_sym_forall] = ACTIONS(191),
    [anon_sym_choose] = ACTIONS(193),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(363)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(892),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(357),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1352),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1308),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(363),
    [sym_block_comment] = STATE(363),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(191),
    [anon_sym_forall] = ACTIONS(191),
    [anon_sym_choose] = ACTIONS(193),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(364)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(893),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(357),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1352),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1308),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(364),
    [sym_block_comment] = STATE(364),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(191),
    [anon_sym_forall] = ACTIONS(191),
    [anon_sym_choose] = ACTIONS(193),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(365)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(896),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(357),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1352),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1308),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(365),
    [sym_block_comment] = STATE(365),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(191),
    [anon_sym_forall] = ACTIONS(191),
    [anon_sym_choose] = ACTIONS(193),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(366)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(909),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(273),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1145),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1150),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(366),
    [sym_block_comment] = STATE(366),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(87),
    [anon_sym_forall] = ACTIONS(87),
    [anon_sym_choose] = ACTIONS(89),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(367)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(907),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(357),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1352),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1308),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(367),
    [sym_block_comment] = STATE(367),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(191),
    [anon_sym_forall] = ACTIONS(191),
    [anon_sym_choose] = ACTIONS(193),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(368)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(909),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(357),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1352),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1308),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(368),
    [sym_block_comment] = STATE(368),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(191),
    [anon_sym_forall] = ACTIONS(191),
    [anon_sym_choose] = ACTIONS(193),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(369)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(916),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(357),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1352),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1308),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(369),
    [sym_block_comment] = STATE(369),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(191),
    [anon_sym_forall] = ACTIONS(191),
    [anon_sym_choose] = ACTIONS(193),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(370)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(917),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(357),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1352),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1308),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(370),
    [sym_block_comment] = STATE(370),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(191),
    [anon_sym_forall] = ACTIONS(191),
    [anon_sym_choose] = ACTIONS(193),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(371)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(922),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(357),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1352),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1308),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(371),
    [sym_block_comment] = STATE(371),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(191),
    [anon_sym_forall] = ACTIONS(191),
    [anon_sym_choose] = ACTIONS(193),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(372)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(895),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(373),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1271),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1253),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(372),
    [sym_block_comment] = STATE(372),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(133),
    [anon_sym_forall] = ACTIONS(133),
    [anon_sym_choose] = ACTIONS(135),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(373)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(906),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(373),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1271),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1253),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(373),
    [sym_block_comment] = STATE(373),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(133),
    [anon_sym_forall] = ACTIONS(133),
    [anon_sym_choose] = ACTIONS(135),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(374)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(916),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(273),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1145),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1150),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(374),
    [sym_block_comment] = STATE(374),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(87),
    [anon_sym_forall] = ACTIONS(87),
    [anon_sym_choose] = ACTIONS(89),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(375)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(887),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(373),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1271),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1253),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(375),
    [sym_block_comment] = STATE(375),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(133),
    [anon_sym_forall] = ACTIONS(133),
    [anon_sym_choose] = ACTIONS(135),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(376)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(933),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(373),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1271),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1253),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(376),
    [sym_block_comment] = STATE(376),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(133),
    [anon_sym_forall] = ACTIONS(133),
    [anon_sym_choose] = ACTIONS(135),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(377)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(939),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(373),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1271),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1253),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(377),
    [sym_block_comment] = STATE(377),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(133),
    [anon_sym_forall] = ACTIONS(133),
    [anon_sym_choose] = ACTIONS(135),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(378)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(892),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(373),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1271),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1253),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(378),
    [sym_block_comment] = STATE(378),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(133),
    [anon_sym_forall] = ACTIONS(133),
    [anon_sym_choose] = ACTIONS(135),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(379)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(893),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(373),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1271),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1253),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(379),
    [sym_block_comment] = STATE(379),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(133),
    [anon_sym_forall] = ACTIONS(133),
    [anon_sym_choose] = ACTIONS(135),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(380)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(896),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(373),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1271),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1253),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(380),
    [sym_block_comment] = STATE(380),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(133),
    [anon_sym_forall] = ACTIONS(133),
    [anon_sym_choose] = ACTIONS(135),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(381)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(917),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(273),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1145),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1150),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(381),
    [sym_block_comment] = STATE(381),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(87),
    [anon_sym_forall] = ACTIONS(87),
    [anon_sym_choose] = ACTIONS(89),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(382)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(907),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(373),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1271),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1253),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(382),
    [sym_block_comment] = STATE(382),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(133),
    [anon_sym_forall] = ACTIONS(133),
    [anon_sym_choose] = ACTIONS(135),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(383)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(909),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(373),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1271),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1253),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(383),
    [sym_block_comment] = STATE(383),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(133),
    [anon_sym_forall] = ACTIONS(133),
    [anon_sym_choose] = ACTIONS(135),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(384)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(916),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(373),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1271),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1253),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(384),
    [sym_block_comment] = STATE(384),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(133),
    [anon_sym_forall] = ACTIONS(133),
    [anon_sym_choose] = ACTIONS(135),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(385)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(917),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(373),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1271),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1253),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(385),
    [sym_block_comment] = STATE(385),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(133),
    [anon_sym_forall] = ACTIONS(133),
    [anon_sym_choose] = ACTIONS(135),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(386)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(922),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(373),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1271),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1253),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(386),
    [sym_block_comment] = STATE(386),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(133),
    [anon_sym_forall] = ACTIONS(133),
    [anon_sym_choose] = ACTIONS(135),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(387)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(922),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(273),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1145),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1150),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(387),
    [sym_block_comment] = STATE(387),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(87),
    [anon_sym_forall] = ACTIONS(87),
    [anon_sym_choose] = ACTIONS(89),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(388)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2323),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(388),
    [sym_block_comment] = STATE(388),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(389)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(895),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(295),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1002),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1001),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(389),
    [sym_block_comment] = STATE(389),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(507),
    [anon_sym_forall] = ACTIONS(507),
    [anon_sym_choose] = ACTIONS(509),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(390)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(906),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(390),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1033),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1023),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(390),
    [sym_block_comment] = STATE(390),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(467),
    [anon_sym_forall] = ACTIONS(467),
    [anon_sym_choose] = ACTIONS(469),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(391)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2221),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(391),
    [sym_block_comment] = STATE(391),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(392)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(895),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(175),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1356),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1335),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(392),
    [sym_block_comment] = STATE(392),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(207),
    [anon_sym_forall] = ACTIONS(207),
    [anon_sym_choose] = ACTIONS(209),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(393)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(906),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(393),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1296),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1319),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(393),
    [sym_block_comment] = STATE(393),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(169),
    [anon_sym_forall] = ACTIONS(169),
    [anon_sym_choose] = ACTIONS(171),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(394)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(887),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(393),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1296),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1319),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(394),
    [sym_block_comment] = STATE(394),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(169),
    [anon_sym_forall] = ACTIONS(169),
    [anon_sym_choose] = ACTIONS(171),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(395)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(933),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(393),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1296),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1319),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(395),
    [sym_block_comment] = STATE(395),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(169),
    [anon_sym_forall] = ACTIONS(169),
    [anon_sym_choose] = ACTIONS(171),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(396)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(939),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(393),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1296),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1319),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(396),
    [sym_block_comment] = STATE(396),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(169),
    [anon_sym_forall] = ACTIONS(169),
    [anon_sym_choose] = ACTIONS(171),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(397)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(892),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(393),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1296),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1319),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(397),
    [sym_block_comment] = STATE(397),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(169),
    [anon_sym_forall] = ACTIONS(169),
    [anon_sym_choose] = ACTIONS(171),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(398)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(893),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(393),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1296),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1319),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(398),
    [sym_block_comment] = STATE(398),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(169),
    [anon_sym_forall] = ACTIONS(169),
    [anon_sym_choose] = ACTIONS(171),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(399)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(896),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(393),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1296),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1319),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(399),
    [sym_block_comment] = STATE(399),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(169),
    [anon_sym_forall] = ACTIONS(169),
    [anon_sym_choose] = ACTIONS(171),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(400)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(907),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(393),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1296),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1319),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(400),
    [sym_block_comment] = STATE(400),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(169),
    [anon_sym_forall] = ACTIONS(169),
    [anon_sym_choose] = ACTIONS(171),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(401)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(909),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(393),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1296),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1319),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(401),
    [sym_block_comment] = STATE(401),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(169),
    [anon_sym_forall] = ACTIONS(169),
    [anon_sym_choose] = ACTIONS(171),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(402)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(916),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(393),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1296),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1319),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(402),
    [sym_block_comment] = STATE(402),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(169),
    [anon_sym_forall] = ACTIONS(169),
    [anon_sym_choose] = ACTIONS(171),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(403)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(917),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(393),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1296),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1319),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(403),
    [sym_block_comment] = STATE(403),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(169),
    [anon_sym_forall] = ACTIONS(169),
    [anon_sym_choose] = ACTIONS(171),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(404)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(922),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(393),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1296),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1319),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(404),
    [sym_block_comment] = STATE(404),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(169),
    [anon_sym_forall] = ACTIONS(169),
    [anon_sym_choose] = ACTIONS(171),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(405)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(895),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(210),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1405),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1396),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(405),
    [sym_block_comment] = STATE(405),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(263),
    [anon_sym_forall] = ACTIONS(263),
    [anon_sym_choose] = ACTIONS(265),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(406)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(906),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(406),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1397),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1392),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(406),
    [sym_block_comment] = STATE(406),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(227),
    [anon_sym_forall] = ACTIONS(227),
    [anon_sym_choose] = ACTIONS(229),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(407)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(887),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(406),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1397),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1392),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(407),
    [sym_block_comment] = STATE(407),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(227),
    [anon_sym_forall] = ACTIONS(227),
    [anon_sym_choose] = ACTIONS(229),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(408)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(933),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(406),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1397),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1392),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(408),
    [sym_block_comment] = STATE(408),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(227),
    [anon_sym_forall] = ACTIONS(227),
    [anon_sym_choose] = ACTIONS(229),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(409)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(939),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(406),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1397),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1392),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(409),
    [sym_block_comment] = STATE(409),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(227),
    [anon_sym_forall] = ACTIONS(227),
    [anon_sym_choose] = ACTIONS(229),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(410)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(892),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(406),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1397),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1392),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(410),
    [sym_block_comment] = STATE(410),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(227),
    [anon_sym_forall] = ACTIONS(227),
    [anon_sym_choose] = ACTIONS(229),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(411)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(893),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(406),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1397),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1392),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(411),
    [sym_block_comment] = STATE(411),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(227),
    [anon_sym_forall] = ACTIONS(227),
    [anon_sym_choose] = ACTIONS(229),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(412)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(896),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(406),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1397),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1392),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(412),
    [sym_block_comment] = STATE(412),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(227),
    [anon_sym_forall] = ACTIONS(227),
    [anon_sym_choose] = ACTIONS(229),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(413)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(907),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(406),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1397),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1392),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(413),
    [sym_block_comment] = STATE(413),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(227),
    [anon_sym_forall] = ACTIONS(227),
    [anon_sym_choose] = ACTIONS(229),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(414)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(909),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(406),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1397),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1392),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(414),
    [sym_block_comment] = STATE(414),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(227),
    [anon_sym_forall] = ACTIONS(227),
    [anon_sym_choose] = ACTIONS(229),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(415)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(916),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(406),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1397),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1392),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(415),
    [sym_block_comment] = STATE(415),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(227),
    [anon_sym_forall] = ACTIONS(227),
    [anon_sym_choose] = ACTIONS(229),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(416)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(917),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(406),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1397),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1392),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(416),
    [sym_block_comment] = STATE(416),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(227),
    [anon_sym_forall] = ACTIONS(227),
    [anon_sym_choose] = ACTIONS(229),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(417)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(922),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(406),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1397),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1392),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(417),
    [sym_block_comment] = STATE(417),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(227),
    [anon_sym_forall] = ACTIONS(227),
    [anon_sym_choose] = ACTIONS(229),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(418)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(895),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(419),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1307),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1275),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(418),
    [sym_block_comment] = STATE(418),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(223),
    [anon_sym_forall] = ACTIONS(223),
    [anon_sym_choose] = ACTIONS(225),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(419)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(906),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(419),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1307),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1275),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(419),
    [sym_block_comment] = STATE(419),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(223),
    [anon_sym_forall] = ACTIONS(223),
    [anon_sym_choose] = ACTIONS(225),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(420)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(887),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(419),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1307),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1275),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(420),
    [sym_block_comment] = STATE(420),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(223),
    [anon_sym_forall] = ACTIONS(223),
    [anon_sym_choose] = ACTIONS(225),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(421)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(933),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(419),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1307),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1275),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(421),
    [sym_block_comment] = STATE(421),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(223),
    [anon_sym_forall] = ACTIONS(223),
    [anon_sym_choose] = ACTIONS(225),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(422)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(939),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(419),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1307),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1275),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(422),
    [sym_block_comment] = STATE(422),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(223),
    [anon_sym_forall] = ACTIONS(223),
    [anon_sym_choose] = ACTIONS(225),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(423)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(892),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(419),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1307),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1275),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(423),
    [sym_block_comment] = STATE(423),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(223),
    [anon_sym_forall] = ACTIONS(223),
    [anon_sym_choose] = ACTIONS(225),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(424)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(893),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(419),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1307),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1275),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(424),
    [sym_block_comment] = STATE(424),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(223),
    [anon_sym_forall] = ACTIONS(223),
    [anon_sym_choose] = ACTIONS(225),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(425)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(896),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(419),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1307),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1275),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(425),
    [sym_block_comment] = STATE(425),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(223),
    [anon_sym_forall] = ACTIONS(223),
    [anon_sym_choose] = ACTIONS(225),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(426)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(907),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(419),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1307),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1275),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(426),
    [sym_block_comment] = STATE(426),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(223),
    [anon_sym_forall] = ACTIONS(223),
    [anon_sym_choose] = ACTIONS(225),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(427)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(909),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(419),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1307),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1275),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(427),
    [sym_block_comment] = STATE(427),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(223),
    [anon_sym_forall] = ACTIONS(223),
    [anon_sym_choose] = ACTIONS(225),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(428)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(916),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(419),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1307),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1275),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(428),
    [sym_block_comment] = STATE(428),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(223),
    [anon_sym_forall] = ACTIONS(223),
    [anon_sym_choose] = ACTIONS(225),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(429)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(917),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(419),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1307),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1275),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(429),
    [sym_block_comment] = STATE(429),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(223),
    [anon_sym_forall] = ACTIONS(223),
    [anon_sym_choose] = ACTIONS(225),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(430)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__expr] = STATE(922),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(419),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1307),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1275),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(430),
    [sym_block_comment] = STATE(430),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(223),
    [anon_sym_forall] = ACTIONS(223),
    [anon_sym_choose] = ACTIONS(225),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(431)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(895),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(432),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1393),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1365),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(431),
    [sym_block_comment] = STATE(431),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(259),
    [anon_sym_forall] = ACTIONS(259),
    [anon_sym_choose] = ACTIONS(261),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(432)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(906),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(432),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1393),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1365),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(432),
    [sym_block_comment] = STATE(432),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(259),
    [anon_sym_forall] = ACTIONS(259),
    [anon_sym_choose] = ACTIONS(261),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(433)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(887),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(432),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1393),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1365),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(433),
    [sym_block_comment] = STATE(433),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(259),
    [anon_sym_forall] = ACTIONS(259),
    [anon_sym_choose] = ACTIONS(261),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(434)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(933),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(432),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1393),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1365),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(434),
    [sym_block_comment] = STATE(434),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(259),
    [anon_sym_forall] = ACTIONS(259),
    [anon_sym_choose] = ACTIONS(261),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(435)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(939),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(432),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1393),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1365),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(435),
    [sym_block_comment] = STATE(435),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(259),
    [anon_sym_forall] = ACTIONS(259),
    [anon_sym_choose] = ACTIONS(261),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(436)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(892),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(432),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1393),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1365),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(436),
    [sym_block_comment] = STATE(436),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(259),
    [anon_sym_forall] = ACTIONS(259),
    [anon_sym_choose] = ACTIONS(261),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(437)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(893),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(432),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1393),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1365),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(437),
    [sym_block_comment] = STATE(437),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(259),
    [anon_sym_forall] = ACTIONS(259),
    [anon_sym_choose] = ACTIONS(261),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(438)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(896),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(432),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1393),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1365),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(438),
    [sym_block_comment] = STATE(438),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(259),
    [anon_sym_forall] = ACTIONS(259),
    [anon_sym_choose] = ACTIONS(261),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(439)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(907),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(432),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1393),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1365),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(439),
    [sym_block_comment] = STATE(439),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(259),
    [anon_sym_forall] = ACTIONS(259),
    [anon_sym_choose] = ACTIONS(261),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(440)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(909),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(432),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1393),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1365),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(440),
    [sym_block_comment] = STATE(440),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(259),
    [anon_sym_forall] = ACTIONS(259),
    [anon_sym_choose] = ACTIONS(261),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(441)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(916),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(432),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1393),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1365),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(441),
    [sym_block_comment] = STATE(441),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(259),
    [anon_sym_forall] = ACTIONS(259),
    [anon_sym_choose] = ACTIONS(261),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(442)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(917),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(432),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1393),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1365),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(442),
    [sym_block_comment] = STATE(442),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(259),
    [anon_sym_forall] = ACTIONS(259),
    [anon_sym_choose] = ACTIONS(261),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(443)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(922),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(432),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1393),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1365),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(443),
    [sym_block_comment] = STATE(443),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(259),
    [anon_sym_forall] = ACTIONS(259),
    [anon_sym_choose] = ACTIONS(261),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(444)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(887),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(498),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1321),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1334),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(444),
    [sym_block_comment] = STATE(444),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(113),
    [anon_sym_forall] = ACTIONS(113),
    [anon_sym_choose] = ACTIONS(115),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(445)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2399),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(445),
    [sym_block_comment] = STATE(445),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(446)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(933),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(498),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1321),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1334),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(446),
    [sym_block_comment] = STATE(446),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(113),
    [anon_sym_forall] = ACTIONS(113),
    [anon_sym_choose] = ACTIONS(115),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(447)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(939),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(498),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1321),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1334),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(447),
    [sym_block_comment] = STATE(447),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(113),
    [anon_sym_forall] = ACTIONS(113),
    [anon_sym_choose] = ACTIONS(115),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(448)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2403),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(448),
    [sym_block_comment] = STATE(448),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(449)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(892),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(498),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1321),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1334),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(449),
    [sym_block_comment] = STATE(449),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(113),
    [anon_sym_forall] = ACTIONS(113),
    [anon_sym_choose] = ACTIONS(115),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(450)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2406),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(450),
    [sym_block_comment] = STATE(450),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(451)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(893),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(498),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1321),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1334),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(451),
    [sym_block_comment] = STATE(451),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(113),
    [anon_sym_forall] = ACTIONS(113),
    [anon_sym_choose] = ACTIONS(115),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(452)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(896),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(498),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1321),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1334),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(452),
    [sym_block_comment] = STATE(452),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(113),
    [anon_sym_forall] = ACTIONS(113),
    [anon_sym_choose] = ACTIONS(115),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(453)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(907),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(498),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1321),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1334),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(453),
    [sym_block_comment] = STATE(453),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(113),
    [anon_sym_forall] = ACTIONS(113),
    [anon_sym_choose] = ACTIONS(115),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(454)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2408),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(454),
    [sym_block_comment] = STATE(454),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(455)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(909),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(498),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1321),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1334),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(455),
    [sym_block_comment] = STATE(455),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(113),
    [anon_sym_forall] = ACTIONS(113),
    [anon_sym_choose] = ACTIONS(115),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(456)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(916),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(498),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1321),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1334),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(456),
    [sym_block_comment] = STATE(456),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(113),
    [anon_sym_forall] = ACTIONS(113),
    [anon_sym_choose] = ACTIONS(115),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(457)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(917),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(498),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1321),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1334),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(457),
    [sym_block_comment] = STATE(457),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(113),
    [anon_sym_forall] = ACTIONS(113),
    [anon_sym_choose] = ACTIONS(115),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(458)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(922),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(498),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1321),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1334),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(458),
    [sym_block_comment] = STATE(458),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(113),
    [anon_sym_forall] = ACTIONS(113),
    [anon_sym_choose] = ACTIONS(115),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(459)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2178),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(459),
    [sym_block_comment] = STATE(459),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(460)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2181),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(460),
    [sym_block_comment] = STATE(460),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(461)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2182),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(461),
    [sym_block_comment] = STATE(461),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(462)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2183),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(462),
    [sym_block_comment] = STATE(462),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(463)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2205),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(463),
    [sym_block_comment] = STATE(463),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(464)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2207),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(464),
    [sym_block_comment] = STATE(464),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(465)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2208),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(465),
    [sym_block_comment] = STATE(465),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(466)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2209),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(466),
    [sym_block_comment] = STATE(466),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(467)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2228),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(467),
    [sym_block_comment] = STATE(467),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(468)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2230),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(468),
    [sym_block_comment] = STATE(468),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(469)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2231),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(469),
    [sym_block_comment] = STATE(469),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(470)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2232),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(470),
    [sym_block_comment] = STATE(470),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(471)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2252),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(471),
    [sym_block_comment] = STATE(471),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(472)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2254),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(472),
    [sym_block_comment] = STATE(472),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(473)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2255),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(473),
    [sym_block_comment] = STATE(473),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(474)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2256),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(474),
    [sym_block_comment] = STATE(474),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(475)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(887),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(390),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1033),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1023),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(475),
    [sym_block_comment] = STATE(475),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(467),
    [anon_sym_forall] = ACTIONS(467),
    [anon_sym_choose] = ACTIONS(469),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(476)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(895),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(476),
    [sym_block_comment] = STATE(476),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(477)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(906),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(477),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1188),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1194),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(477),
    [sym_block_comment] = STATE(477),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(27),
    [anon_sym_forall] = ACTIONS(27),
    [anon_sym_choose] = ACTIONS(29),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(478)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(895),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(479),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1165),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1146),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(478),
    [sym_block_comment] = STATE(478),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(109),
    [anon_sym_forall] = ACTIONS(109),
    [anon_sym_choose] = ACTIONS(111),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(479)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(906),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(479),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1165),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1146),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(479),
    [sym_block_comment] = STATE(479),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(109),
    [anon_sym_forall] = ACTIONS(109),
    [anon_sym_choose] = ACTIONS(111),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(480)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(887),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(479),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1165),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1146),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(480),
    [sym_block_comment] = STATE(480),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(109),
    [anon_sym_forall] = ACTIONS(109),
    [anon_sym_choose] = ACTIONS(111),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(481)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2444),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(481),
    [sym_block_comment] = STATE(481),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(482)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(939),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(479),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1165),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1146),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(482),
    [sym_block_comment] = STATE(482),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(109),
    [anon_sym_forall] = ACTIONS(109),
    [anon_sym_choose] = ACTIONS(111),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(483)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(892),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(479),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1165),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1146),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(483),
    [sym_block_comment] = STATE(483),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(109),
    [anon_sym_forall] = ACTIONS(109),
    [anon_sym_choose] = ACTIONS(111),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(484)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(893),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(479),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1165),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1146),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(484),
    [sym_block_comment] = STATE(484),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(109),
    [anon_sym_forall] = ACTIONS(109),
    [anon_sym_choose] = ACTIONS(111),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(485)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(896),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(479),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1165),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1146),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(485),
    [sym_block_comment] = STATE(485),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(109),
    [anon_sym_forall] = ACTIONS(109),
    [anon_sym_choose] = ACTIONS(111),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(486)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(907),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(479),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1165),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1146),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(486),
    [sym_block_comment] = STATE(486),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(109),
    [anon_sym_forall] = ACTIONS(109),
    [anon_sym_choose] = ACTIONS(111),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(487)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(909),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(479),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1165),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1146),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(487),
    [sym_block_comment] = STATE(487),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(109),
    [anon_sym_forall] = ACTIONS(109),
    [anon_sym_choose] = ACTIONS(111),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(488)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(916),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(479),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1165),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1146),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(488),
    [sym_block_comment] = STATE(488),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(109),
    [anon_sym_forall] = ACTIONS(109),
    [anon_sym_choose] = ACTIONS(111),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(489)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(917),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(479),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1165),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1146),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(489),
    [sym_block_comment] = STATE(489),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(109),
    [anon_sym_forall] = ACTIONS(109),
    [anon_sym_choose] = ACTIONS(111),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(490)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(922),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(479),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1165),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1146),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(490),
    [sym_block_comment] = STATE(490),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(109),
    [anon_sym_forall] = ACTIONS(109),
    [anon_sym_choose] = ACTIONS(111),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(491)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(933),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(390),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1033),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1023),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(491),
    [sym_block_comment] = STATE(491),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(467),
    [anon_sym_forall] = ACTIONS(467),
    [anon_sym_choose] = ACTIONS(469),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(492)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2378),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(492),
    [sym_block_comment] = STATE(492),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(493)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2380),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(493),
    [sym_block_comment] = STATE(493),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(494)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2381),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(494),
    [sym_block_comment] = STATE(494),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(495)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2382),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(495),
    [sym_block_comment] = STATE(495),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(496)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(939),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(390),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1033),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1023),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(496),
    [sym_block_comment] = STATE(496),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(467),
    [anon_sym_forall] = ACTIONS(467),
    [anon_sym_choose] = ACTIONS(469),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(497)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(895),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(357),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1352),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1308),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(497),
    [sym_block_comment] = STATE(497),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(191),
    [anon_sym_forall] = ACTIONS(191),
    [anon_sym_choose] = ACTIONS(193),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(498)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(906),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(498),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1321),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1334),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(498),
    [sym_block_comment] = STATE(498),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(113),
    [anon_sym_forall] = ACTIONS(113),
    [anon_sym_choose] = ACTIONS(115),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(499)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(895),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(500),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1332),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1274),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(499),
    [sym_block_comment] = STATE(499),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(187),
    [anon_sym_forall] = ACTIONS(187),
    [anon_sym_choose] = ACTIONS(189),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(500)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(906),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(500),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1332),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1274),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(500),
    [sym_block_comment] = STATE(500),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(187),
    [anon_sym_forall] = ACTIONS(187),
    [anon_sym_choose] = ACTIONS(189),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(501)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(887),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(500),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1332),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1274),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(501),
    [sym_block_comment] = STATE(501),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(187),
    [anon_sym_forall] = ACTIONS(187),
    [anon_sym_choose] = ACTIONS(189),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(502)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(933),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(500),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1332),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1274),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(502),
    [sym_block_comment] = STATE(502),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(187),
    [anon_sym_forall] = ACTIONS(187),
    [anon_sym_choose] = ACTIONS(189),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(503)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(939),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(500),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1332),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1274),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(503),
    [sym_block_comment] = STATE(503),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(187),
    [anon_sym_forall] = ACTIONS(187),
    [anon_sym_choose] = ACTIONS(189),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(504)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(892),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(500),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1332),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1274),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(504),
    [sym_block_comment] = STATE(504),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(187),
    [anon_sym_forall] = ACTIONS(187),
    [anon_sym_choose] = ACTIONS(189),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(505)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(893),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(500),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1332),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1274),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(505),
    [sym_block_comment] = STATE(505),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(187),
    [anon_sym_forall] = ACTIONS(187),
    [anon_sym_choose] = ACTIONS(189),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(506)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(896),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(500),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1332),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1274),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(506),
    [sym_block_comment] = STATE(506),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(187),
    [anon_sym_forall] = ACTIONS(187),
    [anon_sym_choose] = ACTIONS(189),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(507)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(907),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(500),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1332),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1274),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(507),
    [sym_block_comment] = STATE(507),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(187),
    [anon_sym_forall] = ACTIONS(187),
    [anon_sym_choose] = ACTIONS(189),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(508)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(909),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(500),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1332),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1274),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(508),
    [sym_block_comment] = STATE(508),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(187),
    [anon_sym_forall] = ACTIONS(187),
    [anon_sym_choose] = ACTIONS(189),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(509)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(916),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(500),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1332),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1274),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(509),
    [sym_block_comment] = STATE(509),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(187),
    [anon_sym_forall] = ACTIONS(187),
    [anon_sym_choose] = ACTIONS(189),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(510)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(917),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(500),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1332),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1274),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(510),
    [sym_block_comment] = STATE(510),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(187),
    [anon_sym_forall] = ACTIONS(187),
    [anon_sym_choose] = ACTIONS(189),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(511)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(922),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(500),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1332),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1274),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(511),
    [sym_block_comment] = STATE(511),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(187),
    [anon_sym_forall] = ACTIONS(187),
    [anon_sym_choose] = ACTIONS(189),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(512)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(892),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(390),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1033),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1023),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(512),
    [sym_block_comment] = STATE(512),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(467),
    [anon_sym_forall] = ACTIONS(467),
    [anon_sym_choose] = ACTIONS(469),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(513)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2413),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(513),
    [sym_block_comment] = STATE(513),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(514)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2415),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(514),
    [sym_block_comment] = STATE(514),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(515)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2416),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(515),
    [sym_block_comment] = STATE(515),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(516)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2417),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(516),
    [sym_block_comment] = STATE(516),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(517)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2355),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(517),
    [sym_block_comment] = STATE(517),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(518)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2448),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(518),
    [sym_block_comment] = STATE(518),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(519)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2455),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(519),
    [sym_block_comment] = STATE(519),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(520)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2180),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(520),
    [sym_block_comment] = STATE(520),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(521)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2354),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(521),
    [sym_block_comment] = STATE(521),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(522)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2356),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(522),
    [sym_block_comment] = STATE(522),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(523)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2357),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(523),
    [sym_block_comment] = STATE(523),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(524)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2358),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(524),
    [sym_block_comment] = STATE(524),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(525)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2437),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(525),
    [sym_block_comment] = STATE(525),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(526)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2441),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(526),
    [sym_block_comment] = STATE(526),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(527)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__expr] = STATE(2442),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(275),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1216),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(1180),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(527),
    [sym_block_comment] = STATE(527),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(105),
    [anon_sym_forall] = ACTIONS(105),
    [anon_sym_choose] = ACTIONS(107),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(528)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__expr] = STATE(907),
    [sym_assignment] = STATE(918),
    [sym_lambda_bind_list] = STATE(310),
    [sym_quantifier] = STATE(918),
    [sym__op_expr] = STATE(1014),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(988),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(528),
    [sym_block_comment] = STATE(528),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(479),
    [anon_sym_forall] = ACTIONS(479),
    [anon_sym_choose] = ACTIONS(481),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_PIPE] = ACTIONS(269),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(529)] = {
    [sym_line_comment] = STATE(529),
    [sym_block_comment] = STATE(529),
    [sym_identifier] = ACTIONS(519),
    [anon_sym_u8] = ACTIONS(519),
    [anon_sym_u16] = ACTIONS(519),
    [anon_sym_u32] = ACTIONS(519),
    [anon_sym_u64] = ACTIONS(519),
    [anon_sym_u128] = ACTIONS(519),
    [anon_sym_u256] = ACTIONS(519),
    [anon_sym_exists] = ACTIONS(519),
    [anon_sym_forall] = ACTIONS(519),
    [anon_sym_choose] = ACTIONS(519),
    [anon_sym_min] = ACTIONS(519),
    [anon_sym_bool] = ACTIONS(519),
    [anon_sym_address] = ACTIONS(519),
    [anon_sym_vector] = ACTIONS(519),
    [anon_sym_for] = ACTIONS(519),
    [anon_sym_while] = ACTIONS(519),
    [anon_sym_friend] = ACTIONS(519),
    [anon_sym_match] = ACTIONS(519),
    [aux_sym_number_token1] = ACTIONS(519),
    [aux_sym_number_token2] = ACTIONS(521),
    [aux_sym_number_token4] = ACTIONS(521),
    [aux_sym_number_token6] = ACTIONS(521),
    [anon_sym_STAR] = ACTIONS(521),
    [anon_sym_LPAREN] = ACTIONS(521),
    [anon_sym_COMMA] = ACTIONS(521),
    [anon_sym_RPAREN] = ACTIONS(521),
    [anon_sym_PIPE] = ACTIONS(519),
    [anon_sym_AMP] = ACTIONS(519),
    [anon_sym_EQ] = ACTIONS(519),
    [anon_sym_where] = ACTIONS(519),
    [anon_sym_COLON] = ACTIONS(521),
    [anon_sym_LBRACE] = ACTIONS(521),
    [anon_sym_RBRACE] = ACTIONS(521),
    [anon_sym_EQ_EQ_GT] = ACTIONS(521),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(521),
    [anon_sym_PIPE_PIPE] = ACTIONS(521),
    [anon_sym_AMP_AMP] = ACTIONS(521),
    [anon_sym_EQ_EQ] = ACTIONS(519),
    [anon_sym_BANG_EQ] = ACTIONS(521),
    [anon_sym_LT] = ACTIONS(519),
    [anon_sym_GT] = ACTIONS(519),
    [anon_sym_LT_EQ] = ACTIONS(519),
    [anon_sym_GT_EQ] = ACTIONS(521),
    [anon_sym_DOT_DOT] = ACTIONS(521),
    [anon_sym_CARET] = ACTIONS(521),
    [anon_sym_LT_LT] = ACTIONS(521),
    [anon_sym_GT_GT] = ACTIONS(521),
    [anon_sym_PLUS] = ACTIONS(521),
    [anon_sym_DASH] = ACTIONS(521),
    [anon_sym_SLASH] = ACTIONS(519),
    [anon_sym_PERCENT] = ACTIONS(521),
    [anon_sym_DOT] = ACTIONS(519),
    [anon_sym_LBRACK] = ACTIONS(521),
    [anon_sym_RBRACK] = ACTIONS(521),
    [anon_sym_if] = ACTIONS(519),
    [anon_sym_EQ_GT] = ACTIONS(521),
    [anon_sym_else] = ACTIONS(519),
    [anon_sym_POUND] = ACTIONS(521),
    [anon_sym_spec] = ACTIONS(519),
    [anon_sym_invariant] = ACTIONS(519),
    [anon_sym_update] = ACTIONS(519),
    [anon_sym_SEMI] = ACTIONS(521),
    [anon_sym_assert] = ACTIONS(519),
    [anon_sym_assume] = ACTIONS(519),
    [anon_sym_ensures] = ACTIONS(519),
    [anon_sym_requires] = ACTIONS(519),
    [anon_sym_aborts_if] = ACTIONS(519),
    [anon_sym_with] = ACTIONS(519),
    [anon_sym_aborts_with] = ACTIONS(519),
    [anon_sym_modifies] = ACTIONS(519),
    [anon_sym_emits] = ACTIONS(519),
    [anon_sym_to] = ACTIONS(519),
    [anon_sym_global] = ACTIONS(519),
    [anon_sym_local] = ACTIONS(519),
    [anon_sym_fun] = ACTIONS(519),
    [anon_sym_native] = ACTIONS(519),
    [anon_sym_include] = ACTIONS(519),
    [anon_sym_apply] = ACTIONS(519),
    [anon_sym_public] = ACTIONS(519),
    [anon_sym_pragma] = ACTIONS(519),
    [anon_sym_let] = ACTIONS(519),
    [anon_sym_axiom] = ACTIONS(519),
    [anon_sym_entry] = ACTIONS(519),
    [anon_sym_use] = ACTIONS(519),
    [anon_sym_const] = ACTIONS(519),
    [anon_sym_inline] = ACTIONS(519),
    [anon_sym_struct] = ACTIONS(519),
    [anon_sym_enum] = ACTIONS(519),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(530)] = {
    [sym_line_comment] = STATE(530),
    [sym_block_comment] = STATE(530),
    [sym_identifier] = ACTIONS(523),
    [anon_sym_u8] = ACTIONS(523),
    [anon_sym_u16] = ACTIONS(523),
    [anon_sym_u32] = ACTIONS(523),
    [anon_sym_u64] = ACTIONS(523),
    [anon_sym_u128] = ACTIONS(523),
    [anon_sym_u256] = ACTIONS(523),
    [anon_sym_exists] = ACTIONS(523),
    [anon_sym_forall] = ACTIONS(523),
    [anon_sym_choose] = ACTIONS(523),
    [anon_sym_min] = ACTIONS(523),
    [anon_sym_bool] = ACTIONS(523),
    [anon_sym_address] = ACTIONS(523),
    [anon_sym_vector] = ACTIONS(523),
    [anon_sym_for] = ACTIONS(523),
    [anon_sym_while] = ACTIONS(523),
    [anon_sym_friend] = ACTIONS(523),
    [anon_sym_match] = ACTIONS(523),
    [aux_sym_number_token1] = ACTIONS(523),
    [aux_sym_number_token2] = ACTIONS(525),
    [aux_sym_number_token4] = ACTIONS(525),
    [aux_sym_number_token6] = ACTIONS(525),
    [anon_sym_STAR] = ACTIONS(525),
    [anon_sym_LPAREN] = ACTIONS(525),
    [anon_sym_COMMA] = ACTIONS(525),
    [anon_sym_RPAREN] = ACTIONS(525),
    [anon_sym_PIPE] = ACTIONS(523),
    [anon_sym_AMP] = ACTIONS(523),
    [anon_sym_EQ] = ACTIONS(523),
    [anon_sym_where] = ACTIONS(523),
    [anon_sym_COLON] = ACTIONS(525),
    [anon_sym_LBRACE] = ACTIONS(525),
    [anon_sym_RBRACE] = ACTIONS(525),
    [anon_sym_EQ_EQ_GT] = ACTIONS(525),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(525),
    [anon_sym_PIPE_PIPE] = ACTIONS(525),
    [anon_sym_AMP_AMP] = ACTIONS(525),
    [anon_sym_EQ_EQ] = ACTIONS(523),
    [anon_sym_BANG_EQ] = ACTIONS(525),
    [anon_sym_LT] = ACTIONS(523),
    [anon_sym_GT] = ACTIONS(523),
    [anon_sym_LT_EQ] = ACTIONS(523),
    [anon_sym_GT_EQ] = ACTIONS(525),
    [anon_sym_DOT_DOT] = ACTIONS(525),
    [anon_sym_CARET] = ACTIONS(525),
    [anon_sym_LT_LT] = ACTIONS(525),
    [anon_sym_GT_GT] = ACTIONS(525),
    [anon_sym_PLUS] = ACTIONS(525),
    [anon_sym_DASH] = ACTIONS(525),
    [anon_sym_SLASH] = ACTIONS(523),
    [anon_sym_PERCENT] = ACTIONS(525),
    [anon_sym_DOT] = ACTIONS(523),
    [anon_sym_LBRACK] = ACTIONS(525),
    [anon_sym_RBRACK] = ACTIONS(525),
    [anon_sym_if] = ACTIONS(523),
    [anon_sym_EQ_GT] = ACTIONS(525),
    [anon_sym_else] = ACTIONS(523),
    [anon_sym_POUND] = ACTIONS(525),
    [anon_sym_spec] = ACTIONS(523),
    [anon_sym_invariant] = ACTIONS(523),
    [anon_sym_update] = ACTIONS(523),
    [anon_sym_SEMI] = ACTIONS(525),
    [anon_sym_assert] = ACTIONS(523),
    [anon_sym_assume] = ACTIONS(523),
    [anon_sym_ensures] = ACTIONS(523),
    [anon_sym_requires] = ACTIONS(523),
    [anon_sym_aborts_if] = ACTIONS(523),
    [anon_sym_with] = ACTIONS(523),
    [anon_sym_aborts_with] = ACTIONS(523),
    [anon_sym_modifies] = ACTIONS(523),
    [anon_sym_emits] = ACTIONS(523),
    [anon_sym_to] = ACTIONS(523),
    [anon_sym_global] = ACTIONS(523),
    [anon_sym_local] = ACTIONS(523),
    [anon_sym_fun] = ACTIONS(523),
    [anon_sym_native] = ACTIONS(523),
    [anon_sym_include] = ACTIONS(523),
    [anon_sym_apply] = ACTIONS(523),
    [anon_sym_public] = ACTIONS(523),
    [anon_sym_pragma] = ACTIONS(523),
    [anon_sym_let] = ACTIONS(523),
    [anon_sym_axiom] = ACTIONS(523),
    [anon_sym_entry] = ACTIONS(523),
    [anon_sym_use] = ACTIONS(523),
    [anon_sym_const] = ACTIONS(523),
    [anon_sym_inline] = ACTIONS(523),
    [anon_sym_struct] = ACTIONS(523),
    [anon_sym_enum] = ACTIONS(523),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(531)] = {
    [sym_line_comment] = STATE(531),
    [sym_block_comment] = STATE(531),
    [sym_identifier] = ACTIONS(527),
    [anon_sym_u8] = ACTIONS(527),
    [anon_sym_u16] = ACTIONS(527),
    [anon_sym_u32] = ACTIONS(527),
    [anon_sym_u64] = ACTIONS(527),
    [anon_sym_u128] = ACTIONS(527),
    [anon_sym_u256] = ACTIONS(527),
    [anon_sym_exists] = ACTIONS(527),
    [anon_sym_forall] = ACTIONS(527),
    [anon_sym_choose] = ACTIONS(527),
    [anon_sym_min] = ACTIONS(527),
    [anon_sym_bool] = ACTIONS(527),
    [anon_sym_address] = ACTIONS(527),
    [anon_sym_vector] = ACTIONS(527),
    [anon_sym_for] = ACTIONS(527),
    [anon_sym_while] = ACTIONS(527),
    [anon_sym_friend] = ACTIONS(527),
    [anon_sym_match] = ACTIONS(527),
    [aux_sym_number_token1] = ACTIONS(527),
    [aux_sym_number_token2] = ACTIONS(529),
    [aux_sym_number_token4] = ACTIONS(529),
    [aux_sym_number_token6] = ACTIONS(529),
    [anon_sym_STAR] = ACTIONS(529),
    [anon_sym_LPAREN] = ACTIONS(529),
    [anon_sym_COMMA] = ACTIONS(529),
    [anon_sym_RPAREN] = ACTIONS(529),
    [anon_sym_PIPE] = ACTIONS(527),
    [anon_sym_AMP] = ACTIONS(527),
    [anon_sym_EQ] = ACTIONS(527),
    [anon_sym_where] = ACTIONS(527),
    [anon_sym_COLON] = ACTIONS(529),
    [anon_sym_LBRACE] = ACTIONS(529),
    [anon_sym_RBRACE] = ACTIONS(529),
    [anon_sym_EQ_EQ_GT] = ACTIONS(529),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(529),
    [anon_sym_PIPE_PIPE] = ACTIONS(529),
    [anon_sym_AMP_AMP] = ACTIONS(529),
    [anon_sym_EQ_EQ] = ACTIONS(527),
    [anon_sym_BANG_EQ] = ACTIONS(529),
    [anon_sym_LT] = ACTIONS(527),
    [anon_sym_GT] = ACTIONS(527),
    [anon_sym_LT_EQ] = ACTIONS(527),
    [anon_sym_GT_EQ] = ACTIONS(529),
    [anon_sym_DOT_DOT] = ACTIONS(529),
    [anon_sym_CARET] = ACTIONS(529),
    [anon_sym_LT_LT] = ACTIONS(529),
    [anon_sym_GT_GT] = ACTIONS(529),
    [anon_sym_PLUS] = ACTIONS(529),
    [anon_sym_DASH] = ACTIONS(529),
    [anon_sym_SLASH] = ACTIONS(527),
    [anon_sym_PERCENT] = ACTIONS(529),
    [anon_sym_DOT] = ACTIONS(527),
    [anon_sym_LBRACK] = ACTIONS(529),
    [anon_sym_RBRACK] = ACTIONS(529),
    [anon_sym_if] = ACTIONS(527),
    [anon_sym_EQ_GT] = ACTIONS(529),
    [anon_sym_else] = ACTIONS(527),
    [anon_sym_POUND] = ACTIONS(529),
    [anon_sym_spec] = ACTIONS(527),
    [anon_sym_invariant] = ACTIONS(527),
    [anon_sym_update] = ACTIONS(527),
    [anon_sym_SEMI] = ACTIONS(529),
    [anon_sym_assert] = ACTIONS(527),
    [anon_sym_assume] = ACTIONS(527),
    [anon_sym_ensures] = ACTIONS(527),
    [anon_sym_requires] = ACTIONS(527),
    [anon_sym_aborts_if] = ACTIONS(527),
    [anon_sym_with] = ACTIONS(527),
    [anon_sym_aborts_with] = ACTIONS(527),
    [anon_sym_modifies] = ACTIONS(527),
    [anon_sym_emits] = ACTIONS(527),
    [anon_sym_to] = ACTIONS(527),
    [anon_sym_global] = ACTIONS(527),
    [anon_sym_local] = ACTIONS(527),
    [anon_sym_fun] = ACTIONS(527),
    [anon_sym_native] = ACTIONS(527),
    [anon_sym_include] = ACTIONS(527),
    [anon_sym_apply] = ACTIONS(527),
    [anon_sym_public] = ACTIONS(527),
    [anon_sym_pragma] = ACTIONS(527),
    [anon_sym_let] = ACTIONS(527),
    [anon_sym_axiom] = ACTIONS(527),
    [anon_sym_entry] = ACTIONS(527),
    [anon_sym_use] = ACTIONS(527),
    [anon_sym_const] = ACTIONS(527),
    [anon_sym_inline] = ACTIONS(527),
    [anon_sym_struct] = ACTIONS(527),
    [anon_sym_enum] = ACTIONS(527),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(532)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1036),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(532),
    [sym_block_comment] = STATE(532),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(531),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(533),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(535),
    [anon_sym_AMPmut] = ACTIONS(537),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(539),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(541),
    [anon_sym_loop] = ACTIONS(543),
    [anon_sym_return] = ACTIONS(545),
    [anon_sym_abort] = ACTIONS(547),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(533)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1347),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(533),
    [sym_block_comment] = STATE(533),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(549),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(551),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(553),
    [anon_sym_AMPmut] = ACTIONS(555),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(557),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(559),
    [anon_sym_loop] = ACTIONS(561),
    [anon_sym_return] = ACTIONS(563),
    [anon_sym_abort] = ACTIONS(565),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(534)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1348),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(534),
    [sym_block_comment] = STATE(534),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(549),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(551),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(553),
    [anon_sym_AMPmut] = ACTIONS(555),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(557),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(559),
    [anon_sym_loop] = ACTIONS(561),
    [anon_sym_return] = ACTIONS(563),
    [anon_sym_abort] = ACTIONS(565),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(535)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1349),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(535),
    [sym_block_comment] = STATE(535),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(549),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(551),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(553),
    [anon_sym_AMPmut] = ACTIONS(555),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(557),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(559),
    [anon_sym_loop] = ACTIONS(561),
    [anon_sym_return] = ACTIONS(563),
    [anon_sym_abort] = ACTIONS(565),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(536)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1350),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(536),
    [sym_block_comment] = STATE(536),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(549),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(551),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(553),
    [anon_sym_AMPmut] = ACTIONS(555),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(557),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(559),
    [anon_sym_loop] = ACTIONS(561),
    [anon_sym_return] = ACTIONS(563),
    [anon_sym_abort] = ACTIONS(565),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(537)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1351),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(537),
    [sym_block_comment] = STATE(537),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(549),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(551),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(553),
    [anon_sym_AMPmut] = ACTIONS(555),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(557),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(559),
    [anon_sym_loop] = ACTIONS(561),
    [anon_sym_return] = ACTIONS(563),
    [anon_sym_abort] = ACTIONS(565),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(538)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1373),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(538),
    [sym_block_comment] = STATE(538),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(549),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(551),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(553),
    [anon_sym_AMPmut] = ACTIONS(555),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(557),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(559),
    [anon_sym_loop] = ACTIONS(561),
    [anon_sym_return] = ACTIONS(563),
    [anon_sym_abort] = ACTIONS(565),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(539)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1353),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(539),
    [sym_block_comment] = STATE(539),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(549),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(551),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(553),
    [anon_sym_AMPmut] = ACTIONS(555),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(557),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(559),
    [anon_sym_loop] = ACTIONS(561),
    [anon_sym_return] = ACTIONS(563),
    [anon_sym_abort] = ACTIONS(565),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(540)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1168),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(540),
    [sym_block_comment] = STATE(540),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(541)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1170),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(541),
    [sym_block_comment] = STATE(541),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(542)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1210),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(542),
    [sym_block_comment] = STATE(542),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(567),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(569),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(571),
    [anon_sym_AMPmut] = ACTIONS(573),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(575),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(577),
    [anon_sym_loop] = ACTIONS(579),
    [anon_sym_return] = ACTIONS(581),
    [anon_sym_abort] = ACTIONS(583),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(543)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1171),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(543),
    [sym_block_comment] = STATE(543),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(544)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(886),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(544),
    [sym_block_comment] = STATE(544),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(585),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(587),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(589),
    [anon_sym_AMPmut] = ACTIONS(591),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(593),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(595),
    [anon_sym_loop] = ACTIONS(597),
    [anon_sym_return] = ACTIONS(599),
    [anon_sym_abort] = ACTIONS(601),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(545)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1211),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(545),
    [sym_block_comment] = STATE(545),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(567),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(569),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(571),
    [anon_sym_AMPmut] = ACTIONS(573),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(575),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(577),
    [anon_sym_loop] = ACTIONS(579),
    [anon_sym_return] = ACTIONS(581),
    [anon_sym_abort] = ACTIONS(583),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(546)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(886),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(546),
    [sym_block_comment] = STATE(546),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(547)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1003),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(547),
    [sym_block_comment] = STATE(547),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(548)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1015),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(548),
    [sym_block_comment] = STATE(548),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(549)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1016),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(549),
    [sym_block_comment] = STATE(549),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(550)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1018),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(550),
    [sym_block_comment] = STATE(550),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(551)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1212),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(551),
    [sym_block_comment] = STATE(551),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(567),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(569),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(571),
    [anon_sym_AMPmut] = ACTIONS(573),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(575),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(577),
    [anon_sym_loop] = ACTIONS(579),
    [anon_sym_return] = ACTIONS(581),
    [anon_sym_abort] = ACTIONS(583),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(552)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1213),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(552),
    [sym_block_comment] = STATE(552),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(567),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(569),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(571),
    [anon_sym_AMPmut] = ACTIONS(573),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(575),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(577),
    [anon_sym_loop] = ACTIONS(579),
    [anon_sym_return] = ACTIONS(581),
    [anon_sym_abort] = ACTIONS(583),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(553)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1214),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(553),
    [sym_block_comment] = STATE(553),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(567),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(569),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(571),
    [anon_sym_AMPmut] = ACTIONS(573),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(575),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(577),
    [anon_sym_loop] = ACTIONS(579),
    [anon_sym_return] = ACTIONS(581),
    [anon_sym_abort] = ACTIONS(583),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(554)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1020),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(554),
    [sym_block_comment] = STATE(554),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(555)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(886),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(555),
    [sym_block_comment] = STATE(555),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(556)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1261),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(556),
    [sym_block_comment] = STATE(556),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(557)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1262),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(557),
    [sym_block_comment] = STATE(557),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(558)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1263),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(558),
    [sym_block_comment] = STATE(558),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(559)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1264),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(559),
    [sym_block_comment] = STATE(559),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(560)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1265),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(560),
    [sym_block_comment] = STATE(560),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(561)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1266),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(561),
    [sym_block_comment] = STATE(561),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(562)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1267),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(562),
    [sym_block_comment] = STATE(562),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(563)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1268),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(563),
    [sym_block_comment] = STATE(563),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(564)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1269),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(564),
    [sym_block_comment] = STATE(564),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(565)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1270),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(565),
    [sym_block_comment] = STATE(565),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(566)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1021),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(566),
    [sym_block_comment] = STATE(566),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(567)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1022),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(567),
    [sym_block_comment] = STATE(567),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(568)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(886),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(568),
    [sym_block_comment] = STATE(568),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(569)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(990),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(569),
    [sym_block_comment] = STATE(569),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(570)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(886),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(570),
    [sym_block_comment] = STATE(570),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(571)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1138),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(571),
    [sym_block_comment] = STATE(571),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(572)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1139),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(572),
    [sym_block_comment] = STATE(572),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(573)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1140),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(573),
    [sym_block_comment] = STATE(573),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(574)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1141),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(574),
    [sym_block_comment] = STATE(574),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(575)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1136),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(575),
    [sym_block_comment] = STATE(575),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(576)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1142),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(576),
    [sym_block_comment] = STATE(576),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(577)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1143),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(577),
    [sym_block_comment] = STATE(577),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(578)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1144),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(578),
    [sym_block_comment] = STATE(578),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(579)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1149),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(579),
    [sym_block_comment] = STATE(579),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(580)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1151),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(580),
    [sym_block_comment] = STATE(580),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(581)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(991),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(581),
    [sym_block_comment] = STATE(581),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(582)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(986),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(582),
    [sym_block_comment] = STATE(582),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(583)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(886),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(583),
    [sym_block_comment] = STATE(583),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(584)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1309),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(584),
    [sym_block_comment] = STATE(584),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(585)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1310),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(585),
    [sym_block_comment] = STATE(585),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(586)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1311),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(586),
    [sym_block_comment] = STATE(586),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(587)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1312),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(587),
    [sym_block_comment] = STATE(587),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(588)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1313),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(588),
    [sym_block_comment] = STATE(588),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(589)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1314),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(589),
    [sym_block_comment] = STATE(589),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(590)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1315),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(590),
    [sym_block_comment] = STATE(590),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(591)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1316),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(591),
    [sym_block_comment] = STATE(591),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(592)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1317),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(592),
    [sym_block_comment] = STATE(592),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(593)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1318),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(593),
    [sym_block_comment] = STATE(593),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(594)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(996),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(594),
    [sym_block_comment] = STATE(594),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(595)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(998),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(595),
    [sym_block_comment] = STATE(595),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(596)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(994),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(596),
    [sym_block_comment] = STATE(596),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(597)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(989),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(597),
    [sym_block_comment] = STATE(597),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(598)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(995),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(598),
    [sym_block_comment] = STATE(598),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(599)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(999),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(599),
    [sym_block_comment] = STATE(599),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(600)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(992),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(600),
    [sym_block_comment] = STATE(600),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(601)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(886),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(601),
    [sym_block_comment] = STATE(601),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(567),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(569),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(571),
    [anon_sym_AMPmut] = ACTIONS(573),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(575),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(577),
    [anon_sym_loop] = ACTIONS(579),
    [anon_sym_return] = ACTIONS(581),
    [anon_sym_abort] = ACTIONS(583),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(602)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1205),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(602),
    [sym_block_comment] = STATE(602),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(567),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(569),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(571),
    [anon_sym_AMPmut] = ACTIONS(573),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(575),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(577),
    [anon_sym_loop] = ACTIONS(579),
    [anon_sym_return] = ACTIONS(581),
    [anon_sym_abort] = ACTIONS(583),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(603)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1206),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(603),
    [sym_block_comment] = STATE(603),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(567),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(569),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(571),
    [anon_sym_AMPmut] = ACTIONS(573),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(575),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(577),
    [anon_sym_loop] = ACTIONS(579),
    [anon_sym_return] = ACTIONS(581),
    [anon_sym_abort] = ACTIONS(583),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(604)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1024),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(604),
    [sym_block_comment] = STATE(604),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(605)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1025),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(605),
    [sym_block_comment] = STATE(605),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(606)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1000),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(606),
    [sym_block_comment] = STATE(606),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(607)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(886),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(607),
    [sym_block_comment] = STATE(607),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(531),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(533),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(535),
    [anon_sym_AMPmut] = ACTIONS(537),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(539),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(541),
    [anon_sym_loop] = ACTIONS(543),
    [anon_sym_return] = ACTIONS(545),
    [anon_sym_abort] = ACTIONS(547),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(608)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1027),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(608),
    [sym_block_comment] = STATE(608),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(531),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(533),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(535),
    [anon_sym_AMPmut] = ACTIONS(537),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(539),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(541),
    [anon_sym_loop] = ACTIONS(543),
    [anon_sym_return] = ACTIONS(545),
    [anon_sym_abort] = ACTIONS(547),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(609)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1028),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(609),
    [sym_block_comment] = STATE(609),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(531),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(533),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(535),
    [anon_sym_AMPmut] = ACTIONS(537),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(539),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(541),
    [anon_sym_loop] = ACTIONS(543),
    [anon_sym_return] = ACTIONS(545),
    [anon_sym_abort] = ACTIONS(547),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(610)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1029),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(610),
    [sym_block_comment] = STATE(610),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(531),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(533),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(535),
    [anon_sym_AMPmut] = ACTIONS(537),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(539),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(541),
    [anon_sym_loop] = ACTIONS(543),
    [anon_sym_return] = ACTIONS(545),
    [anon_sym_abort] = ACTIONS(547),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(611)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1337),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(611),
    [sym_block_comment] = STATE(611),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(585),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(587),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(589),
    [anon_sym_AMPmut] = ACTIONS(591),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(593),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(595),
    [anon_sym_loop] = ACTIONS(597),
    [anon_sym_return] = ACTIONS(599),
    [anon_sym_abort] = ACTIONS(601),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(612)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1338),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(612),
    [sym_block_comment] = STATE(612),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(585),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(587),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(589),
    [anon_sym_AMPmut] = ACTIONS(591),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(593),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(595),
    [anon_sym_loop] = ACTIONS(597),
    [anon_sym_return] = ACTIONS(599),
    [anon_sym_abort] = ACTIONS(601),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(613)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1339),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(613),
    [sym_block_comment] = STATE(613),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(585),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(587),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(589),
    [anon_sym_AMPmut] = ACTIONS(591),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(593),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(595),
    [anon_sym_loop] = ACTIONS(597),
    [anon_sym_return] = ACTIONS(599),
    [anon_sym_abort] = ACTIONS(601),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(614)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(886),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(614),
    [sym_block_comment] = STATE(614),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(603),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(605),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(607),
    [anon_sym_AMPmut] = ACTIONS(609),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(611),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(613),
    [anon_sym_loop] = ACTIONS(615),
    [anon_sym_return] = ACTIONS(617),
    [anon_sym_abort] = ACTIONS(619),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(615)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1195),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(615),
    [sym_block_comment] = STATE(615),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(603),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(605),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(607),
    [anon_sym_AMPmut] = ACTIONS(609),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(611),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(613),
    [anon_sym_loop] = ACTIONS(615),
    [anon_sym_return] = ACTIONS(617),
    [anon_sym_abort] = ACTIONS(619),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(616)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1169),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(616),
    [sym_block_comment] = STATE(616),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(603),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(605),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(607),
    [anon_sym_AMPmut] = ACTIONS(609),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(611),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(613),
    [anon_sym_loop] = ACTIONS(615),
    [anon_sym_return] = ACTIONS(617),
    [anon_sym_abort] = ACTIONS(619),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(617)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1166),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(617),
    [sym_block_comment] = STATE(617),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(603),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(605),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(607),
    [anon_sym_AMPmut] = ACTIONS(609),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(611),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(613),
    [anon_sym_loop] = ACTIONS(615),
    [anon_sym_return] = ACTIONS(617),
    [anon_sym_abort] = ACTIONS(619),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(618)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1190),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(618),
    [sym_block_comment] = STATE(618),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(603),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(605),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(607),
    [anon_sym_AMPmut] = ACTIONS(609),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(611),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(613),
    [anon_sym_loop] = ACTIONS(615),
    [anon_sym_return] = ACTIONS(617),
    [anon_sym_abort] = ACTIONS(619),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(619)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1202),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(619),
    [sym_block_comment] = STATE(619),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(603),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(605),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(607),
    [anon_sym_AMPmut] = ACTIONS(609),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(611),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(613),
    [anon_sym_loop] = ACTIONS(615),
    [anon_sym_return] = ACTIONS(617),
    [anon_sym_abort] = ACTIONS(619),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(620)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1182),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(620),
    [sym_block_comment] = STATE(620),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(603),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(605),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(607),
    [anon_sym_AMPmut] = ACTIONS(609),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(611),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(613),
    [anon_sym_loop] = ACTIONS(615),
    [anon_sym_return] = ACTIONS(617),
    [anon_sym_abort] = ACTIONS(619),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(621)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1186),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(621),
    [sym_block_comment] = STATE(621),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(603),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(605),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(607),
    [anon_sym_AMPmut] = ACTIONS(609),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(611),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(613),
    [anon_sym_loop] = ACTIONS(615),
    [anon_sym_return] = ACTIONS(617),
    [anon_sym_abort] = ACTIONS(619),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(622)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1198),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(622),
    [sym_block_comment] = STATE(622),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(603),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(605),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(607),
    [anon_sym_AMPmut] = ACTIONS(609),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(611),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(613),
    [anon_sym_loop] = ACTIONS(615),
    [anon_sym_return] = ACTIONS(617),
    [anon_sym_abort] = ACTIONS(619),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(623)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1199),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(623),
    [sym_block_comment] = STATE(623),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(603),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(605),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(607),
    [anon_sym_AMPmut] = ACTIONS(609),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(611),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(613),
    [anon_sym_loop] = ACTIONS(615),
    [anon_sym_return] = ACTIONS(617),
    [anon_sym_abort] = ACTIONS(619),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(624)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1031),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(624),
    [sym_block_comment] = STATE(624),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(531),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(533),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(535),
    [anon_sym_AMPmut] = ACTIONS(537),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(539),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(541),
    [anon_sym_loop] = ACTIONS(543),
    [anon_sym_return] = ACTIONS(545),
    [anon_sym_abort] = ACTIONS(547),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(625)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1030),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(625),
    [sym_block_comment] = STATE(625),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(531),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(533),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(535),
    [anon_sym_AMPmut] = ACTIONS(537),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(539),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(541),
    [anon_sym_loop] = ACTIONS(543),
    [anon_sym_return] = ACTIONS(545),
    [anon_sym_abort] = ACTIONS(547),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(626)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1164),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(626),
    [sym_block_comment] = STATE(626),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(603),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(605),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(607),
    [anon_sym_AMPmut] = ACTIONS(609),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(611),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(613),
    [anon_sym_loop] = ACTIONS(615),
    [anon_sym_return] = ACTIONS(617),
    [anon_sym_abort] = ACTIONS(619),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(627)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1340),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(627),
    [sym_block_comment] = STATE(627),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(585),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(587),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(589),
    [anon_sym_AMPmut] = ACTIONS(591),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(593),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(595),
    [anon_sym_loop] = ACTIONS(597),
    [anon_sym_return] = ACTIONS(599),
    [anon_sym_abort] = ACTIONS(601),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(628)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(886),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(628),
    [sym_block_comment] = STATE(628),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(629)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1256),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(629),
    [sym_block_comment] = STATE(629),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(630)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1258),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(630),
    [sym_block_comment] = STATE(630),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(631)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1259),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(631),
    [sym_block_comment] = STATE(631),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(632)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1260),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(632),
    [sym_block_comment] = STATE(632),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(633)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1282),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(633),
    [sym_block_comment] = STATE(633),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(634)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1255),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(634),
    [sym_block_comment] = STATE(634),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(635)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1257),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(635),
    [sym_block_comment] = STATE(635),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(636)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1278),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(636),
    [sym_block_comment] = STATE(636),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(637)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1283),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(637),
    [sym_block_comment] = STATE(637),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(638)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1272),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(638),
    [sym_block_comment] = STATE(638),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(639)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1341),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(639),
    [sym_block_comment] = STATE(639),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(585),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(587),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(589),
    [anon_sym_AMPmut] = ACTIONS(591),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(593),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(595),
    [anon_sym_loop] = ACTIONS(597),
    [anon_sym_return] = ACTIONS(599),
    [anon_sym_abort] = ACTIONS(601),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(640)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1342),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(640),
    [sym_block_comment] = STATE(640),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(585),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(587),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(589),
    [anon_sym_AMPmut] = ACTIONS(591),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(593),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(595),
    [anon_sym_loop] = ACTIONS(597),
    [anon_sym_return] = ACTIONS(599),
    [anon_sym_abort] = ACTIONS(601),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(641)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1343),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(641),
    [sym_block_comment] = STATE(641),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(585),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(587),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(589),
    [anon_sym_AMPmut] = ACTIONS(591),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(593),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(595),
    [anon_sym_loop] = ACTIONS(597),
    [anon_sym_return] = ACTIONS(599),
    [anon_sym_abort] = ACTIONS(601),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(642)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(886),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(642),
    [sym_block_comment] = STATE(642),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(643)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1286),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(643),
    [sym_block_comment] = STATE(643),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(644)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1287),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(644),
    [sym_block_comment] = STATE(644),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(645)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1288),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(645),
    [sym_block_comment] = STATE(645),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(646)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1289),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(646),
    [sym_block_comment] = STATE(646),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(647)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1290),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(647),
    [sym_block_comment] = STATE(647),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(648)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1291),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(648),
    [sym_block_comment] = STATE(648),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(649)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1292),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(649),
    [sym_block_comment] = STATE(649),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(650)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1293),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(650),
    [sym_block_comment] = STATE(650),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(651)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1294),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(651),
    [sym_block_comment] = STATE(651),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(652)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(886),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(652),
    [sym_block_comment] = STATE(652),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(549),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(551),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(553),
    [anon_sym_AMPmut] = ACTIONS(555),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(557),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(559),
    [anon_sym_loop] = ACTIONS(561),
    [anon_sym_return] = ACTIONS(563),
    [anon_sym_abort] = ACTIONS(565),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(653)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1037),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(653),
    [sym_block_comment] = STATE(653),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(531),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(533),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(535),
    [anon_sym_AMPmut] = ACTIONS(537),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(539),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(541),
    [anon_sym_loop] = ACTIONS(543),
    [anon_sym_return] = ACTIONS(545),
    [anon_sym_abort] = ACTIONS(547),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(654)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1040),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(654),
    [sym_block_comment] = STATE(654),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(531),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(533),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(535),
    [anon_sym_AMPmut] = ACTIONS(537),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(539),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(541),
    [anon_sym_loop] = ACTIONS(543),
    [anon_sym_return] = ACTIONS(545),
    [anon_sym_abort] = ACTIONS(547),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(655)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1041),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(655),
    [sym_block_comment] = STATE(655),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(531),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(533),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(535),
    [anon_sym_AMPmut] = ACTIONS(537),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(539),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(541),
    [anon_sym_loop] = ACTIONS(543),
    [anon_sym_return] = ACTIONS(545),
    [anon_sym_abort] = ACTIONS(547),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(656)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1295),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(656),
    [sym_block_comment] = STATE(656),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(657)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1344),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(657),
    [sym_block_comment] = STATE(657),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(585),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(587),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(589),
    [anon_sym_AMPmut] = ACTIONS(591),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(593),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(595),
    [anon_sym_loop] = ACTIONS(597),
    [anon_sym_return] = ACTIONS(599),
    [anon_sym_abort] = ACTIONS(601),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(658)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1354),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(658),
    [sym_block_comment] = STATE(658),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(585),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(587),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(589),
    [anon_sym_AMPmut] = ACTIONS(591),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(593),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(595),
    [anon_sym_loop] = ACTIONS(597),
    [anon_sym_return] = ACTIONS(599),
    [anon_sym_abort] = ACTIONS(601),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(659)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1355),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(659),
    [sym_block_comment] = STATE(659),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(585),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(587),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(589),
    [anon_sym_AMPmut] = ACTIONS(591),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(593),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(595),
    [anon_sym_loop] = ACTIONS(597),
    [anon_sym_return] = ACTIONS(599),
    [anon_sym_abort] = ACTIONS(601),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(660)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1163),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(660),
    [sym_block_comment] = STATE(660),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(661)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1189),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(661),
    [sym_block_comment] = STATE(661),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(662)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1172),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(662),
    [sym_block_comment] = STATE(662),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(663)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1191),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(663),
    [sym_block_comment] = STATE(663),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(664)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1345),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(664),
    [sym_block_comment] = STATE(664),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(549),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(551),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(553),
    [anon_sym_AMPmut] = ACTIONS(555),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(557),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(559),
    [anon_sym_loop] = ACTIONS(561),
    [anon_sym_return] = ACTIONS(563),
    [anon_sym_abort] = ACTIONS(565),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(665)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1192),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(665),
    [sym_block_comment] = STATE(665),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(666)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1207),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(666),
    [sym_block_comment] = STATE(666),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(567),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(569),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(571),
    [anon_sym_AMPmut] = ACTIONS(573),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(575),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(577),
    [anon_sym_loop] = ACTIONS(579),
    [anon_sym_return] = ACTIONS(581),
    [anon_sym_abort] = ACTIONS(583),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(667)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(886),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(667),
    [sym_block_comment] = STATE(667),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(621),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(623),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(625),
    [anon_sym_AMPmut] = ACTIONS(627),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(629),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(631),
    [anon_sym_loop] = ACTIONS(633),
    [anon_sym_return] = ACTIONS(635),
    [anon_sym_abort] = ACTIONS(637),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(668)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1004),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(668),
    [sym_block_comment] = STATE(668),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(621),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(623),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(625),
    [anon_sym_AMPmut] = ACTIONS(627),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(629),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(631),
    [anon_sym_loop] = ACTIONS(633),
    [anon_sym_return] = ACTIONS(635),
    [anon_sym_abort] = ACTIONS(637),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(669)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1005),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(669),
    [sym_block_comment] = STATE(669),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(621),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(623),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(625),
    [anon_sym_AMPmut] = ACTIONS(627),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(629),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(631),
    [anon_sym_loop] = ACTIONS(633),
    [anon_sym_return] = ACTIONS(635),
    [anon_sym_abort] = ACTIONS(637),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(670)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(886),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(670),
    [sym_block_comment] = STATE(670),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(671)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(886),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(671),
    [sym_block_comment] = STATE(671),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(639),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(641),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(643),
    [anon_sym_AMPmut] = ACTIONS(645),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(647),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(649),
    [anon_sym_loop] = ACTIONS(651),
    [anon_sym_return] = ACTIONS(653),
    [anon_sym_abort] = ACTIONS(655),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(672)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1322),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(672),
    [sym_block_comment] = STATE(672),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(639),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(641),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(643),
    [anon_sym_AMPmut] = ACTIONS(645),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(647),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(649),
    [anon_sym_loop] = ACTIONS(651),
    [anon_sym_return] = ACTIONS(653),
    [anon_sym_abort] = ACTIONS(655),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(673)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1323),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(673),
    [sym_block_comment] = STATE(673),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(639),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(641),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(643),
    [anon_sym_AMPmut] = ACTIONS(645),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(647),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(649),
    [anon_sym_loop] = ACTIONS(651),
    [anon_sym_return] = ACTIONS(653),
    [anon_sym_abort] = ACTIONS(655),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(674)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1324),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(674),
    [sym_block_comment] = STATE(674),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(639),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(641),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(643),
    [anon_sym_AMPmut] = ACTIONS(645),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(647),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(649),
    [anon_sym_loop] = ACTIONS(651),
    [anon_sym_return] = ACTIONS(653),
    [anon_sym_abort] = ACTIONS(655),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(675)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1325),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(675),
    [sym_block_comment] = STATE(675),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(639),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(641),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(643),
    [anon_sym_AMPmut] = ACTIONS(645),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(647),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(649),
    [anon_sym_loop] = ACTIONS(651),
    [anon_sym_return] = ACTIONS(653),
    [anon_sym_abort] = ACTIONS(655),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(676)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1326),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(676),
    [sym_block_comment] = STATE(676),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(639),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(641),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(643),
    [anon_sym_AMPmut] = ACTIONS(645),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(647),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(649),
    [anon_sym_loop] = ACTIONS(651),
    [anon_sym_return] = ACTIONS(653),
    [anon_sym_abort] = ACTIONS(655),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(677)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1327),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(677),
    [sym_block_comment] = STATE(677),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(639),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(641),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(643),
    [anon_sym_AMPmut] = ACTIONS(645),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(647),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(649),
    [anon_sym_loop] = ACTIONS(651),
    [anon_sym_return] = ACTIONS(653),
    [anon_sym_abort] = ACTIONS(655),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(678)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1328),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(678),
    [sym_block_comment] = STATE(678),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(639),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(641),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(643),
    [anon_sym_AMPmut] = ACTIONS(645),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(647),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(649),
    [anon_sym_loop] = ACTIONS(651),
    [anon_sym_return] = ACTIONS(653),
    [anon_sym_abort] = ACTIONS(655),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(679)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1329),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(679),
    [sym_block_comment] = STATE(679),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(639),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(641),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(643),
    [anon_sym_AMPmut] = ACTIONS(645),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(647),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(649),
    [anon_sym_loop] = ACTIONS(651),
    [anon_sym_return] = ACTIONS(653),
    [anon_sym_abort] = ACTIONS(655),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(680)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1330),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(680),
    [sym_block_comment] = STATE(680),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(639),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(641),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(643),
    [anon_sym_AMPmut] = ACTIONS(645),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(647),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(649),
    [anon_sym_loop] = ACTIONS(651),
    [anon_sym_return] = ACTIONS(653),
    [anon_sym_abort] = ACTIONS(655),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(681)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1331),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(681),
    [sym_block_comment] = STATE(681),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(639),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(641),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(643),
    [anon_sym_AMPmut] = ACTIONS(645),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(647),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(649),
    [anon_sym_loop] = ACTIONS(651),
    [anon_sym_return] = ACTIONS(653),
    [anon_sym_abort] = ACTIONS(655),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(682)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1384),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(682),
    [sym_block_comment] = STATE(682),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(683)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1385),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(683),
    [sym_block_comment] = STATE(683),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(684)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1386),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(684),
    [sym_block_comment] = STATE(684),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(685)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1387),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(685),
    [sym_block_comment] = STATE(685),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(686)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1039),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(686),
    [sym_block_comment] = STATE(686),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(531),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(533),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(535),
    [anon_sym_AMPmut] = ACTIONS(537),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(539),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(541),
    [anon_sym_loop] = ACTIONS(543),
    [anon_sym_return] = ACTIONS(545),
    [anon_sym_abort] = ACTIONS(547),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(687)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1388),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(687),
    [sym_block_comment] = STATE(687),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(688)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1389),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(688),
    [sym_block_comment] = STATE(688),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(689)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1390),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(689),
    [sym_block_comment] = STATE(689),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(690)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1391),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(690),
    [sym_block_comment] = STATE(690),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(691)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1394),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(691),
    [sym_block_comment] = STATE(691),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(692)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1395),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(692),
    [sym_block_comment] = STATE(692),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(693)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(886),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(693),
    [sym_block_comment] = STATE(693),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(657),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(659),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(661),
    [anon_sym_AMPmut] = ACTIONS(663),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(665),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(667),
    [anon_sym_loop] = ACTIONS(669),
    [anon_sym_return] = ACTIONS(671),
    [anon_sym_abort] = ACTIONS(673),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(694)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1297),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(694),
    [sym_block_comment] = STATE(694),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(657),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(659),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(661),
    [anon_sym_AMPmut] = ACTIONS(663),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(665),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(667),
    [anon_sym_loop] = ACTIONS(669),
    [anon_sym_return] = ACTIONS(671),
    [anon_sym_abort] = ACTIONS(673),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(695)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1298),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(695),
    [sym_block_comment] = STATE(695),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(657),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(659),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(661),
    [anon_sym_AMPmut] = ACTIONS(663),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(665),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(667),
    [anon_sym_loop] = ACTIONS(669),
    [anon_sym_return] = ACTIONS(671),
    [anon_sym_abort] = ACTIONS(673),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(696)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1299),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(696),
    [sym_block_comment] = STATE(696),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(657),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(659),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(661),
    [anon_sym_AMPmut] = ACTIONS(663),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(665),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(667),
    [anon_sym_loop] = ACTIONS(669),
    [anon_sym_return] = ACTIONS(671),
    [anon_sym_abort] = ACTIONS(673),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(697)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1284),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(697),
    [sym_block_comment] = STATE(697),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(657),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(659),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(661),
    [anon_sym_AMPmut] = ACTIONS(663),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(665),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(667),
    [anon_sym_loop] = ACTIONS(669),
    [anon_sym_return] = ACTIONS(671),
    [anon_sym_abort] = ACTIONS(673),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(698)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1301),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(698),
    [sym_block_comment] = STATE(698),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(657),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(659),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(661),
    [anon_sym_AMPmut] = ACTIONS(663),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(665),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(667),
    [anon_sym_loop] = ACTIONS(669),
    [anon_sym_return] = ACTIONS(671),
    [anon_sym_abort] = ACTIONS(673),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(699)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1302),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(699),
    [sym_block_comment] = STATE(699),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(657),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(659),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(661),
    [anon_sym_AMPmut] = ACTIONS(663),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(665),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(667),
    [anon_sym_loop] = ACTIONS(669),
    [anon_sym_return] = ACTIONS(671),
    [anon_sym_abort] = ACTIONS(673),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(700)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1303),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(700),
    [sym_block_comment] = STATE(700),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(657),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(659),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(661),
    [anon_sym_AMPmut] = ACTIONS(663),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(665),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(667),
    [anon_sym_loop] = ACTIONS(669),
    [anon_sym_return] = ACTIONS(671),
    [anon_sym_abort] = ACTIONS(673),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(701)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1304),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(701),
    [sym_block_comment] = STATE(701),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(657),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(659),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(661),
    [anon_sym_AMPmut] = ACTIONS(663),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(665),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(667),
    [anon_sym_loop] = ACTIONS(669),
    [anon_sym_return] = ACTIONS(671),
    [anon_sym_abort] = ACTIONS(673),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(702)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1305),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(702),
    [sym_block_comment] = STATE(702),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(657),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(659),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(661),
    [anon_sym_AMPmut] = ACTIONS(663),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(665),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(667),
    [anon_sym_loop] = ACTIONS(669),
    [anon_sym_return] = ACTIONS(671),
    [anon_sym_abort] = ACTIONS(673),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(703)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__op_expr] = STATE(1306),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(703),
    [sym_block_comment] = STATE(703),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(657),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(659),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(661),
    [anon_sym_AMPmut] = ACTIONS(663),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(665),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(667),
    [anon_sym_loop] = ACTIONS(669),
    [anon_sym_return] = ACTIONS(671),
    [anon_sym_abort] = ACTIONS(673),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(704)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(886),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(704),
    [sym_block_comment] = STATE(704),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(675),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(677),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(679),
    [anon_sym_AMPmut] = ACTIONS(681),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(683),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(685),
    [anon_sym_loop] = ACTIONS(687),
    [anon_sym_return] = ACTIONS(689),
    [anon_sym_abort] = ACTIONS(691),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(705)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1404),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(705),
    [sym_block_comment] = STATE(705),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(675),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(677),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(679),
    [anon_sym_AMPmut] = ACTIONS(681),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(683),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(685),
    [anon_sym_loop] = ACTIONS(687),
    [anon_sym_return] = ACTIONS(689),
    [anon_sym_abort] = ACTIONS(691),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(706)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1402),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(706),
    [sym_block_comment] = STATE(706),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(675),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(677),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(679),
    [anon_sym_AMPmut] = ACTIONS(681),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(683),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(685),
    [anon_sym_loop] = ACTIONS(687),
    [anon_sym_return] = ACTIONS(689),
    [anon_sym_abort] = ACTIONS(691),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(707)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1403),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(707),
    [sym_block_comment] = STATE(707),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(675),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(677),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(679),
    [anon_sym_AMPmut] = ACTIONS(681),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(683),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(685),
    [anon_sym_loop] = ACTIONS(687),
    [anon_sym_return] = ACTIONS(689),
    [anon_sym_abort] = ACTIONS(691),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(708)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1374),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(708),
    [sym_block_comment] = STATE(708),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(675),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(677),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(679),
    [anon_sym_AMPmut] = ACTIONS(681),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(683),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(685),
    [anon_sym_loop] = ACTIONS(687),
    [anon_sym_return] = ACTIONS(689),
    [anon_sym_abort] = ACTIONS(691),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(709)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1377),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(709),
    [sym_block_comment] = STATE(709),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(675),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(677),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(679),
    [anon_sym_AMPmut] = ACTIONS(681),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(683),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(685),
    [anon_sym_loop] = ACTIONS(687),
    [anon_sym_return] = ACTIONS(689),
    [anon_sym_abort] = ACTIONS(691),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(710)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1378),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(710),
    [sym_block_comment] = STATE(710),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(675),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(677),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(679),
    [anon_sym_AMPmut] = ACTIONS(681),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(683),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(685),
    [anon_sym_loop] = ACTIONS(687),
    [anon_sym_return] = ACTIONS(689),
    [anon_sym_abort] = ACTIONS(691),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(711)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1379),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(711),
    [sym_block_comment] = STATE(711),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(675),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(677),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(679),
    [anon_sym_AMPmut] = ACTIONS(681),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(683),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(685),
    [anon_sym_loop] = ACTIONS(687),
    [anon_sym_return] = ACTIONS(689),
    [anon_sym_abort] = ACTIONS(691),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(712)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1381),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(712),
    [sym_block_comment] = STATE(712),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(675),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(677),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(679),
    [anon_sym_AMPmut] = ACTIONS(681),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(683),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(685),
    [anon_sym_loop] = ACTIONS(687),
    [anon_sym_return] = ACTIONS(689),
    [anon_sym_abort] = ACTIONS(691),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(713)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1382),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(713),
    [sym_block_comment] = STATE(713),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(675),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(677),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(679),
    [anon_sym_AMPmut] = ACTIONS(681),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(683),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(685),
    [anon_sym_loop] = ACTIONS(687),
    [anon_sym_return] = ACTIONS(689),
    [anon_sym_abort] = ACTIONS(691),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(714)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1383),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(714),
    [sym_block_comment] = STATE(714),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(675),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(677),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(679),
    [anon_sym_AMPmut] = ACTIONS(681),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(683),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(685),
    [anon_sym_loop] = ACTIONS(687),
    [anon_sym_return] = ACTIONS(689),
    [anon_sym_abort] = ACTIONS(691),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(715)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1006),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(715),
    [sym_block_comment] = STATE(715),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(621),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(623),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(625),
    [anon_sym_AMPmut] = ACTIONS(627),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(629),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(631),
    [anon_sym_loop] = ACTIONS(633),
    [anon_sym_return] = ACTIONS(635),
    [anon_sym_abort] = ACTIONS(637),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(716)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1007),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(716),
    [sym_block_comment] = STATE(716),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(621),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(623),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(625),
    [anon_sym_AMPmut] = ACTIONS(627),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(629),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(631),
    [anon_sym_loop] = ACTIONS(633),
    [anon_sym_return] = ACTIONS(635),
    [anon_sym_abort] = ACTIONS(637),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(717)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1008),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(717),
    [sym_block_comment] = STATE(717),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(621),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(623),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(625),
    [anon_sym_AMPmut] = ACTIONS(627),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(629),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(631),
    [anon_sym_loop] = ACTIONS(633),
    [anon_sym_return] = ACTIONS(635),
    [anon_sym_abort] = ACTIONS(637),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(718)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1009),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(718),
    [sym_block_comment] = STATE(718),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(621),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(623),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(625),
    [anon_sym_AMPmut] = ACTIONS(627),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(629),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(631),
    [anon_sym_loop] = ACTIONS(633),
    [anon_sym_return] = ACTIONS(635),
    [anon_sym_abort] = ACTIONS(637),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(719)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1010),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(719),
    [sym_block_comment] = STATE(719),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(621),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(623),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(625),
    [anon_sym_AMPmut] = ACTIONS(627),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(629),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(631),
    [anon_sym_loop] = ACTIONS(633),
    [anon_sym_return] = ACTIONS(635),
    [anon_sym_abort] = ACTIONS(637),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(720)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1011),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(720),
    [sym_block_comment] = STATE(720),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(621),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(623),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(625),
    [anon_sym_AMPmut] = ACTIONS(627),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(629),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(631),
    [anon_sym_loop] = ACTIONS(633),
    [anon_sym_return] = ACTIONS(635),
    [anon_sym_abort] = ACTIONS(637),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(721)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1012),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(721),
    [sym_block_comment] = STATE(721),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(621),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(623),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(625),
    [anon_sym_AMPmut] = ACTIONS(627),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(629),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(631),
    [anon_sym_loop] = ACTIONS(633),
    [anon_sym_return] = ACTIONS(635),
    [anon_sym_abort] = ACTIONS(637),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(722)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__op_expr] = STATE(1013),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(722),
    [sym_block_comment] = STATE(722),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(621),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(623),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(625),
    [anon_sym_AMPmut] = ACTIONS(627),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(629),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(631),
    [anon_sym_loop] = ACTIONS(633),
    [anon_sym_return] = ACTIONS(635),
    [anon_sym_abort] = ACTIONS(637),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(723)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1179),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(723),
    [sym_block_comment] = STATE(723),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(724)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1173),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(724),
    [sym_block_comment] = STATE(724),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(725)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1208),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(725),
    [sym_block_comment] = STATE(725),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(567),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(569),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(571),
    [anon_sym_AMPmut] = ACTIONS(573),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(575),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(577),
    [anon_sym_loop] = ACTIONS(579),
    [anon_sym_return] = ACTIONS(581),
    [anon_sym_abort] = ACTIONS(583),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(726)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1336),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(726),
    [sym_block_comment] = STATE(726),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(549),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(551),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(553),
    [anon_sym_AMPmut] = ACTIONS(555),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(557),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(559),
    [anon_sym_loop] = ACTIONS(561),
    [anon_sym_return] = ACTIONS(563),
    [anon_sym_abort] = ACTIONS(565),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(727)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1209),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(727),
    [sym_block_comment] = STATE(727),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(567),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(569),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(571),
    [anon_sym_AMPmut] = ACTIONS(573),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(575),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(577),
    [anon_sym_loop] = ACTIONS(579),
    [anon_sym_return] = ACTIONS(581),
    [anon_sym_abort] = ACTIONS(583),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(728)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(886),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(728),
    [sym_block_comment] = STATE(728),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(729)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1357),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(729),
    [sym_block_comment] = STATE(729),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(730)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1358),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(730),
    [sym_block_comment] = STATE(730),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(731)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1359),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(731),
    [sym_block_comment] = STATE(731),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(732)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1360),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(732),
    [sym_block_comment] = STATE(732),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(733)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1361),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(733),
    [sym_block_comment] = STATE(733),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(734)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1362),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(734),
    [sym_block_comment] = STATE(734),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(735)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1363),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(735),
    [sym_block_comment] = STATE(735),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(736)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1364),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(736),
    [sym_block_comment] = STATE(736),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(737)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1366),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(737),
    [sym_block_comment] = STATE(737),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(738)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1367),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(738),
    [sym_block_comment] = STATE(738),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(739)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1346),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(739),
    [sym_block_comment] = STATE(739),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(549),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(551),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(553),
    [anon_sym_AMPmut] = ACTIONS(555),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(557),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(559),
    [anon_sym_loop] = ACTIONS(561),
    [anon_sym_return] = ACTIONS(563),
    [anon_sym_abort] = ACTIONS(565),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(740)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(886),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(740),
    [sym_block_comment] = STATE(740),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(693),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(695),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(697),
    [anon_sym_AMPmut] = ACTIONS(699),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(701),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(703),
    [anon_sym_loop] = ACTIONS(705),
    [anon_sym_return] = ACTIONS(707),
    [anon_sym_abort] = ACTIONS(709),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(741)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1419),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(741),
    [sym_block_comment] = STATE(741),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(693),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(695),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(697),
    [anon_sym_AMPmut] = ACTIONS(699),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(701),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(703),
    [anon_sym_loop] = ACTIONS(705),
    [anon_sym_return] = ACTIONS(707),
    [anon_sym_abort] = ACTIONS(709),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(742)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1406),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(742),
    [sym_block_comment] = STATE(742),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(693),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(695),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(697),
    [anon_sym_AMPmut] = ACTIONS(699),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(701),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(703),
    [anon_sym_loop] = ACTIONS(705),
    [anon_sym_return] = ACTIONS(707),
    [anon_sym_abort] = ACTIONS(709),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(743)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1407),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(743),
    [sym_block_comment] = STATE(743),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(693),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(695),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(697),
    [anon_sym_AMPmut] = ACTIONS(699),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(701),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(703),
    [anon_sym_loop] = ACTIONS(705),
    [anon_sym_return] = ACTIONS(707),
    [anon_sym_abort] = ACTIONS(709),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(744)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1412),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(744),
    [sym_block_comment] = STATE(744),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(693),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(695),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(697),
    [anon_sym_AMPmut] = ACTIONS(699),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(701),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(703),
    [anon_sym_loop] = ACTIONS(705),
    [anon_sym_return] = ACTIONS(707),
    [anon_sym_abort] = ACTIONS(709),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(745)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1414),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(745),
    [sym_block_comment] = STATE(745),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(693),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(695),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(697),
    [anon_sym_AMPmut] = ACTIONS(699),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(701),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(703),
    [anon_sym_loop] = ACTIONS(705),
    [anon_sym_return] = ACTIONS(707),
    [anon_sym_abort] = ACTIONS(709),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(746)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1420),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(746),
    [sym_block_comment] = STATE(746),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(693),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(695),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(697),
    [anon_sym_AMPmut] = ACTIONS(699),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(701),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(703),
    [anon_sym_loop] = ACTIONS(705),
    [anon_sym_return] = ACTIONS(707),
    [anon_sym_abort] = ACTIONS(709),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(747)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1421),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(747),
    [sym_block_comment] = STATE(747),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(693),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(695),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(697),
    [anon_sym_AMPmut] = ACTIONS(699),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(701),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(703),
    [anon_sym_loop] = ACTIONS(705),
    [anon_sym_return] = ACTIONS(707),
    [anon_sym_abort] = ACTIONS(709),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(748)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1408),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(748),
    [sym_block_comment] = STATE(748),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(693),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(695),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(697),
    [anon_sym_AMPmut] = ACTIONS(699),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(701),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(703),
    [anon_sym_loop] = ACTIONS(705),
    [anon_sym_return] = ACTIONS(707),
    [anon_sym_abort] = ACTIONS(709),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(749)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1411),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(749),
    [sym_block_comment] = STATE(749),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(693),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(695),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(697),
    [anon_sym_AMPmut] = ACTIONS(699),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(701),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(703),
    [anon_sym_loop] = ACTIONS(705),
    [anon_sym_return] = ACTIONS(707),
    [anon_sym_abort] = ACTIONS(709),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(750)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(1416),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(750),
    [sym_block_comment] = STATE(750),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(693),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(695),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(697),
    [anon_sym_AMPmut] = ACTIONS(699),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(701),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(703),
    [anon_sym_loop] = ACTIONS(705),
    [anon_sym_return] = ACTIONS(707),
    [anon_sym_abort] = ACTIONS(709),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(751)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__op_expr] = STATE(886),
    [sym_bin_op_expr] = STATE(920),
    [sym__unary_expr] = STATE(920),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(751),
    [sym_block_comment] = STATE(751),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(752)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1534),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym_assignment] = STATE(2235),
    [sym__unary_expr] = STATE(2322),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(752),
    [sym_block_comment] = STATE(752),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(711),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(713),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(715),
    [anon_sym_AMPmut] = ACTIONS(717),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(719),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(721),
    [anon_sym_loop] = ACTIONS(723),
    [anon_sym_return] = ACTIONS(725),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(753)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__unary_expr] = STATE(950),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(753),
    [sym_block_comment] = STATE(753),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(657),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(659),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(661),
    [anon_sym_AMPmut] = ACTIONS(663),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(665),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(667),
    [anon_sym_loop] = ACTIONS(669),
    [anon_sym_return] = ACTIONS(671),
    [anon_sym_abort] = ACTIONS(673),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(754)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__unary_expr] = STATE(935),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(754),
    [sym_block_comment] = STATE(754),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(621),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(623),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(625),
    [anon_sym_AMPmut] = ACTIONS(627),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(629),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(631),
    [anon_sym_loop] = ACTIONS(633),
    [anon_sym_return] = ACTIONS(635),
    [anon_sym_abort] = ACTIONS(637),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(755)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__unary_expr] = STATE(941),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(755),
    [sym_block_comment] = STATE(755),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(621),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(623),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(625),
    [anon_sym_AMPmut] = ACTIONS(627),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(629),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(631),
    [anon_sym_loop] = ACTIONS(633),
    [anon_sym_return] = ACTIONS(635),
    [anon_sym_abort] = ACTIONS(637),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(756)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__unary_expr] = STATE(947),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(756),
    [sym_block_comment] = STATE(756),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(621),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(623),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(625),
    [anon_sym_AMPmut] = ACTIONS(627),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(629),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(631),
    [anon_sym_loop] = ACTIONS(633),
    [anon_sym_return] = ACTIONS(635),
    [anon_sym_abort] = ACTIONS(637),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(757)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__unary_expr] = STATE(950),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(757),
    [sym_block_comment] = STATE(757),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(621),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(623),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(625),
    [anon_sym_AMPmut] = ACTIONS(627),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(629),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(631),
    [anon_sym_loop] = ACTIONS(633),
    [anon_sym_return] = ACTIONS(635),
    [anon_sym_abort] = ACTIONS(637),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(758)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(935),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(758),
    [sym_block_comment] = STATE(758),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(567),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(569),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(571),
    [anon_sym_AMPmut] = ACTIONS(573),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(575),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(577),
    [anon_sym_loop] = ACTIONS(579),
    [anon_sym_return] = ACTIONS(581),
    [anon_sym_abort] = ACTIONS(583),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(759)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(941),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(759),
    [sym_block_comment] = STATE(759),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(567),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(569),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(571),
    [anon_sym_AMPmut] = ACTIONS(573),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(575),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(577),
    [anon_sym_loop] = ACTIONS(579),
    [anon_sym_return] = ACTIONS(581),
    [anon_sym_abort] = ACTIONS(583),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(760)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(947),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(760),
    [sym_block_comment] = STATE(760),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(567),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(569),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(571),
    [anon_sym_AMPmut] = ACTIONS(573),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(575),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(577),
    [anon_sym_loop] = ACTIONS(579),
    [anon_sym_return] = ACTIONS(581),
    [anon_sym_abort] = ACTIONS(583),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(761)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(950),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(761),
    [sym_block_comment] = STATE(761),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(567),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(569),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(571),
    [anon_sym_AMPmut] = ACTIONS(573),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(575),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(577),
    [anon_sym_loop] = ACTIONS(579),
    [anon_sym_return] = ACTIONS(581),
    [anon_sym_abort] = ACTIONS(583),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(762)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1534),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(935),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(762),
    [sym_block_comment] = STATE(762),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(711),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(713),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(715),
    [anon_sym_AMPmut] = ACTIONS(717),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(719),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(721),
    [anon_sym_loop] = ACTIONS(723),
    [anon_sym_return] = ACTIONS(725),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(763)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1534),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(941),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(763),
    [sym_block_comment] = STATE(763),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(711),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(713),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(715),
    [anon_sym_AMPmut] = ACTIONS(717),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(719),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(721),
    [anon_sym_loop] = ACTIONS(723),
    [anon_sym_return] = ACTIONS(725),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(764)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1534),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(947),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(764),
    [sym_block_comment] = STATE(764),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(711),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(713),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(715),
    [anon_sym_AMPmut] = ACTIONS(717),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(719),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(721),
    [anon_sym_loop] = ACTIONS(723),
    [anon_sym_return] = ACTIONS(725),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(765)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1534),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(950),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(765),
    [sym_block_comment] = STATE(765),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(711),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(713),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(715),
    [anon_sym_AMPmut] = ACTIONS(717),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(719),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(721),
    [anon_sym_loop] = ACTIONS(723),
    [anon_sym_return] = ACTIONS(725),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(766)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1534),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(935),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(766),
    [sym_block_comment] = STATE(766),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(727),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(729),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_AMPmut] = ACTIONS(733),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(735),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(737),
    [anon_sym_loop] = ACTIONS(739),
    [anon_sym_return] = ACTIONS(741),
    [anon_sym_abort] = ACTIONS(583),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(767)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1534),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(941),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(767),
    [sym_block_comment] = STATE(767),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(727),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(729),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_AMPmut] = ACTIONS(733),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(735),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(737),
    [anon_sym_loop] = ACTIONS(739),
    [anon_sym_return] = ACTIONS(741),
    [anon_sym_abort] = ACTIONS(583),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(768)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1534),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(947),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(768),
    [sym_block_comment] = STATE(768),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(727),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(729),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_AMPmut] = ACTIONS(733),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(735),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(737),
    [anon_sym_loop] = ACTIONS(739),
    [anon_sym_return] = ACTIONS(741),
    [anon_sym_abort] = ACTIONS(583),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(769)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1534),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(950),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(769),
    [sym_block_comment] = STATE(769),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(727),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(729),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_AMPmut] = ACTIONS(733),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(735),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(737),
    [anon_sym_loop] = ACTIONS(739),
    [anon_sym_return] = ACTIONS(741),
    [anon_sym_abort] = ACTIONS(583),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(770)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(935),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(770),
    [sym_block_comment] = STATE(770),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(771)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1534),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(2424),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(771),
    [sym_block_comment] = STATE(771),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(727),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(729),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_AMPmut] = ACTIONS(733),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(735),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(737),
    [anon_sym_loop] = ACTIONS(739),
    [anon_sym_return] = ACTIONS(741),
    [anon_sym_abort] = ACTIONS(583),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(772)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(935),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(772),
    [sym_block_comment] = STATE(772),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(773)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(941),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(773),
    [sym_block_comment] = STATE(773),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(774)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(947),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(774),
    [sym_block_comment] = STATE(774),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(775)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(950),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(775),
    [sym_block_comment] = STATE(775),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(91),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(403),
    [anon_sym_AMPmut] = ACTIONS(93),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(405),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(97),
    [anon_sym_loop] = ACTIONS(99),
    [anon_sym_return] = ACTIONS(101),
    [anon_sym_abort] = ACTIONS(103),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(776)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(935),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(776),
    [sym_block_comment] = STATE(776),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(777)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(941),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(777),
    [sym_block_comment] = STATE(777),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(778)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(947),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(778),
    [sym_block_comment] = STATE(778),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(779)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(950),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(779),
    [sym_block_comment] = STATE(779),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(117),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_AMPmut] = ACTIONS(119),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(429),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(123),
    [anon_sym_loop] = ACTIONS(125),
    [anon_sym_return] = ACTIONS(127),
    [anon_sym_abort] = ACTIONS(129),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(780)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(941),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(780),
    [sym_block_comment] = STATE(780),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(781)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(947),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(781),
    [sym_block_comment] = STATE(781),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(782)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(950),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(782),
    [sym_block_comment] = STATE(782),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(267),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(271),
    [anon_sym_AMPmut] = ACTIONS(63),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(275),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(75),
    [anon_sym_loop] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_abort] = ACTIONS(81),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(783)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1534),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(1812),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(783),
    [sym_block_comment] = STATE(783),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(743),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(745),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(747),
    [anon_sym_AMPmut] = ACTIONS(749),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(751),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(753),
    [anon_sym_loop] = ACTIONS(755),
    [anon_sym_return] = ACTIONS(757),
    [anon_sym_abort] = ACTIONS(619),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(784)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__unary_expr] = STATE(935),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(784),
    [sym_block_comment] = STATE(784),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(785)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__unary_expr] = STATE(941),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(785),
    [sym_block_comment] = STATE(785),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(786)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__unary_expr] = STATE(947),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(786),
    [sym_block_comment] = STATE(786),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(787)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__unary_expr] = STATE(950),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(787),
    [sym_block_comment] = STATE(787),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(173),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(463),
    [anon_sym_AMPmut] = ACTIONS(175),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(465),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_loop] = ACTIONS(181),
    [anon_sym_return] = ACTIONS(183),
    [anon_sym_abort] = ACTIONS(185),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(788)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(935),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(788),
    [sym_block_comment] = STATE(788),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(789)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(941),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(789),
    [sym_block_comment] = STATE(789),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(790)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(947),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(790),
    [sym_block_comment] = STATE(790),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(791)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(950),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(791),
    [sym_block_comment] = STATE(791),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(137),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(435),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(437),
    [anon_sym_AMPmut] = ACTIONS(139),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(439),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(143),
    [anon_sym_loop] = ACTIONS(145),
    [anon_sym_return] = ACTIONS(147),
    [anon_sym_abort] = ACTIONS(149),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(792)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(935),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(792),
    [sym_block_comment] = STATE(792),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(793)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(941),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(793),
    [sym_block_comment] = STATE(793),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(794)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(947),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(794),
    [sym_block_comment] = STATE(794),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(795)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(950),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(795),
    [sym_block_comment] = STATE(795),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(231),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(483),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(485),
    [anon_sym_AMPmut] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(487),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(489),
    [anon_sym_loop] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(239),
    [anon_sym_abort] = ACTIONS(241),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(796)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(935),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(796),
    [sym_block_comment] = STATE(796),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(639),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(641),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(643),
    [anon_sym_AMPmut] = ACTIONS(645),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(647),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(649),
    [anon_sym_loop] = ACTIONS(651),
    [anon_sym_return] = ACTIONS(653),
    [anon_sym_abort] = ACTIONS(655),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(797)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(950),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(797),
    [sym_block_comment] = STATE(797),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(675),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(677),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(679),
    [anon_sym_AMPmut] = ACTIONS(681),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(683),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(685),
    [anon_sym_loop] = ACTIONS(687),
    [anon_sym_return] = ACTIONS(689),
    [anon_sym_abort] = ACTIONS(691),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(798)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(947),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(798),
    [sym_block_comment] = STATE(798),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(639),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(641),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(643),
    [anon_sym_AMPmut] = ACTIONS(645),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(647),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(649),
    [anon_sym_loop] = ACTIONS(651),
    [anon_sym_return] = ACTIONS(653),
    [anon_sym_abort] = ACTIONS(655),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(799)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(950),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(799),
    [sym_block_comment] = STATE(799),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(639),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(641),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(643),
    [anon_sym_AMPmut] = ACTIONS(645),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(647),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(649),
    [anon_sym_loop] = ACTIONS(651),
    [anon_sym_return] = ACTIONS(653),
    [anon_sym_abort] = ACTIONS(655),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(800)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(935),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(800),
    [sym_block_comment] = STATE(800),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(549),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(551),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(553),
    [anon_sym_AMPmut] = ACTIONS(555),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(557),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(559),
    [anon_sym_loop] = ACTIONS(561),
    [anon_sym_return] = ACTIONS(563),
    [anon_sym_abort] = ACTIONS(565),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(801)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(941),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(801),
    [sym_block_comment] = STATE(801),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(549),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(551),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(553),
    [anon_sym_AMPmut] = ACTIONS(555),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(557),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(559),
    [anon_sym_loop] = ACTIONS(561),
    [anon_sym_return] = ACTIONS(563),
    [anon_sym_abort] = ACTIONS(565),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(802)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(947),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(802),
    [sym_block_comment] = STATE(802),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(549),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(551),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(553),
    [anon_sym_AMPmut] = ACTIONS(555),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(557),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(559),
    [anon_sym_loop] = ACTIONS(561),
    [anon_sym_return] = ACTIONS(563),
    [anon_sym_abort] = ACTIONS(565),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(803)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(950),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(803),
    [sym_block_comment] = STATE(803),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(549),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(551),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(553),
    [anon_sym_AMPmut] = ACTIONS(555),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(557),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(559),
    [anon_sym_loop] = ACTIONS(561),
    [anon_sym_return] = ACTIONS(563),
    [anon_sym_abort] = ACTIONS(565),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(804)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__unary_expr] = STATE(935),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(804),
    [sym_block_comment] = STATE(804),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(805)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__unary_expr] = STATE(941),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(805),
    [sym_block_comment] = STATE(805),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(806)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__unary_expr] = STATE(947),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(806),
    [sym_block_comment] = STATE(806),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(807)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__unary_expr] = STATE(950),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(807),
    [sym_block_comment] = STATE(807),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(155),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(491),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(493),
    [anon_sym_AMPmut] = ACTIONS(157),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(495),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(161),
    [anon_sym_loop] = ACTIONS(163),
    [anon_sym_return] = ACTIONS(165),
    [anon_sym_abort] = ACTIONS(167),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(808)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__unary_expr] = STATE(935),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(808),
    [sym_block_comment] = STATE(808),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(585),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(587),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(589),
    [anon_sym_AMPmut] = ACTIONS(591),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(593),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(595),
    [anon_sym_loop] = ACTIONS(597),
    [anon_sym_return] = ACTIONS(599),
    [anon_sym_abort] = ACTIONS(601),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(809)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__unary_expr] = STATE(941),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(809),
    [sym_block_comment] = STATE(809),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(585),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(587),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(589),
    [anon_sym_AMPmut] = ACTIONS(591),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(593),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(595),
    [anon_sym_loop] = ACTIONS(597),
    [anon_sym_return] = ACTIONS(599),
    [anon_sym_abort] = ACTIONS(601),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(810)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__unary_expr] = STATE(947),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(810),
    [sym_block_comment] = STATE(810),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(585),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(587),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(589),
    [anon_sym_AMPmut] = ACTIONS(591),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(593),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(595),
    [anon_sym_loop] = ACTIONS(597),
    [anon_sym_return] = ACTIONS(599),
    [anon_sym_abort] = ACTIONS(601),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(811)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__unary_expr] = STATE(950),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(811),
    [sym_block_comment] = STATE(811),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(585),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(587),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(589),
    [anon_sym_AMPmut] = ACTIONS(591),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(593),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(595),
    [anon_sym_loop] = ACTIONS(597),
    [anon_sym_return] = ACTIONS(599),
    [anon_sym_abort] = ACTIONS(601),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(812)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(935),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(812),
    [sym_block_comment] = STATE(812),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(813)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(941),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(813),
    [sym_block_comment] = STATE(813),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(814)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(947),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(814),
    [sym_block_comment] = STATE(814),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(815)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(950),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(815),
    [sym_block_comment] = STATE(815),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(247),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_AMPmut] = ACTIONS(249),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(501),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(503),
    [anon_sym_loop] = ACTIONS(253),
    [anon_sym_return] = ACTIONS(255),
    [anon_sym_abort] = ACTIONS(257),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(816)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(935),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(816),
    [sym_block_comment] = STATE(816),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(693),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(695),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(697),
    [anon_sym_AMPmut] = ACTIONS(699),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(701),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(703),
    [anon_sym_loop] = ACTIONS(705),
    [anon_sym_return] = ACTIONS(707),
    [anon_sym_abort] = ACTIONS(709),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(817)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(941),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(817),
    [sym_block_comment] = STATE(817),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(693),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(695),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(697),
    [anon_sym_AMPmut] = ACTIONS(699),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(701),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(703),
    [anon_sym_loop] = ACTIONS(705),
    [anon_sym_return] = ACTIONS(707),
    [anon_sym_abort] = ACTIONS(709),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(818)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(947),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(818),
    [sym_block_comment] = STATE(818),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(693),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(695),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(697),
    [anon_sym_AMPmut] = ACTIONS(699),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(701),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(703),
    [anon_sym_loop] = ACTIONS(705),
    [anon_sym_return] = ACTIONS(707),
    [anon_sym_abort] = ACTIONS(709),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(819)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(950),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(819),
    [sym_block_comment] = STATE(819),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(693),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(695),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(697),
    [anon_sym_AMPmut] = ACTIONS(699),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(701),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(703),
    [anon_sym_loop] = ACTIONS(705),
    [anon_sym_return] = ACTIONS(707),
    [anon_sym_abort] = ACTIONS(709),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(820)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1534),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(935),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(820),
    [sym_block_comment] = STATE(820),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(743),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(745),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(747),
    [anon_sym_AMPmut] = ACTIONS(749),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(751),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(753),
    [anon_sym_loop] = ACTIONS(755),
    [anon_sym_return] = ACTIONS(757),
    [anon_sym_abort] = ACTIONS(619),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(821)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1534),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(941),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(821),
    [sym_block_comment] = STATE(821),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(743),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(745),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(747),
    [anon_sym_AMPmut] = ACTIONS(749),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(751),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(753),
    [anon_sym_loop] = ACTIONS(755),
    [anon_sym_return] = ACTIONS(757),
    [anon_sym_abort] = ACTIONS(619),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(822)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1534),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(947),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(822),
    [sym_block_comment] = STATE(822),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(743),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(745),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(747),
    [anon_sym_AMPmut] = ACTIONS(749),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(751),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(753),
    [anon_sym_loop] = ACTIONS(755),
    [anon_sym_return] = ACTIONS(757),
    [anon_sym_abort] = ACTIONS(619),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(823)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1534),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(950),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(823),
    [sym_block_comment] = STATE(823),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(743),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(745),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(747),
    [anon_sym_AMPmut] = ACTIONS(749),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(751),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(753),
    [anon_sym_loop] = ACTIONS(755),
    [anon_sym_return] = ACTIONS(757),
    [anon_sym_abort] = ACTIONS(619),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(824)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(935),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(824),
    [sym_block_comment] = STATE(824),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(603),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(605),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(607),
    [anon_sym_AMPmut] = ACTIONS(609),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(611),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(613),
    [anon_sym_loop] = ACTIONS(615),
    [anon_sym_return] = ACTIONS(617),
    [anon_sym_abort] = ACTIONS(619),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(825)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(941),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(825),
    [sym_block_comment] = STATE(825),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(603),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(605),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(607),
    [anon_sym_AMPmut] = ACTIONS(609),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(611),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(613),
    [anon_sym_loop] = ACTIONS(615),
    [anon_sym_return] = ACTIONS(617),
    [anon_sym_abort] = ACTIONS(619),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(826)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(947),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(826),
    [sym_block_comment] = STATE(826),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(603),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(605),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(607),
    [anon_sym_AMPmut] = ACTIONS(609),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(611),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(613),
    [anon_sym_loop] = ACTIONS(615),
    [anon_sym_return] = ACTIONS(617),
    [anon_sym_abort] = ACTIONS(619),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(827)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(950),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(827),
    [sym_block_comment] = STATE(827),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(603),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(605),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(607),
    [anon_sym_AMPmut] = ACTIONS(609),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(611),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(613),
    [anon_sym_loop] = ACTIONS(615),
    [anon_sym_return] = ACTIONS(617),
    [anon_sym_abort] = ACTIONS(619),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(828)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__unary_expr] = STATE(935),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(828),
    [sym_block_comment] = STATE(828),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(829)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__unary_expr] = STATE(941),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(829),
    [sym_block_comment] = STATE(829),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(830)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__unary_expr] = STATE(947),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(830),
    [sym_block_comment] = STATE(830),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(831)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__unary_expr] = STATE(950),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(831),
    [sym_block_comment] = STATE(831),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(471),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(475),
    [anon_sym_AMPmut] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(477),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_loop] = ACTIONS(201),
    [anon_sym_return] = ACTIONS(203),
    [anon_sym_abort] = ACTIONS(205),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(832)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__unary_expr] = STATE(935),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(832),
    [sym_block_comment] = STATE(832),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(657),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(659),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(661),
    [anon_sym_AMPmut] = ACTIONS(663),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(665),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(667),
    [anon_sym_loop] = ACTIONS(669),
    [anon_sym_return] = ACTIONS(671),
    [anon_sym_abort] = ACTIONS(673),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(833)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__unary_expr] = STATE(941),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(833),
    [sym_block_comment] = STATE(833),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(657),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(659),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(661),
    [anon_sym_AMPmut] = ACTIONS(663),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(665),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(667),
    [anon_sym_loop] = ACTIONS(669),
    [anon_sym_return] = ACTIONS(671),
    [anon_sym_abort] = ACTIONS(673),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(834)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1167),
    [sym__unary_expr] = STATE(947),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(834),
    [sym_block_comment] = STATE(834),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(657),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(659),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(661),
    [anon_sym_AMPmut] = ACTIONS(663),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(665),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(667),
    [anon_sym_loop] = ACTIONS(669),
    [anon_sym_return] = ACTIONS(671),
    [anon_sym_abort] = ACTIONS(673),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(835)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__unary_expr] = STATE(935),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(835),
    [sym_block_comment] = STATE(835),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(836)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__unary_expr] = STATE(941),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(836),
    [sym_block_comment] = STATE(836),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(837)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__unary_expr] = STATE(947),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(837),
    [sym_block_comment] = STATE(837),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(838)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__unary_expr] = STATE(950),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(838),
    [sym_block_comment] = STATE(838),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(453),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(455),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(457),
    [anon_sym_AMPmut] = ACTIONS(211),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(459),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(215),
    [anon_sym_loop] = ACTIONS(217),
    [anon_sym_return] = ACTIONS(219),
    [anon_sym_abort] = ACTIONS(221),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(839)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__unary_expr] = STATE(935),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(839),
    [sym_block_comment] = STATE(839),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(531),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(533),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(535),
    [anon_sym_AMPmut] = ACTIONS(537),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(539),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(541),
    [anon_sym_loop] = ACTIONS(543),
    [anon_sym_return] = ACTIONS(545),
    [anon_sym_abort] = ACTIONS(547),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(840)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__unary_expr] = STATE(941),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(840),
    [sym_block_comment] = STATE(840),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(531),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(533),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(535),
    [anon_sym_AMPmut] = ACTIONS(537),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(539),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(541),
    [anon_sym_loop] = ACTIONS(543),
    [anon_sym_return] = ACTIONS(545),
    [anon_sym_abort] = ACTIONS(547),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(841)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__unary_expr] = STATE(947),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(841),
    [sym_block_comment] = STATE(841),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(531),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(533),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(535),
    [anon_sym_AMPmut] = ACTIONS(537),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(539),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(541),
    [anon_sym_loop] = ACTIONS(543),
    [anon_sym_return] = ACTIONS(545),
    [anon_sym_abort] = ACTIONS(547),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(842)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(984),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(959),
    [sym__unary_expr] = STATE(950),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(842),
    [sym_block_comment] = STATE(842),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(531),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(533),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(535),
    [anon_sym_AMPmut] = ACTIONS(537),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(539),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(541),
    [anon_sym_loop] = ACTIONS(543),
    [anon_sym_return] = ACTIONS(545),
    [anon_sym_abort] = ACTIONS(547),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(843)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(935),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(843),
    [sym_block_comment] = STATE(843),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(675),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(677),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(679),
    [anon_sym_AMPmut] = ACTIONS(681),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(683),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(685),
    [anon_sym_loop] = ACTIONS(687),
    [anon_sym_return] = ACTIONS(689),
    [anon_sym_abort] = ACTIONS(691),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(844)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(941),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(844),
    [sym_block_comment] = STATE(844),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(675),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(677),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(679),
    [anon_sym_AMPmut] = ACTIONS(681),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(683),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(685),
    [anon_sym_loop] = ACTIONS(687),
    [anon_sym_return] = ACTIONS(689),
    [anon_sym_abort] = ACTIONS(691),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(845)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(947),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(845),
    [sym_block_comment] = STATE(845),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(675),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(677),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(679),
    [anon_sym_AMPmut] = ACTIONS(681),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(683),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(685),
    [anon_sym_loop] = ACTIONS(687),
    [anon_sym_return] = ACTIONS(689),
    [anon_sym_abort] = ACTIONS(691),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(846)] = {
    [sym_number_type] = STATE(859),
    [sym__quantifier_directive] = STATE(860),
    [sym_primitive_type] = STATE(860),
    [sym_discouraged_name] = STATE(866),
    [sym__reuseable_keywords] = STATE(860),
    [sym_number] = STATE(1044),
    [sym_numerical_addr] = STATE(883),
    [sym_bool_literal] = STATE(943),
    [sym_typed_number] = STATE(943),
    [sym_byte_string] = STATE(943),
    [sym__leading_name_access] = STATE(2342),
    [sym_name_access_chain] = STATE(1050),
    [sym__unary_expr] = STATE(941),
    [sym_not_expr] = STATE(925),
    [sym_ref_expr] = STATE(925),
    [sym_ref_mut_expr] = STATE(925),
    [sym_deref_expr] = STATE(925),
    [sym_move_expr] = STATE(925),
    [sym_copy_expr] = STATE(925),
    [sym_receiver_call] = STATE(926),
    [sym_mem_access] = STATE(926),
    [sym_access_field] = STATE(926),
    [sym_term] = STATE(926),
    [sym_vector_value_expr] = STATE(908),
    [sym_tuple_expr] = STATE(908),
    [sym_type_hint_expr] = STATE(908),
    [sym_cast_expr] = STATE(908),
    [sym_match_expr] = STATE(908),
    [sym_if_expr] = STATE(908),
    [sym_while_expr] = STATE(908),
    [sym_loop_expr] = STATE(908),
    [sym_return_expr] = STATE(908),
    [sym_abort_expr] = STATE(908),
    [sym_for_loop_expr] = STATE(908),
    [sym_var] = STATE(908),
    [sym_call_expr] = STATE(908),
    [sym_pack_expr] = STATE(908),
    [sym_macro_call_expr] = STATE(908),
    [sym_value] = STATE(908),
    [sym_spec_block] = STATE(908),
    [sym_block] = STATE(908),
    [sym_line_comment] = STATE(846),
    [sym_block_comment] = STATE(846),
    [sym_identifier] = ACTIONS(23),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_u256] = ACTIONS(25),
    [anon_sym_exists] = ACTIONS(31),
    [anon_sym_forall] = ACTIONS(31),
    [anon_sym_choose] = ACTIONS(31),
    [anon_sym_min] = ACTIONS(31),
    [anon_sym_bool] = ACTIONS(33),
    [anon_sym_address] = ACTIONS(33),
    [anon_sym_vector] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(639),
    [anon_sym_friend] = ACTIONS(41),
    [anon_sym_match] = ACTIONS(43),
    [aux_sym_number_token1] = ACTIONS(45),
    [aux_sym_number_token2] = ACTIONS(47),
    [aux_sym_number_token4] = ACTIONS(49),
    [aux_sym_number_token6] = ACTIONS(51),
    [anon_sym_true] = ACTIONS(53),
    [anon_sym_false] = ACTIONS(53),
    [aux_sym_byte_string_token1] = ACTIONS(55),
    [aux_sym_byte_string_token2] = ACTIONS(55),
    [anon_sym_STAR] = ACTIONS(641),
    [anon_sym_LPAREN] = ACTIONS(59),
    [anon_sym_AMP] = ACTIONS(643),
    [anon_sym_AMPmut] = ACTIONS(645),
    [anon_sym_LBRACE] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(647),
    [anon_sym_move] = ACTIONS(69),
    [anon_sym_copy] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(73),
    [anon_sym_if] = ACTIONS(649),
    [anon_sym_loop] = ACTIONS(651),
    [anon_sym_return] = ACTIONS(653),
    [anon_sym_abort] = ACTIONS(655),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_spec] = ACTIONS(131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(847)] = {
    [sym_line_comment] = STATE(847),
    [sym_block_comment] = STATE(847),
    [sym_identifier] = ACTIONS(759),
    [anon_sym_u8] = ACTIONS(759),
    [anon_sym_u16] = ACTIONS(759),
    [anon_sym_u32] = ACTIONS(759),
    [anon_sym_u64] = ACTIONS(759),
    [anon_sym_u128] = ACTIONS(759),
    [anon_sym_u256] = ACTIONS(759),
    [anon_sym_exists] = ACTIONS(759),
    [anon_sym_forall] = ACTIONS(759),
    [anon_sym_choose] = ACTIONS(759),
    [anon_sym_min] = ACTIONS(759),
    [anon_sym_bool] = ACTIONS(759),
    [anon_sym_address] = ACTIONS(759),
    [anon_sym_vector] = ACTIONS(759),
    [anon_sym_for] = ACTIONS(759),
    [anon_sym_while] = ACTIONS(759),
    [anon_sym_friend] = ACTIONS(759),
    [anon_sym_match] = ACTIONS(759),
    [aux_sym_number_token1] = ACTIONS(759),
    [aux_sym_number_token2] = ACTIONS(761),
    [aux_sym_number_token4] = ACTIONS(761),
    [aux_sym_number_token6] = ACTIONS(761),
    [anon_sym_STAR] = ACTIONS(761),
    [anon_sym_LPAREN] = ACTIONS(761),
    [anon_sym_COMMA] = ACTIONS(761),
    [anon_sym_RPAREN] = ACTIONS(761),
    [anon_sym_PIPE] = ACTIONS(759),
    [anon_sym_AMP] = ACTIONS(759),
    [anon_sym_EQ] = ACTIONS(759),
    [anon_sym_where] = ACTIONS(759),
    [anon_sym_COLON] = ACTIONS(761),
    [anon_sym_LBRACE] = ACTIONS(761),
    [anon_sym_RBRACE] = ACTIONS(761),
    [anon_sym_EQ_EQ_GT] = ACTIONS(761),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(761),
    [anon_sym_PIPE_PIPE] = ACTIONS(761),
    [anon_sym_AMP_AMP] = ACTIONS(761),
    [anon_sym_EQ_EQ] = ACTIONS(759),
    [anon_sym_BANG_EQ] = ACTIONS(761),
    [anon_sym_LT] = ACTIONS(759),
    [anon_sym_GT] = ACTIONS(759),
    [anon_sym_LT_EQ] = ACTIONS(759),
    [anon_sym_GT_EQ] = ACTIONS(761),
    [anon_sym_DOT_DOT] = ACTIONS(761),
    [anon_sym_CARET] = ACTIONS(761),
    [anon_sym_LT_LT] = ACTIONS(761),
    [anon_sym_GT_GT] = ACTIONS(761),
    [anon_sym_PLUS] = ACTIONS(761),
    [anon_sym_DASH] = ACTIONS(761),
    [anon_sym_SLASH] = ACTIONS(759),
    [anon_sym_PERCENT] = ACTIONS(761),
    [anon_sym_DOT] = ACTIONS(759),
    [anon_sym_LBRACK] = ACTIONS(761),
    [anon_sym_RBRACK] = ACTIONS(761),
    [anon_sym_as] = ACTIONS(759),
    [anon_sym_if] = ACTIONS(759),
    [anon_sym_EQ_GT] = ACTIONS(761),
    [anon_sym_else] = ACTIONS(759),
    [anon_sym_POUND] = ACTIONS(761),
    [anon_sym_spec] = ACTIONS(759),
    [anon_sym_invariant] = ACTIONS(759),
    [anon_sym_SEMI] = ACTIONS(761),
    [anon_sym_with] = ACTIONS(759),
    [anon_sym_to] = ACTIONS(759),
    [anon_sym_fun] = ACTIONS(759),
    [anon_sym_native] = ACTIONS(759),
    [anon_sym_public] = ACTIONS(759),
    [anon_sym_entry] = ACTIONS(759),
    [anon_sym_use] = ACTIONS(759),
    [anon_sym_const] = ACTIONS(759),
    [anon_sym_inline] = ACTIONS(759),
    [anon_sym_struct] = ACTIONS(759),
    [anon_sym_enum] = ACTIONS(759),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(848)] = {
    [sym_line_comment] = STATE(848),
    [sym_block_comment] = STATE(848),
    [sym_identifier] = ACTIONS(763),
    [anon_sym_u8] = ACTIONS(763),
    [anon_sym_u16] = ACTIONS(763),
    [anon_sym_u32] = ACTIONS(763),
    [anon_sym_u64] = ACTIONS(763),
    [anon_sym_u128] = ACTIONS(763),
    [anon_sym_u256] = ACTIONS(763),
    [anon_sym_exists] = ACTIONS(763),
    [anon_sym_forall] = ACTIONS(763),
    [anon_sym_choose] = ACTIONS(763),
    [anon_sym_min] = ACTIONS(763),
    [anon_sym_bool] = ACTIONS(763),
    [anon_sym_address] = ACTIONS(763),
    [anon_sym_vector] = ACTIONS(763),
    [anon_sym_for] = ACTIONS(763),
    [anon_sym_while] = ACTIONS(763),
    [anon_sym_friend] = ACTIONS(763),
    [anon_sym_match] = ACTIONS(763),
    [aux_sym_number_token1] = ACTIONS(763),
    [aux_sym_number_token2] = ACTIONS(765),
    [aux_sym_number_token4] = ACTIONS(765),
    [aux_sym_number_token6] = ACTIONS(765),
    [anon_sym_STAR] = ACTIONS(765),
    [anon_sym_LPAREN] = ACTIONS(765),
    [anon_sym_COMMA] = ACTIONS(765),
    [anon_sym_RPAREN] = ACTIONS(765),
    [anon_sym_PIPE] = ACTIONS(763),
    [anon_sym_AMP] = ACTIONS(763),
    [anon_sym_EQ] = ACTIONS(763),
    [anon_sym_where] = ACTIONS(763),
    [anon_sym_COLON] = ACTIONS(765),
    [anon_sym_LBRACE] = ACTIONS(765),
    [anon_sym_RBRACE] = ACTIONS(765),
    [anon_sym_EQ_EQ_GT] = ACTIONS(765),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(765),
    [anon_sym_PIPE_PIPE] = ACTIONS(765),
    [anon_sym_AMP_AMP] = ACTIONS(765),
    [anon_sym_EQ_EQ] = ACTIONS(763),
    [anon_sym_BANG_EQ] = ACTIONS(765),
    [anon_sym_LT] = ACTIONS(763),
    [anon_sym_GT] = ACTIONS(763),
    [anon_sym_LT_EQ] = ACTIONS(763),
    [anon_sym_GT_EQ] = ACTIONS(765),
    [anon_sym_DOT_DOT] = ACTIONS(765),
    [anon_sym_CARET] = ACTIONS(765),
    [anon_sym_LT_LT] = ACTIONS(765),
    [anon_sym_GT_GT] = ACTIONS(765),
    [anon_sym_PLUS] = ACTIONS(765),
    [anon_sym_DASH] = ACTIONS(765),
    [anon_sym_SLASH] = ACTIONS(763),
    [anon_sym_PERCENT] = ACTIONS(765),
    [anon_sym_DOT] = ACTIONS(763),
    [anon_sym_LBRACK] = ACTIONS(765),
    [anon_sym_RBRACK] = ACTIONS(765),
    [anon_sym_as] = ACTIONS(763),
    [anon_sym_if] = ACTIONS(763),
    [anon_sym_EQ_GT] = ACTIONS(765),
    [anon_sym_else] = ACTIONS(763),
    [anon_sym_POUND] = ACTIONS(765),
    [anon_sym_spec] = ACTIONS(763),
    [anon_sym_invariant] = ACTIONS(763),
    [anon_sym_SEMI] = ACTIONS(765),
    [anon_sym_with] = ACTIONS(763),
    [anon_sym_to] = ACTIONS(763),
    [anon_sym_fun] = ACTIONS(763),
    [anon_sym_native] = ACTIONS(763),
    [anon_sym_public] = ACTIONS(763),
    [anon_sym_entry] = ACTIONS(763),
    [anon_sym_use] = ACTIONS(763),
    [anon_sym_const] = ACTIONS(763),
    [anon_sym_inline] = ACTIONS(763),
    [anon_sym_struct] = ACTIONS(763),
    [anon_sym_enum] = ACTIONS(763),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(849)] = {
    [sym_line_comment] = STATE(849),
    [sym_block_comment] = STATE(849),
    [sym_identifier] = ACTIONS(767),
    [anon_sym_u8] = ACTIONS(767),
    [anon_sym_u16] = ACTIONS(767),
    [anon_sym_u32] = ACTIONS(767),
    [anon_sym_u64] = ACTIONS(767),
    [anon_sym_u128] = ACTIONS(767),
    [anon_sym_u256] = ACTIONS(767),
    [anon_sym_exists] = ACTIONS(767),
    [anon_sym_forall] = ACTIONS(767),
    [anon_sym_choose] = ACTIONS(767),
    [anon_sym_min] = ACTIONS(767),
    [anon_sym_bool] = ACTIONS(767),
    [anon_sym_address] = ACTIONS(767),
    [anon_sym_vector] = ACTIONS(767),
    [anon_sym_for] = ACTIONS(767),
    [anon_sym_while] = ACTIONS(767),
    [anon_sym_friend] = ACTIONS(767),
    [anon_sym_match] = ACTIONS(767),
    [aux_sym_number_token1] = ACTIONS(767),
    [aux_sym_number_token2] = ACTIONS(769),
    [aux_sym_number_token4] = ACTIONS(769),
    [aux_sym_number_token6] = ACTIONS(769),
    [anon_sym_STAR] = ACTIONS(769),
    [anon_sym_LPAREN] = ACTIONS(769),
    [anon_sym_COMMA] = ACTIONS(769),
    [anon_sym_RPAREN] = ACTIONS(769),
    [anon_sym_PIPE] = ACTIONS(767),
    [anon_sym_AMP] = ACTIONS(767),
    [anon_sym_EQ] = ACTIONS(767),
    [anon_sym_where] = ACTIONS(767),
    [anon_sym_COLON] = ACTIONS(769),
    [anon_sym_LBRACE] = ACTIONS(769),
    [anon_sym_RBRACE] = ACTIONS(769),
    [anon_sym_EQ_EQ_GT] = ACTIONS(769),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(769),
    [anon_sym_PIPE_PIPE] = ACTIONS(769),
    [anon_sym_AMP_AMP] = ACTIONS(769),
    [anon_sym_EQ_EQ] = ACTIONS(767),
    [anon_sym_BANG_EQ] = ACTIONS(769),
    [anon_sym_LT] = ACTIONS(767),
    [anon_sym_GT] = ACTIONS(767),
    [anon_sym_LT_EQ] = ACTIONS(767),
    [anon_sym_GT_EQ] = ACTIONS(769),
    [anon_sym_DOT_DOT] = ACTIONS(769),
    [anon_sym_CARET] = ACTIONS(769),
    [anon_sym_LT_LT] = ACTIONS(769),
    [anon_sym_GT_GT] = ACTIONS(769),
    [anon_sym_PLUS] = ACTIONS(769),
    [anon_sym_DASH] = ACTIONS(769),
    [anon_sym_SLASH] = ACTIONS(767),
    [anon_sym_PERCENT] = ACTIONS(769),
    [anon_sym_DOT] = ACTIONS(767),
    [anon_sym_LBRACK] = ACTIONS(769),
    [anon_sym_RBRACK] = ACTIONS(769),
    [anon_sym_as] = ACTIONS(767),
    [anon_sym_if] = ACTIONS(767),
    [anon_sym_EQ_GT] = ACTIONS(769),
    [anon_sym_else] = ACTIONS(767),
    [anon_sym_POUND] = ACTIONS(769),
    [anon_sym_spec] = ACTIONS(767),
    [anon_sym_invariant] = ACTIONS(767),
    [anon_sym_SEMI] = ACTIONS(769),
    [anon_sym_with] = ACTIONS(767),
    [anon_sym_to] = ACTIONS(767),
    [anon_sym_fun] = ACTIONS(767),
    [anon_sym_native] = ACTIONS(767),
    [anon_sym_public] = ACTIONS(767),
    [anon_sym_entry] = ACTIONS(767),
    [anon_sym_use] = ACTIONS(767),
    [anon_sym_const] = ACTIONS(767),
    [anon_sym_inline] = ACTIONS(767),
    [anon_sym_struct] = ACTIONS(767),
    [anon_sym_enum] = ACTIONS(767),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(850)] = {
    [sym_line_comment] = STATE(850),
    [sym_block_comment] = STATE(850),
    [sym_identifier] = ACTIONS(771),
    [anon_sym_u8] = ACTIONS(771),
    [anon_sym_u16] = ACTIONS(771),
    [anon_sym_u32] = ACTIONS(771),
    [anon_sym_u64] = ACTIONS(771),
    [anon_sym_u128] = ACTIONS(771),
    [anon_sym_u256] = ACTIONS(771),
    [anon_sym_exists] = ACTIONS(771),
    [anon_sym_forall] = ACTIONS(771),
    [anon_sym_choose] = ACTIONS(771),
    [anon_sym_min] = ACTIONS(771),
    [anon_sym_bool] = ACTIONS(771),
    [anon_sym_address] = ACTIONS(771),
    [anon_sym_vector] = ACTIONS(771),
    [anon_sym_for] = ACTIONS(771),
    [anon_sym_while] = ACTIONS(771),
    [anon_sym_friend] = ACTIONS(771),
    [anon_sym_match] = ACTIONS(771),
    [aux_sym_number_token1] = ACTIONS(771),
    [aux_sym_number_token2] = ACTIONS(773),
    [aux_sym_number_token4] = ACTIONS(773),
    [aux_sym_number_token6] = ACTIONS(773),
    [anon_sym_STAR] = ACTIONS(773),
    [anon_sym_LPAREN] = ACTIONS(773),
    [anon_sym_COMMA] = ACTIONS(773),
    [anon_sym_RPAREN] = ACTIONS(773),
    [anon_sym_PIPE] = ACTIONS(771),
    [anon_sym_AMP] = ACTIONS(771),
    [anon_sym_EQ] = ACTIONS(771),
    [anon_sym_where] = ACTIONS(771),
    [anon_sym_COLON] = ACTIONS(773),
    [anon_sym_LBRACE] = ACTIONS(773),
    [anon_sym_RBRACE] = ACTIONS(773),
    [anon_sym_EQ_EQ_GT] = ACTIONS(773),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(773),
    [anon_sym_PIPE_PIPE] = ACTIONS(773),
    [anon_sym_AMP_AMP] = ACTIONS(773),
    [anon_sym_EQ_EQ] = ACTIONS(771),
    [anon_sym_BANG_EQ] = ACTIONS(773),
    [anon_sym_LT] = ACTIONS(771),
    [anon_sym_GT] = ACTIONS(771),
    [anon_sym_LT_EQ] = ACTIONS(771),
    [anon_sym_GT_EQ] = ACTIONS(773),
    [anon_sym_DOT_DOT] = ACTIONS(773),
    [anon_sym_CARET] = ACTIONS(773),
    [anon_sym_LT_LT] = ACTIONS(773),
    [anon_sym_GT_GT] = ACTIONS(773),
    [anon_sym_PLUS] = ACTIONS(773),
    [anon_sym_DASH] = ACTIONS(773),
    [anon_sym_SLASH] = ACTIONS(771),
    [anon_sym_PERCENT] = ACTIONS(773),
    [anon_sym_DOT] = ACTIONS(771),
    [anon_sym_LBRACK] = ACTIONS(773),
    [anon_sym_RBRACK] = ACTIONS(773),
    [anon_sym_as] = ACTIONS(771),
    [anon_sym_if] = ACTIONS(771),
    [anon_sym_EQ_GT] = ACTIONS(773),
    [anon_sym_else] = ACTIONS(771),
    [anon_sym_POUND] = ACTIONS(773),
    [anon_sym_spec] = ACTIONS(771),
    [anon_sym_invariant] = ACTIONS(771),
    [anon_sym_SEMI] = ACTIONS(773),
    [anon_sym_with] = ACTIONS(771),
    [anon_sym_to] = ACTIONS(771),
    [anon_sym_fun] = ACTIONS(771),
    [anon_sym_native] = ACTIONS(771),
    [anon_sym_public] = ACTIONS(771),
    [anon_sym_entry] = ACTIONS(771),
    [anon_sym_use] = ACTIONS(771),
    [anon_sym_const] = ACTIONS(771),
    [anon_sym_inline] = ACTIONS(771),
    [anon_sym_struct] = ACTIONS(771),
    [anon_sym_enum] = ACTIONS(771),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(851)] = {
    [sym_line_comment] = STATE(851),
    [sym_block_comment] = STATE(851),
    [sym_identifier] = ACTIONS(775),
    [anon_sym_u8] = ACTIONS(775),
    [anon_sym_u16] = ACTIONS(775),
    [anon_sym_u32] = ACTIONS(775),
    [anon_sym_u64] = ACTIONS(775),
    [anon_sym_u128] = ACTIONS(775),
    [anon_sym_u256] = ACTIONS(775),
    [anon_sym_exists] = ACTIONS(775),
    [anon_sym_forall] = ACTIONS(775),
    [anon_sym_choose] = ACTIONS(775),
    [anon_sym_min] = ACTIONS(775),
    [anon_sym_bool] = ACTIONS(775),
    [anon_sym_address] = ACTIONS(775),
    [anon_sym_vector] = ACTIONS(775),
    [anon_sym_for] = ACTIONS(775),
    [anon_sym_while] = ACTIONS(775),
    [anon_sym_friend] = ACTIONS(775),
    [anon_sym_match] = ACTIONS(775),
    [aux_sym_number_token1] = ACTIONS(775),
    [aux_sym_number_token2] = ACTIONS(777),
    [aux_sym_number_token4] = ACTIONS(777),
    [aux_sym_number_token6] = ACTIONS(777),
    [anon_sym_STAR] = ACTIONS(777),
    [anon_sym_LPAREN] = ACTIONS(777),
    [anon_sym_COMMA] = ACTIONS(777),
    [anon_sym_RPAREN] = ACTIONS(777),
    [anon_sym_PIPE] = ACTIONS(775),
    [anon_sym_AMP] = ACTIONS(775),
    [anon_sym_EQ] = ACTIONS(775),
    [anon_sym_where] = ACTIONS(775),
    [anon_sym_COLON] = ACTIONS(777),
    [anon_sym_LBRACE] = ACTIONS(777),
    [anon_sym_RBRACE] = ACTIONS(777),
    [anon_sym_EQ_EQ_GT] = ACTIONS(777),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(777),
    [anon_sym_PIPE_PIPE] = ACTIONS(777),
    [anon_sym_AMP_AMP] = ACTIONS(777),
    [anon_sym_EQ_EQ] = ACTIONS(775),
    [anon_sym_BANG_EQ] = ACTIONS(777),
    [anon_sym_LT] = ACTIONS(775),
    [anon_sym_GT] = ACTIONS(775),
    [anon_sym_LT_EQ] = ACTIONS(775),
    [anon_sym_GT_EQ] = ACTIONS(777),
    [anon_sym_DOT_DOT] = ACTIONS(777),
    [anon_sym_CARET] = ACTIONS(777),
    [anon_sym_LT_LT] = ACTIONS(777),
    [anon_sym_GT_GT] = ACTIONS(777),
    [anon_sym_PLUS] = ACTIONS(777),
    [anon_sym_DASH] = ACTIONS(777),
    [anon_sym_SLASH] = ACTIONS(775),
    [anon_sym_PERCENT] = ACTIONS(777),
    [anon_sym_DOT] = ACTIONS(775),
    [anon_sym_LBRACK] = ACTIONS(777),
    [anon_sym_RBRACK] = ACTIONS(777),
    [anon_sym_as] = ACTIONS(775),
    [anon_sym_if] = ACTIONS(775),
    [anon_sym_EQ_GT] = ACTIONS(777),
    [anon_sym_else] = ACTIONS(775),
    [anon_sym_POUND] = ACTIONS(777),
    [anon_sym_spec] = ACTIONS(775),
    [anon_sym_invariant] = ACTIONS(775),
    [anon_sym_SEMI] = ACTIONS(777),
    [anon_sym_with] = ACTIONS(775),
    [anon_sym_to] = ACTIONS(775),
    [anon_sym_fun] = ACTIONS(775),
    [anon_sym_native] = ACTIONS(775),
    [anon_sym_public] = ACTIONS(775),
    [anon_sym_entry] = ACTIONS(775),
    [anon_sym_use] = ACTIONS(775),
    [anon_sym_const] = ACTIONS(775),
    [anon_sym_inline] = ACTIONS(775),
    [anon_sym_struct] = ACTIONS(775),
    [anon_sym_enum] = ACTIONS(775),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(852)] = {
    [sym_line_comment] = STATE(852),
    [sym_block_comment] = STATE(852),
    [sym_identifier] = ACTIONS(779),
    [anon_sym_u8] = ACTIONS(779),
    [anon_sym_u16] = ACTIONS(779),
    [anon_sym_u32] = ACTIONS(779),
    [anon_sym_u64] = ACTIONS(779),
    [anon_sym_u128] = ACTIONS(779),
    [anon_sym_u256] = ACTIONS(779),
    [anon_sym_exists] = ACTIONS(779),
    [anon_sym_forall] = ACTIONS(779),
    [anon_sym_choose] = ACTIONS(779),
    [anon_sym_min] = ACTIONS(779),
    [anon_sym_bool] = ACTIONS(779),
    [anon_sym_address] = ACTIONS(779),
    [anon_sym_vector] = ACTIONS(779),
    [anon_sym_for] = ACTIONS(779),
    [anon_sym_while] = ACTIONS(779),
    [anon_sym_friend] = ACTIONS(779),
    [anon_sym_match] = ACTIONS(779),
    [aux_sym_number_token1] = ACTIONS(779),
    [aux_sym_number_token2] = ACTIONS(781),
    [aux_sym_number_token4] = ACTIONS(781),
    [aux_sym_number_token6] = ACTIONS(781),
    [anon_sym_STAR] = ACTIONS(781),
    [anon_sym_LPAREN] = ACTIONS(781),
    [anon_sym_COMMA] = ACTIONS(781),
    [anon_sym_RPAREN] = ACTIONS(781),
    [anon_sym_PIPE] = ACTIONS(779),
    [anon_sym_AMP] = ACTIONS(779),
    [anon_sym_EQ] = ACTIONS(779),
    [anon_sym_where] = ACTIONS(779),
    [anon_sym_COLON] = ACTIONS(781),
    [anon_sym_LBRACE] = ACTIONS(781),
    [anon_sym_RBRACE] = ACTIONS(781),
    [anon_sym_EQ_EQ_GT] = ACTIONS(781),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(781),
    [anon_sym_PIPE_PIPE] = ACTIONS(781),
    [anon_sym_AMP_AMP] = ACTIONS(781),
    [anon_sym_EQ_EQ] = ACTIONS(779),
    [anon_sym_BANG_EQ] = ACTIONS(781),
    [anon_sym_LT] = ACTIONS(779),
    [anon_sym_GT] = ACTIONS(779),
    [anon_sym_LT_EQ] = ACTIONS(779),
    [anon_sym_GT_EQ] = ACTIONS(781),
    [anon_sym_DOT_DOT] = ACTIONS(781),
    [anon_sym_CARET] = ACTIONS(781),
    [anon_sym_LT_LT] = ACTIONS(781),
    [anon_sym_GT_GT] = ACTIONS(781),
    [anon_sym_PLUS] = ACTIONS(781),
    [anon_sym_DASH] = ACTIONS(781),
    [anon_sym_SLASH] = ACTIONS(779),
    [anon_sym_PERCENT] = ACTIONS(781),
    [anon_sym_DOT] = ACTIONS(779),
    [anon_sym_LBRACK] = ACTIONS(781),
    [anon_sym_RBRACK] = ACTIONS(781),
    [anon_sym_as] = ACTIONS(779),
    [anon_sym_if] = ACTIONS(779),
    [anon_sym_EQ_GT] = ACTIONS(781),
    [anon_sym_else] = ACTIONS(779),
    [anon_sym_POUND] = ACTIONS(781),
    [anon_sym_spec] = ACTIONS(779),
    [anon_sym_invariant] = ACTIONS(779),
    [anon_sym_SEMI] = ACTIONS(781),
    [anon_sym_with] = ACTIONS(779),
    [anon_sym_to] = ACTIONS(779),
    [anon_sym_fun] = ACTIONS(779),
    [anon_sym_native] = ACTIONS(779),
    [anon_sym_public] = ACTIONS(779),
    [anon_sym_entry] = ACTIONS(779),
    [anon_sym_use] = ACTIONS(779),
    [anon_sym_const] = ACTIONS(779),
    [anon_sym_inline] = ACTIONS(779),
    [anon_sym_struct] = ACTIONS(779),
    [anon_sym_enum] = ACTIONS(779),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(853)] = {
    [sym_line_comment] = STATE(853),
    [sym_block_comment] = STATE(853),
    [sym_identifier] = ACTIONS(783),
    [anon_sym_u8] = ACTIONS(783),
    [anon_sym_u16] = ACTIONS(783),
    [anon_sym_u32] = ACTIONS(783),
    [anon_sym_u64] = ACTIONS(783),
    [anon_sym_u128] = ACTIONS(783),
    [anon_sym_u256] = ACTIONS(783),
    [anon_sym_exists] = ACTIONS(783),
    [anon_sym_forall] = ACTIONS(783),
    [anon_sym_choose] = ACTIONS(783),
    [anon_sym_min] = ACTIONS(783),
    [anon_sym_bool] = ACTIONS(783),
    [anon_sym_address] = ACTIONS(783),
    [anon_sym_vector] = ACTIONS(783),
    [anon_sym_for] = ACTIONS(783),
    [anon_sym_while] = ACTIONS(783),
    [anon_sym_friend] = ACTIONS(783),
    [anon_sym_match] = ACTIONS(783),
    [aux_sym_number_token1] = ACTIONS(783),
    [aux_sym_number_token2] = ACTIONS(785),
    [aux_sym_number_token4] = ACTIONS(785),
    [aux_sym_number_token6] = ACTIONS(785),
    [anon_sym_STAR] = ACTIONS(785),
    [anon_sym_LPAREN] = ACTIONS(785),
    [anon_sym_COMMA] = ACTIONS(785),
    [anon_sym_RPAREN] = ACTIONS(785),
    [anon_sym_PIPE] = ACTIONS(783),
    [anon_sym_AMP] = ACTIONS(783),
    [anon_sym_EQ] = ACTIONS(783),
    [anon_sym_where] = ACTIONS(783),
    [anon_sym_COLON] = ACTIONS(785),
    [anon_sym_LBRACE] = ACTIONS(785),
    [anon_sym_RBRACE] = ACTIONS(785),
    [anon_sym_EQ_EQ_GT] = ACTIONS(785),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(785),
    [anon_sym_PIPE_PIPE] = ACTIONS(785),
    [anon_sym_AMP_AMP] = ACTIONS(785),
    [anon_sym_EQ_EQ] = ACTIONS(783),
    [anon_sym_BANG_EQ] = ACTIONS(785),
    [anon_sym_LT] = ACTIONS(783),
    [anon_sym_GT] = ACTIONS(783),
    [anon_sym_LT_EQ] = ACTIONS(783),
    [anon_sym_GT_EQ] = ACTIONS(785),
    [anon_sym_DOT_DOT] = ACTIONS(785),
    [anon_sym_CARET] = ACTIONS(785),
    [anon_sym_LT_LT] = ACTIONS(785),
    [anon_sym_GT_GT] = ACTIONS(785),
    [anon_sym_PLUS] = ACTIONS(785),
    [anon_sym_DASH] = ACTIONS(785),
    [anon_sym_SLASH] = ACTIONS(783),
    [anon_sym_PERCENT] = ACTIONS(785),
    [anon_sym_DOT] = ACTIONS(783),
    [anon_sym_LBRACK] = ACTIONS(785),
    [anon_sym_RBRACK] = ACTIONS(785),
    [anon_sym_as] = ACTIONS(783),
    [anon_sym_if] = ACTIONS(783),
    [anon_sym_EQ_GT] = ACTIONS(785),
    [anon_sym_else] = ACTIONS(783),
    [anon_sym_POUND] = ACTIONS(785),
    [anon_sym_spec] = ACTIONS(783),
    [anon_sym_invariant] = ACTIONS(783),
    [anon_sym_SEMI] = ACTIONS(785),
    [anon_sym_with] = ACTIONS(783),
    [anon_sym_to] = ACTIONS(783),
    [anon_sym_fun] = ACTIONS(783),
    [anon_sym_native] = ACTIONS(783),
    [anon_sym_public] = ACTIONS(783),
    [anon_sym_entry] = ACTIONS(783),
    [anon_sym_use] = ACTIONS(783),
    [anon_sym_const] = ACTIONS(783),
    [anon_sym_inline] = ACTIONS(783),
    [anon_sym_struct] = ACTIONS(783),
    [anon_sym_enum] = ACTIONS(783),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(854)] = {
    [sym_line_comment] = STATE(854),
    [sym_block_comment] = STATE(854),
    [sym_identifier] = ACTIONS(787),
    [anon_sym_u8] = ACTIONS(787),
    [anon_sym_u16] = ACTIONS(787),
    [anon_sym_u32] = ACTIONS(787),
    [anon_sym_u64] = ACTIONS(787),
    [anon_sym_u128] = ACTIONS(787),
    [anon_sym_u256] = ACTIONS(787),
    [anon_sym_exists] = ACTIONS(787),
    [anon_sym_forall] = ACTIONS(787),
    [anon_sym_choose] = ACTIONS(787),
    [anon_sym_min] = ACTIONS(787),
    [anon_sym_bool] = ACTIONS(787),
    [anon_sym_address] = ACTIONS(787),
    [anon_sym_vector] = ACTIONS(787),
    [anon_sym_for] = ACTIONS(787),
    [anon_sym_while] = ACTIONS(787),
    [anon_sym_friend] = ACTIONS(787),
    [anon_sym_match] = ACTIONS(787),
    [aux_sym_number_token1] = ACTIONS(787),
    [aux_sym_number_token2] = ACTIONS(789),
    [aux_sym_number_token4] = ACTIONS(789),
    [aux_sym_number_token6] = ACTIONS(789),
    [anon_sym_STAR] = ACTIONS(789),
    [anon_sym_LPAREN] = ACTIONS(789),
    [anon_sym_COMMA] = ACTIONS(789),
    [anon_sym_RPAREN] = ACTIONS(789),
    [anon_sym_PIPE] = ACTIONS(787),
    [anon_sym_AMP] = ACTIONS(787),
    [anon_sym_EQ] = ACTIONS(787),
    [anon_sym_where] = ACTIONS(787),
    [anon_sym_COLON] = ACTIONS(789),
    [anon_sym_LBRACE] = ACTIONS(789),
    [anon_sym_RBRACE] = ACTIONS(789),
    [anon_sym_EQ_EQ_GT] = ACTIONS(789),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(789),
    [anon_sym_PIPE_PIPE] = ACTIONS(789),
    [anon_sym_AMP_AMP] = ACTIONS(789),
    [anon_sym_EQ_EQ] = ACTIONS(787),
    [anon_sym_BANG_EQ] = ACTIONS(789),
    [anon_sym_LT] = ACTIONS(787),
    [anon_sym_GT] = ACTIONS(787),
    [anon_sym_LT_EQ] = ACTIONS(787),
    [anon_sym_GT_EQ] = ACTIONS(789),
    [anon_sym_DOT_DOT] = ACTIONS(789),
    [anon_sym_CARET] = ACTIONS(789),
    [anon_sym_LT_LT] = ACTIONS(789),
    [anon_sym_GT_GT] = ACTIONS(789),
    [anon_sym_PLUS] = ACTIONS(789),
    [anon_sym_DASH] = ACTIONS(789),
    [anon_sym_SLASH] = ACTIONS(787),
    [anon_sym_PERCENT] = ACTIONS(789),
    [anon_sym_DOT] = ACTIONS(787),
    [anon_sym_LBRACK] = ACTIONS(789),
    [anon_sym_RBRACK] = ACTIONS(789),
    [anon_sym_as] = ACTIONS(787),
    [anon_sym_if] = ACTIONS(787),
    [anon_sym_EQ_GT] = ACTIONS(789),
    [anon_sym_else] = ACTIONS(787),
    [anon_sym_POUND] = ACTIONS(789),
    [anon_sym_spec] = ACTIONS(787),
    [anon_sym_invariant] = ACTIONS(787),
    [anon_sym_SEMI] = ACTIONS(789),
    [anon_sym_with] = ACTIONS(787),
    [anon_sym_to] = ACTIONS(787),
    [anon_sym_fun] = ACTIONS(787),
    [anon_sym_native] = ACTIONS(787),
    [anon_sym_public] = ACTIONS(787),
    [anon_sym_entry] = ACTIONS(787),
    [anon_sym_use] = ACTIONS(787),
    [anon_sym_const] = ACTIONS(787),
    [anon_sym_inline] = ACTIONS(787),
    [anon_sym_struct] = ACTIONS(787),
    [anon_sym_enum] = ACTIONS(787),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(855)] = {
    [sym_line_comment] = STATE(855),
    [sym_block_comment] = STATE(855),
    [sym_identifier] = ACTIONS(791),
    [anon_sym_u8] = ACTIONS(791),
    [anon_sym_u16] = ACTIONS(791),
    [anon_sym_u32] = ACTIONS(791),
    [anon_sym_u64] = ACTIONS(791),
    [anon_sym_u128] = ACTIONS(791),
    [anon_sym_u256] = ACTIONS(791),
    [anon_sym_exists] = ACTIONS(791),
    [anon_sym_forall] = ACTIONS(791),
    [anon_sym_choose] = ACTIONS(791),
    [anon_sym_min] = ACTIONS(791),
    [anon_sym_bool] = ACTIONS(791),
    [anon_sym_address] = ACTIONS(791),
    [anon_sym_vector] = ACTIONS(791),
    [anon_sym_for] = ACTIONS(791),
    [anon_sym_while] = ACTIONS(791),
    [anon_sym_friend] = ACTIONS(791),
    [anon_sym_match] = ACTIONS(791),
    [aux_sym_number_token1] = ACTIONS(791),
    [aux_sym_number_token2] = ACTIONS(793),
    [aux_sym_number_token4] = ACTIONS(793),
    [aux_sym_number_token6] = ACTIONS(793),
    [anon_sym_STAR] = ACTIONS(793),
    [anon_sym_LPAREN] = ACTIONS(793),
    [anon_sym_COMMA] = ACTIONS(793),
    [anon_sym_RPAREN] = ACTIONS(793),
    [anon_sym_PIPE] = ACTIONS(791),
    [anon_sym_AMP] = ACTIONS(791),
    [anon_sym_EQ] = ACTIONS(791),
    [anon_sym_where] = ACTIONS(791),
    [anon_sym_COLON] = ACTIONS(793),
    [anon_sym_LBRACE] = ACTIONS(793),
    [anon_sym_RBRACE] = ACTIONS(793),
    [anon_sym_EQ_EQ_GT] = ACTIONS(793),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(793),
    [anon_sym_PIPE_PIPE] = ACTIONS(793),
    [anon_sym_AMP_AMP] = ACTIONS(793),
    [anon_sym_EQ_EQ] = ACTIONS(791),
    [anon_sym_BANG_EQ] = ACTIONS(793),
    [anon_sym_LT] = ACTIONS(791),
    [anon_sym_GT] = ACTIONS(791),
    [anon_sym_LT_EQ] = ACTIONS(791),
    [anon_sym_GT_EQ] = ACTIONS(793),
    [anon_sym_DOT_DOT] = ACTIONS(793),
    [anon_sym_CARET] = ACTIONS(793),
    [anon_sym_LT_LT] = ACTIONS(793),
    [anon_sym_GT_GT] = ACTIONS(793),
    [anon_sym_PLUS] = ACTIONS(793),
    [anon_sym_DASH] = ACTIONS(793),
    [anon_sym_SLASH] = ACTIONS(791),
    [anon_sym_PERCENT] = ACTIONS(793),
    [anon_sym_DOT] = ACTIONS(791),
    [anon_sym_LBRACK] = ACTIONS(793),
    [anon_sym_RBRACK] = ACTIONS(793),
    [anon_sym_as] = ACTIONS(791),
    [anon_sym_if] = ACTIONS(791),
    [anon_sym_EQ_GT] = ACTIONS(793),
    [anon_sym_else] = ACTIONS(791),
    [anon_sym_POUND] = ACTIONS(793),
    [anon_sym_spec] = ACTIONS(791),
    [anon_sym_invariant] = ACTIONS(791),
    [anon_sym_SEMI] = ACTIONS(793),
    [anon_sym_with] = ACTIONS(791),
    [anon_sym_to] = ACTIONS(791),
    [anon_sym_fun] = ACTIONS(791),
    [anon_sym_native] = ACTIONS(791),
    [anon_sym_public] = ACTIONS(791),
    [anon_sym_entry] = ACTIONS(791),
    [anon_sym_use] = ACTIONS(791),
    [anon_sym_const] = ACTIONS(791),
    [anon_sym_inline] = ACTIONS(791),
    [anon_sym_struct] = ACTIONS(791),
    [anon_sym_enum] = ACTIONS(791),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(856)] = {
    [sym_line_comment] = STATE(856),
    [sym_block_comment] = STATE(856),
    [sym_identifier] = ACTIONS(795),
    [anon_sym_u8] = ACTIONS(795),
    [anon_sym_u16] = ACTIONS(795),
    [anon_sym_u32] = ACTIONS(795),
    [anon_sym_u64] = ACTIONS(795),
    [anon_sym_u128] = ACTIONS(795),
    [anon_sym_u256] = ACTIONS(795),
    [anon_sym_exists] = ACTIONS(795),
    [anon_sym_forall] = ACTIONS(795),
    [anon_sym_choose] = ACTIONS(795),
    [anon_sym_min] = ACTIONS(795),
    [anon_sym_bool] = ACTIONS(795),
    [anon_sym_address] = ACTIONS(795),
    [anon_sym_vector] = ACTIONS(795),
    [anon_sym_for] = ACTIONS(795),
    [anon_sym_while] = ACTIONS(795),
    [anon_sym_friend] = ACTIONS(795),
    [anon_sym_match] = ACTIONS(795),
    [aux_sym_number_token1] = ACTIONS(795),
    [aux_sym_number_token2] = ACTIONS(797),
    [aux_sym_number_token4] = ACTIONS(797),
    [aux_sym_number_token6] = ACTIONS(797),
    [anon_sym_STAR] = ACTIONS(797),
    [anon_sym_LPAREN] = ACTIONS(797),
    [anon_sym_COMMA] = ACTIONS(797),
    [anon_sym_RPAREN] = ACTIONS(797),
    [anon_sym_PIPE] = ACTIONS(795),
    [anon_sym_AMP] = ACTIONS(795),
    [anon_sym_EQ] = ACTIONS(795),
    [anon_sym_where] = ACTIONS(795),
    [anon_sym_COLON] = ACTIONS(797),
    [anon_sym_LBRACE] = ACTIONS(797),
    [anon_sym_RBRACE] = ACTIONS(797),
    [anon_sym_EQ_EQ_GT] = ACTIONS(797),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(797),
    [anon_sym_PIPE_PIPE] = ACTIONS(797),
    [anon_sym_AMP_AMP] = ACTIONS(797),
    [anon_sym_EQ_EQ] = ACTIONS(795),
    [anon_sym_BANG_EQ] = ACTIONS(797),
    [anon_sym_LT] = ACTIONS(795),
    [anon_sym_GT] = ACTIONS(795),
    [anon_sym_LT_EQ] = ACTIONS(795),
    [anon_sym_GT_EQ] = ACTIONS(797),
    [anon_sym_DOT_DOT] = ACTIONS(797),
    [anon_sym_CARET] = ACTIONS(797),
    [anon_sym_LT_LT] = ACTIONS(797),
    [anon_sym_GT_GT] = ACTIONS(797),
    [anon_sym_PLUS] = ACTIONS(797),
    [anon_sym_DASH] = ACTIONS(797),
    [anon_sym_SLASH] = ACTIONS(795),
    [anon_sym_PERCENT] = ACTIONS(797),
    [anon_sym_DOT] = ACTIONS(795),
    [anon_sym_LBRACK] = ACTIONS(797),
    [anon_sym_RBRACK] = ACTIONS(797),
    [anon_sym_as] = ACTIONS(795),
    [anon_sym_if] = ACTIONS(795),
    [anon_sym_EQ_GT] = ACTIONS(797),
    [anon_sym_else] = ACTIONS(795),
    [anon_sym_POUND] = ACTIONS(797),
    [anon_sym_spec] = ACTIONS(795),
    [anon_sym_invariant] = ACTIONS(795),
    [anon_sym_SEMI] = ACTIONS(797),
    [anon_sym_with] = ACTIONS(795),
    [anon_sym_to] = ACTIONS(795),
    [anon_sym_fun] = ACTIONS(795),
    [anon_sym_native] = ACTIONS(795),
    [anon_sym_public] = ACTIONS(795),
    [anon_sym_entry] = ACTIONS(795),
    [anon_sym_use] = ACTIONS(795),
    [anon_sym_const] = ACTIONS(795),
    [anon_sym_inline] = ACTIONS(795),
    [anon_sym_struct] = ACTIONS(795),
    [anon_sym_enum] = ACTIONS(795),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(857)] = {
    [sym_line_comment] = STATE(857),
    [sym_block_comment] = STATE(857),
    [sym_identifier] = ACTIONS(799),
    [anon_sym_u8] = ACTIONS(799),
    [anon_sym_u16] = ACTIONS(799),
    [anon_sym_u32] = ACTIONS(799),
    [anon_sym_u64] = ACTIONS(799),
    [anon_sym_u128] = ACTIONS(799),
    [anon_sym_u256] = ACTIONS(799),
    [anon_sym_exists] = ACTIONS(799),
    [anon_sym_forall] = ACTIONS(799),
    [anon_sym_choose] = ACTIONS(799),
    [anon_sym_min] = ACTIONS(799),
    [anon_sym_bool] = ACTIONS(799),
    [anon_sym_address] = ACTIONS(799),
    [anon_sym_vector] = ACTIONS(799),
    [anon_sym_for] = ACTIONS(799),
    [anon_sym_while] = ACTIONS(799),
    [anon_sym_friend] = ACTIONS(799),
    [anon_sym_match] = ACTIONS(799),
    [aux_sym_number_token1] = ACTIONS(799),
    [aux_sym_number_token2] = ACTIONS(801),
    [aux_sym_number_token4] = ACTIONS(801),
    [aux_sym_number_token6] = ACTIONS(801),
    [anon_sym_STAR] = ACTIONS(801),
    [anon_sym_LPAREN] = ACTIONS(801),
    [anon_sym_COMMA] = ACTIONS(801),
    [anon_sym_RPAREN] = ACTIONS(801),
    [anon_sym_PIPE] = ACTIONS(799),
    [anon_sym_AMP] = ACTIONS(799),
    [anon_sym_EQ] = ACTIONS(799),
    [anon_sym_where] = ACTIONS(799),
    [anon_sym_COLON] = ACTIONS(801),
    [anon_sym_LBRACE] = ACTIONS(801),
    [anon_sym_RBRACE] = ACTIONS(801),
    [anon_sym_EQ_EQ_GT] = ACTIONS(801),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(801),
    [anon_sym_PIPE_PIPE] = ACTIONS(801),
    [anon_sym_AMP_AMP] = ACTIONS(801),
    [anon_sym_EQ_EQ] = ACTIONS(799),
    [anon_sym_BANG_EQ] = ACTIONS(801),
    [anon_sym_LT] = ACTIONS(799),
    [anon_sym_GT] = ACTIONS(799),
    [anon_sym_LT_EQ] = ACTIONS(799),
    [anon_sym_GT_EQ] = ACTIONS(801),
    [anon_sym_DOT_DOT] = ACTIONS(801),
    [anon_sym_CARET] = ACTIONS(801),
    [anon_sym_LT_LT] = ACTIONS(801),
    [anon_sym_GT_GT] = ACTIONS(801),
    [anon_sym_PLUS] = ACTIONS(801),
    [anon_sym_DASH] = ACTIONS(801),
    [anon_sym_SLASH] = ACTIONS(799),
    [anon_sym_PERCENT] = ACTIONS(801),
    [anon_sym_DOT] = ACTIONS(799),
    [anon_sym_LBRACK] = ACTIONS(801),
    [anon_sym_RBRACK] = ACTIONS(801),
    [anon_sym_as] = ACTIONS(799),
    [anon_sym_if] = ACTIONS(799),
    [anon_sym_EQ_GT] = ACTIONS(801),
    [anon_sym_else] = ACTIONS(799),
    [anon_sym_POUND] = ACTIONS(801),
    [anon_sym_spec] = ACTIONS(799),
    [anon_sym_invariant] = ACTIONS(799),
    [anon_sym_SEMI] = ACTIONS(801),
    [anon_sym_with] = ACTIONS(799),
    [anon_sym_to] = ACTIONS(799),
    [anon_sym_fun] = ACTIONS(799),
    [anon_sym_native] = ACTIONS(799),
    [anon_sym_public] = ACTIONS(799),
    [anon_sym_entry] = ACTIONS(799),
    [anon_sym_use] = ACTIONS(799),
    [anon_sym_const] = ACTIONS(799),
    [anon_sym_inline] = ACTIONS(799),
    [anon_sym_struct] = ACTIONS(799),
    [anon_sym_enum] = ACTIONS(799),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(858)] = {
    [sym_line_comment] = STATE(858),
    [sym_block_comment] = STATE(858),
    [sym_identifier] = ACTIONS(803),
    [anon_sym_u8] = ACTIONS(803),
    [anon_sym_u16] = ACTIONS(803),
    [anon_sym_u32] = ACTIONS(803),
    [anon_sym_u64] = ACTIONS(803),
    [anon_sym_u128] = ACTIONS(803),
    [anon_sym_u256] = ACTIONS(803),
    [anon_sym_exists] = ACTIONS(803),
    [anon_sym_forall] = ACTIONS(803),
    [anon_sym_choose] = ACTIONS(803),
    [anon_sym_min] = ACTIONS(803),
    [anon_sym_bool] = ACTIONS(803),
    [anon_sym_address] = ACTIONS(803),
    [anon_sym_vector] = ACTIONS(803),
    [anon_sym_for] = ACTIONS(803),
    [anon_sym_while] = ACTIONS(803),
    [anon_sym_friend] = ACTIONS(803),
    [anon_sym_match] = ACTIONS(803),
    [aux_sym_number_token1] = ACTIONS(803),
    [aux_sym_number_token2] = ACTIONS(805),
    [aux_sym_number_token4] = ACTIONS(805),
    [aux_sym_number_token6] = ACTIONS(805),
    [anon_sym_STAR] = ACTIONS(805),
    [anon_sym_LPAREN] = ACTIONS(805),
    [anon_sym_COMMA] = ACTIONS(805),
    [anon_sym_RPAREN] = ACTIONS(805),
    [anon_sym_PIPE] = ACTIONS(803),
    [anon_sym_AMP] = ACTIONS(803),
    [anon_sym_EQ] = ACTIONS(803),
    [anon_sym_where] = ACTIONS(803),
    [anon_sym_COLON] = ACTIONS(805),
    [anon_sym_LBRACE] = ACTIONS(805),
    [anon_sym_RBRACE] = ACTIONS(805),
    [anon_sym_EQ_EQ_GT] = ACTIONS(805),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(805),
    [anon_sym_PIPE_PIPE] = ACTIONS(805),
    [anon_sym_AMP_AMP] = ACTIONS(805),
    [anon_sym_EQ_EQ] = ACTIONS(803),
    [anon_sym_BANG_EQ] = ACTIONS(805),
    [anon_sym_LT] = ACTIONS(803),
    [anon_sym_GT] = ACTIONS(803),
    [anon_sym_LT_EQ] = ACTIONS(803),
    [anon_sym_GT_EQ] = ACTIONS(805),
    [anon_sym_DOT_DOT] = ACTIONS(805),
    [anon_sym_CARET] = ACTIONS(805),
    [anon_sym_LT_LT] = ACTIONS(805),
    [anon_sym_GT_GT] = ACTIONS(805),
    [anon_sym_PLUS] = ACTIONS(805),
    [anon_sym_DASH] = ACTIONS(805),
    [anon_sym_SLASH] = ACTIONS(803),
    [anon_sym_PERCENT] = ACTIONS(805),
    [anon_sym_DOT] = ACTIONS(803),
    [anon_sym_LBRACK] = ACTIONS(805),
    [anon_sym_RBRACK] = ACTIONS(805),
    [anon_sym_as] = ACTIONS(803),
    [anon_sym_if] = ACTIONS(803),
    [anon_sym_EQ_GT] = ACTIONS(805),
    [anon_sym_else] = ACTIONS(803),
    [anon_sym_POUND] = ACTIONS(805),
    [anon_sym_spec] = ACTIONS(803),
    [anon_sym_invariant] = ACTIONS(803),
    [anon_sym_SEMI] = ACTIONS(805),
    [anon_sym_with] = ACTIONS(803),
    [anon_sym_to] = ACTIONS(803),
    [anon_sym_fun] = ACTIONS(803),
    [anon_sym_native] = ACTIONS(803),
    [anon_sym_public] = ACTIONS(803),
    [anon_sym_entry] = ACTIONS(803),
    [anon_sym_use] = ACTIONS(803),
    [anon_sym_const] = ACTIONS(803),
    [anon_sym_inline] = ACTIONS(803),
    [anon_sym_struct] = ACTIONS(803),
    [anon_sym_enum] = ACTIONS(803),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(859)] = {
    [sym_line_comment] = STATE(859),
    [sym_block_comment] = STATE(859),
    [sym_identifier] = ACTIONS(807),
    [anon_sym_u8] = ACTIONS(807),
    [anon_sym_u16] = ACTIONS(807),
    [anon_sym_u32] = ACTIONS(807),
    [anon_sym_u64] = ACTIONS(807),
    [anon_sym_u128] = ACTIONS(807),
    [anon_sym_u256] = ACTIONS(807),
    [anon_sym_exists] = ACTIONS(807),
    [anon_sym_forall] = ACTIONS(807),
    [anon_sym_choose] = ACTIONS(807),
    [anon_sym_min] = ACTIONS(807),
    [anon_sym_bool] = ACTIONS(807),
    [anon_sym_address] = ACTIONS(807),
    [anon_sym_vector] = ACTIONS(807),
    [anon_sym_for] = ACTIONS(807),
    [anon_sym_while] = ACTIONS(807),
    [anon_sym_friend] = ACTIONS(807),
    [anon_sym_match] = ACTIONS(807),
    [aux_sym_number_token1] = ACTIONS(807),
    [aux_sym_number_token2] = ACTIONS(809),
    [aux_sym_number_token4] = ACTIONS(809),
    [aux_sym_number_token6] = ACTIONS(809),
    [anon_sym_STAR] = ACTIONS(809),
    [anon_sym_COLON_COLON] = ACTIONS(809),
    [anon_sym_LPAREN] = ACTIONS(809),
    [anon_sym_COMMA] = ACTIONS(809),
    [anon_sym_RPAREN] = ACTIONS(809),
    [anon_sym_PIPE] = ACTIONS(807),
    [anon_sym_AMP] = ACTIONS(807),
    [anon_sym_EQ] = ACTIONS(807),
    [anon_sym_where] = ACTIONS(807),
    [anon_sym_COLON] = ACTIONS(807),
    [anon_sym_in] = ACTIONS(807),
    [anon_sym_LBRACE] = ACTIONS(809),
    [anon_sym_RBRACE] = ACTIONS(809),
    [anon_sym_EQ_EQ_GT] = ACTIONS(809),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(809),
    [anon_sym_PIPE_PIPE] = ACTIONS(809),
    [anon_sym_AMP_AMP] = ACTIONS(809),
    [anon_sym_EQ_EQ] = ACTIONS(807),
    [anon_sym_BANG_EQ] = ACTIONS(809),
    [anon_sym_LT] = ACTIONS(807),
    [anon_sym_GT] = ACTIONS(807),
    [anon_sym_LT_EQ] = ACTIONS(807),
    [anon_sym_GT_EQ] = ACTIONS(809),
    [anon_sym_DOT_DOT] = ACTIONS(809),
    [anon_sym_CARET] = ACTIONS(809),
    [anon_sym_LT_LT] = ACTIONS(809),
    [anon_sym_GT_GT] = ACTIONS(809),
    [anon_sym_PLUS] = ACTIONS(809),
    [anon_sym_DASH] = ACTIONS(809),
    [anon_sym_SLASH] = ACTIONS(807),
    [anon_sym_PERCENT] = ACTIONS(809),
    [anon_sym_BANG] = ACTIONS(807),
    [anon_sym_DOT] = ACTIONS(807),
    [anon_sym_LBRACK] = ACTIONS(809),
    [anon_sym_RBRACK] = ACTIONS(809),
    [anon_sym_as] = ACTIONS(807),
    [anon_sym_if] = ACTIONS(807),
    [anon_sym_EQ_GT] = ACTIONS(809),
    [anon_sym_else] = ACTIONS(807),
    [anon_sym_LT2] = ACTIONS(807),
    [anon_sym_spec] = ACTIONS(807),
    [anon_sym_SEMI] = ACTIONS(809),
    [anon_sym_with] = ACTIONS(807),
    [anon_sym_to] = ACTIONS(807),
    [anon_sym_pure] = ACTIONS(807),
    [anon_sym_acquires] = ACTIONS(807),
    [anon_sym_reads] = ACTIONS(807),
    [anon_sym_writes] = ACTIONS(807),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(860)] = {
    [sym_line_comment] = STATE(860),
    [sym_block_comment] = STATE(860),
    [sym_identifier] = ACTIONS(811),
    [anon_sym_u8] = ACTIONS(811),
    [anon_sym_u16] = ACTIONS(811),
    [anon_sym_u32] = ACTIONS(811),
    [anon_sym_u64] = ACTIONS(811),
    [anon_sym_u128] = ACTIONS(811),
    [anon_sym_u256] = ACTIONS(811),
    [anon_sym_exists] = ACTIONS(811),
    [anon_sym_forall] = ACTIONS(811),
    [anon_sym_choose] = ACTIONS(811),
    [anon_sym_min] = ACTIONS(811),
    [anon_sym_bool] = ACTIONS(811),
    [anon_sym_address] = ACTIONS(811),
    [anon_sym_vector] = ACTIONS(811),
    [anon_sym_for] = ACTIONS(811),
    [anon_sym_while] = ACTIONS(811),
    [anon_sym_friend] = ACTIONS(811),
    [anon_sym_match] = ACTIONS(811),
    [aux_sym_number_token1] = ACTIONS(811),
    [aux_sym_number_token2] = ACTIONS(813),
    [aux_sym_number_token4] = ACTIONS(813),
    [aux_sym_number_token6] = ACTIONS(813),
    [anon_sym_STAR] = ACTIONS(813),
    [anon_sym_COLON_COLON] = ACTIONS(813),
    [anon_sym_LPAREN] = ACTIONS(813),
    [anon_sym_COMMA] = ACTIONS(813),
    [anon_sym_RPAREN] = ACTIONS(813),
    [anon_sym_PIPE] = ACTIONS(811),
    [anon_sym_AMP] = ACTIONS(811),
    [anon_sym_EQ] = ACTIONS(811),
    [anon_sym_where] = ACTIONS(811),
    [anon_sym_COLON] = ACTIONS(811),
    [anon_sym_in] = ACTIONS(811),
    [anon_sym_LBRACE] = ACTIONS(813),
    [anon_sym_RBRACE] = ACTIONS(813),
    [anon_sym_EQ_EQ_GT] = ACTIONS(813),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(813),
    [anon_sym_PIPE_PIPE] = ACTIONS(813),
    [anon_sym_AMP_AMP] = ACTIONS(813),
    [anon_sym_EQ_EQ] = ACTIONS(811),
    [anon_sym_BANG_EQ] = ACTIONS(813),
    [anon_sym_LT] = ACTIONS(811),
    [anon_sym_GT] = ACTIONS(811),
    [anon_sym_LT_EQ] = ACTIONS(811),
    [anon_sym_GT_EQ] = ACTIONS(813),
    [anon_sym_DOT_DOT] = ACTIONS(813),
    [anon_sym_CARET] = ACTIONS(813),
    [anon_sym_LT_LT] = ACTIONS(813),
    [anon_sym_GT_GT] = ACTIONS(813),
    [anon_sym_PLUS] = ACTIONS(813),
    [anon_sym_DASH] = ACTIONS(813),
    [anon_sym_SLASH] = ACTIONS(811),
    [anon_sym_PERCENT] = ACTIONS(813),
    [anon_sym_BANG] = ACTIONS(811),
    [anon_sym_DOT] = ACTIONS(811),
    [anon_sym_LBRACK] = ACTIONS(813),
    [anon_sym_RBRACK] = ACTIONS(813),
    [anon_sym_as] = ACTIONS(811),
    [anon_sym_if] = ACTIONS(811),
    [anon_sym_EQ_GT] = ACTIONS(813),
    [anon_sym_else] = ACTIONS(811),
    [anon_sym_LT2] = ACTIONS(811),
    [anon_sym_spec] = ACTIONS(811),
    [anon_sym_SEMI] = ACTIONS(813),
    [anon_sym_with] = ACTIONS(811),
    [anon_sym_to] = ACTIONS(811),
    [anon_sym_pure] = ACTIONS(811),
    [anon_sym_acquires] = ACTIONS(811),
    [anon_sym_reads] = ACTIONS(811),
    [anon_sym_writes] = ACTIONS(811),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(861)] = {
    [sym_line_comment] = STATE(861),
    [sym_block_comment] = STATE(861),
    [sym_identifier] = ACTIONS(815),
    [anon_sym_u8] = ACTIONS(815),
    [anon_sym_u16] = ACTIONS(815),
    [anon_sym_u32] = ACTIONS(815),
    [anon_sym_u64] = ACTIONS(815),
    [anon_sym_u128] = ACTIONS(815),
    [anon_sym_u256] = ACTIONS(815),
    [anon_sym_exists] = ACTIONS(815),
    [anon_sym_forall] = ACTIONS(815),
    [anon_sym_choose] = ACTIONS(815),
    [anon_sym_min] = ACTIONS(815),
    [anon_sym_bool] = ACTIONS(815),
    [anon_sym_address] = ACTIONS(815),
    [anon_sym_vector] = ACTIONS(815),
    [anon_sym_for] = ACTIONS(815),
    [anon_sym_while] = ACTIONS(815),
    [anon_sym_friend] = ACTIONS(815),
    [anon_sym_match] = ACTIONS(815),
    [aux_sym_number_token1] = ACTIONS(815),
    [aux_sym_number_token2] = ACTIONS(817),
    [aux_sym_number_token4] = ACTIONS(817),
    [aux_sym_number_token6] = ACTIONS(817),
    [anon_sym_STAR] = ACTIONS(817),
    [anon_sym_COLON_COLON] = ACTIONS(817),
    [anon_sym_LPAREN] = ACTIONS(817),
    [anon_sym_COMMA] = ACTIONS(817),
    [anon_sym_RPAREN] = ACTIONS(817),
    [anon_sym_PIPE] = ACTIONS(815),
    [anon_sym_AMP] = ACTIONS(815),
    [anon_sym_EQ] = ACTIONS(815),
    [anon_sym_where] = ACTIONS(815),
    [anon_sym_COLON] = ACTIONS(815),
    [anon_sym_in] = ACTIONS(815),
    [anon_sym_LBRACE] = ACTIONS(817),
    [anon_sym_RBRACE] = ACTIONS(817),
    [anon_sym_EQ_EQ_GT] = ACTIONS(817),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(817),
    [anon_sym_PIPE_PIPE] = ACTIONS(817),
    [anon_sym_AMP_AMP] = ACTIONS(817),
    [anon_sym_EQ_EQ] = ACTIONS(815),
    [anon_sym_BANG_EQ] = ACTIONS(817),
    [anon_sym_LT] = ACTIONS(815),
    [anon_sym_GT] = ACTIONS(815),
    [anon_sym_LT_EQ] = ACTIONS(815),
    [anon_sym_GT_EQ] = ACTIONS(817),
    [anon_sym_DOT_DOT] = ACTIONS(817),
    [anon_sym_CARET] = ACTIONS(817),
    [anon_sym_LT_LT] = ACTIONS(817),
    [anon_sym_GT_GT] = ACTIONS(817),
    [anon_sym_PLUS] = ACTIONS(817),
    [anon_sym_DASH] = ACTIONS(817),
    [anon_sym_SLASH] = ACTIONS(815),
    [anon_sym_PERCENT] = ACTIONS(817),
    [anon_sym_BANG] = ACTIONS(815),
    [anon_sym_DOT] = ACTIONS(815),
    [anon_sym_LBRACK] = ACTIONS(817),
    [anon_sym_RBRACK] = ACTIONS(817),
    [anon_sym_as] = ACTIONS(815),
    [anon_sym_if] = ACTIONS(815),
    [anon_sym_EQ_GT] = ACTIONS(817),
    [anon_sym_else] = ACTIONS(815),
    [anon_sym_LT2] = ACTIONS(815),
    [anon_sym_spec] = ACTIONS(815),
    [anon_sym_SEMI] = ACTIONS(817),
    [anon_sym_with] = ACTIONS(815),
    [anon_sym_to] = ACTIONS(815),
    [anon_sym_pure] = ACTIONS(815),
    [anon_sym_acquires] = ACTIONS(815),
    [anon_sym_reads] = ACTIONS(815),
    [anon_sym_writes] = ACTIONS(815),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(862)] = {
    [sym_line_comment] = STATE(862),
    [sym_block_comment] = STATE(862),
    [sym_identifier] = ACTIONS(819),
    [anon_sym_u8] = ACTIONS(819),
    [anon_sym_u16] = ACTIONS(819),
    [anon_sym_u32] = ACTIONS(819),
    [anon_sym_u64] = ACTIONS(819),
    [anon_sym_u128] = ACTIONS(819),
    [anon_sym_u256] = ACTIONS(819),
    [anon_sym_exists] = ACTIONS(819),
    [anon_sym_forall] = ACTIONS(819),
    [anon_sym_choose] = ACTIONS(819),
    [anon_sym_min] = ACTIONS(819),
    [anon_sym_bool] = ACTIONS(819),
    [anon_sym_address] = ACTIONS(819),
    [anon_sym_vector] = ACTIONS(819),
    [anon_sym_for] = ACTIONS(819),
    [anon_sym_while] = ACTIONS(819),
    [anon_sym_friend] = ACTIONS(819),
    [anon_sym_match] = ACTIONS(819),
    [aux_sym_number_token1] = ACTIONS(819),
    [aux_sym_number_token2] = ACTIONS(821),
    [aux_sym_number_token4] = ACTIONS(821),
    [aux_sym_number_token6] = ACTIONS(821),
    [anon_sym_STAR] = ACTIONS(821),
    [anon_sym_COLON_COLON] = ACTIONS(821),
    [anon_sym_LPAREN] = ACTIONS(821),
    [anon_sym_COMMA] = ACTIONS(821),
    [anon_sym_RPAREN] = ACTIONS(821),
    [anon_sym_PIPE] = ACTIONS(819),
    [anon_sym_AMP] = ACTIONS(819),
    [anon_sym_EQ] = ACTIONS(819),
    [anon_sym_where] = ACTIONS(819),
    [anon_sym_COLON] = ACTIONS(819),
    [anon_sym_in] = ACTIONS(819),
    [anon_sym_LBRACE] = ACTIONS(821),
    [anon_sym_RBRACE] = ACTIONS(821),
    [anon_sym_EQ_EQ_GT] = ACTIONS(821),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(821),
    [anon_sym_PIPE_PIPE] = ACTIONS(821),
    [anon_sym_AMP_AMP] = ACTIONS(821),
    [anon_sym_EQ_EQ] = ACTIONS(819),
    [anon_sym_BANG_EQ] = ACTIONS(821),
    [anon_sym_LT] = ACTIONS(819),
    [anon_sym_GT] = ACTIONS(819),
    [anon_sym_LT_EQ] = ACTIONS(819),
    [anon_sym_GT_EQ] = ACTIONS(821),
    [anon_sym_DOT_DOT] = ACTIONS(821),
    [anon_sym_CARET] = ACTIONS(821),
    [anon_sym_LT_LT] = ACTIONS(821),
    [anon_sym_GT_GT] = ACTIONS(821),
    [anon_sym_PLUS] = ACTIONS(821),
    [anon_sym_DASH] = ACTIONS(821),
    [anon_sym_SLASH] = ACTIONS(819),
    [anon_sym_PERCENT] = ACTIONS(821),
    [anon_sym_BANG] = ACTIONS(819),
    [anon_sym_DOT] = ACTIONS(819),
    [anon_sym_LBRACK] = ACTIONS(821),
    [anon_sym_RBRACK] = ACTIONS(821),
    [anon_sym_as] = ACTIONS(819),
    [anon_sym_if] = ACTIONS(819),
    [anon_sym_EQ_GT] = ACTIONS(821),
    [anon_sym_else] = ACTIONS(819),
    [anon_sym_LT2] = ACTIONS(819),
    [anon_sym_spec] = ACTIONS(819),
    [anon_sym_SEMI] = ACTIONS(821),
    [anon_sym_with] = ACTIONS(819),
    [anon_sym_to] = ACTIONS(819),
    [anon_sym_pure] = ACTIONS(819),
    [anon_sym_acquires] = ACTIONS(819),
    [anon_sym_reads] = ACTIONS(819),
    [anon_sym_writes] = ACTIONS(819),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(863)] = {
    [sym_line_comment] = STATE(863),
    [sym_block_comment] = STATE(863),
    [sym_identifier] = ACTIONS(823),
    [anon_sym_u8] = ACTIONS(823),
    [anon_sym_u16] = ACTIONS(823),
    [anon_sym_u32] = ACTIONS(823),
    [anon_sym_u64] = ACTIONS(823),
    [anon_sym_u128] = ACTIONS(823),
    [anon_sym_u256] = ACTIONS(823),
    [anon_sym_exists] = ACTIONS(823),
    [anon_sym_forall] = ACTIONS(823),
    [anon_sym_choose] = ACTIONS(823),
    [anon_sym_min] = ACTIONS(823),
    [anon_sym_bool] = ACTIONS(823),
    [anon_sym_address] = ACTIONS(823),
    [anon_sym_vector] = ACTIONS(823),
    [anon_sym_for] = ACTIONS(823),
    [anon_sym_while] = ACTIONS(823),
    [anon_sym_friend] = ACTIONS(823),
    [anon_sym_match] = ACTIONS(823),
    [aux_sym_number_token1] = ACTIONS(823),
    [aux_sym_number_token2] = ACTIONS(825),
    [aux_sym_number_token4] = ACTIONS(825),
    [aux_sym_number_token6] = ACTIONS(825),
    [anon_sym_STAR] = ACTIONS(825),
    [anon_sym_COLON_COLON] = ACTIONS(825),
    [anon_sym_LPAREN] = ACTIONS(825),
    [anon_sym_COMMA] = ACTIONS(825),
    [anon_sym_RPAREN] = ACTIONS(825),
    [anon_sym_PIPE] = ACTIONS(823),
    [anon_sym_AMP] = ACTIONS(823),
    [anon_sym_EQ] = ACTIONS(823),
    [anon_sym_where] = ACTIONS(823),
    [anon_sym_COLON] = ACTIONS(823),
    [anon_sym_in] = ACTIONS(823),
    [anon_sym_LBRACE] = ACTIONS(825),
    [anon_sym_RBRACE] = ACTIONS(825),
    [anon_sym_EQ_EQ_GT] = ACTIONS(825),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(825),
    [anon_sym_PIPE_PIPE] = ACTIONS(825),
    [anon_sym_AMP_AMP] = ACTIONS(825),
    [anon_sym_EQ_EQ] = ACTIONS(823),
    [anon_sym_BANG_EQ] = ACTIONS(825),
    [anon_sym_LT] = ACTIONS(823),
    [anon_sym_GT] = ACTIONS(823),
    [anon_sym_LT_EQ] = ACTIONS(823),
    [anon_sym_GT_EQ] = ACTIONS(825),
    [anon_sym_DOT_DOT] = ACTIONS(825),
    [anon_sym_CARET] = ACTIONS(825),
    [anon_sym_LT_LT] = ACTIONS(825),
    [anon_sym_GT_GT] = ACTIONS(825),
    [anon_sym_PLUS] = ACTIONS(825),
    [anon_sym_DASH] = ACTIONS(825),
    [anon_sym_SLASH] = ACTIONS(823),
    [anon_sym_PERCENT] = ACTIONS(825),
    [anon_sym_BANG] = ACTIONS(823),
    [anon_sym_DOT] = ACTIONS(823),
    [anon_sym_LBRACK] = ACTIONS(825),
    [anon_sym_RBRACK] = ACTIONS(825),
    [anon_sym_as] = ACTIONS(823),
    [anon_sym_if] = ACTIONS(823),
    [anon_sym_EQ_GT] = ACTIONS(825),
    [anon_sym_else] = ACTIONS(823),
    [anon_sym_LT2] = ACTIONS(823),
    [anon_sym_spec] = ACTIONS(823),
    [anon_sym_SEMI] = ACTIONS(825),
    [anon_sym_with] = ACTIONS(823),
    [anon_sym_to] = ACTIONS(823),
    [anon_sym_pure] = ACTIONS(823),
    [anon_sym_acquires] = ACTIONS(823),
    [anon_sym_reads] = ACTIONS(823),
    [anon_sym_writes] = ACTIONS(823),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(864)] = {
    [sym_line_comment] = STATE(864),
    [sym_block_comment] = STATE(864),
    [sym_identifier] = ACTIONS(827),
    [anon_sym_u8] = ACTIONS(827),
    [anon_sym_u16] = ACTIONS(827),
    [anon_sym_u32] = ACTIONS(827),
    [anon_sym_u64] = ACTIONS(827),
    [anon_sym_u128] = ACTIONS(827),
    [anon_sym_u256] = ACTIONS(827),
    [anon_sym_exists] = ACTIONS(827),
    [anon_sym_forall] = ACTIONS(827),
    [anon_sym_choose] = ACTIONS(827),
    [anon_sym_min] = ACTIONS(827),
    [anon_sym_bool] = ACTIONS(827),
    [anon_sym_address] = ACTIONS(827),
    [anon_sym_vector] = ACTIONS(827),
    [anon_sym_for] = ACTIONS(827),
    [anon_sym_while] = ACTIONS(827),
    [anon_sym_friend] = ACTIONS(827),
    [anon_sym_match] = ACTIONS(827),
    [aux_sym_number_token1] = ACTIONS(827),
    [aux_sym_number_token2] = ACTIONS(829),
    [aux_sym_number_token4] = ACTIONS(829),
    [aux_sym_number_token6] = ACTIONS(829),
    [anon_sym_STAR] = ACTIONS(829),
    [anon_sym_COLON_COLON] = ACTIONS(831),
    [anon_sym_LPAREN] = ACTIONS(829),
    [anon_sym_COMMA] = ACTIONS(829),
    [anon_sym_RPAREN] = ACTIONS(829),
    [anon_sym_PIPE] = ACTIONS(827),
    [anon_sym_AMP] = ACTIONS(827),
    [anon_sym_EQ] = ACTIONS(827),
    [anon_sym_where] = ACTIONS(827),
    [anon_sym_COLON] = ACTIONS(827),
    [anon_sym_LBRACE] = ACTIONS(829),
    [anon_sym_RBRACE] = ACTIONS(829),
    [anon_sym_EQ_EQ_GT] = ACTIONS(829),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(829),
    [anon_sym_PIPE_PIPE] = ACTIONS(829),
    [anon_sym_AMP_AMP] = ACTIONS(829),
    [anon_sym_EQ_EQ] = ACTIONS(827),
    [anon_sym_BANG_EQ] = ACTIONS(829),
    [anon_sym_LT] = ACTIONS(827),
    [anon_sym_GT] = ACTIONS(827),
    [anon_sym_LT_EQ] = ACTIONS(827),
    [anon_sym_GT_EQ] = ACTIONS(829),
    [anon_sym_DOT_DOT] = ACTIONS(829),
    [anon_sym_CARET] = ACTIONS(829),
    [anon_sym_LT_LT] = ACTIONS(829),
    [anon_sym_GT_GT] = ACTIONS(829),
    [anon_sym_PLUS] = ACTIONS(829),
    [anon_sym_DASH] = ACTIONS(829),
    [anon_sym_SLASH] = ACTIONS(827),
    [anon_sym_PERCENT] = ACTIONS(829),
    [anon_sym_BANG] = ACTIONS(827),
    [anon_sym_DOT] = ACTIONS(827),
    [anon_sym_LBRACK] = ACTIONS(829),
    [anon_sym_RBRACK] = ACTIONS(829),
    [anon_sym_as] = ACTIONS(827),
    [anon_sym_if] = ACTIONS(827),
    [anon_sym_EQ_GT] = ACTIONS(829),
    [anon_sym_else] = ACTIONS(827),
    [anon_sym_LT2] = ACTIONS(827),
    [anon_sym_spec] = ACTIONS(827),
    [anon_sym_SEMI] = ACTIONS(829),
    [anon_sym_with] = ACTIONS(827),
    [anon_sym_to] = ACTIONS(827),
    [anon_sym_pure] = ACTIONS(827),
    [anon_sym_acquires] = ACTIONS(827),
    [anon_sym_reads] = ACTIONS(827),
    [anon_sym_writes] = ACTIONS(827),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(865)] = {
    [sym_line_comment] = STATE(865),
    [sym_block_comment] = STATE(865),
    [sym_identifier] = ACTIONS(833),
    [anon_sym_u8] = ACTIONS(833),
    [anon_sym_u16] = ACTIONS(833),
    [anon_sym_u32] = ACTIONS(833),
    [anon_sym_u64] = ACTIONS(833),
    [anon_sym_u128] = ACTIONS(833),
    [anon_sym_u256] = ACTIONS(833),
    [anon_sym_exists] = ACTIONS(833),
    [anon_sym_forall] = ACTIONS(833),
    [anon_sym_choose] = ACTIONS(833),
    [anon_sym_min] = ACTIONS(833),
    [anon_sym_bool] = ACTIONS(833),
    [anon_sym_address] = ACTIONS(833),
    [anon_sym_vector] = ACTIONS(833),
    [anon_sym_for] = ACTIONS(833),
    [anon_sym_while] = ACTIONS(833),
    [anon_sym_friend] = ACTIONS(833),
    [anon_sym_match] = ACTIONS(833),
    [aux_sym_number_token1] = ACTIONS(833),
    [aux_sym_number_token2] = ACTIONS(835),
    [aux_sym_number_token4] = ACTIONS(835),
    [aux_sym_number_token6] = ACTIONS(835),
    [anon_sym_STAR] = ACTIONS(835),
    [anon_sym_COLON_COLON] = ACTIONS(837),
    [anon_sym_LPAREN] = ACTIONS(835),
    [anon_sym_COMMA] = ACTIONS(835),
    [anon_sym_RPAREN] = ACTIONS(835),
    [anon_sym_PIPE] = ACTIONS(833),
    [anon_sym_AMP] = ACTIONS(833),
    [anon_sym_EQ] = ACTIONS(833),
    [anon_sym_where] = ACTIONS(833),
    [anon_sym_COLON] = ACTIONS(833),
    [anon_sym_LBRACE] = ACTIONS(835),
    [anon_sym_RBRACE] = ACTIONS(835),
    [anon_sym_EQ_EQ_GT] = ACTIONS(835),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(835),
    [anon_sym_PIPE_PIPE] = ACTIONS(835),
    [anon_sym_AMP_AMP] = ACTIONS(835),
    [anon_sym_EQ_EQ] = ACTIONS(833),
    [anon_sym_BANG_EQ] = ACTIONS(835),
    [anon_sym_LT] = ACTIONS(833),
    [anon_sym_GT] = ACTIONS(833),
    [anon_sym_LT_EQ] = ACTIONS(833),
    [anon_sym_GT_EQ] = ACTIONS(835),
    [anon_sym_DOT_DOT] = ACTIONS(835),
    [anon_sym_CARET] = ACTIONS(835),
    [anon_sym_LT_LT] = ACTIONS(835),
    [anon_sym_GT_GT] = ACTIONS(835),
    [anon_sym_PLUS] = ACTIONS(835),
    [anon_sym_DASH] = ACTIONS(835),
    [anon_sym_SLASH] = ACTIONS(833),
    [anon_sym_PERCENT] = ACTIONS(835),
    [anon_sym_BANG] = ACTIONS(833),
    [anon_sym_DOT] = ACTIONS(833),
    [anon_sym_LBRACK] = ACTIONS(835),
    [anon_sym_RBRACK] = ACTIONS(835),
    [anon_sym_as] = ACTIONS(833),
    [anon_sym_if] = ACTIONS(833),
    [anon_sym_EQ_GT] = ACTIONS(835),
    [anon_sym_else] = ACTIONS(833),
    [anon_sym_LT2] = ACTIONS(833),
    [anon_sym_spec] = ACTIONS(833),
    [anon_sym_SEMI] = ACTIONS(835),
    [anon_sym_with] = ACTIONS(833),
    [anon_sym_to] = ACTIONS(833),
    [anon_sym_pure] = ACTIONS(833),
    [anon_sym_acquires] = ACTIONS(833),
    [anon_sym_reads] = ACTIONS(833),
    [anon_sym_writes] = ACTIONS(833),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(866)] = {
    [sym_line_comment] = STATE(866),
    [sym_block_comment] = STATE(866),
    [sym_identifier] = ACTIONS(827),
    [anon_sym_u8] = ACTIONS(827),
    [anon_sym_u16] = ACTIONS(827),
    [anon_sym_u32] = ACTIONS(827),
    [anon_sym_u64] = ACTIONS(827),
    [anon_sym_u128] = ACTIONS(827),
    [anon_sym_u256] = ACTIONS(827),
    [anon_sym_exists] = ACTIONS(827),
    [anon_sym_forall] = ACTIONS(827),
    [anon_sym_choose] = ACTIONS(827),
    [anon_sym_min] = ACTIONS(827),
    [anon_sym_bool] = ACTIONS(827),
    [anon_sym_address] = ACTIONS(827),
    [anon_sym_vector] = ACTIONS(827),
    [anon_sym_for] = ACTIONS(827),
    [anon_sym_while] = ACTIONS(827),
    [anon_sym_friend] = ACTIONS(827),
    [anon_sym_match] = ACTIONS(827),
    [aux_sym_number_token1] = ACTIONS(827),
    [aux_sym_number_token2] = ACTIONS(829),
    [aux_sym_number_token4] = ACTIONS(829),
    [aux_sym_number_token6] = ACTIONS(829),
    [anon_sym_STAR] = ACTIONS(829),
    [anon_sym_COLON_COLON] = ACTIONS(839),
    [anon_sym_LPAREN] = ACTIONS(829),
    [anon_sym_COMMA] = ACTIONS(829),
    [anon_sym_RPAREN] = ACTIONS(829),
    [anon_sym_PIPE] = ACTIONS(827),
    [anon_sym_AMP] = ACTIONS(827),
    [anon_sym_EQ] = ACTIONS(827),
    [anon_sym_where] = ACTIONS(827),
    [anon_sym_COLON] = ACTIONS(827),
    [anon_sym_LBRACE] = ACTIONS(829),
    [anon_sym_RBRACE] = ACTIONS(829),
    [anon_sym_EQ_EQ_GT] = ACTIONS(829),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(829),
    [anon_sym_PIPE_PIPE] = ACTIONS(829),
    [anon_sym_AMP_AMP] = ACTIONS(829),
    [anon_sym_EQ_EQ] = ACTIONS(827),
    [anon_sym_BANG_EQ] = ACTIONS(829),
    [anon_sym_LT] = ACTIONS(827),
    [anon_sym_GT] = ACTIONS(827),
    [anon_sym_LT_EQ] = ACTIONS(827),
    [anon_sym_GT_EQ] = ACTIONS(829),
    [anon_sym_DOT_DOT] = ACTIONS(829),
    [anon_sym_CARET] = ACTIONS(829),
    [anon_sym_LT_LT] = ACTIONS(829),
    [anon_sym_GT_GT] = ACTIONS(829),
    [anon_sym_PLUS] = ACTIONS(829),
    [anon_sym_DASH] = ACTIONS(829),
    [anon_sym_SLASH] = ACTIONS(827),
    [anon_sym_PERCENT] = ACTIONS(829),
    [anon_sym_BANG] = ACTIONS(827),
    [anon_sym_DOT] = ACTIONS(827),
    [anon_sym_LBRACK] = ACTIONS(829),
    [anon_sym_RBRACK] = ACTIONS(829),
    [anon_sym_as] = ACTIONS(827),
    [anon_sym_if] = ACTIONS(827),
    [anon_sym_EQ_GT] = ACTIONS(829),
    [anon_sym_else] = ACTIONS(827),
    [anon_sym_LT2] = ACTIONS(827),
    [anon_sym_spec] = ACTIONS(827),
    [anon_sym_SEMI] = ACTIONS(829),
    [anon_sym_with] = ACTIONS(827),
    [anon_sym_to] = ACTIONS(827),
    [anon_sym_pure] = ACTIONS(827),
    [anon_sym_acquires] = ACTIONS(827),
    [anon_sym_reads] = ACTIONS(827),
    [anon_sym_writes] = ACTIONS(827),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(867)] = {
    [sym_line_comment] = STATE(867),
    [sym_block_comment] = STATE(867),
    [sym_identifier] = ACTIONS(841),
    [anon_sym_u8] = ACTIONS(841),
    [anon_sym_u16] = ACTIONS(841),
    [anon_sym_u32] = ACTIONS(841),
    [anon_sym_u64] = ACTIONS(841),
    [anon_sym_u128] = ACTIONS(841),
    [anon_sym_u256] = ACTIONS(841),
    [anon_sym_exists] = ACTIONS(841),
    [anon_sym_forall] = ACTIONS(841),
    [anon_sym_choose] = ACTIONS(841),
    [anon_sym_min] = ACTIONS(841),
    [anon_sym_bool] = ACTIONS(841),
    [anon_sym_address] = ACTIONS(841),
    [anon_sym_vector] = ACTIONS(841),
    [anon_sym_for] = ACTIONS(841),
    [anon_sym_while] = ACTIONS(841),
    [anon_sym_friend] = ACTIONS(841),
    [anon_sym_match] = ACTIONS(841),
    [aux_sym_number_token1] = ACTIONS(841),
    [aux_sym_number_token2] = ACTIONS(843),
    [aux_sym_number_token4] = ACTIONS(843),
    [aux_sym_number_token6] = ACTIONS(843),
    [anon_sym_STAR] = ACTIONS(843),
    [anon_sym_COLON_COLON] = ACTIONS(845),
    [anon_sym_LPAREN] = ACTIONS(843),
    [anon_sym_COMMA] = ACTIONS(843),
    [anon_sym_RPAREN] = ACTIONS(843),
    [anon_sym_PIPE] = ACTIONS(841),
    [anon_sym_AMP] = ACTIONS(841),
    [anon_sym_EQ] = ACTIONS(841),
    [anon_sym_where] = ACTIONS(841),
    [anon_sym_COLON] = ACTIONS(841),
    [anon_sym_LBRACE] = ACTIONS(843),
    [anon_sym_RBRACE] = ACTIONS(843),
    [anon_sym_EQ_EQ_GT] = ACTIONS(843),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(843),
    [anon_sym_PIPE_PIPE] = ACTIONS(843),
    [anon_sym_AMP_AMP] = ACTIONS(843),
    [anon_sym_EQ_EQ] = ACTIONS(841),
    [anon_sym_BANG_EQ] = ACTIONS(843),
    [anon_sym_LT] = ACTIONS(841),
    [anon_sym_GT] = ACTIONS(841),
    [anon_sym_LT_EQ] = ACTIONS(841),
    [anon_sym_GT_EQ] = ACTIONS(843),
    [anon_sym_DOT_DOT] = ACTIONS(843),
    [anon_sym_CARET] = ACTIONS(843),
    [anon_sym_LT_LT] = ACTIONS(843),
    [anon_sym_GT_GT] = ACTIONS(843),
    [anon_sym_PLUS] = ACTIONS(843),
    [anon_sym_DASH] = ACTIONS(843),
    [anon_sym_SLASH] = ACTIONS(841),
    [anon_sym_PERCENT] = ACTIONS(843),
    [anon_sym_BANG] = ACTIONS(841),
    [anon_sym_DOT] = ACTIONS(841),
    [anon_sym_LBRACK] = ACTIONS(843),
    [anon_sym_RBRACK] = ACTIONS(843),
    [anon_sym_as] = ACTIONS(841),
    [anon_sym_if] = ACTIONS(841),
    [anon_sym_EQ_GT] = ACTIONS(843),
    [anon_sym_else] = ACTIONS(841),
    [anon_sym_LT2] = ACTIONS(841),
    [anon_sym_spec] = ACTIONS(841),
    [anon_sym_SEMI] = ACTIONS(843),
    [anon_sym_with] = ACTIONS(841),
    [anon_sym_to] = ACTIONS(841),
    [anon_sym_pure] = ACTIONS(841),
    [anon_sym_acquires] = ACTIONS(841),
    [anon_sym_reads] = ACTIONS(841),
    [anon_sym_writes] = ACTIONS(841),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(868)] = {
    [sym_line_comment] = STATE(868),
    [sym_block_comment] = STATE(868),
    [sym_identifier] = ACTIONS(847),
    [anon_sym_u8] = ACTIONS(847),
    [anon_sym_u16] = ACTIONS(847),
    [anon_sym_u32] = ACTIONS(847),
    [anon_sym_u64] = ACTIONS(847),
    [anon_sym_u128] = ACTIONS(847),
    [anon_sym_u256] = ACTIONS(847),
    [anon_sym_exists] = ACTIONS(847),
    [anon_sym_forall] = ACTIONS(847),
    [anon_sym_choose] = ACTIONS(847),
    [anon_sym_min] = ACTIONS(847),
    [anon_sym_bool] = ACTIONS(847),
    [anon_sym_address] = ACTIONS(847),
    [anon_sym_vector] = ACTIONS(847),
    [anon_sym_for] = ACTIONS(847),
    [anon_sym_while] = ACTIONS(847),
    [anon_sym_friend] = ACTIONS(847),
    [anon_sym_match] = ACTIONS(847),
    [aux_sym_number_token1] = ACTIONS(847),
    [aux_sym_number_token2] = ACTIONS(849),
    [aux_sym_number_token4] = ACTIONS(849),
    [aux_sym_number_token6] = ACTIONS(849),
    [anon_sym_STAR] = ACTIONS(849),
    [anon_sym_LPAREN] = ACTIONS(849),
    [anon_sym_COMMA] = ACTIONS(849),
    [anon_sym_RPAREN] = ACTIONS(849),
    [anon_sym_PIPE] = ACTIONS(847),
    [anon_sym_AMP] = ACTIONS(847),
    [anon_sym_EQ] = ACTIONS(847),
    [anon_sym_where] = ACTIONS(847),
    [anon_sym_COLON] = ACTIONS(849),
    [anon_sym_LBRACE] = ACTIONS(849),
    [anon_sym_RBRACE] = ACTIONS(849),
    [anon_sym_EQ_EQ_GT] = ACTIONS(849),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(849),
    [anon_sym_PIPE_PIPE] = ACTIONS(849),
    [anon_sym_AMP_AMP] = ACTIONS(849),
    [anon_sym_EQ_EQ] = ACTIONS(847),
    [anon_sym_BANG_EQ] = ACTIONS(849),
    [anon_sym_LT] = ACTIONS(847),
    [anon_sym_GT] = ACTIONS(847),
    [anon_sym_LT_EQ] = ACTIONS(847),
    [anon_sym_GT_EQ] = ACTIONS(849),
    [anon_sym_DOT_DOT] = ACTIONS(849),
    [anon_sym_CARET] = ACTIONS(849),
    [anon_sym_LT_LT] = ACTIONS(849),
    [anon_sym_GT_GT] = ACTIONS(849),
    [anon_sym_PLUS] = ACTIONS(849),
    [anon_sym_DASH] = ACTIONS(849),
    [anon_sym_SLASH] = ACTIONS(847),
    [anon_sym_PERCENT] = ACTIONS(849),
    [anon_sym_BANG] = ACTIONS(847),
    [anon_sym_DOT] = ACTIONS(847),
    [anon_sym_LBRACK] = ACTIONS(849),
    [anon_sym_RBRACK] = ACTIONS(849),
    [anon_sym_as] = ACTIONS(847),
    [anon_sym_if] = ACTIONS(847),
    [anon_sym_EQ_GT] = ACTIONS(849),
    [anon_sym_else] = ACTIONS(847),
    [anon_sym_LT2] = ACTIONS(847),
    [anon_sym_spec] = ACTIONS(847),
    [anon_sym_SEMI] = ACTIONS(849),
    [anon_sym_with] = ACTIONS(847),
    [anon_sym_to] = ACTIONS(847),
    [anon_sym_pure] = ACTIONS(847),
    [anon_sym_acquires] = ACTIONS(847),
    [anon_sym_reads] = ACTIONS(847),
    [anon_sym_writes] = ACTIONS(847),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(869)] = {
    [sym_line_comment] = STATE(869),
    [sym_block_comment] = STATE(869),
    [sym_identifier] = ACTIONS(851),
    [anon_sym_u8] = ACTIONS(851),
    [anon_sym_u16] = ACTIONS(851),
    [anon_sym_u32] = ACTIONS(851),
    [anon_sym_u64] = ACTIONS(851),
    [anon_sym_u128] = ACTIONS(851),
    [anon_sym_u256] = ACTIONS(851),
    [anon_sym_exists] = ACTIONS(851),
    [anon_sym_forall] = ACTIONS(851),
    [anon_sym_choose] = ACTIONS(851),
    [anon_sym_min] = ACTIONS(851),
    [anon_sym_bool] = ACTIONS(851),
    [anon_sym_address] = ACTIONS(851),
    [anon_sym_vector] = ACTIONS(851),
    [anon_sym_for] = ACTIONS(851),
    [anon_sym_while] = ACTIONS(851),
    [anon_sym_friend] = ACTIONS(851),
    [anon_sym_match] = ACTIONS(851),
    [aux_sym_number_token1] = ACTIONS(851),
    [aux_sym_number_token2] = ACTIONS(853),
    [aux_sym_number_token4] = ACTIONS(853),
    [aux_sym_number_token6] = ACTIONS(853),
    [anon_sym_STAR] = ACTIONS(853),
    [anon_sym_LPAREN] = ACTIONS(853),
    [anon_sym_COMMA] = ACTIONS(853),
    [anon_sym_RPAREN] = ACTIONS(853),
    [anon_sym_PIPE] = ACTIONS(851),
    [anon_sym_AMP] = ACTIONS(851),
    [anon_sym_EQ] = ACTIONS(851),
    [anon_sym_where] = ACTIONS(851),
    [anon_sym_COLON] = ACTIONS(853),
    [anon_sym_LBRACE] = ACTIONS(853),
    [anon_sym_RBRACE] = ACTIONS(853),
    [anon_sym_EQ_EQ_GT] = ACTIONS(853),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(853),
    [anon_sym_PIPE_PIPE] = ACTIONS(853),
    [anon_sym_AMP_AMP] = ACTIONS(853),
    [anon_sym_EQ_EQ] = ACTIONS(851),
    [anon_sym_BANG_EQ] = ACTIONS(853),
    [anon_sym_LT] = ACTIONS(851),
    [anon_sym_GT] = ACTIONS(851),
    [anon_sym_LT_EQ] = ACTIONS(851),
    [anon_sym_GT_EQ] = ACTIONS(853),
    [anon_sym_DOT_DOT] = ACTIONS(853),
    [anon_sym_CARET] = ACTIONS(853),
    [anon_sym_LT_LT] = ACTIONS(853),
    [anon_sym_GT_GT] = ACTIONS(853),
    [anon_sym_PLUS] = ACTIONS(853),
    [anon_sym_DASH] = ACTIONS(853),
    [anon_sym_SLASH] = ACTIONS(851),
    [anon_sym_PERCENT] = ACTIONS(853),
    [anon_sym_BANG] = ACTIONS(851),
    [anon_sym_DOT] = ACTIONS(851),
    [anon_sym_LBRACK] = ACTIONS(853),
    [anon_sym_RBRACK] = ACTIONS(853),
    [anon_sym_as] = ACTIONS(851),
    [anon_sym_if] = ACTIONS(851),
    [anon_sym_EQ_GT] = ACTIONS(853),
    [anon_sym_else] = ACTIONS(851),
    [anon_sym_spec] = ACTIONS(851),
    [anon_sym_SEMI] = ACTIONS(853),
    [anon_sym_with] = ACTIONS(851),
    [anon_sym_to] = ACTIONS(851),
    [anon_sym_pure] = ACTIONS(851),
    [anon_sym_acquires] = ACTIONS(851),
    [anon_sym_reads] = ACTIONS(851),
    [anon_sym_writes] = ACTIONS(851),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(870)] = {
    [sym_line_comment] = STATE(870),
    [sym_block_comment] = STATE(870),
    [sym_identifier] = ACTIONS(855),
    [anon_sym_u8] = ACTIONS(855),
    [anon_sym_u16] = ACTIONS(855),
    [anon_sym_u32] = ACTIONS(855),
    [anon_sym_u64] = ACTIONS(855),
    [anon_sym_u128] = ACTIONS(855),
    [anon_sym_u256] = ACTIONS(855),
    [anon_sym_exists] = ACTIONS(855),
    [anon_sym_forall] = ACTIONS(855),
    [anon_sym_choose] = ACTIONS(855),
    [anon_sym_min] = ACTIONS(855),
    [anon_sym_bool] = ACTIONS(855),
    [anon_sym_address] = ACTIONS(855),
    [anon_sym_vector] = ACTIONS(855),
    [anon_sym_for] = ACTIONS(855),
    [anon_sym_while] = ACTIONS(855),
    [anon_sym_friend] = ACTIONS(855),
    [anon_sym_match] = ACTIONS(855),
    [aux_sym_number_token1] = ACTIONS(855),
    [aux_sym_number_token2] = ACTIONS(857),
    [aux_sym_number_token4] = ACTIONS(857),
    [aux_sym_number_token6] = ACTIONS(857),
    [anon_sym_STAR] = ACTIONS(857),
    [anon_sym_LPAREN] = ACTIONS(857),
    [anon_sym_COMMA] = ACTIONS(857),
    [anon_sym_RPAREN] = ACTIONS(857),
    [anon_sym_PIPE] = ACTIONS(855),
    [anon_sym_AMP] = ACTIONS(855),
    [anon_sym_EQ] = ACTIONS(855),
    [anon_sym_where] = ACTIONS(855),
    [anon_sym_COLON] = ACTIONS(857),
    [anon_sym_LBRACE] = ACTIONS(857),
    [anon_sym_RBRACE] = ACTIONS(857),
    [anon_sym_EQ_EQ_GT] = ACTIONS(857),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(857),
    [anon_sym_PIPE_PIPE] = ACTIONS(857),
    [anon_sym_AMP_AMP] = ACTIONS(857),
    [anon_sym_EQ_EQ] = ACTIONS(855),
    [anon_sym_BANG_EQ] = ACTIONS(857),
    [anon_sym_LT] = ACTIONS(855),
    [anon_sym_GT] = ACTIONS(855),
    [anon_sym_LT_EQ] = ACTIONS(855),
    [anon_sym_GT_EQ] = ACTIONS(857),
    [anon_sym_DOT_DOT] = ACTIONS(857),
    [anon_sym_CARET] = ACTIONS(857),
    [anon_sym_LT_LT] = ACTIONS(857),
    [anon_sym_GT_GT] = ACTIONS(857),
    [anon_sym_PLUS] = ACTIONS(857),
    [anon_sym_DASH] = ACTIONS(857),
    [anon_sym_SLASH] = ACTIONS(855),
    [anon_sym_PERCENT] = ACTIONS(857),
    [anon_sym_BANG] = ACTIONS(855),
    [anon_sym_DOT] = ACTIONS(855),
    [anon_sym_LBRACK] = ACTIONS(857),
    [anon_sym_RBRACK] = ACTIONS(857),
    [anon_sym_as] = ACTIONS(855),
    [anon_sym_if] = ACTIONS(855),
    [anon_sym_EQ_GT] = ACTIONS(857),
    [anon_sym_else] = ACTIONS(855),
    [anon_sym_spec] = ACTIONS(855),
    [anon_sym_SEMI] = ACTIONS(857),
    [anon_sym_with] = ACTIONS(855),
    [anon_sym_to] = ACTIONS(855),
    [anon_sym_pure] = ACTIONS(855),
    [anon_sym_acquires] = ACTIONS(855),
    [anon_sym_reads] = ACTIONS(855),
    [anon_sym_writes] = ACTIONS(855),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(871)] = {
    [sym_line_comment] = STATE(871),
    [sym_block_comment] = STATE(871),
    [sym_identifier] = ACTIONS(859),
    [anon_sym_u8] = ACTIONS(859),
    [anon_sym_u16] = ACTIONS(859),
    [anon_sym_u32] = ACTIONS(859),
    [anon_sym_u64] = ACTIONS(859),
    [anon_sym_u128] = ACTIONS(859),
    [anon_sym_u256] = ACTIONS(859),
    [anon_sym_exists] = ACTIONS(859),
    [anon_sym_forall] = ACTIONS(859),
    [anon_sym_choose] = ACTIONS(859),
    [anon_sym_min] = ACTIONS(859),
    [anon_sym_bool] = ACTIONS(859),
    [anon_sym_address] = ACTIONS(859),
    [anon_sym_vector] = ACTIONS(859),
    [anon_sym_for] = ACTIONS(859),
    [anon_sym_while] = ACTIONS(859),
    [anon_sym_friend] = ACTIONS(859),
    [anon_sym_match] = ACTIONS(859),
    [aux_sym_number_token1] = ACTIONS(859),
    [aux_sym_number_token2] = ACTIONS(861),
    [aux_sym_number_token4] = ACTIONS(861),
    [aux_sym_number_token6] = ACTIONS(861),
    [anon_sym_STAR] = ACTIONS(861),
    [anon_sym_LPAREN] = ACTIONS(861),
    [anon_sym_COMMA] = ACTIONS(861),
    [anon_sym_RPAREN] = ACTIONS(861),
    [anon_sym_PIPE] = ACTIONS(859),
    [anon_sym_AMP] = ACTIONS(859),
    [anon_sym_EQ] = ACTIONS(859),
    [anon_sym_where] = ACTIONS(859),
    [anon_sym_COLON] = ACTIONS(861),
    [anon_sym_LBRACE] = ACTIONS(861),
    [anon_sym_RBRACE] = ACTIONS(861),
    [anon_sym_EQ_EQ_GT] = ACTIONS(861),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(861),
    [anon_sym_PIPE_PIPE] = ACTIONS(861),
    [anon_sym_AMP_AMP] = ACTIONS(861),
    [anon_sym_EQ_EQ] = ACTIONS(859),
    [anon_sym_BANG_EQ] = ACTIONS(861),
    [anon_sym_LT] = ACTIONS(859),
    [anon_sym_GT] = ACTIONS(859),
    [anon_sym_LT_EQ] = ACTIONS(859),
    [anon_sym_GT_EQ] = ACTIONS(861),
    [anon_sym_DOT_DOT] = ACTIONS(861),
    [anon_sym_CARET] = ACTIONS(861),
    [anon_sym_LT_LT] = ACTIONS(861),
    [anon_sym_GT_GT] = ACTIONS(861),
    [anon_sym_PLUS] = ACTIONS(861),
    [anon_sym_DASH] = ACTIONS(861),
    [anon_sym_SLASH] = ACTIONS(859),
    [anon_sym_PERCENT] = ACTIONS(861),
    [anon_sym_BANG] = ACTIONS(859),
    [anon_sym_DOT] = ACTIONS(859),
    [anon_sym_LBRACK] = ACTIONS(861),
    [anon_sym_RBRACK] = ACTIONS(861),
    [anon_sym_as] = ACTIONS(859),
    [anon_sym_if] = ACTIONS(859),
    [anon_sym_EQ_GT] = ACTIONS(861),
    [anon_sym_else] = ACTIONS(859),
    [anon_sym_spec] = ACTIONS(859),
    [anon_sym_SEMI] = ACTIONS(861),
    [anon_sym_with] = ACTIONS(859),
    [anon_sym_to] = ACTIONS(859),
    [anon_sym_pure] = ACTIONS(859),
    [anon_sym_acquires] = ACTIONS(859),
    [anon_sym_reads] = ACTIONS(859),
    [anon_sym_writes] = ACTIONS(859),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(872)] = {
    [sym_line_comment] = STATE(872),
    [sym_block_comment] = STATE(872),
    [sym_identifier] = ACTIONS(863),
    [anon_sym_u8] = ACTIONS(863),
    [anon_sym_u16] = ACTIONS(863),
    [anon_sym_u32] = ACTIONS(863),
    [anon_sym_u64] = ACTIONS(863),
    [anon_sym_u128] = ACTIONS(863),
    [anon_sym_u256] = ACTIONS(863),
    [anon_sym_exists] = ACTIONS(863),
    [anon_sym_forall] = ACTIONS(863),
    [anon_sym_choose] = ACTIONS(863),
    [anon_sym_min] = ACTIONS(863),
    [anon_sym_bool] = ACTIONS(863),
    [anon_sym_address] = ACTIONS(863),
    [anon_sym_vector] = ACTIONS(863),
    [anon_sym_for] = ACTIONS(863),
    [anon_sym_while] = ACTIONS(863),
    [anon_sym_friend] = ACTIONS(863),
    [anon_sym_match] = ACTIONS(863),
    [aux_sym_number_token1] = ACTIONS(863),
    [aux_sym_number_token2] = ACTIONS(865),
    [aux_sym_number_token4] = ACTIONS(865),
    [aux_sym_number_token6] = ACTIONS(865),
    [anon_sym_STAR] = ACTIONS(865),
    [anon_sym_LPAREN] = ACTIONS(865),
    [anon_sym_COMMA] = ACTIONS(865),
    [anon_sym_RPAREN] = ACTIONS(865),
    [anon_sym_PIPE] = ACTIONS(863),
    [anon_sym_AMP] = ACTIONS(863),
    [anon_sym_EQ] = ACTIONS(863),
    [anon_sym_where] = ACTIONS(863),
    [anon_sym_COLON] = ACTIONS(865),
    [anon_sym_LBRACE] = ACTIONS(865),
    [anon_sym_RBRACE] = ACTIONS(865),
    [anon_sym_EQ_EQ_GT] = ACTIONS(865),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(865),
    [anon_sym_PIPE_PIPE] = ACTIONS(865),
    [anon_sym_AMP_AMP] = ACTIONS(865),
    [anon_sym_EQ_EQ] = ACTIONS(863),
    [anon_sym_BANG_EQ] = ACTIONS(865),
    [anon_sym_LT] = ACTIONS(863),
    [anon_sym_GT] = ACTIONS(863),
    [anon_sym_LT_EQ] = ACTIONS(863),
    [anon_sym_GT_EQ] = ACTIONS(865),
    [anon_sym_DOT_DOT] = ACTIONS(865),
    [anon_sym_CARET] = ACTIONS(865),
    [anon_sym_LT_LT] = ACTIONS(865),
    [anon_sym_GT_GT] = ACTIONS(865),
    [anon_sym_PLUS] = ACTIONS(865),
    [anon_sym_DASH] = ACTIONS(865),
    [anon_sym_SLASH] = ACTIONS(863),
    [anon_sym_PERCENT] = ACTIONS(865),
    [anon_sym_BANG] = ACTIONS(863),
    [anon_sym_DOT] = ACTIONS(863),
    [anon_sym_LBRACK] = ACTIONS(865),
    [anon_sym_RBRACK] = ACTIONS(865),
    [anon_sym_as] = ACTIONS(863),
    [anon_sym_if] = ACTIONS(863),
    [anon_sym_EQ_GT] = ACTIONS(865),
    [anon_sym_else] = ACTIONS(863),
    [anon_sym_spec] = ACTIONS(863),
    [anon_sym_SEMI] = ACTIONS(865),
    [anon_sym_with] = ACTIONS(863),
    [anon_sym_to] = ACTIONS(863),
    [anon_sym_pure] = ACTIONS(863),
    [anon_sym_acquires] = ACTIONS(863),
    [anon_sym_reads] = ACTIONS(863),
    [anon_sym_writes] = ACTIONS(863),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(873)] = {
    [sym_type_args] = STATE(2223),
    [sym_line_comment] = STATE(873),
    [sym_block_comment] = STATE(873),
    [sym_identifier] = ACTIONS(807),
    [anon_sym_u8] = ACTIONS(807),
    [anon_sym_u16] = ACTIONS(807),
    [anon_sym_u32] = ACTIONS(807),
    [anon_sym_u64] = ACTIONS(807),
    [anon_sym_u128] = ACTIONS(807),
    [anon_sym_u256] = ACTIONS(807),
    [anon_sym_exists] = ACTIONS(807),
    [anon_sym_forall] = ACTIONS(807),
    [anon_sym_choose] = ACTIONS(807),
    [anon_sym_min] = ACTIONS(807),
    [anon_sym_bool] = ACTIONS(807),
    [anon_sym_address] = ACTIONS(807),
    [anon_sym_vector] = ACTIONS(807),
    [anon_sym_for] = ACTIONS(807),
    [anon_sym_while] = ACTIONS(807),
    [anon_sym_friend] = ACTIONS(807),
    [anon_sym_match] = ACTIONS(807),
    [aux_sym_number_token1] = ACTIONS(807),
    [aux_sym_number_token2] = ACTIONS(809),
    [aux_sym_number_token4] = ACTIONS(809),
    [aux_sym_number_token6] = ACTIONS(809),
    [anon_sym_STAR] = ACTIONS(809),
    [anon_sym_COLON_COLON] = ACTIONS(809),
    [anon_sym_LPAREN] = ACTIONS(809),
    [anon_sym_COMMA] = ACTIONS(809),
    [anon_sym_RPAREN] = ACTIONS(809),
    [anon_sym_PIPE] = ACTIONS(807),
    [anon_sym_AMP] = ACTIONS(807),
    [anon_sym_EQ] = ACTIONS(807),
    [anon_sym_where] = ACTIONS(807),
    [anon_sym_COLON] = ACTIONS(807),
    [anon_sym_LBRACE] = ACTIONS(809),
    [anon_sym_RBRACE] = ACTIONS(809),
    [anon_sym_EQ_EQ_GT] = ACTIONS(809),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(809),
    [anon_sym_PIPE_PIPE] = ACTIONS(809),
    [anon_sym_AMP_AMP] = ACTIONS(809),
    [anon_sym_EQ_EQ] = ACTIONS(807),
    [anon_sym_BANG_EQ] = ACTIONS(809),
    [anon_sym_LT] = ACTIONS(807),
    [anon_sym_GT] = ACTIONS(807),
    [anon_sym_LT_EQ] = ACTIONS(807),
    [anon_sym_GT_EQ] = ACTIONS(809),
    [anon_sym_DOT_DOT] = ACTIONS(809),
    [anon_sym_CARET] = ACTIONS(809),
    [anon_sym_LT_LT] = ACTIONS(809),
    [anon_sym_GT_GT] = ACTIONS(809),
    [anon_sym_PLUS] = ACTIONS(809),
    [anon_sym_DASH] = ACTIONS(809),
    [anon_sym_SLASH] = ACTIONS(807),
    [anon_sym_PERCENT] = ACTIONS(809),
    [anon_sym_BANG] = ACTIONS(807),
    [anon_sym_DOT] = ACTIONS(807),
    [anon_sym_LBRACK] = ACTIONS(867),
    [anon_sym_RBRACK] = ACTIONS(809),
    [anon_sym_as] = ACTIONS(807),
    [anon_sym_if] = ACTIONS(807),
    [anon_sym_EQ_GT] = ACTIONS(809),
    [anon_sym_else] = ACTIONS(807),
    [anon_sym_LT2] = ACTIONS(869),
    [anon_sym_spec] = ACTIONS(807),
    [anon_sym_SEMI] = ACTIONS(809),
    [anon_sym_with] = ACTIONS(807),
    [anon_sym_to] = ACTIONS(807),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(874)] = {
    [sym_line_comment] = STATE(874),
    [sym_block_comment] = STATE(874),
    [sym_identifier] = ACTIONS(815),
    [anon_sym_u8] = ACTIONS(815),
    [anon_sym_u16] = ACTIONS(815),
    [anon_sym_u32] = ACTIONS(815),
    [anon_sym_u64] = ACTIONS(815),
    [anon_sym_u128] = ACTIONS(815),
    [anon_sym_u256] = ACTIONS(815),
    [anon_sym_exists] = ACTIONS(815),
    [anon_sym_forall] = ACTIONS(815),
    [anon_sym_choose] = ACTIONS(815),
    [anon_sym_min] = ACTIONS(815),
    [anon_sym_bool] = ACTIONS(815),
    [anon_sym_address] = ACTIONS(815),
    [anon_sym_vector] = ACTIONS(815),
    [anon_sym_for] = ACTIONS(815),
    [anon_sym_while] = ACTIONS(815),
    [anon_sym_friend] = ACTIONS(815),
    [anon_sym_match] = ACTIONS(815),
    [aux_sym_number_token1] = ACTIONS(815),
    [aux_sym_number_token2] = ACTIONS(817),
    [aux_sym_number_token4] = ACTIONS(817),
    [aux_sym_number_token6] = ACTIONS(817),
    [anon_sym_STAR] = ACTIONS(817),
    [anon_sym_COLON_COLON] = ACTIONS(817),
    [anon_sym_LPAREN] = ACTIONS(871),
    [anon_sym_COMMA] = ACTIONS(817),
    [anon_sym_RPAREN] = ACTIONS(817),
    [anon_sym_PIPE] = ACTIONS(815),
    [anon_sym_AMP] = ACTIONS(815),
    [anon_sym_EQ] = ACTIONS(815),
    [anon_sym_where] = ACTIONS(815),
    [anon_sym_COLON] = ACTIONS(815),
    [anon_sym_LBRACE] = ACTIONS(817),
    [anon_sym_RBRACE] = ACTIONS(817),
    [anon_sym_EQ_EQ_GT] = ACTIONS(817),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(817),
    [anon_sym_PIPE_PIPE] = ACTIONS(817),
    [anon_sym_AMP_AMP] = ACTIONS(817),
    [anon_sym_EQ_EQ] = ACTIONS(815),
    [anon_sym_BANG_EQ] = ACTIONS(817),
    [anon_sym_LT] = ACTIONS(815),
    [anon_sym_GT] = ACTIONS(815),
    [anon_sym_LT_EQ] = ACTIONS(815),
    [anon_sym_GT_EQ] = ACTIONS(817),
    [anon_sym_DOT_DOT] = ACTIONS(817),
    [anon_sym_CARET] = ACTIONS(817),
    [anon_sym_LT_LT] = ACTIONS(817),
    [anon_sym_GT_GT] = ACTIONS(817),
    [anon_sym_PLUS] = ACTIONS(817),
    [anon_sym_DASH] = ACTIONS(817),
    [anon_sym_SLASH] = ACTIONS(815),
    [anon_sym_PERCENT] = ACTIONS(817),
    [anon_sym_BANG] = ACTIONS(815),
    [anon_sym_DOT] = ACTIONS(815),
    [anon_sym_LBRACK] = ACTIONS(817),
    [anon_sym_RBRACK] = ACTIONS(817),
    [anon_sym_as] = ACTIONS(815),
    [anon_sym_if] = ACTIONS(815),
    [anon_sym_EQ_GT] = ACTIONS(817),
    [anon_sym_else] = ACTIONS(815),
    [anon_sym_LT2] = ACTIONS(815),
    [anon_sym_spec] = ACTIONS(815),
    [anon_sym_SEMI] = ACTIONS(817),
    [anon_sym_with] = ACTIONS(815),
    [anon_sym_to] = ACTIONS(815),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(875)] = {
    [sym_line_comment] = STATE(875),
    [sym_block_comment] = STATE(875),
    [sym_identifier] = ACTIONS(815),
    [anon_sym_u8] = ACTIONS(815),
    [anon_sym_u16] = ACTIONS(815),
    [anon_sym_u32] = ACTIONS(815),
    [anon_sym_u64] = ACTIONS(815),
    [anon_sym_u128] = ACTIONS(815),
    [anon_sym_u256] = ACTIONS(815),
    [anon_sym_exists] = ACTIONS(815),
    [anon_sym_forall] = ACTIONS(815),
    [anon_sym_choose] = ACTIONS(815),
    [anon_sym_min] = ACTIONS(815),
    [anon_sym_bool] = ACTIONS(815),
    [anon_sym_address] = ACTIONS(815),
    [anon_sym_vector] = ACTIONS(815),
    [anon_sym_for] = ACTIONS(815),
    [anon_sym_while] = ACTIONS(815),
    [anon_sym_friend] = ACTIONS(815),
    [anon_sym_match] = ACTIONS(815),
    [aux_sym_number_token1] = ACTIONS(815),
    [aux_sym_number_token2] = ACTIONS(817),
    [aux_sym_number_token4] = ACTIONS(817),
    [aux_sym_number_token6] = ACTIONS(817),
    [anon_sym_STAR] = ACTIONS(817),
    [anon_sym_COLON_COLON] = ACTIONS(817),
    [anon_sym_LPAREN] = ACTIONS(874),
    [anon_sym_COMMA] = ACTIONS(817),
    [anon_sym_RPAREN] = ACTIONS(817),
    [anon_sym_PIPE] = ACTIONS(815),
    [anon_sym_AMP] = ACTIONS(815),
    [anon_sym_EQ] = ACTIONS(815),
    [anon_sym_where] = ACTIONS(815),
    [anon_sym_COLON] = ACTIONS(815),
    [anon_sym_LBRACE] = ACTIONS(817),
    [anon_sym_RBRACE] = ACTIONS(817),
    [anon_sym_EQ_EQ_GT] = ACTIONS(817),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(817),
    [anon_sym_PIPE_PIPE] = ACTIONS(817),
    [anon_sym_AMP_AMP] = ACTIONS(817),
    [anon_sym_EQ_EQ] = ACTIONS(815),
    [anon_sym_BANG_EQ] = ACTIONS(817),
    [anon_sym_LT] = ACTIONS(815),
    [anon_sym_GT] = ACTIONS(815),
    [anon_sym_LT_EQ] = ACTIONS(815),
    [anon_sym_GT_EQ] = ACTIONS(817),
    [anon_sym_DOT_DOT] = ACTIONS(817),
    [anon_sym_CARET] = ACTIONS(817),
    [anon_sym_LT_LT] = ACTIONS(817),
    [anon_sym_GT_GT] = ACTIONS(817),
    [anon_sym_PLUS] = ACTIONS(817),
    [anon_sym_DASH] = ACTIONS(817),
    [anon_sym_SLASH] = ACTIONS(815),
    [anon_sym_PERCENT] = ACTIONS(817),
    [anon_sym_BANG] = ACTIONS(815),
    [anon_sym_DOT] = ACTIONS(815),
    [anon_sym_LBRACK] = ACTIONS(817),
    [anon_sym_RBRACK] = ACTIONS(817),
    [anon_sym_as] = ACTIONS(815),
    [anon_sym_if] = ACTIONS(815),
    [anon_sym_EQ_GT] = ACTIONS(817),
    [anon_sym_else] = ACTIONS(815),
    [anon_sym_LT2] = ACTIONS(815),
    [anon_sym_spec] = ACTIONS(815),
    [anon_sym_SEMI] = ACTIONS(817),
    [anon_sym_with] = ACTIONS(815),
    [anon_sym_to] = ACTIONS(815),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(876)] = {
    [sym_spec_block] = STATE(944),
    [sym_spec_loop_invariant] = STATE(945),
    [sym_line_comment] = STATE(876),
    [sym_block_comment] = STATE(876),
    [sym_identifier] = ACTIONS(877),
    [anon_sym_u8] = ACTIONS(877),
    [anon_sym_u16] = ACTIONS(877),
    [anon_sym_u32] = ACTIONS(877),
    [anon_sym_u64] = ACTIONS(877),
    [anon_sym_u128] = ACTIONS(877),
    [anon_sym_u256] = ACTIONS(877),
    [anon_sym_exists] = ACTIONS(877),
    [anon_sym_forall] = ACTIONS(877),
    [anon_sym_choose] = ACTIONS(877),
    [anon_sym_min] = ACTIONS(877),
    [anon_sym_bool] = ACTIONS(877),
    [anon_sym_address] = ACTIONS(877),
    [anon_sym_vector] = ACTIONS(877),
    [anon_sym_for] = ACTIONS(877),
    [anon_sym_while] = ACTIONS(877),
    [anon_sym_friend] = ACTIONS(877),
    [anon_sym_match] = ACTIONS(877),
    [aux_sym_number_token1] = ACTIONS(877),
    [aux_sym_number_token2] = ACTIONS(879),
    [aux_sym_number_token4] = ACTIONS(879),
    [aux_sym_number_token6] = ACTIONS(879),
    [anon_sym_STAR] = ACTIONS(879),
    [anon_sym_LPAREN] = ACTIONS(879),
    [anon_sym_COMMA] = ACTIONS(879),
    [anon_sym_RPAREN] = ACTIONS(879),
    [anon_sym_PIPE] = ACTIONS(877),
    [anon_sym_AMP] = ACTIONS(877),
    [anon_sym_EQ] = ACTIONS(877),
    [anon_sym_where] = ACTIONS(877),
    [anon_sym_COLON] = ACTIONS(879),
    [anon_sym_LBRACE] = ACTIONS(879),
    [anon_sym_RBRACE] = ACTIONS(879),
    [anon_sym_EQ_EQ_GT] = ACTIONS(879),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(879),
    [anon_sym_PIPE_PIPE] = ACTIONS(879),
    [anon_sym_AMP_AMP] = ACTIONS(879),
    [anon_sym_EQ_EQ] = ACTIONS(877),
    [anon_sym_BANG_EQ] = ACTIONS(879),
    [anon_sym_LT] = ACTIONS(877),
    [anon_sym_GT] = ACTIONS(877),
    [anon_sym_LT_EQ] = ACTIONS(877),
    [anon_sym_GT_EQ] = ACTIONS(879),
    [anon_sym_DOT_DOT] = ACTIONS(879),
    [anon_sym_CARET] = ACTIONS(879),
    [anon_sym_LT_LT] = ACTIONS(879),
    [anon_sym_GT_GT] = ACTIONS(879),
    [anon_sym_PLUS] = ACTIONS(879),
    [anon_sym_DASH] = ACTIONS(879),
    [anon_sym_SLASH] = ACTIONS(877),
    [anon_sym_PERCENT] = ACTIONS(879),
    [anon_sym_DOT] = ACTIONS(877),
    [anon_sym_LBRACK] = ACTIONS(879),
    [anon_sym_RBRACK] = ACTIONS(879),
    [anon_sym_as] = ACTIONS(877),
    [anon_sym_if] = ACTIONS(877),
    [anon_sym_EQ_GT] = ACTIONS(879),
    [anon_sym_else] = ACTIONS(877),
    [anon_sym_spec] = ACTIONS(877),
    [anon_sym_SEMI] = ACTIONS(879),
    [anon_sym_with] = ACTIONS(877),
    [anon_sym_to] = ACTIONS(877),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(877)] = {
    [sym_call_args] = STATE(954),
    [sym_line_comment] = STATE(877),
    [sym_block_comment] = STATE(877),
    [sym_identifier] = ACTIONS(881),
    [anon_sym_u8] = ACTIONS(881),
    [anon_sym_u16] = ACTIONS(881),
    [anon_sym_u32] = ACTIONS(881),
    [anon_sym_u64] = ACTIONS(881),
    [anon_sym_u128] = ACTIONS(881),
    [anon_sym_u256] = ACTIONS(881),
    [anon_sym_exists] = ACTIONS(881),
    [anon_sym_forall] = ACTIONS(881),
    [anon_sym_choose] = ACTIONS(881),
    [anon_sym_min] = ACTIONS(881),
    [anon_sym_bool] = ACTIONS(881),
    [anon_sym_address] = ACTIONS(881),
    [anon_sym_vector] = ACTIONS(881),
    [anon_sym_for] = ACTIONS(881),
    [anon_sym_while] = ACTIONS(881),
    [anon_sym_friend] = ACTIONS(881),
    [anon_sym_match] = ACTIONS(881),
    [aux_sym_number_token1] = ACTIONS(881),
    [aux_sym_number_token2] = ACTIONS(883),
    [aux_sym_number_token4] = ACTIONS(883),
    [aux_sym_number_token6] = ACTIONS(883),
    [anon_sym_STAR] = ACTIONS(883),
    [anon_sym_COLON_COLON] = ACTIONS(885),
    [anon_sym_LPAREN] = ACTIONS(887),
    [anon_sym_COMMA] = ACTIONS(883),
    [anon_sym_RPAREN] = ACTIONS(883),
    [anon_sym_PIPE] = ACTIONS(881),
    [anon_sym_AMP] = ACTIONS(881),
    [anon_sym_EQ] = ACTIONS(881),
    [anon_sym_where] = ACTIONS(881),
    [anon_sym_COLON] = ACTIONS(881),
    [anon_sym_LBRACE] = ACTIONS(883),
    [anon_sym_RBRACE] = ACTIONS(883),
    [anon_sym_EQ_EQ_GT] = ACTIONS(883),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(883),
    [anon_sym_PIPE_PIPE] = ACTIONS(883),
    [anon_sym_AMP_AMP] = ACTIONS(883),
    [anon_sym_EQ_EQ] = ACTIONS(881),
    [anon_sym_BANG_EQ] = ACTIONS(883),
    [anon_sym_LT] = ACTIONS(881),
    [anon_sym_GT] = ACTIONS(881),
    [anon_sym_LT_EQ] = ACTIONS(881),
    [anon_sym_GT_EQ] = ACTIONS(883),
    [anon_sym_DOT_DOT] = ACTIONS(883),
    [anon_sym_CARET] = ACTIONS(883),
    [anon_sym_LT_LT] = ACTIONS(883),
    [anon_sym_GT_GT] = ACTIONS(883),
    [anon_sym_PLUS] = ACTIONS(883),
    [anon_sym_DASH] = ACTIONS(883),
    [anon_sym_SLASH] = ACTIONS(881),
    [anon_sym_PERCENT] = ACTIONS(883),
    [anon_sym_DOT] = ACTIONS(881),
    [anon_sym_LBRACK] = ACTIONS(883),
    [anon_sym_RBRACK] = ACTIONS(883),
    [anon_sym_as] = ACTIONS(881),
    [anon_sym_if] = ACTIONS(881),
    [anon_sym_EQ_GT] = ACTIONS(883),
    [anon_sym_else] = ACTIONS(881),
    [anon_sym_spec] = ACTIONS(881),
    [anon_sym_SEMI] = ACTIONS(883),
    [anon_sym_with] = ACTIONS(881),
    [anon_sym_to] = ACTIONS(881),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(878)] = {
    [sym_line_comment] = STATE(878),
    [sym_block_comment] = STATE(878),
    [sym_identifier] = ACTIONS(889),
    [anon_sym_u8] = ACTIONS(889),
    [anon_sym_u16] = ACTIONS(889),
    [anon_sym_u32] = ACTIONS(889),
    [anon_sym_u64] = ACTIONS(889),
    [anon_sym_u128] = ACTIONS(889),
    [anon_sym_u256] = ACTIONS(889),
    [anon_sym_exists] = ACTIONS(889),
    [anon_sym_forall] = ACTIONS(889),
    [anon_sym_choose] = ACTIONS(889),
    [anon_sym_min] = ACTIONS(889),
    [anon_sym_bool] = ACTIONS(889),
    [anon_sym_address] = ACTIONS(889),
    [anon_sym_vector] = ACTIONS(889),
    [anon_sym_for] = ACTIONS(889),
    [anon_sym_while] = ACTIONS(889),
    [anon_sym_friend] = ACTIONS(889),
    [anon_sym_match] = ACTIONS(889),
    [aux_sym_number_token1] = ACTIONS(889),
    [aux_sym_number_token2] = ACTIONS(891),
    [aux_sym_number_token4] = ACTIONS(891),
    [aux_sym_number_token6] = ACTIONS(891),
    [anon_sym_STAR] = ACTIONS(891),
    [anon_sym_COLON_COLON] = ACTIONS(891),
    [anon_sym_LPAREN] = ACTIONS(891),
    [anon_sym_COMMA] = ACTIONS(891),
    [anon_sym_RPAREN] = ACTIONS(891),
    [anon_sym_PIPE] = ACTIONS(889),
    [anon_sym_AMP] = ACTIONS(889),
    [anon_sym_EQ] = ACTIONS(889),
    [anon_sym_where] = ACTIONS(889),
    [anon_sym_COLON] = ACTIONS(889),
    [anon_sym_LBRACE] = ACTIONS(891),
    [anon_sym_RBRACE] = ACTIONS(891),
    [anon_sym_EQ_EQ_GT] = ACTIONS(891),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(891),
    [anon_sym_PIPE_PIPE] = ACTIONS(891),
    [anon_sym_AMP_AMP] = ACTIONS(891),
    [anon_sym_EQ_EQ] = ACTIONS(889),
    [anon_sym_BANG_EQ] = ACTIONS(891),
    [anon_sym_LT] = ACTIONS(889),
    [anon_sym_GT] = ACTIONS(889),
    [anon_sym_LT_EQ] = ACTIONS(889),
    [anon_sym_GT_EQ] = ACTIONS(891),
    [anon_sym_DOT_DOT] = ACTIONS(891),
    [anon_sym_CARET] = ACTIONS(891),
    [anon_sym_LT_LT] = ACTIONS(891),
    [anon_sym_GT_GT] = ACTIONS(891),
    [anon_sym_PLUS] = ACTIONS(891),
    [anon_sym_DASH] = ACTIONS(891),
    [anon_sym_SLASH] = ACTIONS(889),
    [anon_sym_PERCENT] = ACTIONS(891),
    [anon_sym_DOT] = ACTIONS(889),
    [anon_sym_LBRACK] = ACTIONS(891),
    [anon_sym_RBRACK] = ACTIONS(891),
    [anon_sym_as] = ACTIONS(889),
    [anon_sym_if] = ACTIONS(889),
    [anon_sym_EQ_GT] = ACTIONS(891),
    [anon_sym_else] = ACTIONS(889),
    [anon_sym_spec] = ACTIONS(889),
    [anon_sym_SEMI] = ACTIONS(891),
    [anon_sym_with] = ACTIONS(889),
    [anon_sym_to] = ACTIONS(889),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(879)] = {
    [sym_spec_block] = STATE(944),
    [sym_spec_loop_invariant] = STATE(945),
    [sym_line_comment] = STATE(879),
    [sym_block_comment] = STATE(879),
    [sym_identifier] = ACTIONS(877),
    [anon_sym_u8] = ACTIONS(877),
    [anon_sym_u16] = ACTIONS(877),
    [anon_sym_u32] = ACTIONS(877),
    [anon_sym_u64] = ACTIONS(877),
    [anon_sym_u128] = ACTIONS(877),
    [anon_sym_u256] = ACTIONS(877),
    [anon_sym_exists] = ACTIONS(877),
    [anon_sym_forall] = ACTIONS(877),
    [anon_sym_choose] = ACTIONS(877),
    [anon_sym_min] = ACTIONS(877),
    [anon_sym_bool] = ACTIONS(877),
    [anon_sym_address] = ACTIONS(877),
    [anon_sym_vector] = ACTIONS(877),
    [anon_sym_for] = ACTIONS(877),
    [anon_sym_while] = ACTIONS(877),
    [anon_sym_friend] = ACTIONS(877),
    [anon_sym_match] = ACTIONS(877),
    [aux_sym_number_token1] = ACTIONS(877),
    [aux_sym_number_token2] = ACTIONS(879),
    [aux_sym_number_token4] = ACTIONS(879),
    [aux_sym_number_token6] = ACTIONS(879),
    [anon_sym_STAR] = ACTIONS(879),
    [anon_sym_LPAREN] = ACTIONS(879),
    [anon_sym_COMMA] = ACTIONS(879),
    [anon_sym_RPAREN] = ACTIONS(879),
    [anon_sym_PIPE] = ACTIONS(877),
    [anon_sym_AMP] = ACTIONS(877),
    [anon_sym_EQ] = ACTIONS(877),
    [anon_sym_where] = ACTIONS(877),
    [anon_sym_COLON] = ACTIONS(879),
    [anon_sym_LBRACE] = ACTIONS(879),
    [anon_sym_RBRACE] = ACTIONS(879),
    [anon_sym_EQ_EQ_GT] = ACTIONS(879),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(879),
    [anon_sym_PIPE_PIPE] = ACTIONS(879),
    [anon_sym_AMP_AMP] = ACTIONS(879),
    [anon_sym_EQ_EQ] = ACTIONS(877),
    [anon_sym_BANG_EQ] = ACTIONS(879),
    [anon_sym_LT] = ACTIONS(877),
    [anon_sym_GT] = ACTIONS(877),
    [anon_sym_LT_EQ] = ACTIONS(877),
    [anon_sym_GT_EQ] = ACTIONS(879),
    [anon_sym_DOT_DOT] = ACTIONS(879),
    [anon_sym_CARET] = ACTIONS(879),
    [anon_sym_LT_LT] = ACTIONS(879),
    [anon_sym_GT_GT] = ACTIONS(879),
    [anon_sym_PLUS] = ACTIONS(879),
    [anon_sym_DASH] = ACTIONS(879),
    [anon_sym_SLASH] = ACTIONS(877),
    [anon_sym_PERCENT] = ACTIONS(879),
    [anon_sym_DOT] = ACTIONS(877),
    [anon_sym_LBRACK] = ACTIONS(879),
    [anon_sym_RBRACK] = ACTIONS(879),
    [anon_sym_if] = ACTIONS(877),
    [anon_sym_EQ_GT] = ACTIONS(879),
    [anon_sym_else] = ACTIONS(877),
    [anon_sym_spec] = ACTIONS(85),
    [anon_sym_SEMI] = ACTIONS(879),
    [anon_sym_with] = ACTIONS(877),
    [anon_sym_to] = ACTIONS(877),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(880)] = {
    [sym_line_comment] = STATE(880),
    [sym_block_comment] = STATE(880),
    [sym_identifier] = ACTIONS(893),
    [anon_sym_u8] = ACTIONS(893),
    [anon_sym_u16] = ACTIONS(893),
    [anon_sym_u32] = ACTIONS(893),
    [anon_sym_u64] = ACTIONS(893),
    [anon_sym_u128] = ACTIONS(893),
    [anon_sym_u256] = ACTIONS(893),
    [anon_sym_exists] = ACTIONS(893),
    [anon_sym_forall] = ACTIONS(893),
    [anon_sym_choose] = ACTIONS(893),
    [anon_sym_min] = ACTIONS(893),
    [anon_sym_bool] = ACTIONS(893),
    [anon_sym_address] = ACTIONS(893),
    [anon_sym_vector] = ACTIONS(893),
    [anon_sym_for] = ACTIONS(893),
    [anon_sym_while] = ACTIONS(893),
    [anon_sym_friend] = ACTIONS(893),
    [anon_sym_match] = ACTIONS(893),
    [aux_sym_number_token1] = ACTIONS(893),
    [aux_sym_number_token2] = ACTIONS(895),
    [aux_sym_number_token4] = ACTIONS(895),
    [aux_sym_number_token6] = ACTIONS(895),
    [anon_sym_STAR] = ACTIONS(895),
    [anon_sym_COLON_COLON] = ACTIONS(895),
    [anon_sym_LPAREN] = ACTIONS(895),
    [anon_sym_COMMA] = ACTIONS(895),
    [anon_sym_RPAREN] = ACTIONS(895),
    [anon_sym_PIPE] = ACTIONS(893),
    [anon_sym_AMP] = ACTIONS(893),
    [anon_sym_EQ] = ACTIONS(893),
    [anon_sym_where] = ACTIONS(893),
    [anon_sym_COLON] = ACTIONS(893),
    [anon_sym_LBRACE] = ACTIONS(895),
    [anon_sym_RBRACE] = ACTIONS(895),
    [anon_sym_EQ_EQ_GT] = ACTIONS(895),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(895),
    [anon_sym_PIPE_PIPE] = ACTIONS(895),
    [anon_sym_AMP_AMP] = ACTIONS(895),
    [anon_sym_EQ_EQ] = ACTIONS(893),
    [anon_sym_BANG_EQ] = ACTIONS(895),
    [anon_sym_LT] = ACTIONS(893),
    [anon_sym_GT] = ACTIONS(893),
    [anon_sym_LT_EQ] = ACTIONS(893),
    [anon_sym_GT_EQ] = ACTIONS(895),
    [anon_sym_DOT_DOT] = ACTIONS(895),
    [anon_sym_CARET] = ACTIONS(895),
    [anon_sym_LT_LT] = ACTIONS(895),
    [anon_sym_GT_GT] = ACTIONS(895),
    [anon_sym_PLUS] = ACTIONS(895),
    [anon_sym_DASH] = ACTIONS(895),
    [anon_sym_SLASH] = ACTIONS(893),
    [anon_sym_PERCENT] = ACTIONS(895),
    [anon_sym_DOT] = ACTIONS(893),
    [anon_sym_LBRACK] = ACTIONS(895),
    [anon_sym_RBRACK] = ACTIONS(895),
    [anon_sym_as] = ACTIONS(893),
    [anon_sym_if] = ACTIONS(893),
    [anon_sym_EQ_GT] = ACTIONS(895),
    [anon_sym_else] = ACTIONS(893),
    [anon_sym_spec] = ACTIONS(893),
    [anon_sym_SEMI] = ACTIONS(895),
    [anon_sym_with] = ACTIONS(893),
    [anon_sym_to] = ACTIONS(893),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(881)] = {
    [sym_line_comment] = STATE(881),
    [sym_block_comment] = STATE(881),
    [sym_identifier] = ACTIONS(897),
    [anon_sym_u8] = ACTIONS(897),
    [anon_sym_u16] = ACTIONS(897),
    [anon_sym_u32] = ACTIONS(897),
    [anon_sym_u64] = ACTIONS(897),
    [anon_sym_u128] = ACTIONS(897),
    [anon_sym_u256] = ACTIONS(897),
    [anon_sym_exists] = ACTIONS(897),
    [anon_sym_forall] = ACTIONS(897),
    [anon_sym_choose] = ACTIONS(897),
    [anon_sym_min] = ACTIONS(897),
    [anon_sym_bool] = ACTIONS(897),
    [anon_sym_address] = ACTIONS(897),
    [anon_sym_vector] = ACTIONS(897),
    [anon_sym_for] = ACTIONS(897),
    [anon_sym_while] = ACTIONS(897),
    [anon_sym_friend] = ACTIONS(897),
    [anon_sym_match] = ACTIONS(897),
    [aux_sym_number_token1] = ACTIONS(897),
    [aux_sym_number_token2] = ACTIONS(899),
    [aux_sym_number_token4] = ACTIONS(899),
    [aux_sym_number_token6] = ACTIONS(899),
    [anon_sym_STAR] = ACTIONS(899),
    [anon_sym_COLON_COLON] = ACTIONS(899),
    [anon_sym_LPAREN] = ACTIONS(899),
    [anon_sym_COMMA] = ACTIONS(899),
    [anon_sym_RPAREN] = ACTIONS(899),
    [anon_sym_PIPE] = ACTIONS(897),
    [anon_sym_AMP] = ACTIONS(897),
    [anon_sym_EQ] = ACTIONS(897),
    [anon_sym_where] = ACTIONS(897),
    [anon_sym_COLON] = ACTIONS(897),
    [anon_sym_LBRACE] = ACTIONS(899),
    [anon_sym_RBRACE] = ACTIONS(899),
    [anon_sym_EQ_EQ_GT] = ACTIONS(899),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(899),
    [anon_sym_PIPE_PIPE] = ACTIONS(899),
    [anon_sym_AMP_AMP] = ACTIONS(899),
    [anon_sym_EQ_EQ] = ACTIONS(897),
    [anon_sym_BANG_EQ] = ACTIONS(899),
    [anon_sym_LT] = ACTIONS(897),
    [anon_sym_GT] = ACTIONS(897),
    [anon_sym_LT_EQ] = ACTIONS(897),
    [anon_sym_GT_EQ] = ACTIONS(899),
    [anon_sym_DOT_DOT] = ACTIONS(899),
    [anon_sym_CARET] = ACTIONS(899),
    [anon_sym_LT_LT] = ACTIONS(899),
    [anon_sym_GT_GT] = ACTIONS(899),
    [anon_sym_PLUS] = ACTIONS(899),
    [anon_sym_DASH] = ACTIONS(899),
    [anon_sym_SLASH] = ACTIONS(897),
    [anon_sym_PERCENT] = ACTIONS(899),
    [anon_sym_DOT] = ACTIONS(897),
    [anon_sym_LBRACK] = ACTIONS(899),
    [anon_sym_RBRACK] = ACTIONS(899),
    [anon_sym_as] = ACTIONS(897),
    [anon_sym_if] = ACTIONS(897),
    [anon_sym_EQ_GT] = ACTIONS(899),
    [anon_sym_else] = ACTIONS(897),
    [anon_sym_spec] = ACTIONS(897),
    [anon_sym_SEMI] = ACTIONS(899),
    [anon_sym_with] = ACTIONS(897),
    [anon_sym_to] = ACTIONS(897),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(882)] = {
    [sym_line_comment] = STATE(882),
    [sym_block_comment] = STATE(882),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_u8] = ACTIONS(901),
    [anon_sym_u16] = ACTIONS(901),
    [anon_sym_u32] = ACTIONS(901),
    [anon_sym_u64] = ACTIONS(901),
    [anon_sym_u128] = ACTIONS(901),
    [anon_sym_u256] = ACTIONS(901),
    [anon_sym_exists] = ACTIONS(901),
    [anon_sym_forall] = ACTIONS(901),
    [anon_sym_choose] = ACTIONS(901),
    [anon_sym_min] = ACTIONS(901),
    [anon_sym_bool] = ACTIONS(901),
    [anon_sym_address] = ACTIONS(901),
    [anon_sym_vector] = ACTIONS(901),
    [anon_sym_for] = ACTIONS(901),
    [anon_sym_while] = ACTIONS(901),
    [anon_sym_friend] = ACTIONS(901),
    [anon_sym_match] = ACTIONS(901),
    [aux_sym_number_token1] = ACTIONS(901),
    [aux_sym_number_token2] = ACTIONS(903),
    [aux_sym_number_token4] = ACTIONS(903),
    [aux_sym_number_token6] = ACTIONS(903),
    [anon_sym_STAR] = ACTIONS(903),
    [anon_sym_LPAREN] = ACTIONS(903),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_RPAREN] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(901),
    [anon_sym_AMP] = ACTIONS(901),
    [anon_sym_EQ] = ACTIONS(901),
    [anon_sym_where] = ACTIONS(901),
    [anon_sym_COLON] = ACTIONS(903),
    [anon_sym_LBRACE] = ACTIONS(903),
    [anon_sym_RBRACE] = ACTIONS(903),
    [anon_sym_EQ_EQ_GT] = ACTIONS(903),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(903),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(901),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_LT] = ACTIONS(901),
    [anon_sym_GT] = ACTIONS(901),
    [anon_sym_LT_EQ] = ACTIONS(901),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_DOT_DOT] = ACTIONS(903),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(903),
    [anon_sym_DASH] = ACTIONS(903),
    [anon_sym_SLASH] = ACTIONS(901),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_DOT] = ACTIONS(901),
    [anon_sym_LBRACK] = ACTIONS(903),
    [anon_sym_RBRACK] = ACTIONS(903),
    [anon_sym_as] = ACTIONS(901),
    [anon_sym_if] = ACTIONS(901),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_else] = ACTIONS(901),
    [anon_sym_POUND] = ACTIONS(903),
    [anon_sym_spec] = ACTIONS(901),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_with] = ACTIONS(901),
    [anon_sym_to] = ACTIONS(901),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(883)] = {
    [sym_line_comment] = STATE(883),
    [sym_block_comment] = STATE(883),
    [sym_identifier] = ACTIONS(905),
    [anon_sym_u8] = ACTIONS(905),
    [anon_sym_u16] = ACTIONS(905),
    [anon_sym_u32] = ACTIONS(905),
    [anon_sym_u64] = ACTIONS(905),
    [anon_sym_u128] = ACTIONS(905),
    [anon_sym_u256] = ACTIONS(905),
    [anon_sym_exists] = ACTIONS(905),
    [anon_sym_forall] = ACTIONS(905),
    [anon_sym_choose] = ACTIONS(905),
    [anon_sym_min] = ACTIONS(905),
    [anon_sym_bool] = ACTIONS(905),
    [anon_sym_address] = ACTIONS(905),
    [anon_sym_vector] = ACTIONS(905),
    [anon_sym_for] = ACTIONS(905),
    [anon_sym_while] = ACTIONS(905),
    [anon_sym_friend] = ACTIONS(905),
    [anon_sym_match] = ACTIONS(905),
    [aux_sym_number_token1] = ACTIONS(905),
    [aux_sym_number_token2] = ACTIONS(831),
    [aux_sym_number_token4] = ACTIONS(831),
    [aux_sym_number_token6] = ACTIONS(831),
    [anon_sym_STAR] = ACTIONS(831),
    [anon_sym_COLON_COLON] = ACTIONS(831),
    [anon_sym_LPAREN] = ACTIONS(831),
    [anon_sym_COMMA] = ACTIONS(831),
    [anon_sym_RPAREN] = ACTIONS(831),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_EQ] = ACTIONS(905),
    [anon_sym_where] = ACTIONS(905),
    [anon_sym_COLON] = ACTIONS(905),
    [anon_sym_LBRACE] = ACTIONS(831),
    [anon_sym_RBRACE] = ACTIONS(831),
    [anon_sym_EQ_EQ_GT] = ACTIONS(831),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(831),
    [anon_sym_PIPE_PIPE] = ACTIONS(831),
    [anon_sym_AMP_AMP] = ACTIONS(831),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(831),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(831),
    [anon_sym_DOT_DOT] = ACTIONS(831),
    [anon_sym_CARET] = ACTIONS(831),
    [anon_sym_LT_LT] = ACTIONS(831),
    [anon_sym_GT_GT] = ACTIONS(831),
    [anon_sym_PLUS] = ACTIONS(831),
    [anon_sym_DASH] = ACTIONS(831),
    [anon_sym_SLASH] = ACTIONS(905),
    [anon_sym_PERCENT] = ACTIONS(831),
    [anon_sym_DOT] = ACTIONS(905),
    [anon_sym_LBRACK] = ACTIONS(831),
    [anon_sym_RBRACK] = ACTIONS(831),
    [anon_sym_as] = ACTIONS(905),
    [anon_sym_if] = ACTIONS(905),
    [anon_sym_EQ_GT] = ACTIONS(831),
    [anon_sym_else] = ACTIONS(905),
    [anon_sym_spec] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(831),
    [anon_sym_with] = ACTIONS(905),
    [anon_sym_to] = ACTIONS(905),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(884)] = {
    [sym_line_comment] = STATE(884),
    [sym_block_comment] = STATE(884),
    [sym_identifier] = ACTIONS(907),
    [anon_sym_u8] = ACTIONS(907),
    [anon_sym_u16] = ACTIONS(907),
    [anon_sym_u32] = ACTIONS(907),
    [anon_sym_u64] = ACTIONS(907),
    [anon_sym_u128] = ACTIONS(907),
    [anon_sym_u256] = ACTIONS(907),
    [anon_sym_exists] = ACTIONS(907),
    [anon_sym_forall] = ACTIONS(907),
    [anon_sym_choose] = ACTIONS(907),
    [anon_sym_min] = ACTIONS(907),
    [anon_sym_bool] = ACTIONS(907),
    [anon_sym_address] = ACTIONS(907),
    [anon_sym_vector] = ACTIONS(907),
    [anon_sym_for] = ACTIONS(907),
    [anon_sym_while] = ACTIONS(907),
    [anon_sym_friend] = ACTIONS(907),
    [anon_sym_match] = ACTIONS(907),
    [aux_sym_number_token1] = ACTIONS(907),
    [aux_sym_number_token2] = ACTIONS(909),
    [aux_sym_number_token4] = ACTIONS(909),
    [aux_sym_number_token6] = ACTIONS(909),
    [anon_sym_STAR] = ACTIONS(909),
    [anon_sym_COLON_COLON] = ACTIONS(909),
    [anon_sym_LPAREN] = ACTIONS(909),
    [anon_sym_COMMA] = ACTIONS(909),
    [anon_sym_RPAREN] = ACTIONS(909),
    [anon_sym_PIPE] = ACTIONS(907),
    [anon_sym_AMP] = ACTIONS(907),
    [anon_sym_EQ] = ACTIONS(907),
    [anon_sym_where] = ACTIONS(907),
    [anon_sym_COLON] = ACTIONS(907),
    [anon_sym_LBRACE] = ACTIONS(909),
    [anon_sym_RBRACE] = ACTIONS(909),
    [anon_sym_EQ_EQ_GT] = ACTIONS(909),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(909),
    [anon_sym_PIPE_PIPE] = ACTIONS(909),
    [anon_sym_AMP_AMP] = ACTIONS(909),
    [anon_sym_EQ_EQ] = ACTIONS(907),
    [anon_sym_BANG_EQ] = ACTIONS(909),
    [anon_sym_LT] = ACTIONS(907),
    [anon_sym_GT] = ACTIONS(907),
    [anon_sym_LT_EQ] = ACTIONS(907),
    [anon_sym_GT_EQ] = ACTIONS(909),
    [anon_sym_DOT_DOT] = ACTIONS(909),
    [anon_sym_CARET] = ACTIONS(909),
    [anon_sym_LT_LT] = ACTIONS(909),
    [anon_sym_GT_GT] = ACTIONS(909),
    [anon_sym_PLUS] = ACTIONS(909),
    [anon_sym_DASH] = ACTIONS(909),
    [anon_sym_SLASH] = ACTIONS(907),
    [anon_sym_PERCENT] = ACTIONS(909),
    [anon_sym_DOT] = ACTIONS(907),
    [anon_sym_LBRACK] = ACTIONS(909),
    [anon_sym_RBRACK] = ACTIONS(909),
    [anon_sym_as] = ACTIONS(907),
    [anon_sym_if] = ACTIONS(907),
    [anon_sym_EQ_GT] = ACTIONS(909),
    [anon_sym_else] = ACTIONS(907),
    [anon_sym_spec] = ACTIONS(907),
    [anon_sym_SEMI] = ACTIONS(909),
    [anon_sym_with] = ACTIONS(907),
    [anon_sym_to] = ACTIONS(907),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(885)] = {
    [sym_line_comment] = STATE(885),
    [sym_block_comment] = STATE(885),
    [sym_identifier] = ACTIONS(911),
    [anon_sym_u8] = ACTIONS(911),
    [anon_sym_u16] = ACTIONS(911),
    [anon_sym_u32] = ACTIONS(911),
    [anon_sym_u64] = ACTIONS(911),
    [anon_sym_u128] = ACTIONS(911),
    [anon_sym_u256] = ACTIONS(911),
    [anon_sym_exists] = ACTIONS(911),
    [anon_sym_forall] = ACTIONS(911),
    [anon_sym_choose] = ACTIONS(911),
    [anon_sym_min] = ACTIONS(911),
    [anon_sym_bool] = ACTIONS(911),
    [anon_sym_address] = ACTIONS(911),
    [anon_sym_vector] = ACTIONS(911),
    [anon_sym_for] = ACTIONS(911),
    [anon_sym_while] = ACTIONS(911),
    [anon_sym_friend] = ACTIONS(911),
    [anon_sym_match] = ACTIONS(911),
    [aux_sym_number_token1] = ACTIONS(911),
    [aux_sym_number_token2] = ACTIONS(913),
    [aux_sym_number_token4] = ACTIONS(913),
    [aux_sym_number_token6] = ACTIONS(913),
    [anon_sym_STAR] = ACTIONS(913),
    [anon_sym_LPAREN] = ACTIONS(913),
    [anon_sym_COMMA] = ACTIONS(913),
    [anon_sym_RPAREN] = ACTIONS(913),
    [anon_sym_PIPE] = ACTIONS(911),
    [anon_sym_AMP] = ACTIONS(911),
    [anon_sym_EQ] = ACTIONS(911),
    [anon_sym_where] = ACTIONS(911),
    [anon_sym_COLON] = ACTIONS(913),
    [anon_sym_LBRACE] = ACTIONS(913),
    [anon_sym_RBRACE] = ACTIONS(913),
    [anon_sym_EQ_EQ_GT] = ACTIONS(913),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(913),
    [anon_sym_PIPE_PIPE] = ACTIONS(913),
    [anon_sym_AMP_AMP] = ACTIONS(913),
    [anon_sym_EQ_EQ] = ACTIONS(911),
    [anon_sym_BANG_EQ] = ACTIONS(913),
    [anon_sym_LT] = ACTIONS(911),
    [anon_sym_GT] = ACTIONS(911),
    [anon_sym_LT_EQ] = ACTIONS(911),
    [anon_sym_GT_EQ] = ACTIONS(913),
    [anon_sym_DOT_DOT] = ACTIONS(913),
    [anon_sym_CARET] = ACTIONS(913),
    [anon_sym_LT_LT] = ACTIONS(913),
    [anon_sym_GT_GT] = ACTIONS(913),
    [anon_sym_PLUS] = ACTIONS(913),
    [anon_sym_DASH] = ACTIONS(913),
    [anon_sym_SLASH] = ACTIONS(911),
    [anon_sym_PERCENT] = ACTIONS(913),
    [anon_sym_DOT] = ACTIONS(911),
    [anon_sym_LBRACK] = ACTIONS(913),
    [anon_sym_RBRACK] = ACTIONS(913),
    [anon_sym_as] = ACTIONS(911),
    [anon_sym_if] = ACTIONS(911),
    [anon_sym_EQ_GT] = ACTIONS(913),
    [anon_sym_else] = ACTIONS(911),
    [anon_sym_spec] = ACTIONS(911),
    [anon_sym_SEMI] = ACTIONS(913),
    [anon_sym_with] = ACTIONS(911),
    [anon_sym_to] = ACTIONS(911),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(886)] = {
    [sym_line_comment] = STATE(886),
    [sym_block_comment] = STATE(886),
    [sym_identifier] = ACTIONS(915),
    [anon_sym_u8] = ACTIONS(915),
    [anon_sym_u16] = ACTIONS(915),
    [anon_sym_u32] = ACTIONS(915),
    [anon_sym_u64] = ACTIONS(915),
    [anon_sym_u128] = ACTIONS(915),
    [anon_sym_u256] = ACTIONS(915),
    [anon_sym_exists] = ACTIONS(915),
    [anon_sym_forall] = ACTIONS(915),
    [anon_sym_choose] = ACTIONS(915),
    [anon_sym_min] = ACTIONS(915),
    [anon_sym_bool] = ACTIONS(915),
    [anon_sym_address] = ACTIONS(915),
    [anon_sym_vector] = ACTIONS(915),
    [anon_sym_for] = ACTIONS(915),
    [anon_sym_while] = ACTIONS(915),
    [anon_sym_friend] = ACTIONS(915),
    [anon_sym_match] = ACTIONS(915),
    [aux_sym_number_token1] = ACTIONS(915),
    [aux_sym_number_token2] = ACTIONS(917),
    [aux_sym_number_token4] = ACTIONS(917),
    [aux_sym_number_token6] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(917),
    [anon_sym_LPAREN] = ACTIONS(917),
    [anon_sym_COMMA] = ACTIONS(917),
    [anon_sym_RPAREN] = ACTIONS(917),
    [anon_sym_PIPE] = ACTIONS(915),
    [anon_sym_AMP] = ACTIONS(915),
    [anon_sym_EQ] = ACTIONS(915),
    [anon_sym_where] = ACTIONS(915),
    [anon_sym_COLON] = ACTIONS(917),
    [anon_sym_LBRACE] = ACTIONS(917),
    [anon_sym_RBRACE] = ACTIONS(917),
    [anon_sym_EQ_EQ_GT] = ACTIONS(917),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(917),
    [anon_sym_PIPE_PIPE] = ACTIONS(917),
    [anon_sym_AMP_AMP] = ACTIONS(917),
    [anon_sym_EQ_EQ] = ACTIONS(915),
    [anon_sym_BANG_EQ] = ACTIONS(917),
    [anon_sym_LT] = ACTIONS(915),
    [anon_sym_GT] = ACTIONS(915),
    [anon_sym_LT_EQ] = ACTIONS(915),
    [anon_sym_GT_EQ] = ACTIONS(917),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_CARET] = ACTIONS(917),
    [anon_sym_LT_LT] = ACTIONS(917),
    [anon_sym_GT_GT] = ACTIONS(917),
    [anon_sym_PLUS] = ACTIONS(917),
    [anon_sym_DASH] = ACTIONS(917),
    [anon_sym_SLASH] = ACTIONS(915),
    [anon_sym_PERCENT] = ACTIONS(917),
    [anon_sym_DOT] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_RBRACK] = ACTIONS(917),
    [anon_sym_as] = ACTIONS(915),
    [anon_sym_if] = ACTIONS(915),
    [anon_sym_EQ_GT] = ACTIONS(917),
    [anon_sym_else] = ACTIONS(915),
    [anon_sym_spec] = ACTIONS(915),
    [anon_sym_SEMI] = ACTIONS(917),
    [anon_sym_with] = ACTIONS(915),
    [anon_sym_to] = ACTIONS(915),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(887)] = {
    [sym_line_comment] = STATE(887),
    [sym_block_comment] = STATE(887),
    [sym_identifier] = ACTIONS(919),
    [anon_sym_u8] = ACTIONS(919),
    [anon_sym_u16] = ACTIONS(919),
    [anon_sym_u32] = ACTIONS(919),
    [anon_sym_u64] = ACTIONS(919),
    [anon_sym_u128] = ACTIONS(919),
    [anon_sym_u256] = ACTIONS(919),
    [anon_sym_exists] = ACTIONS(919),
    [anon_sym_forall] = ACTIONS(919),
    [anon_sym_choose] = ACTIONS(919),
    [anon_sym_min] = ACTIONS(919),
    [anon_sym_bool] = ACTIONS(919),
    [anon_sym_address] = ACTIONS(919),
    [anon_sym_vector] = ACTIONS(919),
    [anon_sym_for] = ACTIONS(919),
    [anon_sym_while] = ACTIONS(919),
    [anon_sym_friend] = ACTIONS(919),
    [anon_sym_match] = ACTIONS(919),
    [aux_sym_number_token1] = ACTIONS(919),
    [aux_sym_number_token2] = ACTIONS(921),
    [aux_sym_number_token4] = ACTIONS(921),
    [aux_sym_number_token6] = ACTIONS(921),
    [anon_sym_STAR] = ACTIONS(921),
    [anon_sym_LPAREN] = ACTIONS(921),
    [anon_sym_COMMA] = ACTIONS(921),
    [anon_sym_RPAREN] = ACTIONS(921),
    [anon_sym_PIPE] = ACTIONS(919),
    [anon_sym_AMP] = ACTIONS(919),
    [anon_sym_EQ] = ACTIONS(919),
    [anon_sym_where] = ACTIONS(919),
    [anon_sym_COLON] = ACTIONS(921),
    [anon_sym_LBRACE] = ACTIONS(921),
    [anon_sym_RBRACE] = ACTIONS(921),
    [anon_sym_EQ_EQ_GT] = ACTIONS(921),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(921),
    [anon_sym_PIPE_PIPE] = ACTIONS(921),
    [anon_sym_AMP_AMP] = ACTIONS(921),
    [anon_sym_EQ_EQ] = ACTIONS(919),
    [anon_sym_BANG_EQ] = ACTIONS(921),
    [anon_sym_LT] = ACTIONS(919),
    [anon_sym_GT] = ACTIONS(919),
    [anon_sym_LT_EQ] = ACTIONS(919),
    [anon_sym_GT_EQ] = ACTIONS(921),
    [anon_sym_DOT_DOT] = ACTIONS(921),
    [anon_sym_CARET] = ACTIONS(921),
    [anon_sym_LT_LT] = ACTIONS(921),
    [anon_sym_GT_GT] = ACTIONS(921),
    [anon_sym_PLUS] = ACTIONS(921),
    [anon_sym_DASH] = ACTIONS(921),
    [anon_sym_SLASH] = ACTIONS(919),
    [anon_sym_PERCENT] = ACTIONS(921),
    [anon_sym_DOT] = ACTIONS(919),
    [anon_sym_LBRACK] = ACTIONS(921),
    [anon_sym_RBRACK] = ACTIONS(921),
    [anon_sym_as] = ACTIONS(919),
    [anon_sym_if] = ACTIONS(919),
    [anon_sym_EQ_GT] = ACTIONS(921),
    [anon_sym_else] = ACTIONS(919),
    [anon_sym_spec] = ACTIONS(919),
    [anon_sym_SEMI] = ACTIONS(921),
    [anon_sym_with] = ACTIONS(919),
    [anon_sym_to] = ACTIONS(919),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(888)] = {
    [sym_line_comment] = STATE(888),
    [sym_block_comment] = STATE(888),
    [sym_identifier] = ACTIONS(923),
    [anon_sym_u8] = ACTIONS(923),
    [anon_sym_u16] = ACTIONS(923),
    [anon_sym_u32] = ACTIONS(923),
    [anon_sym_u64] = ACTIONS(923),
    [anon_sym_u128] = ACTIONS(923),
    [anon_sym_u256] = ACTIONS(923),
    [anon_sym_exists] = ACTIONS(923),
    [anon_sym_forall] = ACTIONS(923),
    [anon_sym_choose] = ACTIONS(923),
    [anon_sym_min] = ACTIONS(923),
    [anon_sym_bool] = ACTIONS(923),
    [anon_sym_address] = ACTIONS(923),
    [anon_sym_vector] = ACTIONS(923),
    [anon_sym_for] = ACTIONS(923),
    [anon_sym_while] = ACTIONS(923),
    [anon_sym_friend] = ACTIONS(923),
    [anon_sym_match] = ACTIONS(923),
    [aux_sym_number_token1] = ACTIONS(923),
    [aux_sym_number_token2] = ACTIONS(925),
    [aux_sym_number_token4] = ACTIONS(925),
    [aux_sym_number_token6] = ACTIONS(925),
    [anon_sym_STAR] = ACTIONS(925),
    [anon_sym_LPAREN] = ACTIONS(925),
    [anon_sym_COMMA] = ACTIONS(925),
    [anon_sym_RPAREN] = ACTIONS(925),
    [anon_sym_PIPE] = ACTIONS(923),
    [anon_sym_AMP] = ACTIONS(923),
    [anon_sym_EQ] = ACTIONS(923),
    [anon_sym_where] = ACTIONS(923),
    [anon_sym_COLON] = ACTIONS(925),
    [anon_sym_LBRACE] = ACTIONS(925),
    [anon_sym_RBRACE] = ACTIONS(925),
    [anon_sym_EQ_EQ_GT] = ACTIONS(925),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(925),
    [anon_sym_PIPE_PIPE] = ACTIONS(925),
    [anon_sym_AMP_AMP] = ACTIONS(925),
    [anon_sym_EQ_EQ] = ACTIONS(923),
    [anon_sym_BANG_EQ] = ACTIONS(925),
    [anon_sym_LT] = ACTIONS(923),
    [anon_sym_GT] = ACTIONS(923),
    [anon_sym_LT_EQ] = ACTIONS(923),
    [anon_sym_GT_EQ] = ACTIONS(925),
    [anon_sym_DOT_DOT] = ACTIONS(925),
    [anon_sym_CARET] = ACTIONS(925),
    [anon_sym_LT_LT] = ACTIONS(925),
    [anon_sym_GT_GT] = ACTIONS(925),
    [anon_sym_PLUS] = ACTIONS(925),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_SLASH] = ACTIONS(923),
    [anon_sym_PERCENT] = ACTIONS(925),
    [anon_sym_DOT] = ACTIONS(923),
    [anon_sym_LBRACK] = ACTIONS(925),
    [anon_sym_RBRACK] = ACTIONS(925),
    [anon_sym_as] = ACTIONS(923),
    [anon_sym_if] = ACTIONS(923),
    [anon_sym_EQ_GT] = ACTIONS(925),
    [anon_sym_else] = ACTIONS(923),
    [anon_sym_spec] = ACTIONS(923),
    [anon_sym_SEMI] = ACTIONS(925),
    [anon_sym_with] = ACTIONS(923),
    [anon_sym_to] = ACTIONS(923),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(889)] = {
    [sym_line_comment] = STATE(889),
    [sym_block_comment] = STATE(889),
    [sym_identifier] = ACTIONS(927),
    [anon_sym_u8] = ACTIONS(927),
    [anon_sym_u16] = ACTIONS(927),
    [anon_sym_u32] = ACTIONS(927),
    [anon_sym_u64] = ACTIONS(927),
    [anon_sym_u128] = ACTIONS(927),
    [anon_sym_u256] = ACTIONS(927),
    [anon_sym_exists] = ACTIONS(927),
    [anon_sym_forall] = ACTIONS(927),
    [anon_sym_choose] = ACTIONS(927),
    [anon_sym_min] = ACTIONS(927),
    [anon_sym_bool] = ACTIONS(927),
    [anon_sym_address] = ACTIONS(927),
    [anon_sym_vector] = ACTIONS(927),
    [anon_sym_for] = ACTIONS(927),
    [anon_sym_while] = ACTIONS(927),
    [anon_sym_friend] = ACTIONS(927),
    [anon_sym_match] = ACTIONS(927),
    [aux_sym_number_token1] = ACTIONS(927),
    [aux_sym_number_token2] = ACTIONS(929),
    [aux_sym_number_token4] = ACTIONS(929),
    [aux_sym_number_token6] = ACTIONS(929),
    [anon_sym_STAR] = ACTIONS(929),
    [anon_sym_LPAREN] = ACTIONS(929),
    [anon_sym_COMMA] = ACTIONS(929),
    [anon_sym_RPAREN] = ACTIONS(929),
    [anon_sym_PIPE] = ACTIONS(927),
    [anon_sym_AMP] = ACTIONS(927),
    [anon_sym_EQ] = ACTIONS(927),
    [anon_sym_where] = ACTIONS(927),
    [anon_sym_COLON] = ACTIONS(929),
    [anon_sym_LBRACE] = ACTIONS(929),
    [anon_sym_RBRACE] = ACTIONS(929),
    [anon_sym_EQ_EQ_GT] = ACTIONS(929),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(929),
    [anon_sym_PIPE_PIPE] = ACTIONS(929),
    [anon_sym_AMP_AMP] = ACTIONS(929),
    [anon_sym_EQ_EQ] = ACTIONS(927),
    [anon_sym_BANG_EQ] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(927),
    [anon_sym_GT] = ACTIONS(927),
    [anon_sym_LT_EQ] = ACTIONS(927),
    [anon_sym_GT_EQ] = ACTIONS(929),
    [anon_sym_DOT_DOT] = ACTIONS(929),
    [anon_sym_CARET] = ACTIONS(929),
    [anon_sym_LT_LT] = ACTIONS(929),
    [anon_sym_GT_GT] = ACTIONS(929),
    [anon_sym_PLUS] = ACTIONS(929),
    [anon_sym_DASH] = ACTIONS(929),
    [anon_sym_SLASH] = ACTIONS(927),
    [anon_sym_PERCENT] = ACTIONS(929),
    [anon_sym_DOT] = ACTIONS(927),
    [anon_sym_LBRACK] = ACTIONS(929),
    [anon_sym_RBRACK] = ACTIONS(929),
    [anon_sym_as] = ACTIONS(927),
    [anon_sym_if] = ACTIONS(927),
    [anon_sym_EQ_GT] = ACTIONS(929),
    [anon_sym_else] = ACTIONS(927),
    [anon_sym_spec] = ACTIONS(927),
    [anon_sym_SEMI] = ACTIONS(929),
    [anon_sym_with] = ACTIONS(927),
    [anon_sym_to] = ACTIONS(927),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(890)] = {
    [sym_line_comment] = STATE(890),
    [sym_block_comment] = STATE(890),
    [sym_identifier] = ACTIONS(931),
    [anon_sym_u8] = ACTIONS(931),
    [anon_sym_u16] = ACTIONS(931),
    [anon_sym_u32] = ACTIONS(931),
    [anon_sym_u64] = ACTIONS(931),
    [anon_sym_u128] = ACTIONS(931),
    [anon_sym_u256] = ACTIONS(931),
    [anon_sym_exists] = ACTIONS(931),
    [anon_sym_forall] = ACTIONS(931),
    [anon_sym_choose] = ACTIONS(931),
    [anon_sym_min] = ACTIONS(931),
    [anon_sym_bool] = ACTIONS(931),
    [anon_sym_address] = ACTIONS(931),
    [anon_sym_vector] = ACTIONS(931),
    [anon_sym_for] = ACTIONS(931),
    [anon_sym_while] = ACTIONS(931),
    [anon_sym_friend] = ACTIONS(931),
    [anon_sym_match] = ACTIONS(931),
    [aux_sym_number_token1] = ACTIONS(931),
    [aux_sym_number_token2] = ACTIONS(933),
    [aux_sym_number_token4] = ACTIONS(933),
    [aux_sym_number_token6] = ACTIONS(933),
    [anon_sym_STAR] = ACTIONS(933),
    [anon_sym_LPAREN] = ACTIONS(933),
    [anon_sym_COMMA] = ACTIONS(933),
    [anon_sym_RPAREN] = ACTIONS(933),
    [anon_sym_PIPE] = ACTIONS(931),
    [anon_sym_AMP] = ACTIONS(931),
    [anon_sym_EQ] = ACTIONS(931),
    [anon_sym_where] = ACTIONS(931),
    [anon_sym_COLON] = ACTIONS(933),
    [anon_sym_LBRACE] = ACTIONS(933),
    [anon_sym_RBRACE] = ACTIONS(933),
    [anon_sym_EQ_EQ_GT] = ACTIONS(933),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(933),
    [anon_sym_PIPE_PIPE] = ACTIONS(933),
    [anon_sym_AMP_AMP] = ACTIONS(933),
    [anon_sym_EQ_EQ] = ACTIONS(931),
    [anon_sym_BANG_EQ] = ACTIONS(933),
    [anon_sym_LT] = ACTIONS(931),
    [anon_sym_GT] = ACTIONS(931),
    [anon_sym_LT_EQ] = ACTIONS(931),
    [anon_sym_GT_EQ] = ACTIONS(933),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_CARET] = ACTIONS(933),
    [anon_sym_LT_LT] = ACTIONS(933),
    [anon_sym_GT_GT] = ACTIONS(933),
    [anon_sym_PLUS] = ACTIONS(933),
    [anon_sym_DASH] = ACTIONS(933),
    [anon_sym_SLASH] = ACTIONS(931),
    [anon_sym_PERCENT] = ACTIONS(933),
    [anon_sym_DOT] = ACTIONS(931),
    [anon_sym_LBRACK] = ACTIONS(933),
    [anon_sym_RBRACK] = ACTIONS(933),
    [anon_sym_as] = ACTIONS(931),
    [anon_sym_if] = ACTIONS(931),
    [anon_sym_EQ_GT] = ACTIONS(933),
    [anon_sym_else] = ACTIONS(931),
    [anon_sym_spec] = ACTIONS(931),
    [anon_sym_SEMI] = ACTIONS(933),
    [anon_sym_with] = ACTIONS(931),
    [anon_sym_to] = ACTIONS(931),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(891)] = {
    [sym_line_comment] = STATE(891),
    [sym_block_comment] = STATE(891),
    [sym_identifier] = ACTIONS(927),
    [anon_sym_u8] = ACTIONS(927),
    [anon_sym_u16] = ACTIONS(927),
    [anon_sym_u32] = ACTIONS(927),
    [anon_sym_u64] = ACTIONS(927),
    [anon_sym_u128] = ACTIONS(927),
    [anon_sym_u256] = ACTIONS(927),
    [anon_sym_exists] = ACTIONS(927),
    [anon_sym_forall] = ACTIONS(927),
    [anon_sym_choose] = ACTIONS(927),
    [anon_sym_min] = ACTIONS(927),
    [anon_sym_bool] = ACTIONS(927),
    [anon_sym_address] = ACTIONS(927),
    [anon_sym_vector] = ACTIONS(927),
    [anon_sym_for] = ACTIONS(927),
    [anon_sym_while] = ACTIONS(927),
    [anon_sym_friend] = ACTIONS(927),
    [anon_sym_match] = ACTIONS(927),
    [aux_sym_number_token1] = ACTIONS(927),
    [aux_sym_number_token2] = ACTIONS(929),
    [aux_sym_number_token4] = ACTIONS(929),
    [aux_sym_number_token6] = ACTIONS(929),
    [anon_sym_STAR] = ACTIONS(929),
    [anon_sym_LPAREN] = ACTIONS(929),
    [anon_sym_COMMA] = ACTIONS(929),
    [anon_sym_RPAREN] = ACTIONS(929),
    [anon_sym_PIPE] = ACTIONS(927),
    [anon_sym_AMP] = ACTIONS(927),
    [anon_sym_EQ] = ACTIONS(927),
    [anon_sym_where] = ACTIONS(927),
    [anon_sym_COLON] = ACTIONS(929),
    [anon_sym_LBRACE] = ACTIONS(929),
    [anon_sym_RBRACE] = ACTIONS(929),
    [anon_sym_EQ_EQ_GT] = ACTIONS(929),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(929),
    [anon_sym_PIPE_PIPE] = ACTIONS(929),
    [anon_sym_AMP_AMP] = ACTIONS(929),
    [anon_sym_EQ_EQ] = ACTIONS(927),
    [anon_sym_BANG_EQ] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(927),
    [anon_sym_GT] = ACTIONS(927),
    [anon_sym_LT_EQ] = ACTIONS(927),
    [anon_sym_GT_EQ] = ACTIONS(929),
    [anon_sym_DOT_DOT] = ACTIONS(929),
    [anon_sym_CARET] = ACTIONS(929),
    [anon_sym_LT_LT] = ACTIONS(929),
    [anon_sym_GT_GT] = ACTIONS(929),
    [anon_sym_PLUS] = ACTIONS(929),
    [anon_sym_DASH] = ACTIONS(929),
    [anon_sym_SLASH] = ACTIONS(927),
    [anon_sym_PERCENT] = ACTIONS(929),
    [anon_sym_DOT] = ACTIONS(927),
    [anon_sym_LBRACK] = ACTIONS(929),
    [anon_sym_RBRACK] = ACTIONS(929),
    [anon_sym_as] = ACTIONS(927),
    [anon_sym_if] = ACTIONS(927),
    [anon_sym_EQ_GT] = ACTIONS(929),
    [anon_sym_else] = ACTIONS(927),
    [anon_sym_spec] = ACTIONS(927),
    [anon_sym_SEMI] = ACTIONS(929),
    [anon_sym_with] = ACTIONS(927),
    [anon_sym_to] = ACTIONS(927),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(892)] = {
    [sym_line_comment] = STATE(892),
    [sym_block_comment] = STATE(892),
    [sym_identifier] = ACTIONS(935),
    [anon_sym_u8] = ACTIONS(935),
    [anon_sym_u16] = ACTIONS(935),
    [anon_sym_u32] = ACTIONS(935),
    [anon_sym_u64] = ACTIONS(935),
    [anon_sym_u128] = ACTIONS(935),
    [anon_sym_u256] = ACTIONS(935),
    [anon_sym_exists] = ACTIONS(935),
    [anon_sym_forall] = ACTIONS(935),
    [anon_sym_choose] = ACTIONS(935),
    [anon_sym_min] = ACTIONS(935),
    [anon_sym_bool] = ACTIONS(935),
    [anon_sym_address] = ACTIONS(935),
    [anon_sym_vector] = ACTIONS(935),
    [anon_sym_for] = ACTIONS(935),
    [anon_sym_while] = ACTIONS(935),
    [anon_sym_friend] = ACTIONS(935),
    [anon_sym_match] = ACTIONS(935),
    [aux_sym_number_token1] = ACTIONS(935),
    [aux_sym_number_token2] = ACTIONS(937),
    [aux_sym_number_token4] = ACTIONS(937),
    [aux_sym_number_token6] = ACTIONS(937),
    [anon_sym_STAR] = ACTIONS(937),
    [anon_sym_LPAREN] = ACTIONS(937),
    [anon_sym_COMMA] = ACTIONS(937),
    [anon_sym_RPAREN] = ACTIONS(937),
    [anon_sym_PIPE] = ACTIONS(935),
    [anon_sym_AMP] = ACTIONS(935),
    [anon_sym_EQ] = ACTIONS(935),
    [anon_sym_where] = ACTIONS(935),
    [anon_sym_COLON] = ACTIONS(937),
    [anon_sym_LBRACE] = ACTIONS(937),
    [anon_sym_RBRACE] = ACTIONS(937),
    [anon_sym_EQ_EQ_GT] = ACTIONS(937),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(937),
    [anon_sym_PIPE_PIPE] = ACTIONS(937),
    [anon_sym_AMP_AMP] = ACTIONS(937),
    [anon_sym_EQ_EQ] = ACTIONS(935),
    [anon_sym_BANG_EQ] = ACTIONS(937),
    [anon_sym_LT] = ACTIONS(935),
    [anon_sym_GT] = ACTIONS(935),
    [anon_sym_LT_EQ] = ACTIONS(935),
    [anon_sym_GT_EQ] = ACTIONS(937),
    [anon_sym_DOT_DOT] = ACTIONS(937),
    [anon_sym_CARET] = ACTIONS(937),
    [anon_sym_LT_LT] = ACTIONS(937),
    [anon_sym_GT_GT] = ACTIONS(937),
    [anon_sym_PLUS] = ACTIONS(937),
    [anon_sym_DASH] = ACTIONS(937),
    [anon_sym_SLASH] = ACTIONS(935),
    [anon_sym_PERCENT] = ACTIONS(937),
    [anon_sym_DOT] = ACTIONS(935),
    [anon_sym_LBRACK] = ACTIONS(937),
    [anon_sym_RBRACK] = ACTIONS(937),
    [anon_sym_as] = ACTIONS(935),
    [anon_sym_if] = ACTIONS(935),
    [anon_sym_EQ_GT] = ACTIONS(937),
    [anon_sym_else] = ACTIONS(935),
    [anon_sym_spec] = ACTIONS(935),
    [anon_sym_SEMI] = ACTIONS(937),
    [anon_sym_with] = ACTIONS(935),
    [anon_sym_to] = ACTIONS(935),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(893)] = {
    [sym_line_comment] = STATE(893),
    [sym_block_comment] = STATE(893),
    [sym_identifier] = ACTIONS(939),
    [anon_sym_u8] = ACTIONS(939),
    [anon_sym_u16] = ACTIONS(939),
    [anon_sym_u32] = ACTIONS(939),
    [anon_sym_u64] = ACTIONS(939),
    [anon_sym_u128] = ACTIONS(939),
    [anon_sym_u256] = ACTIONS(939),
    [anon_sym_exists] = ACTIONS(939),
    [anon_sym_forall] = ACTIONS(939),
    [anon_sym_choose] = ACTIONS(939),
    [anon_sym_min] = ACTIONS(939),
    [anon_sym_bool] = ACTIONS(939),
    [anon_sym_address] = ACTIONS(939),
    [anon_sym_vector] = ACTIONS(939),
    [anon_sym_for] = ACTIONS(939),
    [anon_sym_while] = ACTIONS(939),
    [anon_sym_friend] = ACTIONS(939),
    [anon_sym_match] = ACTIONS(939),
    [aux_sym_number_token1] = ACTIONS(939),
    [aux_sym_number_token2] = ACTIONS(941),
    [aux_sym_number_token4] = ACTIONS(941),
    [aux_sym_number_token6] = ACTIONS(941),
    [anon_sym_STAR] = ACTIONS(941),
    [anon_sym_LPAREN] = ACTIONS(941),
    [anon_sym_COMMA] = ACTIONS(941),
    [anon_sym_RPAREN] = ACTIONS(941),
    [anon_sym_PIPE] = ACTIONS(939),
    [anon_sym_AMP] = ACTIONS(939),
    [anon_sym_EQ] = ACTIONS(939),
    [anon_sym_where] = ACTIONS(939),
    [anon_sym_COLON] = ACTIONS(941),
    [anon_sym_LBRACE] = ACTIONS(941),
    [anon_sym_RBRACE] = ACTIONS(941),
    [anon_sym_EQ_EQ_GT] = ACTIONS(941),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(941),
    [anon_sym_PIPE_PIPE] = ACTIONS(941),
    [anon_sym_AMP_AMP] = ACTIONS(941),
    [anon_sym_EQ_EQ] = ACTIONS(939),
    [anon_sym_BANG_EQ] = ACTIONS(941),
    [anon_sym_LT] = ACTIONS(939),
    [anon_sym_GT] = ACTIONS(939),
    [anon_sym_LT_EQ] = ACTIONS(939),
    [anon_sym_GT_EQ] = ACTIONS(941),
    [anon_sym_DOT_DOT] = ACTIONS(941),
    [anon_sym_CARET] = ACTIONS(941),
    [anon_sym_LT_LT] = ACTIONS(941),
    [anon_sym_GT_GT] = ACTIONS(941),
    [anon_sym_PLUS] = ACTIONS(941),
    [anon_sym_DASH] = ACTIONS(941),
    [anon_sym_SLASH] = ACTIONS(939),
    [anon_sym_PERCENT] = ACTIONS(941),
    [anon_sym_DOT] = ACTIONS(939),
    [anon_sym_LBRACK] = ACTIONS(941),
    [anon_sym_RBRACK] = ACTIONS(941),
    [anon_sym_as] = ACTIONS(939),
    [anon_sym_if] = ACTIONS(939),
    [anon_sym_EQ_GT] = ACTIONS(941),
    [anon_sym_else] = ACTIONS(939),
    [anon_sym_spec] = ACTIONS(939),
    [anon_sym_SEMI] = ACTIONS(941),
    [anon_sym_with] = ACTIONS(939),
    [anon_sym_to] = ACTIONS(939),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(894)] = {
    [sym_line_comment] = STATE(894),
    [sym_block_comment] = STATE(894),
    [sym_identifier] = ACTIONS(943),
    [anon_sym_u8] = ACTIONS(943),
    [anon_sym_u16] = ACTIONS(943),
    [anon_sym_u32] = ACTIONS(943),
    [anon_sym_u64] = ACTIONS(943),
    [anon_sym_u128] = ACTIONS(943),
    [anon_sym_u256] = ACTIONS(943),
    [anon_sym_exists] = ACTIONS(943),
    [anon_sym_forall] = ACTIONS(943),
    [anon_sym_choose] = ACTIONS(943),
    [anon_sym_min] = ACTIONS(943),
    [anon_sym_bool] = ACTIONS(943),
    [anon_sym_address] = ACTIONS(943),
    [anon_sym_vector] = ACTIONS(943),
    [anon_sym_for] = ACTIONS(943),
    [anon_sym_while] = ACTIONS(943),
    [anon_sym_friend] = ACTIONS(943),
    [anon_sym_match] = ACTIONS(943),
    [aux_sym_number_token1] = ACTIONS(943),
    [aux_sym_number_token2] = ACTIONS(945),
    [aux_sym_number_token4] = ACTIONS(945),
    [aux_sym_number_token6] = ACTIONS(945),
    [anon_sym_STAR] = ACTIONS(945),
    [anon_sym_LPAREN] = ACTIONS(945),
    [anon_sym_COMMA] = ACTIONS(945),
    [anon_sym_RPAREN] = ACTIONS(945),
    [anon_sym_PIPE] = ACTIONS(943),
    [anon_sym_AMP] = ACTIONS(943),
    [anon_sym_EQ] = ACTIONS(943),
    [anon_sym_where] = ACTIONS(943),
    [anon_sym_COLON] = ACTIONS(945),
    [anon_sym_LBRACE] = ACTIONS(945),
    [anon_sym_RBRACE] = ACTIONS(945),
    [anon_sym_EQ_EQ_GT] = ACTIONS(945),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(945),
    [anon_sym_AMP_AMP] = ACTIONS(945),
    [anon_sym_EQ_EQ] = ACTIONS(943),
    [anon_sym_BANG_EQ] = ACTIONS(945),
    [anon_sym_LT] = ACTIONS(943),
    [anon_sym_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(943),
    [anon_sym_GT_EQ] = ACTIONS(945),
    [anon_sym_DOT_DOT] = ACTIONS(945),
    [anon_sym_CARET] = ACTIONS(945),
    [anon_sym_LT_LT] = ACTIONS(945),
    [anon_sym_GT_GT] = ACTIONS(945),
    [anon_sym_PLUS] = ACTIONS(945),
    [anon_sym_DASH] = ACTIONS(945),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(945),
    [anon_sym_DOT] = ACTIONS(943),
    [anon_sym_LBRACK] = ACTIONS(945),
    [anon_sym_RBRACK] = ACTIONS(945),
    [anon_sym_as] = ACTIONS(943),
    [anon_sym_if] = ACTIONS(943),
    [anon_sym_EQ_GT] = ACTIONS(945),
    [anon_sym_else] = ACTIONS(943),
    [anon_sym_spec] = ACTIONS(943),
    [anon_sym_SEMI] = ACTIONS(945),
    [anon_sym_with] = ACTIONS(943),
    [anon_sym_to] = ACTIONS(943),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(895)] = {
    [sym_line_comment] = STATE(895),
    [sym_block_comment] = STATE(895),
    [sym_identifier] = ACTIONS(947),
    [anon_sym_u8] = ACTIONS(947),
    [anon_sym_u16] = ACTIONS(947),
    [anon_sym_u32] = ACTIONS(947),
    [anon_sym_u64] = ACTIONS(947),
    [anon_sym_u128] = ACTIONS(947),
    [anon_sym_u256] = ACTIONS(947),
    [anon_sym_exists] = ACTIONS(947),
    [anon_sym_forall] = ACTIONS(947),
    [anon_sym_choose] = ACTIONS(947),
    [anon_sym_min] = ACTIONS(947),
    [anon_sym_bool] = ACTIONS(947),
    [anon_sym_address] = ACTIONS(947),
    [anon_sym_vector] = ACTIONS(947),
    [anon_sym_for] = ACTIONS(947),
    [anon_sym_while] = ACTIONS(947),
    [anon_sym_friend] = ACTIONS(947),
    [anon_sym_match] = ACTIONS(947),
    [aux_sym_number_token1] = ACTIONS(947),
    [aux_sym_number_token2] = ACTIONS(949),
    [aux_sym_number_token4] = ACTIONS(949),
    [aux_sym_number_token6] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(949),
    [anon_sym_LPAREN] = ACTIONS(949),
    [anon_sym_COMMA] = ACTIONS(949),
    [anon_sym_RPAREN] = ACTIONS(949),
    [anon_sym_PIPE] = ACTIONS(947),
    [anon_sym_AMP] = ACTIONS(947),
    [anon_sym_EQ] = ACTIONS(947),
    [anon_sym_where] = ACTIONS(947),
    [anon_sym_COLON] = ACTIONS(949),
    [anon_sym_LBRACE] = ACTIONS(949),
    [anon_sym_RBRACE] = ACTIONS(949),
    [anon_sym_EQ_EQ_GT] = ACTIONS(949),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(949),
    [anon_sym_PIPE_PIPE] = ACTIONS(949),
    [anon_sym_AMP_AMP] = ACTIONS(949),
    [anon_sym_EQ_EQ] = ACTIONS(947),
    [anon_sym_BANG_EQ] = ACTIONS(949),
    [anon_sym_LT] = ACTIONS(947),
    [anon_sym_GT] = ACTIONS(947),
    [anon_sym_LT_EQ] = ACTIONS(947),
    [anon_sym_GT_EQ] = ACTIONS(949),
    [anon_sym_DOT_DOT] = ACTIONS(949),
    [anon_sym_CARET] = ACTIONS(949),
    [anon_sym_LT_LT] = ACTIONS(949),
    [anon_sym_GT_GT] = ACTIONS(949),
    [anon_sym_PLUS] = ACTIONS(949),
    [anon_sym_DASH] = ACTIONS(949),
    [anon_sym_SLASH] = ACTIONS(947),
    [anon_sym_PERCENT] = ACTIONS(949),
    [anon_sym_DOT] = ACTIONS(947),
    [anon_sym_LBRACK] = ACTIONS(949),
    [anon_sym_RBRACK] = ACTIONS(949),
    [anon_sym_as] = ACTIONS(947),
    [anon_sym_if] = ACTIONS(947),
    [anon_sym_EQ_GT] = ACTIONS(949),
    [anon_sym_else] = ACTIONS(947),
    [anon_sym_spec] = ACTIONS(947),
    [anon_sym_SEMI] = ACTIONS(949),
    [anon_sym_with] = ACTIONS(947),
    [anon_sym_to] = ACTIONS(947),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(896)] = {
    [sym_line_comment] = STATE(896),
    [sym_block_comment] = STATE(896),
    [sym_identifier] = ACTIONS(951),
    [anon_sym_u8] = ACTIONS(951),
    [anon_sym_u16] = ACTIONS(951),
    [anon_sym_u32] = ACTIONS(951),
    [anon_sym_u64] = ACTIONS(951),
    [anon_sym_u128] = ACTIONS(951),
    [anon_sym_u256] = ACTIONS(951),
    [anon_sym_exists] = ACTIONS(951),
    [anon_sym_forall] = ACTIONS(951),
    [anon_sym_choose] = ACTIONS(951),
    [anon_sym_min] = ACTIONS(951),
    [anon_sym_bool] = ACTIONS(951),
    [anon_sym_address] = ACTIONS(951),
    [anon_sym_vector] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(951),
    [anon_sym_while] = ACTIONS(951),
    [anon_sym_friend] = ACTIONS(951),
    [anon_sym_match] = ACTIONS(951),
    [aux_sym_number_token1] = ACTIONS(951),
    [aux_sym_number_token2] = ACTIONS(953),
    [aux_sym_number_token4] = ACTIONS(953),
    [aux_sym_number_token6] = ACTIONS(953),
    [anon_sym_STAR] = ACTIONS(953),
    [anon_sym_LPAREN] = ACTIONS(953),
    [anon_sym_COMMA] = ACTIONS(953),
    [anon_sym_RPAREN] = ACTIONS(953),
    [anon_sym_PIPE] = ACTIONS(951),
    [anon_sym_AMP] = ACTIONS(951),
    [anon_sym_EQ] = ACTIONS(951),
    [anon_sym_where] = ACTIONS(951),
    [anon_sym_COLON] = ACTIONS(953),
    [anon_sym_LBRACE] = ACTIONS(953),
    [anon_sym_RBRACE] = ACTIONS(953),
    [anon_sym_EQ_EQ_GT] = ACTIONS(953),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(953),
    [anon_sym_PIPE_PIPE] = ACTIONS(953),
    [anon_sym_AMP_AMP] = ACTIONS(953),
    [anon_sym_EQ_EQ] = ACTIONS(951),
    [anon_sym_BANG_EQ] = ACTIONS(953),
    [anon_sym_LT] = ACTIONS(951),
    [anon_sym_GT] = ACTIONS(951),
    [anon_sym_LT_EQ] = ACTIONS(951),
    [anon_sym_GT_EQ] = ACTIONS(953),
    [anon_sym_DOT_DOT] = ACTIONS(953),
    [anon_sym_CARET] = ACTIONS(953),
    [anon_sym_LT_LT] = ACTIONS(953),
    [anon_sym_GT_GT] = ACTIONS(953),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_DASH] = ACTIONS(953),
    [anon_sym_SLASH] = ACTIONS(951),
    [anon_sym_PERCENT] = ACTIONS(953),
    [anon_sym_DOT] = ACTIONS(951),
    [anon_sym_LBRACK] = ACTIONS(953),
    [anon_sym_RBRACK] = ACTIONS(953),
    [anon_sym_as] = ACTIONS(951),
    [anon_sym_if] = ACTIONS(951),
    [anon_sym_EQ_GT] = ACTIONS(953),
    [anon_sym_else] = ACTIONS(951),
    [anon_sym_spec] = ACTIONS(951),
    [anon_sym_SEMI] = ACTIONS(953),
    [anon_sym_with] = ACTIONS(951),
    [anon_sym_to] = ACTIONS(951),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(897)] = {
    [sym_line_comment] = STATE(897),
    [sym_block_comment] = STATE(897),
    [sym_identifier] = ACTIONS(955),
    [anon_sym_u8] = ACTIONS(955),
    [anon_sym_u16] = ACTIONS(955),
    [anon_sym_u32] = ACTIONS(955),
    [anon_sym_u64] = ACTIONS(955),
    [anon_sym_u128] = ACTIONS(955),
    [anon_sym_u256] = ACTIONS(955),
    [anon_sym_exists] = ACTIONS(955),
    [anon_sym_forall] = ACTIONS(955),
    [anon_sym_choose] = ACTIONS(955),
    [anon_sym_min] = ACTIONS(955),
    [anon_sym_bool] = ACTIONS(955),
    [anon_sym_address] = ACTIONS(955),
    [anon_sym_vector] = ACTIONS(955),
    [anon_sym_for] = ACTIONS(955),
    [anon_sym_while] = ACTIONS(955),
    [anon_sym_friend] = ACTIONS(955),
    [anon_sym_match] = ACTIONS(955),
    [aux_sym_number_token1] = ACTIONS(955),
    [aux_sym_number_token2] = ACTIONS(957),
    [aux_sym_number_token4] = ACTIONS(957),
    [aux_sym_number_token6] = ACTIONS(957),
    [anon_sym_STAR] = ACTIONS(957),
    [anon_sym_LPAREN] = ACTIONS(957),
    [anon_sym_COMMA] = ACTIONS(957),
    [anon_sym_RPAREN] = ACTIONS(957),
    [anon_sym_PIPE] = ACTIONS(955),
    [anon_sym_AMP] = ACTIONS(955),
    [anon_sym_EQ] = ACTIONS(955),
    [anon_sym_where] = ACTIONS(955),
    [anon_sym_COLON] = ACTIONS(957),
    [anon_sym_LBRACE] = ACTIONS(957),
    [anon_sym_RBRACE] = ACTIONS(957),
    [anon_sym_EQ_EQ_GT] = ACTIONS(957),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(957),
    [anon_sym_PIPE_PIPE] = ACTIONS(957),
    [anon_sym_AMP_AMP] = ACTIONS(957),
    [anon_sym_EQ_EQ] = ACTIONS(955),
    [anon_sym_BANG_EQ] = ACTIONS(957),
    [anon_sym_LT] = ACTIONS(955),
    [anon_sym_GT] = ACTIONS(955),
    [anon_sym_LT_EQ] = ACTIONS(955),
    [anon_sym_GT_EQ] = ACTIONS(957),
    [anon_sym_DOT_DOT] = ACTIONS(957),
    [anon_sym_CARET] = ACTIONS(957),
    [anon_sym_LT_LT] = ACTIONS(957),
    [anon_sym_GT_GT] = ACTIONS(957),
    [anon_sym_PLUS] = ACTIONS(957),
    [anon_sym_DASH] = ACTIONS(957),
    [anon_sym_SLASH] = ACTIONS(955),
    [anon_sym_PERCENT] = ACTIONS(957),
    [anon_sym_DOT] = ACTIONS(955),
    [anon_sym_LBRACK] = ACTIONS(957),
    [anon_sym_RBRACK] = ACTIONS(957),
    [anon_sym_as] = ACTIONS(955),
    [anon_sym_if] = ACTIONS(955),
    [anon_sym_EQ_GT] = ACTIONS(957),
    [anon_sym_else] = ACTIONS(955),
    [anon_sym_spec] = ACTIONS(955),
    [anon_sym_SEMI] = ACTIONS(957),
    [anon_sym_with] = ACTIONS(955),
    [anon_sym_to] = ACTIONS(955),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(898)] = {
    [sym_line_comment] = STATE(898),
    [sym_block_comment] = STATE(898),
    [sym_identifier] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(959),
    [anon_sym_u16] = ACTIONS(959),
    [anon_sym_u32] = ACTIONS(959),
    [anon_sym_u64] = ACTIONS(959),
    [anon_sym_u128] = ACTIONS(959),
    [anon_sym_u256] = ACTIONS(959),
    [anon_sym_exists] = ACTIONS(959),
    [anon_sym_forall] = ACTIONS(959),
    [anon_sym_choose] = ACTIONS(959),
    [anon_sym_min] = ACTIONS(959),
    [anon_sym_bool] = ACTIONS(959),
    [anon_sym_address] = ACTIONS(959),
    [anon_sym_vector] = ACTIONS(959),
    [anon_sym_for] = ACTIONS(959),
    [anon_sym_while] = ACTIONS(959),
    [anon_sym_friend] = ACTIONS(959),
    [anon_sym_match] = ACTIONS(959),
    [aux_sym_number_token1] = ACTIONS(959),
    [aux_sym_number_token2] = ACTIONS(961),
    [aux_sym_number_token4] = ACTIONS(961),
    [aux_sym_number_token6] = ACTIONS(961),
    [anon_sym_STAR] = ACTIONS(961),
    [anon_sym_LPAREN] = ACTIONS(961),
    [anon_sym_COMMA] = ACTIONS(961),
    [anon_sym_RPAREN] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(959),
    [anon_sym_EQ] = ACTIONS(959),
    [anon_sym_where] = ACTIONS(959),
    [anon_sym_COLON] = ACTIONS(961),
    [anon_sym_LBRACE] = ACTIONS(961),
    [anon_sym_RBRACE] = ACTIONS(961),
    [anon_sym_EQ_EQ_GT] = ACTIONS(961),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(961),
    [anon_sym_PIPE_PIPE] = ACTIONS(961),
    [anon_sym_AMP_AMP] = ACTIONS(961),
    [anon_sym_EQ_EQ] = ACTIONS(959),
    [anon_sym_BANG_EQ] = ACTIONS(961),
    [anon_sym_LT] = ACTIONS(959),
    [anon_sym_GT] = ACTIONS(959),
    [anon_sym_LT_EQ] = ACTIONS(959),
    [anon_sym_GT_EQ] = ACTIONS(961),
    [anon_sym_DOT_DOT] = ACTIONS(961),
    [anon_sym_CARET] = ACTIONS(961),
    [anon_sym_LT_LT] = ACTIONS(961),
    [anon_sym_GT_GT] = ACTIONS(961),
    [anon_sym_PLUS] = ACTIONS(961),
    [anon_sym_DASH] = ACTIONS(961),
    [anon_sym_SLASH] = ACTIONS(959),
    [anon_sym_PERCENT] = ACTIONS(961),
    [anon_sym_DOT] = ACTIONS(959),
    [anon_sym_LBRACK] = ACTIONS(961),
    [anon_sym_RBRACK] = ACTIONS(961),
    [anon_sym_as] = ACTIONS(959),
    [anon_sym_if] = ACTIONS(959),
    [anon_sym_EQ_GT] = ACTIONS(961),
    [anon_sym_else] = ACTIONS(959),
    [anon_sym_spec] = ACTIONS(959),
    [anon_sym_SEMI] = ACTIONS(961),
    [anon_sym_with] = ACTIONS(959),
    [anon_sym_to] = ACTIONS(959),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(899)] = {
    [sym_line_comment] = STATE(899),
    [sym_block_comment] = STATE(899),
    [sym_identifier] = ACTIONS(963),
    [anon_sym_u8] = ACTIONS(963),
    [anon_sym_u16] = ACTIONS(963),
    [anon_sym_u32] = ACTIONS(963),
    [anon_sym_u64] = ACTIONS(963),
    [anon_sym_u128] = ACTIONS(963),
    [anon_sym_u256] = ACTIONS(963),
    [anon_sym_exists] = ACTIONS(963),
    [anon_sym_forall] = ACTIONS(963),
    [anon_sym_choose] = ACTIONS(963),
    [anon_sym_min] = ACTIONS(963),
    [anon_sym_bool] = ACTIONS(963),
    [anon_sym_address] = ACTIONS(963),
    [anon_sym_vector] = ACTIONS(963),
    [anon_sym_for] = ACTIONS(963),
    [anon_sym_while] = ACTIONS(963),
    [anon_sym_friend] = ACTIONS(963),
    [anon_sym_match] = ACTIONS(963),
    [aux_sym_number_token1] = ACTIONS(963),
    [aux_sym_number_token2] = ACTIONS(965),
    [aux_sym_number_token4] = ACTIONS(965),
    [aux_sym_number_token6] = ACTIONS(965),
    [anon_sym_STAR] = ACTIONS(965),
    [anon_sym_LPAREN] = ACTIONS(965),
    [anon_sym_COMMA] = ACTIONS(965),
    [anon_sym_RPAREN] = ACTIONS(965),
    [anon_sym_PIPE] = ACTIONS(963),
    [anon_sym_AMP] = ACTIONS(963),
    [anon_sym_EQ] = ACTIONS(963),
    [anon_sym_where] = ACTIONS(963),
    [anon_sym_COLON] = ACTIONS(965),
    [anon_sym_LBRACE] = ACTIONS(965),
    [anon_sym_RBRACE] = ACTIONS(965),
    [anon_sym_EQ_EQ_GT] = ACTIONS(965),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(965),
    [anon_sym_PIPE_PIPE] = ACTIONS(965),
    [anon_sym_AMP_AMP] = ACTIONS(965),
    [anon_sym_EQ_EQ] = ACTIONS(963),
    [anon_sym_BANG_EQ] = ACTIONS(965),
    [anon_sym_LT] = ACTIONS(963),
    [anon_sym_GT] = ACTIONS(963),
    [anon_sym_LT_EQ] = ACTIONS(963),
    [anon_sym_GT_EQ] = ACTIONS(965),
    [anon_sym_DOT_DOT] = ACTIONS(965),
    [anon_sym_CARET] = ACTIONS(965),
    [anon_sym_LT_LT] = ACTIONS(965),
    [anon_sym_GT_GT] = ACTIONS(965),
    [anon_sym_PLUS] = ACTIONS(965),
    [anon_sym_DASH] = ACTIONS(965),
    [anon_sym_SLASH] = ACTIONS(963),
    [anon_sym_PERCENT] = ACTIONS(965),
    [anon_sym_DOT] = ACTIONS(963),
    [anon_sym_LBRACK] = ACTIONS(965),
    [anon_sym_RBRACK] = ACTIONS(965),
    [anon_sym_as] = ACTIONS(963),
    [anon_sym_if] = ACTIONS(963),
    [anon_sym_EQ_GT] = ACTIONS(965),
    [anon_sym_else] = ACTIONS(963),
    [anon_sym_spec] = ACTIONS(963),
    [anon_sym_SEMI] = ACTIONS(965),
    [anon_sym_with] = ACTIONS(963),
    [anon_sym_to] = ACTIONS(963),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(900)] = {
    [sym_line_comment] = STATE(900),
    [sym_block_comment] = STATE(900),
    [sym_identifier] = ACTIONS(967),
    [anon_sym_u8] = ACTIONS(967),
    [anon_sym_u16] = ACTIONS(967),
    [anon_sym_u32] = ACTIONS(967),
    [anon_sym_u64] = ACTIONS(967),
    [anon_sym_u128] = ACTIONS(967),
    [anon_sym_u256] = ACTIONS(967),
    [anon_sym_exists] = ACTIONS(967),
    [anon_sym_forall] = ACTIONS(967),
    [anon_sym_choose] = ACTIONS(967),
    [anon_sym_min] = ACTIONS(967),
    [anon_sym_bool] = ACTIONS(967),
    [anon_sym_address] = ACTIONS(967),
    [anon_sym_vector] = ACTIONS(967),
    [anon_sym_for] = ACTIONS(967),
    [anon_sym_while] = ACTIONS(967),
    [anon_sym_friend] = ACTIONS(967),
    [anon_sym_match] = ACTIONS(967),
    [aux_sym_number_token1] = ACTIONS(967),
    [aux_sym_number_token2] = ACTIONS(969),
    [aux_sym_number_token4] = ACTIONS(969),
    [aux_sym_number_token6] = ACTIONS(969),
    [anon_sym_STAR] = ACTIONS(969),
    [anon_sym_LPAREN] = ACTIONS(969),
    [anon_sym_COMMA] = ACTIONS(969),
    [anon_sym_RPAREN] = ACTIONS(969),
    [anon_sym_PIPE] = ACTIONS(967),
    [anon_sym_AMP] = ACTIONS(967),
    [anon_sym_EQ] = ACTIONS(967),
    [anon_sym_where] = ACTIONS(967),
    [anon_sym_COLON] = ACTIONS(969),
    [anon_sym_LBRACE] = ACTIONS(969),
    [anon_sym_RBRACE] = ACTIONS(969),
    [anon_sym_EQ_EQ_GT] = ACTIONS(969),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(969),
    [anon_sym_PIPE_PIPE] = ACTIONS(969),
    [anon_sym_AMP_AMP] = ACTIONS(969),
    [anon_sym_EQ_EQ] = ACTIONS(967),
    [anon_sym_BANG_EQ] = ACTIONS(969),
    [anon_sym_LT] = ACTIONS(967),
    [anon_sym_GT] = ACTIONS(967),
    [anon_sym_LT_EQ] = ACTIONS(967),
    [anon_sym_GT_EQ] = ACTIONS(969),
    [anon_sym_DOT_DOT] = ACTIONS(969),
    [anon_sym_CARET] = ACTIONS(969),
    [anon_sym_LT_LT] = ACTIONS(969),
    [anon_sym_GT_GT] = ACTIONS(969),
    [anon_sym_PLUS] = ACTIONS(969),
    [anon_sym_DASH] = ACTIONS(969),
    [anon_sym_SLASH] = ACTIONS(967),
    [anon_sym_PERCENT] = ACTIONS(969),
    [anon_sym_DOT] = ACTIONS(967),
    [anon_sym_LBRACK] = ACTIONS(969),
    [anon_sym_RBRACK] = ACTIONS(969),
    [anon_sym_as] = ACTIONS(967),
    [anon_sym_if] = ACTIONS(967),
    [anon_sym_EQ_GT] = ACTIONS(969),
    [anon_sym_else] = ACTIONS(967),
    [anon_sym_spec] = ACTIONS(967),
    [anon_sym_SEMI] = ACTIONS(969),
    [anon_sym_with] = ACTIONS(967),
    [anon_sym_to] = ACTIONS(967),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(901)] = {
    [sym_line_comment] = STATE(901),
    [sym_block_comment] = STATE(901),
    [sym_identifier] = ACTIONS(971),
    [anon_sym_u8] = ACTIONS(971),
    [anon_sym_u16] = ACTIONS(971),
    [anon_sym_u32] = ACTIONS(971),
    [anon_sym_u64] = ACTIONS(971),
    [anon_sym_u128] = ACTIONS(971),
    [anon_sym_u256] = ACTIONS(971),
    [anon_sym_exists] = ACTIONS(971),
    [anon_sym_forall] = ACTIONS(971),
    [anon_sym_choose] = ACTIONS(971),
    [anon_sym_min] = ACTIONS(971),
    [anon_sym_bool] = ACTIONS(971),
    [anon_sym_address] = ACTIONS(971),
    [anon_sym_vector] = ACTIONS(971),
    [anon_sym_for] = ACTIONS(971),
    [anon_sym_while] = ACTIONS(971),
    [anon_sym_friend] = ACTIONS(971),
    [anon_sym_match] = ACTIONS(971),
    [aux_sym_number_token1] = ACTIONS(971),
    [aux_sym_number_token2] = ACTIONS(973),
    [aux_sym_number_token4] = ACTIONS(973),
    [aux_sym_number_token6] = ACTIONS(973),
    [anon_sym_STAR] = ACTIONS(973),
    [anon_sym_LPAREN] = ACTIONS(973),
    [anon_sym_COMMA] = ACTIONS(973),
    [anon_sym_RPAREN] = ACTIONS(973),
    [anon_sym_PIPE] = ACTIONS(971),
    [anon_sym_AMP] = ACTIONS(971),
    [anon_sym_EQ] = ACTIONS(971),
    [anon_sym_where] = ACTIONS(971),
    [anon_sym_COLON] = ACTIONS(973),
    [anon_sym_LBRACE] = ACTIONS(973),
    [anon_sym_RBRACE] = ACTIONS(973),
    [anon_sym_EQ_EQ_GT] = ACTIONS(973),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(973),
    [anon_sym_PIPE_PIPE] = ACTIONS(973),
    [anon_sym_AMP_AMP] = ACTIONS(973),
    [anon_sym_EQ_EQ] = ACTIONS(971),
    [anon_sym_BANG_EQ] = ACTIONS(973),
    [anon_sym_LT] = ACTIONS(971),
    [anon_sym_GT] = ACTIONS(971),
    [anon_sym_LT_EQ] = ACTIONS(971),
    [anon_sym_GT_EQ] = ACTIONS(973),
    [anon_sym_DOT_DOT] = ACTIONS(973),
    [anon_sym_CARET] = ACTIONS(973),
    [anon_sym_LT_LT] = ACTIONS(973),
    [anon_sym_GT_GT] = ACTIONS(973),
    [anon_sym_PLUS] = ACTIONS(973),
    [anon_sym_DASH] = ACTIONS(973),
    [anon_sym_SLASH] = ACTIONS(971),
    [anon_sym_PERCENT] = ACTIONS(973),
    [anon_sym_DOT] = ACTIONS(971),
    [anon_sym_LBRACK] = ACTIONS(973),
    [anon_sym_RBRACK] = ACTIONS(973),
    [anon_sym_as] = ACTIONS(971),
    [anon_sym_if] = ACTIONS(971),
    [anon_sym_EQ_GT] = ACTIONS(973),
    [anon_sym_else] = ACTIONS(971),
    [anon_sym_spec] = ACTIONS(971),
    [anon_sym_SEMI] = ACTIONS(973),
    [anon_sym_with] = ACTIONS(971),
    [anon_sym_to] = ACTIONS(971),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(902)] = {
    [sym_line_comment] = STATE(902),
    [sym_block_comment] = STATE(902),
    [sym_identifier] = ACTIONS(975),
    [anon_sym_u8] = ACTIONS(975),
    [anon_sym_u16] = ACTIONS(975),
    [anon_sym_u32] = ACTIONS(975),
    [anon_sym_u64] = ACTIONS(975),
    [anon_sym_u128] = ACTIONS(975),
    [anon_sym_u256] = ACTIONS(975),
    [anon_sym_exists] = ACTIONS(975),
    [anon_sym_forall] = ACTIONS(975),
    [anon_sym_choose] = ACTIONS(975),
    [anon_sym_min] = ACTIONS(975),
    [anon_sym_bool] = ACTIONS(975),
    [anon_sym_address] = ACTIONS(975),
    [anon_sym_vector] = ACTIONS(975),
    [anon_sym_for] = ACTIONS(975),
    [anon_sym_while] = ACTIONS(975),
    [anon_sym_friend] = ACTIONS(975),
    [anon_sym_match] = ACTIONS(975),
    [aux_sym_number_token1] = ACTIONS(975),
    [aux_sym_number_token2] = ACTIONS(977),
    [aux_sym_number_token4] = ACTIONS(977),
    [aux_sym_number_token6] = ACTIONS(977),
    [anon_sym_STAR] = ACTIONS(977),
    [anon_sym_LPAREN] = ACTIONS(977),
    [anon_sym_COMMA] = ACTIONS(977),
    [anon_sym_RPAREN] = ACTIONS(977),
    [anon_sym_PIPE] = ACTIONS(975),
    [anon_sym_AMP] = ACTIONS(975),
    [anon_sym_EQ] = ACTIONS(975),
    [anon_sym_where] = ACTIONS(975),
    [anon_sym_COLON] = ACTIONS(977),
    [anon_sym_LBRACE] = ACTIONS(977),
    [anon_sym_RBRACE] = ACTIONS(977),
    [anon_sym_EQ_EQ_GT] = ACTIONS(977),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(977),
    [anon_sym_PIPE_PIPE] = ACTIONS(977),
    [anon_sym_AMP_AMP] = ACTIONS(977),
    [anon_sym_EQ_EQ] = ACTIONS(975),
    [anon_sym_BANG_EQ] = ACTIONS(977),
    [anon_sym_LT] = ACTIONS(975),
    [anon_sym_GT] = ACTIONS(975),
    [anon_sym_LT_EQ] = ACTIONS(975),
    [anon_sym_GT_EQ] = ACTIONS(977),
    [anon_sym_DOT_DOT] = ACTIONS(977),
    [anon_sym_CARET] = ACTIONS(977),
    [anon_sym_LT_LT] = ACTIONS(977),
    [anon_sym_GT_GT] = ACTIONS(977),
    [anon_sym_PLUS] = ACTIONS(977),
    [anon_sym_DASH] = ACTIONS(977),
    [anon_sym_SLASH] = ACTIONS(975),
    [anon_sym_PERCENT] = ACTIONS(977),
    [anon_sym_DOT] = ACTIONS(975),
    [anon_sym_LBRACK] = ACTIONS(977),
    [anon_sym_RBRACK] = ACTIONS(977),
    [anon_sym_as] = ACTIONS(975),
    [anon_sym_if] = ACTIONS(975),
    [anon_sym_EQ_GT] = ACTIONS(977),
    [anon_sym_else] = ACTIONS(975),
    [anon_sym_spec] = ACTIONS(975),
    [anon_sym_SEMI] = ACTIONS(977),
    [anon_sym_with] = ACTIONS(975),
    [anon_sym_to] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(903)] = {
    [sym_line_comment] = STATE(903),
    [sym_block_comment] = STATE(903),
    [sym_identifier] = ACTIONS(979),
    [anon_sym_u8] = ACTIONS(979),
    [anon_sym_u16] = ACTIONS(979),
    [anon_sym_u32] = ACTIONS(979),
    [anon_sym_u64] = ACTIONS(979),
    [anon_sym_u128] = ACTIONS(979),
    [anon_sym_u256] = ACTIONS(979),
    [anon_sym_exists] = ACTIONS(979),
    [anon_sym_forall] = ACTIONS(979),
    [anon_sym_choose] = ACTIONS(979),
    [anon_sym_min] = ACTIONS(979),
    [anon_sym_bool] = ACTIONS(979),
    [anon_sym_address] = ACTIONS(979),
    [anon_sym_vector] = ACTIONS(979),
    [anon_sym_for] = ACTIONS(979),
    [anon_sym_while] = ACTIONS(979),
    [anon_sym_friend] = ACTIONS(979),
    [anon_sym_match] = ACTIONS(979),
    [aux_sym_number_token1] = ACTIONS(979),
    [aux_sym_number_token2] = ACTIONS(981),
    [aux_sym_number_token4] = ACTIONS(981),
    [aux_sym_number_token6] = ACTIONS(981),
    [anon_sym_STAR] = ACTIONS(981),
    [anon_sym_LPAREN] = ACTIONS(981),
    [anon_sym_COMMA] = ACTIONS(981),
    [anon_sym_RPAREN] = ACTIONS(981),
    [anon_sym_PIPE] = ACTIONS(979),
    [anon_sym_AMP] = ACTIONS(979),
    [anon_sym_EQ] = ACTIONS(979),
    [anon_sym_where] = ACTIONS(979),
    [anon_sym_COLON] = ACTIONS(981),
    [anon_sym_LBRACE] = ACTIONS(981),
    [anon_sym_RBRACE] = ACTIONS(981),
    [anon_sym_EQ_EQ_GT] = ACTIONS(981),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(981),
    [anon_sym_PIPE_PIPE] = ACTIONS(981),
    [anon_sym_AMP_AMP] = ACTIONS(981),
    [anon_sym_EQ_EQ] = ACTIONS(979),
    [anon_sym_BANG_EQ] = ACTIONS(981),
    [anon_sym_LT] = ACTIONS(979),
    [anon_sym_GT] = ACTIONS(979),
    [anon_sym_LT_EQ] = ACTIONS(979),
    [anon_sym_GT_EQ] = ACTIONS(981),
    [anon_sym_DOT_DOT] = ACTIONS(981),
    [anon_sym_CARET] = ACTIONS(981),
    [anon_sym_LT_LT] = ACTIONS(981),
    [anon_sym_GT_GT] = ACTIONS(981),
    [anon_sym_PLUS] = ACTIONS(981),
    [anon_sym_DASH] = ACTIONS(981),
    [anon_sym_SLASH] = ACTIONS(979),
    [anon_sym_PERCENT] = ACTIONS(981),
    [anon_sym_DOT] = ACTIONS(979),
    [anon_sym_LBRACK] = ACTIONS(981),
    [anon_sym_RBRACK] = ACTIONS(981),
    [anon_sym_as] = ACTIONS(979),
    [anon_sym_if] = ACTIONS(979),
    [anon_sym_EQ_GT] = ACTIONS(981),
    [anon_sym_else] = ACTIONS(979),
    [anon_sym_spec] = ACTIONS(979),
    [anon_sym_SEMI] = ACTIONS(981),
    [anon_sym_with] = ACTIONS(979),
    [anon_sym_to] = ACTIONS(979),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(904)] = {
    [sym_line_comment] = STATE(904),
    [sym_block_comment] = STATE(904),
    [sym_identifier] = ACTIONS(983),
    [anon_sym_u8] = ACTIONS(983),
    [anon_sym_u16] = ACTIONS(983),
    [anon_sym_u32] = ACTIONS(983),
    [anon_sym_u64] = ACTIONS(983),
    [anon_sym_u128] = ACTIONS(983),
    [anon_sym_u256] = ACTIONS(983),
    [anon_sym_exists] = ACTIONS(983),
    [anon_sym_forall] = ACTIONS(983),
    [anon_sym_choose] = ACTIONS(983),
    [anon_sym_min] = ACTIONS(983),
    [anon_sym_bool] = ACTIONS(983),
    [anon_sym_address] = ACTIONS(983),
    [anon_sym_vector] = ACTIONS(983),
    [anon_sym_for] = ACTIONS(983),
    [anon_sym_while] = ACTIONS(983),
    [anon_sym_friend] = ACTIONS(983),
    [anon_sym_match] = ACTIONS(983),
    [aux_sym_number_token1] = ACTIONS(983),
    [aux_sym_number_token2] = ACTIONS(985),
    [aux_sym_number_token4] = ACTIONS(985),
    [aux_sym_number_token6] = ACTIONS(985),
    [anon_sym_STAR] = ACTIONS(985),
    [anon_sym_LPAREN] = ACTIONS(985),
    [anon_sym_COMMA] = ACTIONS(985),
    [anon_sym_RPAREN] = ACTIONS(985),
    [anon_sym_PIPE] = ACTIONS(983),
    [anon_sym_AMP] = ACTIONS(983),
    [anon_sym_EQ] = ACTIONS(983),
    [anon_sym_where] = ACTIONS(983),
    [anon_sym_COLON] = ACTIONS(985),
    [anon_sym_LBRACE] = ACTIONS(985),
    [anon_sym_RBRACE] = ACTIONS(985),
    [anon_sym_EQ_EQ_GT] = ACTIONS(985),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(985),
    [anon_sym_PIPE_PIPE] = ACTIONS(985),
    [anon_sym_AMP_AMP] = ACTIONS(985),
    [anon_sym_EQ_EQ] = ACTIONS(983),
    [anon_sym_BANG_EQ] = ACTIONS(985),
    [anon_sym_LT] = ACTIONS(983),
    [anon_sym_GT] = ACTIONS(983),
    [anon_sym_LT_EQ] = ACTIONS(983),
    [anon_sym_GT_EQ] = ACTIONS(985),
    [anon_sym_DOT_DOT] = ACTIONS(985),
    [anon_sym_CARET] = ACTIONS(985),
    [anon_sym_LT_LT] = ACTIONS(985),
    [anon_sym_GT_GT] = ACTIONS(985),
    [anon_sym_PLUS] = ACTIONS(985),
    [anon_sym_DASH] = ACTIONS(985),
    [anon_sym_SLASH] = ACTIONS(983),
    [anon_sym_PERCENT] = ACTIONS(985),
    [anon_sym_DOT] = ACTIONS(983),
    [anon_sym_LBRACK] = ACTIONS(985),
    [anon_sym_RBRACK] = ACTIONS(985),
    [anon_sym_as] = ACTIONS(983),
    [anon_sym_if] = ACTIONS(983),
    [anon_sym_EQ_GT] = ACTIONS(985),
    [anon_sym_else] = ACTIONS(983),
    [anon_sym_spec] = ACTIONS(983),
    [anon_sym_SEMI] = ACTIONS(985),
    [anon_sym_with] = ACTIONS(983),
    [anon_sym_to] = ACTIONS(983),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(905)] = {
    [sym_line_comment] = STATE(905),
    [sym_block_comment] = STATE(905),
    [sym_identifier] = ACTIONS(987),
    [anon_sym_u8] = ACTIONS(987),
    [anon_sym_u16] = ACTIONS(987),
    [anon_sym_u32] = ACTIONS(987),
    [anon_sym_u64] = ACTIONS(987),
    [anon_sym_u128] = ACTIONS(987),
    [anon_sym_u256] = ACTIONS(987),
    [anon_sym_exists] = ACTIONS(987),
    [anon_sym_forall] = ACTIONS(987),
    [anon_sym_choose] = ACTIONS(987),
    [anon_sym_min] = ACTIONS(987),
    [anon_sym_bool] = ACTIONS(987),
    [anon_sym_address] = ACTIONS(987),
    [anon_sym_vector] = ACTIONS(987),
    [anon_sym_for] = ACTIONS(987),
    [anon_sym_while] = ACTIONS(987),
    [anon_sym_friend] = ACTIONS(987),
    [anon_sym_match] = ACTIONS(987),
    [aux_sym_number_token1] = ACTIONS(987),
    [aux_sym_number_token2] = ACTIONS(989),
    [aux_sym_number_token4] = ACTIONS(989),
    [aux_sym_number_token6] = ACTIONS(989),
    [anon_sym_STAR] = ACTIONS(989),
    [anon_sym_LPAREN] = ACTIONS(989),
    [anon_sym_COMMA] = ACTIONS(989),
    [anon_sym_RPAREN] = ACTIONS(989),
    [anon_sym_PIPE] = ACTIONS(987),
    [anon_sym_AMP] = ACTIONS(987),
    [anon_sym_EQ] = ACTIONS(987),
    [anon_sym_where] = ACTIONS(987),
    [anon_sym_COLON] = ACTIONS(989),
    [anon_sym_LBRACE] = ACTIONS(989),
    [anon_sym_RBRACE] = ACTIONS(989),
    [anon_sym_EQ_EQ_GT] = ACTIONS(989),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(989),
    [anon_sym_PIPE_PIPE] = ACTIONS(989),
    [anon_sym_AMP_AMP] = ACTIONS(989),
    [anon_sym_EQ_EQ] = ACTIONS(987),
    [anon_sym_BANG_EQ] = ACTIONS(989),
    [anon_sym_LT] = ACTIONS(987),
    [anon_sym_GT] = ACTIONS(987),
    [anon_sym_LT_EQ] = ACTIONS(987),
    [anon_sym_GT_EQ] = ACTIONS(989),
    [anon_sym_DOT_DOT] = ACTIONS(989),
    [anon_sym_CARET] = ACTIONS(989),
    [anon_sym_LT_LT] = ACTIONS(989),
    [anon_sym_GT_GT] = ACTIONS(989),
    [anon_sym_PLUS] = ACTIONS(989),
    [anon_sym_DASH] = ACTIONS(989),
    [anon_sym_SLASH] = ACTIONS(987),
    [anon_sym_PERCENT] = ACTIONS(989),
    [anon_sym_DOT] = ACTIONS(987),
    [anon_sym_LBRACK] = ACTIONS(989),
    [anon_sym_RBRACK] = ACTIONS(989),
    [anon_sym_as] = ACTIONS(987),
    [anon_sym_if] = ACTIONS(987),
    [anon_sym_EQ_GT] = ACTIONS(989),
    [anon_sym_else] = ACTIONS(987),
    [anon_sym_spec] = ACTIONS(987),
    [anon_sym_SEMI] = ACTIONS(989),
    [anon_sym_with] = ACTIONS(987),
    [anon_sym_to] = ACTIONS(987),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(906)] = {
    [sym_line_comment] = STATE(906),
    [sym_block_comment] = STATE(906),
    [sym_identifier] = ACTIONS(991),
    [anon_sym_u8] = ACTIONS(991),
    [anon_sym_u16] = ACTIONS(991),
    [anon_sym_u32] = ACTIONS(991),
    [anon_sym_u64] = ACTIONS(991),
    [anon_sym_u128] = ACTIONS(991),
    [anon_sym_u256] = ACTIONS(991),
    [anon_sym_exists] = ACTIONS(991),
    [anon_sym_forall] = ACTIONS(991),
    [anon_sym_choose] = ACTIONS(991),
    [anon_sym_min] = ACTIONS(991),
    [anon_sym_bool] = ACTIONS(991),
    [anon_sym_address] = ACTIONS(991),
    [anon_sym_vector] = ACTIONS(991),
    [anon_sym_for] = ACTIONS(991),
    [anon_sym_while] = ACTIONS(991),
    [anon_sym_friend] = ACTIONS(991),
    [anon_sym_match] = ACTIONS(991),
    [aux_sym_number_token1] = ACTIONS(991),
    [aux_sym_number_token2] = ACTIONS(993),
    [aux_sym_number_token4] = ACTIONS(993),
    [aux_sym_number_token6] = ACTIONS(993),
    [anon_sym_STAR] = ACTIONS(993),
    [anon_sym_LPAREN] = ACTIONS(993),
    [anon_sym_COMMA] = ACTIONS(993),
    [anon_sym_RPAREN] = ACTIONS(993),
    [anon_sym_PIPE] = ACTIONS(991),
    [anon_sym_AMP] = ACTIONS(991),
    [anon_sym_EQ] = ACTIONS(991),
    [anon_sym_where] = ACTIONS(991),
    [anon_sym_COLON] = ACTIONS(993),
    [anon_sym_LBRACE] = ACTIONS(993),
    [anon_sym_RBRACE] = ACTIONS(993),
    [anon_sym_EQ_EQ_GT] = ACTIONS(993),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(993),
    [anon_sym_PIPE_PIPE] = ACTIONS(993),
    [anon_sym_AMP_AMP] = ACTIONS(993),
    [anon_sym_EQ_EQ] = ACTIONS(991),
    [anon_sym_BANG_EQ] = ACTIONS(993),
    [anon_sym_LT] = ACTIONS(991),
    [anon_sym_GT] = ACTIONS(991),
    [anon_sym_LT_EQ] = ACTIONS(991),
    [anon_sym_GT_EQ] = ACTIONS(993),
    [anon_sym_DOT_DOT] = ACTIONS(993),
    [anon_sym_CARET] = ACTIONS(993),
    [anon_sym_LT_LT] = ACTIONS(993),
    [anon_sym_GT_GT] = ACTIONS(993),
    [anon_sym_PLUS] = ACTIONS(993),
    [anon_sym_DASH] = ACTIONS(993),
    [anon_sym_SLASH] = ACTIONS(991),
    [anon_sym_PERCENT] = ACTIONS(993),
    [anon_sym_DOT] = ACTIONS(991),
    [anon_sym_LBRACK] = ACTIONS(993),
    [anon_sym_RBRACK] = ACTIONS(993),
    [anon_sym_as] = ACTIONS(991),
    [anon_sym_if] = ACTIONS(991),
    [anon_sym_EQ_GT] = ACTIONS(993),
    [anon_sym_else] = ACTIONS(991),
    [anon_sym_spec] = ACTIONS(991),
    [anon_sym_SEMI] = ACTIONS(993),
    [anon_sym_with] = ACTIONS(991),
    [anon_sym_to] = ACTIONS(991),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(907)] = {
    [sym_line_comment] = STATE(907),
    [sym_block_comment] = STATE(907),
    [sym_identifier] = ACTIONS(995),
    [anon_sym_u8] = ACTIONS(995),
    [anon_sym_u16] = ACTIONS(995),
    [anon_sym_u32] = ACTIONS(995),
    [anon_sym_u64] = ACTIONS(995),
    [anon_sym_u128] = ACTIONS(995),
    [anon_sym_u256] = ACTIONS(995),
    [anon_sym_exists] = ACTIONS(995),
    [anon_sym_forall] = ACTIONS(995),
    [anon_sym_choose] = ACTIONS(995),
    [anon_sym_min] = ACTIONS(995),
    [anon_sym_bool] = ACTIONS(995),
    [anon_sym_address] = ACTIONS(995),
    [anon_sym_vector] = ACTIONS(995),
    [anon_sym_for] = ACTIONS(995),
    [anon_sym_while] = ACTIONS(995),
    [anon_sym_friend] = ACTIONS(995),
    [anon_sym_match] = ACTIONS(995),
    [aux_sym_number_token1] = ACTIONS(995),
    [aux_sym_number_token2] = ACTIONS(997),
    [aux_sym_number_token4] = ACTIONS(997),
    [aux_sym_number_token6] = ACTIONS(997),
    [anon_sym_STAR] = ACTIONS(997),
    [anon_sym_LPAREN] = ACTIONS(997),
    [anon_sym_COMMA] = ACTIONS(997),
    [anon_sym_RPAREN] = ACTIONS(997),
    [anon_sym_PIPE] = ACTIONS(995),
    [anon_sym_AMP] = ACTIONS(995),
    [anon_sym_EQ] = ACTIONS(995),
    [anon_sym_where] = ACTIONS(995),
    [anon_sym_COLON] = ACTIONS(997),
    [anon_sym_LBRACE] = ACTIONS(997),
    [anon_sym_RBRACE] = ACTIONS(997),
    [anon_sym_EQ_EQ_GT] = ACTIONS(997),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(997),
    [anon_sym_PIPE_PIPE] = ACTIONS(997),
    [anon_sym_AMP_AMP] = ACTIONS(997),
    [anon_sym_EQ_EQ] = ACTIONS(995),
    [anon_sym_BANG_EQ] = ACTIONS(997),
    [anon_sym_LT] = ACTIONS(995),
    [anon_sym_GT] = ACTIONS(995),
    [anon_sym_LT_EQ] = ACTIONS(995),
    [anon_sym_GT_EQ] = ACTIONS(997),
    [anon_sym_DOT_DOT] = ACTIONS(997),
    [anon_sym_CARET] = ACTIONS(997),
    [anon_sym_LT_LT] = ACTIONS(997),
    [anon_sym_GT_GT] = ACTIONS(997),
    [anon_sym_PLUS] = ACTIONS(997),
    [anon_sym_DASH] = ACTIONS(997),
    [anon_sym_SLASH] = ACTIONS(995),
    [anon_sym_PERCENT] = ACTIONS(997),
    [anon_sym_DOT] = ACTIONS(995),
    [anon_sym_LBRACK] = ACTIONS(997),
    [anon_sym_RBRACK] = ACTIONS(997),
    [anon_sym_as] = ACTIONS(995),
    [anon_sym_if] = ACTIONS(995),
    [anon_sym_EQ_GT] = ACTIONS(997),
    [anon_sym_else] = ACTIONS(995),
    [anon_sym_spec] = ACTIONS(995),
    [anon_sym_SEMI] = ACTIONS(997),
    [anon_sym_with] = ACTIONS(995),
    [anon_sym_to] = ACTIONS(995),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(908)] = {
    [sym_line_comment] = STATE(908),
    [sym_block_comment] = STATE(908),
    [sym_identifier] = ACTIONS(999),
    [anon_sym_u8] = ACTIONS(999),
    [anon_sym_u16] = ACTIONS(999),
    [anon_sym_u32] = ACTIONS(999),
    [anon_sym_u64] = ACTIONS(999),
    [anon_sym_u128] = ACTIONS(999),
    [anon_sym_u256] = ACTIONS(999),
    [anon_sym_exists] = ACTIONS(999),
    [anon_sym_forall] = ACTIONS(999),
    [anon_sym_choose] = ACTIONS(999),
    [anon_sym_min] = ACTIONS(999),
    [anon_sym_bool] = ACTIONS(999),
    [anon_sym_address] = ACTIONS(999),
    [anon_sym_vector] = ACTIONS(999),
    [anon_sym_for] = ACTIONS(999),
    [anon_sym_while] = ACTIONS(999),
    [anon_sym_friend] = ACTIONS(999),
    [anon_sym_match] = ACTIONS(999),
    [aux_sym_number_token1] = ACTIONS(999),
    [aux_sym_number_token2] = ACTIONS(1001),
    [aux_sym_number_token4] = ACTIONS(1001),
    [aux_sym_number_token6] = ACTIONS(1001),
    [anon_sym_STAR] = ACTIONS(1001),
    [anon_sym_LPAREN] = ACTIONS(1001),
    [anon_sym_COMMA] = ACTIONS(1001),
    [anon_sym_RPAREN] = ACTIONS(1001),
    [anon_sym_PIPE] = ACTIONS(999),
    [anon_sym_AMP] = ACTIONS(999),
    [anon_sym_EQ] = ACTIONS(999),
    [anon_sym_where] = ACTIONS(999),
    [anon_sym_COLON] = ACTIONS(1001),
    [anon_sym_LBRACE] = ACTIONS(1001),
    [anon_sym_RBRACE] = ACTIONS(1001),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1001),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1001),
    [anon_sym_PIPE_PIPE] = ACTIONS(1001),
    [anon_sym_AMP_AMP] = ACTIONS(1001),
    [anon_sym_EQ_EQ] = ACTIONS(999),
    [anon_sym_BANG_EQ] = ACTIONS(1001),
    [anon_sym_LT] = ACTIONS(999),
    [anon_sym_GT] = ACTIONS(999),
    [anon_sym_LT_EQ] = ACTIONS(999),
    [anon_sym_GT_EQ] = ACTIONS(1001),
    [anon_sym_DOT_DOT] = ACTIONS(1001),
    [anon_sym_CARET] = ACTIONS(1001),
    [anon_sym_LT_LT] = ACTIONS(1001),
    [anon_sym_GT_GT] = ACTIONS(1001),
    [anon_sym_PLUS] = ACTIONS(1001),
    [anon_sym_DASH] = ACTIONS(1001),
    [anon_sym_SLASH] = ACTIONS(999),
    [anon_sym_PERCENT] = ACTIONS(1001),
    [anon_sym_DOT] = ACTIONS(999),
    [anon_sym_LBRACK] = ACTIONS(1001),
    [anon_sym_RBRACK] = ACTIONS(1001),
    [anon_sym_as] = ACTIONS(999),
    [anon_sym_if] = ACTIONS(999),
    [anon_sym_EQ_GT] = ACTIONS(1001),
    [anon_sym_else] = ACTIONS(999),
    [anon_sym_spec] = ACTIONS(999),
    [anon_sym_SEMI] = ACTIONS(1001),
    [anon_sym_with] = ACTIONS(999),
    [anon_sym_to] = ACTIONS(999),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(909)] = {
    [sym_line_comment] = STATE(909),
    [sym_block_comment] = STATE(909),
    [sym_identifier] = ACTIONS(1003),
    [anon_sym_u8] = ACTIONS(1003),
    [anon_sym_u16] = ACTIONS(1003),
    [anon_sym_u32] = ACTIONS(1003),
    [anon_sym_u64] = ACTIONS(1003),
    [anon_sym_u128] = ACTIONS(1003),
    [anon_sym_u256] = ACTIONS(1003),
    [anon_sym_exists] = ACTIONS(1003),
    [anon_sym_forall] = ACTIONS(1003),
    [anon_sym_choose] = ACTIONS(1003),
    [anon_sym_min] = ACTIONS(1003),
    [anon_sym_bool] = ACTIONS(1003),
    [anon_sym_address] = ACTIONS(1003),
    [anon_sym_vector] = ACTIONS(1003),
    [anon_sym_for] = ACTIONS(1003),
    [anon_sym_while] = ACTIONS(1003),
    [anon_sym_friend] = ACTIONS(1003),
    [anon_sym_match] = ACTIONS(1003),
    [aux_sym_number_token1] = ACTIONS(1003),
    [aux_sym_number_token2] = ACTIONS(1005),
    [aux_sym_number_token4] = ACTIONS(1005),
    [aux_sym_number_token6] = ACTIONS(1005),
    [anon_sym_STAR] = ACTIONS(1005),
    [anon_sym_LPAREN] = ACTIONS(1005),
    [anon_sym_COMMA] = ACTIONS(1005),
    [anon_sym_RPAREN] = ACTIONS(1005),
    [anon_sym_PIPE] = ACTIONS(1003),
    [anon_sym_AMP] = ACTIONS(1003),
    [anon_sym_EQ] = ACTIONS(1003),
    [anon_sym_where] = ACTIONS(1003),
    [anon_sym_COLON] = ACTIONS(1005),
    [anon_sym_LBRACE] = ACTIONS(1005),
    [anon_sym_RBRACE] = ACTIONS(1005),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1005),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1005),
    [anon_sym_PIPE_PIPE] = ACTIONS(1005),
    [anon_sym_AMP_AMP] = ACTIONS(1005),
    [anon_sym_EQ_EQ] = ACTIONS(1003),
    [anon_sym_BANG_EQ] = ACTIONS(1005),
    [anon_sym_LT] = ACTIONS(1003),
    [anon_sym_GT] = ACTIONS(1003),
    [anon_sym_LT_EQ] = ACTIONS(1003),
    [anon_sym_GT_EQ] = ACTIONS(1005),
    [anon_sym_DOT_DOT] = ACTIONS(1005),
    [anon_sym_CARET] = ACTIONS(1005),
    [anon_sym_LT_LT] = ACTIONS(1005),
    [anon_sym_GT_GT] = ACTIONS(1005),
    [anon_sym_PLUS] = ACTIONS(1005),
    [anon_sym_DASH] = ACTIONS(1005),
    [anon_sym_SLASH] = ACTIONS(1003),
    [anon_sym_PERCENT] = ACTIONS(1005),
    [anon_sym_DOT] = ACTIONS(1003),
    [anon_sym_LBRACK] = ACTIONS(1005),
    [anon_sym_RBRACK] = ACTIONS(1005),
    [anon_sym_as] = ACTIONS(1003),
    [anon_sym_if] = ACTIONS(1003),
    [anon_sym_EQ_GT] = ACTIONS(1005),
    [anon_sym_else] = ACTIONS(1003),
    [anon_sym_spec] = ACTIONS(1003),
    [anon_sym_SEMI] = ACTIONS(1005),
    [anon_sym_with] = ACTIONS(1003),
    [anon_sym_to] = ACTIONS(1003),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(910)] = {
    [sym_line_comment] = STATE(910),
    [sym_block_comment] = STATE(910),
    [sym_identifier] = ACTIONS(1007),
    [anon_sym_u8] = ACTIONS(1007),
    [anon_sym_u16] = ACTIONS(1007),
    [anon_sym_u32] = ACTIONS(1007),
    [anon_sym_u64] = ACTIONS(1007),
    [anon_sym_u128] = ACTIONS(1007),
    [anon_sym_u256] = ACTIONS(1007),
    [anon_sym_exists] = ACTIONS(1007),
    [anon_sym_forall] = ACTIONS(1007),
    [anon_sym_choose] = ACTIONS(1007),
    [anon_sym_min] = ACTIONS(1007),
    [anon_sym_bool] = ACTIONS(1007),
    [anon_sym_address] = ACTIONS(1007),
    [anon_sym_vector] = ACTIONS(1007),
    [anon_sym_for] = ACTIONS(1007),
    [anon_sym_while] = ACTIONS(1007),
    [anon_sym_friend] = ACTIONS(1007),
    [anon_sym_match] = ACTIONS(1007),
    [aux_sym_number_token1] = ACTIONS(1007),
    [aux_sym_number_token2] = ACTIONS(1009),
    [aux_sym_number_token4] = ACTIONS(1009),
    [aux_sym_number_token6] = ACTIONS(1009),
    [anon_sym_STAR] = ACTIONS(1009),
    [anon_sym_LPAREN] = ACTIONS(1009),
    [anon_sym_COMMA] = ACTIONS(1009),
    [anon_sym_RPAREN] = ACTIONS(1009),
    [anon_sym_PIPE] = ACTIONS(1007),
    [anon_sym_AMP] = ACTIONS(1007),
    [anon_sym_EQ] = ACTIONS(1007),
    [anon_sym_where] = ACTIONS(1007),
    [anon_sym_COLON] = ACTIONS(1009),
    [anon_sym_LBRACE] = ACTIONS(1009),
    [anon_sym_RBRACE] = ACTIONS(1009),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1009),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1009),
    [anon_sym_PIPE_PIPE] = ACTIONS(1009),
    [anon_sym_AMP_AMP] = ACTIONS(1009),
    [anon_sym_EQ_EQ] = ACTIONS(1007),
    [anon_sym_BANG_EQ] = ACTIONS(1009),
    [anon_sym_LT] = ACTIONS(1007),
    [anon_sym_GT] = ACTIONS(1007),
    [anon_sym_LT_EQ] = ACTIONS(1007),
    [anon_sym_GT_EQ] = ACTIONS(1009),
    [anon_sym_DOT_DOT] = ACTIONS(1009),
    [anon_sym_CARET] = ACTIONS(1009),
    [anon_sym_LT_LT] = ACTIONS(1009),
    [anon_sym_GT_GT] = ACTIONS(1009),
    [anon_sym_PLUS] = ACTIONS(1009),
    [anon_sym_DASH] = ACTIONS(1009),
    [anon_sym_SLASH] = ACTIONS(1007),
    [anon_sym_PERCENT] = ACTIONS(1009),
    [anon_sym_DOT] = ACTIONS(1007),
    [anon_sym_LBRACK] = ACTIONS(1009),
    [anon_sym_RBRACK] = ACTIONS(1009),
    [anon_sym_as] = ACTIONS(1007),
    [anon_sym_if] = ACTIONS(1007),
    [anon_sym_EQ_GT] = ACTIONS(1009),
    [anon_sym_else] = ACTIONS(1007),
    [anon_sym_spec] = ACTIONS(1007),
    [anon_sym_SEMI] = ACTIONS(1009),
    [anon_sym_with] = ACTIONS(1007),
    [anon_sym_to] = ACTIONS(1007),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(911)] = {
    [sym_line_comment] = STATE(911),
    [sym_block_comment] = STATE(911),
    [sym_identifier] = ACTIONS(1011),
    [anon_sym_u8] = ACTIONS(1011),
    [anon_sym_u16] = ACTIONS(1011),
    [anon_sym_u32] = ACTIONS(1011),
    [anon_sym_u64] = ACTIONS(1011),
    [anon_sym_u128] = ACTIONS(1011),
    [anon_sym_u256] = ACTIONS(1011),
    [anon_sym_exists] = ACTIONS(1011),
    [anon_sym_forall] = ACTIONS(1011),
    [anon_sym_choose] = ACTIONS(1011),
    [anon_sym_min] = ACTIONS(1011),
    [anon_sym_bool] = ACTIONS(1011),
    [anon_sym_address] = ACTIONS(1011),
    [anon_sym_vector] = ACTIONS(1011),
    [anon_sym_for] = ACTIONS(1011),
    [anon_sym_while] = ACTIONS(1011),
    [anon_sym_friend] = ACTIONS(1011),
    [anon_sym_match] = ACTIONS(1011),
    [aux_sym_number_token1] = ACTIONS(1011),
    [aux_sym_number_token2] = ACTIONS(1013),
    [aux_sym_number_token4] = ACTIONS(1013),
    [aux_sym_number_token6] = ACTIONS(1013),
    [anon_sym_STAR] = ACTIONS(1013),
    [anon_sym_LPAREN] = ACTIONS(1013),
    [anon_sym_COMMA] = ACTIONS(1013),
    [anon_sym_RPAREN] = ACTIONS(1013),
    [anon_sym_PIPE] = ACTIONS(1011),
    [anon_sym_AMP] = ACTIONS(1011),
    [anon_sym_EQ] = ACTIONS(1011),
    [anon_sym_where] = ACTIONS(1011),
    [anon_sym_COLON] = ACTIONS(1013),
    [anon_sym_LBRACE] = ACTIONS(1013),
    [anon_sym_RBRACE] = ACTIONS(1013),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1013),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1013),
    [anon_sym_PIPE_PIPE] = ACTIONS(1013),
    [anon_sym_AMP_AMP] = ACTIONS(1013),
    [anon_sym_EQ_EQ] = ACTIONS(1011),
    [anon_sym_BANG_EQ] = ACTIONS(1013),
    [anon_sym_LT] = ACTIONS(1011),
    [anon_sym_GT] = ACTIONS(1011),
    [anon_sym_LT_EQ] = ACTIONS(1011),
    [anon_sym_GT_EQ] = ACTIONS(1013),
    [anon_sym_DOT_DOT] = ACTIONS(1013),
    [anon_sym_CARET] = ACTIONS(1013),
    [anon_sym_LT_LT] = ACTIONS(1013),
    [anon_sym_GT_GT] = ACTIONS(1013),
    [anon_sym_PLUS] = ACTIONS(1013),
    [anon_sym_DASH] = ACTIONS(1013),
    [anon_sym_SLASH] = ACTIONS(1011),
    [anon_sym_PERCENT] = ACTIONS(1013),
    [anon_sym_DOT] = ACTIONS(1011),
    [anon_sym_LBRACK] = ACTIONS(1013),
    [anon_sym_RBRACK] = ACTIONS(1013),
    [anon_sym_as] = ACTIONS(1011),
    [anon_sym_if] = ACTIONS(1011),
    [anon_sym_EQ_GT] = ACTIONS(1013),
    [anon_sym_else] = ACTIONS(1011),
    [anon_sym_spec] = ACTIONS(1011),
    [anon_sym_SEMI] = ACTIONS(1013),
    [anon_sym_with] = ACTIONS(1011),
    [anon_sym_to] = ACTIONS(1011),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(912)] = {
    [sym_line_comment] = STATE(912),
    [sym_block_comment] = STATE(912),
    [sym_identifier] = ACTIONS(1015),
    [anon_sym_u8] = ACTIONS(1015),
    [anon_sym_u16] = ACTIONS(1015),
    [anon_sym_u32] = ACTIONS(1015),
    [anon_sym_u64] = ACTIONS(1015),
    [anon_sym_u128] = ACTIONS(1015),
    [anon_sym_u256] = ACTIONS(1015),
    [anon_sym_exists] = ACTIONS(1015),
    [anon_sym_forall] = ACTIONS(1015),
    [anon_sym_choose] = ACTIONS(1015),
    [anon_sym_min] = ACTIONS(1015),
    [anon_sym_bool] = ACTIONS(1015),
    [anon_sym_address] = ACTIONS(1015),
    [anon_sym_vector] = ACTIONS(1015),
    [anon_sym_for] = ACTIONS(1015),
    [anon_sym_while] = ACTIONS(1015),
    [anon_sym_friend] = ACTIONS(1015),
    [anon_sym_match] = ACTIONS(1015),
    [aux_sym_number_token1] = ACTIONS(1015),
    [aux_sym_number_token2] = ACTIONS(1017),
    [aux_sym_number_token4] = ACTIONS(1017),
    [aux_sym_number_token6] = ACTIONS(1017),
    [anon_sym_STAR] = ACTIONS(1017),
    [anon_sym_LPAREN] = ACTIONS(1017),
    [anon_sym_COMMA] = ACTIONS(1017),
    [anon_sym_RPAREN] = ACTIONS(1017),
    [anon_sym_PIPE] = ACTIONS(1015),
    [anon_sym_AMP] = ACTIONS(1015),
    [anon_sym_EQ] = ACTIONS(1015),
    [anon_sym_where] = ACTIONS(1015),
    [anon_sym_COLON] = ACTIONS(1017),
    [anon_sym_LBRACE] = ACTIONS(1017),
    [anon_sym_RBRACE] = ACTIONS(1017),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1017),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1017),
    [anon_sym_PIPE_PIPE] = ACTIONS(1017),
    [anon_sym_AMP_AMP] = ACTIONS(1017),
    [anon_sym_EQ_EQ] = ACTIONS(1015),
    [anon_sym_BANG_EQ] = ACTIONS(1017),
    [anon_sym_LT] = ACTIONS(1015),
    [anon_sym_GT] = ACTIONS(1015),
    [anon_sym_LT_EQ] = ACTIONS(1015),
    [anon_sym_GT_EQ] = ACTIONS(1017),
    [anon_sym_DOT_DOT] = ACTIONS(1017),
    [anon_sym_CARET] = ACTIONS(1017),
    [anon_sym_LT_LT] = ACTIONS(1017),
    [anon_sym_GT_GT] = ACTIONS(1017),
    [anon_sym_PLUS] = ACTIONS(1017),
    [anon_sym_DASH] = ACTIONS(1017),
    [anon_sym_SLASH] = ACTIONS(1015),
    [anon_sym_PERCENT] = ACTIONS(1017),
    [anon_sym_DOT] = ACTIONS(1015),
    [anon_sym_LBRACK] = ACTIONS(1017),
    [anon_sym_RBRACK] = ACTIONS(1017),
    [anon_sym_as] = ACTIONS(1015),
    [anon_sym_if] = ACTIONS(1015),
    [anon_sym_EQ_GT] = ACTIONS(1017),
    [anon_sym_else] = ACTIONS(1015),
    [anon_sym_spec] = ACTIONS(1015),
    [anon_sym_SEMI] = ACTIONS(1017),
    [anon_sym_with] = ACTIONS(1015),
    [anon_sym_to] = ACTIONS(1015),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(913)] = {
    [sym_line_comment] = STATE(913),
    [sym_block_comment] = STATE(913),
    [sym_identifier] = ACTIONS(1019),
    [anon_sym_u8] = ACTIONS(1019),
    [anon_sym_u16] = ACTIONS(1019),
    [anon_sym_u32] = ACTIONS(1019),
    [anon_sym_u64] = ACTIONS(1019),
    [anon_sym_u128] = ACTIONS(1019),
    [anon_sym_u256] = ACTIONS(1019),
    [anon_sym_exists] = ACTIONS(1019),
    [anon_sym_forall] = ACTIONS(1019),
    [anon_sym_choose] = ACTIONS(1019),
    [anon_sym_min] = ACTIONS(1019),
    [anon_sym_bool] = ACTIONS(1019),
    [anon_sym_address] = ACTIONS(1019),
    [anon_sym_vector] = ACTIONS(1019),
    [anon_sym_for] = ACTIONS(1019),
    [anon_sym_while] = ACTIONS(1019),
    [anon_sym_friend] = ACTIONS(1019),
    [anon_sym_match] = ACTIONS(1019),
    [aux_sym_number_token1] = ACTIONS(1019),
    [aux_sym_number_token2] = ACTIONS(1021),
    [aux_sym_number_token4] = ACTIONS(1021),
    [aux_sym_number_token6] = ACTIONS(1021),
    [anon_sym_STAR] = ACTIONS(1021),
    [anon_sym_LPAREN] = ACTIONS(1021),
    [anon_sym_COMMA] = ACTIONS(1021),
    [anon_sym_RPAREN] = ACTIONS(1021),
    [anon_sym_PIPE] = ACTIONS(1019),
    [anon_sym_AMP] = ACTIONS(1019),
    [anon_sym_EQ] = ACTIONS(1019),
    [anon_sym_where] = ACTIONS(1019),
    [anon_sym_COLON] = ACTIONS(1021),
    [anon_sym_LBRACE] = ACTIONS(1021),
    [anon_sym_RBRACE] = ACTIONS(1021),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1021),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1021),
    [anon_sym_PIPE_PIPE] = ACTIONS(1021),
    [anon_sym_AMP_AMP] = ACTIONS(1021),
    [anon_sym_EQ_EQ] = ACTIONS(1019),
    [anon_sym_BANG_EQ] = ACTIONS(1021),
    [anon_sym_LT] = ACTIONS(1019),
    [anon_sym_GT] = ACTIONS(1019),
    [anon_sym_LT_EQ] = ACTIONS(1019),
    [anon_sym_GT_EQ] = ACTIONS(1021),
    [anon_sym_DOT_DOT] = ACTIONS(1021),
    [anon_sym_CARET] = ACTIONS(1021),
    [anon_sym_LT_LT] = ACTIONS(1021),
    [anon_sym_GT_GT] = ACTIONS(1021),
    [anon_sym_PLUS] = ACTIONS(1021),
    [anon_sym_DASH] = ACTIONS(1021),
    [anon_sym_SLASH] = ACTIONS(1019),
    [anon_sym_PERCENT] = ACTIONS(1021),
    [anon_sym_DOT] = ACTIONS(1019),
    [anon_sym_LBRACK] = ACTIONS(1021),
    [anon_sym_RBRACK] = ACTIONS(1021),
    [anon_sym_as] = ACTIONS(1019),
    [anon_sym_if] = ACTIONS(1019),
    [anon_sym_EQ_GT] = ACTIONS(1021),
    [anon_sym_else] = ACTIONS(1019),
    [anon_sym_spec] = ACTIONS(1019),
    [anon_sym_SEMI] = ACTIONS(1021),
    [anon_sym_with] = ACTIONS(1019),
    [anon_sym_to] = ACTIONS(1019),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(914)] = {
    [sym_line_comment] = STATE(914),
    [sym_block_comment] = STATE(914),
    [sym_identifier] = ACTIONS(1023),
    [anon_sym_u8] = ACTIONS(1023),
    [anon_sym_u16] = ACTIONS(1023),
    [anon_sym_u32] = ACTIONS(1023),
    [anon_sym_u64] = ACTIONS(1023),
    [anon_sym_u128] = ACTIONS(1023),
    [anon_sym_u256] = ACTIONS(1023),
    [anon_sym_exists] = ACTIONS(1023),
    [anon_sym_forall] = ACTIONS(1023),
    [anon_sym_choose] = ACTIONS(1023),
    [anon_sym_min] = ACTIONS(1023),
    [anon_sym_bool] = ACTIONS(1023),
    [anon_sym_address] = ACTIONS(1023),
    [anon_sym_vector] = ACTIONS(1023),
    [anon_sym_for] = ACTIONS(1023),
    [anon_sym_while] = ACTIONS(1023),
    [anon_sym_friend] = ACTIONS(1023),
    [anon_sym_match] = ACTIONS(1023),
    [aux_sym_number_token1] = ACTIONS(1023),
    [aux_sym_number_token2] = ACTIONS(1025),
    [aux_sym_number_token4] = ACTIONS(1025),
    [aux_sym_number_token6] = ACTIONS(1025),
    [anon_sym_STAR] = ACTIONS(1025),
    [anon_sym_LPAREN] = ACTIONS(1025),
    [anon_sym_COMMA] = ACTIONS(1025),
    [anon_sym_RPAREN] = ACTIONS(1025),
    [anon_sym_PIPE] = ACTIONS(1023),
    [anon_sym_AMP] = ACTIONS(1023),
    [anon_sym_EQ] = ACTIONS(1023),
    [anon_sym_where] = ACTIONS(1023),
    [anon_sym_COLON] = ACTIONS(1025),
    [anon_sym_LBRACE] = ACTIONS(1025),
    [anon_sym_RBRACE] = ACTIONS(1025),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1025),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1025),
    [anon_sym_PIPE_PIPE] = ACTIONS(1025),
    [anon_sym_AMP_AMP] = ACTIONS(1025),
    [anon_sym_EQ_EQ] = ACTIONS(1023),
    [anon_sym_BANG_EQ] = ACTIONS(1025),
    [anon_sym_LT] = ACTIONS(1023),
    [anon_sym_GT] = ACTIONS(1023),
    [anon_sym_LT_EQ] = ACTIONS(1023),
    [anon_sym_GT_EQ] = ACTIONS(1025),
    [anon_sym_DOT_DOT] = ACTIONS(1025),
    [anon_sym_CARET] = ACTIONS(1025),
    [anon_sym_LT_LT] = ACTIONS(1025),
    [anon_sym_GT_GT] = ACTIONS(1025),
    [anon_sym_PLUS] = ACTIONS(1025),
    [anon_sym_DASH] = ACTIONS(1025),
    [anon_sym_SLASH] = ACTIONS(1023),
    [anon_sym_PERCENT] = ACTIONS(1025),
    [anon_sym_DOT] = ACTIONS(1023),
    [anon_sym_LBRACK] = ACTIONS(1025),
    [anon_sym_RBRACK] = ACTIONS(1025),
    [anon_sym_as] = ACTIONS(1023),
    [anon_sym_if] = ACTIONS(1023),
    [anon_sym_EQ_GT] = ACTIONS(1025),
    [anon_sym_else] = ACTIONS(1023),
    [anon_sym_spec] = ACTIONS(1023),
    [anon_sym_SEMI] = ACTIONS(1025),
    [anon_sym_with] = ACTIONS(1023),
    [anon_sym_to] = ACTIONS(1023),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(915)] = {
    [sym_line_comment] = STATE(915),
    [sym_block_comment] = STATE(915),
    [sym_identifier] = ACTIONS(1027),
    [anon_sym_u8] = ACTIONS(1027),
    [anon_sym_u16] = ACTIONS(1027),
    [anon_sym_u32] = ACTIONS(1027),
    [anon_sym_u64] = ACTIONS(1027),
    [anon_sym_u128] = ACTIONS(1027),
    [anon_sym_u256] = ACTIONS(1027),
    [anon_sym_exists] = ACTIONS(1027),
    [anon_sym_forall] = ACTIONS(1027),
    [anon_sym_choose] = ACTIONS(1027),
    [anon_sym_min] = ACTIONS(1027),
    [anon_sym_bool] = ACTIONS(1027),
    [anon_sym_address] = ACTIONS(1027),
    [anon_sym_vector] = ACTIONS(1027),
    [anon_sym_for] = ACTIONS(1027),
    [anon_sym_while] = ACTIONS(1027),
    [anon_sym_friend] = ACTIONS(1027),
    [anon_sym_match] = ACTIONS(1027),
    [aux_sym_number_token1] = ACTIONS(1027),
    [aux_sym_number_token2] = ACTIONS(1029),
    [aux_sym_number_token4] = ACTIONS(1029),
    [aux_sym_number_token6] = ACTIONS(1029),
    [anon_sym_STAR] = ACTIONS(1029),
    [anon_sym_LPAREN] = ACTIONS(1029),
    [anon_sym_COMMA] = ACTIONS(1029),
    [anon_sym_RPAREN] = ACTIONS(1029),
    [anon_sym_PIPE] = ACTIONS(1027),
    [anon_sym_AMP] = ACTIONS(1027),
    [anon_sym_EQ] = ACTIONS(1027),
    [anon_sym_where] = ACTIONS(1027),
    [anon_sym_COLON] = ACTIONS(1029),
    [anon_sym_LBRACE] = ACTIONS(1029),
    [anon_sym_RBRACE] = ACTIONS(1029),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1029),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1029),
    [anon_sym_PIPE_PIPE] = ACTIONS(1029),
    [anon_sym_AMP_AMP] = ACTIONS(1029),
    [anon_sym_EQ_EQ] = ACTIONS(1027),
    [anon_sym_BANG_EQ] = ACTIONS(1029),
    [anon_sym_LT] = ACTIONS(1027),
    [anon_sym_GT] = ACTIONS(1027),
    [anon_sym_LT_EQ] = ACTIONS(1027),
    [anon_sym_GT_EQ] = ACTIONS(1029),
    [anon_sym_DOT_DOT] = ACTIONS(1029),
    [anon_sym_CARET] = ACTIONS(1029),
    [anon_sym_LT_LT] = ACTIONS(1029),
    [anon_sym_GT_GT] = ACTIONS(1029),
    [anon_sym_PLUS] = ACTIONS(1029),
    [anon_sym_DASH] = ACTIONS(1029),
    [anon_sym_SLASH] = ACTIONS(1027),
    [anon_sym_PERCENT] = ACTIONS(1029),
    [anon_sym_DOT] = ACTIONS(1027),
    [anon_sym_LBRACK] = ACTIONS(1029),
    [anon_sym_RBRACK] = ACTIONS(1029),
    [anon_sym_as] = ACTIONS(1027),
    [anon_sym_if] = ACTIONS(1027),
    [anon_sym_EQ_GT] = ACTIONS(1029),
    [anon_sym_else] = ACTIONS(1027),
    [anon_sym_spec] = ACTIONS(1027),
    [anon_sym_SEMI] = ACTIONS(1029),
    [anon_sym_with] = ACTIONS(1027),
    [anon_sym_to] = ACTIONS(1027),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(916)] = {
    [sym_line_comment] = STATE(916),
    [sym_block_comment] = STATE(916),
    [sym_identifier] = ACTIONS(1031),
    [anon_sym_u8] = ACTIONS(1031),
    [anon_sym_u16] = ACTIONS(1031),
    [anon_sym_u32] = ACTIONS(1031),
    [anon_sym_u64] = ACTIONS(1031),
    [anon_sym_u128] = ACTIONS(1031),
    [anon_sym_u256] = ACTIONS(1031),
    [anon_sym_exists] = ACTIONS(1031),
    [anon_sym_forall] = ACTIONS(1031),
    [anon_sym_choose] = ACTIONS(1031),
    [anon_sym_min] = ACTIONS(1031),
    [anon_sym_bool] = ACTIONS(1031),
    [anon_sym_address] = ACTIONS(1031),
    [anon_sym_vector] = ACTIONS(1031),
    [anon_sym_for] = ACTIONS(1031),
    [anon_sym_while] = ACTIONS(1031),
    [anon_sym_friend] = ACTIONS(1031),
    [anon_sym_match] = ACTIONS(1031),
    [aux_sym_number_token1] = ACTIONS(1031),
    [aux_sym_number_token2] = ACTIONS(1033),
    [aux_sym_number_token4] = ACTIONS(1033),
    [aux_sym_number_token6] = ACTIONS(1033),
    [anon_sym_STAR] = ACTIONS(1033),
    [anon_sym_LPAREN] = ACTIONS(1033),
    [anon_sym_COMMA] = ACTIONS(1033),
    [anon_sym_RPAREN] = ACTIONS(1033),
    [anon_sym_PIPE] = ACTIONS(1031),
    [anon_sym_AMP] = ACTIONS(1031),
    [anon_sym_EQ] = ACTIONS(1031),
    [anon_sym_where] = ACTIONS(1031),
    [anon_sym_COLON] = ACTIONS(1033),
    [anon_sym_LBRACE] = ACTIONS(1033),
    [anon_sym_RBRACE] = ACTIONS(1033),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1033),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1033),
    [anon_sym_PIPE_PIPE] = ACTIONS(1033),
    [anon_sym_AMP_AMP] = ACTIONS(1033),
    [anon_sym_EQ_EQ] = ACTIONS(1031),
    [anon_sym_BANG_EQ] = ACTIONS(1033),
    [anon_sym_LT] = ACTIONS(1031),
    [anon_sym_GT] = ACTIONS(1031),
    [anon_sym_LT_EQ] = ACTIONS(1031),
    [anon_sym_GT_EQ] = ACTIONS(1033),
    [anon_sym_DOT_DOT] = ACTIONS(1033),
    [anon_sym_CARET] = ACTIONS(1033),
    [anon_sym_LT_LT] = ACTIONS(1033),
    [anon_sym_GT_GT] = ACTIONS(1033),
    [anon_sym_PLUS] = ACTIONS(1033),
    [anon_sym_DASH] = ACTIONS(1033),
    [anon_sym_SLASH] = ACTIONS(1031),
    [anon_sym_PERCENT] = ACTIONS(1033),
    [anon_sym_DOT] = ACTIONS(1031),
    [anon_sym_LBRACK] = ACTIONS(1033),
    [anon_sym_RBRACK] = ACTIONS(1033),
    [anon_sym_as] = ACTIONS(1031),
    [anon_sym_if] = ACTIONS(1031),
    [anon_sym_EQ_GT] = ACTIONS(1033),
    [anon_sym_else] = ACTIONS(1031),
    [anon_sym_spec] = ACTIONS(1031),
    [anon_sym_SEMI] = ACTIONS(1033),
    [anon_sym_with] = ACTIONS(1031),
    [anon_sym_to] = ACTIONS(1031),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(917)] = {
    [sym_line_comment] = STATE(917),
    [sym_block_comment] = STATE(917),
    [sym_identifier] = ACTIONS(1035),
    [anon_sym_u8] = ACTIONS(1035),
    [anon_sym_u16] = ACTIONS(1035),
    [anon_sym_u32] = ACTIONS(1035),
    [anon_sym_u64] = ACTIONS(1035),
    [anon_sym_u128] = ACTIONS(1035),
    [anon_sym_u256] = ACTIONS(1035),
    [anon_sym_exists] = ACTIONS(1035),
    [anon_sym_forall] = ACTIONS(1035),
    [anon_sym_choose] = ACTIONS(1035),
    [anon_sym_min] = ACTIONS(1035),
    [anon_sym_bool] = ACTIONS(1035),
    [anon_sym_address] = ACTIONS(1035),
    [anon_sym_vector] = ACTIONS(1035),
    [anon_sym_for] = ACTIONS(1035),
    [anon_sym_while] = ACTIONS(1035),
    [anon_sym_friend] = ACTIONS(1035),
    [anon_sym_match] = ACTIONS(1035),
    [aux_sym_number_token1] = ACTIONS(1035),
    [aux_sym_number_token2] = ACTIONS(1037),
    [aux_sym_number_token4] = ACTIONS(1037),
    [aux_sym_number_token6] = ACTIONS(1037),
    [anon_sym_STAR] = ACTIONS(1037),
    [anon_sym_LPAREN] = ACTIONS(1037),
    [anon_sym_COMMA] = ACTIONS(1037),
    [anon_sym_RPAREN] = ACTIONS(1037),
    [anon_sym_PIPE] = ACTIONS(1035),
    [anon_sym_AMP] = ACTIONS(1035),
    [anon_sym_EQ] = ACTIONS(1035),
    [anon_sym_where] = ACTIONS(1035),
    [anon_sym_COLON] = ACTIONS(1037),
    [anon_sym_LBRACE] = ACTIONS(1037),
    [anon_sym_RBRACE] = ACTIONS(1037),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1037),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1037),
    [anon_sym_PIPE_PIPE] = ACTIONS(1037),
    [anon_sym_AMP_AMP] = ACTIONS(1037),
    [anon_sym_EQ_EQ] = ACTIONS(1035),
    [anon_sym_BANG_EQ] = ACTIONS(1037),
    [anon_sym_LT] = ACTIONS(1035),
    [anon_sym_GT] = ACTIONS(1035),
    [anon_sym_LT_EQ] = ACTIONS(1035),
    [anon_sym_GT_EQ] = ACTIONS(1037),
    [anon_sym_DOT_DOT] = ACTIONS(1037),
    [anon_sym_CARET] = ACTIONS(1037),
    [anon_sym_LT_LT] = ACTIONS(1037),
    [anon_sym_GT_GT] = ACTIONS(1037),
    [anon_sym_PLUS] = ACTIONS(1037),
    [anon_sym_DASH] = ACTIONS(1037),
    [anon_sym_SLASH] = ACTIONS(1035),
    [anon_sym_PERCENT] = ACTIONS(1037),
    [anon_sym_DOT] = ACTIONS(1035),
    [anon_sym_LBRACK] = ACTIONS(1037),
    [anon_sym_RBRACK] = ACTIONS(1037),
    [anon_sym_as] = ACTIONS(1035),
    [anon_sym_if] = ACTIONS(1035),
    [anon_sym_EQ_GT] = ACTIONS(1037),
    [anon_sym_else] = ACTIONS(1035),
    [anon_sym_spec] = ACTIONS(1035),
    [anon_sym_SEMI] = ACTIONS(1037),
    [anon_sym_with] = ACTIONS(1035),
    [anon_sym_to] = ACTIONS(1035),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(918)] = {
    [sym_line_comment] = STATE(918),
    [sym_block_comment] = STATE(918),
    [sym_identifier] = ACTIONS(1039),
    [anon_sym_u8] = ACTIONS(1039),
    [anon_sym_u16] = ACTIONS(1039),
    [anon_sym_u32] = ACTIONS(1039),
    [anon_sym_u64] = ACTIONS(1039),
    [anon_sym_u128] = ACTIONS(1039),
    [anon_sym_u256] = ACTIONS(1039),
    [anon_sym_exists] = ACTIONS(1039),
    [anon_sym_forall] = ACTIONS(1039),
    [anon_sym_choose] = ACTIONS(1039),
    [anon_sym_min] = ACTIONS(1039),
    [anon_sym_bool] = ACTIONS(1039),
    [anon_sym_address] = ACTIONS(1039),
    [anon_sym_vector] = ACTIONS(1039),
    [anon_sym_for] = ACTIONS(1039),
    [anon_sym_while] = ACTIONS(1039),
    [anon_sym_friend] = ACTIONS(1039),
    [anon_sym_match] = ACTIONS(1039),
    [aux_sym_number_token1] = ACTIONS(1039),
    [aux_sym_number_token2] = ACTIONS(1041),
    [aux_sym_number_token4] = ACTIONS(1041),
    [aux_sym_number_token6] = ACTIONS(1041),
    [anon_sym_STAR] = ACTIONS(1041),
    [anon_sym_LPAREN] = ACTIONS(1041),
    [anon_sym_COMMA] = ACTIONS(1041),
    [anon_sym_RPAREN] = ACTIONS(1041),
    [anon_sym_PIPE] = ACTIONS(1039),
    [anon_sym_AMP] = ACTIONS(1039),
    [anon_sym_EQ] = ACTIONS(1039),
    [anon_sym_where] = ACTIONS(1039),
    [anon_sym_COLON] = ACTIONS(1041),
    [anon_sym_LBRACE] = ACTIONS(1041),
    [anon_sym_RBRACE] = ACTIONS(1041),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1041),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1041),
    [anon_sym_PIPE_PIPE] = ACTIONS(1041),
    [anon_sym_AMP_AMP] = ACTIONS(1041),
    [anon_sym_EQ_EQ] = ACTIONS(1039),
    [anon_sym_BANG_EQ] = ACTIONS(1041),
    [anon_sym_LT] = ACTIONS(1039),
    [anon_sym_GT] = ACTIONS(1039),
    [anon_sym_LT_EQ] = ACTIONS(1039),
    [anon_sym_GT_EQ] = ACTIONS(1041),
    [anon_sym_DOT_DOT] = ACTIONS(1041),
    [anon_sym_CARET] = ACTIONS(1041),
    [anon_sym_LT_LT] = ACTIONS(1041),
    [anon_sym_GT_GT] = ACTIONS(1041),
    [anon_sym_PLUS] = ACTIONS(1041),
    [anon_sym_DASH] = ACTIONS(1041),
    [anon_sym_SLASH] = ACTIONS(1039),
    [anon_sym_PERCENT] = ACTIONS(1041),
    [anon_sym_DOT] = ACTIONS(1039),
    [anon_sym_LBRACK] = ACTIONS(1041),
    [anon_sym_RBRACK] = ACTIONS(1041),
    [anon_sym_as] = ACTIONS(1039),
    [anon_sym_if] = ACTIONS(1039),
    [anon_sym_EQ_GT] = ACTIONS(1041),
    [anon_sym_else] = ACTIONS(1039),
    [anon_sym_spec] = ACTIONS(1039),
    [anon_sym_SEMI] = ACTIONS(1041),
    [anon_sym_with] = ACTIONS(1039),
    [anon_sym_to] = ACTIONS(1039),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(919)] = {
    [sym_line_comment] = STATE(919),
    [sym_block_comment] = STATE(919),
    [sym_identifier] = ACTIONS(1043),
    [anon_sym_u8] = ACTIONS(1043),
    [anon_sym_u16] = ACTIONS(1043),
    [anon_sym_u32] = ACTIONS(1043),
    [anon_sym_u64] = ACTIONS(1043),
    [anon_sym_u128] = ACTIONS(1043),
    [anon_sym_u256] = ACTIONS(1043),
    [anon_sym_exists] = ACTIONS(1043),
    [anon_sym_forall] = ACTIONS(1043),
    [anon_sym_choose] = ACTIONS(1043),
    [anon_sym_min] = ACTIONS(1043),
    [anon_sym_bool] = ACTIONS(1043),
    [anon_sym_address] = ACTIONS(1043),
    [anon_sym_vector] = ACTIONS(1043),
    [anon_sym_for] = ACTIONS(1043),
    [anon_sym_while] = ACTIONS(1043),
    [anon_sym_friend] = ACTIONS(1043),
    [anon_sym_match] = ACTIONS(1043),
    [aux_sym_number_token1] = ACTIONS(1043),
    [aux_sym_number_token2] = ACTIONS(1045),
    [aux_sym_number_token4] = ACTIONS(1045),
    [aux_sym_number_token6] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(1045),
    [anon_sym_LPAREN] = ACTIONS(1045),
    [anon_sym_COMMA] = ACTIONS(1045),
    [anon_sym_RPAREN] = ACTIONS(1045),
    [anon_sym_PIPE] = ACTIONS(1043),
    [anon_sym_AMP] = ACTIONS(1043),
    [anon_sym_EQ] = ACTIONS(1043),
    [anon_sym_where] = ACTIONS(1043),
    [anon_sym_COLON] = ACTIONS(1045),
    [anon_sym_LBRACE] = ACTIONS(1045),
    [anon_sym_RBRACE] = ACTIONS(1045),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1045),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1045),
    [anon_sym_PIPE_PIPE] = ACTIONS(1045),
    [anon_sym_AMP_AMP] = ACTIONS(1045),
    [anon_sym_EQ_EQ] = ACTIONS(1043),
    [anon_sym_BANG_EQ] = ACTIONS(1045),
    [anon_sym_LT] = ACTIONS(1043),
    [anon_sym_GT] = ACTIONS(1043),
    [anon_sym_LT_EQ] = ACTIONS(1043),
    [anon_sym_GT_EQ] = ACTIONS(1045),
    [anon_sym_DOT_DOT] = ACTIONS(1045),
    [anon_sym_CARET] = ACTIONS(1045),
    [anon_sym_LT_LT] = ACTIONS(1045),
    [anon_sym_GT_GT] = ACTIONS(1045),
    [anon_sym_PLUS] = ACTIONS(1045),
    [anon_sym_DASH] = ACTIONS(1045),
    [anon_sym_SLASH] = ACTIONS(1043),
    [anon_sym_PERCENT] = ACTIONS(1045),
    [anon_sym_DOT] = ACTIONS(1043),
    [anon_sym_LBRACK] = ACTIONS(1045),
    [anon_sym_RBRACK] = ACTIONS(1045),
    [anon_sym_as] = ACTIONS(1043),
    [anon_sym_if] = ACTIONS(1043),
    [anon_sym_EQ_GT] = ACTIONS(1045),
    [anon_sym_else] = ACTIONS(1043),
    [anon_sym_spec] = ACTIONS(1043),
    [anon_sym_SEMI] = ACTIONS(1045),
    [anon_sym_with] = ACTIONS(1043),
    [anon_sym_to] = ACTIONS(1043),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(920)] = {
    [sym_line_comment] = STATE(920),
    [sym_block_comment] = STATE(920),
    [sym_identifier] = ACTIONS(1047),
    [anon_sym_u8] = ACTIONS(1047),
    [anon_sym_u16] = ACTIONS(1047),
    [anon_sym_u32] = ACTIONS(1047),
    [anon_sym_u64] = ACTIONS(1047),
    [anon_sym_u128] = ACTIONS(1047),
    [anon_sym_u256] = ACTIONS(1047),
    [anon_sym_exists] = ACTIONS(1047),
    [anon_sym_forall] = ACTIONS(1047),
    [anon_sym_choose] = ACTIONS(1047),
    [anon_sym_min] = ACTIONS(1047),
    [anon_sym_bool] = ACTIONS(1047),
    [anon_sym_address] = ACTIONS(1047),
    [anon_sym_vector] = ACTIONS(1047),
    [anon_sym_for] = ACTIONS(1047),
    [anon_sym_while] = ACTIONS(1047),
    [anon_sym_friend] = ACTIONS(1047),
    [anon_sym_match] = ACTIONS(1047),
    [aux_sym_number_token1] = ACTIONS(1047),
    [aux_sym_number_token2] = ACTIONS(1049),
    [aux_sym_number_token4] = ACTIONS(1049),
    [aux_sym_number_token6] = ACTIONS(1049),
    [anon_sym_STAR] = ACTIONS(1049),
    [anon_sym_LPAREN] = ACTIONS(1049),
    [anon_sym_COMMA] = ACTIONS(1049),
    [anon_sym_RPAREN] = ACTIONS(1049),
    [anon_sym_PIPE] = ACTIONS(1047),
    [anon_sym_AMP] = ACTIONS(1047),
    [anon_sym_EQ] = ACTIONS(1047),
    [anon_sym_where] = ACTIONS(1047),
    [anon_sym_COLON] = ACTIONS(1049),
    [anon_sym_LBRACE] = ACTIONS(1049),
    [anon_sym_RBRACE] = ACTIONS(1049),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1049),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1049),
    [anon_sym_PIPE_PIPE] = ACTIONS(1049),
    [anon_sym_AMP_AMP] = ACTIONS(1049),
    [anon_sym_EQ_EQ] = ACTIONS(1047),
    [anon_sym_BANG_EQ] = ACTIONS(1049),
    [anon_sym_LT] = ACTIONS(1047),
    [anon_sym_GT] = ACTIONS(1047),
    [anon_sym_LT_EQ] = ACTIONS(1047),
    [anon_sym_GT_EQ] = ACTIONS(1049),
    [anon_sym_DOT_DOT] = ACTIONS(1049),
    [anon_sym_CARET] = ACTIONS(1049),
    [anon_sym_LT_LT] = ACTIONS(1049),
    [anon_sym_GT_GT] = ACTIONS(1049),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_DASH] = ACTIONS(1049),
    [anon_sym_SLASH] = ACTIONS(1047),
    [anon_sym_PERCENT] = ACTIONS(1049),
    [anon_sym_DOT] = ACTIONS(1047),
    [anon_sym_LBRACK] = ACTIONS(1049),
    [anon_sym_RBRACK] = ACTIONS(1049),
    [anon_sym_as] = ACTIONS(1047),
    [anon_sym_if] = ACTIONS(1047),
    [anon_sym_EQ_GT] = ACTIONS(1049),
    [anon_sym_else] = ACTIONS(1047),
    [anon_sym_spec] = ACTIONS(1047),
    [anon_sym_SEMI] = ACTIONS(1049),
    [anon_sym_with] = ACTIONS(1047),
    [anon_sym_to] = ACTIONS(1047),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(921)] = {
    [sym_line_comment] = STATE(921),
    [sym_block_comment] = STATE(921),
    [sym_identifier] = ACTIONS(1051),
    [anon_sym_u8] = ACTIONS(1051),
    [anon_sym_u16] = ACTIONS(1051),
    [anon_sym_u32] = ACTIONS(1051),
    [anon_sym_u64] = ACTIONS(1051),
    [anon_sym_u128] = ACTIONS(1051),
    [anon_sym_u256] = ACTIONS(1051),
    [anon_sym_exists] = ACTIONS(1051),
    [anon_sym_forall] = ACTIONS(1051),
    [anon_sym_choose] = ACTIONS(1051),
    [anon_sym_min] = ACTIONS(1051),
    [anon_sym_bool] = ACTIONS(1051),
    [anon_sym_address] = ACTIONS(1051),
    [anon_sym_vector] = ACTIONS(1051),
    [anon_sym_for] = ACTIONS(1051),
    [anon_sym_while] = ACTIONS(1051),
    [anon_sym_friend] = ACTIONS(1051),
    [anon_sym_match] = ACTIONS(1051),
    [aux_sym_number_token1] = ACTIONS(1051),
    [aux_sym_number_token2] = ACTIONS(1053),
    [aux_sym_number_token4] = ACTIONS(1053),
    [aux_sym_number_token6] = ACTIONS(1053),
    [anon_sym_STAR] = ACTIONS(1053),
    [anon_sym_LPAREN] = ACTIONS(1053),
    [anon_sym_COMMA] = ACTIONS(1053),
    [anon_sym_RPAREN] = ACTIONS(1053),
    [anon_sym_PIPE] = ACTIONS(1051),
    [anon_sym_AMP] = ACTIONS(1051),
    [anon_sym_EQ] = ACTIONS(1051),
    [anon_sym_where] = ACTIONS(1051),
    [anon_sym_COLON] = ACTIONS(1053),
    [anon_sym_LBRACE] = ACTIONS(1053),
    [anon_sym_RBRACE] = ACTIONS(1053),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1053),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1053),
    [anon_sym_PIPE_PIPE] = ACTIONS(1053),
    [anon_sym_AMP_AMP] = ACTIONS(1053),
    [anon_sym_EQ_EQ] = ACTIONS(1051),
    [anon_sym_BANG_EQ] = ACTIONS(1053),
    [anon_sym_LT] = ACTIONS(1051),
    [anon_sym_GT] = ACTIONS(1051),
    [anon_sym_LT_EQ] = ACTIONS(1051),
    [anon_sym_GT_EQ] = ACTIONS(1053),
    [anon_sym_DOT_DOT] = ACTIONS(1053),
    [anon_sym_CARET] = ACTIONS(1053),
    [anon_sym_LT_LT] = ACTIONS(1053),
    [anon_sym_GT_GT] = ACTIONS(1053),
    [anon_sym_PLUS] = ACTIONS(1053),
    [anon_sym_DASH] = ACTIONS(1053),
    [anon_sym_SLASH] = ACTIONS(1051),
    [anon_sym_PERCENT] = ACTIONS(1053),
    [anon_sym_DOT] = ACTIONS(1051),
    [anon_sym_LBRACK] = ACTIONS(1053),
    [anon_sym_RBRACK] = ACTIONS(1053),
    [anon_sym_as] = ACTIONS(1051),
    [anon_sym_if] = ACTIONS(1051),
    [anon_sym_EQ_GT] = ACTIONS(1053),
    [anon_sym_else] = ACTIONS(1051),
    [anon_sym_spec] = ACTIONS(1051),
    [anon_sym_SEMI] = ACTIONS(1053),
    [anon_sym_with] = ACTIONS(1051),
    [anon_sym_to] = ACTIONS(1051),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(922)] = {
    [sym_line_comment] = STATE(922),
    [sym_block_comment] = STATE(922),
    [sym_identifier] = ACTIONS(1055),
    [anon_sym_u8] = ACTIONS(1055),
    [anon_sym_u16] = ACTIONS(1055),
    [anon_sym_u32] = ACTIONS(1055),
    [anon_sym_u64] = ACTIONS(1055),
    [anon_sym_u128] = ACTIONS(1055),
    [anon_sym_u256] = ACTIONS(1055),
    [anon_sym_exists] = ACTIONS(1055),
    [anon_sym_forall] = ACTIONS(1055),
    [anon_sym_choose] = ACTIONS(1055),
    [anon_sym_min] = ACTIONS(1055),
    [anon_sym_bool] = ACTIONS(1055),
    [anon_sym_address] = ACTIONS(1055),
    [anon_sym_vector] = ACTIONS(1055),
    [anon_sym_for] = ACTIONS(1055),
    [anon_sym_while] = ACTIONS(1055),
    [anon_sym_friend] = ACTIONS(1055),
    [anon_sym_match] = ACTIONS(1055),
    [aux_sym_number_token1] = ACTIONS(1055),
    [aux_sym_number_token2] = ACTIONS(1057),
    [aux_sym_number_token4] = ACTIONS(1057),
    [aux_sym_number_token6] = ACTIONS(1057),
    [anon_sym_STAR] = ACTIONS(1057),
    [anon_sym_LPAREN] = ACTIONS(1057),
    [anon_sym_COMMA] = ACTIONS(1057),
    [anon_sym_RPAREN] = ACTIONS(1057),
    [anon_sym_PIPE] = ACTIONS(1055),
    [anon_sym_AMP] = ACTIONS(1055),
    [anon_sym_EQ] = ACTIONS(1055),
    [anon_sym_where] = ACTIONS(1055),
    [anon_sym_COLON] = ACTIONS(1057),
    [anon_sym_LBRACE] = ACTIONS(1057),
    [anon_sym_RBRACE] = ACTIONS(1057),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1057),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1057),
    [anon_sym_PIPE_PIPE] = ACTIONS(1057),
    [anon_sym_AMP_AMP] = ACTIONS(1057),
    [anon_sym_EQ_EQ] = ACTIONS(1055),
    [anon_sym_BANG_EQ] = ACTIONS(1057),
    [anon_sym_LT] = ACTIONS(1055),
    [anon_sym_GT] = ACTIONS(1055),
    [anon_sym_LT_EQ] = ACTIONS(1055),
    [anon_sym_GT_EQ] = ACTIONS(1057),
    [anon_sym_DOT_DOT] = ACTIONS(1057),
    [anon_sym_CARET] = ACTIONS(1057),
    [anon_sym_LT_LT] = ACTIONS(1057),
    [anon_sym_GT_GT] = ACTIONS(1057),
    [anon_sym_PLUS] = ACTIONS(1057),
    [anon_sym_DASH] = ACTIONS(1057),
    [anon_sym_SLASH] = ACTIONS(1055),
    [anon_sym_PERCENT] = ACTIONS(1057),
    [anon_sym_DOT] = ACTIONS(1055),
    [anon_sym_LBRACK] = ACTIONS(1057),
    [anon_sym_RBRACK] = ACTIONS(1057),
    [anon_sym_as] = ACTIONS(1055),
    [anon_sym_if] = ACTIONS(1055),
    [anon_sym_EQ_GT] = ACTIONS(1057),
    [anon_sym_else] = ACTIONS(1055),
    [anon_sym_spec] = ACTIONS(1055),
    [anon_sym_SEMI] = ACTIONS(1057),
    [anon_sym_with] = ACTIONS(1055),
    [anon_sym_to] = ACTIONS(1055),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(923)] = {
    [sym_line_comment] = STATE(923),
    [sym_block_comment] = STATE(923),
    [sym_identifier] = ACTIONS(1059),
    [anon_sym_u8] = ACTIONS(1059),
    [anon_sym_u16] = ACTIONS(1059),
    [anon_sym_u32] = ACTIONS(1059),
    [anon_sym_u64] = ACTIONS(1059),
    [anon_sym_u128] = ACTIONS(1059),
    [anon_sym_u256] = ACTIONS(1059),
    [anon_sym_exists] = ACTIONS(1059),
    [anon_sym_forall] = ACTIONS(1059),
    [anon_sym_choose] = ACTIONS(1059),
    [anon_sym_min] = ACTIONS(1059),
    [anon_sym_bool] = ACTIONS(1059),
    [anon_sym_address] = ACTIONS(1059),
    [anon_sym_vector] = ACTIONS(1059),
    [anon_sym_for] = ACTIONS(1059),
    [anon_sym_while] = ACTIONS(1059),
    [anon_sym_friend] = ACTIONS(1059),
    [anon_sym_match] = ACTIONS(1059),
    [aux_sym_number_token1] = ACTIONS(1059),
    [aux_sym_number_token2] = ACTIONS(1061),
    [aux_sym_number_token4] = ACTIONS(1061),
    [aux_sym_number_token6] = ACTIONS(1061),
    [anon_sym_STAR] = ACTIONS(1061),
    [anon_sym_LPAREN] = ACTIONS(1061),
    [anon_sym_COMMA] = ACTIONS(1061),
    [anon_sym_RPAREN] = ACTIONS(1061),
    [anon_sym_PIPE] = ACTIONS(1059),
    [anon_sym_AMP] = ACTIONS(1059),
    [anon_sym_EQ] = ACTIONS(1059),
    [anon_sym_where] = ACTIONS(1059),
    [anon_sym_COLON] = ACTIONS(1061),
    [anon_sym_LBRACE] = ACTIONS(1061),
    [anon_sym_RBRACE] = ACTIONS(1061),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1061),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1061),
    [anon_sym_PIPE_PIPE] = ACTIONS(1061),
    [anon_sym_AMP_AMP] = ACTIONS(1061),
    [anon_sym_EQ_EQ] = ACTIONS(1059),
    [anon_sym_BANG_EQ] = ACTIONS(1061),
    [anon_sym_LT] = ACTIONS(1059),
    [anon_sym_GT] = ACTIONS(1059),
    [anon_sym_LT_EQ] = ACTIONS(1059),
    [anon_sym_GT_EQ] = ACTIONS(1061),
    [anon_sym_DOT_DOT] = ACTIONS(1061),
    [anon_sym_CARET] = ACTIONS(1061),
    [anon_sym_LT_LT] = ACTIONS(1061),
    [anon_sym_GT_GT] = ACTIONS(1061),
    [anon_sym_PLUS] = ACTIONS(1061),
    [anon_sym_DASH] = ACTIONS(1061),
    [anon_sym_SLASH] = ACTIONS(1059),
    [anon_sym_PERCENT] = ACTIONS(1061),
    [anon_sym_DOT] = ACTIONS(1059),
    [anon_sym_LBRACK] = ACTIONS(1061),
    [anon_sym_RBRACK] = ACTIONS(1061),
    [anon_sym_as] = ACTIONS(1059),
    [anon_sym_if] = ACTIONS(1059),
    [anon_sym_EQ_GT] = ACTIONS(1061),
    [anon_sym_else] = ACTIONS(1059),
    [anon_sym_spec] = ACTIONS(1059),
    [anon_sym_SEMI] = ACTIONS(1061),
    [anon_sym_with] = ACTIONS(1059),
    [anon_sym_to] = ACTIONS(1059),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(924)] = {
    [sym_line_comment] = STATE(924),
    [sym_block_comment] = STATE(924),
    [sym_identifier] = ACTIONS(1063),
    [anon_sym_u8] = ACTIONS(1063),
    [anon_sym_u16] = ACTIONS(1063),
    [anon_sym_u32] = ACTIONS(1063),
    [anon_sym_u64] = ACTIONS(1063),
    [anon_sym_u128] = ACTIONS(1063),
    [anon_sym_u256] = ACTIONS(1063),
    [anon_sym_exists] = ACTIONS(1063),
    [anon_sym_forall] = ACTIONS(1063),
    [anon_sym_choose] = ACTIONS(1063),
    [anon_sym_min] = ACTIONS(1063),
    [anon_sym_bool] = ACTIONS(1063),
    [anon_sym_address] = ACTIONS(1063),
    [anon_sym_vector] = ACTIONS(1063),
    [anon_sym_for] = ACTIONS(1063),
    [anon_sym_while] = ACTIONS(1063),
    [anon_sym_friend] = ACTIONS(1063),
    [anon_sym_match] = ACTIONS(1063),
    [aux_sym_number_token1] = ACTIONS(1063),
    [aux_sym_number_token2] = ACTIONS(1065),
    [aux_sym_number_token4] = ACTIONS(1065),
    [aux_sym_number_token6] = ACTIONS(1065),
    [anon_sym_STAR] = ACTIONS(1065),
    [anon_sym_LPAREN] = ACTIONS(1065),
    [anon_sym_COMMA] = ACTIONS(1065),
    [anon_sym_RPAREN] = ACTIONS(1065),
    [anon_sym_PIPE] = ACTIONS(1063),
    [anon_sym_AMP] = ACTIONS(1063),
    [anon_sym_EQ] = ACTIONS(1063),
    [anon_sym_where] = ACTIONS(1063),
    [anon_sym_COLON] = ACTIONS(1065),
    [anon_sym_LBRACE] = ACTIONS(1065),
    [anon_sym_RBRACE] = ACTIONS(1065),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1065),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1065),
    [anon_sym_PIPE_PIPE] = ACTIONS(1065),
    [anon_sym_AMP_AMP] = ACTIONS(1065),
    [anon_sym_EQ_EQ] = ACTIONS(1063),
    [anon_sym_BANG_EQ] = ACTIONS(1065),
    [anon_sym_LT] = ACTIONS(1063),
    [anon_sym_GT] = ACTIONS(1063),
    [anon_sym_LT_EQ] = ACTIONS(1063),
    [anon_sym_GT_EQ] = ACTIONS(1065),
    [anon_sym_DOT_DOT] = ACTIONS(1065),
    [anon_sym_CARET] = ACTIONS(1065),
    [anon_sym_LT_LT] = ACTIONS(1065),
    [anon_sym_GT_GT] = ACTIONS(1065),
    [anon_sym_PLUS] = ACTIONS(1065),
    [anon_sym_DASH] = ACTIONS(1065),
    [anon_sym_SLASH] = ACTIONS(1063),
    [anon_sym_PERCENT] = ACTIONS(1065),
    [anon_sym_DOT] = ACTIONS(1063),
    [anon_sym_LBRACK] = ACTIONS(1065),
    [anon_sym_RBRACK] = ACTIONS(1065),
    [anon_sym_as] = ACTIONS(1063),
    [anon_sym_if] = ACTIONS(1063),
    [anon_sym_EQ_GT] = ACTIONS(1065),
    [anon_sym_else] = ACTIONS(1063),
    [anon_sym_spec] = ACTIONS(1063),
    [anon_sym_SEMI] = ACTIONS(1065),
    [anon_sym_with] = ACTIONS(1063),
    [anon_sym_to] = ACTIONS(1063),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(925)] = {
    [sym_line_comment] = STATE(925),
    [sym_block_comment] = STATE(925),
    [sym_identifier] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(1067),
    [anon_sym_u16] = ACTIONS(1067),
    [anon_sym_u32] = ACTIONS(1067),
    [anon_sym_u64] = ACTIONS(1067),
    [anon_sym_u128] = ACTIONS(1067),
    [anon_sym_u256] = ACTIONS(1067),
    [anon_sym_exists] = ACTIONS(1067),
    [anon_sym_forall] = ACTIONS(1067),
    [anon_sym_choose] = ACTIONS(1067),
    [anon_sym_min] = ACTIONS(1067),
    [anon_sym_bool] = ACTIONS(1067),
    [anon_sym_address] = ACTIONS(1067),
    [anon_sym_vector] = ACTIONS(1067),
    [anon_sym_for] = ACTIONS(1067),
    [anon_sym_while] = ACTIONS(1067),
    [anon_sym_friend] = ACTIONS(1067),
    [anon_sym_match] = ACTIONS(1067),
    [aux_sym_number_token1] = ACTIONS(1067),
    [aux_sym_number_token2] = ACTIONS(1069),
    [aux_sym_number_token4] = ACTIONS(1069),
    [aux_sym_number_token6] = ACTIONS(1069),
    [anon_sym_STAR] = ACTIONS(1069),
    [anon_sym_LPAREN] = ACTIONS(1069),
    [anon_sym_COMMA] = ACTIONS(1069),
    [anon_sym_RPAREN] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1067),
    [anon_sym_EQ] = ACTIONS(1067),
    [anon_sym_where] = ACTIONS(1067),
    [anon_sym_COLON] = ACTIONS(1069),
    [anon_sym_LBRACE] = ACTIONS(1069),
    [anon_sym_RBRACE] = ACTIONS(1069),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1069),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1069),
    [anon_sym_PIPE_PIPE] = ACTIONS(1069),
    [anon_sym_AMP_AMP] = ACTIONS(1069),
    [anon_sym_EQ_EQ] = ACTIONS(1067),
    [anon_sym_BANG_EQ] = ACTIONS(1069),
    [anon_sym_LT] = ACTIONS(1067),
    [anon_sym_GT] = ACTIONS(1067),
    [anon_sym_LT_EQ] = ACTIONS(1067),
    [anon_sym_GT_EQ] = ACTIONS(1069),
    [anon_sym_DOT_DOT] = ACTIONS(1069),
    [anon_sym_CARET] = ACTIONS(1069),
    [anon_sym_LT_LT] = ACTIONS(1069),
    [anon_sym_GT_GT] = ACTIONS(1069),
    [anon_sym_PLUS] = ACTIONS(1069),
    [anon_sym_DASH] = ACTIONS(1069),
    [anon_sym_SLASH] = ACTIONS(1067),
    [anon_sym_PERCENT] = ACTIONS(1069),
    [anon_sym_DOT] = ACTIONS(1067),
    [anon_sym_LBRACK] = ACTIONS(1069),
    [anon_sym_RBRACK] = ACTIONS(1069),
    [anon_sym_as] = ACTIONS(1067),
    [anon_sym_if] = ACTIONS(1067),
    [anon_sym_EQ_GT] = ACTIONS(1069),
    [anon_sym_else] = ACTIONS(1067),
    [anon_sym_spec] = ACTIONS(1067),
    [anon_sym_SEMI] = ACTIONS(1069),
    [anon_sym_with] = ACTIONS(1067),
    [anon_sym_to] = ACTIONS(1067),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(926)] = {
    [sym_line_comment] = STATE(926),
    [sym_block_comment] = STATE(926),
    [sym_identifier] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(1067),
    [anon_sym_u16] = ACTIONS(1067),
    [anon_sym_u32] = ACTIONS(1067),
    [anon_sym_u64] = ACTIONS(1067),
    [anon_sym_u128] = ACTIONS(1067),
    [anon_sym_u256] = ACTIONS(1067),
    [anon_sym_exists] = ACTIONS(1067),
    [anon_sym_forall] = ACTIONS(1067),
    [anon_sym_choose] = ACTIONS(1067),
    [anon_sym_min] = ACTIONS(1067),
    [anon_sym_bool] = ACTIONS(1067),
    [anon_sym_address] = ACTIONS(1067),
    [anon_sym_vector] = ACTIONS(1067),
    [anon_sym_for] = ACTIONS(1067),
    [anon_sym_while] = ACTIONS(1067),
    [anon_sym_friend] = ACTIONS(1067),
    [anon_sym_match] = ACTIONS(1067),
    [aux_sym_number_token1] = ACTIONS(1067),
    [aux_sym_number_token2] = ACTIONS(1069),
    [aux_sym_number_token4] = ACTIONS(1069),
    [aux_sym_number_token6] = ACTIONS(1069),
    [anon_sym_STAR] = ACTIONS(1069),
    [anon_sym_LPAREN] = ACTIONS(1069),
    [anon_sym_COMMA] = ACTIONS(1069),
    [anon_sym_RPAREN] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1067),
    [anon_sym_EQ] = ACTIONS(1067),
    [anon_sym_where] = ACTIONS(1067),
    [anon_sym_COLON] = ACTIONS(1069),
    [anon_sym_LBRACE] = ACTIONS(1069),
    [anon_sym_RBRACE] = ACTIONS(1069),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1069),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1069),
    [anon_sym_PIPE_PIPE] = ACTIONS(1069),
    [anon_sym_AMP_AMP] = ACTIONS(1069),
    [anon_sym_EQ_EQ] = ACTIONS(1067),
    [anon_sym_BANG_EQ] = ACTIONS(1069),
    [anon_sym_LT] = ACTIONS(1067),
    [anon_sym_GT] = ACTIONS(1067),
    [anon_sym_LT_EQ] = ACTIONS(1067),
    [anon_sym_GT_EQ] = ACTIONS(1069),
    [anon_sym_DOT_DOT] = ACTIONS(1069),
    [anon_sym_CARET] = ACTIONS(1069),
    [anon_sym_LT_LT] = ACTIONS(1069),
    [anon_sym_GT_GT] = ACTIONS(1069),
    [anon_sym_PLUS] = ACTIONS(1069),
    [anon_sym_DASH] = ACTIONS(1069),
    [anon_sym_SLASH] = ACTIONS(1067),
    [anon_sym_PERCENT] = ACTIONS(1069),
    [anon_sym_DOT] = ACTIONS(1071),
    [anon_sym_LBRACK] = ACTIONS(1073),
    [anon_sym_RBRACK] = ACTIONS(1069),
    [anon_sym_as] = ACTIONS(1067),
    [anon_sym_if] = ACTIONS(1067),
    [anon_sym_EQ_GT] = ACTIONS(1069),
    [anon_sym_else] = ACTIONS(1067),
    [anon_sym_spec] = ACTIONS(1067),
    [anon_sym_SEMI] = ACTIONS(1069),
    [anon_sym_with] = ACTIONS(1067),
    [anon_sym_to] = ACTIONS(1067),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(927)] = {
    [sym_line_comment] = STATE(927),
    [sym_block_comment] = STATE(927),
    [sym_identifier] = ACTIONS(1075),
    [anon_sym_u8] = ACTIONS(1075),
    [anon_sym_u16] = ACTIONS(1075),
    [anon_sym_u32] = ACTIONS(1075),
    [anon_sym_u64] = ACTIONS(1075),
    [anon_sym_u128] = ACTIONS(1075),
    [anon_sym_u256] = ACTIONS(1075),
    [anon_sym_exists] = ACTIONS(1075),
    [anon_sym_forall] = ACTIONS(1075),
    [anon_sym_choose] = ACTIONS(1075),
    [anon_sym_min] = ACTIONS(1075),
    [anon_sym_bool] = ACTIONS(1075),
    [anon_sym_address] = ACTIONS(1075),
    [anon_sym_vector] = ACTIONS(1075),
    [anon_sym_for] = ACTIONS(1075),
    [anon_sym_while] = ACTIONS(1075),
    [anon_sym_friend] = ACTIONS(1075),
    [anon_sym_match] = ACTIONS(1075),
    [aux_sym_number_token1] = ACTIONS(1075),
    [aux_sym_number_token2] = ACTIONS(1077),
    [aux_sym_number_token4] = ACTIONS(1077),
    [aux_sym_number_token6] = ACTIONS(1077),
    [anon_sym_STAR] = ACTIONS(1077),
    [anon_sym_LPAREN] = ACTIONS(1077),
    [anon_sym_COMMA] = ACTIONS(1077),
    [anon_sym_RPAREN] = ACTIONS(1077),
    [anon_sym_PIPE] = ACTIONS(1075),
    [anon_sym_AMP] = ACTIONS(1075),
    [anon_sym_EQ] = ACTIONS(1075),
    [anon_sym_where] = ACTIONS(1075),
    [anon_sym_COLON] = ACTIONS(1077),
    [anon_sym_LBRACE] = ACTIONS(1077),
    [anon_sym_RBRACE] = ACTIONS(1077),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1077),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1077),
    [anon_sym_PIPE_PIPE] = ACTIONS(1077),
    [anon_sym_AMP_AMP] = ACTIONS(1077),
    [anon_sym_EQ_EQ] = ACTIONS(1075),
    [anon_sym_BANG_EQ] = ACTIONS(1077),
    [anon_sym_LT] = ACTIONS(1075),
    [anon_sym_GT] = ACTIONS(1075),
    [anon_sym_LT_EQ] = ACTIONS(1075),
    [anon_sym_GT_EQ] = ACTIONS(1077),
    [anon_sym_DOT_DOT] = ACTIONS(1077),
    [anon_sym_CARET] = ACTIONS(1077),
    [anon_sym_LT_LT] = ACTIONS(1077),
    [anon_sym_GT_GT] = ACTIONS(1077),
    [anon_sym_PLUS] = ACTIONS(1077),
    [anon_sym_DASH] = ACTIONS(1077),
    [anon_sym_SLASH] = ACTIONS(1075),
    [anon_sym_PERCENT] = ACTIONS(1077),
    [anon_sym_DOT] = ACTIONS(1075),
    [anon_sym_LBRACK] = ACTIONS(1077),
    [anon_sym_RBRACK] = ACTIONS(1077),
    [anon_sym_as] = ACTIONS(1075),
    [anon_sym_if] = ACTIONS(1075),
    [anon_sym_EQ_GT] = ACTIONS(1077),
    [anon_sym_else] = ACTIONS(1075),
    [anon_sym_spec] = ACTIONS(1075),
    [anon_sym_SEMI] = ACTIONS(1077),
    [anon_sym_with] = ACTIONS(1075),
    [anon_sym_to] = ACTIONS(1075),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(928)] = {
    [sym_line_comment] = STATE(928),
    [sym_block_comment] = STATE(928),
    [sym_identifier] = ACTIONS(819),
    [anon_sym_u8] = ACTIONS(819),
    [anon_sym_u16] = ACTIONS(819),
    [anon_sym_u32] = ACTIONS(819),
    [anon_sym_u64] = ACTIONS(819),
    [anon_sym_u128] = ACTIONS(819),
    [anon_sym_u256] = ACTIONS(819),
    [anon_sym_exists] = ACTIONS(819),
    [anon_sym_forall] = ACTIONS(819),
    [anon_sym_choose] = ACTIONS(819),
    [anon_sym_min] = ACTIONS(819),
    [anon_sym_bool] = ACTIONS(819),
    [anon_sym_address] = ACTIONS(819),
    [anon_sym_vector] = ACTIONS(819),
    [anon_sym_for] = ACTIONS(819),
    [anon_sym_while] = ACTIONS(819),
    [anon_sym_friend] = ACTIONS(819),
    [anon_sym_match] = ACTIONS(819),
    [aux_sym_number_token1] = ACTIONS(819),
    [aux_sym_number_token2] = ACTIONS(821),
    [aux_sym_number_token4] = ACTIONS(821),
    [aux_sym_number_token6] = ACTIONS(821),
    [anon_sym_STAR] = ACTIONS(821),
    [anon_sym_LPAREN] = ACTIONS(821),
    [anon_sym_COMMA] = ACTIONS(821),
    [anon_sym_RPAREN] = ACTIONS(821),
    [anon_sym_PIPE] = ACTIONS(819),
    [anon_sym_AMP] = ACTIONS(819),
    [anon_sym_EQ] = ACTIONS(819),
    [anon_sym_where] = ACTIONS(819),
    [anon_sym_COLON] = ACTIONS(821),
    [anon_sym_LBRACE] = ACTIONS(821),
    [anon_sym_RBRACE] = ACTIONS(821),
    [anon_sym_EQ_EQ_GT] = ACTIONS(821),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(821),
    [anon_sym_PIPE_PIPE] = ACTIONS(821),
    [anon_sym_AMP_AMP] = ACTIONS(821),
    [anon_sym_EQ_EQ] = ACTIONS(819),
    [anon_sym_BANG_EQ] = ACTIONS(821),
    [anon_sym_LT] = ACTIONS(819),
    [anon_sym_GT] = ACTIONS(819),
    [anon_sym_LT_EQ] = ACTIONS(819),
    [anon_sym_GT_EQ] = ACTIONS(821),
    [anon_sym_DOT_DOT] = ACTIONS(821),
    [anon_sym_CARET] = ACTIONS(821),
    [anon_sym_LT_LT] = ACTIONS(821),
    [anon_sym_GT_GT] = ACTIONS(821),
    [anon_sym_PLUS] = ACTIONS(821),
    [anon_sym_DASH] = ACTIONS(821),
    [anon_sym_SLASH] = ACTIONS(819),
    [anon_sym_PERCENT] = ACTIONS(821),
    [anon_sym_DOT] = ACTIONS(819),
    [anon_sym_LBRACK] = ACTIONS(821),
    [anon_sym_RBRACK] = ACTIONS(821),
    [anon_sym_as] = ACTIONS(819),
    [anon_sym_if] = ACTIONS(819),
    [anon_sym_EQ_GT] = ACTIONS(821),
    [anon_sym_else] = ACTIONS(819),
    [anon_sym_spec] = ACTIONS(819),
    [anon_sym_SEMI] = ACTIONS(821),
    [anon_sym_with] = ACTIONS(819),
    [anon_sym_to] = ACTIONS(819),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(929)] = {
    [sym_line_comment] = STATE(929),
    [sym_block_comment] = STATE(929),
    [sym_identifier] = ACTIONS(1079),
    [anon_sym_u8] = ACTIONS(1079),
    [anon_sym_u16] = ACTIONS(1079),
    [anon_sym_u32] = ACTIONS(1079),
    [anon_sym_u64] = ACTIONS(1079),
    [anon_sym_u128] = ACTIONS(1079),
    [anon_sym_u256] = ACTIONS(1079),
    [anon_sym_exists] = ACTIONS(1079),
    [anon_sym_forall] = ACTIONS(1079),
    [anon_sym_choose] = ACTIONS(1079),
    [anon_sym_min] = ACTIONS(1079),
    [anon_sym_bool] = ACTIONS(1079),
    [anon_sym_address] = ACTIONS(1079),
    [anon_sym_vector] = ACTIONS(1079),
    [anon_sym_for] = ACTIONS(1079),
    [anon_sym_while] = ACTIONS(1079),
    [anon_sym_friend] = ACTIONS(1079),
    [anon_sym_match] = ACTIONS(1079),
    [aux_sym_number_token1] = ACTIONS(1079),
    [aux_sym_number_token2] = ACTIONS(1081),
    [aux_sym_number_token4] = ACTIONS(1081),
    [aux_sym_number_token6] = ACTIONS(1081),
    [anon_sym_STAR] = ACTIONS(1081),
    [anon_sym_LPAREN] = ACTIONS(1081),
    [anon_sym_COMMA] = ACTIONS(1081),
    [anon_sym_RPAREN] = ACTIONS(1081),
    [anon_sym_PIPE] = ACTIONS(1079),
    [anon_sym_AMP] = ACTIONS(1079),
    [anon_sym_EQ] = ACTIONS(1079),
    [anon_sym_where] = ACTIONS(1079),
    [anon_sym_COLON] = ACTIONS(1081),
    [anon_sym_LBRACE] = ACTIONS(1081),
    [anon_sym_RBRACE] = ACTIONS(1081),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1081),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1081),
    [anon_sym_PIPE_PIPE] = ACTIONS(1081),
    [anon_sym_AMP_AMP] = ACTIONS(1081),
    [anon_sym_EQ_EQ] = ACTIONS(1079),
    [anon_sym_BANG_EQ] = ACTIONS(1081),
    [anon_sym_LT] = ACTIONS(1079),
    [anon_sym_GT] = ACTIONS(1079),
    [anon_sym_LT_EQ] = ACTIONS(1079),
    [anon_sym_GT_EQ] = ACTIONS(1081),
    [anon_sym_DOT_DOT] = ACTIONS(1081),
    [anon_sym_CARET] = ACTIONS(1081),
    [anon_sym_LT_LT] = ACTIONS(1081),
    [anon_sym_GT_GT] = ACTIONS(1081),
    [anon_sym_PLUS] = ACTIONS(1081),
    [anon_sym_DASH] = ACTIONS(1081),
    [anon_sym_SLASH] = ACTIONS(1079),
    [anon_sym_PERCENT] = ACTIONS(1081),
    [anon_sym_DOT] = ACTIONS(1079),
    [anon_sym_LBRACK] = ACTIONS(1081),
    [anon_sym_RBRACK] = ACTIONS(1081),
    [anon_sym_as] = ACTIONS(1079),
    [anon_sym_if] = ACTIONS(1079),
    [anon_sym_EQ_GT] = ACTIONS(1081),
    [anon_sym_else] = ACTIONS(1079),
    [anon_sym_spec] = ACTIONS(1079),
    [anon_sym_SEMI] = ACTIONS(1081),
    [anon_sym_with] = ACTIONS(1079),
    [anon_sym_to] = ACTIONS(1079),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(930)] = {
    [sym_line_comment] = STATE(930),
    [sym_block_comment] = STATE(930),
    [sym_identifier] = ACTIONS(1083),
    [anon_sym_u8] = ACTIONS(1083),
    [anon_sym_u16] = ACTIONS(1083),
    [anon_sym_u32] = ACTIONS(1083),
    [anon_sym_u64] = ACTIONS(1083),
    [anon_sym_u128] = ACTIONS(1083),
    [anon_sym_u256] = ACTIONS(1083),
    [anon_sym_exists] = ACTIONS(1083),
    [anon_sym_forall] = ACTIONS(1083),
    [anon_sym_choose] = ACTIONS(1083),
    [anon_sym_min] = ACTIONS(1083),
    [anon_sym_bool] = ACTIONS(1083),
    [anon_sym_address] = ACTIONS(1083),
    [anon_sym_vector] = ACTIONS(1083),
    [anon_sym_for] = ACTIONS(1083),
    [anon_sym_while] = ACTIONS(1083),
    [anon_sym_friend] = ACTIONS(1083),
    [anon_sym_match] = ACTIONS(1083),
    [aux_sym_number_token1] = ACTIONS(1083),
    [aux_sym_number_token2] = ACTIONS(1085),
    [aux_sym_number_token4] = ACTIONS(1085),
    [aux_sym_number_token6] = ACTIONS(1085),
    [anon_sym_STAR] = ACTIONS(1085),
    [anon_sym_LPAREN] = ACTIONS(1085),
    [anon_sym_COMMA] = ACTIONS(1085),
    [anon_sym_RPAREN] = ACTIONS(1085),
    [anon_sym_PIPE] = ACTIONS(1083),
    [anon_sym_AMP] = ACTIONS(1083),
    [anon_sym_EQ] = ACTIONS(1083),
    [anon_sym_where] = ACTIONS(1083),
    [anon_sym_COLON] = ACTIONS(1085),
    [anon_sym_LBRACE] = ACTIONS(1085),
    [anon_sym_RBRACE] = ACTIONS(1085),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1085),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1085),
    [anon_sym_PIPE_PIPE] = ACTIONS(1085),
    [anon_sym_AMP_AMP] = ACTIONS(1085),
    [anon_sym_EQ_EQ] = ACTIONS(1083),
    [anon_sym_BANG_EQ] = ACTIONS(1085),
    [anon_sym_LT] = ACTIONS(1083),
    [anon_sym_GT] = ACTIONS(1083),
    [anon_sym_LT_EQ] = ACTIONS(1083),
    [anon_sym_GT_EQ] = ACTIONS(1085),
    [anon_sym_DOT_DOT] = ACTIONS(1085),
    [anon_sym_CARET] = ACTIONS(1085),
    [anon_sym_LT_LT] = ACTIONS(1085),
    [anon_sym_GT_GT] = ACTIONS(1085),
    [anon_sym_PLUS] = ACTIONS(1085),
    [anon_sym_DASH] = ACTIONS(1085),
    [anon_sym_SLASH] = ACTIONS(1083),
    [anon_sym_PERCENT] = ACTIONS(1085),
    [anon_sym_DOT] = ACTIONS(1083),
    [anon_sym_LBRACK] = ACTIONS(1085),
    [anon_sym_RBRACK] = ACTIONS(1085),
    [anon_sym_as] = ACTIONS(1083),
    [anon_sym_if] = ACTIONS(1083),
    [anon_sym_EQ_GT] = ACTIONS(1085),
    [anon_sym_else] = ACTIONS(1083),
    [anon_sym_spec] = ACTIONS(1083),
    [anon_sym_SEMI] = ACTIONS(1085),
    [anon_sym_with] = ACTIONS(1083),
    [anon_sym_to] = ACTIONS(1083),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(931)] = {
    [sym_line_comment] = STATE(931),
    [sym_block_comment] = STATE(931),
    [sym_identifier] = ACTIONS(1087),
    [anon_sym_u8] = ACTIONS(1087),
    [anon_sym_u16] = ACTIONS(1087),
    [anon_sym_u32] = ACTIONS(1087),
    [anon_sym_u64] = ACTIONS(1087),
    [anon_sym_u128] = ACTIONS(1087),
    [anon_sym_u256] = ACTIONS(1087),
    [anon_sym_exists] = ACTIONS(1087),
    [anon_sym_forall] = ACTIONS(1087),
    [anon_sym_choose] = ACTIONS(1087),
    [anon_sym_min] = ACTIONS(1087),
    [anon_sym_bool] = ACTIONS(1087),
    [anon_sym_address] = ACTIONS(1087),
    [anon_sym_vector] = ACTIONS(1087),
    [anon_sym_for] = ACTIONS(1087),
    [anon_sym_while] = ACTIONS(1087),
    [anon_sym_friend] = ACTIONS(1087),
    [anon_sym_match] = ACTIONS(1087),
    [aux_sym_number_token1] = ACTIONS(1087),
    [aux_sym_number_token2] = ACTIONS(1089),
    [aux_sym_number_token4] = ACTIONS(1089),
    [aux_sym_number_token6] = ACTIONS(1089),
    [anon_sym_STAR] = ACTIONS(1089),
    [anon_sym_LPAREN] = ACTIONS(1089),
    [anon_sym_COMMA] = ACTIONS(1089),
    [anon_sym_RPAREN] = ACTIONS(1089),
    [anon_sym_PIPE] = ACTIONS(1087),
    [anon_sym_AMP] = ACTIONS(1087),
    [anon_sym_EQ] = ACTIONS(1087),
    [anon_sym_where] = ACTIONS(1087),
    [anon_sym_COLON] = ACTIONS(1089),
    [anon_sym_LBRACE] = ACTIONS(1089),
    [anon_sym_RBRACE] = ACTIONS(1089),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1089),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1089),
    [anon_sym_PIPE_PIPE] = ACTIONS(1089),
    [anon_sym_AMP_AMP] = ACTIONS(1089),
    [anon_sym_EQ_EQ] = ACTIONS(1087),
    [anon_sym_BANG_EQ] = ACTIONS(1089),
    [anon_sym_LT] = ACTIONS(1087),
    [anon_sym_GT] = ACTIONS(1087),
    [anon_sym_LT_EQ] = ACTIONS(1087),
    [anon_sym_GT_EQ] = ACTIONS(1089),
    [anon_sym_DOT_DOT] = ACTIONS(1089),
    [anon_sym_CARET] = ACTIONS(1089),
    [anon_sym_LT_LT] = ACTIONS(1089),
    [anon_sym_GT_GT] = ACTIONS(1089),
    [anon_sym_PLUS] = ACTIONS(1089),
    [anon_sym_DASH] = ACTIONS(1089),
    [anon_sym_SLASH] = ACTIONS(1087),
    [anon_sym_PERCENT] = ACTIONS(1089),
    [anon_sym_DOT] = ACTIONS(1087),
    [anon_sym_LBRACK] = ACTIONS(1089),
    [anon_sym_RBRACK] = ACTIONS(1089),
    [anon_sym_as] = ACTIONS(1087),
    [anon_sym_if] = ACTIONS(1087),
    [anon_sym_EQ_GT] = ACTIONS(1089),
    [anon_sym_else] = ACTIONS(1087),
    [anon_sym_spec] = ACTIONS(1087),
    [anon_sym_SEMI] = ACTIONS(1089),
    [anon_sym_with] = ACTIONS(1087),
    [anon_sym_to] = ACTIONS(1087),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(932)] = {
    [sym_line_comment] = STATE(932),
    [sym_block_comment] = STATE(932),
    [sym_identifier] = ACTIONS(1091),
    [anon_sym_u8] = ACTIONS(1091),
    [anon_sym_u16] = ACTIONS(1091),
    [anon_sym_u32] = ACTIONS(1091),
    [anon_sym_u64] = ACTIONS(1091),
    [anon_sym_u128] = ACTIONS(1091),
    [anon_sym_u256] = ACTIONS(1091),
    [anon_sym_exists] = ACTIONS(1091),
    [anon_sym_forall] = ACTIONS(1091),
    [anon_sym_choose] = ACTIONS(1091),
    [anon_sym_min] = ACTIONS(1091),
    [anon_sym_bool] = ACTIONS(1091),
    [anon_sym_address] = ACTIONS(1091),
    [anon_sym_vector] = ACTIONS(1091),
    [anon_sym_for] = ACTIONS(1091),
    [anon_sym_while] = ACTIONS(1091),
    [anon_sym_friend] = ACTIONS(1091),
    [anon_sym_match] = ACTIONS(1091),
    [aux_sym_number_token1] = ACTIONS(1091),
    [aux_sym_number_token2] = ACTIONS(1093),
    [aux_sym_number_token4] = ACTIONS(1093),
    [aux_sym_number_token6] = ACTIONS(1093),
    [anon_sym_STAR] = ACTIONS(1093),
    [anon_sym_LPAREN] = ACTIONS(1093),
    [anon_sym_COMMA] = ACTIONS(1093),
    [anon_sym_RPAREN] = ACTIONS(1093),
    [anon_sym_PIPE] = ACTIONS(1091),
    [anon_sym_AMP] = ACTIONS(1091),
    [anon_sym_EQ] = ACTIONS(1091),
    [anon_sym_where] = ACTIONS(1091),
    [anon_sym_COLON] = ACTIONS(1093),
    [anon_sym_LBRACE] = ACTIONS(1093),
    [anon_sym_RBRACE] = ACTIONS(1093),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1093),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1093),
    [anon_sym_PIPE_PIPE] = ACTIONS(1093),
    [anon_sym_AMP_AMP] = ACTIONS(1093),
    [anon_sym_EQ_EQ] = ACTIONS(1091),
    [anon_sym_BANG_EQ] = ACTIONS(1093),
    [anon_sym_LT] = ACTIONS(1091),
    [anon_sym_GT] = ACTIONS(1091),
    [anon_sym_LT_EQ] = ACTIONS(1091),
    [anon_sym_GT_EQ] = ACTIONS(1093),
    [anon_sym_DOT_DOT] = ACTIONS(1093),
    [anon_sym_CARET] = ACTIONS(1093),
    [anon_sym_LT_LT] = ACTIONS(1093),
    [anon_sym_GT_GT] = ACTIONS(1093),
    [anon_sym_PLUS] = ACTIONS(1093),
    [anon_sym_DASH] = ACTIONS(1093),
    [anon_sym_SLASH] = ACTIONS(1091),
    [anon_sym_PERCENT] = ACTIONS(1093),
    [anon_sym_DOT] = ACTIONS(1091),
    [anon_sym_LBRACK] = ACTIONS(1093),
    [anon_sym_RBRACK] = ACTIONS(1093),
    [anon_sym_as] = ACTIONS(1091),
    [anon_sym_if] = ACTIONS(1091),
    [anon_sym_EQ_GT] = ACTIONS(1093),
    [anon_sym_else] = ACTIONS(1091),
    [anon_sym_spec] = ACTIONS(1091),
    [anon_sym_SEMI] = ACTIONS(1093),
    [anon_sym_with] = ACTIONS(1091),
    [anon_sym_to] = ACTIONS(1091),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(933)] = {
    [sym_line_comment] = STATE(933),
    [sym_block_comment] = STATE(933),
    [sym_identifier] = ACTIONS(1095),
    [anon_sym_u8] = ACTIONS(1095),
    [anon_sym_u16] = ACTIONS(1095),
    [anon_sym_u32] = ACTIONS(1095),
    [anon_sym_u64] = ACTIONS(1095),
    [anon_sym_u128] = ACTIONS(1095),
    [anon_sym_u256] = ACTIONS(1095),
    [anon_sym_exists] = ACTIONS(1095),
    [anon_sym_forall] = ACTIONS(1095),
    [anon_sym_choose] = ACTIONS(1095),
    [anon_sym_min] = ACTIONS(1095),
    [anon_sym_bool] = ACTIONS(1095),
    [anon_sym_address] = ACTIONS(1095),
    [anon_sym_vector] = ACTIONS(1095),
    [anon_sym_for] = ACTIONS(1095),
    [anon_sym_while] = ACTIONS(1095),
    [anon_sym_friend] = ACTIONS(1095),
    [anon_sym_match] = ACTIONS(1095),
    [aux_sym_number_token1] = ACTIONS(1095),
    [aux_sym_number_token2] = ACTIONS(1097),
    [aux_sym_number_token4] = ACTIONS(1097),
    [aux_sym_number_token6] = ACTIONS(1097),
    [anon_sym_STAR] = ACTIONS(1097),
    [anon_sym_LPAREN] = ACTIONS(1097),
    [anon_sym_COMMA] = ACTIONS(1097),
    [anon_sym_RPAREN] = ACTIONS(1097),
    [anon_sym_PIPE] = ACTIONS(1095),
    [anon_sym_AMP] = ACTIONS(1095),
    [anon_sym_EQ] = ACTIONS(1095),
    [anon_sym_where] = ACTIONS(1095),
    [anon_sym_COLON] = ACTIONS(1097),
    [anon_sym_LBRACE] = ACTIONS(1097),
    [anon_sym_RBRACE] = ACTIONS(1097),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1097),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1097),
    [anon_sym_PIPE_PIPE] = ACTIONS(1097),
    [anon_sym_AMP_AMP] = ACTIONS(1097),
    [anon_sym_EQ_EQ] = ACTIONS(1095),
    [anon_sym_BANG_EQ] = ACTIONS(1097),
    [anon_sym_LT] = ACTIONS(1095),
    [anon_sym_GT] = ACTIONS(1095),
    [anon_sym_LT_EQ] = ACTIONS(1095),
    [anon_sym_GT_EQ] = ACTIONS(1097),
    [anon_sym_DOT_DOT] = ACTIONS(1097),
    [anon_sym_CARET] = ACTIONS(1097),
    [anon_sym_LT_LT] = ACTIONS(1097),
    [anon_sym_GT_GT] = ACTIONS(1097),
    [anon_sym_PLUS] = ACTIONS(1097),
    [anon_sym_DASH] = ACTIONS(1097),
    [anon_sym_SLASH] = ACTIONS(1095),
    [anon_sym_PERCENT] = ACTIONS(1097),
    [anon_sym_DOT] = ACTIONS(1095),
    [anon_sym_LBRACK] = ACTIONS(1097),
    [anon_sym_RBRACK] = ACTIONS(1097),
    [anon_sym_as] = ACTIONS(1095),
    [anon_sym_if] = ACTIONS(1095),
    [anon_sym_EQ_GT] = ACTIONS(1097),
    [anon_sym_else] = ACTIONS(1095),
    [anon_sym_spec] = ACTIONS(1095),
    [anon_sym_SEMI] = ACTIONS(1097),
    [anon_sym_with] = ACTIONS(1095),
    [anon_sym_to] = ACTIONS(1095),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(934)] = {
    [sym_line_comment] = STATE(934),
    [sym_block_comment] = STATE(934),
    [sym_identifier] = ACTIONS(927),
    [anon_sym_u8] = ACTIONS(927),
    [anon_sym_u16] = ACTIONS(927),
    [anon_sym_u32] = ACTIONS(927),
    [anon_sym_u64] = ACTIONS(927),
    [anon_sym_u128] = ACTIONS(927),
    [anon_sym_u256] = ACTIONS(927),
    [anon_sym_exists] = ACTIONS(927),
    [anon_sym_forall] = ACTIONS(927),
    [anon_sym_choose] = ACTIONS(927),
    [anon_sym_min] = ACTIONS(927),
    [anon_sym_bool] = ACTIONS(927),
    [anon_sym_address] = ACTIONS(927),
    [anon_sym_vector] = ACTIONS(927),
    [anon_sym_for] = ACTIONS(927),
    [anon_sym_while] = ACTIONS(927),
    [anon_sym_friend] = ACTIONS(927),
    [anon_sym_match] = ACTIONS(927),
    [aux_sym_number_token1] = ACTIONS(927),
    [aux_sym_number_token2] = ACTIONS(929),
    [aux_sym_number_token4] = ACTIONS(929),
    [aux_sym_number_token6] = ACTIONS(929),
    [anon_sym_STAR] = ACTIONS(929),
    [anon_sym_LPAREN] = ACTIONS(929),
    [anon_sym_COMMA] = ACTIONS(929),
    [anon_sym_RPAREN] = ACTIONS(929),
    [anon_sym_PIPE] = ACTIONS(927),
    [anon_sym_AMP] = ACTIONS(927),
    [anon_sym_EQ] = ACTIONS(999),
    [anon_sym_where] = ACTIONS(927),
    [anon_sym_COLON] = ACTIONS(929),
    [anon_sym_LBRACE] = ACTIONS(929),
    [anon_sym_RBRACE] = ACTIONS(929),
    [anon_sym_EQ_EQ_GT] = ACTIONS(929),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(929),
    [anon_sym_PIPE_PIPE] = ACTIONS(929),
    [anon_sym_AMP_AMP] = ACTIONS(929),
    [anon_sym_EQ_EQ] = ACTIONS(927),
    [anon_sym_BANG_EQ] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(927),
    [anon_sym_GT] = ACTIONS(927),
    [anon_sym_LT_EQ] = ACTIONS(927),
    [anon_sym_GT_EQ] = ACTIONS(929),
    [anon_sym_DOT_DOT] = ACTIONS(929),
    [anon_sym_CARET] = ACTIONS(929),
    [anon_sym_LT_LT] = ACTIONS(929),
    [anon_sym_GT_GT] = ACTIONS(929),
    [anon_sym_PLUS] = ACTIONS(929),
    [anon_sym_DASH] = ACTIONS(929),
    [anon_sym_SLASH] = ACTIONS(927),
    [anon_sym_PERCENT] = ACTIONS(929),
    [anon_sym_DOT] = ACTIONS(927),
    [anon_sym_LBRACK] = ACTIONS(929),
    [anon_sym_RBRACK] = ACTIONS(929),
    [anon_sym_as] = ACTIONS(927),
    [anon_sym_if] = ACTIONS(927),
    [anon_sym_EQ_GT] = ACTIONS(929),
    [anon_sym_else] = ACTIONS(927),
    [anon_sym_spec] = ACTIONS(927),
    [anon_sym_SEMI] = ACTIONS(929),
    [anon_sym_with] = ACTIONS(927),
    [anon_sym_to] = ACTIONS(927),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(935)] = {
    [sym_line_comment] = STATE(935),
    [sym_block_comment] = STATE(935),
    [sym_identifier] = ACTIONS(1099),
    [anon_sym_u8] = ACTIONS(1099),
    [anon_sym_u16] = ACTIONS(1099),
    [anon_sym_u32] = ACTIONS(1099),
    [anon_sym_u64] = ACTIONS(1099),
    [anon_sym_u128] = ACTIONS(1099),
    [anon_sym_u256] = ACTIONS(1099),
    [anon_sym_exists] = ACTIONS(1099),
    [anon_sym_forall] = ACTIONS(1099),
    [anon_sym_choose] = ACTIONS(1099),
    [anon_sym_min] = ACTIONS(1099),
    [anon_sym_bool] = ACTIONS(1099),
    [anon_sym_address] = ACTIONS(1099),
    [anon_sym_vector] = ACTIONS(1099),
    [anon_sym_for] = ACTIONS(1099),
    [anon_sym_while] = ACTIONS(1099),
    [anon_sym_friend] = ACTIONS(1099),
    [anon_sym_match] = ACTIONS(1099),
    [aux_sym_number_token1] = ACTIONS(1099),
    [aux_sym_number_token2] = ACTIONS(1101),
    [aux_sym_number_token4] = ACTIONS(1101),
    [aux_sym_number_token6] = ACTIONS(1101),
    [anon_sym_STAR] = ACTIONS(1101),
    [anon_sym_LPAREN] = ACTIONS(1101),
    [anon_sym_COMMA] = ACTIONS(1101),
    [anon_sym_RPAREN] = ACTIONS(1101),
    [anon_sym_PIPE] = ACTIONS(1099),
    [anon_sym_AMP] = ACTIONS(1099),
    [anon_sym_EQ] = ACTIONS(1099),
    [anon_sym_where] = ACTIONS(1099),
    [anon_sym_COLON] = ACTIONS(1101),
    [anon_sym_LBRACE] = ACTIONS(1101),
    [anon_sym_RBRACE] = ACTIONS(1101),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1101),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1101),
    [anon_sym_PIPE_PIPE] = ACTIONS(1101),
    [anon_sym_AMP_AMP] = ACTIONS(1101),
    [anon_sym_EQ_EQ] = ACTIONS(1099),
    [anon_sym_BANG_EQ] = ACTIONS(1101),
    [anon_sym_LT] = ACTIONS(1099),
    [anon_sym_GT] = ACTIONS(1099),
    [anon_sym_LT_EQ] = ACTIONS(1099),
    [anon_sym_GT_EQ] = ACTIONS(1101),
    [anon_sym_DOT_DOT] = ACTIONS(1101),
    [anon_sym_CARET] = ACTIONS(1101),
    [anon_sym_LT_LT] = ACTIONS(1101),
    [anon_sym_GT_GT] = ACTIONS(1101),
    [anon_sym_PLUS] = ACTIONS(1101),
    [anon_sym_DASH] = ACTIONS(1101),
    [anon_sym_SLASH] = ACTIONS(1099),
    [anon_sym_PERCENT] = ACTIONS(1101),
    [anon_sym_DOT] = ACTIONS(1099),
    [anon_sym_LBRACK] = ACTIONS(1101),
    [anon_sym_RBRACK] = ACTIONS(1101),
    [anon_sym_as] = ACTIONS(1099),
    [anon_sym_if] = ACTIONS(1099),
    [anon_sym_EQ_GT] = ACTIONS(1101),
    [anon_sym_else] = ACTIONS(1099),
    [anon_sym_spec] = ACTIONS(1099),
    [anon_sym_SEMI] = ACTIONS(1101),
    [anon_sym_with] = ACTIONS(1099),
    [anon_sym_to] = ACTIONS(1099),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(936)] = {
    [sym_line_comment] = STATE(936),
    [sym_block_comment] = STATE(936),
    [sym_identifier] = ACTIONS(1103),
    [anon_sym_u8] = ACTIONS(1103),
    [anon_sym_u16] = ACTIONS(1103),
    [anon_sym_u32] = ACTIONS(1103),
    [anon_sym_u64] = ACTIONS(1103),
    [anon_sym_u128] = ACTIONS(1103),
    [anon_sym_u256] = ACTIONS(1103),
    [anon_sym_exists] = ACTIONS(1103),
    [anon_sym_forall] = ACTIONS(1103),
    [anon_sym_choose] = ACTIONS(1103),
    [anon_sym_min] = ACTIONS(1103),
    [anon_sym_bool] = ACTIONS(1103),
    [anon_sym_address] = ACTIONS(1103),
    [anon_sym_vector] = ACTIONS(1103),
    [anon_sym_for] = ACTIONS(1103),
    [anon_sym_while] = ACTIONS(1103),
    [anon_sym_friend] = ACTIONS(1103),
    [anon_sym_match] = ACTIONS(1103),
    [aux_sym_number_token1] = ACTIONS(1103),
    [aux_sym_number_token2] = ACTIONS(1105),
    [aux_sym_number_token4] = ACTIONS(1105),
    [aux_sym_number_token6] = ACTIONS(1105),
    [anon_sym_STAR] = ACTIONS(1105),
    [anon_sym_LPAREN] = ACTIONS(1105),
    [anon_sym_COMMA] = ACTIONS(1105),
    [anon_sym_RPAREN] = ACTIONS(1105),
    [anon_sym_PIPE] = ACTIONS(1103),
    [anon_sym_AMP] = ACTIONS(1103),
    [anon_sym_EQ] = ACTIONS(1103),
    [anon_sym_where] = ACTIONS(1103),
    [anon_sym_COLON] = ACTIONS(1105),
    [anon_sym_LBRACE] = ACTIONS(1105),
    [anon_sym_RBRACE] = ACTIONS(1105),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1105),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1105),
    [anon_sym_PIPE_PIPE] = ACTIONS(1105),
    [anon_sym_AMP_AMP] = ACTIONS(1105),
    [anon_sym_EQ_EQ] = ACTIONS(1103),
    [anon_sym_BANG_EQ] = ACTIONS(1105),
    [anon_sym_LT] = ACTIONS(1103),
    [anon_sym_GT] = ACTIONS(1103),
    [anon_sym_LT_EQ] = ACTIONS(1103),
    [anon_sym_GT_EQ] = ACTIONS(1105),
    [anon_sym_DOT_DOT] = ACTIONS(1105),
    [anon_sym_CARET] = ACTIONS(1105),
    [anon_sym_LT_LT] = ACTIONS(1105),
    [anon_sym_GT_GT] = ACTIONS(1105),
    [anon_sym_PLUS] = ACTIONS(1105),
    [anon_sym_DASH] = ACTIONS(1105),
    [anon_sym_SLASH] = ACTIONS(1103),
    [anon_sym_PERCENT] = ACTIONS(1105),
    [anon_sym_DOT] = ACTIONS(1103),
    [anon_sym_LBRACK] = ACTIONS(1105),
    [anon_sym_RBRACK] = ACTIONS(1105),
    [anon_sym_as] = ACTIONS(1103),
    [anon_sym_if] = ACTIONS(1103),
    [anon_sym_EQ_GT] = ACTIONS(1105),
    [anon_sym_else] = ACTIONS(1103),
    [anon_sym_spec] = ACTIONS(1103),
    [anon_sym_SEMI] = ACTIONS(1105),
    [anon_sym_with] = ACTIONS(1103),
    [anon_sym_to] = ACTIONS(1103),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(937)] = {
    [sym_line_comment] = STATE(937),
    [sym_block_comment] = STATE(937),
    [sym_identifier] = ACTIONS(1107),
    [anon_sym_u8] = ACTIONS(1107),
    [anon_sym_u16] = ACTIONS(1107),
    [anon_sym_u32] = ACTIONS(1107),
    [anon_sym_u64] = ACTIONS(1107),
    [anon_sym_u128] = ACTIONS(1107),
    [anon_sym_u256] = ACTIONS(1107),
    [anon_sym_exists] = ACTIONS(1107),
    [anon_sym_forall] = ACTIONS(1107),
    [anon_sym_choose] = ACTIONS(1107),
    [anon_sym_min] = ACTIONS(1107),
    [anon_sym_bool] = ACTIONS(1107),
    [anon_sym_address] = ACTIONS(1107),
    [anon_sym_vector] = ACTIONS(1107),
    [anon_sym_for] = ACTIONS(1107),
    [anon_sym_while] = ACTIONS(1107),
    [anon_sym_friend] = ACTIONS(1107),
    [anon_sym_match] = ACTIONS(1107),
    [aux_sym_number_token1] = ACTIONS(1107),
    [aux_sym_number_token2] = ACTIONS(1109),
    [aux_sym_number_token4] = ACTIONS(1109),
    [aux_sym_number_token6] = ACTIONS(1109),
    [anon_sym_STAR] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1109),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_RPAREN] = ACTIONS(1109),
    [anon_sym_PIPE] = ACTIONS(1107),
    [anon_sym_AMP] = ACTIONS(1107),
    [anon_sym_EQ] = ACTIONS(1107),
    [anon_sym_where] = ACTIONS(1107),
    [anon_sym_COLON] = ACTIONS(1109),
    [anon_sym_LBRACE] = ACTIONS(1109),
    [anon_sym_RBRACE] = ACTIONS(1109),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1109),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_EQ_EQ] = ACTIONS(1107),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_LT] = ACTIONS(1107),
    [anon_sym_GT] = ACTIONS(1107),
    [anon_sym_LT_EQ] = ACTIONS(1107),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_DOT_DOT] = ACTIONS(1109),
    [anon_sym_CARET] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1109),
    [anon_sym_GT_GT] = ACTIONS(1109),
    [anon_sym_PLUS] = ACTIONS(1109),
    [anon_sym_DASH] = ACTIONS(1109),
    [anon_sym_SLASH] = ACTIONS(1107),
    [anon_sym_PERCENT] = ACTIONS(1109),
    [anon_sym_DOT] = ACTIONS(1107),
    [anon_sym_LBRACK] = ACTIONS(1109),
    [anon_sym_RBRACK] = ACTIONS(1109),
    [anon_sym_as] = ACTIONS(1107),
    [anon_sym_if] = ACTIONS(1107),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_else] = ACTIONS(1107),
    [anon_sym_spec] = ACTIONS(1107),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_with] = ACTIONS(1107),
    [anon_sym_to] = ACTIONS(1107),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(938)] = {
    [sym_line_comment] = STATE(938),
    [sym_block_comment] = STATE(938),
    [sym_identifier] = ACTIONS(1111),
    [anon_sym_u8] = ACTIONS(1111),
    [anon_sym_u16] = ACTIONS(1111),
    [anon_sym_u32] = ACTIONS(1111),
    [anon_sym_u64] = ACTIONS(1111),
    [anon_sym_u128] = ACTIONS(1111),
    [anon_sym_u256] = ACTIONS(1111),
    [anon_sym_exists] = ACTIONS(1111),
    [anon_sym_forall] = ACTIONS(1111),
    [anon_sym_choose] = ACTIONS(1111),
    [anon_sym_min] = ACTIONS(1111),
    [anon_sym_bool] = ACTIONS(1111),
    [anon_sym_address] = ACTIONS(1111),
    [anon_sym_vector] = ACTIONS(1111),
    [anon_sym_for] = ACTIONS(1111),
    [anon_sym_while] = ACTIONS(1111),
    [anon_sym_friend] = ACTIONS(1111),
    [anon_sym_match] = ACTIONS(1111),
    [aux_sym_number_token1] = ACTIONS(1111),
    [aux_sym_number_token2] = ACTIONS(1113),
    [aux_sym_number_token4] = ACTIONS(1113),
    [aux_sym_number_token6] = ACTIONS(1113),
    [anon_sym_STAR] = ACTIONS(1113),
    [anon_sym_LPAREN] = ACTIONS(1113),
    [anon_sym_COMMA] = ACTIONS(1113),
    [anon_sym_RPAREN] = ACTIONS(1113),
    [anon_sym_PIPE] = ACTIONS(1111),
    [anon_sym_AMP] = ACTIONS(1111),
    [anon_sym_EQ] = ACTIONS(1111),
    [anon_sym_where] = ACTIONS(1111),
    [anon_sym_COLON] = ACTIONS(1113),
    [anon_sym_LBRACE] = ACTIONS(1113),
    [anon_sym_RBRACE] = ACTIONS(1113),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1113),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1113),
    [anon_sym_PIPE_PIPE] = ACTIONS(1113),
    [anon_sym_AMP_AMP] = ACTIONS(1113),
    [anon_sym_EQ_EQ] = ACTIONS(1111),
    [anon_sym_BANG_EQ] = ACTIONS(1113),
    [anon_sym_LT] = ACTIONS(1111),
    [anon_sym_GT] = ACTIONS(1111),
    [anon_sym_LT_EQ] = ACTIONS(1111),
    [anon_sym_GT_EQ] = ACTIONS(1113),
    [anon_sym_DOT_DOT] = ACTIONS(1113),
    [anon_sym_CARET] = ACTIONS(1113),
    [anon_sym_LT_LT] = ACTIONS(1113),
    [anon_sym_GT_GT] = ACTIONS(1113),
    [anon_sym_PLUS] = ACTIONS(1113),
    [anon_sym_DASH] = ACTIONS(1113),
    [anon_sym_SLASH] = ACTIONS(1111),
    [anon_sym_PERCENT] = ACTIONS(1113),
    [anon_sym_DOT] = ACTIONS(1111),
    [anon_sym_LBRACK] = ACTIONS(1113),
    [anon_sym_RBRACK] = ACTIONS(1113),
    [anon_sym_as] = ACTIONS(1111),
    [anon_sym_if] = ACTIONS(1111),
    [anon_sym_EQ_GT] = ACTIONS(1113),
    [anon_sym_else] = ACTIONS(1111),
    [anon_sym_spec] = ACTIONS(1111),
    [anon_sym_SEMI] = ACTIONS(1113),
    [anon_sym_with] = ACTIONS(1111),
    [anon_sym_to] = ACTIONS(1111),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(939)] = {
    [sym_line_comment] = STATE(939),
    [sym_block_comment] = STATE(939),
    [sym_identifier] = ACTIONS(1115),
    [anon_sym_u8] = ACTIONS(1115),
    [anon_sym_u16] = ACTIONS(1115),
    [anon_sym_u32] = ACTIONS(1115),
    [anon_sym_u64] = ACTIONS(1115),
    [anon_sym_u128] = ACTIONS(1115),
    [anon_sym_u256] = ACTIONS(1115),
    [anon_sym_exists] = ACTIONS(1115),
    [anon_sym_forall] = ACTIONS(1115),
    [anon_sym_choose] = ACTIONS(1115),
    [anon_sym_min] = ACTIONS(1115),
    [anon_sym_bool] = ACTIONS(1115),
    [anon_sym_address] = ACTIONS(1115),
    [anon_sym_vector] = ACTIONS(1115),
    [anon_sym_for] = ACTIONS(1115),
    [anon_sym_while] = ACTIONS(1115),
    [anon_sym_friend] = ACTIONS(1115),
    [anon_sym_match] = ACTIONS(1115),
    [aux_sym_number_token1] = ACTIONS(1115),
    [aux_sym_number_token2] = ACTIONS(1117),
    [aux_sym_number_token4] = ACTIONS(1117),
    [aux_sym_number_token6] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_LPAREN] = ACTIONS(1117),
    [anon_sym_COMMA] = ACTIONS(1117),
    [anon_sym_RPAREN] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1115),
    [anon_sym_AMP] = ACTIONS(1115),
    [anon_sym_EQ] = ACTIONS(1115),
    [anon_sym_where] = ACTIONS(1115),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_LBRACE] = ACTIONS(1117),
    [anon_sym_RBRACE] = ACTIONS(1117),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1117),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1117),
    [anon_sym_PIPE_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1115),
    [anon_sym_BANG_EQ] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1115),
    [anon_sym_GT] = ACTIONS(1115),
    [anon_sym_LT_EQ] = ACTIONS(1115),
    [anon_sym_GT_EQ] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1115),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1115),
    [anon_sym_LBRACK] = ACTIONS(1117),
    [anon_sym_RBRACK] = ACTIONS(1117),
    [anon_sym_as] = ACTIONS(1115),
    [anon_sym_if] = ACTIONS(1115),
    [anon_sym_EQ_GT] = ACTIONS(1117),
    [anon_sym_else] = ACTIONS(1115),
    [anon_sym_spec] = ACTIONS(1115),
    [anon_sym_SEMI] = ACTIONS(1117),
    [anon_sym_with] = ACTIONS(1115),
    [anon_sym_to] = ACTIONS(1115),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(940)] = {
    [sym_line_comment] = STATE(940),
    [sym_block_comment] = STATE(940),
    [sym_identifier] = ACTIONS(1119),
    [anon_sym_u8] = ACTIONS(1119),
    [anon_sym_u16] = ACTIONS(1119),
    [anon_sym_u32] = ACTIONS(1119),
    [anon_sym_u64] = ACTIONS(1119),
    [anon_sym_u128] = ACTIONS(1119),
    [anon_sym_u256] = ACTIONS(1119),
    [anon_sym_exists] = ACTIONS(1119),
    [anon_sym_forall] = ACTIONS(1119),
    [anon_sym_choose] = ACTIONS(1119),
    [anon_sym_min] = ACTIONS(1119),
    [anon_sym_bool] = ACTIONS(1119),
    [anon_sym_address] = ACTIONS(1119),
    [anon_sym_vector] = ACTIONS(1119),
    [anon_sym_for] = ACTIONS(1119),
    [anon_sym_while] = ACTIONS(1119),
    [anon_sym_friend] = ACTIONS(1119),
    [anon_sym_match] = ACTIONS(1119),
    [aux_sym_number_token1] = ACTIONS(1119),
    [aux_sym_number_token2] = ACTIONS(1121),
    [aux_sym_number_token4] = ACTIONS(1121),
    [aux_sym_number_token6] = ACTIONS(1121),
    [anon_sym_STAR] = ACTIONS(1121),
    [anon_sym_LPAREN] = ACTIONS(1121),
    [anon_sym_COMMA] = ACTIONS(1121),
    [anon_sym_RPAREN] = ACTIONS(1121),
    [anon_sym_PIPE] = ACTIONS(1119),
    [anon_sym_AMP] = ACTIONS(1119),
    [anon_sym_EQ] = ACTIONS(1119),
    [anon_sym_where] = ACTIONS(1119),
    [anon_sym_COLON] = ACTIONS(1121),
    [anon_sym_LBRACE] = ACTIONS(1121),
    [anon_sym_RBRACE] = ACTIONS(1121),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1121),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1121),
    [anon_sym_PIPE_PIPE] = ACTIONS(1121),
    [anon_sym_AMP_AMP] = ACTIONS(1121),
    [anon_sym_EQ_EQ] = ACTIONS(1119),
    [anon_sym_BANG_EQ] = ACTIONS(1121),
    [anon_sym_LT] = ACTIONS(1119),
    [anon_sym_GT] = ACTIONS(1119),
    [anon_sym_LT_EQ] = ACTIONS(1119),
    [anon_sym_GT_EQ] = ACTIONS(1121),
    [anon_sym_DOT_DOT] = ACTIONS(1121),
    [anon_sym_CARET] = ACTIONS(1121),
    [anon_sym_LT_LT] = ACTIONS(1121),
    [anon_sym_GT_GT] = ACTIONS(1121),
    [anon_sym_PLUS] = ACTIONS(1121),
    [anon_sym_DASH] = ACTIONS(1121),
    [anon_sym_SLASH] = ACTIONS(1119),
    [anon_sym_PERCENT] = ACTIONS(1121),
    [anon_sym_DOT] = ACTIONS(1119),
    [anon_sym_LBRACK] = ACTIONS(1121),
    [anon_sym_RBRACK] = ACTIONS(1121),
    [anon_sym_as] = ACTIONS(1119),
    [anon_sym_if] = ACTIONS(1119),
    [anon_sym_EQ_GT] = ACTIONS(1121),
    [anon_sym_else] = ACTIONS(1119),
    [anon_sym_spec] = ACTIONS(1119),
    [anon_sym_SEMI] = ACTIONS(1121),
    [anon_sym_with] = ACTIONS(1119),
    [anon_sym_to] = ACTIONS(1119),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(941)] = {
    [sym_line_comment] = STATE(941),
    [sym_block_comment] = STATE(941),
    [sym_identifier] = ACTIONS(1123),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_u256] = ACTIONS(1123),
    [anon_sym_exists] = ACTIONS(1123),
    [anon_sym_forall] = ACTIONS(1123),
    [anon_sym_choose] = ACTIONS(1123),
    [anon_sym_min] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_address] = ACTIONS(1123),
    [anon_sym_vector] = ACTIONS(1123),
    [anon_sym_for] = ACTIONS(1123),
    [anon_sym_while] = ACTIONS(1123),
    [anon_sym_friend] = ACTIONS(1123),
    [anon_sym_match] = ACTIONS(1123),
    [aux_sym_number_token1] = ACTIONS(1123),
    [aux_sym_number_token2] = ACTIONS(1125),
    [aux_sym_number_token4] = ACTIONS(1125),
    [aux_sym_number_token6] = ACTIONS(1125),
    [anon_sym_STAR] = ACTIONS(1125),
    [anon_sym_LPAREN] = ACTIONS(1125),
    [anon_sym_COMMA] = ACTIONS(1125),
    [anon_sym_RPAREN] = ACTIONS(1125),
    [anon_sym_PIPE] = ACTIONS(1123),
    [anon_sym_AMP] = ACTIONS(1123),
    [anon_sym_EQ] = ACTIONS(1123),
    [anon_sym_where] = ACTIONS(1123),
    [anon_sym_COLON] = ACTIONS(1125),
    [anon_sym_LBRACE] = ACTIONS(1125),
    [anon_sym_RBRACE] = ACTIONS(1125),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1125),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1125),
    [anon_sym_PIPE_PIPE] = ACTIONS(1125),
    [anon_sym_AMP_AMP] = ACTIONS(1125),
    [anon_sym_EQ_EQ] = ACTIONS(1123),
    [anon_sym_BANG_EQ] = ACTIONS(1125),
    [anon_sym_LT] = ACTIONS(1123),
    [anon_sym_GT] = ACTIONS(1123),
    [anon_sym_LT_EQ] = ACTIONS(1123),
    [anon_sym_GT_EQ] = ACTIONS(1125),
    [anon_sym_DOT_DOT] = ACTIONS(1125),
    [anon_sym_CARET] = ACTIONS(1125),
    [anon_sym_LT_LT] = ACTIONS(1125),
    [anon_sym_GT_GT] = ACTIONS(1125),
    [anon_sym_PLUS] = ACTIONS(1125),
    [anon_sym_DASH] = ACTIONS(1125),
    [anon_sym_SLASH] = ACTIONS(1123),
    [anon_sym_PERCENT] = ACTIONS(1125),
    [anon_sym_DOT] = ACTIONS(1123),
    [anon_sym_LBRACK] = ACTIONS(1125),
    [anon_sym_RBRACK] = ACTIONS(1125),
    [anon_sym_as] = ACTIONS(1123),
    [anon_sym_if] = ACTIONS(1123),
    [anon_sym_EQ_GT] = ACTIONS(1125),
    [anon_sym_else] = ACTIONS(1123),
    [anon_sym_spec] = ACTIONS(1123),
    [anon_sym_SEMI] = ACTIONS(1125),
    [anon_sym_with] = ACTIONS(1123),
    [anon_sym_to] = ACTIONS(1123),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(942)] = {
    [sym_line_comment] = STATE(942),
    [sym_block_comment] = STATE(942),
    [sym_identifier] = ACTIONS(1127),
    [anon_sym_u8] = ACTIONS(1127),
    [anon_sym_u16] = ACTIONS(1127),
    [anon_sym_u32] = ACTIONS(1127),
    [anon_sym_u64] = ACTIONS(1127),
    [anon_sym_u128] = ACTIONS(1127),
    [anon_sym_u256] = ACTIONS(1127),
    [anon_sym_exists] = ACTIONS(1127),
    [anon_sym_forall] = ACTIONS(1127),
    [anon_sym_choose] = ACTIONS(1127),
    [anon_sym_min] = ACTIONS(1127),
    [anon_sym_bool] = ACTIONS(1127),
    [anon_sym_address] = ACTIONS(1127),
    [anon_sym_vector] = ACTIONS(1127),
    [anon_sym_for] = ACTIONS(1127),
    [anon_sym_while] = ACTIONS(1127),
    [anon_sym_friend] = ACTIONS(1127),
    [anon_sym_match] = ACTIONS(1127),
    [aux_sym_number_token1] = ACTIONS(1127),
    [aux_sym_number_token2] = ACTIONS(1129),
    [aux_sym_number_token4] = ACTIONS(1129),
    [aux_sym_number_token6] = ACTIONS(1129),
    [anon_sym_STAR] = ACTIONS(1129),
    [anon_sym_LPAREN] = ACTIONS(1129),
    [anon_sym_COMMA] = ACTIONS(1129),
    [anon_sym_RPAREN] = ACTIONS(1129),
    [anon_sym_PIPE] = ACTIONS(1127),
    [anon_sym_AMP] = ACTIONS(1127),
    [anon_sym_EQ] = ACTIONS(1127),
    [anon_sym_where] = ACTIONS(1127),
    [anon_sym_COLON] = ACTIONS(1129),
    [anon_sym_LBRACE] = ACTIONS(1129),
    [anon_sym_RBRACE] = ACTIONS(1129),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1129),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1129),
    [anon_sym_PIPE_PIPE] = ACTIONS(1129),
    [anon_sym_AMP_AMP] = ACTIONS(1129),
    [anon_sym_EQ_EQ] = ACTIONS(1127),
    [anon_sym_BANG_EQ] = ACTIONS(1129),
    [anon_sym_LT] = ACTIONS(1127),
    [anon_sym_GT] = ACTIONS(1127),
    [anon_sym_LT_EQ] = ACTIONS(1127),
    [anon_sym_GT_EQ] = ACTIONS(1129),
    [anon_sym_DOT_DOT] = ACTIONS(1129),
    [anon_sym_CARET] = ACTIONS(1129),
    [anon_sym_LT_LT] = ACTIONS(1129),
    [anon_sym_GT_GT] = ACTIONS(1129),
    [anon_sym_PLUS] = ACTIONS(1129),
    [anon_sym_DASH] = ACTIONS(1129),
    [anon_sym_SLASH] = ACTIONS(1127),
    [anon_sym_PERCENT] = ACTIONS(1129),
    [anon_sym_DOT] = ACTIONS(1127),
    [anon_sym_LBRACK] = ACTIONS(1129),
    [anon_sym_RBRACK] = ACTIONS(1129),
    [anon_sym_as] = ACTIONS(1127),
    [anon_sym_if] = ACTIONS(1127),
    [anon_sym_EQ_GT] = ACTIONS(1129),
    [anon_sym_else] = ACTIONS(1127),
    [anon_sym_spec] = ACTIONS(1127),
    [anon_sym_SEMI] = ACTIONS(1129),
    [anon_sym_with] = ACTIONS(1127),
    [anon_sym_to] = ACTIONS(1127),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(943)] = {
    [sym_line_comment] = STATE(943),
    [sym_block_comment] = STATE(943),
    [sym_identifier] = ACTIONS(1131),
    [anon_sym_u8] = ACTIONS(1131),
    [anon_sym_u16] = ACTIONS(1131),
    [anon_sym_u32] = ACTIONS(1131),
    [anon_sym_u64] = ACTIONS(1131),
    [anon_sym_u128] = ACTIONS(1131),
    [anon_sym_u256] = ACTIONS(1131),
    [anon_sym_exists] = ACTIONS(1131),
    [anon_sym_forall] = ACTIONS(1131),
    [anon_sym_choose] = ACTIONS(1131),
    [anon_sym_min] = ACTIONS(1131),
    [anon_sym_bool] = ACTIONS(1131),
    [anon_sym_address] = ACTIONS(1131),
    [anon_sym_vector] = ACTIONS(1131),
    [anon_sym_for] = ACTIONS(1131),
    [anon_sym_while] = ACTIONS(1131),
    [anon_sym_friend] = ACTIONS(1131),
    [anon_sym_match] = ACTIONS(1131),
    [aux_sym_number_token1] = ACTIONS(1131),
    [aux_sym_number_token2] = ACTIONS(1133),
    [aux_sym_number_token4] = ACTIONS(1133),
    [aux_sym_number_token6] = ACTIONS(1133),
    [anon_sym_STAR] = ACTIONS(1133),
    [anon_sym_LPAREN] = ACTIONS(1133),
    [anon_sym_COMMA] = ACTIONS(1133),
    [anon_sym_RPAREN] = ACTIONS(1133),
    [anon_sym_PIPE] = ACTIONS(1131),
    [anon_sym_AMP] = ACTIONS(1131),
    [anon_sym_EQ] = ACTIONS(1131),
    [anon_sym_where] = ACTIONS(1131),
    [anon_sym_COLON] = ACTIONS(1133),
    [anon_sym_LBRACE] = ACTIONS(1133),
    [anon_sym_RBRACE] = ACTIONS(1133),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1133),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1133),
    [anon_sym_PIPE_PIPE] = ACTIONS(1133),
    [anon_sym_AMP_AMP] = ACTIONS(1133),
    [anon_sym_EQ_EQ] = ACTIONS(1131),
    [anon_sym_BANG_EQ] = ACTIONS(1133),
    [anon_sym_LT] = ACTIONS(1131),
    [anon_sym_GT] = ACTIONS(1131),
    [anon_sym_LT_EQ] = ACTIONS(1131),
    [anon_sym_GT_EQ] = ACTIONS(1133),
    [anon_sym_DOT_DOT] = ACTIONS(1133),
    [anon_sym_CARET] = ACTIONS(1133),
    [anon_sym_LT_LT] = ACTIONS(1133),
    [anon_sym_GT_GT] = ACTIONS(1133),
    [anon_sym_PLUS] = ACTIONS(1133),
    [anon_sym_DASH] = ACTIONS(1133),
    [anon_sym_SLASH] = ACTIONS(1131),
    [anon_sym_PERCENT] = ACTIONS(1133),
    [anon_sym_DOT] = ACTIONS(1131),
    [anon_sym_LBRACK] = ACTIONS(1133),
    [anon_sym_RBRACK] = ACTIONS(1133),
    [anon_sym_as] = ACTIONS(1131),
    [anon_sym_if] = ACTIONS(1131),
    [anon_sym_EQ_GT] = ACTIONS(1133),
    [anon_sym_else] = ACTIONS(1131),
    [anon_sym_spec] = ACTIONS(1131),
    [anon_sym_SEMI] = ACTIONS(1133),
    [anon_sym_with] = ACTIONS(1131),
    [anon_sym_to] = ACTIONS(1131),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(944)] = {
    [sym_line_comment] = STATE(944),
    [sym_block_comment] = STATE(944),
    [sym_identifier] = ACTIONS(1135),
    [anon_sym_u8] = ACTIONS(1135),
    [anon_sym_u16] = ACTIONS(1135),
    [anon_sym_u32] = ACTIONS(1135),
    [anon_sym_u64] = ACTIONS(1135),
    [anon_sym_u128] = ACTIONS(1135),
    [anon_sym_u256] = ACTIONS(1135),
    [anon_sym_exists] = ACTIONS(1135),
    [anon_sym_forall] = ACTIONS(1135),
    [anon_sym_choose] = ACTIONS(1135),
    [anon_sym_min] = ACTIONS(1135),
    [anon_sym_bool] = ACTIONS(1135),
    [anon_sym_address] = ACTIONS(1135),
    [anon_sym_vector] = ACTIONS(1135),
    [anon_sym_for] = ACTIONS(1135),
    [anon_sym_while] = ACTIONS(1135),
    [anon_sym_friend] = ACTIONS(1135),
    [anon_sym_match] = ACTIONS(1135),
    [aux_sym_number_token1] = ACTIONS(1135),
    [aux_sym_number_token2] = ACTIONS(1137),
    [aux_sym_number_token4] = ACTIONS(1137),
    [aux_sym_number_token6] = ACTIONS(1137),
    [anon_sym_STAR] = ACTIONS(1137),
    [anon_sym_LPAREN] = ACTIONS(1137),
    [anon_sym_COMMA] = ACTIONS(1137),
    [anon_sym_RPAREN] = ACTIONS(1137),
    [anon_sym_PIPE] = ACTIONS(1135),
    [anon_sym_AMP] = ACTIONS(1135),
    [anon_sym_EQ] = ACTIONS(1135),
    [anon_sym_where] = ACTIONS(1135),
    [anon_sym_COLON] = ACTIONS(1137),
    [anon_sym_LBRACE] = ACTIONS(1137),
    [anon_sym_RBRACE] = ACTIONS(1137),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1137),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1137),
    [anon_sym_PIPE_PIPE] = ACTIONS(1137),
    [anon_sym_AMP_AMP] = ACTIONS(1137),
    [anon_sym_EQ_EQ] = ACTIONS(1135),
    [anon_sym_BANG_EQ] = ACTIONS(1137),
    [anon_sym_LT] = ACTIONS(1135),
    [anon_sym_GT] = ACTIONS(1135),
    [anon_sym_LT_EQ] = ACTIONS(1135),
    [anon_sym_GT_EQ] = ACTIONS(1137),
    [anon_sym_DOT_DOT] = ACTIONS(1137),
    [anon_sym_CARET] = ACTIONS(1137),
    [anon_sym_LT_LT] = ACTIONS(1137),
    [anon_sym_GT_GT] = ACTIONS(1137),
    [anon_sym_PLUS] = ACTIONS(1137),
    [anon_sym_DASH] = ACTIONS(1137),
    [anon_sym_SLASH] = ACTIONS(1135),
    [anon_sym_PERCENT] = ACTIONS(1137),
    [anon_sym_DOT] = ACTIONS(1135),
    [anon_sym_LBRACK] = ACTIONS(1137),
    [anon_sym_RBRACK] = ACTIONS(1137),
    [anon_sym_as] = ACTIONS(1135),
    [anon_sym_if] = ACTIONS(1135),
    [anon_sym_EQ_GT] = ACTIONS(1137),
    [anon_sym_else] = ACTIONS(1135),
    [anon_sym_spec] = ACTIONS(1135),
    [anon_sym_SEMI] = ACTIONS(1137),
    [anon_sym_with] = ACTIONS(1135),
    [anon_sym_to] = ACTIONS(1135),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(945)] = {
    [sym_line_comment] = STATE(945),
    [sym_block_comment] = STATE(945),
    [sym_identifier] = ACTIONS(1139),
    [anon_sym_u8] = ACTIONS(1139),
    [anon_sym_u16] = ACTIONS(1139),
    [anon_sym_u32] = ACTIONS(1139),
    [anon_sym_u64] = ACTIONS(1139),
    [anon_sym_u128] = ACTIONS(1139),
    [anon_sym_u256] = ACTIONS(1139),
    [anon_sym_exists] = ACTIONS(1139),
    [anon_sym_forall] = ACTIONS(1139),
    [anon_sym_choose] = ACTIONS(1139),
    [anon_sym_min] = ACTIONS(1139),
    [anon_sym_bool] = ACTIONS(1139),
    [anon_sym_address] = ACTIONS(1139),
    [anon_sym_vector] = ACTIONS(1139),
    [anon_sym_for] = ACTIONS(1139),
    [anon_sym_while] = ACTIONS(1139),
    [anon_sym_friend] = ACTIONS(1139),
    [anon_sym_match] = ACTIONS(1139),
    [aux_sym_number_token1] = ACTIONS(1139),
    [aux_sym_number_token2] = ACTIONS(1141),
    [aux_sym_number_token4] = ACTIONS(1141),
    [aux_sym_number_token6] = ACTIONS(1141),
    [anon_sym_STAR] = ACTIONS(1141),
    [anon_sym_LPAREN] = ACTIONS(1141),
    [anon_sym_COMMA] = ACTIONS(1141),
    [anon_sym_RPAREN] = ACTIONS(1141),
    [anon_sym_PIPE] = ACTIONS(1139),
    [anon_sym_AMP] = ACTIONS(1139),
    [anon_sym_EQ] = ACTIONS(1139),
    [anon_sym_where] = ACTIONS(1139),
    [anon_sym_COLON] = ACTIONS(1141),
    [anon_sym_LBRACE] = ACTIONS(1141),
    [anon_sym_RBRACE] = ACTIONS(1141),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1141),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1141),
    [anon_sym_PIPE_PIPE] = ACTIONS(1141),
    [anon_sym_AMP_AMP] = ACTIONS(1141),
    [anon_sym_EQ_EQ] = ACTIONS(1139),
    [anon_sym_BANG_EQ] = ACTIONS(1141),
    [anon_sym_LT] = ACTIONS(1139),
    [anon_sym_GT] = ACTIONS(1139),
    [anon_sym_LT_EQ] = ACTIONS(1139),
    [anon_sym_GT_EQ] = ACTIONS(1141),
    [anon_sym_DOT_DOT] = ACTIONS(1141),
    [anon_sym_CARET] = ACTIONS(1141),
    [anon_sym_LT_LT] = ACTIONS(1141),
    [anon_sym_GT_GT] = ACTIONS(1141),
    [anon_sym_PLUS] = ACTIONS(1141),
    [anon_sym_DASH] = ACTIONS(1141),
    [anon_sym_SLASH] = ACTIONS(1139),
    [anon_sym_PERCENT] = ACTIONS(1141),
    [anon_sym_DOT] = ACTIONS(1139),
    [anon_sym_LBRACK] = ACTIONS(1141),
    [anon_sym_RBRACK] = ACTIONS(1141),
    [anon_sym_as] = ACTIONS(1139),
    [anon_sym_if] = ACTIONS(1139),
    [anon_sym_EQ_GT] = ACTIONS(1141),
    [anon_sym_else] = ACTIONS(1139),
    [anon_sym_spec] = ACTIONS(1139),
    [anon_sym_SEMI] = ACTIONS(1141),
    [anon_sym_with] = ACTIONS(1139),
    [anon_sym_to] = ACTIONS(1139),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(946)] = {
    [sym_line_comment] = STATE(946),
    [sym_block_comment] = STATE(946),
    [sym_identifier] = ACTIONS(1143),
    [anon_sym_u8] = ACTIONS(1143),
    [anon_sym_u16] = ACTIONS(1143),
    [anon_sym_u32] = ACTIONS(1143),
    [anon_sym_u64] = ACTIONS(1143),
    [anon_sym_u128] = ACTIONS(1143),
    [anon_sym_u256] = ACTIONS(1143),
    [anon_sym_exists] = ACTIONS(1143),
    [anon_sym_forall] = ACTIONS(1143),
    [anon_sym_choose] = ACTIONS(1143),
    [anon_sym_min] = ACTIONS(1143),
    [anon_sym_bool] = ACTIONS(1143),
    [anon_sym_address] = ACTIONS(1143),
    [anon_sym_vector] = ACTIONS(1143),
    [anon_sym_for] = ACTIONS(1143),
    [anon_sym_while] = ACTIONS(1143),
    [anon_sym_friend] = ACTIONS(1143),
    [anon_sym_match] = ACTIONS(1143),
    [aux_sym_number_token1] = ACTIONS(1143),
    [aux_sym_number_token2] = ACTIONS(1145),
    [aux_sym_number_token4] = ACTIONS(1145),
    [aux_sym_number_token6] = ACTIONS(1145),
    [anon_sym_STAR] = ACTIONS(1145),
    [anon_sym_LPAREN] = ACTIONS(1145),
    [anon_sym_COMMA] = ACTIONS(1145),
    [anon_sym_RPAREN] = ACTIONS(1145),
    [anon_sym_PIPE] = ACTIONS(1143),
    [anon_sym_AMP] = ACTIONS(1143),
    [anon_sym_EQ] = ACTIONS(1143),
    [anon_sym_where] = ACTIONS(1143),
    [anon_sym_COLON] = ACTIONS(1145),
    [anon_sym_LBRACE] = ACTIONS(1145),
    [anon_sym_RBRACE] = ACTIONS(1145),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1145),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1145),
    [anon_sym_PIPE_PIPE] = ACTIONS(1145),
    [anon_sym_AMP_AMP] = ACTIONS(1145),
    [anon_sym_EQ_EQ] = ACTIONS(1143),
    [anon_sym_BANG_EQ] = ACTIONS(1145),
    [anon_sym_LT] = ACTIONS(1143),
    [anon_sym_GT] = ACTIONS(1143),
    [anon_sym_LT_EQ] = ACTIONS(1143),
    [anon_sym_GT_EQ] = ACTIONS(1145),
    [anon_sym_DOT_DOT] = ACTIONS(1145),
    [anon_sym_CARET] = ACTIONS(1145),
    [anon_sym_LT_LT] = ACTIONS(1145),
    [anon_sym_GT_GT] = ACTIONS(1145),
    [anon_sym_PLUS] = ACTIONS(1145),
    [anon_sym_DASH] = ACTIONS(1145),
    [anon_sym_SLASH] = ACTIONS(1143),
    [anon_sym_PERCENT] = ACTIONS(1145),
    [anon_sym_DOT] = ACTIONS(1143),
    [anon_sym_LBRACK] = ACTIONS(1145),
    [anon_sym_RBRACK] = ACTIONS(1145),
    [anon_sym_as] = ACTIONS(1143),
    [anon_sym_if] = ACTIONS(1143),
    [anon_sym_EQ_GT] = ACTIONS(1145),
    [anon_sym_else] = ACTIONS(1143),
    [anon_sym_spec] = ACTIONS(1143),
    [anon_sym_SEMI] = ACTIONS(1145),
    [anon_sym_with] = ACTIONS(1143),
    [anon_sym_to] = ACTIONS(1143),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(947)] = {
    [sym_line_comment] = STATE(947),
    [sym_block_comment] = STATE(947),
    [sym_identifier] = ACTIONS(1147),
    [anon_sym_u8] = ACTIONS(1147),
    [anon_sym_u16] = ACTIONS(1147),
    [anon_sym_u32] = ACTIONS(1147),
    [anon_sym_u64] = ACTIONS(1147),
    [anon_sym_u128] = ACTIONS(1147),
    [anon_sym_u256] = ACTIONS(1147),
    [anon_sym_exists] = ACTIONS(1147),
    [anon_sym_forall] = ACTIONS(1147),
    [anon_sym_choose] = ACTIONS(1147),
    [anon_sym_min] = ACTIONS(1147),
    [anon_sym_bool] = ACTIONS(1147),
    [anon_sym_address] = ACTIONS(1147),
    [anon_sym_vector] = ACTIONS(1147),
    [anon_sym_for] = ACTIONS(1147),
    [anon_sym_while] = ACTIONS(1147),
    [anon_sym_friend] = ACTIONS(1147),
    [anon_sym_match] = ACTIONS(1147),
    [aux_sym_number_token1] = ACTIONS(1147),
    [aux_sym_number_token2] = ACTIONS(1149),
    [aux_sym_number_token4] = ACTIONS(1149),
    [aux_sym_number_token6] = ACTIONS(1149),
    [anon_sym_STAR] = ACTIONS(1149),
    [anon_sym_LPAREN] = ACTIONS(1149),
    [anon_sym_COMMA] = ACTIONS(1149),
    [anon_sym_RPAREN] = ACTIONS(1149),
    [anon_sym_PIPE] = ACTIONS(1147),
    [anon_sym_AMP] = ACTIONS(1147),
    [anon_sym_EQ] = ACTIONS(1147),
    [anon_sym_where] = ACTIONS(1147),
    [anon_sym_COLON] = ACTIONS(1149),
    [anon_sym_LBRACE] = ACTIONS(1149),
    [anon_sym_RBRACE] = ACTIONS(1149),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1149),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1149),
    [anon_sym_PIPE_PIPE] = ACTIONS(1149),
    [anon_sym_AMP_AMP] = ACTIONS(1149),
    [anon_sym_EQ_EQ] = ACTIONS(1147),
    [anon_sym_BANG_EQ] = ACTIONS(1149),
    [anon_sym_LT] = ACTIONS(1147),
    [anon_sym_GT] = ACTIONS(1147),
    [anon_sym_LT_EQ] = ACTIONS(1147),
    [anon_sym_GT_EQ] = ACTIONS(1149),
    [anon_sym_DOT_DOT] = ACTIONS(1149),
    [anon_sym_CARET] = ACTIONS(1149),
    [anon_sym_LT_LT] = ACTIONS(1149),
    [anon_sym_GT_GT] = ACTIONS(1149),
    [anon_sym_PLUS] = ACTIONS(1149),
    [anon_sym_DASH] = ACTIONS(1149),
    [anon_sym_SLASH] = ACTIONS(1147),
    [anon_sym_PERCENT] = ACTIONS(1149),
    [anon_sym_DOT] = ACTIONS(1147),
    [anon_sym_LBRACK] = ACTIONS(1149),
    [anon_sym_RBRACK] = ACTIONS(1149),
    [anon_sym_as] = ACTIONS(1147),
    [anon_sym_if] = ACTIONS(1147),
    [anon_sym_EQ_GT] = ACTIONS(1149),
    [anon_sym_else] = ACTIONS(1147),
    [anon_sym_spec] = ACTIONS(1147),
    [anon_sym_SEMI] = ACTIONS(1149),
    [anon_sym_with] = ACTIONS(1147),
    [anon_sym_to] = ACTIONS(1147),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(948)] = {
    [sym_line_comment] = STATE(948),
    [sym_block_comment] = STATE(948),
    [sym_identifier] = ACTIONS(1151),
    [anon_sym_u8] = ACTIONS(1151),
    [anon_sym_u16] = ACTIONS(1151),
    [anon_sym_u32] = ACTIONS(1151),
    [anon_sym_u64] = ACTIONS(1151),
    [anon_sym_u128] = ACTIONS(1151),
    [anon_sym_u256] = ACTIONS(1151),
    [anon_sym_exists] = ACTIONS(1151),
    [anon_sym_forall] = ACTIONS(1151),
    [anon_sym_choose] = ACTIONS(1151),
    [anon_sym_min] = ACTIONS(1151),
    [anon_sym_bool] = ACTIONS(1151),
    [anon_sym_address] = ACTIONS(1151),
    [anon_sym_vector] = ACTIONS(1151),
    [anon_sym_for] = ACTIONS(1151),
    [anon_sym_while] = ACTIONS(1151),
    [anon_sym_friend] = ACTIONS(1151),
    [anon_sym_match] = ACTIONS(1151),
    [aux_sym_number_token1] = ACTIONS(1151),
    [aux_sym_number_token2] = ACTIONS(1153),
    [aux_sym_number_token4] = ACTIONS(1153),
    [aux_sym_number_token6] = ACTIONS(1153),
    [anon_sym_STAR] = ACTIONS(1153),
    [anon_sym_LPAREN] = ACTIONS(1153),
    [anon_sym_COMMA] = ACTIONS(1153),
    [anon_sym_RPAREN] = ACTIONS(1153),
    [anon_sym_PIPE] = ACTIONS(1151),
    [anon_sym_AMP] = ACTIONS(1151),
    [anon_sym_EQ] = ACTIONS(1151),
    [anon_sym_where] = ACTIONS(1151),
    [anon_sym_COLON] = ACTIONS(1153),
    [anon_sym_LBRACE] = ACTIONS(1153),
    [anon_sym_RBRACE] = ACTIONS(1153),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1153),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1153),
    [anon_sym_PIPE_PIPE] = ACTIONS(1153),
    [anon_sym_AMP_AMP] = ACTIONS(1153),
    [anon_sym_EQ_EQ] = ACTIONS(1151),
    [anon_sym_BANG_EQ] = ACTIONS(1153),
    [anon_sym_LT] = ACTIONS(1151),
    [anon_sym_GT] = ACTIONS(1151),
    [anon_sym_LT_EQ] = ACTIONS(1151),
    [anon_sym_GT_EQ] = ACTIONS(1153),
    [anon_sym_DOT_DOT] = ACTIONS(1153),
    [anon_sym_CARET] = ACTIONS(1153),
    [anon_sym_LT_LT] = ACTIONS(1153),
    [anon_sym_GT_GT] = ACTIONS(1153),
    [anon_sym_PLUS] = ACTIONS(1153),
    [anon_sym_DASH] = ACTIONS(1153),
    [anon_sym_SLASH] = ACTIONS(1151),
    [anon_sym_PERCENT] = ACTIONS(1153),
    [anon_sym_DOT] = ACTIONS(1151),
    [anon_sym_LBRACK] = ACTIONS(1153),
    [anon_sym_RBRACK] = ACTIONS(1153),
    [anon_sym_as] = ACTIONS(1151),
    [anon_sym_if] = ACTIONS(1151),
    [anon_sym_EQ_GT] = ACTIONS(1153),
    [anon_sym_else] = ACTIONS(1151),
    [anon_sym_spec] = ACTIONS(1151),
    [anon_sym_SEMI] = ACTIONS(1153),
    [anon_sym_with] = ACTIONS(1151),
    [anon_sym_to] = ACTIONS(1151),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(949)] = {
    [sym_line_comment] = STATE(949),
    [sym_block_comment] = STATE(949),
    [sym_identifier] = ACTIONS(1155),
    [anon_sym_u8] = ACTIONS(1155),
    [anon_sym_u16] = ACTIONS(1155),
    [anon_sym_u32] = ACTIONS(1155),
    [anon_sym_u64] = ACTIONS(1155),
    [anon_sym_u128] = ACTIONS(1155),
    [anon_sym_u256] = ACTIONS(1155),
    [anon_sym_exists] = ACTIONS(1155),
    [anon_sym_forall] = ACTIONS(1155),
    [anon_sym_choose] = ACTIONS(1155),
    [anon_sym_min] = ACTIONS(1155),
    [anon_sym_bool] = ACTIONS(1155),
    [anon_sym_address] = ACTIONS(1155),
    [anon_sym_vector] = ACTIONS(1155),
    [anon_sym_for] = ACTIONS(1155),
    [anon_sym_while] = ACTIONS(1155),
    [anon_sym_friend] = ACTIONS(1155),
    [anon_sym_match] = ACTIONS(1155),
    [aux_sym_number_token1] = ACTIONS(1155),
    [aux_sym_number_token2] = ACTIONS(1157),
    [aux_sym_number_token4] = ACTIONS(1157),
    [aux_sym_number_token6] = ACTIONS(1157),
    [anon_sym_STAR] = ACTIONS(1157),
    [anon_sym_LPAREN] = ACTIONS(1157),
    [anon_sym_COMMA] = ACTIONS(1157),
    [anon_sym_RPAREN] = ACTIONS(1157),
    [anon_sym_PIPE] = ACTIONS(1155),
    [anon_sym_AMP] = ACTIONS(1155),
    [anon_sym_EQ] = ACTIONS(1155),
    [anon_sym_where] = ACTIONS(1155),
    [anon_sym_COLON] = ACTIONS(1157),
    [anon_sym_LBRACE] = ACTIONS(1157),
    [anon_sym_RBRACE] = ACTIONS(1157),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1157),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1157),
    [anon_sym_PIPE_PIPE] = ACTIONS(1157),
    [anon_sym_AMP_AMP] = ACTIONS(1157),
    [anon_sym_EQ_EQ] = ACTIONS(1155),
    [anon_sym_BANG_EQ] = ACTIONS(1157),
    [anon_sym_LT] = ACTIONS(1155),
    [anon_sym_GT] = ACTIONS(1155),
    [anon_sym_LT_EQ] = ACTIONS(1155),
    [anon_sym_GT_EQ] = ACTIONS(1157),
    [anon_sym_DOT_DOT] = ACTIONS(1157),
    [anon_sym_CARET] = ACTIONS(1157),
    [anon_sym_LT_LT] = ACTIONS(1157),
    [anon_sym_GT_GT] = ACTIONS(1157),
    [anon_sym_PLUS] = ACTIONS(1157),
    [anon_sym_DASH] = ACTIONS(1157),
    [anon_sym_SLASH] = ACTIONS(1155),
    [anon_sym_PERCENT] = ACTIONS(1157),
    [anon_sym_DOT] = ACTIONS(1155),
    [anon_sym_LBRACK] = ACTIONS(1157),
    [anon_sym_RBRACK] = ACTIONS(1157),
    [anon_sym_as] = ACTIONS(1155),
    [anon_sym_if] = ACTIONS(1155),
    [anon_sym_EQ_GT] = ACTIONS(1157),
    [anon_sym_else] = ACTIONS(1155),
    [anon_sym_spec] = ACTIONS(1155),
    [anon_sym_SEMI] = ACTIONS(1157),
    [anon_sym_with] = ACTIONS(1155),
    [anon_sym_to] = ACTIONS(1155),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(950)] = {
    [sym_line_comment] = STATE(950),
    [sym_block_comment] = STATE(950),
    [sym_identifier] = ACTIONS(1159),
    [anon_sym_u8] = ACTIONS(1159),
    [anon_sym_u16] = ACTIONS(1159),
    [anon_sym_u32] = ACTIONS(1159),
    [anon_sym_u64] = ACTIONS(1159),
    [anon_sym_u128] = ACTIONS(1159),
    [anon_sym_u256] = ACTIONS(1159),
    [anon_sym_exists] = ACTIONS(1159),
    [anon_sym_forall] = ACTIONS(1159),
    [anon_sym_choose] = ACTIONS(1159),
    [anon_sym_min] = ACTIONS(1159),
    [anon_sym_bool] = ACTIONS(1159),
    [anon_sym_address] = ACTIONS(1159),
    [anon_sym_vector] = ACTIONS(1159),
    [anon_sym_for] = ACTIONS(1159),
    [anon_sym_while] = ACTIONS(1159),
    [anon_sym_friend] = ACTIONS(1159),
    [anon_sym_match] = ACTIONS(1159),
    [aux_sym_number_token1] = ACTIONS(1159),
    [aux_sym_number_token2] = ACTIONS(1161),
    [aux_sym_number_token4] = ACTIONS(1161),
    [aux_sym_number_token6] = ACTIONS(1161),
    [anon_sym_STAR] = ACTIONS(1161),
    [anon_sym_LPAREN] = ACTIONS(1161),
    [anon_sym_COMMA] = ACTIONS(1161),
    [anon_sym_RPAREN] = ACTIONS(1161),
    [anon_sym_PIPE] = ACTIONS(1159),
    [anon_sym_AMP] = ACTIONS(1159),
    [anon_sym_EQ] = ACTIONS(1159),
    [anon_sym_where] = ACTIONS(1159),
    [anon_sym_COLON] = ACTIONS(1161),
    [anon_sym_LBRACE] = ACTIONS(1161),
    [anon_sym_RBRACE] = ACTIONS(1161),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1161),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1161),
    [anon_sym_PIPE_PIPE] = ACTIONS(1161),
    [anon_sym_AMP_AMP] = ACTIONS(1161),
    [anon_sym_EQ_EQ] = ACTIONS(1159),
    [anon_sym_BANG_EQ] = ACTIONS(1161),
    [anon_sym_LT] = ACTIONS(1159),
    [anon_sym_GT] = ACTIONS(1159),
    [anon_sym_LT_EQ] = ACTIONS(1159),
    [anon_sym_GT_EQ] = ACTIONS(1161),
    [anon_sym_DOT_DOT] = ACTIONS(1161),
    [anon_sym_CARET] = ACTIONS(1161),
    [anon_sym_LT_LT] = ACTIONS(1161),
    [anon_sym_GT_GT] = ACTIONS(1161),
    [anon_sym_PLUS] = ACTIONS(1161),
    [anon_sym_DASH] = ACTIONS(1161),
    [anon_sym_SLASH] = ACTIONS(1159),
    [anon_sym_PERCENT] = ACTIONS(1161),
    [anon_sym_DOT] = ACTIONS(1159),
    [anon_sym_LBRACK] = ACTIONS(1161),
    [anon_sym_RBRACK] = ACTIONS(1161),
    [anon_sym_as] = ACTIONS(1159),
    [anon_sym_if] = ACTIONS(1159),
    [anon_sym_EQ_GT] = ACTIONS(1161),
    [anon_sym_else] = ACTIONS(1159),
    [anon_sym_spec] = ACTIONS(1159),
    [anon_sym_SEMI] = ACTIONS(1161),
    [anon_sym_with] = ACTIONS(1159),
    [anon_sym_to] = ACTIONS(1159),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(951)] = {
    [sym_line_comment] = STATE(951),
    [sym_block_comment] = STATE(951),
    [sym_identifier] = ACTIONS(1163),
    [anon_sym_u8] = ACTIONS(1163),
    [anon_sym_u16] = ACTIONS(1163),
    [anon_sym_u32] = ACTIONS(1163),
    [anon_sym_u64] = ACTIONS(1163),
    [anon_sym_u128] = ACTIONS(1163),
    [anon_sym_u256] = ACTIONS(1163),
    [anon_sym_exists] = ACTIONS(1163),
    [anon_sym_forall] = ACTIONS(1163),
    [anon_sym_choose] = ACTIONS(1163),
    [anon_sym_min] = ACTIONS(1163),
    [anon_sym_bool] = ACTIONS(1163),
    [anon_sym_address] = ACTIONS(1163),
    [anon_sym_vector] = ACTIONS(1163),
    [anon_sym_for] = ACTIONS(1163),
    [anon_sym_while] = ACTIONS(1163),
    [anon_sym_friend] = ACTIONS(1163),
    [anon_sym_match] = ACTIONS(1163),
    [aux_sym_number_token1] = ACTIONS(1163),
    [aux_sym_number_token2] = ACTIONS(1165),
    [aux_sym_number_token4] = ACTIONS(1165),
    [aux_sym_number_token6] = ACTIONS(1165),
    [anon_sym_STAR] = ACTIONS(1165),
    [anon_sym_LPAREN] = ACTIONS(1165),
    [anon_sym_COMMA] = ACTIONS(1165),
    [anon_sym_RPAREN] = ACTIONS(1165),
    [anon_sym_PIPE] = ACTIONS(1163),
    [anon_sym_AMP] = ACTIONS(1163),
    [anon_sym_EQ] = ACTIONS(1163),
    [anon_sym_where] = ACTIONS(1163),
    [anon_sym_COLON] = ACTIONS(1165),
    [anon_sym_LBRACE] = ACTIONS(1165),
    [anon_sym_RBRACE] = ACTIONS(1165),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1165),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1165),
    [anon_sym_PIPE_PIPE] = ACTIONS(1165),
    [anon_sym_AMP_AMP] = ACTIONS(1165),
    [anon_sym_EQ_EQ] = ACTIONS(1163),
    [anon_sym_BANG_EQ] = ACTIONS(1165),
    [anon_sym_LT] = ACTIONS(1163),
    [anon_sym_GT] = ACTIONS(1163),
    [anon_sym_LT_EQ] = ACTIONS(1163),
    [anon_sym_GT_EQ] = ACTIONS(1165),
    [anon_sym_DOT_DOT] = ACTIONS(1165),
    [anon_sym_CARET] = ACTIONS(1165),
    [anon_sym_LT_LT] = ACTIONS(1165),
    [anon_sym_GT_GT] = ACTIONS(1165),
    [anon_sym_PLUS] = ACTIONS(1165),
    [anon_sym_DASH] = ACTIONS(1165),
    [anon_sym_SLASH] = ACTIONS(1163),
    [anon_sym_PERCENT] = ACTIONS(1165),
    [anon_sym_DOT] = ACTIONS(1163),
    [anon_sym_LBRACK] = ACTIONS(1165),
    [anon_sym_RBRACK] = ACTIONS(1165),
    [anon_sym_as] = ACTIONS(1163),
    [anon_sym_if] = ACTIONS(1163),
    [anon_sym_EQ_GT] = ACTIONS(1165),
    [anon_sym_else] = ACTIONS(1163),
    [anon_sym_spec] = ACTIONS(1163),
    [anon_sym_SEMI] = ACTIONS(1165),
    [anon_sym_with] = ACTIONS(1163),
    [anon_sym_to] = ACTIONS(1163),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(952)] = {
    [sym_line_comment] = STATE(952),
    [sym_block_comment] = STATE(952),
    [sym_identifier] = ACTIONS(1167),
    [anon_sym_u8] = ACTIONS(1167),
    [anon_sym_u16] = ACTIONS(1167),
    [anon_sym_u32] = ACTIONS(1167),
    [anon_sym_u64] = ACTIONS(1167),
    [anon_sym_u128] = ACTIONS(1167),
    [anon_sym_u256] = ACTIONS(1167),
    [anon_sym_exists] = ACTIONS(1167),
    [anon_sym_forall] = ACTIONS(1167),
    [anon_sym_choose] = ACTIONS(1167),
    [anon_sym_min] = ACTIONS(1167),
    [anon_sym_bool] = ACTIONS(1167),
    [anon_sym_address] = ACTIONS(1167),
    [anon_sym_vector] = ACTIONS(1167),
    [anon_sym_for] = ACTIONS(1167),
    [anon_sym_while] = ACTIONS(1167),
    [anon_sym_friend] = ACTIONS(1167),
    [anon_sym_match] = ACTIONS(1167),
    [aux_sym_number_token1] = ACTIONS(1167),
    [aux_sym_number_token2] = ACTIONS(1169),
    [aux_sym_number_token4] = ACTIONS(1169),
    [aux_sym_number_token6] = ACTIONS(1169),
    [anon_sym_STAR] = ACTIONS(1169),
    [anon_sym_LPAREN] = ACTIONS(1169),
    [anon_sym_COMMA] = ACTIONS(1169),
    [anon_sym_RPAREN] = ACTIONS(1169),
    [anon_sym_PIPE] = ACTIONS(1167),
    [anon_sym_AMP] = ACTIONS(1167),
    [anon_sym_EQ] = ACTIONS(1167),
    [anon_sym_where] = ACTIONS(1167),
    [anon_sym_COLON] = ACTIONS(1169),
    [anon_sym_LBRACE] = ACTIONS(1169),
    [anon_sym_RBRACE] = ACTIONS(1169),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1169),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1169),
    [anon_sym_PIPE_PIPE] = ACTIONS(1169),
    [anon_sym_AMP_AMP] = ACTIONS(1169),
    [anon_sym_EQ_EQ] = ACTIONS(1167),
    [anon_sym_BANG_EQ] = ACTIONS(1169),
    [anon_sym_LT] = ACTIONS(1167),
    [anon_sym_GT] = ACTIONS(1167),
    [anon_sym_LT_EQ] = ACTIONS(1167),
    [anon_sym_GT_EQ] = ACTIONS(1169),
    [anon_sym_DOT_DOT] = ACTIONS(1169),
    [anon_sym_CARET] = ACTIONS(1169),
    [anon_sym_LT_LT] = ACTIONS(1169),
    [anon_sym_GT_GT] = ACTIONS(1169),
    [anon_sym_PLUS] = ACTIONS(1169),
    [anon_sym_DASH] = ACTIONS(1169),
    [anon_sym_SLASH] = ACTIONS(1167),
    [anon_sym_PERCENT] = ACTIONS(1169),
    [anon_sym_DOT] = ACTIONS(1167),
    [anon_sym_LBRACK] = ACTIONS(1169),
    [anon_sym_RBRACK] = ACTIONS(1169),
    [anon_sym_as] = ACTIONS(1167),
    [anon_sym_if] = ACTIONS(1167),
    [anon_sym_EQ_GT] = ACTIONS(1169),
    [anon_sym_else] = ACTIONS(1167),
    [anon_sym_spec] = ACTIONS(1167),
    [anon_sym_SEMI] = ACTIONS(1169),
    [anon_sym_with] = ACTIONS(1167),
    [anon_sym_to] = ACTIONS(1167),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(953)] = {
    [sym_line_comment] = STATE(953),
    [sym_block_comment] = STATE(953),
    [sym_identifier] = ACTIONS(1171),
    [anon_sym_u8] = ACTIONS(1171),
    [anon_sym_u16] = ACTIONS(1171),
    [anon_sym_u32] = ACTIONS(1171),
    [anon_sym_u64] = ACTIONS(1171),
    [anon_sym_u128] = ACTIONS(1171),
    [anon_sym_u256] = ACTIONS(1171),
    [anon_sym_exists] = ACTIONS(1171),
    [anon_sym_forall] = ACTIONS(1171),
    [anon_sym_choose] = ACTIONS(1171),
    [anon_sym_min] = ACTIONS(1171),
    [anon_sym_bool] = ACTIONS(1171),
    [anon_sym_address] = ACTIONS(1171),
    [anon_sym_vector] = ACTIONS(1171),
    [anon_sym_for] = ACTIONS(1171),
    [anon_sym_while] = ACTIONS(1171),
    [anon_sym_friend] = ACTIONS(1171),
    [anon_sym_match] = ACTIONS(1171),
    [aux_sym_number_token1] = ACTIONS(1171),
    [aux_sym_number_token2] = ACTIONS(1173),
    [aux_sym_number_token4] = ACTIONS(1173),
    [aux_sym_number_token6] = ACTIONS(1173),
    [anon_sym_STAR] = ACTIONS(1173),
    [anon_sym_LPAREN] = ACTIONS(1173),
    [anon_sym_COMMA] = ACTIONS(1173),
    [anon_sym_RPAREN] = ACTIONS(1173),
    [anon_sym_PIPE] = ACTIONS(1171),
    [anon_sym_AMP] = ACTIONS(1171),
    [anon_sym_EQ] = ACTIONS(1171),
    [anon_sym_where] = ACTIONS(1171),
    [anon_sym_COLON] = ACTIONS(1173),
    [anon_sym_LBRACE] = ACTIONS(1173),
    [anon_sym_RBRACE] = ACTIONS(1173),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1173),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1173),
    [anon_sym_PIPE_PIPE] = ACTIONS(1173),
    [anon_sym_AMP_AMP] = ACTIONS(1173),
    [anon_sym_EQ_EQ] = ACTIONS(1171),
    [anon_sym_BANG_EQ] = ACTIONS(1173),
    [anon_sym_LT] = ACTIONS(1171),
    [anon_sym_GT] = ACTIONS(1171),
    [anon_sym_LT_EQ] = ACTIONS(1171),
    [anon_sym_GT_EQ] = ACTIONS(1173),
    [anon_sym_DOT_DOT] = ACTIONS(1173),
    [anon_sym_CARET] = ACTIONS(1173),
    [anon_sym_LT_LT] = ACTIONS(1173),
    [anon_sym_GT_GT] = ACTIONS(1173),
    [anon_sym_PLUS] = ACTIONS(1173),
    [anon_sym_DASH] = ACTIONS(1173),
    [anon_sym_SLASH] = ACTIONS(1171),
    [anon_sym_PERCENT] = ACTIONS(1173),
    [anon_sym_DOT] = ACTIONS(1171),
    [anon_sym_LBRACK] = ACTIONS(1173),
    [anon_sym_RBRACK] = ACTIONS(1173),
    [anon_sym_as] = ACTIONS(1171),
    [anon_sym_if] = ACTIONS(1171),
    [anon_sym_EQ_GT] = ACTIONS(1173),
    [anon_sym_else] = ACTIONS(1171),
    [anon_sym_spec] = ACTIONS(1171),
    [anon_sym_SEMI] = ACTIONS(1173),
    [anon_sym_with] = ACTIONS(1171),
    [anon_sym_to] = ACTIONS(1171),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(954)] = {
    [sym_line_comment] = STATE(954),
    [sym_block_comment] = STATE(954),
    [sym_identifier] = ACTIONS(1175),
    [anon_sym_u8] = ACTIONS(1175),
    [anon_sym_u16] = ACTIONS(1175),
    [anon_sym_u32] = ACTIONS(1175),
    [anon_sym_u64] = ACTIONS(1175),
    [anon_sym_u128] = ACTIONS(1175),
    [anon_sym_u256] = ACTIONS(1175),
    [anon_sym_exists] = ACTIONS(1175),
    [anon_sym_forall] = ACTIONS(1175),
    [anon_sym_choose] = ACTIONS(1175),
    [anon_sym_min] = ACTIONS(1175),
    [anon_sym_bool] = ACTIONS(1175),
    [anon_sym_address] = ACTIONS(1175),
    [anon_sym_vector] = ACTIONS(1175),
    [anon_sym_for] = ACTIONS(1175),
    [anon_sym_while] = ACTIONS(1175),
    [anon_sym_friend] = ACTIONS(1175),
    [anon_sym_match] = ACTIONS(1175),
    [aux_sym_number_token1] = ACTIONS(1175),
    [aux_sym_number_token2] = ACTIONS(1177),
    [aux_sym_number_token4] = ACTIONS(1177),
    [aux_sym_number_token6] = ACTIONS(1177),
    [anon_sym_STAR] = ACTIONS(1177),
    [anon_sym_LPAREN] = ACTIONS(1177),
    [anon_sym_COMMA] = ACTIONS(1177),
    [anon_sym_RPAREN] = ACTIONS(1177),
    [anon_sym_PIPE] = ACTIONS(1175),
    [anon_sym_AMP] = ACTIONS(1175),
    [anon_sym_EQ] = ACTIONS(1175),
    [anon_sym_where] = ACTIONS(1175),
    [anon_sym_COLON] = ACTIONS(1177),
    [anon_sym_LBRACE] = ACTIONS(1177),
    [anon_sym_RBRACE] = ACTIONS(1177),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1177),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1177),
    [anon_sym_PIPE_PIPE] = ACTIONS(1177),
    [anon_sym_AMP_AMP] = ACTIONS(1177),
    [anon_sym_EQ_EQ] = ACTIONS(1175),
    [anon_sym_BANG_EQ] = ACTIONS(1177),
    [anon_sym_LT] = ACTIONS(1175),
    [anon_sym_GT] = ACTIONS(1175),
    [anon_sym_LT_EQ] = ACTIONS(1175),
    [anon_sym_GT_EQ] = ACTIONS(1177),
    [anon_sym_DOT_DOT] = ACTIONS(1177),
    [anon_sym_CARET] = ACTIONS(1177),
    [anon_sym_LT_LT] = ACTIONS(1177),
    [anon_sym_GT_GT] = ACTIONS(1177),
    [anon_sym_PLUS] = ACTIONS(1177),
    [anon_sym_DASH] = ACTIONS(1177),
    [anon_sym_SLASH] = ACTIONS(1175),
    [anon_sym_PERCENT] = ACTIONS(1177),
    [anon_sym_DOT] = ACTIONS(1175),
    [anon_sym_LBRACK] = ACTIONS(1177),
    [anon_sym_RBRACK] = ACTIONS(1177),
    [anon_sym_as] = ACTIONS(1175),
    [anon_sym_if] = ACTIONS(1175),
    [anon_sym_EQ_GT] = ACTIONS(1177),
    [anon_sym_else] = ACTIONS(1175),
    [anon_sym_spec] = ACTIONS(1175),
    [anon_sym_SEMI] = ACTIONS(1177),
    [anon_sym_with] = ACTIONS(1175),
    [anon_sym_to] = ACTIONS(1175),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(955)] = {
    [sym_line_comment] = STATE(955),
    [sym_block_comment] = STATE(955),
    [sym_identifier] = ACTIONS(1179),
    [anon_sym_u8] = ACTIONS(1179),
    [anon_sym_u16] = ACTIONS(1179),
    [anon_sym_u32] = ACTIONS(1179),
    [anon_sym_u64] = ACTIONS(1179),
    [anon_sym_u128] = ACTIONS(1179),
    [anon_sym_u256] = ACTIONS(1179),
    [anon_sym_exists] = ACTIONS(1179),
    [anon_sym_forall] = ACTIONS(1179),
    [anon_sym_choose] = ACTIONS(1179),
    [anon_sym_min] = ACTIONS(1179),
    [anon_sym_bool] = ACTIONS(1179),
    [anon_sym_address] = ACTIONS(1179),
    [anon_sym_vector] = ACTIONS(1179),
    [anon_sym_for] = ACTIONS(1179),
    [anon_sym_while] = ACTIONS(1179),
    [anon_sym_friend] = ACTIONS(1179),
    [anon_sym_match] = ACTIONS(1179),
    [aux_sym_number_token1] = ACTIONS(1179),
    [aux_sym_number_token2] = ACTIONS(1181),
    [aux_sym_number_token4] = ACTIONS(1181),
    [aux_sym_number_token6] = ACTIONS(1181),
    [anon_sym_STAR] = ACTIONS(1181),
    [anon_sym_LPAREN] = ACTIONS(1181),
    [anon_sym_COMMA] = ACTIONS(1181),
    [anon_sym_RPAREN] = ACTIONS(1181),
    [anon_sym_PIPE] = ACTIONS(1179),
    [anon_sym_AMP] = ACTIONS(1179),
    [anon_sym_EQ] = ACTIONS(1179),
    [anon_sym_where] = ACTIONS(1179),
    [anon_sym_COLON] = ACTIONS(1181),
    [anon_sym_LBRACE] = ACTIONS(1181),
    [anon_sym_RBRACE] = ACTIONS(1181),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1181),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1181),
    [anon_sym_PIPE_PIPE] = ACTIONS(1181),
    [anon_sym_AMP_AMP] = ACTIONS(1181),
    [anon_sym_EQ_EQ] = ACTIONS(1179),
    [anon_sym_BANG_EQ] = ACTIONS(1181),
    [anon_sym_LT] = ACTIONS(1179),
    [anon_sym_GT] = ACTIONS(1179),
    [anon_sym_LT_EQ] = ACTIONS(1179),
    [anon_sym_GT_EQ] = ACTIONS(1181),
    [anon_sym_DOT_DOT] = ACTIONS(1181),
    [anon_sym_CARET] = ACTIONS(1181),
    [anon_sym_LT_LT] = ACTIONS(1181),
    [anon_sym_GT_GT] = ACTIONS(1181),
    [anon_sym_PLUS] = ACTIONS(1181),
    [anon_sym_DASH] = ACTIONS(1181),
    [anon_sym_SLASH] = ACTIONS(1179),
    [anon_sym_PERCENT] = ACTIONS(1181),
    [anon_sym_DOT] = ACTIONS(1179),
    [anon_sym_LBRACK] = ACTIONS(1181),
    [anon_sym_RBRACK] = ACTIONS(1181),
    [anon_sym_as] = ACTIONS(1179),
    [anon_sym_if] = ACTIONS(1179),
    [anon_sym_EQ_GT] = ACTIONS(1181),
    [anon_sym_else] = ACTIONS(1179),
    [anon_sym_spec] = ACTIONS(1179),
    [anon_sym_SEMI] = ACTIONS(1181),
    [anon_sym_with] = ACTIONS(1179),
    [anon_sym_to] = ACTIONS(1179),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
  [STATE(956)] = {
    [sym_line_comment] = STATE(956),
    [sym_block_comment] = STATE(956),
    [sym_identifier] = ACTIONS(1183),
    [anon_sym_u8] = ACTIONS(1183),
    [anon_sym_u16] = ACTIONS(1183),
    [anon_sym_u32] = ACTIONS(1183),
    [anon_sym_u64] = ACTIONS(1183),
    [anon_sym_u128] = ACTIONS(1183),
    [anon_sym_u256] = ACTIONS(1183),
    [anon_sym_exists] = ACTIONS(1183),
    [anon_sym_forall] = ACTIONS(1183),
    [anon_sym_choose] = ACTIONS(1183),
    [anon_sym_min] = ACTIONS(1183),
    [anon_sym_bool] = ACTIONS(1183),
    [anon_sym_address] = ACTIONS(1183),
    [anon_sym_vector] = ACTIONS(1183),
    [anon_sym_for] = ACTIONS(1183),
    [anon_sym_while] = ACTIONS(1183),
    [anon_sym_friend] = ACTIONS(1183),
    [anon_sym_match] = ACTIONS(1183),
    [aux_sym_number_token1] = ACTIONS(1183),
    [aux_sym_number_token2] = ACTIONS(1185),
    [aux_sym_number_token4] = ACTIONS(1185),
    [aux_sym_number_token6] = ACTIONS(1185),
    [anon_sym_STAR] = ACTIONS(1185),
    [anon_sym_LPAREN] = ACTIONS(1185),
    [anon_sym_COMMA] = ACTIONS(1185),
    [anon_sym_RPAREN] = ACTIONS(1185),
    [anon_sym_PIPE] = ACTIONS(1183),
    [anon_sym_AMP] = ACTIONS(1183),
    [anon_sym_EQ] = ACTIONS(1183),
    [anon_sym_where] = ACTIONS(1183),
    [anon_sym_COLON] = ACTIONS(1185),
    [anon_sym_LBRACE] = ACTIONS(1185),
    [anon_sym_RBRACE] = ACTIONS(1185),
    [anon_sym_EQ_EQ_GT] = ACTIONS(1185),
    [anon_sym_LT_EQ_EQ_GT] = ACTIONS(1185),
    [anon_sym_PIPE_PIPE] = ACTIONS(1185),
    [anon_sym_AMP_AMP] = ACTIONS(1185),
    [anon_sym_EQ_EQ] = ACTIONS(1183),
    [anon_sym_BANG_EQ] = ACTIONS(1185),
    [anon_sym_LT] = ACTIONS(1183),
    [anon_sym_GT] = ACTIONS(1183),
    [anon_sym_LT_EQ] = ACTIONS(1183),
    [anon_sym_GT_EQ] = ACTIONS(1185),
    [anon_sym_DOT_DOT] = ACTIONS(1185),
    [anon_sym_CARET] = ACTIONS(1185),
    [anon_sym_LT_LT] = ACTIONS(1185),
    [anon_sym_GT_GT] = ACTIONS(1185),
    [anon_sym_PLUS] = ACTIONS(1185),
    [anon_sym_DASH] = ACTIONS(1185),
    [anon_sym_SLASH] = ACTIONS(1183),
    [anon_sym_PERCENT] = ACTIONS(1185),
    [anon_sym_DOT] = ACTIONS(1183),
    [anon_sym_LBRACK] = ACTIONS(1185),
    [anon_sym_RBRACK] = ACTIONS(1185),
    [anon_sym_as] = ACTIONS(1183),
    [anon_sym_if] = ACTIONS(1183),
    [anon_sym_EQ_GT] = ACTIONS(1185),
    [anon_sym_else] = ACTIONS(1183),
    [anon_sym_spec] = ACTIONS(1183),
    [anon_sym_SEMI] = ACTIONS(1185),
    [anon_sym_with] = ACTIONS(1183),
    [anon_sym_to] = ACTIONS(1183),
    [anon_sym_SLASH_SLASH] = ACTIONS(19),
    [anon_sym_SLASH_STAR] = ACTIONS(21),
  },
};

static const uint16_t ts_small_parse_table[] = {
  [0] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(957), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1189), 14,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_BANG,
      anon_sym_AT,
      anon_sym_POUND,
    ACTIONS(1187), 42,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_let,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [71] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1191), 1,
      sym_identifier,
    ACTIONS(1194), 1,
      anon_sym_min,
    STATE(2303), 1,
      sym_quantifier_bind,
    STATE(958), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(825), 23,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
    ACTIONS(823), 30,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_spec,
  [148] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(869), 1,
      anon_sym_LT2,
    ACTIONS(1201), 1,
      anon_sym_LPAREN,
    ACTIONS(1204), 1,
      anon_sym_LBRACE,
    ACTIONS(1206), 1,
      anon_sym_BANG,
    STATE(905), 1,
      sym_call_args,
    STATE(982), 1,
      sym_type_args,
    STATE(959), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1199), 20,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
    ACTIONS(1197), 30,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [231] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(960), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1210), 14,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_BANG,
      anon_sym_AT,
      anon_sym_POUND,
    ACTIONS(1208), 42,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_let,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [302] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(60), 1,
      sym_parenthesized_expr,
    STATE(961), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(817), 22,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
    ACTIONS(815), 32,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_spec,
  [377] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1214), 1,
      sym_identifier,
    STATE(1699), 1,
      sym_quantifier_bind,
    STATE(962), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(825), 23,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
    ACTIONS(823), 31,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_spec,
  [452] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1214), 1,
      sym_identifier,
    STATE(1723), 1,
      sym_quantifier_bind,
    STATE(963), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(825), 23,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
    ACTIONS(823), 31,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_spec,
  [527] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1191), 1,
      sym_identifier,
    ACTIONS(1217), 1,
      anon_sym_min,
    STATE(2203), 1,
      sym_quantifier_bind,
    STATE(964), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(825), 23,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
    ACTIONS(823), 30,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_spec,
  [604] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(965), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1222), 14,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_BANG,
      anon_sym_AT,
      anon_sym_POUND,
    ACTIONS(1220), 42,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_let,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [675] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(966), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1226), 14,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_BANG,
      anon_sym_AT,
      anon_sym_POUND,
    ACTIONS(1224), 42,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_let,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [746] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(967), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1230), 14,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_BANG,
      anon_sym_AT,
      anon_sym_POUND,
    ACTIONS(1228), 42,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_let,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [817] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(968), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1234), 14,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_BANG,
      anon_sym_AT,
      anon_sym_POUND,
    ACTIONS(1232), 42,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_let,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [888] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(969), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1238), 14,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_BANG,
      anon_sym_AT,
      anon_sym_POUND,
    ACTIONS(1236), 42,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_let,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [959] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1214), 1,
      sym_identifier,
    STATE(1697), 1,
      sym_quantifier_bind,
    STATE(970), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(825), 23,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
    ACTIONS(823), 30,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
  [1033] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(80), 1,
      sym_parenthesized_expr,
    STATE(971), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(817), 22,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
    ACTIONS(815), 31,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_spec,
  [1107] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(126), 1,
      sym_parenthesized_expr,
    STATE(972), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(817), 22,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
    ACTIONS(815), 31,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
  [1181] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1191), 1,
      sym_identifier,
    ACTIONS(1240), 1,
      anon_sym_min,
    STATE(2285), 1,
      sym_quantifier_bind,
    STATE(973), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(825), 23,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
    ACTIONS(823), 29,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
  [1257] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1191), 1,
      sym_identifier,
    ACTIONS(1243), 1,
      anon_sym_min,
    STATE(2405), 1,
      sym_quantifier_bind,
    STATE(974), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(825), 23,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
    ACTIONS(823), 29,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
  [1333] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1214), 1,
      sym_identifier,
    STATE(1743), 1,
      sym_quantifier_bind,
    STATE(975), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(825), 23,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
    ACTIONS(823), 30,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
  [1407] = 30,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1246), 1,
      sym_identifier,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1260), 1,
      anon_sym_PIPE,
    ACTIONS(1262), 1,
      anon_sym_AMP,
    ACTIONS(1264), 1,
      anon_sym_AMPmut,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1521), 1,
      sym_primitive_type,
    STATE(1526), 1,
      sym_number_type,
    STATE(1533), 1,
      sym_name_access_chain,
    STATE(1539), 1,
      sym_discouraged_name,
    STATE(1555), 1,
      sym_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1565), 1,
      sym__type,
    STATE(2175), 1,
      sym__leading_name_access,
    STATE(976), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1527), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(1252), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(1250), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(1254), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(1266), 5,
      anon_sym_where,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
    ACTIONS(1248), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
    ACTIONS(1258), 9,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
  [1526] = 30,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1246), 1,
      sym_identifier,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1260), 1,
      anon_sym_PIPE,
    ACTIONS(1262), 1,
      anon_sym_AMP,
    ACTIONS(1264), 1,
      anon_sym_AMPmut,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1521), 1,
      sym_primitive_type,
    STATE(1526), 1,
      sym_number_type,
    STATE(1533), 1,
      sym_name_access_chain,
    STATE(1539), 1,
      sym_discouraged_name,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1560), 1,
      sym_type,
    STATE(1565), 1,
      sym__type,
    STATE(2175), 1,
      sym__leading_name_access,
    STATE(977), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1527), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(1252), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(1250), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(1254), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(1270), 5,
      anon_sym_where,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
    ACTIONS(1248), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
    ACTIONS(1268), 9,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
  [1645] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(70), 1,
      sym_parenthesized_expr,
    STATE(978), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(817), 22,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
    ACTIONS(815), 30,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
  [1718] = 31,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1246), 1,
      sym_identifier,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1262), 1,
      anon_sym_AMP,
    ACTIONS(1264), 1,
      anon_sym_AMPmut,
    ACTIONS(1272), 1,
      anon_sym_COMMA,
    ACTIONS(1276), 1,
      anon_sym_PIPE,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1521), 1,
      sym_primitive_type,
    STATE(1526), 1,
      sym_number_type,
    STATE(1533), 1,
      sym_name_access_chain,
    STATE(1539), 1,
      sym_discouraged_name,
    STATE(1543), 1,
      sym_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1565), 1,
      sym__type,
    STATE(2175), 1,
      sym__leading_name_access,
    STATE(979), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1527), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(1252), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(1250), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(1254), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(1278), 5,
      anon_sym_where,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
    ACTIONS(1248), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
    ACTIONS(1274), 8,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
  [1839] = 31,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1246), 1,
      sym_identifier,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1262), 1,
      anon_sym_AMP,
    ACTIONS(1264), 1,
      anon_sym_AMPmut,
    ACTIONS(1272), 1,
      anon_sym_COMMA,
    ACTIONS(1276), 1,
      anon_sym_PIPE,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1521), 1,
      sym_primitive_type,
    STATE(1526), 1,
      sym_number_type,
    STATE(1533), 1,
      sym_name_access_chain,
    STATE(1539), 1,
      sym_discouraged_name,
    STATE(1542), 1,
      sym_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1565), 1,
      sym__type,
    STATE(2175), 1,
      sym__leading_name_access,
    STATE(980), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1527), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(1252), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(1250), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(1254), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(1270), 5,
      anon_sym_where,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
    ACTIONS(1248), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
    ACTIONS(1268), 8,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
  [1960] = 31,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1246), 1,
      sym_identifier,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1262), 1,
      anon_sym_AMP,
    ACTIONS(1264), 1,
      anon_sym_AMPmut,
    ACTIONS(1272), 1,
      anon_sym_COMMA,
    ACTIONS(1276), 1,
      anon_sym_PIPE,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1521), 1,
      sym_primitive_type,
    STATE(1526), 1,
      sym_number_type,
    STATE(1533), 1,
      sym_name_access_chain,
    STATE(1539), 1,
      sym_discouraged_name,
    STATE(1544), 1,
      sym_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1565), 1,
      sym__type,
    STATE(2175), 1,
      sym__leading_name_access,
    STATE(981), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1527), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(1252), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(1250), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(1254), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(1282), 5,
      anon_sym_where,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
    ACTIONS(1248), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
    ACTIONS(1280), 8,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
  [2081] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1288), 1,
      anon_sym_LPAREN,
    ACTIONS(1291), 1,
      anon_sym_LBRACE,
    STATE(885), 1,
      sym_call_args,
    STATE(982), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1286), 20,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
    ACTIONS(1284), 30,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [2155] = 30,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1560), 1,
      sym_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(983), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(1270), 5,
      anon_sym_where,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
    ACTIONS(1268), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_BANG,
      anon_sym_SEMI,
  [2273] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(895), 1,
      anon_sym_COLON_COLON,
    STATE(937), 1,
      sym_number_type,
    STATE(984), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1133), 21,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
    ACTIONS(1131), 30,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [2345] = 30,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1555), 1,
      sym_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(985), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(1266), 5,
      anon_sym_where,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
    ACTIONS(1258), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_BANG,
      anon_sym_SEMI,
  [2463] = 17,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1301), 1,
      anon_sym_PIPE,
    ACTIONS(1303), 1,
      anon_sym_AMP,
    ACTIONS(1305), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1307), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1313), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1315), 1,
      anon_sym_CARET,
    ACTIONS(1321), 1,
      anon_sym_SLASH,
    ACTIONS(1299), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1311), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1317), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1319), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(986), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1309), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 9,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_LBRACK,
    ACTIONS(915), 23,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_EQ,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [2553] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1323), 1,
      anon_sym_EQ,
    STATE(987), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1049), 21,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
    ACTIONS(1047), 29,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [2621] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1325), 1,
      anon_sym_EQ,
    STATE(988), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1049), 21,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
    ACTIONS(1047), 29,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [2689] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1301), 1,
      anon_sym_PIPE,
    ACTIONS(1303), 1,
      anon_sym_AMP,
    ACTIONS(1315), 1,
      anon_sym_CARET,
    ACTIONS(1321), 1,
      anon_sym_SLASH,
    ACTIONS(1299), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1317), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1319), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(989), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 14,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
    ACTIONS(915), 27,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [2769] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1303), 1,
      anon_sym_AMP,
    ACTIONS(1315), 1,
      anon_sym_CARET,
    ACTIONS(1321), 1,
      anon_sym_SLASH,
    ACTIONS(1299), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1317), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1319), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(990), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 14,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
    ACTIONS(915), 28,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [2847] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1321), 1,
      anon_sym_SLASH,
    ACTIONS(1299), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1317), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1319), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(991), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 15,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
    ACTIONS(915), 29,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [2921] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1321), 1,
      anon_sym_SLASH,
    ACTIONS(1299), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    STATE(992), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 19,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_LBRACK,
    ACTIONS(915), 29,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [2991] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1331), 1,
      anon_sym_else,
    STATE(993), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1329), 21,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
    ACTIONS(1327), 29,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
      anon_sym_spec,
  [3059] = 13,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1301), 1,
      anon_sym_PIPE,
    ACTIONS(1303), 1,
      anon_sym_AMP,
    ACTIONS(1313), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1315), 1,
      anon_sym_CARET,
    ACTIONS(1321), 1,
      anon_sym_SLASH,
    ACTIONS(1299), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1317), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1319), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(994), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 13,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LBRACK,
    ACTIONS(915), 27,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [3141] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1303), 1,
      anon_sym_AMP,
    ACTIONS(1321), 1,
      anon_sym_SLASH,
    ACTIONS(1299), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1317), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1319), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(995), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 15,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
    ACTIONS(915), 28,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [3217] = 16,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1301), 1,
      anon_sym_PIPE,
    ACTIONS(1303), 1,
      anon_sym_AMP,
    ACTIONS(1307), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1313), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1315), 1,
      anon_sym_CARET,
    ACTIONS(1321), 1,
      anon_sym_SLASH,
    ACTIONS(1299), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1311), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1317), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1319), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(996), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1309), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 10,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_LBRACK,
    ACTIONS(915), 23,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_EQ,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [3305] = 18,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1301), 1,
      anon_sym_PIPE,
    ACTIONS(1303), 1,
      anon_sym_AMP,
    ACTIONS(1305), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1307), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1313), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1315), 1,
      anon_sym_CARET,
    ACTIONS(1321), 1,
      anon_sym_SLASH,
    ACTIONS(1299), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1311), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1317), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1319), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(1333), 2,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
    STATE(997), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1309), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(1041), 7,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
    ACTIONS(1039), 23,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_EQ,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [3397] = 15,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1301), 1,
      anon_sym_PIPE,
    ACTIONS(1303), 1,
      anon_sym_AMP,
    ACTIONS(1313), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1315), 1,
      anon_sym_CARET,
    ACTIONS(1321), 1,
      anon_sym_SLASH,
    ACTIONS(1299), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1311), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1317), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1319), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(998), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1309), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 11,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LBRACK,
    ACTIONS(915), 23,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_EQ,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [3483] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1321), 1,
      anon_sym_SLASH,
    ACTIONS(1299), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1319), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(999), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 17,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LBRACK,
    ACTIONS(915), 29,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [3555] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1337), 1,
      anon_sym_SLASH,
    ACTIONS(1335), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    STATE(1000), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 19,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_LBRACK,
    ACTIONS(915), 28,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
  [3624] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1339), 1,
      anon_sym_EQ,
    STATE(1001), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1049), 21,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
    ACTIONS(1047), 28,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
      anon_sym_else,
  [3691] = 18,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1337), 1,
      anon_sym_SLASH,
    ACTIONS(1341), 1,
      anon_sym_PIPE,
    ACTIONS(1343), 1,
      anon_sym_AMP,
    ACTIONS(1347), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1349), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1355), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1357), 1,
      anon_sym_CARET,
    ACTIONS(1335), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1345), 2,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
    ACTIONS(1353), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1359), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1361), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1002), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1351), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(1041), 7,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
    ACTIONS(1039), 22,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_EQ,
      anon_sym_DOT,
      anon_sym_else,
  [3782] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1337), 1,
      anon_sym_SLASH,
    ACTIONS(1343), 1,
      anon_sym_AMP,
    ACTIONS(1357), 1,
      anon_sym_CARET,
    ACTIONS(1335), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1359), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1361), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1003), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 14,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
    ACTIONS(915), 27,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
  [3859] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1365), 1,
      anon_sym_AMP,
    ACTIONS(1367), 1,
      anon_sym_CARET,
    ACTIONS(1373), 1,
      anon_sym_SLASH,
    ACTIONS(1363), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1369), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1371), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1004), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 14,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
    ACTIONS(915), 27,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [3936] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1373), 1,
      anon_sym_SLASH,
    ACTIONS(1363), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1369), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1371), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1005), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 15,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
    ACTIONS(915), 28,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [4009] = 17,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1365), 1,
      anon_sym_AMP,
    ACTIONS(1367), 1,
      anon_sym_CARET,
    ACTIONS(1373), 1,
      anon_sym_SLASH,
    ACTIONS(1375), 1,
      anon_sym_PIPE,
    ACTIONS(1377), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1379), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1385), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1363), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1369), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1371), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(1383), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1006), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1381), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 9,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_LBRACK,
    ACTIONS(915), 22,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [4098] = 16,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1365), 1,
      anon_sym_AMP,
    ACTIONS(1367), 1,
      anon_sym_CARET,
    ACTIONS(1373), 1,
      anon_sym_SLASH,
    ACTIONS(1375), 1,
      anon_sym_PIPE,
    ACTIONS(1379), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1385), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1363), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1369), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1371), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(1383), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1007), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1381), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 10,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_LBRACK,
    ACTIONS(915), 22,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [4185] = 15,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1365), 1,
      anon_sym_AMP,
    ACTIONS(1367), 1,
      anon_sym_CARET,
    ACTIONS(1373), 1,
      anon_sym_SLASH,
    ACTIONS(1375), 1,
      anon_sym_PIPE,
    ACTIONS(1385), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1363), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1369), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1371), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(1383), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1008), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1381), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 11,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LBRACK,
    ACTIONS(915), 22,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [4270] = 13,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1365), 1,
      anon_sym_AMP,
    ACTIONS(1367), 1,
      anon_sym_CARET,
    ACTIONS(1373), 1,
      anon_sym_SLASH,
    ACTIONS(1375), 1,
      anon_sym_PIPE,
    ACTIONS(1385), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1363), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1369), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1371), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1009), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 13,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LBRACK,
    ACTIONS(915), 26,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [4351] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1365), 1,
      anon_sym_AMP,
    ACTIONS(1367), 1,
      anon_sym_CARET,
    ACTIONS(1373), 1,
      anon_sym_SLASH,
    ACTIONS(1375), 1,
      anon_sym_PIPE,
    ACTIONS(1363), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1369), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1371), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1010), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 14,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
    ACTIONS(915), 26,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [4430] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1365), 1,
      anon_sym_AMP,
    ACTIONS(1373), 1,
      anon_sym_SLASH,
    ACTIONS(1363), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1369), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1371), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1011), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 15,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
    ACTIONS(915), 27,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [4505] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1373), 1,
      anon_sym_SLASH,
    ACTIONS(1363), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1371), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1012), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 17,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LBRACK,
    ACTIONS(915), 28,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [4576] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1373), 1,
      anon_sym_SLASH,
    ACTIONS(1363), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    STATE(1013), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 19,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_LBRACK,
    ACTIONS(915), 28,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [4645] = 18,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1365), 1,
      anon_sym_AMP,
    ACTIONS(1367), 1,
      anon_sym_CARET,
    ACTIONS(1373), 1,
      anon_sym_SLASH,
    ACTIONS(1375), 1,
      anon_sym_PIPE,
    ACTIONS(1377), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1379), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1385), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1363), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1369), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1371), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(1383), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1387), 2,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
    STATE(1014), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1381), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(1041), 7,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
    ACTIONS(1039), 22,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_spec,
  [4736] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1337), 1,
      anon_sym_SLASH,
    ACTIONS(1335), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1359), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1361), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1015), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 15,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
    ACTIONS(915), 28,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
  [4809] = 17,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1337), 1,
      anon_sym_SLASH,
    ACTIONS(1341), 1,
      anon_sym_PIPE,
    ACTIONS(1343), 1,
      anon_sym_AMP,
    ACTIONS(1347), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1349), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1355), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1357), 1,
      anon_sym_CARET,
    ACTIONS(1335), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1353), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1359), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1361), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1016), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1351), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 9,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_LBRACK,
    ACTIONS(915), 22,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_EQ,
      anon_sym_DOT,
      anon_sym_else,
  [4898] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1389), 1,
      anon_sym_else,
    STATE(1017), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1329), 21,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
    ACTIONS(1327), 28,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
      anon_sym_spec,
  [4965] = 16,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1337), 1,
      anon_sym_SLASH,
    ACTIONS(1341), 1,
      anon_sym_PIPE,
    ACTIONS(1343), 1,
      anon_sym_AMP,
    ACTIONS(1349), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1355), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1357), 1,
      anon_sym_CARET,
    ACTIONS(1335), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1353), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1359), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1361), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1018), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1351), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 10,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_LBRACK,
    ACTIONS(915), 22,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_EQ,
      anon_sym_DOT,
      anon_sym_else,
  [5052] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1391), 1,
      anon_sym_else,
    STATE(1019), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1329), 21,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
    ACTIONS(1327), 28,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
  [5119] = 15,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1337), 1,
      anon_sym_SLASH,
    ACTIONS(1341), 1,
      anon_sym_PIPE,
    ACTIONS(1343), 1,
      anon_sym_AMP,
    ACTIONS(1355), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1357), 1,
      anon_sym_CARET,
    ACTIONS(1335), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1353), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1359), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1361), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1020), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1351), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 11,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LBRACK,
    ACTIONS(915), 22,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_EQ,
      anon_sym_DOT,
      anon_sym_else,
  [5204] = 13,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1337), 1,
      anon_sym_SLASH,
    ACTIONS(1341), 1,
      anon_sym_PIPE,
    ACTIONS(1343), 1,
      anon_sym_AMP,
    ACTIONS(1355), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1357), 1,
      anon_sym_CARET,
    ACTIONS(1335), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1359), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1361), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1021), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 13,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LBRACK,
    ACTIONS(915), 26,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
  [5285] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1337), 1,
      anon_sym_SLASH,
    ACTIONS(1341), 1,
      anon_sym_PIPE,
    ACTIONS(1343), 1,
      anon_sym_AMP,
    ACTIONS(1357), 1,
      anon_sym_CARET,
    ACTIONS(1335), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1359), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1361), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1022), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 14,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
    ACTIONS(915), 26,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
  [5364] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1393), 1,
      anon_sym_EQ,
    STATE(1023), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1049), 21,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
    ACTIONS(1047), 28,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
      anon_sym_else,
  [5431] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1337), 1,
      anon_sym_SLASH,
    ACTIONS(1343), 1,
      anon_sym_AMP,
    ACTIONS(1335), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1359), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1361), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1024), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 15,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
    ACTIONS(915), 27,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
  [5506] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1337), 1,
      anon_sym_SLASH,
    ACTIONS(1335), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1361), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1025), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 17,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LBRACK,
    ACTIONS(915), 28,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
  [5577] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1026), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1397), 16,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_AT,
      anon_sym_SEMI,
    ACTIONS(1395), 33,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
      anon_sym_update,
      anon_sym_except,
  [5641] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1401), 1,
      anon_sym_AMP,
    ACTIONS(1403), 1,
      anon_sym_CARET,
    ACTIONS(1409), 1,
      anon_sym_SLASH,
    ACTIONS(1399), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1405), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1407), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1027), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 14,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
    ACTIONS(915), 26,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
  [5717] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1409), 1,
      anon_sym_SLASH,
    ACTIONS(1399), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1405), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1407), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1028), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 15,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
    ACTIONS(915), 27,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
  [5789] = 17,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1401), 1,
      anon_sym_AMP,
    ACTIONS(1403), 1,
      anon_sym_CARET,
    ACTIONS(1409), 1,
      anon_sym_SLASH,
    ACTIONS(1411), 1,
      anon_sym_PIPE,
    ACTIONS(1413), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1415), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1421), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1399), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1405), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1407), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(1419), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1029), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1417), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 9,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_LBRACK,
    ACTIONS(915), 21,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_DOT,
      anon_sym_else,
  [5877] = 15,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1401), 1,
      anon_sym_AMP,
    ACTIONS(1403), 1,
      anon_sym_CARET,
    ACTIONS(1409), 1,
      anon_sym_SLASH,
    ACTIONS(1411), 1,
      anon_sym_PIPE,
    ACTIONS(1421), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1399), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1405), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1407), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(1419), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1030), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1417), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 11,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LBRACK,
    ACTIONS(915), 21,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_DOT,
      anon_sym_else,
  [5961] = 16,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1401), 1,
      anon_sym_AMP,
    ACTIONS(1403), 1,
      anon_sym_CARET,
    ACTIONS(1409), 1,
      anon_sym_SLASH,
    ACTIONS(1411), 1,
      anon_sym_PIPE,
    ACTIONS(1415), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1421), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1399), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1405), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1407), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(1419), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1031), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1417), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 10,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_LBRACK,
    ACTIONS(915), 21,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_DOT,
      anon_sym_else,
  [6047] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1032), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(929), 6,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
    ACTIONS(999), 9,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1001), 15,
      anon_sym_STAR,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
    ACTIONS(927), 19,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
  [6115] = 18,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1401), 1,
      anon_sym_AMP,
    ACTIONS(1403), 1,
      anon_sym_CARET,
    ACTIONS(1409), 1,
      anon_sym_SLASH,
    ACTIONS(1411), 1,
      anon_sym_PIPE,
    ACTIONS(1413), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1415), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1421), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1399), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1405), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1407), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(1419), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1423), 2,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
    STATE(1033), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1417), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(1041), 7,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
    ACTIONS(1039), 21,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_DOT,
      anon_sym_else,
  [6205] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1034), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1427), 16,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_AT,
      anon_sym_SEMI,
    ACTIONS(1425), 33,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
      anon_sym_update,
      anon_sym_except,
  [6269] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1035), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1431), 16,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_AT,
      anon_sym_SEMI,
    ACTIONS(1429), 33,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
      anon_sym_update,
      anon_sym_except,
  [6333] = 13,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1401), 1,
      anon_sym_AMP,
    ACTIONS(1403), 1,
      anon_sym_CARET,
    ACTIONS(1409), 1,
      anon_sym_SLASH,
    ACTIONS(1411), 1,
      anon_sym_PIPE,
    ACTIONS(1421), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1399), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1405), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1407), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1036), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 13,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LBRACK,
    ACTIONS(915), 25,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
  [6413] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1401), 1,
      anon_sym_AMP,
    ACTIONS(1403), 1,
      anon_sym_CARET,
    ACTIONS(1409), 1,
      anon_sym_SLASH,
    ACTIONS(1411), 1,
      anon_sym_PIPE,
    ACTIONS(1399), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1405), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1407), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1037), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 14,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
    ACTIONS(915), 25,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
  [6491] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1433), 1,
      anon_sym_else,
    STATE(1038), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1329), 21,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
    ACTIONS(1327), 27,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
  [6557] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1409), 1,
      anon_sym_SLASH,
    ACTIONS(1399), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    STATE(1039), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 19,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_LBRACK,
    ACTIONS(915), 27,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
  [6625] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1401), 1,
      anon_sym_AMP,
    ACTIONS(1409), 1,
      anon_sym_SLASH,
    ACTIONS(1399), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1405), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1407), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1040), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 15,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
    ACTIONS(915), 26,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
  [6699] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1409), 1,
      anon_sym_SLASH,
    ACTIONS(1399), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1407), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1041), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(917), 17,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LBRACK,
    ACTIONS(915), 27,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_else,
  [6769] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1042), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1437), 16,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_AT,
      anon_sym_SEMI,
    ACTIONS(1435), 33,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
      anon_sym_update,
      anon_sym_except,
  [6833] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1443), 1,
      anon_sym_use,
    STATE(1045), 1,
      sym_use_decl,
    STATE(1043), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_spec_block_repeat1,
    ACTIONS(1441), 13,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_BANG,
      anon_sym_AT,
    ACTIONS(1439), 32,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
      anon_sym_let,
  [6900] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(895), 1,
      anon_sym_COLON_COLON,
    STATE(937), 1,
      sym_number_type,
    STATE(1044), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1446), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
    ACTIONS(1131), 10,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1133), 29,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_as,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [6968] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1045), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1450), 13,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_BANG,
      anon_sym_AT,
    ACTIONS(1448), 33,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
      anon_sym_let,
      anon_sym_use,
  [7029] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1046), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1454), 13,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_BANG,
      anon_sym_AT,
    ACTIONS(1452), 32,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
      anon_sym_let,
  [7089] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1047), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1458), 13,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_BANG,
      anon_sym_AT,
    ACTIONS(1456), 32,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
      anon_sym_let,
  [7149] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1460), 1,
      sym_identifier,
    ACTIONS(1462), 1,
      anon_sym_min,
    STATE(2393), 1,
      sym_quantifier_bind,
    STATE(1048), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 17,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_spec,
      anon_sym_with,
      anon_sym_to,
    ACTIONS(825), 24,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [7214] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1460), 1,
      sym_identifier,
    ACTIONS(1464), 1,
      anon_sym_min,
    STATE(2374), 1,
      sym_quantifier_bind,
    STATE(1049), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 17,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_spec,
      anon_sym_with,
      anon_sym_to,
    ACTIONS(825), 24,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [7279] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(869), 1,
      anon_sym_LT2,
    ACTIONS(887), 1,
      anon_sym_LPAREN,
    ACTIONS(1204), 1,
      anon_sym_LBRACE,
    ACTIONS(1206), 1,
      anon_sym_BANG,
    STATE(905), 1,
      sym_call_args,
    STATE(1081), 1,
      sym_type_args,
    STATE(1050), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1197), 9,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1199), 29,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_as,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [7350] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1051), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1468), 12,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_AT,
    ACTIONS(1466), 31,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
  [7408] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1460), 1,
      sym_identifier,
    ACTIONS(1470), 1,
      anon_sym_min,
    STATE(2312), 1,
      sym_quantifier_bind,
    STATE(1052), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 16,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_with,
      anon_sym_to,
    ACTIONS(825), 24,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [7472] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1053), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1474), 12,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_AT,
    ACTIONS(1472), 31,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
  [7530] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1054), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1478), 12,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_AT,
    ACTIONS(1476), 31,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
  [7588] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1480), 1,
      sym_identifier,
    STATE(1719), 1,
      sym_quantifier_bind,
    STATE(1055), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 17,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_spec,
      anon_sym_with,
      anon_sym_to,
    ACTIONS(825), 24,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [7650] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1056), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1484), 12,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_AT,
    ACTIONS(1482), 31,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
  [7708] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1057), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1488), 12,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_AT,
    ACTIONS(1486), 31,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
  [7766] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1058), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1492), 12,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_AT,
    ACTIONS(1490), 31,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
  [7824] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1480), 1,
      sym_identifier,
    STATE(1740), 1,
      sym_quantifier_bind,
    STATE(1059), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 17,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_spec,
      anon_sym_with,
      anon_sym_to,
    ACTIONS(825), 24,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [7886] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1060), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1496), 12,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_AT,
    ACTIONS(1494), 31,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
  [7944] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1460), 1,
      sym_identifier,
    ACTIONS(1498), 1,
      anon_sym_min,
    STATE(2173), 1,
      sym_quantifier_bind,
    STATE(1061), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 16,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_with,
      anon_sym_to,
    ACTIONS(825), 24,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [8008] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1062), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1502), 12,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_AT,
    ACTIONS(1500), 31,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
  [8066] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1063), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1506), 12,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_AT,
    ACTIONS(1504), 31,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
  [8124] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1064), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1510), 12,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_AT,
    ACTIONS(1508), 31,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
  [8182] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1065), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1514), 12,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_AT,
    ACTIONS(1512), 31,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
  [8240] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1066), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1518), 12,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_AT,
    ACTIONS(1516), 31,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
  [8298] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1067), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1522), 12,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
      anon_sym_STAR,
      anon_sym_LPAREN,
      anon_sym_PIPE,
      anon_sym_AMPmut,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_AT,
    ACTIONS(1520), 31,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
      anon_sym_true,
      anon_sym_false,
      anon_sym_AMP,
      anon_sym_move,
      anon_sym_copy,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_return,
      anon_sym_abort,
      anon_sym_spec,
  [8356] = 30,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    ACTIONS(1524), 1,
      anon_sym_COMMA,
    ACTIONS(1526), 1,
      anon_sym_RPAREN,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(1950), 1,
      sym_type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1068), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [8463] = 30,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1246), 1,
      sym_identifier,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1260), 1,
      anon_sym_PIPE,
    ACTIONS(1262), 1,
      anon_sym_AMP,
    ACTIONS(1264), 1,
      anon_sym_AMPmut,
    ACTIONS(1528), 1,
      anon_sym_COMMA,
    ACTIONS(1530), 1,
      anon_sym_GT,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1521), 1,
      sym_primitive_type,
    STATE(1526), 1,
      sym_number_type,
    STATE(1533), 1,
      sym_name_access_chain,
    STATE(1539), 1,
      sym_discouraged_name,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1565), 1,
      sym__type,
    STATE(1895), 1,
      sym_type,
    STATE(2175), 1,
      sym__leading_name_access,
    STATE(1069), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1527), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(1252), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(1250), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(1254), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(1248), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [8570] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1480), 1,
      sym_identifier,
    STATE(1734), 1,
      sym_quantifier_bind,
    STATE(1070), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 16,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_with,
      anon_sym_to,
    ACTIONS(825), 24,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [8631] = 30,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1246), 1,
      sym_identifier,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1260), 1,
      anon_sym_PIPE,
    ACTIONS(1262), 1,
      anon_sym_AMP,
    ACTIONS(1264), 1,
      anon_sym_AMPmut,
    ACTIONS(1532), 1,
      anon_sym_COMMA,
    ACTIONS(1534), 1,
      anon_sym_GT,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1521), 1,
      sym_primitive_type,
    STATE(1526), 1,
      sym_number_type,
    STATE(1533), 1,
      sym_name_access_chain,
    STATE(1539), 1,
      sym_discouraged_name,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1565), 1,
      sym__type,
    STATE(1991), 1,
      sym_type,
    STATE(2175), 1,
      sym__leading_name_access,
    STATE(1071), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1527), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(1252), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(1250), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(1254), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(1248), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [8738] = 30,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    ACTIONS(1536), 1,
      anon_sym_COMMA,
    ACTIONS(1538), 1,
      anon_sym_RPAREN,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(1920), 1,
      sym_type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1072), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [8845] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1480), 1,
      sym_identifier,
    STATE(1704), 1,
      sym_quantifier_bind,
    STATE(1073), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 16,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_with,
      anon_sym_to,
    ACTIONS(825), 24,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [8906] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(66), 1,
      sym_parenthesized_expr,
    STATE(1074), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(815), 12,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_LT2,
    ACTIONS(817), 28,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [8967] = 25,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(83), 1,
      anon_sym_AT,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1534), 1,
      sym_number,
    STATE(1879), 1,
      sym_value,
    STATE(1881), 1,
      sym__attribute_val,
    STATE(1900), 1,
      sym_name_access_chain,
    STATE(2342), 1,
      sym__leading_name_access,
    ACTIONS(53), 2,
      anon_sym_true,
      anon_sym_false,
    ACTIONS(55), 2,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
    STATE(1075), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    STATE(943), 3,
      sym_bool_literal,
      sym_typed_number,
      sym_byte_string,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [9063] = 29,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1246), 1,
      sym_identifier,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1260), 1,
      anon_sym_PIPE,
    ACTIONS(1262), 1,
      anon_sym_AMP,
    ACTIONS(1264), 1,
      anon_sym_AMPmut,
    ACTIONS(1540), 1,
      anon_sym_GT,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1521), 1,
      sym_primitive_type,
    STATE(1526), 1,
      sym_number_type,
    STATE(1533), 1,
      sym_name_access_chain,
    STATE(1539), 1,
      sym_discouraged_name,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1565), 1,
      sym__type,
    STATE(1809), 1,
      sym_type,
    STATE(2175), 1,
      sym__leading_name_access,
    STATE(1076), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1527), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(1252), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(1250), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(1254), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(1248), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [9167] = 29,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    ACTIONS(1542), 1,
      anon_sym_RPAREN,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(1809), 1,
      sym_type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1077), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [9271] = 29,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    ACTIONS(1544), 1,
      anon_sym_RPAREN,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(1809), 1,
      sym_type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1078), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [9375] = 29,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1246), 1,
      sym_identifier,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1260), 1,
      anon_sym_PIPE,
    ACTIONS(1262), 1,
      anon_sym_AMP,
    ACTIONS(1264), 1,
      anon_sym_AMPmut,
    ACTIONS(1546), 1,
      anon_sym_GT,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1521), 1,
      sym_primitive_type,
    STATE(1526), 1,
      sym_number_type,
    STATE(1533), 1,
      sym_name_access_chain,
    STATE(1539), 1,
      sym_discouraged_name,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1565), 1,
      sym__type,
    STATE(1809), 1,
      sym_type,
    STATE(2175), 1,
      sym__leading_name_access,
    STATE(1079), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1527), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(1252), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(1250), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(1254), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(1248), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [9479] = 29,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1246), 1,
      sym_identifier,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1262), 1,
      anon_sym_AMP,
    ACTIONS(1264), 1,
      anon_sym_AMPmut,
    ACTIONS(1272), 1,
      anon_sym_COMMA,
    ACTIONS(1276), 1,
      anon_sym_PIPE,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1521), 1,
      sym_primitive_type,
    STATE(1526), 1,
      sym_number_type,
    STATE(1533), 1,
      sym_name_access_chain,
    STATE(1539), 1,
      sym_discouraged_name,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1565), 1,
      sym__type,
    STATE(1999), 1,
      sym_type,
    STATE(2175), 1,
      sym__leading_name_access,
    STATE(1080), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1527), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(1252), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(1250), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(1254), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(1248), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [9583] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(887), 1,
      anon_sym_LPAREN,
    ACTIONS(1291), 1,
      anon_sym_LBRACE,
    STATE(885), 1,
      sym_call_args,
    STATE(1081), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1284), 9,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1286), 29,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_as,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [9645] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(124), 1,
      sym_parenthesized_expr,
    STATE(1082), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(815), 12,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_LT2,
    ACTIONS(817), 27,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [9705] = 29,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1246), 1,
      sym_identifier,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1260), 1,
      anon_sym_PIPE,
    ACTIONS(1262), 1,
      anon_sym_AMP,
    ACTIONS(1264), 1,
      anon_sym_AMPmut,
    ACTIONS(1548), 1,
      anon_sym_GT,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1521), 1,
      sym_primitive_type,
    STATE(1526), 1,
      sym_number_type,
    STATE(1533), 1,
      sym_name_access_chain,
    STATE(1539), 1,
      sym_discouraged_name,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1565), 1,
      sym__type,
    STATE(1809), 1,
      sym_type,
    STATE(2175), 1,
      sym__leading_name_access,
    STATE(1083), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1527), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(1252), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(1250), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(1254), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(1248), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [9809] = 29,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1246), 1,
      sym_identifier,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1262), 1,
      anon_sym_AMP,
    ACTIONS(1264), 1,
      anon_sym_AMPmut,
    ACTIONS(1276), 1,
      anon_sym_PIPE,
    ACTIONS(1550), 1,
      anon_sym_COMMA,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1521), 1,
      sym_primitive_type,
    STATE(1526), 1,
      sym_number_type,
    STATE(1533), 1,
      sym_name_access_chain,
    STATE(1539), 1,
      sym_discouraged_name,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1565), 1,
      sym__type,
    STATE(1864), 1,
      sym_type,
    STATE(2175), 1,
      sym__leading_name_access,
    STATE(1084), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1527), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(1252), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(1250), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(1254), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(1248), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [9913] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(117), 1,
      sym_parenthesized_expr,
    STATE(1085), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(815), 11,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_LT2,
    ACTIONS(817), 28,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [9973] = 29,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1246), 1,
      sym_identifier,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1260), 1,
      anon_sym_PIPE,
    ACTIONS(1262), 1,
      anon_sym_AMP,
    ACTIONS(1264), 1,
      anon_sym_AMPmut,
    ACTIONS(1552), 1,
      anon_sym_GT,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1521), 1,
      sym_primitive_type,
    STATE(1526), 1,
      sym_number_type,
    STATE(1533), 1,
      sym_name_access_chain,
    STATE(1539), 1,
      sym_discouraged_name,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1565), 1,
      sym__type,
    STATE(1809), 1,
      sym_type,
    STATE(2175), 1,
      sym__leading_name_access,
    STATE(1086), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1527), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(1252), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(1250), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(1254), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(1248), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [10077] = 29,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    ACTIONS(1554), 1,
      anon_sym_RPAREN,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(1809), 1,
      sym_type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1087), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [10181] = 29,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    ACTIONS(1556), 1,
      anon_sym_RPAREN,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(1809), 1,
      sym_type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1088), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [10285] = 28,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(2276), 1,
      sym_type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1089), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [10386] = 28,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(2443), 1,
      sym_type,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1090), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [10487] = 28,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(1809), 1,
      sym_type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1091), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [10588] = 26,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1558), 1,
      sym_identifier,
    ACTIONS(1560), 1,
      anon_sym_RBRACE,
    ACTIONS(1562), 1,
      anon_sym_invariant,
    ACTIONS(1564), 1,
      anon_sym_update,
    ACTIONS(1568), 1,
      anon_sym_aborts_if,
    ACTIONS(1572), 1,
      anon_sym_emits,
    ACTIONS(1576), 1,
      anon_sym_fun,
    ACTIONS(1578), 1,
      anon_sym_native,
    ACTIONS(1580), 1,
      anon_sym_include,
    ACTIONS(1582), 1,
      anon_sym_apply,
    ACTIONS(1584), 1,
      anon_sym_pragma,
    ACTIONS(1586), 1,
      anon_sym_let,
    ACTIONS(1588), 1,
      anon_sym_axiom,
    ACTIONS(1590), 1,
      anon_sym_use,
    STATE(1125), 1,
      aux_sym_spec_block_repeat2,
    STATE(1434), 1,
      aux_sym_spec_block_repeat1,
    STATE(1440), 1,
      sym_use_decl,
    STATE(1452), 1,
      sym__spec_block_member,
    ACTIONS(1570), 2,
      anon_sym_aborts_with,
      anon_sym_modifies,
    ACTIONS(1574), 2,
      anon_sym_global,
      anon_sym_local,
    STATE(1092), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1566), 4,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
    STATE(2453), 4,
      sym_asserts,
      sym_aborts_if,
      sym_aborts_with_or_modifies,
      sym_emits,
    STATE(1453), 10,
      sym_spec_invariant,
      sym_spec_condition,
      sym_spec_variable,
      sym_spec_func,
      sym_spec_include,
      sym_spec_apply,
      sym_spec_pragma,
      sym_spec_let,
      sym_spec_update,
      sym_spec_axiom,
  [10685] = 28,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(2112), 1,
      sym_type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1093), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [10786] = 28,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(2150), 1,
      sym_type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1094), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [10887] = 28,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(2024), 1,
      sym_type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1095), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [10988] = 28,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(2102), 1,
      sym_type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1096), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [11089] = 28,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(2216), 1,
      sym_type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1097), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [11190] = 26,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1558), 1,
      sym_identifier,
    ACTIONS(1562), 1,
      anon_sym_invariant,
    ACTIONS(1564), 1,
      anon_sym_update,
    ACTIONS(1568), 1,
      anon_sym_aborts_if,
    ACTIONS(1572), 1,
      anon_sym_emits,
    ACTIONS(1576), 1,
      anon_sym_fun,
    ACTIONS(1578), 1,
      anon_sym_native,
    ACTIONS(1580), 1,
      anon_sym_include,
    ACTIONS(1582), 1,
      anon_sym_apply,
    ACTIONS(1584), 1,
      anon_sym_pragma,
    ACTIONS(1586), 1,
      anon_sym_let,
    ACTIONS(1588), 1,
      anon_sym_axiom,
    ACTIONS(1590), 1,
      anon_sym_use,
    ACTIONS(1592), 1,
      anon_sym_RBRACE,
    STATE(1130), 1,
      aux_sym_spec_block_repeat2,
    STATE(1434), 1,
      aux_sym_spec_block_repeat1,
    STATE(1440), 1,
      sym_use_decl,
    STATE(1452), 1,
      sym__spec_block_member,
    ACTIONS(1570), 2,
      anon_sym_aborts_with,
      anon_sym_modifies,
    ACTIONS(1574), 2,
      anon_sym_global,
      anon_sym_local,
    STATE(1098), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1566), 4,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
    STATE(2453), 4,
      sym_asserts,
      sym_aborts_if,
      sym_aborts_with_or_modifies,
      sym_emits,
    STATE(1453), 10,
      sym_spec_invariant,
      sym_spec_condition,
      sym_spec_variable,
      sym_spec_func,
      sym_spec_include,
      sym_spec_apply,
      sym_spec_pragma,
      sym_spec_let,
      sym_spec_update,
      sym_spec_axiom,
  [11287] = 28,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(2013), 1,
      sym_type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1099), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [11388] = 28,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(1642), 1,
      sym_type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1100), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [11489] = 28,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(2042), 1,
      sym_type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1101), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [11590] = 28,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(2283), 1,
      sym_type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1102), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [11691] = 28,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(1816), 1,
      sym_type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1103), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [11792] = 23,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(83), 1,
      anon_sym_AT,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1534), 1,
      sym_number,
    STATE(2342), 1,
      sym__leading_name_access,
    ACTIONS(53), 2,
      anon_sym_true,
      anon_sym_false,
    ACTIONS(55), 2,
      aux_sym_byte_string_token1,
      aux_sym_byte_string_token2,
    STATE(1104), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1974), 2,
      sym_name_access_chain,
      sym_value,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    STATE(943), 3,
      sym_bool_literal,
      sym_typed_number,
      sym_byte_string,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [11883] = 28,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1246), 1,
      sym_identifier,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1262), 1,
      anon_sym_AMP,
    ACTIONS(1264), 1,
      anon_sym_AMPmut,
    ACTIONS(1594), 1,
      anon_sym_PIPE,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1521), 1,
      sym_primitive_type,
    STATE(1526), 1,
      sym_number_type,
    STATE(1533), 1,
      sym_name_access_chain,
    STATE(1539), 1,
      sym_discouraged_name,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1565), 1,
      sym__type,
    STATE(1809), 1,
      sym_type,
    STATE(2175), 1,
      sym__leading_name_access,
    STATE(1105), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1527), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(1252), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(1250), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(1254), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(1248), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [11984] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(76), 1,
      sym_parenthesized_expr,
    STATE(1106), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(815), 11,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_LT2,
    ACTIONS(817), 27,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [12043] = 26,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1558), 1,
      sym_identifier,
    ACTIONS(1562), 1,
      anon_sym_invariant,
    ACTIONS(1564), 1,
      anon_sym_update,
    ACTIONS(1568), 1,
      anon_sym_aborts_if,
    ACTIONS(1572), 1,
      anon_sym_emits,
    ACTIONS(1576), 1,
      anon_sym_fun,
    ACTIONS(1578), 1,
      anon_sym_native,
    ACTIONS(1580), 1,
      anon_sym_include,
    ACTIONS(1582), 1,
      anon_sym_apply,
    ACTIONS(1584), 1,
      anon_sym_pragma,
    ACTIONS(1586), 1,
      anon_sym_let,
    ACTIONS(1588), 1,
      anon_sym_axiom,
    ACTIONS(1590), 1,
      anon_sym_use,
    ACTIONS(1596), 1,
      anon_sym_RBRACE,
    STATE(1098), 1,
      aux_sym_spec_block_repeat1,
    STATE(1129), 1,
      aux_sym_spec_block_repeat2,
    STATE(1440), 1,
      sym_use_decl,
    STATE(1452), 1,
      sym__spec_block_member,
    ACTIONS(1570), 2,
      anon_sym_aborts_with,
      anon_sym_modifies,
    ACTIONS(1574), 2,
      anon_sym_global,
      anon_sym_local,
    STATE(1107), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1566), 4,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
    STATE(2453), 4,
      sym_asserts,
      sym_aborts_if,
      sym_aborts_with_or_modifies,
      sym_emits,
    STATE(1453), 10,
      sym_spec_invariant,
      sym_spec_condition,
      sym_spec_variable,
      sym_spec_func,
      sym_spec_include,
      sym_spec_apply,
      sym_spec_pragma,
      sym_spec_let,
      sym_spec_update,
      sym_spec_axiom,
  [12140] = 28,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(2049), 1,
      sym_type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1108), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [12241] = 28,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(1622), 1,
      sym_type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1109), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [12342] = 26,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1558), 1,
      sym_identifier,
    ACTIONS(1562), 1,
      anon_sym_invariant,
    ACTIONS(1564), 1,
      anon_sym_update,
    ACTIONS(1568), 1,
      anon_sym_aborts_if,
    ACTIONS(1572), 1,
      anon_sym_emits,
    ACTIONS(1576), 1,
      anon_sym_fun,
    ACTIONS(1578), 1,
      anon_sym_native,
    ACTIONS(1580), 1,
      anon_sym_include,
    ACTIONS(1582), 1,
      anon_sym_apply,
    ACTIONS(1584), 1,
      anon_sym_pragma,
    ACTIONS(1586), 1,
      anon_sym_let,
    ACTIONS(1588), 1,
      anon_sym_axiom,
    ACTIONS(1590), 1,
      anon_sym_use,
    ACTIONS(1598), 1,
      anon_sym_RBRACE,
    STATE(1092), 1,
      aux_sym_spec_block_repeat1,
    STATE(1128), 1,
      aux_sym_spec_block_repeat2,
    STATE(1440), 1,
      sym_use_decl,
    STATE(1452), 1,
      sym__spec_block_member,
    ACTIONS(1570), 2,
      anon_sym_aborts_with,
      anon_sym_modifies,
    ACTIONS(1574), 2,
      anon_sym_global,
      anon_sym_local,
    STATE(1110), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1566), 4,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
    STATE(2453), 4,
      sym_asserts,
      sym_aborts_if,
      sym_aborts_with_or_modifies,
      sym_emits,
    STATE(1453), 10,
      sym_spec_invariant,
      sym_spec_condition,
      sym_spec_variable,
      sym_spec_func,
      sym_spec_include,
      sym_spec_apply,
      sym_spec_pragma,
      sym_spec_let,
      sym_spec_update,
      sym_spec_axiom,
  [12439] = 28,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(2282), 1,
      sym_type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1111), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [12540] = 28,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(2009), 1,
      sym_type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1112), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [12641] = 28,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(1639), 1,
      sym_type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1113), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [12742] = 28,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1246), 1,
      sym_identifier,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1260), 1,
      anon_sym_PIPE,
    ACTIONS(1262), 1,
      anon_sym_AMP,
    ACTIONS(1264), 1,
      anon_sym_AMPmut,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1521), 1,
      sym_primitive_type,
    STATE(1526), 1,
      sym_number_type,
    STATE(1533), 1,
      sym_name_access_chain,
    STATE(1539), 1,
      sym_discouraged_name,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1565), 1,
      sym__type,
    STATE(1809), 1,
      sym_type,
    STATE(2175), 1,
      sym__leading_name_access,
    STATE(1114), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1527), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(1252), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(1250), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(1254), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(1248), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [12843] = 28,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(1632), 1,
      sym_type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1115), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [12944] = 28,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    ACTIONS(1295), 1,
      anon_sym_AMP,
    ACTIONS(1297), 1,
      anon_sym_AMPmut,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1565), 1,
      sym__type,
    STATE(2126), 1,
      sym_type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1116), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [13045] = 28,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1246), 1,
      sym_identifier,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1262), 1,
      anon_sym_AMP,
    ACTIONS(1264), 1,
      anon_sym_AMPmut,
    ACTIONS(1600), 1,
      anon_sym_PIPE,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1521), 1,
      sym_primitive_type,
    STATE(1526), 1,
      sym_number_type,
    STATE(1533), 1,
      sym_name_access_chain,
    STATE(1539), 1,
      sym_discouraged_name,
    STATE(1556), 1,
      sym__ref_type,
    STATE(1565), 1,
      sym__type,
    STATE(1809), 1,
      sym_type,
    STATE(2175), 1,
      sym__leading_name_access,
    STATE(1117), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1527), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(1252), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(1250), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(1254), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(1248), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [13146] = 23,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1602), 1,
      sym_identifier,
    ACTIONS(1604), 1,
      anon_sym_STAR,
    STATE(859), 1,
      sym_number_type,
    STATE(880), 1,
      sym_number,
    STATE(1604), 1,
      sym_discouraged_name,
    STATE(1610), 1,
      sym_name_access_chain_wildcard,
    STATE(1736), 1,
      sym_access_specifier,
    STATE(2334), 1,
      sym_numerical_addr,
    STATE(2340), 1,
      sym__leading_name_access_wildcard,
    STATE(1118), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(1606), 3,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_SEMI,
    ACTIONS(1608), 3,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [13236] = 23,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1602), 1,
      sym_identifier,
    ACTIONS(1604), 1,
      anon_sym_STAR,
    STATE(859), 1,
      sym_number_type,
    STATE(880), 1,
      sym_number,
    STATE(1604), 1,
      sym_discouraged_name,
    STATE(1610), 1,
      sym_name_access_chain_wildcard,
    STATE(1736), 1,
      sym_access_specifier,
    STATE(2334), 1,
      sym_numerical_addr,
    STATE(2340), 1,
      sym__leading_name_access_wildcard,
    STATE(1119), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(1610), 3,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_SEMI,
    ACTIONS(1612), 3,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [13326] = 26,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1614), 1,
      sym_identifier,
    ACTIONS(1616), 1,
      anon_sym_LPAREN,
    ACTIONS(1618), 1,
      anon_sym_RBRACE,
    STATE(859), 1,
      sym_number_type,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1124), 1,
      aux_sym_match_expr_repeat1,
    STATE(1429), 1,
      sym_match_arm,
    STATE(1579), 1,
      sym_name_access_chain,
    STATE(1595), 1,
      sym_discouraged_name,
    STATE(1641), 1,
      sym_var_name,
    STATE(1791), 1,
      sym__bind,
    STATE(2007), 1,
      sym_bind_list,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(1120), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [13421] = 25,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1620), 1,
      sym_identifier,
    ACTIONS(1635), 1,
      aux_sym_number_token1,
    ACTIONS(1638), 1,
      aux_sym_number_token2,
    ACTIONS(1641), 1,
      aux_sym_number_token4,
    ACTIONS(1644), 1,
      aux_sym_number_token6,
    ACTIONS(1647), 1,
      anon_sym_LPAREN,
    ACTIONS(1650), 1,
      anon_sym_RBRACE,
    STATE(859), 1,
      sym_number_type,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1429), 1,
      sym_match_arm,
    STATE(1579), 1,
      sym_name_access_chain,
    STATE(1595), 1,
      sym_discouraged_name,
    STATE(1641), 1,
      sym_var_name,
    STATE(1791), 1,
      sym__bind,
    STATE(2007), 1,
      sym_bind_list,
    STATE(2342), 1,
      sym__leading_name_access,
    ACTIONS(1629), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    STATE(1121), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_match_expr_repeat1,
    ACTIONS(1626), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(1632), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(1623), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [13514] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1460), 1,
      sym_identifier,
    ACTIONS(1652), 1,
      anon_sym_min,
    STATE(2409), 1,
      sym_quantifier_bind,
    STATE(1122), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 15,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_as,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_spec,
    ACTIONS(825), 20,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
  [13573] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1460), 1,
      sym_identifier,
    ACTIONS(1654), 1,
      anon_sym_min,
    STATE(2329), 1,
      sym_quantifier_bind,
    STATE(1123), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 15,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_as,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_spec,
    ACTIONS(825), 20,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
  [13632] = 26,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1614), 1,
      sym_identifier,
    ACTIONS(1616), 1,
      anon_sym_LPAREN,
    ACTIONS(1656), 1,
      anon_sym_RBRACE,
    STATE(859), 1,
      sym_number_type,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1121), 1,
      aux_sym_match_expr_repeat1,
    STATE(1429), 1,
      sym_match_arm,
    STATE(1579), 1,
      sym_name_access_chain,
    STATE(1595), 1,
      sym_discouraged_name,
    STATE(1641), 1,
      sym_var_name,
    STATE(1791), 1,
      sym__bind,
    STATE(2007), 1,
      sym_bind_list,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(1124), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [13727] = 23,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1558), 1,
      sym_identifier,
    ACTIONS(1562), 1,
      anon_sym_invariant,
    ACTIONS(1564), 1,
      anon_sym_update,
    ACTIONS(1568), 1,
      anon_sym_aborts_if,
    ACTIONS(1572), 1,
      anon_sym_emits,
    ACTIONS(1576), 1,
      anon_sym_fun,
    ACTIONS(1578), 1,
      anon_sym_native,
    ACTIONS(1580), 1,
      anon_sym_include,
    ACTIONS(1582), 1,
      anon_sym_apply,
    ACTIONS(1584), 1,
      anon_sym_pragma,
    ACTIONS(1586), 1,
      anon_sym_let,
    ACTIONS(1588), 1,
      anon_sym_axiom,
    ACTIONS(1658), 1,
      anon_sym_RBRACE,
    STATE(1132), 1,
      aux_sym_spec_block_repeat2,
    STATE(1452), 1,
      sym__spec_block_member,
    ACTIONS(1570), 2,
      anon_sym_aborts_with,
      anon_sym_modifies,
    ACTIONS(1574), 2,
      anon_sym_global,
      anon_sym_local,
    STATE(1125), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1566), 4,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
    STATE(2453), 4,
      sym_asserts,
      sym_aborts_if,
      sym_aborts_with_or_modifies,
      sym_emits,
    STATE(1453), 10,
      sym_spec_invariant,
      sym_spec_condition,
      sym_spec_variable,
      sym_spec_func,
      sym_spec_include,
      sym_spec_apply,
      sym_spec_pragma,
      sym_spec_let,
      sym_spec_update,
      sym_spec_axiom,
  [13815] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1480), 1,
      sym_identifier,
    STATE(1745), 1,
      sym_quantifier_bind,
    STATE(1126), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 15,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_as,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_spec,
    ACTIONS(825), 20,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
  [13871] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1480), 1,
      sym_identifier,
    STATE(1714), 1,
      sym_quantifier_bind,
    STATE(1127), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 15,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_as,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_spec,
    ACTIONS(825), 20,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
  [13927] = 23,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1558), 1,
      sym_identifier,
    ACTIONS(1562), 1,
      anon_sym_invariant,
    ACTIONS(1564), 1,
      anon_sym_update,
    ACTIONS(1568), 1,
      anon_sym_aborts_if,
    ACTIONS(1572), 1,
      anon_sym_emits,
    ACTIONS(1576), 1,
      anon_sym_fun,
    ACTIONS(1578), 1,
      anon_sym_native,
    ACTIONS(1580), 1,
      anon_sym_include,
    ACTIONS(1582), 1,
      anon_sym_apply,
    ACTIONS(1584), 1,
      anon_sym_pragma,
    ACTIONS(1586), 1,
      anon_sym_let,
    ACTIONS(1588), 1,
      anon_sym_axiom,
    ACTIONS(1660), 1,
      anon_sym_RBRACE,
    STATE(1132), 1,
      aux_sym_spec_block_repeat2,
    STATE(1452), 1,
      sym__spec_block_member,
    ACTIONS(1570), 2,
      anon_sym_aborts_with,
      anon_sym_modifies,
    ACTIONS(1574), 2,
      anon_sym_global,
      anon_sym_local,
    STATE(1128), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1566), 4,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
    STATE(2453), 4,
      sym_asserts,
      sym_aborts_if,
      sym_aborts_with_or_modifies,
      sym_emits,
    STATE(1453), 10,
      sym_spec_invariant,
      sym_spec_condition,
      sym_spec_variable,
      sym_spec_func,
      sym_spec_include,
      sym_spec_apply,
      sym_spec_pragma,
      sym_spec_let,
      sym_spec_update,
      sym_spec_axiom,
  [14015] = 23,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1558), 1,
      sym_identifier,
    ACTIONS(1562), 1,
      anon_sym_invariant,
    ACTIONS(1564), 1,
      anon_sym_update,
    ACTIONS(1568), 1,
      anon_sym_aborts_if,
    ACTIONS(1572), 1,
      anon_sym_emits,
    ACTIONS(1576), 1,
      anon_sym_fun,
    ACTIONS(1578), 1,
      anon_sym_native,
    ACTIONS(1580), 1,
      anon_sym_include,
    ACTIONS(1582), 1,
      anon_sym_apply,
    ACTIONS(1584), 1,
      anon_sym_pragma,
    ACTIONS(1586), 1,
      anon_sym_let,
    ACTIONS(1588), 1,
      anon_sym_axiom,
    ACTIONS(1662), 1,
      anon_sym_RBRACE,
    STATE(1132), 1,
      aux_sym_spec_block_repeat2,
    STATE(1452), 1,
      sym__spec_block_member,
    ACTIONS(1570), 2,
      anon_sym_aborts_with,
      anon_sym_modifies,
    ACTIONS(1574), 2,
      anon_sym_global,
      anon_sym_local,
    STATE(1129), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1566), 4,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
    STATE(2453), 4,
      sym_asserts,
      sym_aborts_if,
      sym_aborts_with_or_modifies,
      sym_emits,
    STATE(1453), 10,
      sym_spec_invariant,
      sym_spec_condition,
      sym_spec_variable,
      sym_spec_func,
      sym_spec_include,
      sym_spec_apply,
      sym_spec_pragma,
      sym_spec_let,
      sym_spec_update,
      sym_spec_axiom,
  [14103] = 23,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1558), 1,
      sym_identifier,
    ACTIONS(1562), 1,
      anon_sym_invariant,
    ACTIONS(1564), 1,
      anon_sym_update,
    ACTIONS(1568), 1,
      anon_sym_aborts_if,
    ACTIONS(1572), 1,
      anon_sym_emits,
    ACTIONS(1576), 1,
      anon_sym_fun,
    ACTIONS(1578), 1,
      anon_sym_native,
    ACTIONS(1580), 1,
      anon_sym_include,
    ACTIONS(1582), 1,
      anon_sym_apply,
    ACTIONS(1584), 1,
      anon_sym_pragma,
    ACTIONS(1586), 1,
      anon_sym_let,
    ACTIONS(1588), 1,
      anon_sym_axiom,
    ACTIONS(1664), 1,
      anon_sym_RBRACE,
    STATE(1132), 1,
      aux_sym_spec_block_repeat2,
    STATE(1452), 1,
      sym__spec_block_member,
    ACTIONS(1570), 2,
      anon_sym_aborts_with,
      anon_sym_modifies,
    ACTIONS(1574), 2,
      anon_sym_global,
      anon_sym_local,
    STATE(1130), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1566), 4,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
    STATE(2453), 4,
      sym_asserts,
      sym_aborts_if,
      sym_aborts_with_or_modifies,
      sym_emits,
    STATE(1453), 10,
      sym_spec_invariant,
      sym_spec_condition,
      sym_spec_variable,
      sym_spec_func,
      sym_spec_include,
      sym_spec_apply,
      sym_spec_pragma,
      sym_spec_let,
      sym_spec_update,
      sym_spec_axiom,
  [14191] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1460), 1,
      sym_identifier,
    ACTIONS(1666), 1,
      anon_sym_min,
    STATE(2224), 1,
      sym_quantifier_bind,
    STATE(1131), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 15,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_spec,
    ACTIONS(825), 19,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
  [14249] = 22,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1668), 1,
      sym_identifier,
    ACTIONS(1671), 1,
      anon_sym_RBRACE,
    ACTIONS(1673), 1,
      anon_sym_invariant,
    ACTIONS(1676), 1,
      anon_sym_update,
    ACTIONS(1682), 1,
      anon_sym_aborts_if,
    ACTIONS(1688), 1,
      anon_sym_emits,
    ACTIONS(1694), 1,
      anon_sym_fun,
    ACTIONS(1697), 1,
      anon_sym_native,
    ACTIONS(1700), 1,
      anon_sym_include,
    ACTIONS(1703), 1,
      anon_sym_apply,
    ACTIONS(1706), 1,
      anon_sym_pragma,
    ACTIONS(1709), 1,
      anon_sym_let,
    ACTIONS(1712), 1,
      anon_sym_axiom,
    STATE(1452), 1,
      sym__spec_block_member,
    ACTIONS(1685), 2,
      anon_sym_aborts_with,
      anon_sym_modifies,
    ACTIONS(1691), 2,
      anon_sym_global,
      anon_sym_local,
    STATE(1132), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_spec_block_repeat2,
    ACTIONS(1679), 4,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
    STATE(2453), 4,
      sym_asserts,
      sym_aborts_if,
      sym_aborts_with_or_modifies,
      sym_emits,
    STATE(1453), 10,
      sym_spec_invariant,
      sym_spec_condition,
      sym_spec_variable,
      sym_spec_func,
      sym_spec_include,
      sym_spec_apply,
      sym_spec_pragma,
      sym_spec_let,
      sym_spec_update,
      sym_spec_axiom,
  [14335] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1460), 1,
      sym_identifier,
    ACTIONS(1715), 1,
      anon_sym_min,
    STATE(2350), 1,
      sym_quantifier_bind,
    STATE(1133), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 14,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_as,
      anon_sym_else,
      anon_sym_LT2,
    ACTIONS(825), 20,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
  [14393] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1460), 1,
      sym_identifier,
    ACTIONS(1717), 1,
      anon_sym_min,
    STATE(2234), 1,
      sym_quantifier_bind,
    STATE(1134), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 15,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_spec,
    ACTIONS(825), 19,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
  [14451] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1460), 1,
      sym_identifier,
    ACTIONS(1719), 1,
      anon_sym_min,
    STATE(2431), 1,
      sym_quantifier_bind,
    STATE(1135), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 14,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_as,
      anon_sym_else,
      anon_sym_LT2,
    ACTIONS(825), 20,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
  [14509] = 15,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1723), 1,
      anon_sym_PIPE,
    ACTIONS(1725), 1,
      anon_sym_AMP,
    ACTIONS(1731), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1733), 1,
      anon_sym_CARET,
    ACTIONS(1739), 1,
      anon_sym_SLASH,
    ACTIONS(915), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(1721), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1729), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1735), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1737), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1136), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1727), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 17,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [14580] = 24,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1246), 1,
      sym_identifier,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1260), 1,
      anon_sym_PIPE,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1521), 1,
      sym_primitive_type,
    STATE(1526), 1,
      sym_number_type,
    STATE(1533), 1,
      sym_name_access_chain,
    STATE(1539), 1,
      sym_discouraged_name,
    STATE(1567), 1,
      sym__type,
    STATE(2175), 1,
      sym__leading_name_access,
    STATE(1137), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1527), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(1252), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(1250), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(1254), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(1248), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [14669] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1725), 1,
      anon_sym_AMP,
    ACTIONS(1733), 1,
      anon_sym_CARET,
    ACTIONS(1739), 1,
      anon_sym_SLASH,
    ACTIONS(1721), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1735), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1737), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1138), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 20,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [14732] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1739), 1,
      anon_sym_SLASH,
    ACTIONS(1721), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1735), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1737), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1139), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 21,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [14791] = 17,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1723), 1,
      anon_sym_PIPE,
    ACTIONS(1725), 1,
      anon_sym_AMP,
    ACTIONS(1731), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1733), 1,
      anon_sym_CARET,
    ACTIONS(1739), 1,
      anon_sym_SLASH,
    ACTIONS(1741), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1743), 1,
      anon_sym_AMP_AMP,
    ACTIONS(915), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(1721), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1729), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1735), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1737), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1140), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1727), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [14866] = 16,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1723), 1,
      anon_sym_PIPE,
    ACTIONS(1725), 1,
      anon_sym_AMP,
    ACTIONS(1731), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1733), 1,
      anon_sym_CARET,
    ACTIONS(1739), 1,
      anon_sym_SLASH,
    ACTIONS(1743), 1,
      anon_sym_AMP_AMP,
    ACTIONS(915), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(1721), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1729), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1735), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1737), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1141), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1727), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 16,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [14939] = 13,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1723), 1,
      anon_sym_PIPE,
    ACTIONS(1725), 1,
      anon_sym_AMP,
    ACTIONS(1731), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1733), 1,
      anon_sym_CARET,
    ACTIONS(1739), 1,
      anon_sym_SLASH,
    ACTIONS(1721), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1735), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1737), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1142), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 19,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [15006] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1723), 1,
      anon_sym_PIPE,
    ACTIONS(1725), 1,
      anon_sym_AMP,
    ACTIONS(1733), 1,
      anon_sym_CARET,
    ACTIONS(1739), 1,
      anon_sym_SLASH,
    ACTIONS(1721), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1735), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1737), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1143), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 20,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [15071] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1725), 1,
      anon_sym_AMP,
    ACTIONS(1739), 1,
      anon_sym_SLASH,
    ACTIONS(1721), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1735), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1737), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1144), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 21,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [15132] = 18,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1723), 1,
      anon_sym_PIPE,
    ACTIONS(1725), 1,
      anon_sym_AMP,
    ACTIONS(1731), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1733), 1,
      anon_sym_CARET,
    ACTIONS(1739), 1,
      anon_sym_SLASH,
    ACTIONS(1741), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1743), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1039), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(1721), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1729), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1735), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1737), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(1745), 2,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
    STATE(1145), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1727), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(1041), 13,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [15209] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1747), 1,
      anon_sym_EQ,
    STATE(1146), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1047), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1049), 27,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [15262] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1749), 1,
      anon_sym_else,
    STATE(1147), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1327), 9,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1329), 26,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [15315] = 24,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1246), 1,
      sym_identifier,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1260), 1,
      anon_sym_PIPE,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1521), 1,
      sym_primitive_type,
    STATE(1526), 1,
      sym_number_type,
    STATE(1533), 1,
      sym_name_access_chain,
    STATE(1539), 1,
      sym_discouraged_name,
    STATE(1557), 1,
      sym__type,
    STATE(2175), 1,
      sym__leading_name_access,
    STATE(1148), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1527), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(1252), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(1250), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(1254), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(1248), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [15404] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1739), 1,
      anon_sym_SLASH,
    ACTIONS(1721), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1737), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1149), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 23,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [15461] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1751), 1,
      anon_sym_EQ,
    STATE(1150), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1047), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1049), 27,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [15514] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1739), 1,
      anon_sym_SLASH,
    ACTIONS(1721), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    STATE(1151), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 25,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [15569] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1480), 1,
      sym_identifier,
    STATE(1727), 1,
      sym_quantifier_bind,
    STATE(1152), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 15,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_spec,
    ACTIONS(825), 19,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
  [15624] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1460), 1,
      sym_identifier,
    ACTIONS(1753), 1,
      anon_sym_min,
    STATE(2338), 1,
      sym_quantifier_bind,
    STATE(1153), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 14,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_if,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_spec,
    ACTIONS(825), 19,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_SEMI,
  [15681] = 24,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(1567), 1,
      sym__type,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1154), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [15770] = 24,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    ACTIONS(1293), 1,
      anon_sym_PIPE,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1554), 1,
      sym_primitive_type,
    STATE(1557), 1,
      sym__type,
    STATE(1564), 1,
      sym_name_access_chain,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(860), 2,
      sym__quantifier_directive,
      sym__reuseable_keywords,
    STATE(1155), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1566), 2,
      sym_tuple_type,
      sym_closure_type,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [15859] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1460), 1,
      sym_identifier,
    ACTIONS(1755), 1,
      anon_sym_min,
    STATE(2172), 1,
      sym_quantifier_bind,
    STATE(1156), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 14,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
    ACTIONS(825), 19,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
  [15916] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1460), 1,
      sym_identifier,
    ACTIONS(1757), 1,
      anon_sym_min,
    STATE(2179), 1,
      sym_quantifier_bind,
    STATE(1157), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 14,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
    ACTIONS(825), 19,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
  [15973] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1460), 1,
      sym_identifier,
    ACTIONS(1759), 1,
      anon_sym_min,
    STATE(2248), 1,
      sym_quantifier_bind,
    STATE(1158), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 14,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_if,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_spec,
    ACTIONS(825), 19,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_SEMI,
  [16030] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(138), 1,
      sym_parenthesized_expr,
    STATE(1159), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(815), 12,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_LT2,
    ACTIONS(817), 22,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [16085] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1480), 1,
      sym_identifier,
    STATE(1738), 1,
      sym_quantifier_bind,
    STATE(1160), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 14,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_as,
      anon_sym_else,
      anon_sym_LT2,
    ACTIONS(825), 20,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
  [16140] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1480), 1,
      sym_identifier,
    STATE(1726), 1,
      sym_quantifier_bind,
    STATE(1161), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 15,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_spec,
    ACTIONS(825), 19,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
  [16195] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1480), 1,
      sym_identifier,
    STATE(1748), 1,
      sym_quantifier_bind,
    STATE(1162), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 14,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_as,
      anon_sym_else,
      anon_sym_LT2,
    ACTIONS(825), 20,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
  [16250] = 16,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1763), 1,
      anon_sym_PIPE,
    ACTIONS(1765), 1,
      anon_sym_AMP,
    ACTIONS(1767), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1773), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1775), 1,
      anon_sym_CARET,
    ACTIONS(1781), 1,
      anon_sym_SLASH,
    ACTIONS(915), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1771), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1777), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1779), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1163), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1769), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [16322] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1785), 1,
      anon_sym_SLASH,
    ACTIONS(1783), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    STATE(1164), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 25,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [16376] = 18,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1039), 1,
      anon_sym_DOT,
    ACTIONS(1785), 1,
      anon_sym_SLASH,
    ACTIONS(1787), 1,
      anon_sym_PIPE,
    ACTIONS(1789), 1,
      anon_sym_AMP,
    ACTIONS(1793), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1795), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1801), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1803), 1,
      anon_sym_CARET,
    ACTIONS(1783), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1791), 2,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
    ACTIONS(1799), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1805), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1807), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1165), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1797), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(1041), 13,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [16452] = 17,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(915), 1,
      anon_sym_DOT,
    ACTIONS(1785), 1,
      anon_sym_SLASH,
    ACTIONS(1787), 1,
      anon_sym_PIPE,
    ACTIONS(1789), 1,
      anon_sym_AMP,
    ACTIONS(1793), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1795), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1801), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1803), 1,
      anon_sym_CARET,
    ACTIONS(1783), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1799), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1805), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1807), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1166), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1797), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [16526] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(869), 1,
      anon_sym_LT2,
    ACTIONS(887), 1,
      anon_sym_LPAREN,
    ACTIONS(1206), 1,
      anon_sym_BANG,
    ACTIONS(1809), 1,
      anon_sym_LBRACE,
    STATE(905), 1,
      sym_call_args,
    STATE(1243), 1,
      sym_type_args,
    STATE(1167), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1197), 9,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1199), 20,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [16588] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1765), 1,
      anon_sym_AMP,
    ACTIONS(1775), 1,
      anon_sym_CARET,
    ACTIONS(1781), 1,
      anon_sym_SLASH,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1777), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1779), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1168), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 19,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [16650] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1785), 1,
      anon_sym_SLASH,
    ACTIONS(1783), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1805), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1807), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1169), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 21,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [16708] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1781), 1,
      anon_sym_SLASH,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1777), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1779), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1170), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 20,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [16766] = 17,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1763), 1,
      anon_sym_PIPE,
    ACTIONS(1765), 1,
      anon_sym_AMP,
    ACTIONS(1767), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1773), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1775), 1,
      anon_sym_CARET,
    ACTIONS(1781), 1,
      anon_sym_SLASH,
    ACTIONS(1812), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(915), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1771), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1777), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1779), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1171), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1769), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 14,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [16840] = 13,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1763), 1,
      anon_sym_PIPE,
    ACTIONS(1765), 1,
      anon_sym_AMP,
    ACTIONS(1773), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1775), 1,
      anon_sym_CARET,
    ACTIONS(1781), 1,
      anon_sym_SLASH,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1777), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1779), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1172), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 18,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [16906] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1781), 1,
      anon_sym_SLASH,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    STATE(1173), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 24,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [16960] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1460), 1,
      sym_identifier,
    ACTIONS(1814), 1,
      anon_sym_min,
    STATE(2200), 1,
      sym_quantifier_bind,
    STATE(1174), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 13,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_if,
      anon_sym_else,
      anon_sym_LT2,
    ACTIONS(825), 19,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_SEMI,
  [17016] = 23,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1614), 1,
      sym_identifier,
    ACTIONS(1816), 1,
      anon_sym_COMMA,
    ACTIONS(1818), 1,
      anon_sym_RPAREN,
    STATE(859), 1,
      sym_number_type,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1579), 1,
      sym_name_access_chain,
    STATE(1595), 1,
      sym_discouraged_name,
    STATE(1641), 1,
      sym_var_name,
    STATE(1888), 1,
      sym__bind,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(1175), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [17102] = 23,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1614), 1,
      sym_identifier,
    ACTIONS(1616), 1,
      anon_sym_LPAREN,
    STATE(859), 1,
      sym_number_type,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1579), 1,
      sym_name_access_chain,
    STATE(1595), 1,
      sym_discouraged_name,
    STATE(1641), 1,
      sym_var_name,
    STATE(1791), 1,
      sym__bind,
    STATE(1997), 1,
      sym_bind_list,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(1176), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [17188] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1460), 1,
      sym_identifier,
    ACTIONS(1820), 1,
      anon_sym_min,
    STATE(2259), 1,
      sym_quantifier_bind,
    STATE(1177), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 13,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_if,
      anon_sym_else,
      anon_sym_LT2,
    ACTIONS(825), 19,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_SEMI,
  [17244] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1480), 1,
      sym_identifier,
    STATE(1720), 1,
      sym_quantifier_bind,
    STATE(1178), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 14,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
    ACTIONS(825), 19,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
  [17298] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1781), 1,
      anon_sym_SLASH,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1779), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1179), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 22,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [17354] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1822), 1,
      anon_sym_EQ,
    STATE(1180), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1047), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1049), 26,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [17406] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1824), 1,
      anon_sym_else,
    STATE(1181), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1327), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1329), 26,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [17458] = 13,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1785), 1,
      anon_sym_SLASH,
    ACTIONS(1787), 1,
      anon_sym_PIPE,
    ACTIONS(1789), 1,
      anon_sym_AMP,
    ACTIONS(1801), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1803), 1,
      anon_sym_CARET,
    ACTIONS(1783), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1805), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1807), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1182), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 5,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 19,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [17524] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1480), 1,
      sym_identifier,
    STATE(1730), 1,
      sym_quantifier_bind,
    STATE(1183), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 14,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_if,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_spec,
    ACTIONS(825), 19,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_SEMI,
  [17578] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1826), 1,
      anon_sym_else,
    STATE(1184), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1327), 9,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1329), 25,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [17630] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(127), 1,
      sym_parenthesized_expr,
    STATE(1185), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(815), 12,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_LT2,
    ACTIONS(817), 21,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [17684] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1785), 1,
      anon_sym_SLASH,
    ACTIONS(1787), 1,
      anon_sym_PIPE,
    ACTIONS(1789), 1,
      anon_sym_AMP,
    ACTIONS(1803), 1,
      anon_sym_CARET,
    ACTIONS(1783), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1805), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1807), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1186), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 5,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 20,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [17748] = 23,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1614), 1,
      sym_identifier,
    ACTIONS(1828), 1,
      anon_sym_COMMA,
    ACTIONS(1830), 1,
      anon_sym_RPAREN,
    STATE(859), 1,
      sym_number_type,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1579), 1,
      sym_name_access_chain,
    STATE(1595), 1,
      sym_discouraged_name,
    STATE(1641), 1,
      sym_var_name,
    STATE(1940), 1,
      sym__bind,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(1187), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [17834] = 18,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1763), 1,
      anon_sym_PIPE,
    ACTIONS(1765), 1,
      anon_sym_AMP,
    ACTIONS(1767), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1773), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1775), 1,
      anon_sym_CARET,
    ACTIONS(1781), 1,
      anon_sym_SLASH,
    ACTIONS(1812), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1039), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1771), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1777), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1779), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(1832), 2,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
    STATE(1188), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1769), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(1041), 12,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [17910] = 15,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1763), 1,
      anon_sym_PIPE,
    ACTIONS(1765), 1,
      anon_sym_AMP,
    ACTIONS(1773), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1775), 1,
      anon_sym_CARET,
    ACTIONS(1781), 1,
      anon_sym_SLASH,
    ACTIONS(915), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1771), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1777), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1779), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1189), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1769), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 16,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [17980] = 16,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(915), 1,
      anon_sym_DOT,
    ACTIONS(1785), 1,
      anon_sym_SLASH,
    ACTIONS(1787), 1,
      anon_sym_PIPE,
    ACTIONS(1789), 1,
      anon_sym_AMP,
    ACTIONS(1795), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1801), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1803), 1,
      anon_sym_CARET,
    ACTIONS(1783), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1799), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1805), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1807), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1190), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1797), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 16,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [18052] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1763), 1,
      anon_sym_PIPE,
    ACTIONS(1765), 1,
      anon_sym_AMP,
    ACTIONS(1775), 1,
      anon_sym_CARET,
    ACTIONS(1781), 1,
      anon_sym_SLASH,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1777), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1779), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1191), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 19,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [18116] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1765), 1,
      anon_sym_AMP,
    ACTIONS(1781), 1,
      anon_sym_SLASH,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1777), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1779), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1192), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 20,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [18176] = 23,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1614), 1,
      sym_identifier,
    ACTIONS(1834), 1,
      anon_sym_COMMA,
    ACTIONS(1836), 1,
      anon_sym_PIPE,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1526), 1,
      sym_number_type,
    STATE(1641), 1,
      sym_var_name,
    STATE(1669), 1,
      sym_name_access_chain,
    STATE(1759), 1,
      sym_discouraged_name,
    STATE(1876), 1,
      sym__bind,
    STATE(2175), 1,
      sym__leading_name_access,
    STATE(1193), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1252), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(1527), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(1250), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(1254), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(1248), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [18262] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1838), 1,
      anon_sym_EQ,
    STATE(1194), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1047), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1049), 26,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [18314] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1785), 1,
      anon_sym_SLASH,
    ACTIONS(1789), 1,
      anon_sym_AMP,
    ACTIONS(1803), 1,
      anon_sym_CARET,
    ACTIONS(1783), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1805), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1807), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1195), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_PIPE,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 20,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [18376] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1480), 1,
      sym_identifier,
    STATE(1737), 1,
      sym_quantifier_bind,
    STATE(1196), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 14,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_if,
      anon_sym_else,
      anon_sym_LT2,
      anon_sym_spec,
    ACTIONS(825), 19,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_SEMI,
  [18430] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(50), 1,
      sym_parenthesized_expr,
    STATE(1197), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(815), 11,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_LT2,
    ACTIONS(817), 22,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [18484] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1785), 1,
      anon_sym_SLASH,
    ACTIONS(1789), 1,
      anon_sym_AMP,
    ACTIONS(1783), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1805), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1807), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1198), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_PIPE,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 21,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [18544] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1785), 1,
      anon_sym_SLASH,
    ACTIONS(1783), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1807), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1199), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 23,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [18600] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(98), 1,
      sym_parenthesized_expr,
    STATE(1200), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(815), 12,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_LT2,
    ACTIONS(817), 21,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [18654] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1480), 1,
      sym_identifier,
    STATE(1732), 1,
      sym_quantifier_bind,
    STATE(1201), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 14,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_else,
      anon_sym_LT2,
    ACTIONS(825), 19,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
  [18708] = 15,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(915), 1,
      anon_sym_DOT,
    ACTIONS(1785), 1,
      anon_sym_SLASH,
    ACTIONS(1787), 1,
      anon_sym_PIPE,
    ACTIONS(1789), 1,
      anon_sym_AMP,
    ACTIONS(1801), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1803), 1,
      anon_sym_CARET,
    ACTIONS(1783), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1799), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1805), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1807), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1202), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1797), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 17,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [18778] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1480), 1,
      sym_identifier,
    STATE(1724), 1,
      sym_quantifier_bind,
    STATE(1203), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 13,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_if,
      anon_sym_else,
      anon_sym_LT2,
    ACTIONS(825), 19,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_SEMI,
  [18831] = 22,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1614), 1,
      sym_identifier,
    ACTIONS(1840), 1,
      anon_sym_PIPE,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1526), 1,
      sym_number_type,
    STATE(1641), 1,
      sym_var_name,
    STATE(1669), 1,
      sym_name_access_chain,
    STATE(1759), 1,
      sym_discouraged_name,
    STATE(1946), 1,
      sym__bind,
    STATE(2175), 1,
      sym__leading_name_access,
    STATE(1204), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1252), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(1527), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(1250), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(1254), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(1248), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [18914] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1844), 1,
      anon_sym_AMP,
    ACTIONS(1846), 1,
      anon_sym_CARET,
    ACTIONS(1852), 1,
      anon_sym_SLASH,
    ACTIONS(1842), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1848), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1850), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1205), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_PIPE,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 19,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [18975] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1852), 1,
      anon_sym_SLASH,
    ACTIONS(1842), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1848), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1850), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1206), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 20,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [19032] = 17,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(915), 1,
      anon_sym_DOT,
    ACTIONS(1844), 1,
      anon_sym_AMP,
    ACTIONS(1846), 1,
      anon_sym_CARET,
    ACTIONS(1852), 1,
      anon_sym_SLASH,
    ACTIONS(1854), 1,
      anon_sym_PIPE,
    ACTIONS(1856), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1858), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1864), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1842), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1848), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1850), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(1862), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1207), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1860), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 14,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [19105] = 16,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(915), 1,
      anon_sym_DOT,
    ACTIONS(1844), 1,
      anon_sym_AMP,
    ACTIONS(1846), 1,
      anon_sym_CARET,
    ACTIONS(1852), 1,
      anon_sym_SLASH,
    ACTIONS(1854), 1,
      anon_sym_PIPE,
    ACTIONS(1858), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1864), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1842), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1848), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1850), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(1862), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1208), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1860), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [19176] = 15,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(915), 1,
      anon_sym_DOT,
    ACTIONS(1844), 1,
      anon_sym_AMP,
    ACTIONS(1846), 1,
      anon_sym_CARET,
    ACTIONS(1852), 1,
      anon_sym_SLASH,
    ACTIONS(1854), 1,
      anon_sym_PIPE,
    ACTIONS(1864), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1842), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1848), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1850), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(1862), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1209), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1860), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 16,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [19245] = 13,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1844), 1,
      anon_sym_AMP,
    ACTIONS(1846), 1,
      anon_sym_CARET,
    ACTIONS(1852), 1,
      anon_sym_SLASH,
    ACTIONS(1854), 1,
      anon_sym_PIPE,
    ACTIONS(1864), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1842), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1848), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1850), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1210), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 5,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 18,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [19310] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1844), 1,
      anon_sym_AMP,
    ACTIONS(1846), 1,
      anon_sym_CARET,
    ACTIONS(1852), 1,
      anon_sym_SLASH,
    ACTIONS(1854), 1,
      anon_sym_PIPE,
    ACTIONS(1842), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1848), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1850), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1211), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 5,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 19,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [19373] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1844), 1,
      anon_sym_AMP,
    ACTIONS(1852), 1,
      anon_sym_SLASH,
    ACTIONS(1842), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1848), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1850), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1212), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_PIPE,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 20,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [19432] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1852), 1,
      anon_sym_SLASH,
    ACTIONS(1842), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1850), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1213), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 22,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [19487] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1852), 1,
      anon_sym_SLASH,
    ACTIONS(1842), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    STATE(1214), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 24,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [19540] = 22,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1602), 1,
      sym_identifier,
    ACTIONS(1604), 1,
      anon_sym_STAR,
    STATE(859), 1,
      sym_number_type,
    STATE(880), 1,
      sym_number,
    STATE(1604), 1,
      sym_discouraged_name,
    STATE(1610), 1,
      sym_name_access_chain_wildcard,
    STATE(1668), 1,
      sym_access_specifier,
    STATE(1752), 1,
      sym_access_specifier_list,
    STATE(2334), 1,
      sym_numerical_addr,
    STATE(2340), 1,
      sym__leading_name_access_wildcard,
    STATE(1215), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [19623] = 18,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1039), 1,
      anon_sym_DOT,
    ACTIONS(1844), 1,
      anon_sym_AMP,
    ACTIONS(1846), 1,
      anon_sym_CARET,
    ACTIONS(1852), 1,
      anon_sym_SLASH,
    ACTIONS(1854), 1,
      anon_sym_PIPE,
    ACTIONS(1856), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1858), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1864), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1842), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1848), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1850), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(1862), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1866), 2,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
    STATE(1216), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1860), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(1041), 12,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_else,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [19698] = 22,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1614), 1,
      sym_identifier,
    ACTIONS(1868), 1,
      anon_sym_PIPE,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1526), 1,
      sym_number_type,
    STATE(1641), 1,
      sym_var_name,
    STATE(1669), 1,
      sym_name_access_chain,
    STATE(1759), 1,
      sym_discouraged_name,
    STATE(1946), 1,
      sym__bind,
    STATE(2175), 1,
      sym__leading_name_access,
    STATE(1217), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1252), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(1527), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(1250), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(1254), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(1248), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [19781] = 22,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1614), 1,
      sym_identifier,
    ACTIONS(1870), 1,
      anon_sym_RPAREN,
    STATE(859), 1,
      sym_number_type,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1579), 1,
      sym_name_access_chain,
    STATE(1595), 1,
      sym_discouraged_name,
    STATE(1641), 1,
      sym_var_name,
    STATE(1946), 1,
      sym__bind,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(1218), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [19864] = 22,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1614), 1,
      sym_identifier,
    ACTIONS(1872), 1,
      anon_sym_RPAREN,
    STATE(859), 1,
      sym_number_type,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1579), 1,
      sym_name_access_chain,
    STATE(1595), 1,
      sym_discouraged_name,
    STATE(1641), 1,
      sym_var_name,
    STATE(1946), 1,
      sym__bind,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(1219), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [19947] = 22,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1614), 1,
      sym_identifier,
    ACTIONS(1874), 1,
      anon_sym_RPAREN,
    STATE(859), 1,
      sym_number_type,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1579), 1,
      sym_name_access_chain,
    STATE(1595), 1,
      sym_discouraged_name,
    STATE(1641), 1,
      sym_var_name,
    STATE(1946), 1,
      sym__bind,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(1220), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [20030] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1876), 1,
      anon_sym_else,
    STATE(1221), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1327), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1329), 25,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
      anon_sym_with,
      anon_sym_to,
  [20081] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1480), 1,
      sym_identifier,
    STATE(1709), 1,
      sym_quantifier_bind,
    STATE(1222), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(823), 13,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_if,
      anon_sym_else,
      anon_sym_LT2,
    ACTIONS(825), 19,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_SEMI,
  [20134] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(86), 1,
      sym_parenthesized_expr,
    STATE(1223), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(815), 12,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_LT2,
    ACTIONS(817), 20,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_else,
  [20187] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(106), 1,
      sym_parenthesized_expr,
    STATE(1224), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(815), 11,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_LT2,
    ACTIONS(817), 21,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [20240] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(119), 1,
      sym_parenthesized_expr,
    STATE(1225), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(815), 11,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_LT2,
    ACTIONS(817), 21,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [20293] = 22,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1614), 1,
      sym_identifier,
    ACTIONS(1878), 1,
      anon_sym_RPAREN,
    STATE(859), 1,
      sym_number_type,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1579), 1,
      sym_name_access_chain,
    STATE(1595), 1,
      sym_discouraged_name,
    STATE(1641), 1,
      sym_var_name,
    STATE(1946), 1,
      sym__bind,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(1226), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [20376] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(142), 1,
      sym_parenthesized_expr,
    STATE(1227), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(815), 11,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_LT2,
    ACTIONS(817), 21,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [20429] = 22,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1602), 1,
      sym_identifier,
    ACTIONS(1604), 1,
      anon_sym_STAR,
    STATE(859), 1,
      sym_number_type,
    STATE(880), 1,
      sym_number,
    STATE(1604), 1,
      sym_discouraged_name,
    STATE(1610), 1,
      sym_name_access_chain_wildcard,
    STATE(1668), 1,
      sym_access_specifier,
    STATE(1769), 1,
      sym_access_specifier_list,
    STATE(2334), 1,
      sym_numerical_addr,
    STATE(2340), 1,
      sym__leading_name_access_wildcard,
    STATE(1228), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [20512] = 28,
    ACTIONS(11), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1880), 1,
      anon_sym_friend,
    ACTIONS(1882), 1,
      anon_sym_RBRACE,
    ACTIONS(1884), 1,
      anon_sym_spec,
    ACTIONS(1886), 1,
      anon_sym_invariant,
    ACTIONS(1888), 1,
      anon_sym_fun,
    ACTIONS(1892), 1,
      anon_sym_public,
    ACTIONS(1894), 1,
      anon_sym_use,
    ACTIONS(1896), 1,
      anon_sym_const,
    ACTIONS(1898), 1,
      anon_sym_inline,
    ACTIONS(1900), 1,
      anon_sym_struct,
    ACTIONS(1902), 1,
      anon_sym_enum,
    STATE(1230), 1,
      aux_sym_module_repeat1,
    STATE(1380), 1,
      sym_attributes,
    STATE(1518), 1,
      aux_sym_declaration_repeat1,
    STATE(1536), 1,
      aux_sym_attributes_repeat2,
    STATE(1570), 1,
      sym_declaration,
    STATE(1671), 1,
      sym_module_member_modifier,
    STATE(1686), 1,
      sym_visibility,
    STATE(1868), 1,
      sym__struct_signature,
    STATE(1948), 1,
      sym__function_signature,
    STATE(2063), 1,
      sym__enum_signature,
    ACTIONS(1890), 2,
      anon_sym_native,
      anon_sym_entry,
    STATE(1229), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1577), 2,
      sym_struct_decl,
      sym_enum_decl,
    STATE(1571), 7,
      sym_spec_block,
      sym_spec_invariant,
      sym_use_decl,
      sym_friend_decl,
      sym_constant_decl,
      sym_function_decl,
      sym__struct_or_enum_decl,
  [20606] = 27,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1904), 1,
      anon_sym_friend,
    ACTIONS(1907), 1,
      anon_sym_RBRACE,
    ACTIONS(1909), 1,
      anon_sym_POUND,
    ACTIONS(1912), 1,
      anon_sym_spec,
    ACTIONS(1915), 1,
      anon_sym_invariant,
    ACTIONS(1918), 1,
      anon_sym_fun,
    ACTIONS(1924), 1,
      anon_sym_public,
    ACTIONS(1927), 1,
      anon_sym_use,
    ACTIONS(1930), 1,
      anon_sym_const,
    ACTIONS(1933), 1,
      anon_sym_inline,
    ACTIONS(1936), 1,
      anon_sym_struct,
    ACTIONS(1939), 1,
      anon_sym_enum,
    STATE(1380), 1,
      sym_attributes,
    STATE(1518), 1,
      aux_sym_declaration_repeat1,
    STATE(1536), 1,
      aux_sym_attributes_repeat2,
    STATE(1570), 1,
      sym_declaration,
    STATE(1671), 1,
      sym_module_member_modifier,
    STATE(1686), 1,
      sym_visibility,
    STATE(1868), 1,
      sym__struct_signature,
    STATE(1948), 1,
      sym__function_signature,
    STATE(2063), 1,
      sym__enum_signature,
    ACTIONS(1921), 2,
      anon_sym_native,
      anon_sym_entry,
    STATE(1577), 2,
      sym_struct_decl,
      sym_enum_decl,
    STATE(1230), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_module_repeat1,
    STATE(1571), 7,
      sym_spec_block,
      sym_spec_invariant,
      sym_use_decl,
      sym_friend_decl,
      sym_constant_decl,
      sym_function_decl,
      sym__struct_or_enum_decl,
  [20698] = 28,
    ACTIONS(11), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1880), 1,
      anon_sym_friend,
    ACTIONS(1884), 1,
      anon_sym_spec,
    ACTIONS(1886), 1,
      anon_sym_invariant,
    ACTIONS(1888), 1,
      anon_sym_fun,
    ACTIONS(1892), 1,
      anon_sym_public,
    ACTIONS(1894), 1,
      anon_sym_use,
    ACTIONS(1896), 1,
      anon_sym_const,
    ACTIONS(1898), 1,
      anon_sym_inline,
    ACTIONS(1900), 1,
      anon_sym_struct,
    ACTIONS(1902), 1,
      anon_sym_enum,
    ACTIONS(1942), 1,
      anon_sym_RBRACE,
    STATE(1229), 1,
      aux_sym_module_repeat1,
    STATE(1380), 1,
      sym_attributes,
    STATE(1518), 1,
      aux_sym_declaration_repeat1,
    STATE(1536), 1,
      aux_sym_attributes_repeat2,
    STATE(1570), 1,
      sym_declaration,
    STATE(1671), 1,
      sym_module_member_modifier,
    STATE(1686), 1,
      sym_visibility,
    STATE(1868), 1,
      sym__struct_signature,
    STATE(1948), 1,
      sym__function_signature,
    STATE(2063), 1,
      sym__enum_signature,
    ACTIONS(1890), 2,
      anon_sym_native,
      anon_sym_entry,
    STATE(1231), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1577), 2,
      sym_struct_decl,
      sym_enum_decl,
    STATE(1571), 7,
      sym_spec_block,
      sym_spec_invariant,
      sym_use_decl,
      sym_friend_decl,
      sym_constant_decl,
      sym_function_decl,
      sym__struct_or_enum_decl,
  [20792] = 21,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1944), 1,
      anon_sym_STAR,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(1835), 1,
      sym_name_access_chain,
    STATE(2058), 1,
      sym_numerical_addr,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(2349), 1,
      sym__address_specifier,
    STATE(1232), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [20872] = 21,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1614), 1,
      sym_identifier,
    STATE(859), 1,
      sym_number_type,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1579), 1,
      sym_name_access_chain,
    STATE(1595), 1,
      sym_discouraged_name,
    STATE(1641), 1,
      sym_var_name,
    STATE(2010), 1,
      sym__bind,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(1233), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [20952] = 21,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1614), 1,
      sym_identifier,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1526), 1,
      sym_number_type,
    STATE(1641), 1,
      sym_var_name,
    STATE(1669), 1,
      sym_name_access_chain,
    STATE(1759), 1,
      sym_discouraged_name,
    STATE(1946), 1,
      sym__bind,
    STATE(2175), 1,
      sym__leading_name_access,
    STATE(1234), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1252), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(1527), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(1250), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(1254), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(1248), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [21032] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(152), 1,
      sym_parenthesized_expr,
    STATE(1235), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(815), 11,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_COLON,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_LT2,
    ACTIONS(817), 20,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_else,
  [21084] = 21,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1944), 1,
      anon_sym_STAR,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(1835), 1,
      sym_name_access_chain,
    STATE(2058), 1,
      sym_numerical_addr,
    STATE(2238), 1,
      sym__address_specifier,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(1236), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [21164] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(90), 1,
      sym_parenthesized_expr,
    STATE(1237), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(815), 11,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_LT2,
    ACTIONS(817), 20,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
  [21216] = 21,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1602), 1,
      sym_identifier,
    ACTIONS(1604), 1,
      anon_sym_STAR,
    STATE(859), 1,
      sym_number_type,
    STATE(880), 1,
      sym_number,
    STATE(1604), 1,
      sym_discouraged_name,
    STATE(1610), 1,
      sym_name_access_chain_wildcard,
    STATE(1736), 1,
      sym_access_specifier,
    STATE(2334), 1,
      sym_numerical_addr,
    STATE(2340), 1,
      sym__leading_name_access_wildcard,
    STATE(1238), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [21296] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(121), 1,
      sym_parenthesized_expr,
    STATE(1239), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(815), 10,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_LT2,
    ACTIONS(817), 21,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [21348] = 21,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(1614), 1,
      sym_identifier,
    STATE(859), 1,
      sym_number_type,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1579), 1,
      sym_name_access_chain,
    STATE(1595), 1,
      sym_discouraged_name,
    STATE(1641), 1,
      sym_var_name,
    STATE(1946), 1,
      sym__bind,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(1240), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [21428] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1946), 1,
      anon_sym_spec,
    STATE(944), 1,
      sym_spec_block,
    STATE(945), 1,
      sym_spec_loop_invariant,
    STATE(1241), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(877), 9,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(879), 20,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [21481] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(110), 1,
      sym_parenthesized_expr,
    STATE(1242), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(815), 10,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_LT2,
    ACTIONS(817), 20,
      anon_sym_STAR,
      anon_sym_COLON_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
  [21532] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(887), 1,
      anon_sym_LPAREN,
    ACTIONS(1948), 1,
      anon_sym_LBRACE,
    STATE(885), 1,
      sym_call_args,
    STATE(1243), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1284), 9,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1286), 20,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [21585] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1953), 2,
      anon_sym_RBRACE,
      anon_sym_POUND,
    STATE(1244), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1951), 29,
      sym_identifier,
      anon_sym_friend,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_public,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [21631] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1957), 2,
      anon_sym_RBRACE,
      anon_sym_POUND,
    STATE(1245), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1955), 29,
      sym_identifier,
      anon_sym_friend,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_public,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [21677] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1961), 2,
      anon_sym_RBRACE,
      anon_sym_POUND,
    STATE(1246), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1959), 29,
      sym_identifier,
      anon_sym_friend,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_public,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [21723] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1965), 2,
      anon_sym_RBRACE,
      anon_sym_POUND,
    STATE(1247), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1963), 29,
      sym_identifier,
      anon_sym_friend,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_public,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [21769] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1969), 2,
      anon_sym_RBRACE,
      anon_sym_POUND,
    STATE(1248), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1967), 29,
      sym_identifier,
      anon_sym_friend,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_public,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [21815] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1973), 2,
      anon_sym_RBRACE,
      anon_sym_POUND,
    STATE(1249), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1971), 29,
      sym_identifier,
      anon_sym_friend,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_public,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [21861] = 19,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(23), 1,
      sym_identifier,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    STATE(859), 1,
      sym_number_type,
    STATE(866), 1,
      sym_discouraged_name,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(2342), 1,
      sym__leading_name_access,
    STATE(2418), 1,
      sym_name_access_chain,
    STATE(1250), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [21935] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1977), 2,
      anon_sym_RBRACE,
      anon_sym_POUND,
    STATE(1251), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1975), 29,
      sym_identifier,
      anon_sym_friend,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_public,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [21981] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1981), 2,
      anon_sym_RBRACE,
      anon_sym_POUND,
    STATE(1252), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1979), 29,
      sym_identifier,
      anon_sym_friend,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_public,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [22027] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1983), 1,
      anon_sym_EQ,
    STATE(1253), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1047), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1049), 21,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [22074] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1985), 1,
      anon_sym_else,
    STATE(1254), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1327), 9,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1329), 20,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_spec,
  [22121] = 13,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1989), 1,
      anon_sym_PIPE,
    ACTIONS(1991), 1,
      anon_sym_AMP,
    ACTIONS(1993), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1995), 1,
      anon_sym_CARET,
    ACTIONS(2001), 1,
      anon_sym_SLASH,
    ACTIONS(1987), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1997), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1999), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1255), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 13,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [22182] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1991), 1,
      anon_sym_AMP,
    ACTIONS(1995), 1,
      anon_sym_CARET,
    ACTIONS(2001), 1,
      anon_sym_SLASH,
    ACTIONS(1987), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1997), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1999), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1256), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 14,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [22239] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1989), 1,
      anon_sym_PIPE,
    ACTIONS(1991), 1,
      anon_sym_AMP,
    ACTIONS(1995), 1,
      anon_sym_CARET,
    ACTIONS(2001), 1,
      anon_sym_SLASH,
    ACTIONS(1987), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1997), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1999), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1257), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 14,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [22298] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2001), 1,
      anon_sym_SLASH,
    ACTIONS(1987), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1997), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1999), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1258), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 15,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [22351] = 17,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1989), 1,
      anon_sym_PIPE,
    ACTIONS(1991), 1,
      anon_sym_AMP,
    ACTIONS(1993), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1995), 1,
      anon_sym_CARET,
    ACTIONS(2001), 1,
      anon_sym_SLASH,
    ACTIONS(2003), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2005), 1,
      anon_sym_AMP_AMP,
    ACTIONS(915), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(1987), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1997), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1999), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2009), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1259), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2007), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 9,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [22420] = 16,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1989), 1,
      anon_sym_PIPE,
    ACTIONS(1991), 1,
      anon_sym_AMP,
    ACTIONS(1993), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1995), 1,
      anon_sym_CARET,
    ACTIONS(2001), 1,
      anon_sym_SLASH,
    ACTIONS(2005), 1,
      anon_sym_AMP_AMP,
    ACTIONS(915), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(1987), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1997), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1999), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2009), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1260), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2007), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 10,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [22487] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2013), 1,
      anon_sym_AMP,
    ACTIONS(2015), 1,
      anon_sym_CARET,
    ACTIONS(2021), 1,
      anon_sym_SLASH,
    ACTIONS(2011), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2017), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2019), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1261), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 14,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [22544] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2021), 1,
      anon_sym_SLASH,
    ACTIONS(2011), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2017), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2019), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1262), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [22597] = 17,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2013), 1,
      anon_sym_AMP,
    ACTIONS(2015), 1,
      anon_sym_CARET,
    ACTIONS(2021), 1,
      anon_sym_SLASH,
    ACTIONS(2023), 1,
      anon_sym_PIPE,
    ACTIONS(2025), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2027), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2033), 1,
      anon_sym_DOT_DOT,
    ACTIONS(915), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2011), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2017), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2019), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2031), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1263), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2029), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 9,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [22666] = 16,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2013), 1,
      anon_sym_AMP,
    ACTIONS(2015), 1,
      anon_sym_CARET,
    ACTIONS(2021), 1,
      anon_sym_SLASH,
    ACTIONS(2023), 1,
      anon_sym_PIPE,
    ACTIONS(2027), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2033), 1,
      anon_sym_DOT_DOT,
    ACTIONS(915), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2011), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2017), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2019), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2031), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1264), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2029), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 10,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [22733] = 15,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2013), 1,
      anon_sym_AMP,
    ACTIONS(2015), 1,
      anon_sym_CARET,
    ACTIONS(2021), 1,
      anon_sym_SLASH,
    ACTIONS(2023), 1,
      anon_sym_PIPE,
    ACTIONS(2033), 1,
      anon_sym_DOT_DOT,
    ACTIONS(915), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2011), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2017), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2019), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2031), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1265), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2029), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 11,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [22798] = 13,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2013), 1,
      anon_sym_AMP,
    ACTIONS(2015), 1,
      anon_sym_CARET,
    ACTIONS(2021), 1,
      anon_sym_SLASH,
    ACTIONS(2023), 1,
      anon_sym_PIPE,
    ACTIONS(2033), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2011), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2017), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2019), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1266), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 13,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [22859] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2013), 1,
      anon_sym_AMP,
    ACTIONS(2015), 1,
      anon_sym_CARET,
    ACTIONS(2021), 1,
      anon_sym_SLASH,
    ACTIONS(2023), 1,
      anon_sym_PIPE,
    ACTIONS(2011), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2017), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2019), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1267), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 14,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [22918] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2013), 1,
      anon_sym_AMP,
    ACTIONS(2021), 1,
      anon_sym_SLASH,
    ACTIONS(2011), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2017), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2019), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1268), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [22973] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2021), 1,
      anon_sym_SLASH,
    ACTIONS(2011), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2019), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1269), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 17,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [23024] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2021), 1,
      anon_sym_SLASH,
    ACTIONS(2011), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    STATE(1270), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 19,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [23073] = 18,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2013), 1,
      anon_sym_AMP,
    ACTIONS(2015), 1,
      anon_sym_CARET,
    ACTIONS(2021), 1,
      anon_sym_SLASH,
    ACTIONS(2023), 1,
      anon_sym_PIPE,
    ACTIONS(2025), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2027), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2033), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1039), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2011), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2017), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2019), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2031), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(2035), 2,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
    STATE(1271), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2029), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(1041), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [23144] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2001), 1,
      anon_sym_SLASH,
    ACTIONS(1987), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    STATE(1272), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 19,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [23193] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2037), 1,
      anon_sym_else,
    STATE(1273), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1327), 9,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1329), 20,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_spec,
  [23240] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2039), 1,
      anon_sym_EQ,
    STATE(1274), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1047), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1049), 21,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [23287] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2041), 1,
      anon_sym_EQ,
    STATE(1275), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1047), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1049), 21,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [23334] = 18,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1989), 1,
      anon_sym_PIPE,
    ACTIONS(1991), 1,
      anon_sym_AMP,
    ACTIONS(1993), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1995), 1,
      anon_sym_CARET,
    ACTIONS(2001), 1,
      anon_sym_SLASH,
    ACTIONS(2003), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2005), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1039), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(1987), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1997), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1999), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2009), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(2043), 2,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
    STATE(1276), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2007), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(1041), 7,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [23405] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2045), 1,
      anon_sym_EQ,
    STATE(1277), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1047), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1049), 21,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [23452] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1991), 1,
      anon_sym_AMP,
    ACTIONS(2001), 1,
      anon_sym_SLASH,
    ACTIONS(1987), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1997), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1999), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1278), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 15,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [23507] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1279), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(519), 9,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(521), 21,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [23552] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1280), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(527), 9,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(529), 21,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [23597] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1281), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(523), 9,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(525), 21,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [23642] = 15,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1989), 1,
      anon_sym_PIPE,
    ACTIONS(1991), 1,
      anon_sym_AMP,
    ACTIONS(1993), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1995), 1,
      anon_sym_CARET,
    ACTIONS(2001), 1,
      anon_sym_SLASH,
    ACTIONS(915), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(1987), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1997), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1999), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2009), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1282), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2007), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 11,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [23707] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2001), 1,
      anon_sym_SLASH,
    ACTIONS(1987), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1999), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1283), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 17,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [23758] = 16,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(915), 1,
      anon_sym_DOT,
    ACTIONS(2049), 1,
      anon_sym_PIPE,
    ACTIONS(2051), 1,
      anon_sym_AMP,
    ACTIONS(2053), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2059), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2061), 1,
      anon_sym_CARET,
    ACTIONS(2067), 1,
      anon_sym_SLASH,
    ACTIONS(2047), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2057), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(2063), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2065), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1284), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2055), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 10,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [23824] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2069), 1,
      anon_sym_else,
    STATE(1285), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1327), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1329), 20,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_spec,
  [23870] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2073), 1,
      anon_sym_AMP,
    ACTIONS(2075), 1,
      anon_sym_CARET,
    ACTIONS(2081), 1,
      anon_sym_SLASH,
    ACTIONS(2071), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2077), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1286), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 13,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_else,
  [23926] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2081), 1,
      anon_sym_SLASH,
    ACTIONS(2071), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2077), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1287), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 14,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_else,
  [23978] = 17,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2073), 1,
      anon_sym_AMP,
    ACTIONS(2075), 1,
      anon_sym_CARET,
    ACTIONS(2081), 1,
      anon_sym_SLASH,
    ACTIONS(2083), 1,
      anon_sym_PIPE,
    ACTIONS(2085), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2087), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2093), 1,
      anon_sym_DOT_DOT,
    ACTIONS(915), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2071), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2077), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2091), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1288), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2089), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 8,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_else,
  [24046] = 16,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2073), 1,
      anon_sym_AMP,
    ACTIONS(2075), 1,
      anon_sym_CARET,
    ACTIONS(2081), 1,
      anon_sym_SLASH,
    ACTIONS(2083), 1,
      anon_sym_PIPE,
    ACTIONS(2087), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2093), 1,
      anon_sym_DOT_DOT,
    ACTIONS(915), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2071), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2077), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2091), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1289), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2089), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 9,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_LBRACK,
      anon_sym_else,
  [24112] = 15,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2073), 1,
      anon_sym_AMP,
    ACTIONS(2075), 1,
      anon_sym_CARET,
    ACTIONS(2081), 1,
      anon_sym_SLASH,
    ACTIONS(2083), 1,
      anon_sym_PIPE,
    ACTIONS(2093), 1,
      anon_sym_DOT_DOT,
    ACTIONS(915), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2071), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2077), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2091), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1290), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2089), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 10,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LBRACK,
      anon_sym_else,
  [24176] = 13,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2073), 1,
      anon_sym_AMP,
    ACTIONS(2075), 1,
      anon_sym_CARET,
    ACTIONS(2081), 1,
      anon_sym_SLASH,
    ACTIONS(2083), 1,
      anon_sym_PIPE,
    ACTIONS(2093), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2071), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2077), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1291), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 12,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LBRACK,
      anon_sym_else,
  [24236] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2073), 1,
      anon_sym_AMP,
    ACTIONS(2075), 1,
      anon_sym_CARET,
    ACTIONS(2081), 1,
      anon_sym_SLASH,
    ACTIONS(2083), 1,
      anon_sym_PIPE,
    ACTIONS(2071), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2077), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1292), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 13,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_else,
  [24294] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2073), 1,
      anon_sym_AMP,
    ACTIONS(2081), 1,
      anon_sym_SLASH,
    ACTIONS(2071), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2077), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1293), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 14,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_else,
  [24348] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2081), 1,
      anon_sym_SLASH,
    ACTIONS(2071), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1294), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 16,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LBRACK,
      anon_sym_else,
  [24398] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2081), 1,
      anon_sym_SLASH,
    ACTIONS(2071), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    STATE(1295), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 18,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_LBRACK,
      anon_sym_else,
  [24446] = 18,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2073), 1,
      anon_sym_AMP,
    ACTIONS(2075), 1,
      anon_sym_CARET,
    ACTIONS(2081), 1,
      anon_sym_SLASH,
    ACTIONS(2083), 1,
      anon_sym_PIPE,
    ACTIONS(2085), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2087), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2093), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1039), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2071), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2077), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2091), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(2095), 2,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
    STATE(1296), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2089), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(1041), 6,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_else,
  [24516] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2051), 1,
      anon_sym_AMP,
    ACTIONS(2061), 1,
      anon_sym_CARET,
    ACTIONS(2067), 1,
      anon_sym_SLASH,
    ACTIONS(2047), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2063), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2065), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1297), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_PIPE,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 14,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [24572] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2067), 1,
      anon_sym_SLASH,
    ACTIONS(2047), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2063), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2065), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1298), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 15,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [24624] = 17,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(915), 1,
      anon_sym_DOT,
    ACTIONS(2049), 1,
      anon_sym_PIPE,
    ACTIONS(2051), 1,
      anon_sym_AMP,
    ACTIONS(2053), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2059), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2061), 1,
      anon_sym_CARET,
    ACTIONS(2067), 1,
      anon_sym_SLASH,
    ACTIONS(2097), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2047), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2057), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(2063), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2065), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1299), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2055), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 9,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [24692] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2099), 1,
      anon_sym_else,
    STATE(1300), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1327), 9,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1329), 19,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_as,
  [24738] = 15,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(915), 1,
      anon_sym_DOT,
    ACTIONS(2049), 1,
      anon_sym_PIPE,
    ACTIONS(2051), 1,
      anon_sym_AMP,
    ACTIONS(2059), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2061), 1,
      anon_sym_CARET,
    ACTIONS(2067), 1,
      anon_sym_SLASH,
    ACTIONS(2047), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2057), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(2063), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2065), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1301), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2055), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 11,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [24802] = 13,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2049), 1,
      anon_sym_PIPE,
    ACTIONS(2051), 1,
      anon_sym_AMP,
    ACTIONS(2059), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2061), 1,
      anon_sym_CARET,
    ACTIONS(2067), 1,
      anon_sym_SLASH,
    ACTIONS(2047), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2063), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2065), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1302), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 5,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 13,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [24862] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2049), 1,
      anon_sym_PIPE,
    ACTIONS(2051), 1,
      anon_sym_AMP,
    ACTIONS(2061), 1,
      anon_sym_CARET,
    ACTIONS(2067), 1,
      anon_sym_SLASH,
    ACTIONS(2047), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2063), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2065), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1303), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 5,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 14,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [24920] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2051), 1,
      anon_sym_AMP,
    ACTIONS(2067), 1,
      anon_sym_SLASH,
    ACTIONS(2047), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2063), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2065), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1304), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_PIPE,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 15,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [24974] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2067), 1,
      anon_sym_SLASH,
    ACTIONS(2047), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2065), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1305), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 17,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [25024] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2067), 1,
      anon_sym_SLASH,
    ACTIONS(2047), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    STATE(1306), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 19,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [25072] = 18,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1039), 1,
      anon_sym_DOT,
    ACTIONS(2049), 1,
      anon_sym_PIPE,
    ACTIONS(2051), 1,
      anon_sym_AMP,
    ACTIONS(2053), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2059), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2061), 1,
      anon_sym_CARET,
    ACTIONS(2067), 1,
      anon_sym_SLASH,
    ACTIONS(2097), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2047), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2057), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(2063), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2065), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2101), 2,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
    STATE(1307), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2055), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(1041), 7,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
  [25142] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2103), 1,
      anon_sym_EQ,
    STATE(1308), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1047), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1049), 20,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [25188] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2107), 1,
      anon_sym_AMP,
    ACTIONS(2109), 1,
      anon_sym_CARET,
    ACTIONS(2115), 1,
      anon_sym_SLASH,
    ACTIONS(2105), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2111), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2113), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1309), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 13,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [25244] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2115), 1,
      anon_sym_SLASH,
    ACTIONS(2105), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2111), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2113), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1310), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 14,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [25296] = 17,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2107), 1,
      anon_sym_AMP,
    ACTIONS(2109), 1,
      anon_sym_CARET,
    ACTIONS(2115), 1,
      anon_sym_SLASH,
    ACTIONS(2117), 1,
      anon_sym_PIPE,
    ACTIONS(2119), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2121), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2127), 1,
      anon_sym_DOT_DOT,
    ACTIONS(915), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2105), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2111), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2113), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2125), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1311), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2123), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [25364] = 16,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2107), 1,
      anon_sym_AMP,
    ACTIONS(2109), 1,
      anon_sym_CARET,
    ACTIONS(2115), 1,
      anon_sym_SLASH,
    ACTIONS(2117), 1,
      anon_sym_PIPE,
    ACTIONS(2121), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2127), 1,
      anon_sym_DOT_DOT,
    ACTIONS(915), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2105), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2111), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2113), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2125), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1312), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2123), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 9,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [25430] = 15,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2107), 1,
      anon_sym_AMP,
    ACTIONS(2109), 1,
      anon_sym_CARET,
    ACTIONS(2115), 1,
      anon_sym_SLASH,
    ACTIONS(2117), 1,
      anon_sym_PIPE,
    ACTIONS(2127), 1,
      anon_sym_DOT_DOT,
    ACTIONS(915), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2105), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2111), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2113), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2125), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1313), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2123), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 10,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [25494] = 13,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2107), 1,
      anon_sym_AMP,
    ACTIONS(2109), 1,
      anon_sym_CARET,
    ACTIONS(2115), 1,
      anon_sym_SLASH,
    ACTIONS(2117), 1,
      anon_sym_PIPE,
    ACTIONS(2127), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2105), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2111), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2113), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1314), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 12,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [25554] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2107), 1,
      anon_sym_AMP,
    ACTIONS(2109), 1,
      anon_sym_CARET,
    ACTIONS(2115), 1,
      anon_sym_SLASH,
    ACTIONS(2117), 1,
      anon_sym_PIPE,
    ACTIONS(2105), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2111), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2113), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1315), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 13,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [25612] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2107), 1,
      anon_sym_AMP,
    ACTIONS(2115), 1,
      anon_sym_SLASH,
    ACTIONS(2105), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2111), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2113), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1316), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 14,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [25666] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2115), 1,
      anon_sym_SLASH,
    ACTIONS(2105), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2113), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1317), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 16,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [25716] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2115), 1,
      anon_sym_SLASH,
    ACTIONS(2105), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    STATE(1318), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 18,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [25764] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2129), 1,
      anon_sym_EQ,
    STATE(1319), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1047), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1049), 20,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_else,
  [25810] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2131), 1,
      anon_sym_else,
    STATE(1320), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1327), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1329), 20,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_spec,
  [25856] = 18,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2107), 1,
      anon_sym_AMP,
    ACTIONS(2109), 1,
      anon_sym_CARET,
    ACTIONS(2115), 1,
      anon_sym_SLASH,
    ACTIONS(2117), 1,
      anon_sym_PIPE,
    ACTIONS(2119), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2121), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2127), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1039), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2105), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2111), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2113), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2125), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(2133), 2,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
    STATE(1321), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2123), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(1041), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [25926] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2137), 1,
      anon_sym_AMP,
    ACTIONS(2139), 1,
      anon_sym_CARET,
    ACTIONS(2145), 1,
      anon_sym_SLASH,
    ACTIONS(2135), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2141), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2143), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1322), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_PIPE,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 14,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [25982] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2145), 1,
      anon_sym_SLASH,
    ACTIONS(2135), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2141), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2143), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1323), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [26034] = 17,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(915), 1,
      anon_sym_DOT,
    ACTIONS(2137), 1,
      anon_sym_AMP,
    ACTIONS(2139), 1,
      anon_sym_CARET,
    ACTIONS(2145), 1,
      anon_sym_SLASH,
    ACTIONS(2147), 1,
      anon_sym_PIPE,
    ACTIONS(2149), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2151), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2157), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2135), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2141), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2143), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2155), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1324), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2153), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 9,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [26102] = 16,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(915), 1,
      anon_sym_DOT,
    ACTIONS(2137), 1,
      anon_sym_AMP,
    ACTIONS(2139), 1,
      anon_sym_CARET,
    ACTIONS(2145), 1,
      anon_sym_SLASH,
    ACTIONS(2147), 1,
      anon_sym_PIPE,
    ACTIONS(2151), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2157), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2135), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2141), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2143), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2155), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1325), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2153), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 10,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [26168] = 15,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(915), 1,
      anon_sym_DOT,
    ACTIONS(2137), 1,
      anon_sym_AMP,
    ACTIONS(2139), 1,
      anon_sym_CARET,
    ACTIONS(2145), 1,
      anon_sym_SLASH,
    ACTIONS(2147), 1,
      anon_sym_PIPE,
    ACTIONS(2157), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2135), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2141), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2143), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2155), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1326), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2153), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 11,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [26232] = 13,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2137), 1,
      anon_sym_AMP,
    ACTIONS(2139), 1,
      anon_sym_CARET,
    ACTIONS(2145), 1,
      anon_sym_SLASH,
    ACTIONS(2147), 1,
      anon_sym_PIPE,
    ACTIONS(2157), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2135), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2141), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2143), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1327), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 5,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 13,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [26292] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2137), 1,
      anon_sym_AMP,
    ACTIONS(2139), 1,
      anon_sym_CARET,
    ACTIONS(2145), 1,
      anon_sym_SLASH,
    ACTIONS(2147), 1,
      anon_sym_PIPE,
    ACTIONS(2135), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2141), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2143), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1328), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 5,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 14,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [26350] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2137), 1,
      anon_sym_AMP,
    ACTIONS(2145), 1,
      anon_sym_SLASH,
    ACTIONS(2135), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2141), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2143), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1329), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_PIPE,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [26404] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2145), 1,
      anon_sym_SLASH,
    ACTIONS(2135), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2143), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1330), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 17,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [26454] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2145), 1,
      anon_sym_SLASH,
    ACTIONS(2135), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    STATE(1331), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 19,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [26502] = 18,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1039), 1,
      anon_sym_DOT,
    ACTIONS(2137), 1,
      anon_sym_AMP,
    ACTIONS(2139), 1,
      anon_sym_CARET,
    ACTIONS(2145), 1,
      anon_sym_SLASH,
    ACTIONS(2147), 1,
      anon_sym_PIPE,
    ACTIONS(2149), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2151), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2157), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2135), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2141), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2143), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2155), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(2159), 2,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
    STATE(1332), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2153), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(1041), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
      anon_sym_spec,
  [26572] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2161), 1,
      anon_sym_else,
    STATE(1333), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1327), 9,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1329), 19,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
  [26618] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2163), 1,
      anon_sym_EQ,
    STATE(1334), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1047), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1049), 20,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [26664] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2165), 1,
      anon_sym_EQ,
    STATE(1335), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1047), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1049), 20,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_else,
  [26710] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2173), 1,
      anon_sym_SLASH,
    ACTIONS(2167), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2169), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2171), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1336), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 14,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [26761] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2177), 1,
      anon_sym_AMP,
    ACTIONS(2179), 1,
      anon_sym_CARET,
    ACTIONS(2185), 1,
      anon_sym_SLASH,
    ACTIONS(2175), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2181), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2183), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1337), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_PIPE,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 13,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_else,
  [26816] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2185), 1,
      anon_sym_SLASH,
    ACTIONS(2175), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2181), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2183), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1338), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 14,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_else,
  [26867] = 17,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(915), 1,
      anon_sym_DOT,
    ACTIONS(2177), 1,
      anon_sym_AMP,
    ACTIONS(2179), 1,
      anon_sym_CARET,
    ACTIONS(2185), 1,
      anon_sym_SLASH,
    ACTIONS(2187), 1,
      anon_sym_PIPE,
    ACTIONS(2189), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2191), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2197), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2175), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2181), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2183), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2195), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1339), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2193), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 8,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_else,
  [26934] = 16,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(915), 1,
      anon_sym_DOT,
    ACTIONS(2177), 1,
      anon_sym_AMP,
    ACTIONS(2179), 1,
      anon_sym_CARET,
    ACTIONS(2185), 1,
      anon_sym_SLASH,
    ACTIONS(2187), 1,
      anon_sym_PIPE,
    ACTIONS(2191), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2197), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2175), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2181), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2183), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2195), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1340), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2193), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 9,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_LBRACK,
      anon_sym_else,
  [26999] = 15,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(915), 1,
      anon_sym_DOT,
    ACTIONS(2177), 1,
      anon_sym_AMP,
    ACTIONS(2179), 1,
      anon_sym_CARET,
    ACTIONS(2185), 1,
      anon_sym_SLASH,
    ACTIONS(2187), 1,
      anon_sym_PIPE,
    ACTIONS(2197), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2175), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2181), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2183), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2195), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1341), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2193), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 10,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LBRACK,
      anon_sym_else,
  [27062] = 13,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2177), 1,
      anon_sym_AMP,
    ACTIONS(2179), 1,
      anon_sym_CARET,
    ACTIONS(2185), 1,
      anon_sym_SLASH,
    ACTIONS(2187), 1,
      anon_sym_PIPE,
    ACTIONS(2197), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2175), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2181), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2183), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1342), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 5,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 12,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LBRACK,
      anon_sym_else,
  [27121] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2177), 1,
      anon_sym_AMP,
    ACTIONS(2179), 1,
      anon_sym_CARET,
    ACTIONS(2185), 1,
      anon_sym_SLASH,
    ACTIONS(2187), 1,
      anon_sym_PIPE,
    ACTIONS(2175), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2181), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2183), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1343), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 5,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 13,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_else,
  [27178] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2177), 1,
      anon_sym_AMP,
    ACTIONS(2185), 1,
      anon_sym_SLASH,
    ACTIONS(2175), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2181), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2183), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1344), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_PIPE,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 14,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_else,
  [27231] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2173), 1,
      anon_sym_SLASH,
    ACTIONS(2199), 1,
      anon_sym_AMP,
    ACTIONS(2201), 1,
      anon_sym_CARET,
    ACTIONS(2167), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2169), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2171), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1345), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_PIPE,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 13,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [27286] = 17,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(915), 1,
      anon_sym_DOT,
    ACTIONS(2173), 1,
      anon_sym_SLASH,
    ACTIONS(2199), 1,
      anon_sym_AMP,
    ACTIONS(2201), 1,
      anon_sym_CARET,
    ACTIONS(2203), 1,
      anon_sym_PIPE,
    ACTIONS(2205), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2207), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2213), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2167), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2169), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2171), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2211), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1346), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2209), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [27353] = 16,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(915), 1,
      anon_sym_DOT,
    ACTIONS(2173), 1,
      anon_sym_SLASH,
    ACTIONS(2199), 1,
      anon_sym_AMP,
    ACTIONS(2201), 1,
      anon_sym_CARET,
    ACTIONS(2203), 1,
      anon_sym_PIPE,
    ACTIONS(2207), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2213), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2167), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2169), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2171), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2211), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1347), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2209), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 9,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [27418] = 15,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(915), 1,
      anon_sym_DOT,
    ACTIONS(2173), 1,
      anon_sym_SLASH,
    ACTIONS(2199), 1,
      anon_sym_AMP,
    ACTIONS(2201), 1,
      anon_sym_CARET,
    ACTIONS(2203), 1,
      anon_sym_PIPE,
    ACTIONS(2213), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2167), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2169), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2171), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2211), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1348), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2209), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 10,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [27481] = 13,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2173), 1,
      anon_sym_SLASH,
    ACTIONS(2199), 1,
      anon_sym_AMP,
    ACTIONS(2201), 1,
      anon_sym_CARET,
    ACTIONS(2203), 1,
      anon_sym_PIPE,
    ACTIONS(2213), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2167), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2169), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2171), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1349), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 5,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 12,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [27540] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2173), 1,
      anon_sym_SLASH,
    ACTIONS(2199), 1,
      anon_sym_AMP,
    ACTIONS(2201), 1,
      anon_sym_CARET,
    ACTIONS(2203), 1,
      anon_sym_PIPE,
    ACTIONS(2167), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2169), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2171), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1350), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 5,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 13,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [27597] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2173), 1,
      anon_sym_SLASH,
    ACTIONS(2199), 1,
      anon_sym_AMP,
    ACTIONS(2167), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2169), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2171), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1351), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_PIPE,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 14,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [27650] = 18,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1039), 1,
      anon_sym_DOT,
    ACTIONS(2173), 1,
      anon_sym_SLASH,
    ACTIONS(2199), 1,
      anon_sym_AMP,
    ACTIONS(2201), 1,
      anon_sym_CARET,
    ACTIONS(2203), 1,
      anon_sym_PIPE,
    ACTIONS(2205), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2207), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2213), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2167), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2169), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2171), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2211), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(2215), 2,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
    STATE(1352), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2209), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(1041), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [27719] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2173), 1,
      anon_sym_SLASH,
    ACTIONS(2167), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    STATE(1353), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 18,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [27766] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2185), 1,
      anon_sym_SLASH,
    ACTIONS(2175), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2183), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1354), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 16,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LBRACK,
      anon_sym_else,
  [27815] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2185), 1,
      anon_sym_SLASH,
    ACTIONS(2175), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    STATE(1355), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 18,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_LBRACK,
      anon_sym_else,
  [27862] = 18,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1039), 1,
      anon_sym_DOT,
    ACTIONS(2177), 1,
      anon_sym_AMP,
    ACTIONS(2179), 1,
      anon_sym_CARET,
    ACTIONS(2185), 1,
      anon_sym_SLASH,
    ACTIONS(2187), 1,
      anon_sym_PIPE,
    ACTIONS(2189), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2191), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2197), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2175), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2181), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2183), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2195), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(2217), 2,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
    STATE(1356), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2193), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(1041), 6,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_else,
  [27931] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2221), 1,
      anon_sym_AMP,
    ACTIONS(2223), 1,
      anon_sym_CARET,
    ACTIONS(2229), 1,
      anon_sym_SLASH,
    ACTIONS(2219), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2225), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2227), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1357), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 12,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [27986] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2229), 1,
      anon_sym_SLASH,
    ACTIONS(2219), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2225), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2227), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1358), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 13,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [28037] = 17,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2221), 1,
      anon_sym_AMP,
    ACTIONS(2223), 1,
      anon_sym_CARET,
    ACTIONS(2229), 1,
      anon_sym_SLASH,
    ACTIONS(2231), 1,
      anon_sym_PIPE,
    ACTIONS(2233), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2235), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2241), 1,
      anon_sym_DOT_DOT,
    ACTIONS(915), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2219), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2225), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2227), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2239), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1359), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2237), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 7,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [28104] = 16,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2221), 1,
      anon_sym_AMP,
    ACTIONS(2223), 1,
      anon_sym_CARET,
    ACTIONS(2229), 1,
      anon_sym_SLASH,
    ACTIONS(2231), 1,
      anon_sym_PIPE,
    ACTIONS(2235), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2241), 1,
      anon_sym_DOT_DOT,
    ACTIONS(915), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2219), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2225), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2227), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2239), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1360), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2237), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 8,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [28169] = 15,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2221), 1,
      anon_sym_AMP,
    ACTIONS(2223), 1,
      anon_sym_CARET,
    ACTIONS(2229), 1,
      anon_sym_SLASH,
    ACTIONS(2231), 1,
      anon_sym_PIPE,
    ACTIONS(2241), 1,
      anon_sym_DOT_DOT,
    ACTIONS(915), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2219), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2225), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2227), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2239), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1361), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2237), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 9,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [28232] = 13,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2221), 1,
      anon_sym_AMP,
    ACTIONS(2223), 1,
      anon_sym_CARET,
    ACTIONS(2229), 1,
      anon_sym_SLASH,
    ACTIONS(2231), 1,
      anon_sym_PIPE,
    ACTIONS(2241), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2219), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2225), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2227), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1362), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 11,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [28291] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2221), 1,
      anon_sym_AMP,
    ACTIONS(2223), 1,
      anon_sym_CARET,
    ACTIONS(2229), 1,
      anon_sym_SLASH,
    ACTIONS(2231), 1,
      anon_sym_PIPE,
    ACTIONS(2219), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2225), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2227), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1363), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 12,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [28348] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2221), 1,
      anon_sym_AMP,
    ACTIONS(2229), 1,
      anon_sym_SLASH,
    ACTIONS(2219), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2225), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2227), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1364), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 13,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [28401] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2243), 1,
      anon_sym_EQ,
    STATE(1365), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1047), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1049), 19,
      anon_sym_STAR,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [28446] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2229), 1,
      anon_sym_SLASH,
    ACTIONS(2219), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2227), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1366), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 15,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [28495] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2229), 1,
      anon_sym_SLASH,
    ACTIONS(2219), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    STATE(1367), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 17,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [28542] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2245), 1,
      anon_sym_else,
    STATE(1368), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1327), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1329), 19,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_as,
  [28587] = 18,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2221), 1,
      anon_sym_AMP,
    ACTIONS(2223), 1,
      anon_sym_CARET,
    ACTIONS(2229), 1,
      anon_sym_SLASH,
    ACTIONS(2231), 1,
      anon_sym_PIPE,
    ACTIONS(2233), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2235), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2241), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1039), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2219), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2225), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2227), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2239), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(2247), 2,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
    STATE(1369), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2237), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(1041), 5,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [28656] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2249), 1,
      anon_sym_EQ,
    STATE(1370), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1047), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1049), 19,
      anon_sym_STAR,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [28701] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2251), 1,
      anon_sym_else,
    STATE(1371), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1327), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1329), 19,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
  [28746] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2253), 1,
      anon_sym_else,
    STATE(1372), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1327), 9,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1329), 18,
      anon_sym_STAR,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_spec,
      anon_sym_SEMI,
  [28791] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2173), 1,
      anon_sym_SLASH,
    ACTIONS(2167), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2171), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1373), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 16,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LBRACK,
      anon_sym_as,
      anon_sym_else,
  [28840] = 16,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(915), 1,
      anon_sym_DOT,
    ACTIONS(2257), 1,
      anon_sym_PIPE,
    ACTIONS(2259), 1,
      anon_sym_AMP,
    ACTIONS(2261), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2267), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2269), 1,
      anon_sym_CARET,
    ACTIONS(2275), 1,
      anon_sym_SLASH,
    ACTIONS(2255), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2265), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(2271), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2273), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1374), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2263), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 8,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [28904] = 15,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2277), 1,
      sym_identifier,
    ACTIONS(2279), 1,
      anon_sym_COMMA,
    ACTIONS(2281), 1,
      anon_sym_RBRACK,
    STATE(859), 1,
      sym_number_type,
    STATE(1713), 1,
      sym_discouraged_name,
    STATE(1810), 1,
      sym_var_name,
    STATE(1861), 1,
      sym__spec_pragma_prop,
    STATE(1375), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [28966] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(927), 1,
      anon_sym_DOT,
    STATE(1376), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(929), 4,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
    ACTIONS(999), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
    ACTIONS(1001), 14,
      anon_sym_STAR,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
  [29012] = 15,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(915), 1,
      anon_sym_DOT,
    ACTIONS(2257), 1,
      anon_sym_PIPE,
    ACTIONS(2259), 1,
      anon_sym_AMP,
    ACTIONS(2267), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2269), 1,
      anon_sym_CARET,
    ACTIONS(2275), 1,
      anon_sym_SLASH,
    ACTIONS(2255), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2265), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(2271), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2273), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1377), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2263), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 9,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [29074] = 13,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2257), 1,
      anon_sym_PIPE,
    ACTIONS(2259), 1,
      anon_sym_AMP,
    ACTIONS(2267), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2269), 1,
      anon_sym_CARET,
    ACTIONS(2275), 1,
      anon_sym_SLASH,
    ACTIONS(2255), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2271), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2273), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1378), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 5,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 11,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [29132] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2257), 1,
      anon_sym_PIPE,
    ACTIONS(2259), 1,
      anon_sym_AMP,
    ACTIONS(2269), 1,
      anon_sym_CARET,
    ACTIONS(2275), 1,
      anon_sym_SLASH,
    ACTIONS(2255), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2271), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2273), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1379), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 5,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 12,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [29188] = 22,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1880), 1,
      anon_sym_friend,
    ACTIONS(1886), 1,
      anon_sym_invariant,
    ACTIONS(1888), 1,
      anon_sym_fun,
    ACTIONS(1892), 1,
      anon_sym_public,
    ACTIONS(1894), 1,
      anon_sym_use,
    ACTIONS(1896), 1,
      anon_sym_const,
    ACTIONS(1898), 1,
      anon_sym_inline,
    ACTIONS(1900), 1,
      anon_sym_struct,
    ACTIONS(1902), 1,
      anon_sym_enum,
    ACTIONS(2283), 1,
      anon_sym_spec,
    STATE(1517), 1,
      aux_sym_declaration_repeat1,
    STATE(1671), 1,
      sym_module_member_modifier,
    STATE(1686), 1,
      sym_visibility,
    STATE(1868), 1,
      sym__struct_signature,
    STATE(1948), 1,
      sym__function_signature,
    STATE(2063), 1,
      sym__enum_signature,
    ACTIONS(1890), 2,
      anon_sym_native,
      anon_sym_entry,
    STATE(1380), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1577), 2,
      sym_struct_decl,
      sym_enum_decl,
    STATE(1584), 7,
      sym_spec_block,
      sym_spec_invariant,
      sym_use_decl,
      sym_friend_decl,
      sym_constant_decl,
      sym_function_decl,
      sym__struct_or_enum_decl,
  [29264] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2259), 1,
      anon_sym_AMP,
    ACTIONS(2275), 1,
      anon_sym_SLASH,
    ACTIONS(2255), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2271), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2273), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1381), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_PIPE,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 13,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [29316] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2275), 1,
      anon_sym_SLASH,
    ACTIONS(2255), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2273), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1382), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 15,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [29364] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2275), 1,
      anon_sym_SLASH,
    ACTIONS(2255), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    STATE(1383), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 17,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [29410] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2287), 1,
      anon_sym_AMP,
    ACTIONS(2289), 1,
      anon_sym_CARET,
    ACTIONS(2295), 1,
      anon_sym_SLASH,
    ACTIONS(2285), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2291), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2293), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1384), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 11,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
  [29464] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2295), 1,
      anon_sym_SLASH,
    ACTIONS(2285), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2291), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2293), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1385), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 12,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
  [29514] = 17,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2287), 1,
      anon_sym_AMP,
    ACTIONS(2289), 1,
      anon_sym_CARET,
    ACTIONS(2295), 1,
      anon_sym_SLASH,
    ACTIONS(2297), 1,
      anon_sym_PIPE,
    ACTIONS(2299), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2301), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2307), 1,
      anon_sym_DOT_DOT,
    ACTIONS(915), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2285), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2291), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2293), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2305), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1386), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2303), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 6,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
  [29580] = 16,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2287), 1,
      anon_sym_AMP,
    ACTIONS(2289), 1,
      anon_sym_CARET,
    ACTIONS(2295), 1,
      anon_sym_SLASH,
    ACTIONS(2297), 1,
      anon_sym_PIPE,
    ACTIONS(2301), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2307), 1,
      anon_sym_DOT_DOT,
    ACTIONS(915), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2285), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2291), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2293), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2305), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1387), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2303), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 7,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
  [29644] = 15,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2287), 1,
      anon_sym_AMP,
    ACTIONS(2289), 1,
      anon_sym_CARET,
    ACTIONS(2295), 1,
      anon_sym_SLASH,
    ACTIONS(2297), 1,
      anon_sym_PIPE,
    ACTIONS(2307), 1,
      anon_sym_DOT_DOT,
    ACTIONS(915), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2285), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2291), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2293), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2305), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    STATE(1388), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2303), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 8,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
  [29706] = 13,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2287), 1,
      anon_sym_AMP,
    ACTIONS(2289), 1,
      anon_sym_CARET,
    ACTIONS(2295), 1,
      anon_sym_SLASH,
    ACTIONS(2297), 1,
      anon_sym_PIPE,
    ACTIONS(2307), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2285), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2291), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2293), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1389), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 10,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
  [29764] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2287), 1,
      anon_sym_AMP,
    ACTIONS(2289), 1,
      anon_sym_CARET,
    ACTIONS(2295), 1,
      anon_sym_SLASH,
    ACTIONS(2297), 1,
      anon_sym_PIPE,
    ACTIONS(2285), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2291), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2293), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1390), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 11,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
  [29820] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2287), 1,
      anon_sym_AMP,
    ACTIONS(2295), 1,
      anon_sym_SLASH,
    ACTIONS(2285), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2291), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2293), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1391), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 12,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
  [29872] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2309), 1,
      anon_sym_EQ,
    STATE(1392), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1047), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1049), 18,
      anon_sym_STAR,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
  [29916] = 18,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1039), 1,
      anon_sym_DOT,
    ACTIONS(2257), 1,
      anon_sym_PIPE,
    ACTIONS(2259), 1,
      anon_sym_AMP,
    ACTIONS(2261), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2267), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2269), 1,
      anon_sym_CARET,
    ACTIONS(2275), 1,
      anon_sym_SLASH,
    ACTIONS(2313), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2255), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2265), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(2271), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2273), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2311), 2,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
    STATE(1393), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2263), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(1041), 5,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [29984] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2295), 1,
      anon_sym_SLASH,
    ACTIONS(2285), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2293), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1394), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 14,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
  [30032] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2295), 1,
      anon_sym_SLASH,
    ACTIONS(2285), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    STATE(1395), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 16,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
  [30078] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2315), 1,
      anon_sym_EQ,
    STATE(1396), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1047), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1049), 18,
      anon_sym_STAR,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
  [30122] = 18,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2287), 1,
      anon_sym_AMP,
    ACTIONS(2289), 1,
      anon_sym_CARET,
    ACTIONS(2295), 1,
      anon_sym_SLASH,
    ACTIONS(2297), 1,
      anon_sym_PIPE,
    ACTIONS(2299), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2301), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2307), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1039), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2285), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2291), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2293), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(2305), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(2317), 2,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
    STATE(1397), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1041), 4,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
    ACTIONS(2303), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
  [30190] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2319), 1,
      anon_sym_else,
    STATE(1398), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1327), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1329), 18,
      anon_sym_STAR,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_spec,
      anon_sym_SEMI,
  [30234] = 15,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2277), 1,
      sym_identifier,
    ACTIONS(2321), 1,
      anon_sym_COMMA,
    ACTIONS(2323), 1,
      anon_sym_SEMI,
    STATE(859), 1,
      sym_number_type,
    STATE(1713), 1,
      sym_discouraged_name,
    STATE(1810), 1,
      sym_var_name,
    STATE(1913), 1,
      sym__spec_pragma_prop,
    STATE(1399), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [30296] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2325), 1,
      anon_sym_else,
    STATE(1400), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1327), 9,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1329), 17,
      anon_sym_STAR,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_SEMI,
  [30340] = 15,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2277), 1,
      sym_identifier,
    ACTIONS(2327), 1,
      anon_sym_COMMA,
    ACTIONS(2329), 1,
      anon_sym_RBRACE,
    STATE(859), 1,
      sym_number_type,
    STATE(1713), 1,
      sym_discouraged_name,
    STATE(1942), 1,
      sym_var_name,
    STATE(1943), 1,
      sym_bind_field,
    STATE(1401), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [30402] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2275), 1,
      anon_sym_SLASH,
    ACTIONS(2255), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2271), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2273), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1402), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 13,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [30452] = 17,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(915), 1,
      anon_sym_DOT,
    ACTIONS(2257), 1,
      anon_sym_PIPE,
    ACTIONS(2259), 1,
      anon_sym_AMP,
    ACTIONS(2261), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2267), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2269), 1,
      anon_sym_CARET,
    ACTIONS(2275), 1,
      anon_sym_SLASH,
    ACTIONS(2313), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2255), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2265), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(2271), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2273), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1403), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2263), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 7,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [30518] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2259), 1,
      anon_sym_AMP,
    ACTIONS(2269), 1,
      anon_sym_CARET,
    ACTIONS(2275), 1,
      anon_sym_SLASH,
    ACTIONS(2255), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2271), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2273), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1404), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_PIPE,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 12,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_spec,
      anon_sym_SEMI,
  [30572] = 18,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1039), 1,
      anon_sym_DOT,
    ACTIONS(2333), 1,
      anon_sym_PIPE,
    ACTIONS(2335), 1,
      anon_sym_AMP,
    ACTIONS(2339), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2341), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2347), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2349), 1,
      anon_sym_CARET,
    ACTIONS(2355), 1,
      anon_sym_SLASH,
    ACTIONS(2331), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2337), 2,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
    ACTIONS(2345), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(2351), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2353), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1405), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1041), 4,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
    ACTIONS(2343), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
  [30639] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2355), 1,
      anon_sym_SLASH,
    ACTIONS(2331), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2351), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2353), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1406), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 12,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
  [30688] = 17,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(915), 1,
      anon_sym_DOT,
    ACTIONS(2333), 1,
      anon_sym_PIPE,
    ACTIONS(2335), 1,
      anon_sym_AMP,
    ACTIONS(2339), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2341), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2347), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2349), 1,
      anon_sym_CARET,
    ACTIONS(2355), 1,
      anon_sym_SLASH,
    ACTIONS(2331), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2345), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(2351), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2353), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1407), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2343), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 6,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
  [30753] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2335), 1,
      anon_sym_AMP,
    ACTIONS(2355), 1,
      anon_sym_SLASH,
    ACTIONS(2331), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2351), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2353), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1408), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_PIPE,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 12,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
  [30804] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2357), 1,
      anon_sym_else,
    STATE(1409), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1327), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
      anon_sym_DOT,
    ACTIONS(1329), 17,
      anon_sym_STAR,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_SEMI,
  [30847] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(927), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    STATE(1410), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(929), 3,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
    ACTIONS(999), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
    ACTIONS(1001), 14,
      anon_sym_STAR,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
  [30892] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2355), 1,
      anon_sym_SLASH,
    ACTIONS(2331), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2353), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1411), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 14,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
  [30939] = 16,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(915), 1,
      anon_sym_DOT,
    ACTIONS(2333), 1,
      anon_sym_PIPE,
    ACTIONS(2335), 1,
      anon_sym_AMP,
    ACTIONS(2341), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2347), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2349), 1,
      anon_sym_CARET,
    ACTIONS(2355), 1,
      anon_sym_SLASH,
    ACTIONS(2331), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2345), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(2351), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2353), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1412), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2343), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 7,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
  [31002] = 14,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2277), 1,
      sym_identifier,
    ACTIONS(2359), 1,
      anon_sym_RBRACE,
    STATE(859), 1,
      sym_number_type,
    STATE(1713), 1,
      sym_discouraged_name,
    STATE(1942), 1,
      sym_var_name,
    STATE(2012), 1,
      sym_bind_field,
    STATE(1413), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [31061] = 15,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(915), 1,
      anon_sym_DOT,
    ACTIONS(2333), 1,
      anon_sym_PIPE,
    ACTIONS(2335), 1,
      anon_sym_AMP,
    ACTIONS(2347), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2349), 1,
      anon_sym_CARET,
    ACTIONS(2355), 1,
      anon_sym_SLASH,
    ACTIONS(2331), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2345), 2,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
    ACTIONS(2351), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2353), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1414), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2343), 4,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
    ACTIONS(917), 8,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
  [31122] = 14,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2277), 1,
      sym_identifier,
    ACTIONS(2361), 1,
      anon_sym_SEMI,
    STATE(859), 1,
      sym_number_type,
    STATE(1713), 1,
      sym_discouraged_name,
    STATE(1810), 1,
      sym_var_name,
    STATE(1979), 1,
      sym__spec_pragma_prop,
    STATE(1415), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [31181] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2355), 1,
      anon_sym_SLASH,
    ACTIONS(2331), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    STATE(1416), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 16,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
  [31226] = 14,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2277), 1,
      sym_identifier,
    ACTIONS(2363), 1,
      anon_sym_RBRACK,
    STATE(859), 1,
      sym_number_type,
    STATE(1713), 1,
      sym_discouraged_name,
    STATE(1810), 1,
      sym_var_name,
    STATE(1979), 1,
      sym__spec_pragma_prop,
    STATE(1417), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [31285] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(927), 1,
      anon_sym_DOT,
    STATE(1418), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(929), 4,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_else,
      anon_sym_spec,
    ACTIONS(999), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_SLASH,
    ACTIONS(1001), 13,
      anon_sym_STAR,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
  [31330] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2335), 1,
      anon_sym_AMP,
    ACTIONS(2349), 1,
      anon_sym_CARET,
    ACTIONS(2355), 1,
      anon_sym_SLASH,
    ACTIONS(2331), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2351), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2353), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1419), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 6,
      anon_sym_PIPE,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 11,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
  [31383] = 13,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2333), 1,
      anon_sym_PIPE,
    ACTIONS(2335), 1,
      anon_sym_AMP,
    ACTIONS(2347), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2349), 1,
      anon_sym_CARET,
    ACTIONS(2355), 1,
      anon_sym_SLASH,
    ACTIONS(2331), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2351), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2353), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1420), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 5,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 10,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
  [31440] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2333), 1,
      anon_sym_PIPE,
    ACTIONS(2335), 1,
      anon_sym_AMP,
    ACTIONS(2349), 1,
      anon_sym_CARET,
    ACTIONS(2355), 1,
      anon_sym_SLASH,
    ACTIONS(2331), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2351), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2353), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1421), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(915), 5,
      anon_sym_EQ_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
    ACTIONS(917), 11,
      anon_sym_EQ_EQ_GT,
      anon_sym_LT_EQ_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_if,
      anon_sym_else,
      anon_sym_SEMI,
  [31495] = 14,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2277), 1,
      sym_identifier,
    ACTIONS(2365), 1,
      anon_sym_RBRACE,
    STATE(859), 1,
      sym_number_type,
    STATE(1713), 1,
      sym_discouraged_name,
    STATE(1942), 1,
      sym_var_name,
    STATE(2012), 1,
      sym_bind_field,
    STATE(1422), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [31554] = 14,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2277), 1,
      sym_identifier,
    ACTIONS(2367), 1,
      anon_sym_SEMI,
    STATE(859), 1,
      sym_number_type,
    STATE(1713), 1,
      sym_discouraged_name,
    STATE(1810), 1,
      sym_var_name,
    STATE(1979), 1,
      sym__spec_pragma_prop,
    STATE(1423), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [31613] = 14,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2277), 1,
      sym_identifier,
    ACTIONS(2369), 1,
      anon_sym_RBRACK,
    STATE(859), 1,
      sym_number_type,
    STATE(1713), 1,
      sym_discouraged_name,
    STATE(1810), 1,
      sym_var_name,
    STATE(1979), 1,
      sym__spec_pragma_prop,
    STATE(1424), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [31672] = 13,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2277), 1,
      sym_identifier,
    STATE(859), 1,
      sym_number_type,
    STATE(1713), 1,
      sym_discouraged_name,
    STATE(1942), 1,
      sym_var_name,
    STATE(2012), 1,
      sym_bind_field,
    STATE(1425), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [31728] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1426), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2373), 6,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
    ACTIONS(2371), 19,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
  [31768] = 13,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2277), 1,
      sym_identifier,
    STATE(859), 1,
      sym_number_type,
    STATE(1713), 1,
      sym_discouraged_name,
    STATE(1810), 1,
      sym_var_name,
    STATE(1979), 1,
      sym__spec_pragma_prop,
    STATE(1427), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [31824] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1428), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2377), 6,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
    ACTIONS(2375), 19,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
  [31864] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2383), 1,
      anon_sym_COMMA,
    STATE(1429), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2381), 5,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_RBRACE,
    ACTIONS(2379), 19,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
  [31906] = 13,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2277), 1,
      sym_identifier,
    ACTIONS(2385), 1,
      anon_sym_post,
    STATE(859), 1,
      sym_number_type,
    STATE(1713), 1,
      sym_discouraged_name,
    STATE(2213), 1,
      sym_var_name,
    STATE(1430), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [31962] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1431), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1650), 5,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
      anon_sym_LPAREN,
      anon_sym_RBRACE,
    ACTIONS(2387), 19,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
      sym_identifier,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
      aux_sym_number_token1,
  [32001] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2277), 1,
      sym_identifier,
    STATE(859), 1,
      sym_number_type,
    STATE(1713), 1,
      sym_discouraged_name,
    STATE(2375), 1,
      sym_var_name,
    STATE(1432), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [32054] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2277), 1,
      sym_identifier,
    STATE(859), 1,
      sym_number_type,
    STATE(1713), 1,
      sym_discouraged_name,
    STATE(2220), 1,
      sym_var_name,
    STATE(1433), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(33), 3,
      anon_sym_bool,
      anon_sym_address,
      anon_sym_vector,
    STATE(860), 3,
      sym__quantifier_directive,
      sym_primitive_type,
      sym__reuseable_keywords,
    ACTIONS(31), 4,
      anon_sym_exists,
      anon_sym_forall,
      anon_sym_choose,
      anon_sym_min,
    ACTIONS(41), 4,
      anon_sym_for,
      anon_sym_while,
      anon_sym_friend,
      anon_sym_match,
    ACTIONS(25), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
  [32107] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1441), 1,
      anon_sym_RBRACE,
    ACTIONS(2389), 1,
      anon_sym_use,
    STATE(1440), 1,
      sym_use_decl,
    STATE(1434), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_spec_block_repeat1,
    ACTIONS(1439), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [32150] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1210), 1,
      anon_sym_RBRACE,
    STATE(1435), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1208), 21,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
      anon_sym_use,
  [32187] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1222), 1,
      anon_sym_RBRACE,
    STATE(1436), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1220), 21,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
      anon_sym_use,
  [32224] = 24,
    ACTIONS(11), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1888), 1,
      anon_sym_fun,
    ACTIONS(1892), 1,
      anon_sym_public,
    ACTIONS(1894), 1,
      anon_sym_use,
    ACTIONS(1896), 1,
      anon_sym_const,
    ACTIONS(1898), 1,
      anon_sym_inline,
    STATE(1532), 1,
      aux_sym_script_repeat2,
    STATE(1536), 1,
      aux_sym_attributes_repeat2,
    STATE(1588), 1,
      sym_attributes,
    STATE(1594), 1,
      aux_sym_script_repeat1,
    STATE(1605), 1,
      aux_sym_declaration_repeat1,
    STATE(1659), 1,
      sym_use_decl,
    STATE(1660), 1,
      sym__script_use_decl,
    STATE(1671), 1,
      sym_module_member_modifier,
    STATE(1686), 1,
      sym_visibility,
    STATE(1693), 1,
      sym__script_func_decl,
    STATE(1703), 1,
      sym_constant_decl,
    STATE(1717), 1,
      sym__script_constant_decl,
    STATE(1936), 1,
      sym_function_decl,
    STATE(1948), 1,
      sym__function_signature,
    ACTIONS(1890), 2,
      anon_sym_native,
      anon_sym_entry,
    STATE(1437), 2,
      sym_line_comment,
      sym_block_comment,
  [32299] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1234), 1,
      anon_sym_RBRACE,
    STATE(1438), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1232), 21,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
      anon_sym_use,
  [32336] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1189), 1,
      anon_sym_RBRACE,
    STATE(1439), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1187), 21,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
      anon_sym_use,
  [32373] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1450), 1,
      anon_sym_RBRACE,
    STATE(1440), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1448), 21,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
      anon_sym_use,
  [32410] = 24,
    ACTIONS(11), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1888), 1,
      anon_sym_fun,
    ACTIONS(1892), 1,
      anon_sym_public,
    ACTIONS(1894), 1,
      anon_sym_use,
    ACTIONS(1896), 1,
      anon_sym_const,
    ACTIONS(1898), 1,
      anon_sym_inline,
    STATE(1437), 1,
      aux_sym_script_repeat1,
    STATE(1523), 1,
      aux_sym_script_repeat2,
    STATE(1536), 1,
      aux_sym_attributes_repeat2,
    STATE(1588), 1,
      sym_attributes,
    STATE(1605), 1,
      aux_sym_declaration_repeat1,
    STATE(1659), 1,
      sym_use_decl,
    STATE(1660), 1,
      sym__script_use_decl,
    STATE(1661), 1,
      sym__script_func_decl,
    STATE(1671), 1,
      sym_module_member_modifier,
    STATE(1686), 1,
      sym_visibility,
    STATE(1703), 1,
      sym_constant_decl,
    STATE(1717), 1,
      sym__script_constant_decl,
    STATE(1936), 1,
      sym_function_decl,
    STATE(1948), 1,
      sym__function_signature,
    ACTIONS(1890), 2,
      anon_sym_native,
      anon_sym_entry,
    STATE(1441), 2,
      sym_line_comment,
      sym_block_comment,
  [32485] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1230), 1,
      anon_sym_RBRACE,
    STATE(1442), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1228), 21,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
      anon_sym_use,
  [32522] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1226), 1,
      anon_sym_RBRACE,
    STATE(1443), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1224), 21,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
      anon_sym_use,
  [32559] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1238), 1,
      anon_sym_RBRACE,
    STATE(1444), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1236), 21,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
      anon_sym_use,
  [32596] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2394), 1,
      anon_sym_RBRACE,
    STATE(1445), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2392), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [32632] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2398), 1,
      anon_sym_RBRACE,
    STATE(1446), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2396), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [32668] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2402), 1,
      anon_sym_RBRACE,
    STATE(1447), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2400), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [32704] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2406), 1,
      anon_sym_RBRACE,
    STATE(1448), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2404), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [32740] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2410), 1,
      anon_sym_RBRACE,
    STATE(1449), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2408), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [32776] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2414), 1,
      anon_sym_RBRACE,
    STATE(1450), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2412), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [32812] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2418), 1,
      anon_sym_RBRACE,
    STATE(1451), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2416), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [32848] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2422), 1,
      anon_sym_RBRACE,
    STATE(1452), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2420), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [32884] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2426), 1,
      anon_sym_RBRACE,
    STATE(1453), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2424), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [32920] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2430), 1,
      anon_sym_RBRACE,
    STATE(1454), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2428), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [32956] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2434), 1,
      anon_sym_RBRACE,
    STATE(1455), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2432), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [32992] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2438), 1,
      anon_sym_RBRACE,
    STATE(1456), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2436), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33028] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2442), 1,
      anon_sym_RBRACE,
    STATE(1457), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2440), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33064] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2446), 1,
      anon_sym_RBRACE,
    STATE(1458), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2444), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33100] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2450), 1,
      anon_sym_RBRACE,
    STATE(1459), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2448), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33136] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2454), 1,
      anon_sym_RBRACE,
    STATE(1460), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2452), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33172] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2458), 1,
      anon_sym_RBRACE,
    STATE(1461), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2456), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33208] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2462), 1,
      anon_sym_RBRACE,
    STATE(1462), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2460), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33244] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2466), 1,
      anon_sym_RBRACE,
    STATE(1463), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2464), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33280] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2470), 1,
      anon_sym_RBRACE,
    STATE(1464), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2468), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33316] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2474), 1,
      anon_sym_RBRACE,
    STATE(1465), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2472), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33352] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2478), 1,
      anon_sym_RBRACE,
    STATE(1466), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2476), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33388] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2482), 1,
      anon_sym_RBRACE,
    STATE(1467), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2480), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33424] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2486), 1,
      anon_sym_RBRACE,
    STATE(1468), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2484), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33460] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2490), 1,
      anon_sym_RBRACE,
    STATE(1469), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2488), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33496] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2494), 1,
      anon_sym_RBRACE,
    STATE(1470), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2492), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33532] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2498), 1,
      anon_sym_RBRACE,
    STATE(1471), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2496), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33568] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2502), 1,
      anon_sym_RBRACE,
    STATE(1472), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2500), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33604] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2506), 1,
      anon_sym_RBRACE,
    STATE(1473), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2504), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33640] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2510), 1,
      anon_sym_RBRACE,
    STATE(1474), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2508), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33676] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2514), 1,
      anon_sym_RBRACE,
    STATE(1475), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2512), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33712] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2518), 1,
      anon_sym_RBRACE,
    STATE(1476), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2516), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33748] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2522), 1,
      anon_sym_RBRACE,
    STATE(1477), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2520), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33784] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2526), 1,
      anon_sym_RBRACE,
    STATE(1478), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2524), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33820] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2530), 1,
      anon_sym_RBRACE,
    STATE(1479), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2528), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33856] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2534), 1,
      anon_sym_RBRACE,
    STATE(1480), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2532), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33892] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2538), 1,
      anon_sym_RBRACE,
    STATE(1481), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2536), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33928] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2542), 1,
      anon_sym_RBRACE,
    STATE(1482), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2540), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [33964] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2546), 1,
      anon_sym_RBRACE,
    STATE(1483), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2544), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34000] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2550), 1,
      anon_sym_RBRACE,
    STATE(1484), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2548), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34036] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2554), 1,
      anon_sym_RBRACE,
    STATE(1485), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2552), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34072] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2558), 1,
      anon_sym_RBRACE,
    STATE(1486), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2556), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34108] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2562), 1,
      anon_sym_RBRACE,
    STATE(1487), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2560), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34144] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2566), 1,
      anon_sym_RBRACE,
    STATE(1488), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2564), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34180] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2570), 1,
      anon_sym_RBRACE,
    STATE(1489), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2568), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34216] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2574), 1,
      anon_sym_RBRACE,
    STATE(1490), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2572), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34252] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2578), 1,
      anon_sym_RBRACE,
    STATE(1491), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2576), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34288] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2582), 1,
      anon_sym_RBRACE,
    STATE(1492), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2580), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34324] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2586), 1,
      anon_sym_RBRACE,
    STATE(1493), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2584), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34360] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2590), 1,
      anon_sym_RBRACE,
    STATE(1494), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2588), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34396] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2594), 1,
      anon_sym_RBRACE,
    STATE(1495), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2592), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34432] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2598), 1,
      anon_sym_RBRACE,
    STATE(1496), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2596), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34468] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2602), 1,
      anon_sym_RBRACE,
    STATE(1497), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2600), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34504] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2606), 1,
      anon_sym_RBRACE,
    STATE(1498), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2604), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34540] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2610), 1,
      anon_sym_RBRACE,
    STATE(1499), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2608), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34576] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2614), 1,
      anon_sym_RBRACE,
    STATE(1500), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2612), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34612] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2618), 1,
      anon_sym_RBRACE,
    STATE(1501), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2616), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34648] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2622), 1,
      anon_sym_RBRACE,
    STATE(1502), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2620), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34684] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2626), 1,
      anon_sym_RBRACE,
    STATE(1503), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2624), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34720] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(773), 1,
      anon_sym_RBRACE,
    STATE(1504), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(771), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34756] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2630), 1,
      anon_sym_RBRACE,
    STATE(1505), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2628), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34792] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2634), 1,
      anon_sym_RBRACE,
    STATE(1506), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2632), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34828] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2638), 1,
      anon_sym_RBRACE,
    STATE(1507), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2636), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34864] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2642), 1,
      anon_sym_RBRACE,
    STATE(1508), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2640), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34900] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2646), 1,
      anon_sym_RBRACE,
    STATE(1509), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2644), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34936] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(765), 1,
      anon_sym_RBRACE,
    STATE(1510), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(763), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [34972] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2650), 1,
      anon_sym_RBRACE,
    STATE(1511), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2648), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [35008] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(781), 1,
      anon_sym_RBRACE,
    STATE(1512), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(779), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [35044] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2654), 1,
      anon_sym_RBRACE,
    STATE(1513), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2652), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [35080] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(789), 1,
      anon_sym_RBRACE,
    STATE(1514), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(787), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [35116] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2658), 1,
      anon_sym_RBRACE,
    STATE(1515), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2656), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [35152] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2662), 1,
      anon_sym_RBRACE,
    STATE(1516), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2660), 20,
      sym_identifier,
      anon_sym_invariant,
      anon_sym_update,
      anon_sym_assert,
      anon_sym_assume,
      anon_sym_ensures,
      anon_sym_requires,
      anon_sym_aborts_if,
      anon_sym_aborts_with,
      anon_sym_modifies,
      anon_sym_emits,
      anon_sym_global,
      anon_sym_local,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_include,
      anon_sym_apply,
      anon_sym_pragma,
      anon_sym_let,
      anon_sym_axiom,
  [35188] = 18,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1888), 1,
      anon_sym_fun,
    ACTIONS(1892), 1,
      anon_sym_public,
    ACTIONS(1896), 1,
      anon_sym_const,
    ACTIONS(1898), 1,
      anon_sym_inline,
    ACTIONS(1900), 1,
      anon_sym_struct,
    ACTIONS(1902), 1,
      anon_sym_enum,
    STATE(1598), 1,
      aux_sym_declaration_repeat1,
    STATE(1671), 1,
      sym_module_member_modifier,
    STATE(1686), 1,
      sym_visibility,
    STATE(1868), 1,
      sym__struct_signature,
    STATE(1948), 1,
      sym__function_signature,
    STATE(2063), 1,
      sym__enum_signature,
    ACTIONS(1890), 2,
      anon_sym_native,
      anon_sym_entry,
    STATE(1517), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1577), 2,
      sym_struct_decl,
      sym_enum_decl,
    STATE(1576), 3,
      sym_constant_decl,
      sym_function_decl,
      sym__struct_or_enum_decl,
  [35248] = 18,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1888), 1,
      anon_sym_fun,
    ACTIONS(1892), 1,
      anon_sym_public,
    ACTIONS(1896), 1,
      anon_sym_const,
    ACTIONS(1898), 1,
      anon_sym_inline,
    ACTIONS(1900), 1,
      anon_sym_struct,
    ACTIONS(1902), 1,
      anon_sym_enum,
    STATE(1598), 1,
      aux_sym_declaration_repeat1,
    STATE(1671), 1,
      sym_module_member_modifier,
    STATE(1686), 1,
      sym_visibility,
    STATE(1868), 1,
      sym__struct_signature,
    STATE(1948), 1,
      sym__function_signature,
    STATE(2063), 1,
      sym__enum_signature,
    ACTIONS(1890), 2,
      anon_sym_native,
      anon_sym_entry,
    STATE(1518), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1577), 2,
      sym_struct_decl,
      sym_enum_decl,
    STATE(1584), 3,
      sym_constant_decl,
      sym_function_decl,
      sym__struct_or_enum_decl,
  [35308] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1519), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2666), 4,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_SEMI,
    ACTIONS(2664), 14,
      sym_identifier,
      anon_sym_friend,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
      anon_sym_has,
  [35341] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1520), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2670), 4,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_SEMI,
    ACTIONS(2668), 14,
      sym_identifier,
      anon_sym_friend,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
      anon_sym_has,
  [35374] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(813), 1,
      anon_sym_COLON_COLON,
    ACTIONS(2675), 1,
      anon_sym_COLON,
    ACTIONS(2678), 1,
      anon_sym_LT2,
    STATE(1561), 1,
      sym_type_args,
    STATE(1521), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2672), 14,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [35413] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1522), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2683), 4,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_SEMI,
    ACTIONS(2681), 14,
      sym_identifier,
      anon_sym_friend,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
      anon_sym_has,
  [35446] = 20,
    ACTIONS(11), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1888), 1,
      anon_sym_fun,
    ACTIONS(1892), 1,
      anon_sym_public,
    ACTIONS(1896), 1,
      anon_sym_const,
    ACTIONS(1898), 1,
      anon_sym_inline,
    STATE(1536), 1,
      aux_sym_attributes_repeat2,
    STATE(1596), 1,
      aux_sym_script_repeat2,
    STATE(1597), 1,
      sym_attributes,
    STATE(1605), 1,
      aux_sym_declaration_repeat1,
    STATE(1671), 1,
      sym_module_member_modifier,
    STATE(1686), 1,
      sym_visibility,
    STATE(1693), 1,
      sym__script_func_decl,
    STATE(1703), 1,
      sym_constant_decl,
    STATE(1717), 1,
      sym__script_constant_decl,
    STATE(1936), 1,
      sym_function_decl,
    STATE(1948), 1,
      sym__function_signature,
    ACTIONS(1890), 2,
      anon_sym_native,
      anon_sym_entry,
    STATE(1523), 2,
      sym_line_comment,
      sym_block_comment,
  [35509] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(819), 1,
      anon_sym_COLON,
    STATE(1524), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(821), 17,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_LT2,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [35542] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(823), 1,
      anon_sym_COLON,
    STATE(1525), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(825), 17,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_LT2,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [35575] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(807), 1,
      anon_sym_COLON,
    STATE(1526), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(809), 17,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_LT2,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [35608] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(811), 1,
      anon_sym_COLON,
    STATE(1527), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(813), 17,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_LT2,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [35641] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(815), 1,
      anon_sym_COLON,
    STATE(1528), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(817), 17,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_LT2,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [35674] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(841), 1,
      anon_sym_COLON,
    ACTIONS(2685), 1,
      anon_sym_COLON_COLON,
    STATE(1529), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(843), 16,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_LT2,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [35709] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1530), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2689), 4,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_SEMI,
    ACTIONS(2687), 14,
      sym_identifier,
      anon_sym_friend,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
      anon_sym_has,
  [35742] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(833), 1,
      anon_sym_COLON,
    ACTIONS(2691), 1,
      anon_sym_COLON_COLON,
    STATE(1531), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(835), 16,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_LT2,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [35777] = 20,
    ACTIONS(11), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1888), 1,
      anon_sym_fun,
    ACTIONS(1892), 1,
      anon_sym_public,
    ACTIONS(1896), 1,
      anon_sym_const,
    ACTIONS(1898), 1,
      anon_sym_inline,
    STATE(1536), 1,
      aux_sym_attributes_repeat2,
    STATE(1596), 1,
      aux_sym_script_repeat2,
    STATE(1597), 1,
      sym_attributes,
    STATE(1605), 1,
      aux_sym_declaration_repeat1,
    STATE(1654), 1,
      sym__script_func_decl,
    STATE(1671), 1,
      sym_module_member_modifier,
    STATE(1686), 1,
      sym_visibility,
    STATE(1703), 1,
      sym_constant_decl,
    STATE(1717), 1,
      sym__script_constant_decl,
    STATE(1936), 1,
      sym_function_decl,
    STATE(1948), 1,
      sym__function_signature,
    ACTIONS(1890), 2,
      anon_sym_native,
      anon_sym_entry,
    STATE(1532), 2,
      sym_line_comment,
      sym_block_comment,
  [35840] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2695), 1,
      anon_sym_LT2,
    STATE(1561), 1,
      sym_type_args,
    STATE(1533), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2693), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [35874] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(895), 1,
      anon_sym_COLON_COLON,
    ACTIONS(1131), 1,
      anon_sym_DOT,
    STATE(937), 1,
      sym_number_type,
    STATE(1534), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2697), 6,
      anon_sym_u8,
      anon_sym_u16,
      anon_sym_u32,
      anon_sym_u64,
      anon_sym_u128,
      anon_sym_u256,
    ACTIONS(1133), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_spec,
      anon_sym_SEMI,
  [35912] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2701), 1,
      anon_sym_POUND,
    STATE(1535), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_attributes_repeat2,
    ACTIONS(2699), 15,
      anon_sym_address,
      anon_sym_friend,
      anon_sym_module,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_script,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [35944] = 6,
    ACTIONS(11), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1535), 1,
      aux_sym_attributes_repeat2,
    STATE(1536), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2704), 15,
      anon_sym_address,
      anon_sym_friend,
      anon_sym_module,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_script,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [35978] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(827), 1,
      anon_sym_COLON,
    ACTIONS(831), 1,
      anon_sym_COLON_COLON,
    STATE(1537), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(829), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_LT2,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [36012] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1538), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(849), 17,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_LT2,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [36042] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(827), 1,
      anon_sym_COLON,
    ACTIONS(2706), 1,
      anon_sym_COLON_COLON,
    STATE(1539), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(829), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_LT2,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [36076] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1540), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2708), 16,
      anon_sym_address,
      anon_sym_friend,
      anon_sym_POUND,
      anon_sym_module,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_script,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [36105] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1541), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2710), 16,
      anon_sym_address,
      anon_sym_friend,
      anon_sym_POUND,
      anon_sym_module,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_script,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [36134] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2712), 1,
      anon_sym_COMMA,
    ACTIONS(2716), 1,
      anon_sym_PIPE,
    STATE(1867), 1,
      aux_sym_tuple_type_repeat1,
    STATE(1542), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2714), 13,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [36169] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2712), 1,
      anon_sym_COMMA,
    ACTIONS(2716), 1,
      anon_sym_PIPE,
    STATE(1867), 1,
      aux_sym_tuple_type_repeat1,
    STATE(1543), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2718), 13,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [36204] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2712), 1,
      anon_sym_COMMA,
    ACTIONS(2716), 1,
      anon_sym_PIPE,
    STATE(1867), 1,
      aux_sym_tuple_type_repeat1,
    STATE(1544), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2720), 13,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [36239] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1545), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2722), 16,
      anon_sym_address,
      anon_sym_friend,
      anon_sym_POUND,
      anon_sym_module,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_script,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [36268] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1546), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(865), 16,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [36297] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1547), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2724), 16,
      anon_sym_address,
      anon_sym_friend,
      anon_sym_POUND,
      anon_sym_module,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_script,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [36326] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1548), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2726), 16,
      anon_sym_friend,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_SEMI,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
      anon_sym_has,
  [36355] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1549), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(853), 16,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [36384] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1550), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(861), 16,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [36413] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1551), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(857), 16,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [36442] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1552), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2728), 16,
      anon_sym_friend,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_SEMI,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
      anon_sym_has,
  [36471] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1553), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2730), 16,
      anon_sym_friend,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_SEMI,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
      anon_sym_has,
  [36500] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(813), 1,
      anon_sym_COLON_COLON,
    ACTIONS(2675), 1,
      anon_sym_COLON,
    ACTIONS(2732), 1,
      anon_sym_LT2,
    STATE(1561), 1,
      sym_type_args,
    STATE(1554), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2672), 12,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [36537] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1555), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2735), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [36565] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1556), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2737), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [36593] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1557), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2739), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [36621] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1558), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2741), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [36649] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1559), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2743), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [36677] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1560), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2714), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [36705] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1561), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2745), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [36733] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1562), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2747), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [36761] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1563), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2749), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [36789] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2751), 1,
      anon_sym_LT2,
    STATE(1561), 1,
      sym_type_args,
    STATE(1564), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2693), 13,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [36821] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1565), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2753), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [36849] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1566), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2693), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [36877] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1567), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2755), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_GT,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [36905] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1568), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2757), 14,
      anon_sym_friend,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [36932] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1569), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2759), 14,
      anon_sym_friend,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [36959] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1570), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2761), 14,
      anon_sym_friend,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [36986] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1571), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2763), 14,
      anon_sym_friend,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [37013] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1572), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2765), 14,
      anon_sym_friend,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [37040] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1573), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2767), 14,
      anon_sym_friend,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [37067] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1574), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2769), 14,
      anon_sym_friend,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [37094] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1575), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2771), 14,
      anon_sym_friend,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [37121] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1576), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2765), 14,
      anon_sym_friend,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [37148] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1577), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2773), 14,
      anon_sym_friend,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [37175] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1578), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2769), 14,
      anon_sym_friend,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [37202] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2751), 1,
      anon_sym_LT2,
    ACTIONS(2775), 1,
      anon_sym_LPAREN,
    ACTIONS(2779), 1,
      anon_sym_EQ,
    ACTIONS(2781), 1,
      anon_sym_LBRACE,
    STATE(1590), 1,
      sym_type_args,
    STATE(1624), 1,
      sym__bind_tuple,
    STATE(1650), 1,
      sym__bind_fields,
    STATE(1579), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2777), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [37243] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1580), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2757), 14,
      anon_sym_friend,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [37270] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1581), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2783), 14,
      anon_sym_friend,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [37297] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1582), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2785), 14,
      anon_sym_friend,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [37324] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1583), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2787), 14,
      anon_sym_friend,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [37351] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1584), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2789), 14,
      anon_sym_friend,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [37378] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1585), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2791), 14,
      anon_sym_friend,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [37405] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1586), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2789), 14,
      anon_sym_friend,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [37432] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1587), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2793), 14,
      anon_sym_friend,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [37459] = 16,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1888), 1,
      anon_sym_fun,
    ACTIONS(1892), 1,
      anon_sym_public,
    ACTIONS(1894), 1,
      anon_sym_use,
    ACTIONS(1896), 1,
      anon_sym_const,
    ACTIONS(1898), 1,
      anon_sym_inline,
    STATE(1601), 1,
      aux_sym_declaration_repeat1,
    STATE(1671), 1,
      sym_module_member_modifier,
    STATE(1686), 1,
      sym_visibility,
    STATE(1687), 1,
      sym_use_decl,
    STATE(1735), 1,
      sym_constant_decl,
    STATE(1937), 1,
      sym_function_decl,
    STATE(1948), 1,
      sym__function_signature,
    ACTIONS(1890), 2,
      anon_sym_native,
      anon_sym_entry,
    STATE(1588), 2,
      sym_line_comment,
      sym_block_comment,
  [37510] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1589), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2795), 14,
      anon_sym_friend,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
      anon_sym_invariant,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [37537] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2775), 1,
      anon_sym_LPAREN,
    ACTIONS(2781), 1,
      anon_sym_LBRACE,
    ACTIONS(2799), 1,
      anon_sym_EQ,
    STATE(1627), 1,
      sym__bind_tuple,
    STATE(1640), 1,
      sym__bind_fields,
    STATE(1590), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2797), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [37573] = 13,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2801), 1,
      ts_builtin_sym_end,
    ACTIONS(2803), 1,
      anon_sym_address,
    ACTIONS(2806), 1,
      anon_sym_POUND,
    ACTIONS(2809), 1,
      anon_sym_module,
    ACTIONS(2812), 1,
      anon_sym_spec,
    ACTIONS(2815), 1,
      anon_sym_script,
    STATE(1536), 1,
      aux_sym_attributes_repeat2,
    STATE(1646), 1,
      sym__module_keyword,
    STATE(1662), 1,
      sym_attributes,
    STATE(1591), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_source_file_repeat1,
    STATE(1767), 3,
      sym_address_block,
      sym_module,
      sym_script,
  [37617] = 14,
    ACTIONS(9), 1,
      anon_sym_address,
    ACTIONS(11), 1,
      anon_sym_POUND,
    ACTIONS(13), 1,
      anon_sym_module,
    ACTIONS(15), 1,
      anon_sym_spec,
    ACTIONS(17), 1,
      anon_sym_script,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2818), 1,
      ts_builtin_sym_end,
    STATE(1536), 1,
      aux_sym_attributes_repeat2,
    STATE(1591), 1,
      aux_sym_source_file_repeat1,
    STATE(1646), 1,
      sym__module_keyword,
    STATE(1662), 1,
      sym_attributes,
    STATE(1592), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1767), 3,
      sym_address_block,
      sym_module,
      sym_script,
  [37663] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(831), 1,
      anon_sym_COLON_COLON,
    ACTIONS(2822), 2,
      anon_sym_EQ,
      anon_sym_COLON,
    STATE(1593), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(829), 3,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_LT2,
    ACTIONS(2820), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_RBRACE,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [37695] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2824), 1,
      anon_sym_POUND,
    ACTIONS(2829), 1,
      anon_sym_use,
    STATE(1536), 1,
      aux_sym_attributes_repeat2,
    STATE(1659), 1,
      sym_use_decl,
    STATE(1660), 1,
      sym__script_use_decl,
    STATE(2105), 1,
      sym_attributes,
    STATE(1594), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_script_repeat1,
    ACTIONS(2827), 6,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_const,
      anon_sym_inline,
  [37733] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(839), 1,
      anon_sym_COLON_COLON,
    ACTIONS(2822), 2,
      anon_sym_EQ,
      anon_sym_COLON,
    STATE(1595), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(829), 3,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_LT2,
    ACTIONS(2820), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [37764] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2832), 1,
      anon_sym_POUND,
    ACTIONS(2837), 1,
      anon_sym_const,
    STATE(1536), 1,
      aux_sym_attributes_repeat2,
    STATE(1703), 1,
      sym_constant_decl,
    STATE(1717), 1,
      sym__script_constant_decl,
    STATE(2065), 1,
      sym_attributes,
    STATE(1596), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_script_repeat2,
    ACTIONS(2835), 5,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_inline,
  [37801] = 14,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1888), 1,
      anon_sym_fun,
    ACTIONS(1892), 1,
      anon_sym_public,
    ACTIONS(1896), 1,
      anon_sym_const,
    ACTIONS(1898), 1,
      anon_sym_inline,
    STATE(1601), 1,
      aux_sym_declaration_repeat1,
    STATE(1671), 1,
      sym_module_member_modifier,
    STATE(1686), 1,
      sym_visibility,
    STATE(1735), 1,
      sym_constant_decl,
    STATE(1937), 1,
      sym_function_decl,
    STATE(1948), 1,
      sym__function_signature,
    ACTIONS(1890), 2,
      anon_sym_native,
      anon_sym_entry,
    STATE(1597), 2,
      sym_line_comment,
      sym_block_comment,
  [37846] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2845), 1,
      anon_sym_public,
    STATE(1671), 1,
      sym_module_member_modifier,
    STATE(1686), 1,
      sym_visibility,
    ACTIONS(2842), 2,
      anon_sym_native,
      anon_sym_entry,
    STATE(1598), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_declaration_repeat1,
    ACTIONS(2840), 5,
      anon_sym_fun,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [37878] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(114), 1,
      sym_parenthesized_expr,
    STATE(1599), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(817), 8,
      anon_sym_COLON_COLON,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LT2,
      anon_sym_spec,
  [37905] = 13,
    ACTIONS(11), 1,
      anon_sym_POUND,
    ACTIONS(13), 1,
      anon_sym_module,
    ACTIONS(15), 1,
      anon_sym_spec,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2848), 1,
      anon_sym_RBRACE,
    STATE(1536), 1,
      aux_sym_attributes_repeat2,
    STATE(1607), 1,
      aux_sym_address_block_repeat1,
    STATE(1646), 1,
      sym__module_keyword,
    STATE(1820), 1,
      sym_attributes,
    STATE(1824), 1,
      sym__address_member,
    STATE(1843), 1,
      sym_module,
    STATE(1600), 2,
      sym_line_comment,
      sym_block_comment,
  [37946] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1888), 1,
      anon_sym_fun,
    ACTIONS(1892), 1,
      anon_sym_public,
    ACTIONS(1898), 1,
      anon_sym_inline,
    STATE(1598), 1,
      aux_sym_declaration_repeat1,
    STATE(1671), 1,
      sym_module_member_modifier,
    STATE(1686), 1,
      sym_visibility,
    STATE(1925), 1,
      sym_function_decl,
    STATE(1948), 1,
      sym__function_signature,
    ACTIONS(1890), 2,
      anon_sym_native,
      anon_sym_entry,
    STATE(1601), 2,
      sym_line_comment,
      sym_block_comment,
  [37985] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2850), 1,
      sym_identifier,
    ACTIONS(2852), 1,
      anon_sym_STAR,
    ACTIONS(2854), 1,
      anon_sym_COMMA,
    ACTIONS(2856), 1,
      anon_sym_SEMI,
    ACTIONS(2858), 1,
      anon_sym_except,
    STATE(1637), 1,
      aux_sym__spec_apply_pattern_repeat1,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    STATE(1801), 1,
      sym__spec_apply_pattern,
    ACTIONS(2860), 2,
      anon_sym_public,
      anon_sym_internal,
    STATE(1602), 2,
      sym_line_comment,
      sym_block_comment,
  [38024] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2862), 1,
      anon_sym_COLON_COLON,
    STATE(1603), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2864), 9,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_LT2,
      anon_sym_SEMI,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [38049] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2866), 1,
      anon_sym_COLON_COLON,
    STATE(1604), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2864), 9,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_LT2,
      anon_sym_SEMI,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [38074] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1888), 1,
      anon_sym_fun,
    ACTIONS(1892), 1,
      anon_sym_public,
    ACTIONS(1898), 1,
      anon_sym_inline,
    STATE(1598), 1,
      aux_sym_declaration_repeat1,
    STATE(1671), 1,
      sym_module_member_modifier,
    STATE(1686), 1,
      sym_visibility,
    STATE(1937), 1,
      sym_function_decl,
    STATE(1948), 1,
      sym__function_signature,
    ACTIONS(1890), 2,
      anon_sym_native,
      anon_sym_entry,
    STATE(1605), 2,
      sym_line_comment,
      sym_block_comment,
  [38113] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2868), 1,
      anon_sym_COLON_COLON,
    STATE(1606), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2870), 9,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_LT2,
      anon_sym_SEMI,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [38138] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2872), 1,
      anon_sym_RBRACE,
    ACTIONS(2874), 1,
      anon_sym_POUND,
    ACTIONS(2877), 1,
      anon_sym_module,
    ACTIONS(2880), 1,
      anon_sym_spec,
    STATE(1536), 1,
      aux_sym_attributes_repeat2,
    STATE(1646), 1,
      sym__module_keyword,
    STATE(1820), 1,
      sym_attributes,
    STATE(1824), 1,
      sym__address_member,
    STATE(1843), 1,
      sym_module,
    STATE(1607), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_address_block_repeat1,
  [38177] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2883), 1,
      anon_sym_COLON,
    ACTIONS(2887), 1,
      anon_sym_BANG,
    ACTIONS(2889), 1,
      anon_sym_pure,
    STATE(1744), 1,
      aux_sym__specifier_repeat1,
    STATE(2138), 1,
      sym__specifier,
    ACTIONS(2885), 2,
      anon_sym_LBRACE,
      anon_sym_SEMI,
    STATE(1608), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2891), 3,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [38212] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2887), 1,
      anon_sym_BANG,
    ACTIONS(2889), 1,
      anon_sym_pure,
    ACTIONS(2893), 1,
      anon_sym_COLON,
    STATE(1744), 1,
      aux_sym__specifier_repeat1,
    STATE(2148), 1,
      sym__specifier,
    ACTIONS(2895), 2,
      anon_sym_LBRACE,
      anon_sym_SEMI,
    STATE(1609), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2891), 3,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [38247] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2751), 1,
      anon_sym_LT2,
    ACTIONS(2897), 1,
      anon_sym_LPAREN,
    STATE(1657), 1,
      sym_type_args,
    STATE(1610), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2899), 7,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [38276] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2901), 1,
      anon_sym_COLON_COLON,
    STATE(1611), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2903), 9,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_LT2,
      anon_sym_SEMI,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [38301] = 13,
    ACTIONS(11), 1,
      anon_sym_POUND,
    ACTIONS(13), 1,
      anon_sym_module,
    ACTIONS(15), 1,
      anon_sym_spec,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2905), 1,
      anon_sym_RBRACE,
    STATE(1536), 1,
      aux_sym_attributes_repeat2,
    STATE(1600), 1,
      aux_sym_address_block_repeat1,
    STATE(1646), 1,
      sym__module_keyword,
    STATE(1820), 1,
      sym_attributes,
    STATE(1824), 1,
      sym__address_member,
    STATE(1843), 1,
      sym_module,
    STATE(1612), 2,
      sym_line_comment,
      sym_block_comment,
  [38342] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2887), 1,
      anon_sym_BANG,
    ACTIONS(2889), 1,
      anon_sym_pure,
    ACTIONS(2907), 1,
      anon_sym_COLON,
    STATE(1744), 1,
      aux_sym__specifier_repeat1,
    STATE(2153), 1,
      sym__specifier,
    ACTIONS(2909), 2,
      anon_sym_LBRACE,
      anon_sym_SEMI,
    STATE(1613), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2891), 3,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [38377] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2887), 1,
      anon_sym_BANG,
    ACTIONS(2889), 1,
      anon_sym_pure,
    ACTIONS(2911), 1,
      anon_sym_COLON,
    STATE(1744), 1,
      aux_sym__specifier_repeat1,
    STATE(2008), 1,
      sym__specifier,
    ACTIONS(2913), 2,
      anon_sym_LBRACE,
      anon_sym_SEMI,
    STATE(1614), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2891), 3,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [38412] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2850), 1,
      sym_identifier,
    ACTIONS(2852), 1,
      anon_sym_STAR,
    ACTIONS(2915), 1,
      anon_sym_COMMA,
    ACTIONS(2917), 1,
      anon_sym_SEMI,
    STATE(1637), 1,
      aux_sym__spec_apply_pattern_repeat1,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    STATE(1857), 1,
      sym__spec_apply_pattern,
    ACTIONS(2860), 2,
      anon_sym_public,
      anon_sym_internal,
    STATE(1615), 2,
      sym_line_comment,
      sym_block_comment,
  [38448] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2921), 1,
      anon_sym_EQ,
    STATE(1616), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2919), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [38472] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2925), 1,
      anon_sym_EQ,
    STATE(1617), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2923), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [38496] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(2927), 1,
      sym_identifier,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1837), 1,
      sym_module_ident,
    STATE(2290), 1,
      sym__leading_name_access,
    STATE(1618), 2,
      sym_line_comment,
      sym_block_comment,
  [38534] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(2927), 1,
      sym_identifier,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(1834), 1,
      sym_module_ident,
    STATE(2290), 1,
      sym__leading_name_access,
    STATE(1619), 2,
      sym_line_comment,
      sym_block_comment,
  [38572] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2931), 1,
      anon_sym_EQ,
    STATE(1620), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2929), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [38596] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(393), 1,
      anon_sym_LT,
    ACTIONS(2850), 1,
      sym_identifier,
    ACTIONS(2852), 1,
      anon_sym_STAR,
    ACTIONS(2935), 1,
      anon_sym_except,
    STATE(1651), 1,
      aux_sym__spec_apply_pattern_repeat1,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    STATE(1988), 1,
      sym_type_params,
    ACTIONS(2933), 2,
      anon_sym_COMMA,
      anon_sym_SEMI,
    STATE(1621), 2,
      sym_line_comment,
      sym_block_comment,
  [38632] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2887), 1,
      anon_sym_BANG,
    ACTIONS(2889), 1,
      anon_sym_pure,
    STATE(1744), 1,
      aux_sym__specifier_repeat1,
    STATE(2088), 1,
      sym__specifier,
    ACTIONS(2937), 2,
      anon_sym_LBRACE,
      anon_sym_SEMI,
    STATE(1622), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2891), 3,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [38664] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2941), 1,
      anon_sym_EQ,
    STATE(1623), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2939), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [38688] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2945), 1,
      anon_sym_EQ,
    STATE(1624), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2943), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [38712] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2949), 1,
      anon_sym_EQ,
    STATE(1625), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2947), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [38736] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(55), 1,
      sym_parenthesized_expr,
    STATE(1626), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(817), 7,
      anon_sym_COLON_COLON,
      anon_sym_EQ,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_LT2,
  [38762] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2953), 1,
      anon_sym_EQ,
    STATE(1627), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2951), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [38786] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2957), 1,
      anon_sym_EQ,
    STATE(1628), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2955), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [38810] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2850), 1,
      sym_identifier,
    ACTIONS(2852), 1,
      anon_sym_STAR,
    ACTIONS(2959), 1,
      anon_sym_COMMA,
    ACTIONS(2961), 1,
      anon_sym_SEMI,
    STATE(1637), 1,
      aux_sym__spec_apply_pattern_repeat1,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    STATE(1893), 1,
      sym__spec_apply_pattern,
    ACTIONS(2860), 2,
      anon_sym_public,
      anon_sym_internal,
    STATE(1629), 2,
      sym_line_comment,
      sym_block_comment,
  [38846] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(815), 1,
      anon_sym_DOT,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(94), 1,
      sym_parenthesized_expr,
    STATE(1630), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(817), 6,
      anon_sym_COLON_COLON,
      anon_sym_LBRACE,
      anon_sym_DOT_DOT,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_LT2,
  [38874] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2965), 1,
      anon_sym_EQ,
    STATE(1631), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2963), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [38898] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2887), 1,
      anon_sym_BANG,
    ACTIONS(2889), 1,
      anon_sym_pure,
    STATE(1744), 1,
      aux_sym__specifier_repeat1,
    STATE(2033), 1,
      sym__specifier,
    ACTIONS(2967), 2,
      anon_sym_LBRACE,
      anon_sym_SEMI,
    STATE(1632), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2891), 3,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [38930] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2850), 1,
      sym_identifier,
    ACTIONS(2852), 1,
      anon_sym_STAR,
    ACTIONS(2969), 1,
      anon_sym_SEMI,
    ACTIONS(2971), 1,
      anon_sym_except,
    STATE(1637), 1,
      aux_sym__spec_apply_pattern_repeat1,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    STATE(1851), 1,
      sym__spec_apply_pattern,
    ACTIONS(2860), 2,
      anon_sym_public,
      anon_sym_internal,
    STATE(1633), 2,
      sym_line_comment,
      sym_block_comment,
  [38966] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2975), 1,
      anon_sym_EQ,
    STATE(1634), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2973), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [38990] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2850), 1,
      sym_identifier,
    ACTIONS(2852), 1,
      anon_sym_STAR,
    ACTIONS(2977), 1,
      anon_sym_COMMA,
    ACTIONS(2979), 1,
      anon_sym_SEMI,
    STATE(1637), 1,
      aux_sym__spec_apply_pattern_repeat1,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    STATE(1860), 1,
      sym__spec_apply_pattern,
    ACTIONS(2860), 2,
      anon_sym_public,
      anon_sym_internal,
    STATE(1635), 2,
      sym_line_comment,
      sym_block_comment,
  [39026] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2850), 1,
      sym_identifier,
    ACTIONS(2852), 1,
      anon_sym_STAR,
    ACTIONS(2981), 1,
      anon_sym_COMMA,
    ACTIONS(2983), 1,
      anon_sym_SEMI,
    STATE(1637), 1,
      aux_sym__spec_apply_pattern_repeat1,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    STATE(1908), 1,
      sym__spec_apply_pattern,
    ACTIONS(2860), 2,
      anon_sym_public,
      anon_sym_internal,
    STATE(1636), 2,
      sym_line_comment,
      sym_block_comment,
  [39062] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(393), 1,
      anon_sym_LT,
    ACTIONS(2850), 1,
      sym_identifier,
    ACTIONS(2852), 1,
      anon_sym_STAR,
    ACTIONS(2987), 1,
      anon_sym_except,
    STATE(1651), 1,
      aux_sym__spec_apply_pattern_repeat1,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    STATE(1873), 1,
      sym_type_params,
    ACTIONS(2985), 2,
      anon_sym_COMMA,
      anon_sym_SEMI,
    STATE(1637), 2,
      sym_line_comment,
      sym_block_comment,
  [39098] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2850), 1,
      sym_identifier,
    ACTIONS(2852), 1,
      anon_sym_STAR,
    ACTIONS(2989), 1,
      anon_sym_COMMA,
    ACTIONS(2991), 1,
      anon_sym_SEMI,
    STATE(1637), 1,
      aux_sym__spec_apply_pattern_repeat1,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    STATE(1983), 1,
      sym__spec_apply_pattern,
    ACTIONS(2860), 2,
      anon_sym_public,
      anon_sym_internal,
    STATE(1638), 2,
      sym_line_comment,
      sym_block_comment,
  [39134] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2887), 1,
      anon_sym_BANG,
    ACTIONS(2889), 1,
      anon_sym_pure,
    STATE(1744), 1,
      aux_sym__specifier_repeat1,
    STATE(2040), 1,
      sym__specifier,
    ACTIONS(2993), 2,
      anon_sym_LBRACE,
      anon_sym_SEMI,
    STATE(1639), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2891), 3,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [39166] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_EQ,
    STATE(1640), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2995), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [39190] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3001), 1,
      anon_sym_EQ,
    STATE(1641), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2999), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [39214] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2887), 1,
      anon_sym_BANG,
    ACTIONS(2889), 1,
      anon_sym_pure,
    STATE(1744), 1,
      aux_sym__specifier_repeat1,
    STATE(2068), 1,
      sym__specifier,
    ACTIONS(3003), 2,
      anon_sym_LBRACE,
      anon_sym_SEMI,
    STATE(1642), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2891), 3,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [39246] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3005), 1,
      anon_sym_LPAREN,
    STATE(1643), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3007), 8,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [39270] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1644), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3009), 9,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_LT2,
      anon_sym_SEMI,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [39292] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2850), 1,
      sym_identifier,
    ACTIONS(2852), 1,
      anon_sym_STAR,
    ACTIONS(3011), 1,
      anon_sym_COMMA,
    ACTIONS(3013), 1,
      anon_sym_SEMI,
    STATE(1637), 1,
      aux_sym__spec_apply_pattern_repeat1,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    STATE(1886), 1,
      sym__spec_apply_pattern,
    ACTIONS(2860), 2,
      anon_sym_public,
      anon_sym_internal,
    STATE(1645), 2,
      sym_line_comment,
      sym_block_comment,
  [39328] = 12,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(3015), 1,
      sym_identifier,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(2237), 1,
      sym__module_path,
    STATE(2371), 1,
      sym__leading_name_access,
    STATE(1646), 2,
      sym_line_comment,
      sym_block_comment,
  [39366] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3019), 1,
      anon_sym_EQ,
    STATE(1647), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3017), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [39390] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2850), 1,
      sym_identifier,
    ACTIONS(2852), 1,
      anon_sym_STAR,
    ACTIONS(3021), 1,
      anon_sym_SEMI,
    ACTIONS(3023), 1,
      anon_sym_except,
    STATE(1637), 1,
      aux_sym__spec_apply_pattern_repeat1,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    STATE(1851), 1,
      sym__spec_apply_pattern,
    ACTIONS(2860), 2,
      anon_sym_public,
      anon_sym_internal,
    STATE(1648), 2,
      sym_line_comment,
      sym_block_comment,
  [39426] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3027), 1,
      anon_sym_EQ,
    STATE(1649), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3025), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [39450] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2799), 1,
      anon_sym_EQ,
    STATE(1650), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2797), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_RBRACE,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [39474] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3029), 1,
      sym_identifier,
    ACTIONS(3032), 1,
      anon_sym_STAR,
    ACTIONS(3037), 1,
      anon_sym_except,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    ACTIONS(3035), 3,
      anon_sym_COMMA,
      anon_sym_LT,
      anon_sym_SEMI,
    STATE(1651), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym__spec_apply_pattern_repeat1,
  [39503] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3039), 1,
      anon_sym_RBRACE,
    ACTIONS(3041), 1,
      anon_sym_POUND,
    ACTIONS(3044), 1,
      anon_sym_spec,
    STATE(1536), 1,
      aux_sym_attributes_repeat2,
    STATE(1951), 1,
      sym_spec_block,
    STATE(1954), 1,
      sym__script_spec_block,
    STATE(2119), 1,
      sym_attributes,
    STATE(1652), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_script_repeat3,
  [39536] = 11,
    ACTIONS(11), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3047), 1,
      anon_sym_RBRACE,
    ACTIONS(3049), 1,
      anon_sym_spec,
    STATE(1536), 1,
      aux_sym_attributes_repeat2,
    STATE(1652), 1,
      aux_sym_script_repeat3,
    STATE(1951), 1,
      sym_spec_block,
    STATE(1954), 1,
      sym__script_spec_block,
    STATE(2119), 1,
      sym_attributes,
    STATE(1653), 2,
      sym_line_comment,
      sym_block_comment,
  [39571] = 11,
    ACTIONS(11), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3047), 1,
      anon_sym_RBRACE,
    ACTIONS(3049), 1,
      anon_sym_spec,
    STATE(1536), 1,
      aux_sym_attributes_repeat2,
    STATE(1666), 1,
      aux_sym_script_repeat3,
    STATE(1951), 1,
      sym_spec_block,
    STATE(1954), 1,
      sym__script_spec_block,
    STATE(2119), 1,
      sym_attributes,
    STATE(1654), 2,
      sym_line_comment,
      sym_block_comment,
  [39606] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(2927), 1,
      sym_identifier,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(2343), 1,
      sym__leading_name_access,
    STATE(1655), 2,
      sym_line_comment,
      sym_block_comment,
  [39641] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1656), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3051), 8,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [39662] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3053), 1,
      anon_sym_LPAREN,
    STATE(1657), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3055), 7,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [39685] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3057), 1,
      anon_sym_COMMA,
    STATE(1664), 1,
      aux_sym_access_specifier_list_repeat1,
    STATE(1658), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1606), 6,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [39710] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1659), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3059), 8,
      anon_sym_POUND,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
  [39731] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1660), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3061), 8,
      anon_sym_POUND,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
  [39752] = 11,
    ACTIONS(11), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3049), 1,
      anon_sym_spec,
    ACTIONS(3063), 1,
      anon_sym_RBRACE,
    STATE(1536), 1,
      aux_sym_attributes_repeat2,
    STATE(1691), 1,
      aux_sym_script_repeat3,
    STATE(1951), 1,
      sym_spec_block,
    STATE(1954), 1,
      sym__script_spec_block,
    STATE(2119), 1,
      sym_attributes,
    STATE(1661), 2,
      sym_line_comment,
      sym_block_comment,
  [39787] = 9,
    ACTIONS(9), 1,
      anon_sym_address,
    ACTIONS(13), 1,
      anon_sym_module,
    ACTIONS(15), 1,
      anon_sym_spec,
    ACTIONS(17), 1,
      anon_sym_script,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1646), 1,
      sym__module_keyword,
    STATE(1662), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1766), 3,
      sym_address_block,
      sym_module,
      sym_script,
  [39818] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1663), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3065), 8,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [39839] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3067), 1,
      anon_sym_COMMA,
    STATE(1664), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_access_specifier_list_repeat1,
    ACTIONS(3070), 6,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [39862] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(45), 1,
      aux_sym_number_token1,
    ACTIONS(47), 1,
      aux_sym_number_token2,
    ACTIONS(49), 1,
      aux_sym_number_token4,
    ACTIONS(51), 1,
      aux_sym_number_token6,
    ACTIONS(2927), 1,
      sym_identifier,
    STATE(880), 1,
      sym_number,
    STATE(883), 1,
      sym_numerical_addr,
    STATE(930), 1,
      sym__leading_name_access,
    STATE(1665), 2,
      sym_line_comment,
      sym_block_comment,
  [39897] = 11,
    ACTIONS(11), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3049), 1,
      anon_sym_spec,
    ACTIONS(3072), 1,
      anon_sym_RBRACE,
    STATE(1536), 1,
      aux_sym_attributes_repeat2,
    STATE(1652), 1,
      aux_sym_script_repeat3,
    STATE(1951), 1,
      sym_spec_block,
    STATE(1954), 1,
      sym__script_spec_block,
    STATE(2119), 1,
      sym_attributes,
    STATE(1666), 2,
      sym_line_comment,
      sym_block_comment,
  [39932] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1667), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3074), 8,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [39953] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3076), 1,
      anon_sym_COMMA,
    STATE(1658), 1,
      aux_sym_access_specifier_list_repeat1,
    STATE(1668), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3078), 6,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [39978] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2695), 1,
      anon_sym_LT2,
    ACTIONS(2775), 1,
      anon_sym_LPAREN,
    ACTIONS(2781), 1,
      anon_sym_LBRACE,
    STATE(1590), 1,
      sym_type_args,
    STATE(1624), 1,
      sym__bind_tuple,
    STATE(1650), 1,
      sym__bind_fields,
    ACTIONS(2777), 2,
      anon_sym_COMMA,
      anon_sym_PIPE,
    STATE(1669), 2,
      sym_line_comment,
      sym_block_comment,
  [40011] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3080), 1,
      sym_identifier,
    ACTIONS(3082), 1,
      anon_sym_LBRACE,
    ACTIONS(3084), 1,
      anon_sym_module,
    ACTIONS(3086), 1,
      anon_sym_schema,
    ACTIONS(3088), 1,
      anon_sym_fun,
    ACTIONS(3090), 1,
      anon_sym_native,
    STATE(882), 1,
      sym_spec_func,
    STATE(2288), 1,
      sym_spec_block_target,
    STATE(1670), 2,
      sym_line_comment,
      sym_block_comment,
  [40046] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1671), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3092), 8,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [40067] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2850), 1,
      sym_identifier,
    ACTIONS(2852), 1,
      anon_sym_STAR,
    ACTIONS(3094), 1,
      anon_sym_SEMI,
    STATE(1637), 1,
      aux_sym__spec_apply_pattern_repeat1,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    STATE(1851), 1,
      sym__spec_apply_pattern,
    ACTIONS(2860), 2,
      anon_sym_public,
      anon_sym_internal,
    STATE(1672), 2,
      sym_line_comment,
      sym_block_comment,
  [40100] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1578), 1,
      anon_sym_native,
    ACTIONS(3080), 1,
      sym_identifier,
    ACTIONS(3082), 1,
      anon_sym_LBRACE,
    ACTIONS(3084), 1,
      anon_sym_module,
    ACTIONS(3086), 1,
      anon_sym_schema,
    ACTIONS(3096), 1,
      anon_sym_fun,
    STATE(1572), 1,
      sym_spec_func,
    STATE(2288), 1,
      sym_spec_block_target,
    STATE(1673), 2,
      sym_line_comment,
      sym_block_comment,
  [40135] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2850), 1,
      sym_identifier,
    ACTIONS(2852), 1,
      anon_sym_STAR,
    ACTIONS(3098), 1,
      anon_sym_SEMI,
    STATE(1637), 1,
      aux_sym__spec_apply_pattern_repeat1,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    STATE(1851), 1,
      sym__spec_apply_pattern,
    ACTIONS(2860), 2,
      anon_sym_public,
      anon_sym_internal,
    STATE(1674), 2,
      sym_line_comment,
      sym_block_comment,
  [40168] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2850), 1,
      sym_identifier,
    ACTIONS(2852), 1,
      anon_sym_STAR,
    ACTIONS(3100), 1,
      anon_sym_SEMI,
    STATE(1637), 1,
      aux_sym__spec_apply_pattern_repeat1,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    STATE(1851), 1,
      sym__spec_apply_pattern,
    ACTIONS(2860), 2,
      anon_sym_public,
      anon_sym_internal,
    STATE(1675), 2,
      sym_line_comment,
      sym_block_comment,
  [40201] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3102), 1,
      sym_identifier,
    ACTIONS(3104), 1,
      anon_sym_RBRACE,
    STATE(1689), 1,
      aux_sym_enum_body_repeat1,
    STATE(1968), 1,
      sym_enum_variant_struct,
    STATE(2135), 1,
      sym__variant,
    STATE(2370), 1,
      sym__variant_last,
    STATE(1676), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(2136), 2,
      sym_enum_variant_posit,
      sym_enum_variant,
  [40234] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2850), 1,
      sym_identifier,
    ACTIONS(2852), 1,
      anon_sym_STAR,
    ACTIONS(3106), 1,
      anon_sym_SEMI,
    STATE(1637), 1,
      aux_sym__spec_apply_pattern_repeat1,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    STATE(1851), 1,
      sym__spec_apply_pattern,
    ACTIONS(2860), 2,
      anon_sym_public,
      anon_sym_internal,
    STATE(1677), 2,
      sym_line_comment,
      sym_block_comment,
  [40267] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2850), 1,
      sym_identifier,
    ACTIONS(2852), 1,
      anon_sym_STAR,
    ACTIONS(3108), 1,
      anon_sym_SEMI,
    STATE(1637), 1,
      aux_sym__spec_apply_pattern_repeat1,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    STATE(1851), 1,
      sym__spec_apply_pattern,
    ACTIONS(2860), 2,
      anon_sym_public,
      anon_sym_internal,
    STATE(1678), 2,
      sym_line_comment,
      sym_block_comment,
  [40300] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2850), 1,
      sym_identifier,
    ACTIONS(2852), 1,
      anon_sym_STAR,
    ACTIONS(3110), 1,
      anon_sym_SEMI,
    STATE(1637), 1,
      aux_sym__spec_apply_pattern_repeat1,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    STATE(1851), 1,
      sym__spec_apply_pattern,
    ACTIONS(2860), 2,
      anon_sym_public,
      anon_sym_internal,
    STATE(1679), 2,
      sym_line_comment,
      sym_block_comment,
  [40333] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2850), 1,
      sym_identifier,
    ACTIONS(2852), 1,
      anon_sym_STAR,
    ACTIONS(3112), 1,
      anon_sym_SEMI,
    STATE(1637), 1,
      aux_sym__spec_apply_pattern_repeat1,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    STATE(1851), 1,
      sym__spec_apply_pattern,
    ACTIONS(2860), 2,
      anon_sym_public,
      anon_sym_internal,
    STATE(1680), 2,
      sym_line_comment,
      sym_block_comment,
  [40366] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2850), 1,
      sym_identifier,
    ACTIONS(2852), 1,
      anon_sym_STAR,
    ACTIONS(3114), 1,
      anon_sym_SEMI,
    STATE(1637), 1,
      aux_sym__spec_apply_pattern_repeat1,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    STATE(1851), 1,
      sym__spec_apply_pattern,
    ACTIONS(2860), 2,
      anon_sym_public,
      anon_sym_internal,
    STATE(1681), 2,
      sym_line_comment,
      sym_block_comment,
  [40399] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2850), 1,
      sym_identifier,
    ACTIONS(2852), 1,
      anon_sym_STAR,
    ACTIONS(3116), 1,
      anon_sym_SEMI,
    STATE(1637), 1,
      aux_sym__spec_apply_pattern_repeat1,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    STATE(1851), 1,
      sym__spec_apply_pattern,
    ACTIONS(2860), 2,
      anon_sym_public,
      anon_sym_internal,
    STATE(1682), 2,
      sym_line_comment,
      sym_block_comment,
  [40432] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2850), 1,
      sym_identifier,
    ACTIONS(2852), 1,
      anon_sym_STAR,
    ACTIONS(3118), 1,
      anon_sym_SEMI,
    STATE(1637), 1,
      aux_sym__spec_apply_pattern_repeat1,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    STATE(1851), 1,
      sym__spec_apply_pattern,
    ACTIONS(2860), 2,
      anon_sym_public,
      anon_sym_internal,
    STATE(1683), 2,
      sym_line_comment,
      sym_block_comment,
  [40465] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2850), 1,
      sym_identifier,
    ACTIONS(2852), 1,
      anon_sym_STAR,
    ACTIONS(3120), 1,
      anon_sym_SEMI,
    STATE(1637), 1,
      aux_sym__spec_apply_pattern_repeat1,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    STATE(1851), 1,
      sym__spec_apply_pattern,
    ACTIONS(2860), 2,
      anon_sym_public,
      anon_sym_internal,
    STATE(1684), 2,
      sym_line_comment,
      sym_block_comment,
  [40498] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2850), 1,
      sym_identifier,
    ACTIONS(2852), 1,
      anon_sym_STAR,
    ACTIONS(3122), 1,
      anon_sym_SEMI,
    STATE(1637), 1,
      aux_sym__spec_apply_pattern_repeat1,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    STATE(1851), 1,
      sym__spec_apply_pattern,
    ACTIONS(2860), 2,
      anon_sym_public,
      anon_sym_internal,
    STATE(1685), 2,
      sym_line_comment,
      sym_block_comment,
  [40531] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1686), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3124), 8,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [40552] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1687), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3126), 8,
      anon_sym_POUND,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_use,
      anon_sym_const,
      anon_sym_inline,
  [40573] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1578), 1,
      anon_sym_native,
    ACTIONS(3080), 1,
      sym_identifier,
    ACTIONS(3082), 1,
      anon_sym_LBRACE,
    ACTIONS(3084), 1,
      anon_sym_module,
    ACTIONS(3086), 1,
      anon_sym_schema,
    ACTIONS(3096), 1,
      anon_sym_fun,
    STATE(882), 1,
      sym_spec_func,
    STATE(2288), 1,
      sym_spec_block_target,
    STATE(1688), 2,
      sym_line_comment,
      sym_block_comment,
  [40608] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3102), 1,
      sym_identifier,
    ACTIONS(3128), 1,
      anon_sym_RBRACE,
    STATE(1696), 1,
      aux_sym_enum_body_repeat1,
    STATE(1968), 1,
      sym_enum_variant_struct,
    STATE(2135), 1,
      sym__variant,
    STATE(2219), 1,
      sym__variant_last,
    STATE(1689), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(2136), 2,
      sym_enum_variant_posit,
      sym_enum_variant,
  [40641] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1690), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3130), 8,
      anon_sym_COLON,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_pure,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [40662] = 11,
    ACTIONS(11), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3049), 1,
      anon_sym_spec,
    ACTIONS(3132), 1,
      anon_sym_RBRACE,
    STATE(1536), 1,
      aux_sym_attributes_repeat2,
    STATE(1652), 1,
      aux_sym_script_repeat3,
    STATE(1951), 1,
      sym_spec_block,
    STATE(1954), 1,
      sym__script_spec_block,
    STATE(2119), 1,
      sym_attributes,
    STATE(1691), 2,
      sym_line_comment,
      sym_block_comment,
  [40697] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1692), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3134), 8,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_const,
      anon_sym_inline,
      anon_sym_struct,
      anon_sym_enum,
  [40718] = 11,
    ACTIONS(11), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3049), 1,
      anon_sym_spec,
    ACTIONS(3132), 1,
      anon_sym_RBRACE,
    STATE(1536), 1,
      aux_sym_attributes_repeat2,
    STATE(1653), 1,
      aux_sym_script_repeat3,
    STATE(1951), 1,
      sym_spec_block,
    STATE(1954), 1,
      sym__script_spec_block,
    STATE(2119), 1,
      sym_attributes,
    STATE(1693), 2,
      sym_line_comment,
      sym_block_comment,
  [40753] = 11,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1578), 1,
      anon_sym_native,
    ACTIONS(3080), 1,
      sym_identifier,
    ACTIONS(3082), 1,
      anon_sym_LBRACE,
    ACTIONS(3084), 1,
      anon_sym_module,
    ACTIONS(3086), 1,
      anon_sym_schema,
    ACTIONS(3096), 1,
      anon_sym_fun,
    STATE(1586), 1,
      sym_spec_func,
    STATE(2288), 1,
      sym_spec_block_target,
    STATE(1694), 2,
      sym_line_comment,
      sym_block_comment,
  [40788] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3138), 1,
      anon_sym_where,
    ACTIONS(3140), 1,
      anon_sym_COLON,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    STATE(1784), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2134), 1,
      sym_triggers,
    STATE(1695), 2,
      sym_line_comment,
      sym_block_comment,
  [40820] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3144), 1,
      sym_identifier,
    ACTIONS(3147), 1,
      anon_sym_RBRACE,
    STATE(1967), 1,
      sym_enum_variant_struct,
    STATE(2135), 1,
      sym__variant,
    STATE(2307), 2,
      sym_enum_variant_posit,
      sym_enum_variant,
    STATE(1696), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_enum_body_repeat1,
  [40848] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3149), 1,
      anon_sym_where,
    ACTIONS(3151), 1,
      anon_sym_COLON,
    STATE(1698), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2106), 1,
      sym_triggers,
    STATE(1697), 2,
      sym_line_comment,
      sym_block_comment,
  [40880] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3153), 1,
      anon_sym_where,
    ACTIONS(3155), 1,
      anon_sym_COLON,
    STATE(1784), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2108), 1,
      sym_triggers,
    STATE(1698), 2,
      sym_line_comment,
      sym_block_comment,
  [40912] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3157), 1,
      anon_sym_where,
    ACTIONS(3159), 1,
      anon_sym_COLON,
    STATE(1700), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2111), 1,
      sym_triggers,
    STATE(1699), 2,
      sym_line_comment,
      sym_block_comment,
  [40944] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3161), 1,
      anon_sym_where,
    ACTIONS(3163), 1,
      anon_sym_COLON,
    STATE(1784), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2113), 1,
      sym_triggers,
    STATE(1700), 2,
      sym_line_comment,
      sym_block_comment,
  [40976] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3165), 1,
      anon_sym_COLON_COLON,
    STATE(1712), 1,
      aux_sym__attribute_name_repeat1,
    STATE(1701), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3167), 5,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_RBRACK,
  [41000] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3171), 1,
      anon_sym_BANG,
    ACTIONS(3169), 2,
      anon_sym_LBRACE,
      anon_sym_SEMI,
    ACTIONS(3174), 3,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
    STATE(1702), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym__specifier_repeat1,
  [41024] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1703), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3177), 7,
      anon_sym_POUND,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_const,
      anon_sym_inline,
  [41044] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3179), 1,
      anon_sym_where,
    ACTIONS(3181), 1,
      anon_sym_COLON,
    STATE(1707), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2022), 1,
      sym_triggers,
    STATE(1704), 2,
      sym_line_comment,
      sym_block_comment,
  [41076] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1705), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3183), 7,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [41096] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3185), 1,
      anon_sym_where,
    ACTIONS(3187), 1,
      anon_sym_COLON,
    STATE(1784), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2077), 1,
      sym_triggers,
    STATE(1706), 2,
      sym_line_comment,
      sym_block_comment,
  [41128] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3189), 1,
      anon_sym_where,
    ACTIONS(3191), 1,
      anon_sym_COLON,
    STATE(1784), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2031), 1,
      sym_triggers,
    STATE(1707), 2,
      sym_line_comment,
      sym_block_comment,
  [41160] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1708), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3193), 7,
      ts_builtin_sym_end,
      anon_sym_address,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_module,
      anon_sym_spec,
      anon_sym_script,
  [41180] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3195), 1,
      anon_sym_where,
    ACTIONS(3197), 1,
      anon_sym_COLON,
    STATE(1710), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2027), 1,
      sym_triggers,
    STATE(1709), 2,
      sym_line_comment,
      sym_block_comment,
  [41212] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3199), 1,
      anon_sym_where,
    ACTIONS(3201), 1,
      anon_sym_COLON,
    STATE(1784), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2029), 1,
      sym_triggers,
    STATE(1710), 2,
      sym_line_comment,
      sym_block_comment,
  [41244] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3165), 1,
      anon_sym_COLON_COLON,
    STATE(1701), 1,
      aux_sym__attribute_name_repeat1,
    STATE(1711), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3203), 5,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_RBRACK,
  [41268] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3205), 1,
      anon_sym_COLON_COLON,
    STATE(1712), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym__attribute_name_repeat1,
    ACTIONS(3208), 5,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_RBRACK,
  [41290] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1713), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2820), 7,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_RBRACE,
      anon_sym_RBRACK,
      anon_sym_SEMI,
  [41310] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3210), 1,
      anon_sym_where,
    ACTIONS(3212), 1,
      anon_sym_COLON,
    STATE(1715), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2059), 1,
      sym_triggers,
    STATE(1714), 2,
      sym_line_comment,
      sym_block_comment,
  [41342] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3214), 1,
      anon_sym_where,
    ACTIONS(3216), 1,
      anon_sym_COLON,
    STATE(1784), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2061), 1,
      sym_triggers,
    STATE(1715), 2,
      sym_line_comment,
      sym_block_comment,
  [41374] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3218), 1,
      anon_sym_LPAREN,
    ACTIONS(3220), 1,
      anon_sym_LBRACE,
    ACTIONS(3223), 1,
      anon_sym_SEMI,
    ACTIONS(3225), 1,
      anon_sym_has,
    STATE(1959), 1,
      sym_struct_body,
    STATE(1960), 1,
      sym_anon_fields,
    STATE(2127), 1,
      sym_abilities,
    STATE(1716), 2,
      sym_line_comment,
      sym_block_comment,
  [41406] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1717), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3227), 7,
      anon_sym_POUND,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_const,
      anon_sym_inline,
  [41426] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1718), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3229), 7,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [41446] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3231), 1,
      anon_sym_where,
    ACTIONS(3233), 1,
      anon_sym_COLON,
    STATE(1750), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2100), 1,
      sym_triggers,
    STATE(1719), 2,
      sym_line_comment,
      sym_block_comment,
  [41478] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3235), 1,
      anon_sym_where,
    ACTIONS(3237), 1,
      anon_sym_COLON,
    STATE(1721), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2069), 1,
      sym_triggers,
    STATE(1720), 2,
      sym_line_comment,
      sym_block_comment,
  [41510] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3239), 1,
      anon_sym_where,
    ACTIONS(3241), 1,
      anon_sym_COLON,
    STATE(1784), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2070), 1,
      sym_triggers,
    STATE(1721), 2,
      sym_line_comment,
      sym_block_comment,
  [41542] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1722), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3243), 7,
      ts_builtin_sym_end,
      anon_sym_address,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_module,
      anon_sym_spec,
      anon_sym_script,
  [41562] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3245), 1,
      anon_sym_where,
    ACTIONS(3247), 1,
      anon_sym_COLON,
    STATE(1742), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2062), 1,
      sym_triggers,
    STATE(1723), 2,
      sym_line_comment,
      sym_block_comment,
  [41594] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3249), 1,
      anon_sym_where,
    ACTIONS(3251), 1,
      anon_sym_COLON,
    STATE(1725), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2079), 1,
      sym_triggers,
    STATE(1724), 2,
      sym_line_comment,
      sym_block_comment,
  [41626] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3253), 1,
      anon_sym_where,
    ACTIONS(3255), 1,
      anon_sym_COLON,
    STATE(1784), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2080), 1,
      sym_triggers,
    STATE(1725), 2,
      sym_line_comment,
      sym_block_comment,
  [41658] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3257), 1,
      anon_sym_where,
    ACTIONS(3259), 1,
      anon_sym_COLON,
    STATE(1729), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2089), 1,
      sym_triggers,
    STATE(1726), 2,
      sym_line_comment,
      sym_block_comment,
  [41690] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3261), 1,
      anon_sym_where,
    ACTIONS(3263), 1,
      anon_sym_COLON,
    STATE(1728), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2086), 1,
      sym_triggers,
    STATE(1727), 2,
      sym_line_comment,
      sym_block_comment,
  [41722] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3265), 1,
      anon_sym_where,
    ACTIONS(3267), 1,
      anon_sym_COLON,
    STATE(1784), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2087), 1,
      sym_triggers,
    STATE(1728), 2,
      sym_line_comment,
      sym_block_comment,
  [41754] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3269), 1,
      anon_sym_where,
    ACTIONS(3271), 1,
      anon_sym_COLON,
    STATE(1784), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2099), 1,
      sym_triggers,
    STATE(1729), 2,
      sym_line_comment,
      sym_block_comment,
  [41786] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3273), 1,
      anon_sym_where,
    ACTIONS(3275), 1,
      anon_sym_COLON,
    STATE(1731), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2095), 1,
      sym_triggers,
    STATE(1730), 2,
      sym_line_comment,
      sym_block_comment,
  [41818] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3277), 1,
      anon_sym_where,
    ACTIONS(3279), 1,
      anon_sym_COLON,
    STATE(1784), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2096), 1,
      sym_triggers,
    STATE(1731), 2,
      sym_line_comment,
      sym_block_comment,
  [41850] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3281), 1,
      anon_sym_where,
    ACTIONS(3283), 1,
      anon_sym_COLON,
    STATE(1733), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2037), 1,
      sym_triggers,
    STATE(1732), 2,
      sym_line_comment,
      sym_block_comment,
  [41882] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3285), 1,
      anon_sym_where,
    ACTIONS(3287), 1,
      anon_sym_COLON,
    STATE(1784), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2060), 1,
      sym_triggers,
    STATE(1733), 2,
      sym_line_comment,
      sym_block_comment,
  [41914] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3289), 1,
      anon_sym_where,
    ACTIONS(3291), 1,
      anon_sym_COLON,
    STATE(1706), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2016), 1,
      sym_triggers,
    STATE(1734), 2,
      sym_line_comment,
      sym_block_comment,
  [41946] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1735), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3293), 7,
      anon_sym_POUND,
      anon_sym_fun,
      anon_sym_native,
      anon_sym_public,
      anon_sym_entry,
      anon_sym_const,
      anon_sym_inline,
  [41966] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1736), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3070), 7,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [41986] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3295), 1,
      anon_sym_where,
    ACTIONS(3297), 1,
      anon_sym_COLON,
    STATE(1695), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2128), 1,
      sym_triggers,
    STATE(1737), 2,
      sym_line_comment,
      sym_block_comment,
  [42018] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3299), 1,
      anon_sym_where,
    ACTIONS(3301), 1,
      anon_sym_COLON,
    STATE(1739), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2130), 1,
      sym_triggers,
    STATE(1738), 2,
      sym_line_comment,
      sym_block_comment,
  [42050] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3303), 1,
      anon_sym_where,
    ACTIONS(3305), 1,
      anon_sym_COLON,
    STATE(1784), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2132), 1,
      sym_triggers,
    STATE(1739), 2,
      sym_line_comment,
      sym_block_comment,
  [42082] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3307), 1,
      anon_sym_where,
    ACTIONS(3309), 1,
      anon_sym_COLON,
    STATE(1741), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2137), 1,
      sym_triggers,
    STATE(1740), 2,
      sym_line_comment,
      sym_block_comment,
  [42114] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3311), 1,
      anon_sym_where,
    ACTIONS(3313), 1,
      anon_sym_COLON,
    STATE(1784), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2139), 1,
      sym_triggers,
    STATE(1741), 2,
      sym_line_comment,
      sym_block_comment,
  [42146] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3315), 1,
      anon_sym_where,
    ACTIONS(3317), 1,
      anon_sym_COLON,
    STATE(1784), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2032), 1,
      sym_triggers,
    STATE(1742), 2,
      sym_line_comment,
      sym_block_comment,
  [42178] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3319), 1,
      anon_sym_where,
    ACTIONS(3321), 1,
      anon_sym_COLON,
    STATE(1747), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2157), 1,
      sym_triggers,
    STATE(1743), 2,
      sym_line_comment,
      sym_block_comment,
  [42210] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2887), 1,
      anon_sym_BANG,
    STATE(1702), 1,
      aux_sym__specifier_repeat1,
    ACTIONS(3323), 2,
      anon_sym_LBRACE,
      anon_sym_SEMI,
    STATE(1744), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2891), 3,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [42236] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3325), 1,
      anon_sym_where,
    ACTIONS(3327), 1,
      anon_sym_COLON,
    STATE(1746), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2151), 1,
      sym_triggers,
    STATE(1745), 2,
      sym_line_comment,
      sym_block_comment,
  [42268] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3329), 1,
      anon_sym_where,
    ACTIONS(3331), 1,
      anon_sym_COLON,
    STATE(1784), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2152), 1,
      sym_triggers,
    STATE(1746), 2,
      sym_line_comment,
      sym_block_comment,
  [42300] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3333), 1,
      anon_sym_where,
    ACTIONS(3335), 1,
      anon_sym_COLON,
    STATE(1784), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2161), 1,
      sym_triggers,
    STATE(1747), 2,
      sym_line_comment,
      sym_block_comment,
  [42332] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3337), 1,
      anon_sym_where,
    ACTIONS(3339), 1,
      anon_sym_COLON,
    STATE(1749), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2162), 1,
      sym_triggers,
    STATE(1748), 2,
      sym_line_comment,
      sym_block_comment,
  [42364] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3341), 1,
      anon_sym_where,
    ACTIONS(3343), 1,
      anon_sym_COLON,
    STATE(1784), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2163), 1,
      sym_triggers,
    STATE(1749), 2,
      sym_line_comment,
      sym_block_comment,
  [42396] = 10,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3136), 1,
      anon_sym_COMMA,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    ACTIONS(3345), 1,
      anon_sym_where,
    ACTIONS(3347), 1,
      anon_sym_COLON,
    STATE(1784), 1,
      aux_sym_quantifier_repeat1,
    STATE(1807), 1,
      aux_sym_triggers_repeat2,
    STATE(2051), 1,
      sym_triggers,
    STATE(1750), 2,
      sym_line_comment,
      sym_block_comment,
  [42428] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2850), 1,
      sym_identifier,
    ACTIONS(2852), 1,
      anon_sym_STAR,
    STATE(1637), 1,
      aux_sym__spec_apply_pattern_repeat1,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    STATE(1851), 1,
      sym__spec_apply_pattern,
    ACTIONS(2860), 2,
      anon_sym_public,
      anon_sym_internal,
    STATE(1751), 2,
      sym_line_comment,
      sym_block_comment,
  [42458] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1752), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3349), 6,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [42477] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1753), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3351), 6,
      ts_builtin_sym_end,
      anon_sym_address,
      anon_sym_POUND,
      anon_sym_module,
      anon_sym_spec,
      anon_sym_script,
  [42496] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3355), 1,
      anon_sym_LT,
    STATE(1839), 1,
      sym__struct_type_params,
    STATE(1754), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3353), 4,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_SEMI,
      anon_sym_has,
  [42519] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(393), 1,
      anon_sym_LT,
    ACTIONS(3357), 1,
      anon_sym_LPAREN,
    ACTIONS(3359), 1,
      anon_sym_LBRACE,
    STATE(2090), 1,
      sym_parameters,
    STATE(2104), 1,
      sym_type_params,
    STATE(2267), 1,
      sym__spec_target_signature_opt,
    STATE(1755), 2,
      sym_line_comment,
      sym_block_comment,
  [42548] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1756), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3361), 6,
      anon_sym_COLON_COLON,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_RBRACK,
  [42567] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1757), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3363), 6,
      ts_builtin_sym_end,
      anon_sym_address,
      anon_sym_POUND,
      anon_sym_module,
      anon_sym_spec,
      anon_sym_script,
  [42586] = 9,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3365), 1,
      sym_identifier,
    ACTIONS(3367), 1,
      anon_sym_COMMA,
    ACTIONS(3369), 1,
      anon_sym_GT,
    ACTIONS(3371), 1,
      sym_phantom,
    STATE(1907), 1,
      sym__struct_type_parameter,
    STATE(2035), 1,
      sym_type_param,
    STATE(1758), 2,
      sym_line_comment,
      sym_block_comment,
  [42615] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2706), 1,
      anon_sym_COLON_COLON,
    ACTIONS(2820), 2,
      anon_sym_COMMA,
      anon_sym_PIPE,
    STATE(1759), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(829), 3,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_LT2,
  [42638] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1760), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3373), 6,
      ts_builtin_sym_end,
      anon_sym_address,
      anon_sym_POUND,
      anon_sym_module,
      anon_sym_spec,
      anon_sym_script,
  [42657] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3218), 1,
      anon_sym_LPAREN,
    ACTIONS(3377), 1,
      anon_sym_LBRACE,
    STATE(1971), 1,
      sym_struct_body,
    STATE(2082), 1,
      sym_anon_fields,
    ACTIONS(3375), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
    STATE(1761), 2,
      sym_line_comment,
      sym_block_comment,
  [42684] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1762), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3379), 6,
      ts_builtin_sym_end,
      anon_sym_address,
      anon_sym_POUND,
      anon_sym_module,
      anon_sym_spec,
      anon_sym_script,
  [42703] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3381), 2,
      sym_identifier,
      anon_sym_except,
    STATE(1763), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3383), 4,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_LT,
      anon_sym_SEMI,
  [42724] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3385), 2,
      sym_identifier,
      anon_sym_except,
    STATE(1764), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3387), 4,
      anon_sym_STAR,
      anon_sym_COMMA,
      anon_sym_LT,
      anon_sym_SEMI,
  [42745] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1765), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3389), 6,
      ts_builtin_sym_end,
      anon_sym_address,
      anon_sym_POUND,
      anon_sym_module,
      anon_sym_spec,
      anon_sym_script,
  [42764] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1766), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2801), 6,
      ts_builtin_sym_end,
      anon_sym_address,
      anon_sym_POUND,
      anon_sym_module,
      anon_sym_spec,
      anon_sym_script,
  [42783] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1767), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3391), 6,
      ts_builtin_sym_end,
      anon_sym_address,
      anon_sym_POUND,
      anon_sym_module,
      anon_sym_spec,
      anon_sym_script,
  [42802] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1768), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3393), 6,
      ts_builtin_sym_end,
      anon_sym_address,
      anon_sym_POUND,
      anon_sym_module,
      anon_sym_spec,
      anon_sym_script,
  [42821] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1769), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3395), 6,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_SEMI,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [42840] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1896), 1,
      sym_ability,
    STATE(1770), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3397), 4,
      anon_sym_copy,
      anon_sym_drop,
      anon_sym_store,
      anon_sym_key,
  [42860] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3399), 1,
      anon_sym_COMMA,
    ACTIONS(3401), 1,
      anon_sym_RPAREN,
    ACTIONS(3403), 1,
      anon_sym_COLON,
    ACTIONS(3405), 1,
      anon_sym_as,
    STATE(1935), 1,
      aux_sym_vector_value_expr_repeat1,
    STATE(1771), 2,
      sym_line_comment,
      sym_block_comment,
  [42886] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1841), 1,
      sym_ability,
    STATE(1772), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3397), 4,
      anon_sym_copy,
      anon_sym_drop,
      anon_sym_store,
      anon_sym_key,
  [42906] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3407), 1,
      anon_sym_LPAREN,
    ACTIONS(3411), 1,
      anon_sym_EQ,
    STATE(1773), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3409), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
  [42928] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3413), 1,
      sym_identifier,
    ACTIONS(3415), 1,
      anon_sym_COMMA,
    ACTIONS(3417), 1,
      anon_sym_RBRACK,
    STATE(1773), 1,
      sym__attribute_name,
    STATE(1852), 1,
      sym_attribute,
    STATE(1774), 2,
      sym_line_comment,
      sym_block_comment,
  [42954] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3421), 1,
      aux_sym_number_token1,
    STATE(1775), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3419), 4,
      sym_identifier,
      aux_sym_number_token2,
      aux_sym_number_token4,
      aux_sym_number_token6,
  [42974] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3423), 1,
      anon_sym_COMMA,
    ACTIONS(3426), 3,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_SEMI,
    STATE(1776), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_vector_value_expr_repeat1,
  [42994] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3365), 1,
      sym_identifier,
    ACTIONS(3371), 1,
      sym_phantom,
    ACTIONS(3428), 1,
      anon_sym_GT,
    STATE(2035), 1,
      sym_type_param,
    STATE(2046), 1,
      sym__struct_type_parameter,
    STATE(1777), 2,
      sym_line_comment,
      sym_block_comment,
  [43020] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1778), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3430), 5,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SEMI,
  [43038] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3432), 1,
      anon_sym_EQ,
    STATE(1779), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3434), 4,
      anon_sym_COLON,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [43058] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3413), 1,
      sym_identifier,
    ACTIONS(3436), 1,
      anon_sym_COMMA,
    ACTIONS(3438), 1,
      anon_sym_RPAREN,
    STATE(1773), 1,
      sym__attribute_name,
    STATE(1947), 1,
      sym_attribute,
    STATE(1780), 2,
      sym_line_comment,
      sym_block_comment,
  [43084] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3440), 1,
      anon_sym_EQ,
    STATE(1781), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3442), 4,
      anon_sym_COLON,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [43104] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3444), 1,
      anon_sym_EQ,
    STATE(1782), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3446), 4,
      anon_sym_COLON,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [43124] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3448), 1,
      anon_sym_EQ,
    STATE(1783), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3450), 4,
      anon_sym_COLON,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [43144] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3452), 1,
      anon_sym_COMMA,
    ACTIONS(3455), 3,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
    STATE(1784), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_quantifier_repeat1,
  [43164] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3459), 1,
      anon_sym_as,
    STATE(1878), 1,
      sym__use_alias,
    STATE(1785), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3457), 3,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [43186] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3461), 1,
      anon_sym_EQ,
    STATE(1786), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3463), 4,
      anon_sym_COLON,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [43206] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3465), 1,
      anon_sym_else,
    STATE(1787), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1329), 4,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_LBRACK,
      anon_sym_spec,
  [43226] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1927), 1,
      sym_ability,
    STATE(1788), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3397), 4,
      anon_sym_copy,
      anon_sym_drop,
      anon_sym_store,
      anon_sym_key,
  [43246] = 8,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3365), 1,
      sym_identifier,
    ACTIONS(3371), 1,
      sym_phantom,
    ACTIONS(3467), 1,
      anon_sym_GT,
    STATE(2035), 1,
      sym_type_param,
    STATE(2046), 1,
      sym__struct_type_parameter,
    STATE(1789), 2,
      sym_line_comment,
      sym_block_comment,
  [43272] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3469), 1,
      anon_sym_EQ,
    STATE(1790), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3471), 4,
      anon_sym_COLON,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [43292] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3473), 1,
      anon_sym_EQ,
    STATE(1791), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3475), 4,
      anon_sym_COLON,
      anon_sym_if,
      anon_sym_EQ_GT,
      anon_sym_SEMI,
  [43312] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1840), 1,
      sym_ability,
    STATE(1792), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3397), 4,
      anon_sym_copy,
      anon_sym_drop,
      anon_sym_store,
      anon_sym_key,
  [43332] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3477), 1,
      anon_sym_COMMA,
    ACTIONS(3480), 2,
      anon_sym_SEMI,
      anon_sym_except,
    STATE(1793), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_spec_apply_repeat1,
  [43351] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3482), 1,
      sym_identifier,
    ACTIONS(3484), 1,
      anon_sym_COMMA,
    ACTIONS(3486), 1,
      anon_sym_GT,
    STATE(1911), 1,
      sym_type_param,
    STATE(1794), 2,
      sym_line_comment,
      sym_block_comment,
  [43374] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3488), 1,
      sym_identifier,
    ACTIONS(3490), 1,
      anon_sym_COMMA,
    ACTIONS(3492), 1,
      anon_sym_RBRACE,
    STATE(1866), 1,
      sym__use_member,
    STATE(1795), 2,
      sym_line_comment,
      sym_block_comment,
  [43397] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1327), 1,
      anon_sym_DOT,
    ACTIONS(3494), 1,
      anon_sym_else,
    ACTIONS(1329), 2,
      anon_sym_DOT_DOT,
      anon_sym_LBRACK,
    STATE(1796), 2,
      sym_line_comment,
      sym_block_comment,
  [43418] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3496), 1,
      anon_sym_COMMA,
    ACTIONS(3498), 1,
      anon_sym_SEMI,
    ACTIONS(3500), 1,
      anon_sym_except,
    STATE(1793), 1,
      aux_sym_spec_apply_repeat1,
    STATE(1797), 2,
      sym_line_comment,
      sym_block_comment,
  [43441] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3502), 1,
      sym_identifier,
    ACTIONS(3504), 1,
      anon_sym_COMMA,
    ACTIONS(3506), 1,
      anon_sym_RBRACE,
    STATE(1984), 1,
      sym_expr_field,
    STATE(1798), 2,
      sym_line_comment,
      sym_block_comment,
  [43464] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3508), 1,
      anon_sym_else,
    STATE(1799), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1329), 3,
      anon_sym_EQ,
      anon_sym_DOT,
      anon_sym_LBRACK,
  [43483] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1621), 1,
      aux_sym__spec_apply_pattern_repeat1,
    STATE(1764), 1,
      sym__spec_apply_fragment,
    ACTIONS(2852), 2,
      sym_identifier,
      anon_sym_STAR,
    STATE(1800), 2,
      sym_line_comment,
      sym_block_comment,
  [43504] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3510), 1,
      anon_sym_COMMA,
    ACTIONS(3512), 1,
      anon_sym_SEMI,
    ACTIONS(3514), 1,
      anon_sym_except,
    STATE(1797), 1,
      aux_sym_spec_apply_repeat1,
    STATE(1801), 2,
      sym_line_comment,
      sym_block_comment,
  [43527] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3518), 1,
      anon_sym_PLUS,
    ACTIONS(3516), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(1802), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_constraints_repeat1,
  [43546] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3523), 1,
      anon_sym_COLON,
    STATE(2109), 1,
      sym_constraints,
    ACTIONS(3521), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(1803), 2,
      sym_line_comment,
      sym_block_comment,
  [43567] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1804), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3525), 4,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_SEMI,
      anon_sym_has,
  [43584] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3365), 1,
      sym_identifier,
    ACTIONS(3371), 1,
      sym_phantom,
    STATE(2035), 1,
      sym_type_param,
    STATE(2046), 1,
      sym__struct_type_parameter,
    STATE(1805), 2,
      sym_line_comment,
      sym_block_comment,
  [43607] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1806), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3527), 4,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_SEMI,
      anon_sym_has,
  [43624] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3142), 1,
      anon_sym_LBRACE,
    STATE(1825), 1,
      aux_sym_triggers_repeat2,
    ACTIONS(3529), 2,
      anon_sym_where,
      anon_sym_COLON,
    STATE(1807), 2,
      sym_line_comment,
      sym_block_comment,
  [43645] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3413), 1,
      sym_identifier,
    ACTIONS(3531), 1,
      anon_sym_RPAREN,
    STATE(1773), 1,
      sym__attribute_name,
    STATE(1874), 1,
      sym_attribute,
    STATE(1808), 2,
      sym_line_comment,
      sym_block_comment,
  [43668] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1809), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3533), 4,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_GT,
  [43685] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3537), 1,
      anon_sym_EQ,
    STATE(1810), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3535), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_SEMI,
  [43704] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3539), 1,
      anon_sym_COMMA,
    ACTIONS(3533), 2,
      anon_sym_PIPE,
      anon_sym_GT,
    STATE(1811), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_tuple_type_repeat1,
  [43723] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3049), 1,
      anon_sym_spec,
    ACTIONS(3542), 1,
      anon_sym_RPAREN,
    STATE(944), 1,
      sym_spec_block,
    STATE(2194), 1,
      sym_spec_loop_invariant,
    STATE(1812), 2,
      sym_line_comment,
      sym_block_comment,
  [43746] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3225), 1,
      anon_sym_has,
    ACTIONS(3544), 1,
      anon_sym_LBRACE,
    STATE(1962), 1,
      sym_enum_body,
    STATE(2346), 1,
      sym_abilities,
    STATE(1813), 2,
      sym_line_comment,
      sym_block_comment,
  [43769] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3547), 1,
      sym_identifier,
    ACTIONS(3549), 1,
      anon_sym_COMMA,
    ACTIONS(3551), 1,
      anon_sym_RBRACE,
    STATE(1966), 1,
      sym_field_annot,
    STATE(1814), 2,
      sym_line_comment,
      sym_block_comment,
  [43792] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1815), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3553), 4,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_SEMI,
      anon_sym_has,
  [43809] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1816), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3555), 4,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
  [43826] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1817), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3557), 4,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
  [43843] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1818), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3455), 4,
      anon_sym_COMMA,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
  [43860] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3561), 1,
      anon_sym_PLUS,
    STATE(1802), 1,
      aux_sym_constraints_repeat1,
    ACTIONS(3559), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(1819), 2,
      sym_line_comment,
      sym_block_comment,
  [43881] = 7,
    ACTIONS(13), 1,
      anon_sym_module,
    ACTIONS(15), 1,
      anon_sym_spec,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1646), 1,
      sym__module_keyword,
    STATE(1821), 1,
      sym_module,
    STATE(1820), 2,
      sym_line_comment,
      sym_block_comment,
  [43904] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1821), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3563), 4,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_module,
      anon_sym_spec,
  [43921] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3565), 1,
      sym_identifier,
    ACTIONS(3567), 1,
      anon_sym_COMMA,
    ACTIONS(3569), 1,
      anon_sym_RPAREN,
    STATE(1976), 1,
      sym_parameter,
    STATE(1822), 2,
      sym_line_comment,
      sym_block_comment,
  [43944] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(393), 1,
      anon_sym_LT,
    ACTIONS(3357), 1,
      anon_sym_LPAREN,
    STATE(2075), 1,
      sym_type_params,
    STATE(2192), 1,
      sym_parameters,
    STATE(1823), 2,
      sym_line_comment,
      sym_block_comment,
  [43967] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1824), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3571), 4,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_module,
      anon_sym_spec,
  [43984] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3575), 1,
      anon_sym_LBRACE,
    ACTIONS(3573), 2,
      anon_sym_where,
      anon_sym_COLON,
    STATE(1825), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_triggers_repeat2,
  [44003] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3413), 1,
      sym_identifier,
    ACTIONS(3578), 1,
      anon_sym_RBRACK,
    STATE(1773), 1,
      sym__attribute_name,
    STATE(1874), 1,
      sym_attribute,
    STATE(1826), 2,
      sym_line_comment,
      sym_block_comment,
  [44026] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3502), 1,
      sym_identifier,
    ACTIONS(3580), 1,
      anon_sym_COMMA,
    ACTIONS(3582), 1,
      anon_sym_RBRACE,
    STATE(1995), 1,
      sym_expr_field,
    STATE(1827), 2,
      sym_line_comment,
      sym_block_comment,
  [44049] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1828), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3584), 4,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_SEMI,
      anon_sym_has,
  [44066] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1829), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3586), 4,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_SEMI,
      anon_sym_has,
  [44083] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(393), 1,
      anon_sym_LT,
    ACTIONS(3357), 1,
      anon_sym_LPAREN,
    STATE(1614), 1,
      sym_parameters,
    STATE(2167), 1,
      sym_type_params,
    STATE(1830), 2,
      sym_line_comment,
      sym_block_comment,
  [44106] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1831), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3426), 4,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_SEMI,
  [44123] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3588), 1,
      anon_sym_COMMA,
    ACTIONS(3591), 2,
      anon_sym_LBRACE,
      anon_sym_SEMI,
    STATE(1832), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_abilities_repeat1,
  [44142] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(393), 1,
      anon_sym_LT,
    ACTIONS(3357), 1,
      anon_sym_LPAREN,
    STATE(1609), 1,
      sym_parameters,
    STATE(2083), 1,
      sym_type_params,
    STATE(1833), 2,
      sym_line_comment,
      sym_block_comment,
  [44165] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3459), 1,
      anon_sym_as,
    ACTIONS(3593), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3595), 1,
      anon_sym_SEMI,
    STATE(2275), 1,
      sym__use_alias,
    STATE(1834), 2,
      sym_line_comment,
      sym_block_comment,
  [44188] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2751), 1,
      anon_sym_LT2,
    ACTIONS(3597), 1,
      anon_sym_LPAREN,
    ACTIONS(3599), 1,
      anon_sym_RPAREN,
    STATE(2229), 1,
      sym_type_args,
    STATE(1835), 2,
      sym_line_comment,
      sym_block_comment,
  [44211] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3601), 1,
      anon_sym_COMMA,
    ACTIONS(3604), 2,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
    STATE(1836), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_attributes_repeat1,
  [44230] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3459), 1,
      anon_sym_as,
    ACTIONS(3606), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3608), 1,
      anon_sym_SEMI,
    STATE(2426), 1,
      sym__use_alias,
    STATE(1837), 2,
      sym_line_comment,
      sym_block_comment,
  [44253] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1838), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3610), 4,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_SEMI,
      anon_sym_has,
  [44270] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1839), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3612), 4,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_SEMI,
      anon_sym_has,
  [44287] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3614), 1,
      anon_sym_COMMA,
    STATE(1846), 1,
      aux_sym_abilities_repeat1,
    ACTIONS(3616), 2,
      anon_sym_LBRACE,
      anon_sym_SEMI,
    STATE(1840), 2,
      sym_line_comment,
      sym_block_comment,
  [44308] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3561), 1,
      anon_sym_PLUS,
    STATE(1819), 1,
      aux_sym_constraints_repeat1,
    ACTIONS(3618), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(1841), 2,
      sym_line_comment,
      sym_block_comment,
  [44329] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3413), 1,
      sym_identifier,
    ACTIONS(3620), 1,
      anon_sym_RPAREN,
    STATE(1773), 1,
      sym__attribute_name,
    STATE(1874), 1,
      sym_attribute,
    STATE(1842), 2,
      sym_line_comment,
      sym_block_comment,
  [44352] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1843), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3622), 4,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_module,
      anon_sym_spec,
  [44369] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1844), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3624), 4,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_SEMI,
      anon_sym_has,
  [44386] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    ACTIONS(3629), 2,
      anon_sym_RBRACK,
      anon_sym_SEMI,
    STATE(1845), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_condition_props_repeat1,
  [44405] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3614), 1,
      anon_sym_COMMA,
    STATE(1832), 1,
      aux_sym_abilities_repeat1,
    ACTIONS(3631), 2,
      anon_sym_LBRACE,
      anon_sym_SEMI,
    STATE(1846), 2,
      sym_line_comment,
      sym_block_comment,
  [44426] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3413), 1,
      sym_identifier,
    ACTIONS(3633), 1,
      anon_sym_RBRACK,
    STATE(1773), 1,
      sym__attribute_name,
    STATE(1874), 1,
      sym_attribute,
    STATE(1847), 2,
      sym_line_comment,
      sym_block_comment,
  [44449] = 7,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3488), 1,
      sym_identifier,
    ACTIONS(3635), 1,
      anon_sym_COMMA,
    ACTIONS(3637), 1,
      anon_sym_RBRACE,
    STATE(1897), 1,
      sym__use_member,
    STATE(1848), 2,
      sym_line_comment,
      sym_block_comment,
  [44472] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1849), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3639), 4,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_SEMI,
      anon_sym_has,
  [44489] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3565), 1,
      sym_identifier,
    ACTIONS(3641), 1,
      anon_sym_RPAREN,
    STATE(2023), 1,
      sym_parameter,
    STATE(1850), 2,
      sym_line_comment,
      sym_block_comment,
  [44509] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1851), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3643), 3,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_except,
  [44525] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3645), 1,
      anon_sym_COMMA,
    ACTIONS(3647), 1,
      anon_sym_RBRACK,
    STATE(1924), 1,
      aux_sym_attributes_repeat1,
    STATE(1852), 2,
      sym_line_comment,
      sym_block_comment,
  [44545] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3565), 1,
      sym_identifier,
    ACTIONS(3649), 1,
      anon_sym_RPAREN,
    STATE(2023), 1,
      sym_parameter,
    STATE(1853), 2,
      sym_line_comment,
      sym_block_comment,
  [44565] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3651), 1,
      anon_sym_COMMA,
    ACTIONS(3653), 1,
      anon_sym_SEMI,
    STATE(1776), 1,
      aux_sym_vector_value_expr_repeat1,
    STATE(1854), 2,
      sym_line_comment,
      sym_block_comment,
  [44585] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3651), 1,
      anon_sym_COMMA,
    ACTIONS(3653), 1,
      anon_sym_SEMI,
    STATE(2000), 1,
      aux_sym_vector_value_expr_repeat1,
    STATE(1855), 2,
      sym_line_comment,
      sym_block_comment,
  [44605] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3655), 1,
      anon_sym_COMMA,
    ACTIONS(3657), 1,
      anon_sym_RBRACE,
    STATE(1902), 1,
      aux_sym_pack_expr_repeat1,
    STATE(1856), 2,
      sym_line_comment,
      sym_block_comment,
  [44625] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3659), 1,
      anon_sym_COMMA,
    ACTIONS(3661), 1,
      anon_sym_SEMI,
    STATE(1890), 1,
      aux_sym_spec_apply_repeat1,
    STATE(1857), 2,
      sym_line_comment,
      sym_block_comment,
  [44645] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1858), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3663), 3,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
  [44661] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1859), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3665), 3,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
  [44677] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3667), 1,
      anon_sym_COMMA,
    ACTIONS(3669), 1,
      anon_sym_SEMI,
    STATE(1987), 1,
      aux_sym_spec_apply_repeat1,
    STATE(1860), 2,
      sym_line_comment,
      sym_block_comment,
  [44697] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3671), 1,
      anon_sym_COMMA,
    ACTIONS(3673), 1,
      anon_sym_RBRACK,
    STATE(1884), 1,
      aux_sym_condition_props_repeat1,
    STATE(1861), 2,
      sym_line_comment,
      sym_block_comment,
  [44717] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1546), 1,
      anon_sym_GT,
    ACTIONS(3675), 1,
      anon_sym_COMMA,
    STATE(1811), 1,
      aux_sym_tuple_type_repeat1,
    STATE(1862), 2,
      sym_line_comment,
      sym_block_comment,
  [44737] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3677), 1,
      anon_sym_COMMA,
    ACTIONS(3680), 1,
      anon_sym_RPAREN,
    STATE(1863), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_lambda_bind_list_repeat1,
  [44755] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2712), 1,
      anon_sym_COMMA,
    ACTIONS(3682), 1,
      anon_sym_PIPE,
    STATE(1975), 1,
      aux_sym_tuple_type_repeat1,
    STATE(1864), 2,
      sym_line_comment,
      sym_block_comment,
  [44775] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(441), 1,
      anon_sym_RBRACK,
    ACTIONS(3684), 1,
      anon_sym_COMMA,
    STATE(1776), 1,
      aux_sym_vector_value_expr_repeat1,
    STATE(1865), 2,
      sym_line_comment,
      sym_block_comment,
  [44795] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3686), 1,
      anon_sym_COMMA,
    ACTIONS(3688), 1,
      anon_sym_RBRACE,
    STATE(1887), 1,
      aux_sym_use_decl_repeat1,
    STATE(1866), 2,
      sym_line_comment,
      sym_block_comment,
  [44815] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3690), 1,
      anon_sym_COMMA,
    ACTIONS(3692), 1,
      anon_sym_PIPE,
    STATE(1811), 1,
      aux_sym_tuple_type_repeat1,
    STATE(1867), 2,
      sym_line_comment,
      sym_block_comment,
  [44835] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3377), 1,
      anon_sym_LBRACE,
    ACTIONS(3694), 1,
      anon_sym_SEMI,
    STATE(1575), 1,
      sym_struct_body,
    STATE(1868), 2,
      sym_line_comment,
      sym_block_comment,
  [44855] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3533), 1,
      anon_sym_RPAREN,
    ACTIONS(3696), 1,
      anon_sym_COMMA,
    STATE(1869), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_tuple_type_repeat1,
  [44873] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3699), 1,
      anon_sym_COMMA,
    ACTIONS(3702), 1,
      anon_sym_RPAREN,
    STATE(1870), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_parameters_repeat1,
  [44891] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3704), 1,
      anon_sym_COMMA,
    ACTIONS(3706), 1,
      anon_sym_SEMI,
    STATE(1845), 1,
      aux_sym_condition_props_repeat1,
    STATE(1871), 2,
      sym_line_comment,
      sym_block_comment,
  [44911] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3428), 1,
      anon_sym_GT,
    ACTIONS(3708), 1,
      anon_sym_COMMA,
    STATE(1916), 1,
      aux_sym__struct_type_params_repeat1,
    STATE(1872), 2,
      sym_line_comment,
      sym_block_comment,
  [44931] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1873), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3710), 3,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_except,
  [44947] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1874), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3604), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
  [44963] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3712), 1,
      anon_sym_COMMA,
    ACTIONS(3714), 1,
      anon_sym_RPAREN,
    STATE(1863), 1,
      aux_sym_lambda_bind_list_repeat1,
    STATE(1875), 2,
      sym_line_comment,
      sym_block_comment,
  [44983] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3716), 1,
      anon_sym_COMMA,
    ACTIONS(3718), 1,
      anon_sym_PIPE,
    STATE(1994), 1,
      aux_sym_lambda_bind_list_repeat1,
    STATE(1876), 2,
      sym_line_comment,
      sym_block_comment,
  [45003] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(415), 1,
      anon_sym_RBRACK,
    ACTIONS(3720), 1,
      anon_sym_COMMA,
    STATE(1917), 1,
      aux_sym_vector_value_expr_repeat1,
    STATE(1877), 2,
      sym_line_comment,
      sym_block_comment,
  [45023] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1878), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3722), 3,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [45039] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1879), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3724), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
  [45055] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3726), 1,
      anon_sym_COMMA,
    ACTIONS(3728), 1,
      anon_sym_SEMI,
    STATE(1793), 1,
      aux_sym_spec_apply_repeat1,
    STATE(1880), 2,
      sym_line_comment,
      sym_block_comment,
  [45075] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1881), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3730), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
  [45091] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2359), 1,
      anon_sym_RBRACE,
    ACTIONS(3732), 1,
      anon_sym_COMMA,
    STATE(1894), 1,
      aux_sym__bind_fields_repeat1,
    STATE(1882), 2,
      sym_line_comment,
      sym_block_comment,
  [45111] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3488), 1,
      sym_identifier,
    ACTIONS(3734), 1,
      anon_sym_RBRACE,
    STATE(2071), 1,
      sym__use_member,
    STATE(1883), 2,
      sym_line_comment,
      sym_block_comment,
  [45131] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3736), 1,
      anon_sym_COMMA,
    ACTIONS(3738), 1,
      anon_sym_RBRACK,
    STATE(1845), 1,
      aux_sym_condition_props_repeat1,
    STATE(1884), 2,
      sym_line_comment,
      sym_block_comment,
  [45151] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3502), 1,
      sym_identifier,
    ACTIONS(3657), 1,
      anon_sym_RBRACE,
    STATE(2021), 1,
      sym_expr_field,
    STATE(1885), 2,
      sym_line_comment,
      sym_block_comment,
  [45171] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3740), 1,
      anon_sym_COMMA,
    ACTIONS(3742), 1,
      anon_sym_SEMI,
    STATE(1903), 1,
      aux_sym_spec_apply_repeat1,
    STATE(1886), 2,
      sym_line_comment,
      sym_block_comment,
  [45191] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3734), 1,
      anon_sym_RBRACE,
    ACTIONS(3744), 1,
      anon_sym_COMMA,
    STATE(1965), 1,
      aux_sym_use_decl_repeat1,
    STATE(1887), 2,
      sym_line_comment,
      sym_block_comment,
  [45211] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3746), 1,
      anon_sym_COMMA,
    ACTIONS(3748), 1,
      anon_sym_RPAREN,
    STATE(1985), 1,
      aux_sym_lambda_bind_list_repeat1,
    STATE(1888), 2,
      sym_line_comment,
      sym_block_comment,
  [45231] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1889), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3750), 3,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
  [45247] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3752), 1,
      anon_sym_COMMA,
    ACTIONS(3754), 1,
      anon_sym_SEMI,
    STATE(1793), 1,
      aux_sym_spec_apply_repeat1,
    STATE(1890), 2,
      sym_line_comment,
      sym_block_comment,
  [45267] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3756), 1,
      anon_sym_COMMA,
    ACTIONS(3758), 1,
      anon_sym_RBRACE,
    STATE(1957), 1,
      aux_sym_triggers_repeat1,
    STATE(1891), 2,
      sym_line_comment,
      sym_block_comment,
  [45287] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(393), 1,
      anon_sym_LT,
    ACTIONS(3760), 1,
      anon_sym_LBRACE,
    STATE(2190), 1,
      sym_type_params,
    STATE(1892), 2,
      sym_line_comment,
      sym_block_comment,
  [45307] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3762), 1,
      anon_sym_COMMA,
    ACTIONS(3764), 1,
      anon_sym_SEMI,
    STATE(1910), 1,
      aux_sym_spec_apply_repeat1,
    STATE(1893), 2,
      sym_line_comment,
      sym_block_comment,
  [45327] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3766), 1,
      anon_sym_COMMA,
    ACTIONS(3769), 1,
      anon_sym_RBRACE,
    STATE(1894), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym__bind_fields_repeat1,
  [45345] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3771), 1,
      anon_sym_COMMA,
    ACTIONS(3773), 1,
      anon_sym_GT,
    STATE(1921), 1,
      aux_sym_tuple_type_repeat1,
    STATE(1895), 2,
      sym_line_comment,
      sym_block_comment,
  [45365] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1896), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3516), 3,
      anon_sym_COMMA,
      anon_sym_GT,
      anon_sym_PLUS,
  [45381] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3775), 1,
      anon_sym_COMMA,
    ACTIONS(3777), 1,
      anon_sym_RBRACE,
    STATE(1923), 1,
      aux_sym_use_decl_repeat1,
    STATE(1897), 2,
      sym_line_comment,
      sym_block_comment,
  [45401] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(65), 1,
      anon_sym_LBRACE,
    ACTIONS(3779), 1,
      anon_sym_SEMI,
    STATE(531), 1,
      sym_block,
    STATE(1898), 2,
      sym_line_comment,
      sym_block_comment,
  [45421] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3547), 1,
      sym_identifier,
    ACTIONS(3781), 1,
      anon_sym_RBRACE,
    STATE(2133), 1,
      sym_field_annot,
    STATE(1899), 2,
      sym_line_comment,
      sym_block_comment,
  [45441] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1900), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3724), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
  [45457] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3502), 1,
      sym_identifier,
    ACTIONS(3783), 1,
      anon_sym_RBRACE,
    STATE(2021), 1,
      sym_expr_field,
    STATE(1901), 2,
      sym_line_comment,
      sym_block_comment,
  [45477] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3785), 1,
      anon_sym_COMMA,
    ACTIONS(3788), 1,
      anon_sym_RBRACE,
    STATE(1902), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_pack_expr_repeat1,
  [45495] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3790), 1,
      anon_sym_COMMA,
    ACTIONS(3792), 1,
      anon_sym_SEMI,
    STATE(1793), 1,
      aux_sym_spec_apply_repeat1,
    STATE(1903), 2,
      sym_line_comment,
      sym_block_comment,
  [45515] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3502), 1,
      sym_identifier,
    ACTIONS(3794), 1,
      anon_sym_RBRACE,
    STATE(2021), 1,
      sym_expr_field,
    STATE(1904), 2,
      sym_line_comment,
      sym_block_comment,
  [45535] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(65), 1,
      anon_sym_LBRACE,
    ACTIONS(3796), 1,
      anon_sym_SEMI,
    STATE(1280), 1,
      sym_block,
    STATE(1905), 2,
      sym_line_comment,
      sym_block_comment,
  [45555] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1554), 1,
      anon_sym_RPAREN,
    ACTIONS(3798), 1,
      anon_sym_COMMA,
    STATE(1869), 1,
      aux_sym_tuple_type_repeat1,
    STATE(1906), 2,
      sym_line_comment,
      sym_block_comment,
  [45575] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3800), 1,
      anon_sym_COMMA,
    ACTIONS(3802), 1,
      anon_sym_GT,
    STATE(1872), 1,
      aux_sym__struct_type_params_repeat1,
    STATE(1907), 2,
      sym_line_comment,
      sym_block_comment,
  [45595] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3804), 1,
      anon_sym_COMMA,
    ACTIONS(3806), 1,
      anon_sym_SEMI,
    STATE(1918), 1,
      aux_sym_spec_apply_repeat1,
    STATE(1908), 2,
      sym_line_comment,
      sym_block_comment,
  [45615] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3502), 1,
      sym_identifier,
    ACTIONS(3808), 1,
      anon_sym_RBRACE,
    STATE(2021), 1,
      sym_expr_field,
    STATE(1909), 2,
      sym_line_comment,
      sym_block_comment,
  [45635] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3810), 1,
      anon_sym_COMMA,
    ACTIONS(3812), 1,
      anon_sym_SEMI,
    STATE(1793), 1,
      aux_sym_spec_apply_repeat1,
    STATE(1910), 2,
      sym_line_comment,
      sym_block_comment,
  [45655] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3814), 1,
      anon_sym_COMMA,
    ACTIONS(3816), 1,
      anon_sym_GT,
    STATE(1998), 1,
      aux_sym_type_params_repeat1,
    STATE(1911), 2,
      sym_line_comment,
      sym_block_comment,
  [45675] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3808), 1,
      anon_sym_RBRACE,
    ACTIONS(3818), 1,
      anon_sym_COMMA,
    STATE(1902), 1,
      aux_sym_pack_expr_repeat1,
    STATE(1912), 2,
      sym_line_comment,
      sym_block_comment,
  [45695] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3820), 1,
      anon_sym_COMMA,
    ACTIONS(3822), 1,
      anon_sym_SEMI,
    STATE(1871), 1,
      aux_sym_condition_props_repeat1,
    STATE(1913), 2,
      sym_line_comment,
      sym_block_comment,
  [45715] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1914), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3824), 3,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
  [45731] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3488), 1,
      sym_identifier,
    ACTIONS(3826), 1,
      anon_sym_RBRACE,
    STATE(2071), 1,
      sym__use_member,
    STATE(1915), 2,
      sym_line_comment,
      sym_block_comment,
  [45751] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3828), 1,
      anon_sym_COMMA,
    ACTIONS(3831), 1,
      anon_sym_GT,
    STATE(1916), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym__struct_type_params_repeat1,
  [45769] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(513), 1,
      anon_sym_RBRACK,
    ACTIONS(3833), 1,
      anon_sym_COMMA,
    STATE(1776), 1,
      aux_sym_vector_value_expr_repeat1,
    STATE(1917), 2,
      sym_line_comment,
      sym_block_comment,
  [45789] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3835), 1,
      anon_sym_COMMA,
    ACTIONS(3837), 1,
      anon_sym_SEMI,
    STATE(1793), 1,
      aux_sym_spec_apply_repeat1,
    STATE(1918), 2,
      sym_line_comment,
      sym_block_comment,
  [45809] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3488), 1,
      sym_identifier,
    ACTIONS(3839), 1,
      anon_sym_LBRACE,
    STATE(2421), 1,
      sym__use_member,
    STATE(1919), 2,
      sym_line_comment,
      sym_block_comment,
  [45829] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3841), 1,
      anon_sym_COMMA,
    ACTIONS(3843), 1,
      anon_sym_RPAREN,
    STATE(1952), 1,
      aux_sym_tuple_type_repeat1,
    STATE(1920), 2,
      sym_line_comment,
      sym_block_comment,
  [45849] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1548), 1,
      anon_sym_GT,
    ACTIONS(3845), 1,
      anon_sym_COMMA,
    STATE(1811), 1,
      aux_sym_tuple_type_repeat1,
    STATE(1921), 2,
      sym_line_comment,
      sym_block_comment,
  [45869] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(513), 1,
      anon_sym_RBRACK,
    ACTIONS(3833), 1,
      anon_sym_COMMA,
    STATE(1865), 1,
      aux_sym_vector_value_expr_repeat1,
    STATE(1922), 2,
      sym_line_comment,
      sym_block_comment,
  [45889] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3826), 1,
      anon_sym_RBRACE,
    ACTIONS(3847), 1,
      anon_sym_COMMA,
    STATE(1965), 1,
      aux_sym_use_decl_repeat1,
    STATE(1923), 2,
      sym_line_comment,
      sym_block_comment,
  [45909] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3633), 1,
      anon_sym_RBRACK,
    ACTIONS(3849), 1,
      anon_sym_COMMA,
    STATE(1836), 1,
      aux_sym_attributes_repeat1,
    STATE(1924), 2,
      sym_line_comment,
      sym_block_comment,
  [45929] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1925), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3851), 3,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
  [45945] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1926), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3853), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
  [45961] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1927), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3591), 3,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_SEMI,
  [45977] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3488), 1,
      sym_identifier,
    ACTIONS(3855), 1,
      anon_sym_RBRACE,
    STATE(2071), 1,
      sym__use_member,
    STATE(1928), 2,
      sym_line_comment,
      sym_block_comment,
  [45997] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3779), 1,
      anon_sym_SEMI,
    ACTIONS(3857), 1,
      anon_sym_LBRACE,
    STATE(531), 1,
      sym_block,
    STATE(1929), 2,
      sym_line_comment,
      sym_block_comment,
  [46017] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3488), 1,
      sym_identifier,
    ACTIONS(3859), 1,
      anon_sym_LBRACE,
    STATE(2226), 1,
      sym__use_member,
    STATE(1930), 2,
      sym_line_comment,
      sym_block_comment,
  [46037] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(393), 1,
      anon_sym_LT,
    ACTIONS(3861), 1,
      anon_sym_COLON,
    STATE(2309), 1,
      sym_type_params,
    STATE(1931), 2,
      sym_line_comment,
      sym_block_comment,
  [46057] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3413), 1,
      sym_identifier,
    STATE(1773), 1,
      sym__attribute_name,
    STATE(1874), 1,
      sym_attribute,
    STATE(1932), 2,
      sym_line_comment,
      sym_block_comment,
  [46077] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1933), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3853), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
  [46093] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3863), 1,
      anon_sym_SLASH_SLASH2,
    ACTIONS(3865), 1,
      anon_sym_SLASH2,
    ACTIONS(3867), 1,
      aux_sym_line_comment_token2,
    STATE(1934), 2,
      sym_line_comment,
      sym_block_comment,
  [46113] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(511), 1,
      anon_sym_RPAREN,
    ACTIONS(3869), 1,
      anon_sym_COMMA,
    STATE(1776), 1,
      aux_sym_vector_value_expr_repeat1,
    STATE(1935), 2,
      sym_line_comment,
      sym_block_comment,
  [46133] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1936), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3871), 3,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
  [46149] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1937), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3873), 3,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
  [46165] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3875), 1,
      anon_sym_STAR_SLASH,
    ACTIONS(3877), 1,
      sym__block_doc_comment_marker,
    ACTIONS(3879), 1,
      sym__block_comment_content,
    STATE(1938), 2,
      sym_line_comment,
      sym_block_comment,
  [46185] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1939), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3881), 3,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
  [46201] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3883), 1,
      anon_sym_COMMA,
    ACTIONS(3885), 1,
      anon_sym_RPAREN,
    STATE(1875), 1,
      aux_sym_lambda_bind_list_repeat1,
    STATE(1940), 2,
      sym_line_comment,
      sym_block_comment,
  [46221] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3651), 1,
      anon_sym_COMMA,
    ACTIONS(3887), 1,
      anon_sym_SEMI,
    STATE(1854), 1,
      aux_sym_vector_value_expr_repeat1,
    STATE(1941), 2,
      sym_line_comment,
      sym_block_comment,
  [46241] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3891), 1,
      anon_sym_COLON,
    ACTIONS(3889), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
    STATE(1942), 2,
      sym_line_comment,
      sym_block_comment,
  [46259] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3893), 1,
      anon_sym_COMMA,
    ACTIONS(3895), 1,
      anon_sym_RBRACE,
    STATE(1882), 1,
      aux_sym__bind_fields_repeat1,
    STATE(1943), 2,
      sym_line_comment,
      sym_block_comment,
  [46279] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1944), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3897), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
  [46295] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1945), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3899), 3,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
  [46311] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1946), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3901), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE,
  [46327] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3903), 1,
      anon_sym_COMMA,
    ACTIONS(3905), 1,
      anon_sym_RPAREN,
    STATE(1992), 1,
      aux_sym_attributes_repeat1,
    STATE(1947), 2,
      sym_line_comment,
      sym_block_comment,
  [46347] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(65), 1,
      anon_sym_LBRACE,
    ACTIONS(3907), 1,
      anon_sym_SEMI,
    STATE(1569), 1,
      sym_block,
    STATE(1948), 2,
      sym_line_comment,
      sym_block_comment,
  [46367] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(877), 1,
      sym__identifier_or_anon_field,
    ACTIONS(3909), 2,
      sym_identifier,
      sym_anon_field_index,
    STATE(1949), 2,
      sym_line_comment,
      sym_block_comment,
  [46385] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3911), 1,
      anon_sym_COMMA,
    ACTIONS(3913), 1,
      anon_sym_RPAREN,
    STATE(1906), 1,
      aux_sym_tuple_type_repeat1,
    STATE(1950), 2,
      sym_line_comment,
      sym_block_comment,
  [46405] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1951), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3915), 3,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
  [46421] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1556), 1,
      anon_sym_RPAREN,
    ACTIONS(3917), 1,
      anon_sym_COMMA,
    STATE(1869), 1,
      aux_sym_tuple_type_repeat1,
    STATE(1952), 2,
      sym_line_comment,
      sym_block_comment,
  [46441] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1953), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3919), 3,
      anon_sym_where,
      anon_sym_COLON,
      anon_sym_LBRACE,
  [46457] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1954), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3921), 3,
      anon_sym_RBRACE,
      anon_sym_POUND,
      anon_sym_spec,
  [46473] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3680), 1,
      anon_sym_PIPE,
    ACTIONS(3923), 1,
      anon_sym_COMMA,
    STATE(1955), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_lambda_bind_list_repeat1,
  [46491] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(393), 1,
      anon_sym_LT,
    ACTIONS(3926), 1,
      anon_sym_COLON,
    STATE(2274), 1,
      sym_type_params,
    STATE(1956), 2,
      sym_line_comment,
      sym_block_comment,
  [46511] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3928), 1,
      anon_sym_COMMA,
    ACTIONS(3930), 1,
      anon_sym_RBRACE,
    STATE(1963), 1,
      aux_sym_triggers_repeat1,
    STATE(1957), 2,
      sym_line_comment,
      sym_block_comment,
  [46531] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1958), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3932), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
  [46547] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3225), 1,
      anon_sym_has,
    ACTIONS(3934), 1,
      anon_sym_SEMI,
    STATE(2440), 1,
      sym_abilities,
    STATE(1959), 2,
      sym_line_comment,
      sym_block_comment,
  [46567] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3225), 1,
      anon_sym_has,
    ACTIONS(3936), 1,
      anon_sym_SEMI,
    STATE(2186), 1,
      sym_abilities,
    STATE(1960), 2,
      sym_line_comment,
      sym_block_comment,
  [46587] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3938), 1,
      anon_sym_COMMA,
    ACTIONS(3941), 1,
      anon_sym_RBRACE,
    STATE(1961), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_struct_body_repeat1,
  [46605] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3225), 1,
      anon_sym_has,
    ACTIONS(3943), 1,
      anon_sym_SEMI,
    STATE(2191), 1,
      sym_abilities,
    STATE(1962), 2,
      sym_line_comment,
      sym_block_comment,
  [46625] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3945), 1,
      anon_sym_COMMA,
    ACTIONS(3948), 1,
      anon_sym_RBRACE,
    STATE(1963), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_triggers_repeat1,
  [46643] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3488), 1,
      sym_identifier,
    ACTIONS(3950), 1,
      anon_sym_RBRACE,
    STATE(2071), 1,
      sym__use_member,
    STATE(1964), 2,
      sym_line_comment,
      sym_block_comment,
  [46663] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3952), 1,
      anon_sym_COMMA,
    ACTIONS(3955), 1,
      anon_sym_RBRACE,
    STATE(1965), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_use_decl_repeat1,
  [46681] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3957), 1,
      anon_sym_COMMA,
    ACTIONS(3959), 1,
      anon_sym_RBRACE,
    STATE(1970), 1,
      aux_sym_struct_body_repeat1,
    STATE(1966), 2,
      sym_line_comment,
      sym_block_comment,
  [46701] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3963), 1,
      anon_sym_COMMA,
    ACTIONS(3961), 2,
      sym_identifier,
      anon_sym_RBRACE,
    STATE(1967), 2,
      sym_line_comment,
      sym_block_comment,
  [46719] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3961), 1,
      sym_identifier,
    ACTIONS(3965), 1,
      anon_sym_COMMA,
    ACTIONS(3967), 1,
      anon_sym_RBRACE,
    STATE(1968), 2,
      sym_line_comment,
      sym_block_comment,
  [46739] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3547), 1,
      sym_identifier,
    ACTIONS(3970), 1,
      anon_sym_RBRACE,
    STATE(2133), 1,
      sym_field_annot,
    STATE(1969), 2,
      sym_line_comment,
      sym_block_comment,
  [46759] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3970), 1,
      anon_sym_RBRACE,
    ACTIONS(3972), 1,
      anon_sym_COMMA,
    STATE(1961), 1,
      aux_sym_struct_body_repeat1,
    STATE(1970), 2,
      sym_line_comment,
      sym_block_comment,
  [46779] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1971), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3974), 3,
      sym_identifier,
      anon_sym_COMMA,
      anon_sym_RBRACE,
  [46795] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3482), 1,
      sym_identifier,
    ACTIONS(3976), 1,
      anon_sym_GT,
    STATE(2141), 1,
      sym_type_param,
    STATE(1972), 2,
      sym_line_comment,
      sym_block_comment,
  [46815] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3978), 1,
      anon_sym_COMMA,
    ACTIONS(3981), 1,
      anon_sym_GT,
    STATE(1973), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_type_params_repeat1,
  [46833] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1974), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3983), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_SEMI,
  [46849] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3690), 1,
      anon_sym_COMMA,
    ACTIONS(3985), 1,
      anon_sym_PIPE,
    STATE(1811), 1,
      aux_sym_tuple_type_repeat1,
    STATE(1975), 2,
      sym_line_comment,
      sym_block_comment,
  [46869] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3987), 1,
      anon_sym_COMMA,
    ACTIONS(3989), 1,
      anon_sym_RPAREN,
    STATE(1990), 1,
      aux_sym_parameters_repeat1,
    STATE(1976), 2,
      sym_line_comment,
      sym_block_comment,
  [46889] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1977), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3932), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
  [46905] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1978), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3991), 3,
      anon_sym_acquires,
      anon_sym_reads,
      anon_sym_writes,
  [46921] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1979), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3993), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_SEMI,
  [46937] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3995), 1,
      anon_sym_COMMA,
    ACTIONS(3997), 1,
      anon_sym_RPAREN,
    STATE(1996), 1,
      aux_sym_vector_value_expr_repeat1,
    STATE(1980), 2,
      sym_line_comment,
      sym_block_comment,
  [46957] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1981), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3999), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
  [46973] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4003), 1,
      anon_sym_COLON,
    ACTIONS(4001), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
    STATE(1982), 2,
      sym_line_comment,
      sym_block_comment,
  [46991] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4005), 1,
      anon_sym_COMMA,
    ACTIONS(4007), 1,
      anon_sym_SEMI,
    STATE(1880), 1,
      aux_sym_spec_apply_repeat1,
    STATE(1983), 2,
      sym_line_comment,
      sym_block_comment,
  [47011] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4009), 1,
      anon_sym_COMMA,
    ACTIONS(4011), 1,
      anon_sym_RBRACE,
    STATE(1912), 1,
      aux_sym_pack_expr_repeat1,
    STATE(1984), 2,
      sym_line_comment,
      sym_block_comment,
  [47031] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4013), 1,
      anon_sym_COMMA,
    ACTIONS(4015), 1,
      anon_sym_RPAREN,
    STATE(1863), 1,
      aux_sym_lambda_bind_list_repeat1,
    STATE(1985), 2,
      sym_line_comment,
      sym_block_comment,
  [47051] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1986), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4017), 3,
      anon_sym_COLON_COLON,
      anon_sym_as,
      anon_sym_SEMI,
  [47067] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4019), 1,
      anon_sym_COMMA,
    ACTIONS(4021), 1,
      anon_sym_SEMI,
    STATE(1793), 1,
      aux_sym_spec_apply_repeat1,
    STATE(1987), 2,
      sym_line_comment,
      sym_block_comment,
  [47087] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1988), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4023), 3,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_except,
  [47103] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(1989), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4025), 3,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_SEMI,
  [47119] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3641), 1,
      anon_sym_RPAREN,
    ACTIONS(4027), 1,
      anon_sym_COMMA,
    STATE(1870), 1,
      aux_sym_parameters_repeat1,
    STATE(1990), 2,
      sym_line_comment,
      sym_block_comment,
  [47139] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4029), 1,
      anon_sym_COMMA,
    ACTIONS(4031), 1,
      anon_sym_GT,
    STATE(1862), 1,
      aux_sym_tuple_type_repeat1,
    STATE(1991), 2,
      sym_line_comment,
      sym_block_comment,
  [47159] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4033), 1,
      anon_sym_COMMA,
    ACTIONS(4035), 1,
      anon_sym_RPAREN,
    STATE(1836), 1,
      aux_sym_attributes_repeat1,
    STATE(1992), 2,
      sym_line_comment,
      sym_block_comment,
  [47179] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3482), 1,
      sym_identifier,
    ACTIONS(4037), 1,
      anon_sym_GT,
    STATE(2141), 1,
      sym_type_param,
    STATE(1993), 2,
      sym_line_comment,
      sym_block_comment,
  [47199] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4039), 1,
      anon_sym_COMMA,
    ACTIONS(4041), 1,
      anon_sym_PIPE,
    STATE(1955), 1,
      aux_sym_lambda_bind_list_repeat1,
    STATE(1994), 2,
      sym_line_comment,
      sym_block_comment,
  [47219] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4043), 1,
      anon_sym_COMMA,
    ACTIONS(4045), 1,
      anon_sym_RBRACE,
    STATE(1856), 1,
      aux_sym_pack_expr_repeat1,
    STATE(1995), 2,
      sym_line_comment,
      sym_block_comment,
  [47239] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(517), 1,
      anon_sym_RPAREN,
    ACTIONS(4047), 1,
      anon_sym_COMMA,
    STATE(1776), 1,
      aux_sym_vector_value_expr_repeat1,
    STATE(1996), 2,
      sym_line_comment,
      sym_block_comment,
  [47259] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4049), 1,
      anon_sym_EQ,
    ACTIONS(4051), 1,
      anon_sym_COLON,
    ACTIONS(4053), 1,
      anon_sym_SEMI,
    STATE(1997), 2,
      sym_line_comment,
      sym_block_comment,
  [47279] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4037), 1,
      anon_sym_GT,
    ACTIONS(4055), 1,
      anon_sym_COMMA,
    STATE(1973), 1,
      aux_sym_type_params_repeat1,
    STATE(1998), 2,
      sym_line_comment,
      sym_block_comment,
  [47299] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2712), 1,
      anon_sym_COMMA,
    ACTIONS(2716), 1,
      anon_sym_PIPE,
    STATE(1867), 1,
      aux_sym_tuple_type_repeat1,
    STATE(1999), 2,
      sym_line_comment,
      sym_block_comment,
  [47319] = 6,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3651), 1,
      anon_sym_COMMA,
    ACTIONS(4057), 1,
      anon_sym_SEMI,
    STATE(1776), 1,
      aux_sym_vector_value_expr_repeat1,
    STATE(2000), 2,
      sym_line_comment,
      sym_block_comment,
  [47339] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    STATE(2001), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4059), 3,
      anon_sym_friend,
      anon_sym_script,
      anon_sym_package,
  [47355] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3488), 1,
      sym_identifier,
    STATE(2071), 1,
      sym__use_member,
    STATE(2002), 2,
      sym_line_comment,
      sym_block_comment,
  [47372] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4061), 1,
      sym_identifier,
    STATE(1818), 1,
      sym_quantifier_bind,
    STATE(2003), 2,
      sym_line_comment,
      sym_block_comment,
  [47389] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4063), 1,
      sym_identifier,
    STATE(2310), 1,
      sym_quantifier_bind,
    STATE(2004), 2,
      sym_line_comment,
      sym_block_comment,
  [47406] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(134), 1,
      sym_parenthesized_expr,
    STATE(2005), 2,
      sym_line_comment,
      sym_block_comment,
  [47423] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(83), 1,
      sym_parenthesized_expr,
    STATE(2006), 2,
      sym_line_comment,
      sym_block_comment,
  [47440] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4065), 1,
      anon_sym_if,
    ACTIONS(4067), 1,
      anon_sym_EQ_GT,
    STATE(2007), 2,
      sym_line_comment,
      sym_block_comment,
  [47457] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4069), 2,
      anon_sym_LBRACE,
      anon_sym_SEMI,
    STATE(2008), 2,
      sym_line_comment,
      sym_block_comment,
  [47472] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4071), 1,
      anon_sym_EQ,
    ACTIONS(4073), 1,
      anon_sym_SEMI,
    STATE(2009), 2,
      sym_line_comment,
      sym_block_comment,
  [47489] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4075), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
    STATE(2010), 2,
      sym_line_comment,
      sym_block_comment,
  [47504] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(71), 1,
      sym_parenthesized_expr,
    STATE(2011), 2,
      sym_line_comment,
      sym_block_comment,
  [47521] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3769), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
    STATE(2012), 2,
      sym_line_comment,
      sym_block_comment,
  [47536] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4077), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(2013), 2,
      sym_line_comment,
      sym_block_comment,
  [47551] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3482), 1,
      sym_identifier,
    STATE(2141), 1,
      sym_type_param,
    STATE(2014), 2,
      sym_line_comment,
      sym_block_comment,
  [47568] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3502), 1,
      sym_identifier,
    STATE(2021), 1,
      sym_expr_field,
    STATE(2015), 2,
      sym_line_comment,
      sym_block_comment,
  [47585] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4079), 1,
      anon_sym_where,
    ACTIONS(4081), 1,
      anon_sym_COLON,
    STATE(2016), 2,
      sym_line_comment,
      sym_block_comment,
  [47602] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4083), 1,
      sym_identifier,
    STATE(1929), 1,
      sym__spec_func_signatures,
    STATE(2017), 2,
      sym_line_comment,
      sym_block_comment,
  [47619] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4083), 1,
      sym_identifier,
    STATE(2222), 1,
      sym__spec_func_signatures,
    STATE(2018), 2,
      sym_line_comment,
      sym_block_comment,
  [47636] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4085), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
    STATE(2019), 2,
      sym_line_comment,
      sym_block_comment,
  [47651] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4087), 2,
      sym_identifier,
      anon_sym_STAR,
    STATE(2020), 2,
      sym_line_comment,
      sym_block_comment,
  [47666] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3788), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
    STATE(2021), 2,
      sym_line_comment,
      sym_block_comment,
  [47681] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4089), 1,
      anon_sym_where,
    ACTIONS(4091), 1,
      anon_sym_COLON,
    STATE(2022), 2,
      sym_line_comment,
      sym_block_comment,
  [47698] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3702), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(2023), 2,
      sym_line_comment,
      sym_block_comment,
  [47713] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4093), 1,
      anon_sym_EQ,
    ACTIONS(4095), 1,
      anon_sym_SEMI,
    STATE(2024), 2,
      sym_line_comment,
      sym_block_comment,
  [47730] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(61), 1,
      sym_parenthesized_expr,
    STATE(2025), 2,
      sym_line_comment,
      sym_block_comment,
  [47747] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3482), 1,
      sym_identifier,
    STATE(2030), 1,
      sym_type_param,
    STATE(2026), 2,
      sym_line_comment,
      sym_block_comment,
  [47764] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4097), 1,
      anon_sym_where,
    ACTIONS(4099), 1,
      anon_sym_COLON,
    STATE(2027), 2,
      sym_line_comment,
      sym_block_comment,
  [47781] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4101), 1,
      anon_sym_COLON,
    ACTIONS(4103), 1,
      anon_sym_LBRACE,
    STATE(2028), 2,
      sym_line_comment,
      sym_block_comment,
  [47798] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4105), 1,
      anon_sym_where,
    ACTIONS(4107), 1,
      anon_sym_COLON,
    STATE(2029), 2,
      sym_line_comment,
      sym_block_comment,
  [47815] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4109), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(2030), 2,
      sym_line_comment,
      sym_block_comment,
  [47830] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4111), 1,
      anon_sym_where,
    ACTIONS(4113), 1,
      anon_sym_COLON,
    STATE(2031), 2,
      sym_line_comment,
      sym_block_comment,
  [47847] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4115), 1,
      anon_sym_where,
    ACTIONS(4117), 1,
      anon_sym_COLON,
    STATE(2032), 2,
      sym_line_comment,
      sym_block_comment,
  [47864] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4119), 2,
      anon_sym_LBRACE,
      anon_sym_SEMI,
    STATE(2033), 2,
      sym_line_comment,
      sym_block_comment,
  [47879] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4121), 1,
      anon_sym_RBRACE,
    ACTIONS(4123), 1,
      anon_sym_SEMI,
    STATE(2034), 2,
      sym_line_comment,
      sym_block_comment,
  [47896] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4125), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(2035), 2,
      sym_line_comment,
      sym_block_comment,
  [47911] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(56), 1,
      sym_parenthesized_expr,
    STATE(2036), 2,
      sym_line_comment,
      sym_block_comment,
  [47928] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4127), 1,
      anon_sym_where,
    ACTIONS(4129), 1,
      anon_sym_COLON,
    STATE(2037), 2,
      sym_line_comment,
      sym_block_comment,
  [47945] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4131), 1,
      anon_sym_COLON,
    ACTIONS(4133), 1,
      anon_sym_in,
    STATE(2038), 2,
      sym_line_comment,
      sym_block_comment,
  [47962] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4135), 1,
      anon_sym_if,
    ACTIONS(4137), 1,
      anon_sym_SEMI,
    STATE(2039), 2,
      sym_line_comment,
      sym_block_comment,
  [47979] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4139), 2,
      anon_sym_LBRACE,
      anon_sym_SEMI,
    STATE(2040), 2,
      sym_line_comment,
      sym_block_comment,
  [47994] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(887), 1,
      anon_sym_LPAREN,
    STATE(915), 1,
      sym_call_args,
    STATE(2041), 2,
      sym_line_comment,
      sym_block_comment,
  [48011] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4141), 1,
      anon_sym_EQ,
    ACTIONS(4143), 1,
      anon_sym_SEMI,
    STATE(2042), 2,
      sym_line_comment,
      sym_block_comment,
  [48028] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4083), 1,
      sym_identifier,
    STATE(2264), 1,
      sym__spec_func_signatures,
    STATE(2043), 2,
      sym_line_comment,
      sym_block_comment,
  [48045] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4063), 1,
      sym_identifier,
    STATE(2268), 1,
      sym_quantifier_bind,
    STATE(2044), 2,
      sym_line_comment,
      sym_block_comment,
  [48062] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4123), 1,
      anon_sym_SEMI,
    ACTIONS(4145), 1,
      anon_sym_RBRACE,
    STATE(2045), 2,
      sym_line_comment,
      sym_block_comment,
  [48079] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3831), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(2046), 2,
      sym_line_comment,
      sym_block_comment,
  [48094] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4147), 1,
      anon_sym_SEMI,
    ACTIONS(4149), 1,
      anon_sym_with,
    STATE(2047), 2,
      sym_line_comment,
      sym_block_comment,
  [48111] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(107), 1,
      sym_parenthesized_expr,
    STATE(2048), 2,
      sym_line_comment,
      sym_block_comment,
  [48128] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4151), 1,
      anon_sym_EQ,
    ACTIONS(4153), 1,
      anon_sym_SEMI,
    STATE(2049), 2,
      sym_line_comment,
      sym_block_comment,
  [48145] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4155), 2,
      sym_identifier,
      anon_sym_STAR,
    STATE(2050), 2,
      sym_line_comment,
      sym_block_comment,
  [48160] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4157), 1,
      anon_sym_where,
    ACTIONS(4159), 1,
      anon_sym_COLON,
    STATE(2051), 2,
      sym_line_comment,
      sym_block_comment,
  [48177] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4161), 1,
      sym_identifier,
    STATE(1716), 1,
      sym__struct_def_name,
    STATE(2052), 2,
      sym_line_comment,
      sym_block_comment,
  [48194] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3547), 1,
      sym_identifier,
    STATE(2133), 1,
      sym_field_annot,
    STATE(2053), 2,
      sym_line_comment,
      sym_block_comment,
  [48211] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4161), 1,
      sym_identifier,
    STATE(1813), 1,
      sym__struct_def_name,
    STATE(2054), 2,
      sym_line_comment,
      sym_block_comment,
  [48228] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4163), 2,
      sym_identifier,
      anon_sym_RBRACE,
    STATE(2055), 2,
      sym_line_comment,
      sym_block_comment,
  [48243] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(130), 1,
      sym_parenthesized_expr,
    STATE(2056), 2,
      sym_line_comment,
      sym_block_comment,
  [48260] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3565), 1,
      sym_identifier,
    STATE(2023), 1,
      sym_parameter,
    STATE(2057), 2,
      sym_line_comment,
      sym_block_comment,
  [48277] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(831), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4165), 1,
      anon_sym_RPAREN,
    STATE(2058), 2,
      sym_line_comment,
      sym_block_comment,
  [48294] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4167), 1,
      anon_sym_where,
    ACTIONS(4169), 1,
      anon_sym_COLON,
    STATE(2059), 2,
      sym_line_comment,
      sym_block_comment,
  [48311] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4171), 1,
      anon_sym_where,
    ACTIONS(4173), 1,
      anon_sym_COLON,
    STATE(2060), 2,
      sym_line_comment,
      sym_block_comment,
  [48328] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4175), 1,
      anon_sym_where,
    ACTIONS(4177), 1,
      anon_sym_COLON,
    STATE(2061), 2,
      sym_line_comment,
      sym_block_comment,
  [48345] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4179), 1,
      anon_sym_where,
    ACTIONS(4181), 1,
      anon_sym_COLON,
    STATE(2062), 2,
      sym_line_comment,
      sym_block_comment,
  [48362] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4183), 1,
      anon_sym_LBRACE,
    STATE(1585), 1,
      sym_enum_body,
    STATE(2063), 2,
      sym_line_comment,
      sym_block_comment,
  [48379] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4185), 1,
      anon_sym_SEMI,
    ACTIONS(4187), 1,
      anon_sym_except,
    STATE(2064), 2,
      sym_line_comment,
      sym_block_comment,
  [48396] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1896), 1,
      anon_sym_const,
    STATE(1735), 1,
      sym_constant_decl,
    STATE(2065), 2,
      sym_line_comment,
      sym_block_comment,
  [48413] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4063), 1,
      sym_identifier,
    STATE(2300), 1,
      sym_quantifier_bind,
    STATE(2066), 2,
      sym_line_comment,
      sym_block_comment,
  [48430] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(103), 1,
      sym_parenthesized_expr,
    STATE(2067), 2,
      sym_line_comment,
      sym_block_comment,
  [48447] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4189), 2,
      anon_sym_LBRACE,
      anon_sym_SEMI,
    STATE(2068), 2,
      sym_line_comment,
      sym_block_comment,
  [48462] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4191), 1,
      anon_sym_where,
    ACTIONS(4193), 1,
      anon_sym_COLON,
    STATE(2069), 2,
      sym_line_comment,
      sym_block_comment,
  [48479] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4195), 1,
      anon_sym_where,
    ACTIONS(4197), 1,
      anon_sym_COLON,
    STATE(2070), 2,
      sym_line_comment,
      sym_block_comment,
  [48496] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3955), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
    STATE(2071), 2,
      sym_line_comment,
      sym_block_comment,
  [48511] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(63), 1,
      sym_parenthesized_expr,
    STATE(2072), 2,
      sym_line_comment,
      sym_block_comment,
  [48528] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(831), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4199), 1,
      anon_sym_LBRACE,
    STATE(2073), 2,
      sym_line_comment,
      sym_block_comment,
  [48545] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(65), 1,
      anon_sym_LBRACE,
    STATE(923), 1,
      sym_block,
    STATE(2074), 2,
      sym_line_comment,
      sym_block_comment,
  [48562] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3357), 1,
      anon_sym_LPAREN,
    STATE(2397), 1,
      sym_parameters,
    STATE(2075), 2,
      sym_line_comment,
      sym_block_comment,
  [48579] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4201), 2,
      sym_identifier,
      anon_sym_STAR,
    STATE(2076), 2,
      sym_line_comment,
      sym_block_comment,
  [48594] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4203), 1,
      anon_sym_where,
    ACTIONS(4205), 1,
      anon_sym_COLON,
    STATE(2077), 2,
      sym_line_comment,
      sym_block_comment,
  [48611] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(285), 1,
      anon_sym_RBRACE,
    ACTIONS(4123), 1,
      anon_sym_SEMI,
    STATE(2078), 2,
      sym_line_comment,
      sym_block_comment,
  [48628] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4207), 1,
      anon_sym_where,
    ACTIONS(4209), 1,
      anon_sym_COLON,
    STATE(2079), 2,
      sym_line_comment,
      sym_block_comment,
  [48645] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4211), 1,
      anon_sym_where,
    ACTIONS(4213), 1,
      anon_sym_COLON,
    STATE(2080), 2,
      sym_line_comment,
      sym_block_comment,
  [48662] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2751), 1,
      anon_sym_LT2,
    STATE(2041), 1,
      sym_type_args,
    STATE(2081), 2,
      sym_line_comment,
      sym_block_comment,
  [48679] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4215), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
    STATE(2082), 2,
      sym_line_comment,
      sym_block_comment,
  [48694] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3357), 1,
      anon_sym_LPAREN,
    STATE(1608), 1,
      sym_parameters,
    STATE(2083), 2,
      sym_line_comment,
      sym_block_comment,
  [48711] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(81), 1,
      sym_parenthesized_expr,
    STATE(2084), 2,
      sym_line_comment,
      sym_block_comment,
  [48728] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4163), 1,
      sym_identifier,
    ACTIONS(4217), 1,
      anon_sym_RBRACE,
    STATE(2085), 2,
      sym_line_comment,
      sym_block_comment,
  [48745] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4220), 1,
      anon_sym_where,
    ACTIONS(4222), 1,
      anon_sym_COLON,
    STATE(2086), 2,
      sym_line_comment,
      sym_block_comment,
  [48762] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4224), 1,
      anon_sym_where,
    ACTIONS(4226), 1,
      anon_sym_COLON,
    STATE(2087), 2,
      sym_line_comment,
      sym_block_comment,
  [48779] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4228), 2,
      anon_sym_LBRACE,
      anon_sym_SEMI,
    STATE(2088), 2,
      sym_line_comment,
      sym_block_comment,
  [48794] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4230), 1,
      anon_sym_where,
    ACTIONS(4232), 1,
      anon_sym_COLON,
    STATE(2089), 2,
      sym_line_comment,
      sym_block_comment,
  [48811] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4234), 1,
      anon_sym_COLON,
    ACTIONS(4236), 1,
      anon_sym_LBRACE,
    STATE(2090), 2,
      sym_line_comment,
      sym_block_comment,
  [48828] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(65), 1,
      anon_sym_LBRACE,
    STATE(924), 1,
      sym_block,
    STATE(2091), 2,
      sym_line_comment,
      sym_block_comment,
  [48845] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(887), 1,
      anon_sym_LPAREN,
    STATE(932), 1,
      sym_call_args,
    STATE(2092), 2,
      sym_line_comment,
      sym_block_comment,
  [48862] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(391), 1,
      anon_sym_RBRACE,
    ACTIONS(4123), 1,
      anon_sym_SEMI,
    STATE(2093), 2,
      sym_line_comment,
      sym_block_comment,
  [48879] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(399), 1,
      anon_sym_RBRACE,
    ACTIONS(4123), 1,
      anon_sym_SEMI,
    STATE(2094), 2,
      sym_line_comment,
      sym_block_comment,
  [48896] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4238), 1,
      anon_sym_where,
    ACTIONS(4240), 1,
      anon_sym_COLON,
    STATE(2095), 2,
      sym_line_comment,
      sym_block_comment,
  [48913] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4242), 1,
      anon_sym_where,
    ACTIONS(4244), 1,
      anon_sym_COLON,
    STATE(2096), 2,
      sym_line_comment,
      sym_block_comment,
  [48930] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4246), 1,
      anon_sym_STAR_SLASH,
    ACTIONS(4248), 1,
      sym__block_comment_content,
    STATE(2097), 2,
      sym_line_comment,
      sym_block_comment,
  [48947] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(143), 1,
      sym_parenthesized_expr,
    STATE(2098), 2,
      sym_line_comment,
      sym_block_comment,
  [48964] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4250), 1,
      anon_sym_where,
    ACTIONS(4252), 1,
      anon_sym_COLON,
    STATE(2099), 2,
      sym_line_comment,
      sym_block_comment,
  [48981] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4254), 1,
      anon_sym_where,
    ACTIONS(4256), 1,
      anon_sym_COLON,
    STATE(2100), 2,
      sym_line_comment,
      sym_block_comment,
  [48998] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4131), 1,
      anon_sym_COLON,
    ACTIONS(4258), 1,
      anon_sym_in,
    STATE(2101), 2,
      sym_line_comment,
      sym_block_comment,
  [49015] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4260), 2,
      anon_sym_LBRACE,
      anon_sym_SEMI,
    STATE(2102), 2,
      sym_line_comment,
      sym_block_comment,
  [49030] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4063), 1,
      sym_identifier,
    STATE(2366), 1,
      sym_quantifier_bind,
    STATE(2103), 2,
      sym_line_comment,
      sym_block_comment,
  [49047] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3357), 1,
      anon_sym_LPAREN,
    STATE(2028), 1,
      sym_parameters,
    STATE(2104), 2,
      sym_line_comment,
      sym_block_comment,
  [49064] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1894), 1,
      anon_sym_use,
    STATE(1687), 1,
      sym_use_decl,
    STATE(2105), 2,
      sym_line_comment,
      sym_block_comment,
  [49081] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4262), 1,
      anon_sym_where,
    ACTIONS(4264), 1,
      anon_sym_COLON,
    STATE(2106), 2,
      sym_line_comment,
      sym_block_comment,
  [49098] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(73), 1,
      sym_parenthesized_expr,
    STATE(2107), 2,
      sym_line_comment,
      sym_block_comment,
  [49115] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4266), 1,
      anon_sym_where,
    ACTIONS(4268), 1,
      anon_sym_COLON,
    STATE(2108), 2,
      sym_line_comment,
      sym_block_comment,
  [49132] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4270), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(2109), 2,
      sym_line_comment,
      sym_block_comment,
  [49147] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4272), 1,
      anon_sym_SEMI,
    ACTIONS(4274), 1,
      anon_sym_with,
    STATE(2110), 2,
      sym_line_comment,
      sym_block_comment,
  [49164] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4276), 1,
      anon_sym_where,
    ACTIONS(4278), 1,
      anon_sym_COLON,
    STATE(2111), 2,
      sym_line_comment,
      sym_block_comment,
  [49181] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4280), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
    STATE(2112), 2,
      sym_line_comment,
      sym_block_comment,
  [49196] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4282), 1,
      anon_sym_where,
    ACTIONS(4284), 1,
      anon_sym_COLON,
    STATE(2113), 2,
      sym_line_comment,
      sym_block_comment,
  [49213] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(139), 1,
      sym_parenthesized_expr,
    STATE(2114), 2,
      sym_line_comment,
      sym_block_comment,
  [49230] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4063), 1,
      sym_identifier,
    STATE(2177), 1,
      sym_quantifier_bind,
    STATE(2115), 2,
      sym_line_comment,
      sym_block_comment,
  [49247] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4083), 1,
      sym_identifier,
    STATE(1905), 1,
      sym__spec_func_signatures,
    STATE(2116), 2,
      sym_line_comment,
      sym_block_comment,
  [49264] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(283), 1,
      anon_sym_RBRACE,
    ACTIONS(4123), 1,
      anon_sym_SEMI,
    STATE(2117), 2,
      sym_line_comment,
      sym_block_comment,
  [49281] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4063), 1,
      sym_identifier,
    STATE(2204), 1,
      sym_quantifier_bind,
    STATE(2118), 2,
      sym_line_comment,
      sym_block_comment,
  [49298] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3049), 1,
      anon_sym_spec,
    STATE(1858), 1,
      sym_spec_block,
    STATE(2119), 2,
      sym_line_comment,
      sym_block_comment,
  [49315] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(67), 1,
      sym_parenthesized_expr,
    STATE(2120), 2,
      sym_line_comment,
      sym_block_comment,
  [49332] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(87), 1,
      sym_parenthesized_expr,
    STATE(2121), 2,
      sym_line_comment,
      sym_block_comment,
  [49349] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4063), 1,
      sym_identifier,
    STATE(2304), 1,
      sym_quantifier_bind,
    STATE(2122), 2,
      sym_line_comment,
      sym_block_comment,
  [49366] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4063), 1,
      sym_identifier,
    STATE(2227), 1,
      sym_quantifier_bind,
    STATE(2123), 2,
      sym_line_comment,
      sym_block_comment,
  [49383] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4286), 1,
      anon_sym_if,
    ACTIONS(4288), 1,
      anon_sym_SEMI,
    STATE(2124), 2,
      sym_line_comment,
      sym_block_comment,
  [49400] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4063), 1,
      sym_identifier,
    STATE(2251), 1,
      sym_quantifier_bind,
    STATE(2125), 2,
      sym_line_comment,
      sym_block_comment,
  [49417] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4290), 1,
      anon_sym_EQ,
    ACTIONS(4292), 1,
      anon_sym_SEMI,
    STATE(2126), 2,
      sym_line_comment,
      sym_block_comment,
  [49434] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4294), 2,
      anon_sym_LBRACE,
      anon_sym_SEMI,
    STATE(2127), 2,
      sym_line_comment,
      sym_block_comment,
  [49449] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4296), 1,
      anon_sym_where,
    ACTIONS(4298), 1,
      anon_sym_COLON,
    STATE(2128), 2,
      sym_line_comment,
      sym_block_comment,
  [49466] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(77), 1,
      sym_parenthesized_expr,
    STATE(2129), 2,
      sym_line_comment,
      sym_block_comment,
  [49483] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4300), 1,
      anon_sym_where,
    ACTIONS(4302), 1,
      anon_sym_COLON,
    STATE(2130), 2,
      sym_line_comment,
      sym_block_comment,
  [49500] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4063), 1,
      sym_identifier,
    STATE(2315), 1,
      sym_quantifier_bind,
    STATE(2131), 2,
      sym_line_comment,
      sym_block_comment,
  [49517] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4304), 1,
      anon_sym_where,
    ACTIONS(4306), 1,
      anon_sym_COLON,
    STATE(2132), 2,
      sym_line_comment,
      sym_block_comment,
  [49534] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3941), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
    STATE(2133), 2,
      sym_line_comment,
      sym_block_comment,
  [49549] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4308), 1,
      anon_sym_where,
    ACTIONS(4310), 1,
      anon_sym_COLON,
    STATE(2134), 2,
      sym_line_comment,
      sym_block_comment,
  [49566] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4312), 2,
      sym_identifier,
      anon_sym_RBRACE,
    STATE(2135), 2,
      sym_line_comment,
      sym_block_comment,
  [49581] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3965), 1,
      anon_sym_COMMA,
    ACTIONS(4314), 1,
      anon_sym_RBRACE,
    STATE(2136), 2,
      sym_line_comment,
      sym_block_comment,
  [49598] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4316), 1,
      anon_sym_where,
    ACTIONS(4318), 1,
      anon_sym_COLON,
    STATE(2137), 2,
      sym_line_comment,
      sym_block_comment,
  [49615] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4320), 2,
      anon_sym_LBRACE,
      anon_sym_SEMI,
    STATE(2138), 2,
      sym_line_comment,
      sym_block_comment,
  [49630] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4322), 1,
      anon_sym_where,
    ACTIONS(4324), 1,
      anon_sym_COLON,
    STATE(2139), 2,
      sym_line_comment,
      sym_block_comment,
  [49647] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4063), 1,
      sym_identifier,
    STATE(2377), 1,
      sym_quantifier_bind,
    STATE(2140), 2,
      sym_line_comment,
      sym_block_comment,
  [49664] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3981), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(2141), 2,
      sym_line_comment,
      sym_block_comment,
  [49679] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4063), 1,
      sym_identifier,
    STATE(2245), 1,
      sym_quantifier_bind,
    STATE(2142), 2,
      sym_line_comment,
      sym_block_comment,
  [49696] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(111), 1,
      sym_parenthesized_expr,
    STATE(2143), 2,
      sym_line_comment,
      sym_block_comment,
  [49713] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(91), 1,
      sym_parenthesized_expr,
    STATE(2144), 2,
      sym_line_comment,
      sym_block_comment,
  [49730] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(99), 1,
      sym_parenthesized_expr,
    STATE(2145), 2,
      sym_line_comment,
      sym_block_comment,
  [49747] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(132), 1,
      sym_parenthesized_expr,
    STATE(2146), 2,
      sym_line_comment,
      sym_block_comment,
  [49764] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4326), 2,
      anon_sym_LBRACE,
      anon_sym_SEMI,
    STATE(2147), 2,
      sym_line_comment,
      sym_block_comment,
  [49779] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4328), 2,
      anon_sym_LBRACE,
      anon_sym_SEMI,
    STATE(2148), 2,
      sym_line_comment,
      sym_block_comment,
  [49794] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4063), 1,
      sym_identifier,
    STATE(2359), 1,
      sym_quantifier_bind,
    STATE(2149), 2,
      sym_line_comment,
      sym_block_comment,
  [49811] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4330), 2,
      anon_sym_LBRACE,
      anon_sym_SEMI,
    STATE(2150), 2,
      sym_line_comment,
      sym_block_comment,
  [49826] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4332), 1,
      anon_sym_where,
    ACTIONS(4334), 1,
      anon_sym_COLON,
    STATE(2151), 2,
      sym_line_comment,
      sym_block_comment,
  [49843] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4336), 1,
      anon_sym_where,
    ACTIONS(4338), 1,
      anon_sym_COLON,
    STATE(2152), 2,
      sym_line_comment,
      sym_block_comment,
  [49860] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4340), 2,
      anon_sym_LBRACE,
      anon_sym_SEMI,
    STATE(2153), 2,
      sym_line_comment,
      sym_block_comment,
  [49875] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4063), 1,
      sym_identifier,
    STATE(2412), 1,
      sym_quantifier_bind,
    STATE(2154), 2,
      sym_line_comment,
      sym_block_comment,
  [49892] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4063), 1,
      sym_identifier,
    STATE(2425), 1,
      sym_quantifier_bind,
    STATE(2155), 2,
      sym_line_comment,
      sym_block_comment,
  [49909] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4342), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
    STATE(2156), 2,
      sym_line_comment,
      sym_block_comment,
  [49924] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4344), 1,
      anon_sym_where,
    ACTIONS(4346), 1,
      anon_sym_COLON,
    STATE(2157), 2,
      sym_line_comment,
      sym_block_comment,
  [49941] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(128), 1,
      sym_parenthesized_expr,
    STATE(2158), 2,
      sym_line_comment,
      sym_block_comment,
  [49958] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(135), 1,
      sym_parenthesized_expr,
    STATE(2159), 2,
      sym_line_comment,
      sym_block_comment,
  [49975] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4063), 1,
      sym_identifier,
    STATE(2246), 1,
      sym_quantifier_bind,
    STATE(2160), 2,
      sym_line_comment,
      sym_block_comment,
  [49992] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4348), 1,
      anon_sym_where,
    ACTIONS(4350), 1,
      anon_sym_COLON,
    STATE(2161), 2,
      sym_line_comment,
      sym_block_comment,
  [50009] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4352), 1,
      anon_sym_where,
    ACTIONS(4354), 1,
      anon_sym_COLON,
    STATE(2162), 2,
      sym_line_comment,
      sym_block_comment,
  [50026] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4356), 1,
      anon_sym_where,
    ACTIONS(4358), 1,
      anon_sym_COLON,
    STATE(2163), 2,
      sym_line_comment,
      sym_block_comment,
  [50043] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(95), 1,
      sym_parenthesized_expr,
    STATE(2164), 2,
      sym_line_comment,
      sym_block_comment,
  [50060] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4063), 1,
      sym_identifier,
    STATE(2353), 1,
      sym_quantifier_bind,
    STATE(2165), 2,
      sym_line_comment,
      sym_block_comment,
  [50077] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4063), 1,
      sym_identifier,
    STATE(2291), 1,
      sym_quantifier_bind,
    STATE(2166), 2,
      sym_line_comment,
      sym_block_comment,
  [50094] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3357), 1,
      anon_sym_LPAREN,
    STATE(1613), 1,
      sym_parameters,
    STATE(2167), 2,
      sym_line_comment,
      sym_block_comment,
  [50111] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4063), 1,
      sym_identifier,
    STATE(2435), 1,
      sym_quantifier_bind,
    STATE(2168), 2,
      sym_line_comment,
      sym_block_comment,
  [50128] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4063), 1,
      sym_identifier,
    STATE(2266), 1,
      sym_quantifier_bind,
    STATE(2169), 2,
      sym_line_comment,
      sym_block_comment,
  [50145] = 5,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4083), 1,
      sym_identifier,
    STATE(1898), 1,
      sym__spec_func_signatures,
    STATE(2170), 2,
      sym_line_comment,
      sym_block_comment,
  [50162] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4360), 1,
      aux_sym_number_token5,
    STATE(2171), 2,
      sym_line_comment,
      sym_block_comment,
  [50176] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4362), 1,
      anon_sym_where,
    STATE(2172), 2,
      sym_line_comment,
      sym_block_comment,
  [50190] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4364), 1,
      anon_sym_where,
    STATE(2173), 2,
      sym_line_comment,
      sym_block_comment,
  [50204] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4366), 1,
      anon_sym_RBRACE,
    STATE(2174), 2,
      sym_line_comment,
      sym_block_comment,
  [50218] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2706), 1,
      anon_sym_COLON_COLON,
    STATE(2175), 2,
      sym_line_comment,
      sym_block_comment,
  [50232] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4368), 1,
      anon_sym_SEMI,
    STATE(2176), 2,
      sym_line_comment,
      sym_block_comment,
  [50246] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4370), 1,
      anon_sym_where,
    STATE(2177), 2,
      sym_line_comment,
      sym_block_comment,
  [50260] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4372), 1,
      anon_sym_COLON,
    STATE(2178), 2,
      sym_line_comment,
      sym_block_comment,
  [50274] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4374), 1,
      anon_sym_where,
    STATE(2179), 2,
      sym_line_comment,
      sym_block_comment,
  [50288] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4376), 1,
      anon_sym_COLON,
    STATE(2180), 2,
      sym_line_comment,
      sym_block_comment,
  [50302] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4378), 1,
      anon_sym_COLON,
    STATE(2181), 2,
      sym_line_comment,
      sym_block_comment,
  [50316] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4380), 1,
      anon_sym_COLON,
    STATE(2182), 2,
      sym_line_comment,
      sym_block_comment,
  [50330] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4382), 1,
      anon_sym_COLON,
    STATE(2183), 2,
      sym_line_comment,
      sym_block_comment,
  [50344] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4384), 1,
      anon_sym_SEMI,
    STATE(2184), 2,
      sym_line_comment,
      sym_block_comment,
  [50358] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4386), 1,
      anon_sym_fun,
    STATE(2185), 2,
      sym_line_comment,
      sym_block_comment,
  [50372] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4388), 1,
      anon_sym_SEMI,
    STATE(2186), 2,
      sym_line_comment,
      sym_block_comment,
  [50386] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4390), 1,
      anon_sym_LBRACE,
    STATE(2187), 2,
      sym_line_comment,
      sym_block_comment,
  [50400] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4392), 1,
      anon_sym_SEMI,
    STATE(2188), 2,
      sym_line_comment,
      sym_block_comment,
  [50414] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4394), 1,
      anon_sym_SEMI,
    STATE(2189), 2,
      sym_line_comment,
      sym_block_comment,
  [50428] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4396), 1,
      anon_sym_LBRACE,
    STATE(2190), 2,
      sym_line_comment,
      sym_block_comment,
  [50442] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4398), 1,
      anon_sym_SEMI,
    STATE(2191), 2,
      sym_line_comment,
      sym_block_comment,
  [50456] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4400), 1,
      anon_sym_COLON,
    STATE(2192), 2,
      sym_line_comment,
      sym_block_comment,
  [50470] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4402), 1,
      sym_identifier,
    STATE(2193), 2,
      sym_line_comment,
      sym_block_comment,
  [50484] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4404), 1,
      anon_sym_RPAREN,
    STATE(2194), 2,
      sym_line_comment,
      sym_block_comment,
  [50498] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4406), 1,
      anon_sym_EQ_GT,
    STATE(2195), 2,
      sym_line_comment,
      sym_block_comment,
  [50512] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4408), 1,
      anon_sym_SEMI,
    STATE(2196), 2,
      sym_line_comment,
      sym_block_comment,
  [50526] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4410), 1,
      anon_sym_SEMI,
    STATE(2197), 2,
      sym_line_comment,
      sym_block_comment,
  [50540] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4412), 1,
      anon_sym_RPAREN,
    STATE(2198), 2,
      sym_line_comment,
      sym_block_comment,
  [50554] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4414), 1,
      anon_sym_SEMI,
    STATE(2199), 2,
      sym_line_comment,
      sym_block_comment,
  [50568] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4416), 1,
      anon_sym_where,
    STATE(2200), 2,
      sym_line_comment,
      sym_block_comment,
  [50582] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4418), 1,
      anon_sym_RPAREN,
    STATE(2201), 2,
      sym_line_comment,
      sym_block_comment,
  [50596] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4420), 1,
      anon_sym_RBRACK,
    STATE(2202), 2,
      sym_line_comment,
      sym_block_comment,
  [50610] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4422), 1,
      anon_sym_where,
    STATE(2203), 2,
      sym_line_comment,
      sym_block_comment,
  [50624] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4424), 1,
      anon_sym_where,
    STATE(2204), 2,
      sym_line_comment,
      sym_block_comment,
  [50638] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4426), 1,
      anon_sym_COLON,
    STATE(2205), 2,
      sym_line_comment,
      sym_block_comment,
  [50652] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4428), 1,
      anon_sym_SEMI,
    STATE(2206), 2,
      sym_line_comment,
      sym_block_comment,
  [50666] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4430), 1,
      anon_sym_COLON,
    STATE(2207), 2,
      sym_line_comment,
      sym_block_comment,
  [50680] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4432), 1,
      anon_sym_COLON,
    STATE(2208), 2,
      sym_line_comment,
      sym_block_comment,
  [50694] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4434), 1,
      anon_sym_COLON,
    STATE(2209), 2,
      sym_line_comment,
      sym_block_comment,
  [50708] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4436), 1,
      anon_sym_RPAREN,
    STATE(2210), 2,
      sym_line_comment,
      sym_block_comment,
  [50722] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4438), 1,
      anon_sym_SEMI,
    STATE(2211), 2,
      sym_line_comment,
      sym_block_comment,
  [50736] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4440), 1,
      sym_identifier,
    STATE(2212), 2,
      sym_line_comment,
      sym_block_comment,
  [50750] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4442), 1,
      anon_sym_EQ,
    STATE(2213), 2,
      sym_line_comment,
      sym_block_comment,
  [50764] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4444), 1,
      sym_identifier,
    STATE(2214), 2,
      sym_line_comment,
      sym_block_comment,
  [50778] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4446), 1,
      anon_sym_LBRACE,
    STATE(2215), 2,
      sym_line_comment,
      sym_block_comment,
  [50792] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4448), 1,
      anon_sym_LBRACE,
    STATE(2216), 2,
      sym_line_comment,
      sym_block_comment,
  [50806] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4450), 1,
      sym_identifier,
    STATE(2217), 2,
      sym_line_comment,
      sym_block_comment,
  [50820] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4452), 1,
      anon_sym_RPAREN,
    STATE(2218), 2,
      sym_line_comment,
      sym_block_comment,
  [50834] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4454), 1,
      anon_sym_RBRACE,
    STATE(2219), 2,
      sym_line_comment,
      sym_block_comment,
  [50848] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4456), 1,
      anon_sym_in,
    STATE(2220), 2,
      sym_line_comment,
      sym_block_comment,
  [50862] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4458), 1,
      anon_sym_RPAREN,
    STATE(2221), 2,
      sym_line_comment,
      sym_block_comment,
  [50876] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4460), 1,
      anon_sym_SEMI,
    STATE(2222), 2,
      sym_line_comment,
      sym_block_comment,
  [50890] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4462), 1,
      anon_sym_LBRACK,
    STATE(2223), 2,
      sym_line_comment,
      sym_block_comment,
  [50904] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4464), 1,
      anon_sym_where,
    STATE(2224), 2,
      sym_line_comment,
      sym_block_comment,
  [50918] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3647), 1,
      anon_sym_RBRACK,
    STATE(2225), 2,
      sym_line_comment,
      sym_block_comment,
  [50932] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4466), 1,
      anon_sym_SEMI,
    STATE(2226), 2,
      sym_line_comment,
      sym_block_comment,
  [50946] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4468), 1,
      anon_sym_where,
    STATE(2227), 2,
      sym_line_comment,
      sym_block_comment,
  [50960] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4470), 1,
      anon_sym_COLON,
    STATE(2228), 2,
      sym_line_comment,
      sym_block_comment,
  [50974] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4472), 1,
      anon_sym_LPAREN,
    STATE(2229), 2,
      sym_line_comment,
      sym_block_comment,
  [50988] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4474), 1,
      anon_sym_COLON,
    STATE(2230), 2,
      sym_line_comment,
      sym_block_comment,
  [51002] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4476), 1,
      anon_sym_COLON,
    STATE(2231), 2,
      sym_line_comment,
      sym_block_comment,
  [51016] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4478), 1,
      anon_sym_COLON,
    STATE(2232), 2,
      sym_line_comment,
      sym_block_comment,
  [51030] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4480), 1,
      anon_sym_SEMI,
    STATE(2233), 2,
      sym_line_comment,
      sym_block_comment,
  [51044] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4482), 1,
      anon_sym_where,
    STATE(2234), 2,
      sym_line_comment,
      sym_block_comment,
  [51058] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4484), 1,
      anon_sym_SEMI,
    STATE(2235), 2,
      sym_line_comment,
      sym_block_comment,
  [51072] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4486), 1,
      sym_identifier,
    STATE(2236), 2,
      sym_line_comment,
      sym_block_comment,
  [51086] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4488), 1,
      anon_sym_LBRACE,
    STATE(2237), 2,
      sym_line_comment,
      sym_block_comment,
  [51100] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4490), 1,
      anon_sym_RPAREN,
    STATE(2238), 2,
      sym_line_comment,
      sym_block_comment,
  [51114] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4492), 1,
      sym_identifier,
    STATE(2239), 2,
      sym_line_comment,
      sym_block_comment,
  [51128] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4494), 1,
      anon_sym_SEMI,
    STATE(2240), 2,
      sym_line_comment,
      sym_block_comment,
  [51142] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4496), 1,
      anon_sym_SEMI,
    STATE(2241), 2,
      sym_line_comment,
      sym_block_comment,
  [51156] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3802), 1,
      anon_sym_GT,
    STATE(2242), 2,
      sym_line_comment,
      sym_block_comment,
  [51170] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4498), 1,
      anon_sym_SEMI,
    STATE(2243), 2,
      sym_line_comment,
      sym_block_comment,
  [51184] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4500), 1,
      anon_sym_RPAREN,
    STATE(2244), 2,
      sym_line_comment,
      sym_block_comment,
  [51198] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4502), 1,
      anon_sym_where,
    STATE(2245), 2,
      sym_line_comment,
      sym_block_comment,
  [51212] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4504), 1,
      anon_sym_where,
    STATE(2246), 2,
      sym_line_comment,
      sym_block_comment,
  [51226] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4506), 1,
      anon_sym_SEMI,
    STATE(2247), 2,
      sym_line_comment,
      sym_block_comment,
  [51240] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4508), 1,
      anon_sym_where,
    STATE(2248), 2,
      sym_line_comment,
      sym_block_comment,
  [51254] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4510), 1,
      anon_sym_LBRACK,
    STATE(2249), 2,
      sym_line_comment,
      sym_block_comment,
  [51268] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4512), 1,
      sym_identifier,
    STATE(2250), 2,
      sym_line_comment,
      sym_block_comment,
  [51282] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4514), 1,
      anon_sym_where,
    STATE(2251), 2,
      sym_line_comment,
      sym_block_comment,
  [51296] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4516), 1,
      anon_sym_COLON,
    STATE(2252), 2,
      sym_line_comment,
      sym_block_comment,
  [51310] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4518), 1,
      anon_sym_COLON,
    STATE(2253), 2,
      sym_line_comment,
      sym_block_comment,
  [51324] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4520), 1,
      anon_sym_COLON,
    STATE(2254), 2,
      sym_line_comment,
      sym_block_comment,
  [51338] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4522), 1,
      anon_sym_COLON,
    STATE(2255), 2,
      sym_line_comment,
      sym_block_comment,
  [51352] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4524), 1,
      anon_sym_COLON,
    STATE(2256), 2,
      sym_line_comment,
      sym_block_comment,
  [51366] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4526), 1,
      anon_sym_COLON,
    STATE(2257), 2,
      sym_line_comment,
      sym_block_comment,
  [51380] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4528), 1,
      anon_sym_COLON,
    STATE(2258), 2,
      sym_line_comment,
      sym_block_comment,
  [51394] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4530), 1,
      anon_sym_where,
    STATE(2259), 2,
      sym_line_comment,
      sym_block_comment,
  [51408] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4532), 1,
      sym_identifier,
    STATE(2260), 2,
      sym_line_comment,
      sym_block_comment,
  [51422] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4534), 1,
      anon_sym_COLON,
    STATE(2261), 2,
      sym_line_comment,
      sym_block_comment,
  [51436] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4536), 1,
      anon_sym_COLON,
    STATE(2262), 2,
      sym_line_comment,
      sym_block_comment,
  [51450] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4538), 1,
      anon_sym_COLON,
    STATE(2263), 2,
      sym_line_comment,
      sym_block_comment,
  [51464] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4540), 1,
      anon_sym_SEMI,
    STATE(2264), 2,
      sym_line_comment,
      sym_block_comment,
  [51478] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4542), 1,
      anon_sym_to,
    STATE(2265), 2,
      sym_line_comment,
      sym_block_comment,
  [51492] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4544), 1,
      anon_sym_where,
    STATE(2266), 2,
      sym_line_comment,
      sym_block_comment,
  [51506] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4546), 1,
      anon_sym_LBRACE,
    STATE(2267), 2,
      sym_line_comment,
      sym_block_comment,
  [51520] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4548), 1,
      anon_sym_where,
    STATE(2268), 2,
      sym_line_comment,
      sym_block_comment,
  [51534] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4550), 1,
      sym_identifier,
    STATE(2269), 2,
      sym_line_comment,
      sym_block_comment,
  [51548] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4552), 1,
      anon_sym_SEMI,
    STATE(2270), 2,
      sym_line_comment,
      sym_block_comment,
  [51562] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(513), 1,
      anon_sym_RBRACK,
    STATE(2271), 2,
      sym_line_comment,
      sym_block_comment,
  [51576] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4554), 1,
      anon_sym_SEMI,
    STATE(2272), 2,
      sym_line_comment,
      sym_block_comment,
  [51590] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4556), 1,
      anon_sym_RPAREN,
    STATE(2273), 2,
      sym_line_comment,
      sym_block_comment,
  [51604] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4558), 1,
      anon_sym_COLON,
    STATE(2274), 2,
      sym_line_comment,
      sym_block_comment,
  [51618] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4560), 1,
      anon_sym_SEMI,
    STATE(2275), 2,
      sym_line_comment,
      sym_block_comment,
  [51632] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4562), 1,
      anon_sym_LBRACE,
    STATE(2276), 2,
      sym_line_comment,
      sym_block_comment,
  [51646] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4564), 1,
      sym_identifier,
    STATE(2277), 2,
      sym_line_comment,
      sym_block_comment,
  [51660] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4031), 1,
      anon_sym_GT,
    STATE(2278), 2,
      sym_line_comment,
      sym_block_comment,
  [51674] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4566), 1,
      anon_sym_LBRACE,
    STATE(2279), 2,
      sym_line_comment,
      sym_block_comment,
  [51688] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4568), 1,
      anon_sym_SEMI,
    STATE(2280), 2,
      sym_line_comment,
      sym_block_comment,
  [51702] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4570), 1,
      sym_identifier,
    STATE(2281), 2,
      sym_line_comment,
      sym_block_comment,
  [51716] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4572), 1,
      anon_sym_RPAREN,
    STATE(2282), 2,
      sym_line_comment,
      sym_block_comment,
  [51730] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4574), 1,
      anon_sym_RPAREN,
    STATE(2283), 2,
      sym_line_comment,
      sym_block_comment,
  [51744] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4576), 1,
      sym_identifier,
    STATE(2284), 2,
      sym_line_comment,
      sym_block_comment,
  [51758] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4578), 1,
      anon_sym_where,
    STATE(2285), 2,
      sym_line_comment,
      sym_block_comment,
  [51772] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4580), 1,
      anon_sym_fun,
    STATE(2286), 2,
      sym_line_comment,
      sym_block_comment,
  [51786] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4582), 1,
      anon_sym_COLON,
    STATE(2287), 2,
      sym_line_comment,
      sym_block_comment,
  [51800] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4584), 1,
      anon_sym_LBRACE,
    STATE(2288), 2,
      sym_line_comment,
      sym_block_comment,
  [51814] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4586), 1,
      anon_sym_SEMI,
    STATE(2289), 2,
      sym_line_comment,
      sym_block_comment,
  [51828] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4588), 1,
      anon_sym_COLON_COLON,
    STATE(2290), 2,
      sym_line_comment,
      sym_block_comment,
  [51842] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4590), 1,
      anon_sym_where,
    STATE(2291), 2,
      sym_line_comment,
      sym_block_comment,
  [51856] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4592), 1,
      anon_sym_COLON,
    STATE(2292), 2,
      sym_line_comment,
      sym_block_comment,
  [51870] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4594), 1,
      sym_identifier,
    STATE(2293), 2,
      sym_line_comment,
      sym_block_comment,
  [51884] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4596), 1,
      anon_sym_COLON,
    STATE(2294), 2,
      sym_line_comment,
      sym_block_comment,
  [51898] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4598), 1,
      anon_sym_COLON,
    STATE(2295), 2,
      sym_line_comment,
      sym_block_comment,
  [51912] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4600), 1,
      anon_sym_COLON,
    STATE(2296), 2,
      sym_line_comment,
      sym_block_comment,
  [51926] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4602), 1,
      anon_sym_COLON,
    STATE(2297), 2,
      sym_line_comment,
      sym_block_comment,
  [51940] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4604), 1,
      anon_sym_COLON,
    STATE(2298), 2,
      sym_line_comment,
      sym_block_comment,
  [51954] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4606), 1,
      anon_sym_SEMI,
    STATE(2299), 2,
      sym_line_comment,
      sym_block_comment,
  [51968] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4608), 1,
      anon_sym_where,
    STATE(2300), 2,
      sym_line_comment,
      sym_block_comment,
  [51982] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4360), 1,
      aux_sym_number_token3,
    STATE(2301), 2,
      sym_line_comment,
      sym_block_comment,
  [51996] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4610), 1,
      anon_sym_SEMI,
    STATE(2302), 2,
      sym_line_comment,
      sym_block_comment,
  [52010] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4612), 1,
      anon_sym_where,
    STATE(2303), 2,
      sym_line_comment,
      sym_block_comment,
  [52024] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4614), 1,
      anon_sym_where,
    STATE(2304), 2,
      sym_line_comment,
      sym_block_comment,
  [52038] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4616), 1,
      anon_sym_RPAREN,
    STATE(2305), 2,
      sym_line_comment,
      sym_block_comment,
  [52052] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4618), 1,
      anon_sym_RPAREN,
    STATE(2306), 2,
      sym_line_comment,
      sym_block_comment,
  [52066] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3963), 1,
      anon_sym_COMMA,
    STATE(2307), 2,
      sym_line_comment,
      sym_block_comment,
  [52080] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3895), 1,
      anon_sym_RBRACE,
    STATE(2308), 2,
      sym_line_comment,
      sym_block_comment,
  [52094] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4620), 1,
      anon_sym_COLON,
    STATE(2309), 2,
      sym_line_comment,
      sym_block_comment,
  [52108] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4622), 1,
      anon_sym_where,
    STATE(2310), 2,
      sym_line_comment,
      sym_block_comment,
  [52122] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4624), 1,
      anon_sym_COLON,
    STATE(2311), 2,
      sym_line_comment,
      sym_block_comment,
  [52136] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4626), 1,
      anon_sym_where,
    STATE(2312), 2,
      sym_line_comment,
      sym_block_comment,
  [52150] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4628), 1,
      anon_sym_COLON,
    STATE(2313), 2,
      sym_line_comment,
      sym_block_comment,
  [52164] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4630), 1,
      anon_sym_LBRACE,
    STATE(2314), 2,
      sym_line_comment,
      sym_block_comment,
  [52178] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4632), 1,
      anon_sym_where,
    STATE(2315), 2,
      sym_line_comment,
      sym_block_comment,
  [52192] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2716), 1,
      anon_sym_PIPE,
    STATE(2316), 2,
      sym_line_comment,
      sym_block_comment,
  [52206] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4634), 1,
      anon_sym_to,
    STATE(2317), 2,
      sym_line_comment,
      sym_block_comment,
  [52220] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4636), 1,
      anon_sym_COLON,
    STATE(2318), 2,
      sym_line_comment,
      sym_block_comment,
  [52234] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4638), 1,
      anon_sym_COLON,
    STATE(2319), 2,
      sym_line_comment,
      sym_block_comment,
  [52248] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4640), 1,
      anon_sym_COLON,
    STATE(2320), 2,
      sym_line_comment,
      sym_block_comment,
  [52262] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3997), 1,
      anon_sym_RPAREN,
    STATE(2321), 2,
      sym_line_comment,
      sym_block_comment,
  [52276] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4642), 1,
      anon_sym_EQ,
    STATE(2322), 2,
      sym_line_comment,
      sym_block_comment,
  [52290] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4644), 1,
      anon_sym_SEMI,
    STATE(2323), 2,
      sym_line_comment,
      sym_block_comment,
  [52304] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4646), 1,
      anon_sym_COLON,
    STATE(2324), 2,
      sym_line_comment,
      sym_block_comment,
  [52318] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4045), 1,
      anon_sym_RBRACE,
    STATE(2325), 2,
      sym_line_comment,
      sym_block_comment,
  [52332] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4648), 1,
      anon_sym_COLON,
    STATE(2326), 2,
      sym_line_comment,
      sym_block_comment,
  [52346] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4650), 1,
      anon_sym_SEMI,
    STATE(2327), 2,
      sym_line_comment,
      sym_block_comment,
  [52360] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4652), 1,
      anon_sym_SEMI,
    STATE(2328), 2,
      sym_line_comment,
      sym_block_comment,
  [52374] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4654), 1,
      anon_sym_where,
    STATE(2329), 2,
      sym_line_comment,
      sym_block_comment,
  [52388] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4011), 1,
      anon_sym_RBRACE,
    STATE(2330), 2,
      sym_line_comment,
      sym_block_comment,
  [52402] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4656), 1,
      anon_sym_SEMI,
    STATE(2331), 2,
      sym_line_comment,
      sym_block_comment,
  [52416] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4658), 1,
      anon_sym_SEMI,
    STATE(2332), 2,
      sym_line_comment,
      sym_block_comment,
  [52430] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4246), 1,
      anon_sym_STAR_SLASH,
    STATE(2333), 2,
      sym_line_comment,
      sym_block_comment,
  [52444] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2862), 1,
      anon_sym_COLON_COLON,
    STATE(2334), 2,
      sym_line_comment,
      sym_block_comment,
  [52458] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4660), 1,
      anon_sym_COLON,
    STATE(2335), 2,
      sym_line_comment,
      sym_block_comment,
  [52472] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4662), 1,
      anon_sym_SEMI,
    STATE(2336), 2,
      sym_line_comment,
      sym_block_comment,
  [52486] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4664), 1,
      anon_sym_COLON,
    STATE(2337), 2,
      sym_line_comment,
      sym_block_comment,
  [52500] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4666), 1,
      anon_sym_where,
    STATE(2338), 2,
      sym_line_comment,
      sym_block_comment,
  [52514] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4668), 1,
      anon_sym_COLON,
    STATE(2339), 2,
      sym_line_comment,
      sym_block_comment,
  [52528] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2866), 1,
      anon_sym_COLON_COLON,
    STATE(2340), 2,
      sym_line_comment,
      sym_block_comment,
  [52542] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3688), 1,
      anon_sym_RBRACE,
    STATE(2341), 2,
      sym_line_comment,
      sym_block_comment,
  [52556] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(839), 1,
      anon_sym_COLON_COLON,
    STATE(2342), 2,
      sym_line_comment,
      sym_block_comment,
  [52570] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4670), 1,
      anon_sym_LBRACE,
    STATE(2343), 2,
      sym_line_comment,
      sym_block_comment,
  [52584] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4672), 1,
      anon_sym_SEMI,
    STATE(2344), 2,
      sym_line_comment,
      sym_block_comment,
  [52598] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4674), 1,
      anon_sym_COLON,
    STATE(2345), 2,
      sym_line_comment,
      sym_block_comment,
  [52612] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4676), 1,
      anon_sym_LBRACE,
    STATE(2346), 2,
      sym_line_comment,
      sym_block_comment,
  [52626] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(415), 1,
      anon_sym_RBRACK,
    STATE(2347), 2,
      sym_line_comment,
      sym_block_comment,
  [52640] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4678), 1,
      anon_sym_SEMI,
    STATE(2348), 2,
      sym_line_comment,
      sym_block_comment,
  [52654] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4680), 1,
      anon_sym_RPAREN,
    STATE(2349), 2,
      sym_line_comment,
      sym_block_comment,
  [52668] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4682), 1,
      anon_sym_where,
    STATE(2350), 2,
      sym_line_comment,
      sym_block_comment,
  [52682] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4684), 1,
      anon_sym_SEMI,
    STATE(2351), 2,
      sym_line_comment,
      sym_block_comment,
  [52696] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4686), 1,
      anon_sym_SEMI,
    STATE(2352), 2,
      sym_line_comment,
      sym_block_comment,
  [52710] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4688), 1,
      anon_sym_where,
    STATE(2353), 2,
      sym_line_comment,
      sym_block_comment,
  [52724] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4690), 1,
      anon_sym_COLON,
    STATE(2354), 2,
      sym_line_comment,
      sym_block_comment,
  [52738] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4692), 1,
      anon_sym_COLON,
    STATE(2355), 2,
      sym_line_comment,
      sym_block_comment,
  [52752] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4694), 1,
      anon_sym_COLON,
    STATE(2356), 2,
      sym_line_comment,
      sym_block_comment,
  [52766] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4696), 1,
      anon_sym_COLON,
    STATE(2357), 2,
      sym_line_comment,
      sym_block_comment,
  [52780] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4698), 1,
      anon_sym_COLON,
    STATE(2358), 2,
      sym_line_comment,
      sym_block_comment,
  [52794] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4700), 1,
      anon_sym_where,
    STATE(2359), 2,
      sym_line_comment,
      sym_block_comment,
  [52808] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4702), 1,
      anon_sym_COLON,
    STATE(2360), 2,
      sym_line_comment,
      sym_block_comment,
  [52822] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3959), 1,
      anon_sym_RBRACE,
    STATE(2361), 2,
      sym_line_comment,
      sym_block_comment,
  [52836] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4704), 1,
      ts_builtin_sym_end,
    STATE(2362), 2,
      sym_line_comment,
      sym_block_comment,
  [52850] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4706), 1,
      anon_sym_SEMI,
    STATE(2363), 2,
      sym_line_comment,
      sym_block_comment,
  [52864] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4708), 1,
      anon_sym_SEMI,
    STATE(2364), 2,
      sym_line_comment,
      sym_block_comment,
  [52878] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4710), 1,
      anon_sym_COLON,
    STATE(2365), 2,
      sym_line_comment,
      sym_block_comment,
  [52892] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4712), 1,
      anon_sym_where,
    STATE(2366), 2,
      sym_line_comment,
      sym_block_comment,
  [52906] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4714), 1,
      anon_sym_COLON,
    STATE(2367), 2,
      sym_line_comment,
      sym_block_comment,
  [52920] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4716), 1,
      anon_sym_COLON,
    STATE(2368), 2,
      sym_line_comment,
      sym_block_comment,
  [52934] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4718), 1,
      sym_identifier,
    STATE(2369), 2,
      sym_line_comment,
      sym_block_comment,
  [52948] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3128), 1,
      anon_sym_RBRACE,
    STATE(2370), 2,
      sym_line_comment,
      sym_block_comment,
  [52962] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4720), 1,
      anon_sym_COLON_COLON,
    STATE(2371), 2,
      sym_line_comment,
      sym_block_comment,
  [52976] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4722), 1,
      anon_sym_COLON,
    STATE(2372), 2,
      sym_line_comment,
      sym_block_comment,
  [52990] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3913), 1,
      anon_sym_RPAREN,
    STATE(2373), 2,
      sym_line_comment,
      sym_block_comment,
  [53004] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4724), 1,
      anon_sym_where,
    STATE(2374), 2,
      sym_line_comment,
      sym_block_comment,
  [53018] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4726), 1,
      anon_sym_EQ,
    STATE(2375), 2,
      sym_line_comment,
      sym_block_comment,
  [53032] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4728), 1,
      anon_sym_COLON,
    STATE(2376), 2,
      sym_line_comment,
      sym_block_comment,
  [53046] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4730), 1,
      anon_sym_where,
    STATE(2377), 2,
      sym_line_comment,
      sym_block_comment,
  [53060] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4732), 1,
      anon_sym_COLON,
    STATE(2378), 2,
      sym_line_comment,
      sym_block_comment,
  [53074] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4734), 1,
      anon_sym_COLON,
    STATE(2379), 2,
      sym_line_comment,
      sym_block_comment,
  [53088] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4736), 1,
      anon_sym_COLON,
    STATE(2380), 2,
      sym_line_comment,
      sym_block_comment,
  [53102] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4738), 1,
      anon_sym_COLON,
    STATE(2381), 2,
      sym_line_comment,
      sym_block_comment,
  [53116] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4740), 1,
      anon_sym_COLON,
    STATE(2382), 2,
      sym_line_comment,
      sym_block_comment,
  [53130] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4742), 1,
      anon_sym_to,
    STATE(2383), 2,
      sym_line_comment,
      sym_block_comment,
  [53144] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4744), 1,
      sym_identifier,
    STATE(2384), 2,
      sym_line_comment,
      sym_block_comment,
  [53158] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4746), 1,
      anon_sym_SEMI,
    STATE(2385), 2,
      sym_line_comment,
      sym_block_comment,
  [53172] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4748), 1,
      anon_sym_SEMI,
    STATE(2386), 2,
      sym_line_comment,
      sym_block_comment,
  [53186] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3777), 1,
      anon_sym_RBRACE,
    STATE(2387), 2,
      sym_line_comment,
      sym_block_comment,
  [53200] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4750), 1,
      anon_sym_COLON,
    STATE(2388), 2,
      sym_line_comment,
      sym_block_comment,
  [53214] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3989), 1,
      anon_sym_RPAREN,
    STATE(2389), 2,
      sym_line_comment,
      sym_block_comment,
  [53228] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4752), 1,
      anon_sym_SEMI,
    STATE(2390), 2,
      sym_line_comment,
      sym_block_comment,
  [53242] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4754), 1,
      anon_sym_COLON,
    STATE(2391), 2,
      sym_line_comment,
      sym_block_comment,
  [53256] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4756), 1,
      sym_identifier,
    STATE(2392), 2,
      sym_line_comment,
      sym_block_comment,
  [53270] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4758), 1,
      anon_sym_where,
    STATE(2393), 2,
      sym_line_comment,
      sym_block_comment,
  [53284] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4760), 1,
      anon_sym_SEMI,
    STATE(2394), 2,
      sym_line_comment,
      sym_block_comment,
  [53298] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4762), 1,
      anon_sym_COLON,
    STATE(2395), 2,
      sym_line_comment,
      sym_block_comment,
  [53312] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4764), 1,
      anon_sym_SEMI,
    STATE(2396), 2,
      sym_line_comment,
      sym_block_comment,
  [53326] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4766), 1,
      anon_sym_COLON,
    STATE(2397), 2,
      sym_line_comment,
      sym_block_comment,
  [53340] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4768), 1,
      anon_sym_SEMI,
    STATE(2398), 2,
      sym_line_comment,
      sym_block_comment,
  [53354] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4770), 1,
      anon_sym_COLON,
    STATE(2399), 2,
      sym_line_comment,
      sym_block_comment,
  [53368] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3816), 1,
      anon_sym_GT,
    STATE(2400), 2,
      sym_line_comment,
      sym_block_comment,
  [53382] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4123), 1,
      anon_sym_SEMI,
    STATE(2401), 2,
      sym_line_comment,
      sym_block_comment,
  [53396] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4772), 1,
      anon_sym_STAR_SLASH,
    STATE(2402), 2,
      sym_line_comment,
      sym_block_comment,
  [53410] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4774), 1,
      anon_sym_COLON,
    STATE(2403), 2,
      sym_line_comment,
      sym_block_comment,
  [53424] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3843), 1,
      anon_sym_RPAREN,
    STATE(2404), 2,
      sym_line_comment,
      sym_block_comment,
  [53438] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4776), 1,
      anon_sym_where,
    STATE(2405), 2,
      sym_line_comment,
      sym_block_comment,
  [53452] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4778), 1,
      anon_sym_COLON,
    STATE(2406), 2,
      sym_line_comment,
      sym_block_comment,
  [53466] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4780), 1,
      sym_identifier,
    STATE(2407), 2,
      sym_line_comment,
      sym_block_comment,
  [53480] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4782), 1,
      anon_sym_COLON,
    STATE(2408), 2,
      sym_line_comment,
      sym_block_comment,
  [53494] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4784), 1,
      anon_sym_where,
    STATE(2409), 2,
      sym_line_comment,
      sym_block_comment,
  [53508] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4786), 1,
      anon_sym_RPAREN,
    STATE(2410), 2,
      sym_line_comment,
      sym_block_comment,
  [53522] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3401), 1,
      anon_sym_RPAREN,
    STATE(2411), 2,
      sym_line_comment,
      sym_block_comment,
  [53536] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4788), 1,
      anon_sym_where,
    STATE(2412), 2,
      sym_line_comment,
      sym_block_comment,
  [53550] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4790), 1,
      anon_sym_COLON,
    STATE(2413), 2,
      sym_line_comment,
      sym_block_comment,
  [53564] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4792), 1,
      sym_identifier,
    STATE(2414), 2,
      sym_line_comment,
      sym_block_comment,
  [53578] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4794), 1,
      anon_sym_COLON,
    STATE(2415), 2,
      sym_line_comment,
      sym_block_comment,
  [53592] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4796), 1,
      anon_sym_COLON,
    STATE(2416), 2,
      sym_line_comment,
      sym_block_comment,
  [53606] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4798), 1,
      anon_sym_COLON,
    STATE(2417), 2,
      sym_line_comment,
      sym_block_comment,
  [53620] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4800), 1,
      anon_sym_SEMI,
    STATE(2418), 2,
      sym_line_comment,
      sym_block_comment,
  [53634] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4802), 1,
      anon_sym_SEMI,
    STATE(2419), 2,
      sym_line_comment,
      sym_block_comment,
  [53648] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4804), 1,
      anon_sym_COLON,
    STATE(2420), 2,
      sym_line_comment,
      sym_block_comment,
  [53662] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4806), 1,
      anon_sym_SEMI,
    STATE(2421), 2,
      sym_line_comment,
      sym_block_comment,
  [53676] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4808), 1,
      anon_sym_COLON,
    STATE(2422), 2,
      sym_line_comment,
      sym_block_comment,
  [53690] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4810), 1,
      anon_sym_SEMI,
    STATE(2423), 2,
      sym_line_comment,
      sym_block_comment,
  [53704] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4812), 1,
      anon_sym_DOT_DOT,
    STATE(2424), 2,
      sym_line_comment,
      sym_block_comment,
  [53718] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4814), 1,
      anon_sym_where,
    STATE(2425), 2,
      sym_line_comment,
      sym_block_comment,
  [53732] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4816), 1,
      anon_sym_SEMI,
    STATE(2426), 2,
      sym_line_comment,
      sym_block_comment,
  [53746] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4818), 1,
      anon_sym_SEMI,
    STATE(2427), 2,
      sym_line_comment,
      sym_block_comment,
  [53760] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4820), 1,
      anon_sym_COLON,
    STATE(2428), 2,
      sym_line_comment,
      sym_block_comment,
  [53774] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4822), 1,
      anon_sym_SEMI,
    STATE(2429), 2,
      sym_line_comment,
      sym_block_comment,
  [53788] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3773), 1,
      anon_sym_GT,
    STATE(2430), 2,
      sym_line_comment,
      sym_block_comment,
  [53802] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4824), 1,
      anon_sym_where,
    STATE(2431), 2,
      sym_line_comment,
      sym_block_comment,
  [53816] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4826), 1,
      sym_identifier,
    STATE(2432), 2,
      sym_line_comment,
      sym_block_comment,
  [53830] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4828), 1,
      aux_sym_line_comment_token1,
    STATE(2433), 2,
      sym_line_comment,
      sym_block_comment,
  [53844] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4360), 1,
      aux_sym_number_token7,
    STATE(2434), 2,
      sym_line_comment,
      sym_block_comment,
  [53858] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4830), 1,
      anon_sym_where,
    STATE(2435), 2,
      sym_line_comment,
      sym_block_comment,
  [53872] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4832), 1,
      anon_sym_COLON,
    STATE(2436), 2,
      sym_line_comment,
      sym_block_comment,
  [53886] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4834), 1,
      anon_sym_COLON,
    STATE(2437), 2,
      sym_line_comment,
      sym_block_comment,
  [53900] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4836), 1,
      anon_sym_COLON,
    STATE(2438), 2,
      sym_line_comment,
      sym_block_comment,
  [53914] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4838), 1,
      anon_sym_COLON,
    STATE(2439), 2,
      sym_line_comment,
      sym_block_comment,
  [53928] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4840), 1,
      anon_sym_SEMI,
    STATE(2440), 2,
      sym_line_comment,
      sym_block_comment,
  [53942] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4842), 1,
      anon_sym_COLON,
    STATE(2441), 2,
      sym_line_comment,
      sym_block_comment,
  [53956] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4844), 1,
      anon_sym_COLON,
    STATE(2442), 2,
      sym_line_comment,
      sym_block_comment,
  [53970] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4846), 1,
      anon_sym_EQ,
    STATE(2443), 2,
      sym_line_comment,
      sym_block_comment,
  [53984] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4848), 1,
      anon_sym_COLON,
    STATE(2444), 2,
      sym_line_comment,
      sym_block_comment,
  [53998] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4850), 1,
      anon_sym_fun,
    STATE(2445), 2,
      sym_line_comment,
      sym_block_comment,
  [54012] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4852), 1,
      sym__doc_line_comment,
    STATE(2446), 2,
      sym_line_comment,
      sym_block_comment,
  [54026] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4854), 1,
      anon_sym_SEMI,
    STATE(2447), 2,
      sym_line_comment,
      sym_block_comment,
  [54040] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4856), 1,
      anon_sym_COLON,
    STATE(2448), 2,
      sym_line_comment,
      sym_block_comment,
  [54054] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3682), 1,
      anon_sym_PIPE,
    STATE(2449), 2,
      sym_line_comment,
      sym_block_comment,
  [54068] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4858), 1,
      anon_sym_SEMI,
    STATE(2450), 2,
      sym_line_comment,
      sym_block_comment,
  [54082] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4860), 1,
      anon_sym_COLON,
    STATE(2451), 2,
      sym_line_comment,
      sym_block_comment,
  [54096] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4862), 1,
      anon_sym_PIPE,
    STATE(2452), 2,
      sym_line_comment,
      sym_block_comment,
  [54110] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4864), 1,
      anon_sym_SEMI,
    STATE(2453), 2,
      sym_line_comment,
      sym_block_comment,
  [54124] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4866), 1,
      anon_sym_COLON,
    STATE(2454), 2,
      sym_line_comment,
      sym_block_comment,
  [54138] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4868), 1,
      anon_sym_COLON,
    STATE(2455), 2,
      sym_line_comment,
      sym_block_comment,
  [54152] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4870), 1,
      anon_sym_COLON,
    STATE(2456), 2,
      sym_line_comment,
      sym_block_comment,
  [54166] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4872), 1,
      anon_sym_RBRACK,
    STATE(2457), 2,
      sym_line_comment,
      sym_block_comment,
  [54180] = 4,
    ACTIONS(19), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(21), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4874), 1,
      anon_sym_RPAREN,
    STATE(2458), 2,
      sym_line_comment,
      sym_block_comment,
  [54194] = 1,
    ACTIONS(4876), 1,
      ts_builtin_sym_end,
  [54198] = 1,
    ACTIONS(4878), 1,
      ts_builtin_sym_end,
  [54202] = 1,
    ACTIONS(4880), 1,
      ts_builtin_sym_end,
  [54206] = 1,
    ACTIONS(4882), 1,
      ts_builtin_sym_end,
  [54210] = 1,
    ACTIONS(4884), 1,
      ts_builtin_sym_end,
};

static const uint32_t ts_small_parse_table_map[] = {
  [SMALL_STATE(957)] = 0,
  [SMALL_STATE(958)] = 71,
  [SMALL_STATE(959)] = 148,
  [SMALL_STATE(960)] = 231,
  [SMALL_STATE(961)] = 302,
  [SMALL_STATE(962)] = 377,
  [SMALL_STATE(963)] = 452,
  [SMALL_STATE(964)] = 527,
  [SMALL_STATE(965)] = 604,
  [SMALL_STATE(966)] = 675,
  [SMALL_STATE(967)] = 746,
  [SMALL_STATE(968)] = 817,
  [SMALL_STATE(969)] = 888,
  [SMALL_STATE(970)] = 959,
  [SMALL_STATE(971)] = 1033,
  [SMALL_STATE(972)] = 1107,
  [SMALL_STATE(973)] = 1181,
  [SMALL_STATE(974)] = 1257,
  [SMALL_STATE(975)] = 1333,
  [SMALL_STATE(976)] = 1407,
  [SMALL_STATE(977)] = 1526,
  [SMALL_STATE(978)] = 1645,
  [SMALL_STATE(979)] = 1718,
  [SMALL_STATE(980)] = 1839,
  [SMALL_STATE(981)] = 1960,
  [SMALL_STATE(982)] = 2081,
  [SMALL_STATE(983)] = 2155,
  [SMALL_STATE(984)] = 2273,
  [SMALL_STATE(985)] = 2345,
  [SMALL_STATE(986)] = 2463,
  [SMALL_STATE(987)] = 2553,
  [SMALL_STATE(988)] = 2621,
  [SMALL_STATE(989)] = 2689,
  [SMALL_STATE(990)] = 2769,
  [SMALL_STATE(991)] = 2847,
  [SMALL_STATE(992)] = 2921,
  [SMALL_STATE(993)] = 2991,
  [SMALL_STATE(994)] = 3059,
  [SMALL_STATE(995)] = 3141,
  [SMALL_STATE(996)] = 3217,
  [SMALL_STATE(997)] = 3305,
  [SMALL_STATE(998)] = 3397,
  [SMALL_STATE(999)] = 3483,
  [SMALL_STATE(1000)] = 3555,
  [SMALL_STATE(1001)] = 3624,
  [SMALL_STATE(1002)] = 3691,
  [SMALL_STATE(1003)] = 3782,
  [SMALL_STATE(1004)] = 3859,
  [SMALL_STATE(1005)] = 3936,
  [SMALL_STATE(1006)] = 4009,
  [SMALL_STATE(1007)] = 4098,
  [SMALL_STATE(1008)] = 4185,
  [SMALL_STATE(1009)] = 4270,
  [SMALL_STATE(1010)] = 4351,
  [SMALL_STATE(1011)] = 4430,
  [SMALL_STATE(1012)] = 4505,
  [SMALL_STATE(1013)] = 4576,
  [SMALL_STATE(1014)] = 4645,
  [SMALL_STATE(1015)] = 4736,
  [SMALL_STATE(1016)] = 4809,
  [SMALL_STATE(1017)] = 4898,
  [SMALL_STATE(1018)] = 4965,
  [SMALL_STATE(1019)] = 5052,
  [SMALL_STATE(1020)] = 5119,
  [SMALL_STATE(1021)] = 5204,
  [SMALL_STATE(1022)] = 5285,
  [SMALL_STATE(1023)] = 5364,
  [SMALL_STATE(1024)] = 5431,
  [SMALL_STATE(1025)] = 5506,
  [SMALL_STATE(1026)] = 5577,
  [SMALL_STATE(1027)] = 5641,
  [SMALL_STATE(1028)] = 5717,
  [SMALL_STATE(1029)] = 5789,
  [SMALL_STATE(1030)] = 5877,
  [SMALL_STATE(1031)] = 5961,
  [SMALL_STATE(1032)] = 6047,
  [SMALL_STATE(1033)] = 6115,
  [SMALL_STATE(1034)] = 6205,
  [SMALL_STATE(1035)] = 6269,
  [SMALL_STATE(1036)] = 6333,
  [SMALL_STATE(1037)] = 6413,
  [SMALL_STATE(1038)] = 6491,
  [SMALL_STATE(1039)] = 6557,
  [SMALL_STATE(1040)] = 6625,
  [SMALL_STATE(1041)] = 6699,
  [SMALL_STATE(1042)] = 6769,
  [SMALL_STATE(1043)] = 6833,
  [SMALL_STATE(1044)] = 6900,
  [SMALL_STATE(1045)] = 6968,
  [SMALL_STATE(1046)] = 7029,
  [SMALL_STATE(1047)] = 7089,
  [SMALL_STATE(1048)] = 7149,
  [SMALL_STATE(1049)] = 7214,
  [SMALL_STATE(1050)] = 7279,
  [SMALL_STATE(1051)] = 7350,
  [SMALL_STATE(1052)] = 7408,
  [SMALL_STATE(1053)] = 7472,
  [SMALL_STATE(1054)] = 7530,
  [SMALL_STATE(1055)] = 7588,
  [SMALL_STATE(1056)] = 7650,
  [SMALL_STATE(1057)] = 7708,
  [SMALL_STATE(1058)] = 7766,
  [SMALL_STATE(1059)] = 7824,
  [SMALL_STATE(1060)] = 7886,
  [SMALL_STATE(1061)] = 7944,
  [SMALL_STATE(1062)] = 8008,
  [SMALL_STATE(1063)] = 8066,
  [SMALL_STATE(1064)] = 8124,
  [SMALL_STATE(1065)] = 8182,
  [SMALL_STATE(1066)] = 8240,
  [SMALL_STATE(1067)] = 8298,
  [SMALL_STATE(1068)] = 8356,
  [SMALL_STATE(1069)] = 8463,
  [SMALL_STATE(1070)] = 8570,
  [SMALL_STATE(1071)] = 8631,
  [SMALL_STATE(1072)] = 8738,
  [SMALL_STATE(1073)] = 8845,
  [SMALL_STATE(1074)] = 8906,
  [SMALL_STATE(1075)] = 8967,
  [SMALL_STATE(1076)] = 9063,
  [SMALL_STATE(1077)] = 9167,
  [SMALL_STATE(1078)] = 9271,
  [SMALL_STATE(1079)] = 9375,
  [SMALL_STATE(1080)] = 9479,
  [SMALL_STATE(1081)] = 9583,
  [SMALL_STATE(1082)] = 9645,
  [SMALL_STATE(1083)] = 9705,
  [SMALL_STATE(1084)] = 9809,
  [SMALL_STATE(1085)] = 9913,
  [SMALL_STATE(1086)] = 9973,
  [SMALL_STATE(1087)] = 10077,
  [SMALL_STATE(1088)] = 10181,
  [SMALL_STATE(1089)] = 10285,
  [SMALL_STATE(1090)] = 10386,
  [SMALL_STATE(1091)] = 10487,
  [SMALL_STATE(1092)] = 10588,
  [SMALL_STATE(1093)] = 10685,
  [SMALL_STATE(1094)] = 10786,
  [SMALL_STATE(1095)] = 10887,
  [SMALL_STATE(1096)] = 10988,
  [SMALL_STATE(1097)] = 11089,
  [SMALL_STATE(1098)] = 11190,
  [SMALL_STATE(1099)] = 11287,
  [SMALL_STATE(1100)] = 11388,
  [SMALL_STATE(1101)] = 11489,
  [SMALL_STATE(1102)] = 11590,
  [SMALL_STATE(1103)] = 11691,
  [SMALL_STATE(1104)] = 11792,
  [SMALL_STATE(1105)] = 11883,
  [SMALL_STATE(1106)] = 11984,
  [SMALL_STATE(1107)] = 12043,
  [SMALL_STATE(1108)] = 12140,
  [SMALL_STATE(1109)] = 12241,
  [SMALL_STATE(1110)] = 12342,
  [SMALL_STATE(1111)] = 12439,
  [SMALL_STATE(1112)] = 12540,
  [SMALL_STATE(1113)] = 12641,
  [SMALL_STATE(1114)] = 12742,
  [SMALL_STATE(1115)] = 12843,
  [SMALL_STATE(1116)] = 12944,
  [SMALL_STATE(1117)] = 13045,
  [SMALL_STATE(1118)] = 13146,
  [SMALL_STATE(1119)] = 13236,
  [SMALL_STATE(1120)] = 13326,
  [SMALL_STATE(1121)] = 13421,
  [SMALL_STATE(1122)] = 13514,
  [SMALL_STATE(1123)] = 13573,
  [SMALL_STATE(1124)] = 13632,
  [SMALL_STATE(1125)] = 13727,
  [SMALL_STATE(1126)] = 13815,
  [SMALL_STATE(1127)] = 13871,
  [SMALL_STATE(1128)] = 13927,
  [SMALL_STATE(1129)] = 14015,
  [SMALL_STATE(1130)] = 14103,
  [SMALL_STATE(1131)] = 14191,
  [SMALL_STATE(1132)] = 14249,
  [SMALL_STATE(1133)] = 14335,
  [SMALL_STATE(1134)] = 14393,
  [SMALL_STATE(1135)] = 14451,
  [SMALL_STATE(1136)] = 14509,
  [SMALL_STATE(1137)] = 14580,
  [SMALL_STATE(1138)] = 14669,
  [SMALL_STATE(1139)] = 14732,
  [SMALL_STATE(1140)] = 14791,
  [SMALL_STATE(1141)] = 14866,
  [SMALL_STATE(1142)] = 14939,
  [SMALL_STATE(1143)] = 15006,
  [SMALL_STATE(1144)] = 15071,
  [SMALL_STATE(1145)] = 15132,
  [SMALL_STATE(1146)] = 15209,
  [SMALL_STATE(1147)] = 15262,
  [SMALL_STATE(1148)] = 15315,
  [SMALL_STATE(1149)] = 15404,
  [SMALL_STATE(1150)] = 15461,
  [SMALL_STATE(1151)] = 15514,
  [SMALL_STATE(1152)] = 15569,
  [SMALL_STATE(1153)] = 15624,
  [SMALL_STATE(1154)] = 15681,
  [SMALL_STATE(1155)] = 15770,
  [SMALL_STATE(1156)] = 15859,
  [SMALL_STATE(1157)] = 15916,
  [SMALL_STATE(1158)] = 15973,
  [SMALL_STATE(1159)] = 16030,
  [SMALL_STATE(1160)] = 16085,
  [SMALL_STATE(1161)] = 16140,
  [SMALL_STATE(1162)] = 16195,
  [SMALL_STATE(1163)] = 16250,
  [SMALL_STATE(1164)] = 16322,
  [SMALL_STATE(1165)] = 16376,
  [SMALL_STATE(1166)] = 16452,
  [SMALL_STATE(1167)] = 16526,
  [SMALL_STATE(1168)] = 16588,
  [SMALL_STATE(1169)] = 16650,
  [SMALL_STATE(1170)] = 16708,
  [SMALL_STATE(1171)] = 16766,
  [SMALL_STATE(1172)] = 16840,
  [SMALL_STATE(1173)] = 16906,
  [SMALL_STATE(1174)] = 16960,
  [SMALL_STATE(1175)] = 17016,
  [SMALL_STATE(1176)] = 17102,
  [SMALL_STATE(1177)] = 17188,
  [SMALL_STATE(1178)] = 17244,
  [SMALL_STATE(1179)] = 17298,
  [SMALL_STATE(1180)] = 17354,
  [SMALL_STATE(1181)] = 17406,
  [SMALL_STATE(1182)] = 17458,
  [SMALL_STATE(1183)] = 17524,
  [SMALL_STATE(1184)] = 17578,
  [SMALL_STATE(1185)] = 17630,
  [SMALL_STATE(1186)] = 17684,
  [SMALL_STATE(1187)] = 17748,
  [SMALL_STATE(1188)] = 17834,
  [SMALL_STATE(1189)] = 17910,
  [SMALL_STATE(1190)] = 17980,
  [SMALL_STATE(1191)] = 18052,
  [SMALL_STATE(1192)] = 18116,
  [SMALL_STATE(1193)] = 18176,
  [SMALL_STATE(1194)] = 18262,
  [SMALL_STATE(1195)] = 18314,
  [SMALL_STATE(1196)] = 18376,
  [SMALL_STATE(1197)] = 18430,
  [SMALL_STATE(1198)] = 18484,
  [SMALL_STATE(1199)] = 18544,
  [SMALL_STATE(1200)] = 18600,
  [SMALL_STATE(1201)] = 18654,
  [SMALL_STATE(1202)] = 18708,
  [SMALL_STATE(1203)] = 18778,
  [SMALL_STATE(1204)] = 18831,
  [SMALL_STATE(1205)] = 18914,
  [SMALL_STATE(1206)] = 18975,
  [SMALL_STATE(1207)] = 19032,
  [SMALL_STATE(1208)] = 19105,
  [SMALL_STATE(1209)] = 19176,
  [SMALL_STATE(1210)] = 19245,
  [SMALL_STATE(1211)] = 19310,
  [SMALL_STATE(1212)] = 19373,
  [SMALL_STATE(1213)] = 19432,
  [SMALL_STATE(1214)] = 19487,
  [SMALL_STATE(1215)] = 19540,
  [SMALL_STATE(1216)] = 19623,
  [SMALL_STATE(1217)] = 19698,
  [SMALL_STATE(1218)] = 19781,
  [SMALL_STATE(1219)] = 19864,
  [SMALL_STATE(1220)] = 19947,
  [SMALL_STATE(1221)] = 20030,
  [SMALL_STATE(1222)] = 20081,
  [SMALL_STATE(1223)] = 20134,
  [SMALL_STATE(1224)] = 20187,
  [SMALL_STATE(1225)] = 20240,
  [SMALL_STATE(1226)] = 20293,
  [SMALL_STATE(1227)] = 20376,
  [SMALL_STATE(1228)] = 20429,
  [SMALL_STATE(1229)] = 20512,
  [SMALL_STATE(1230)] = 20606,
  [SMALL_STATE(1231)] = 20698,
  [SMALL_STATE(1232)] = 20792,
  [SMALL_STATE(1233)] = 20872,
  [SMALL_STATE(1234)] = 20952,
  [SMALL_STATE(1235)] = 21032,
  [SMALL_STATE(1236)] = 21084,
  [SMALL_STATE(1237)] = 21164,
  [SMALL_STATE(1238)] = 21216,
  [SMALL_STATE(1239)] = 21296,
  [SMALL_STATE(1240)] = 21348,
  [SMALL_STATE(1241)] = 21428,
  [SMALL_STATE(1242)] = 21481,
  [SMALL_STATE(1243)] = 21532,
  [SMALL_STATE(1244)] = 21585,
  [SMALL_STATE(1245)] = 21631,
  [SMALL_STATE(1246)] = 21677,
  [SMALL_STATE(1247)] = 21723,
  [SMALL_STATE(1248)] = 21769,
  [SMALL_STATE(1249)] = 21815,
  [SMALL_STATE(1250)] = 21861,
  [SMALL_STATE(1251)] = 21935,
  [SMALL_STATE(1252)] = 21981,
  [SMALL_STATE(1253)] = 22027,
  [SMALL_STATE(1254)] = 22074,
  [SMALL_STATE(1255)] = 22121,
  [SMALL_STATE(1256)] = 22182,
  [SMALL_STATE(1257)] = 22239,
  [SMALL_STATE(1258)] = 22298,
  [SMALL_STATE(1259)] = 22351,
  [SMALL_STATE(1260)] = 22420,
  [SMALL_STATE(1261)] = 22487,
  [SMALL_STATE(1262)] = 22544,
  [SMALL_STATE(1263)] = 22597,
  [SMALL_STATE(1264)] = 22666,
  [SMALL_STATE(1265)] = 22733,
  [SMALL_STATE(1266)] = 22798,
  [SMALL_STATE(1267)] = 22859,
  [SMALL_STATE(1268)] = 22918,
  [SMALL_STATE(1269)] = 22973,
  [SMALL_STATE(1270)] = 23024,
  [SMALL_STATE(1271)] = 23073,
  [SMALL_STATE(1272)] = 23144,
  [SMALL_STATE(1273)] = 23193,
  [SMALL_STATE(1274)] = 23240,
  [SMALL_STATE(1275)] = 23287,
  [SMALL_STATE(1276)] = 23334,
  [SMALL_STATE(1277)] = 23405,
  [SMALL_STATE(1278)] = 23452,
  [SMALL_STATE(1279)] = 23507,
  [SMALL_STATE(1280)] = 23552,
  [SMALL_STATE(1281)] = 23597,
  [SMALL_STATE(1282)] = 23642,
  [SMALL_STATE(1283)] = 23707,
  [SMALL_STATE(1284)] = 23758,
  [SMALL_STATE(1285)] = 23824,
  [SMALL_STATE(1286)] = 23870,
  [SMALL_STATE(1287)] = 23926,
  [SMALL_STATE(1288)] = 23978,
  [SMALL_STATE(1289)] = 24046,
  [SMALL_STATE(1290)] = 24112,
  [SMALL_STATE(1291)] = 24176,
  [SMALL_STATE(1292)] = 24236,
  [SMALL_STATE(1293)] = 24294,
  [SMALL_STATE(1294)] = 24348,
  [SMALL_STATE(1295)] = 24398,
  [SMALL_STATE(1296)] = 24446,
  [SMALL_STATE(1297)] = 24516,
  [SMALL_STATE(1298)] = 24572,
  [SMALL_STATE(1299)] = 24624,
  [SMALL_STATE(1300)] = 24692,
  [SMALL_STATE(1301)] = 24738,
  [SMALL_STATE(1302)] = 24802,
  [SMALL_STATE(1303)] = 24862,
  [SMALL_STATE(1304)] = 24920,
  [SMALL_STATE(1305)] = 24974,
  [SMALL_STATE(1306)] = 25024,
  [SMALL_STATE(1307)] = 25072,
  [SMALL_STATE(1308)] = 25142,
  [SMALL_STATE(1309)] = 25188,
  [SMALL_STATE(1310)] = 25244,
  [SMALL_STATE(1311)] = 25296,
  [SMALL_STATE(1312)] = 25364,
  [SMALL_STATE(1313)] = 25430,
  [SMALL_STATE(1314)] = 25494,
  [SMALL_STATE(1315)] = 25554,
  [SMALL_STATE(1316)] = 25612,
  [SMALL_STATE(1317)] = 25666,
  [SMALL_STATE(1318)] = 25716,
  [SMALL_STATE(1319)] = 25764,
  [SMALL_STATE(1320)] = 25810,
  [SMALL_STATE(1321)] = 25856,
  [SMALL_STATE(1322)] = 25926,
  [SMALL_STATE(1323)] = 25982,
  [SMALL_STATE(1324)] = 26034,
  [SMALL_STATE(1325)] = 26102,
  [SMALL_STATE(1326)] = 26168,
  [SMALL_STATE(1327)] = 26232,
  [SMALL_STATE(1328)] = 26292,
  [SMALL_STATE(1329)] = 26350,
  [SMALL_STATE(1330)] = 26404,
  [SMALL_STATE(1331)] = 26454,
  [SMALL_STATE(1332)] = 26502,
  [SMALL_STATE(1333)] = 26572,
  [SMALL_STATE(1334)] = 26618,
  [SMALL_STATE(1335)] = 26664,
  [SMALL_STATE(1336)] = 26710,
  [SMALL_STATE(1337)] = 26761,
  [SMALL_STATE(1338)] = 26816,
  [SMALL_STATE(1339)] = 26867,
  [SMALL_STATE(1340)] = 26934,
  [SMALL_STATE(1341)] = 26999,
  [SMALL_STATE(1342)] = 27062,
  [SMALL_STATE(1343)] = 27121,
  [SMALL_STATE(1344)] = 27178,
  [SMALL_STATE(1345)] = 27231,
  [SMALL_STATE(1346)] = 27286,
  [SMALL_STATE(1347)] = 27353,
  [SMALL_STATE(1348)] = 27418,
  [SMALL_STATE(1349)] = 27481,
  [SMALL_STATE(1350)] = 27540,
  [SMALL_STATE(1351)] = 27597,
  [SMALL_STATE(1352)] = 27650,
  [SMALL_STATE(1353)] = 27719,
  [SMALL_STATE(1354)] = 27766,
  [SMALL_STATE(1355)] = 27815,
  [SMALL_STATE(1356)] = 27862,
  [SMALL_STATE(1357)] = 27931,
  [SMALL_STATE(1358)] = 27986,
  [SMALL_STATE(1359)] = 28037,
  [SMALL_STATE(1360)] = 28104,
  [SMALL_STATE(1361)] = 28169,
  [SMALL_STATE(1362)] = 28232,
  [SMALL_STATE(1363)] = 28291,
  [SMALL_STATE(1364)] = 28348,
  [SMALL_STATE(1365)] = 28401,
  [SMALL_STATE(1366)] = 28446,
  [SMALL_STATE(1367)] = 28495,
  [SMALL_STATE(1368)] = 28542,
  [SMALL_STATE(1369)] = 28587,
  [SMALL_STATE(1370)] = 28656,
  [SMALL_STATE(1371)] = 28701,
  [SMALL_STATE(1372)] = 28746,
  [SMALL_STATE(1373)] = 28791,
  [SMALL_STATE(1374)] = 28840,
  [SMALL_STATE(1375)] = 28904,
  [SMALL_STATE(1376)] = 28966,
  [SMALL_STATE(1377)] = 29012,
  [SMALL_STATE(1378)] = 29074,
  [SMALL_STATE(1379)] = 29132,
  [SMALL_STATE(1380)] = 29188,
  [SMALL_STATE(1381)] = 29264,
  [SMALL_STATE(1382)] = 29316,
  [SMALL_STATE(1383)] = 29364,
  [SMALL_STATE(1384)] = 29410,
  [SMALL_STATE(1385)] = 29464,
  [SMALL_STATE(1386)] = 29514,
  [SMALL_STATE(1387)] = 29580,
  [SMALL_STATE(1388)] = 29644,
  [SMALL_STATE(1389)] = 29706,
  [SMALL_STATE(1390)] = 29764,
  [SMALL_STATE(1391)] = 29820,
  [SMALL_STATE(1392)] = 29872,
  [SMALL_STATE(1393)] = 29916,
  [SMALL_STATE(1394)] = 29984,
  [SMALL_STATE(1395)] = 30032,
  [SMALL_STATE(1396)] = 30078,
  [SMALL_STATE(1397)] = 30122,
  [SMALL_STATE(1398)] = 30190,
  [SMALL_STATE(1399)] = 30234,
  [SMALL_STATE(1400)] = 30296,
  [SMALL_STATE(1401)] = 30340,
  [SMALL_STATE(1402)] = 30402,
  [SMALL_STATE(1403)] = 30452,
  [SMALL_STATE(1404)] = 30518,
  [SMALL_STATE(1405)] = 30572,
  [SMALL_STATE(1406)] = 30639,
  [SMALL_STATE(1407)] = 30688,
  [SMALL_STATE(1408)] = 30753,
  [SMALL_STATE(1409)] = 30804,
  [SMALL_STATE(1410)] = 30847,
  [SMALL_STATE(1411)] = 30892,
  [SMALL_STATE(1412)] = 30939,
  [SMALL_STATE(1413)] = 31002,
  [SMALL_STATE(1414)] = 31061,
  [SMALL_STATE(1415)] = 31122,
  [SMALL_STATE(1416)] = 31181,
  [SMALL_STATE(1417)] = 31226,
  [SMALL_STATE(1418)] = 31285,
  [SMALL_STATE(1419)] = 31330,
  [SMALL_STATE(1420)] = 31383,
  [SMALL_STATE(1421)] = 31440,
  [SMALL_STATE(1422)] = 31495,
  [SMALL_STATE(1423)] = 31554,
  [SMALL_STATE(1424)] = 31613,
  [SMALL_STATE(1425)] = 31672,
  [SMALL_STATE(1426)] = 31728,
  [SMALL_STATE(1427)] = 31768,
  [SMALL_STATE(1428)] = 31824,
  [SMALL_STATE(1429)] = 31864,
  [SMALL_STATE(1430)] = 31906,
  [SMALL_STATE(1431)] = 31962,
  [SMALL_STATE(1432)] = 32001,
  [SMALL_STATE(1433)] = 32054,
  [SMALL_STATE(1434)] = 32107,
  [SMALL_STATE(1435)] = 32150,
  [SMALL_STATE(1436)] = 32187,
  [SMALL_STATE(1437)] = 32224,
  [SMALL_STATE(1438)] = 32299,
  [SMALL_STATE(1439)] = 32336,
  [SMALL_STATE(1440)] = 32373,
  [SMALL_STATE(1441)] = 32410,
  [SMALL_STATE(1442)] = 32485,
  [SMALL_STATE(1443)] = 32522,
  [SMALL_STATE(1444)] = 32559,
  [SMALL_STATE(1445)] = 32596,
  [SMALL_STATE(1446)] = 32632,
  [SMALL_STATE(1447)] = 32668,
  [SMALL_STATE(1448)] = 32704,
  [SMALL_STATE(1449)] = 32740,
  [SMALL_STATE(1450)] = 32776,
  [SMALL_STATE(1451)] = 32812,
  [SMALL_STATE(1452)] = 32848,
  [SMALL_STATE(1453)] = 32884,
  [SMALL_STATE(1454)] = 32920,
  [SMALL_STATE(1455)] = 32956,
  [SMALL_STATE(1456)] = 32992,
  [SMALL_STATE(1457)] = 33028,
  [SMALL_STATE(1458)] = 33064,
  [SMALL_STATE(1459)] = 33100,
  [SMALL_STATE(1460)] = 33136,
  [SMALL_STATE(1461)] = 33172,
  [SMALL_STATE(1462)] = 33208,
  [SMALL_STATE(1463)] = 33244,
  [SMALL_STATE(1464)] = 33280,
  [SMALL_STATE(1465)] = 33316,
  [SMALL_STATE(1466)] = 33352,
  [SMALL_STATE(1467)] = 33388,
  [SMALL_STATE(1468)] = 33424,
  [SMALL_STATE(1469)] = 33460,
  [SMALL_STATE(1470)] = 33496,
  [SMALL_STATE(1471)] = 33532,
  [SMALL_STATE(1472)] = 33568,
  [SMALL_STATE(1473)] = 33604,
  [SMALL_STATE(1474)] = 33640,
  [SMALL_STATE(1475)] = 33676,
  [SMALL_STATE(1476)] = 33712,
  [SMALL_STATE(1477)] = 33748,
  [SMALL_STATE(1478)] = 33784,
  [SMALL_STATE(1479)] = 33820,
  [SMALL_STATE(1480)] = 33856,
  [SMALL_STATE(1481)] = 33892,
  [SMALL_STATE(1482)] = 33928,
  [SMALL_STATE(1483)] = 33964,
  [SMALL_STATE(1484)] = 34000,
  [SMALL_STATE(1485)] = 34036,
  [SMALL_STATE(1486)] = 34072,
  [SMALL_STATE(1487)] = 34108,
  [SMALL_STATE(1488)] = 34144,
  [SMALL_STATE(1489)] = 34180,
  [SMALL_STATE(1490)] = 34216,
  [SMALL_STATE(1491)] = 34252,
  [SMALL_STATE(1492)] = 34288,
  [SMALL_STATE(1493)] = 34324,
  [SMALL_STATE(1494)] = 34360,
  [SMALL_STATE(1495)] = 34396,
  [SMALL_STATE(1496)] = 34432,
  [SMALL_STATE(1497)] = 34468,
  [SMALL_STATE(1498)] = 34504,
  [SMALL_STATE(1499)] = 34540,
  [SMALL_STATE(1500)] = 34576,
  [SMALL_STATE(1501)] = 34612,
  [SMALL_STATE(1502)] = 34648,
  [SMALL_STATE(1503)] = 34684,
  [SMALL_STATE(1504)] = 34720,
  [SMALL_STATE(1505)] = 34756,
  [SMALL_STATE(1506)] = 34792,
  [SMALL_STATE(1507)] = 34828,
  [SMALL_STATE(1508)] = 34864,
  [SMALL_STATE(1509)] = 34900,
  [SMALL_STATE(1510)] = 34936,
  [SMALL_STATE(1511)] = 34972,
  [SMALL_STATE(1512)] = 35008,
  [SMALL_STATE(1513)] = 35044,
  [SMALL_STATE(1514)] = 35080,
  [SMALL_STATE(1515)] = 35116,
  [SMALL_STATE(1516)] = 35152,
  [SMALL_STATE(1517)] = 35188,
  [SMALL_STATE(1518)] = 35248,
  [SMALL_STATE(1519)] = 35308,
  [SMALL_STATE(1520)] = 35341,
  [SMALL_STATE(1521)] = 35374,
  [SMALL_STATE(1522)] = 35413,
  [SMALL_STATE(1523)] = 35446,
  [SMALL_STATE(1524)] = 35509,
  [SMALL_STATE(1525)] = 35542,
  [SMALL_STATE(1526)] = 35575,
  [SMALL_STATE(1527)] = 35608,
  [SMALL_STATE(1528)] = 35641,
  [SMALL_STATE(1529)] = 35674,
  [SMALL_STATE(1530)] = 35709,
  [SMALL_STATE(1531)] = 35742,
  [SMALL_STATE(1532)] = 35777,
  [SMALL_STATE(1533)] = 35840,
  [SMALL_STATE(1534)] = 35874,
  [SMALL_STATE(1535)] = 35912,
  [SMALL_STATE(1536)] = 35944,
  [SMALL_STATE(1537)] = 35978,
  [SMALL_STATE(1538)] = 36012,
  [SMALL_STATE(1539)] = 36042,
  [SMALL_STATE(1540)] = 36076,
  [SMALL_STATE(1541)] = 36105,
  [SMALL_STATE(1542)] = 36134,
  [SMALL_STATE(1543)] = 36169,
  [SMALL_STATE(1544)] = 36204,
  [SMALL_STATE(1545)] = 36239,
  [SMALL_STATE(1546)] = 36268,
  [SMALL_STATE(1547)] = 36297,
  [SMALL_STATE(1548)] = 36326,
  [SMALL_STATE(1549)] = 36355,
  [SMALL_STATE(1550)] = 36384,
  [SMALL_STATE(1551)] = 36413,
  [SMALL_STATE(1552)] = 36442,
  [SMALL_STATE(1553)] = 36471,
  [SMALL_STATE(1554)] = 36500,
  [SMALL_STATE(1555)] = 36537,
  [SMALL_STATE(1556)] = 36565,
  [SMALL_STATE(1557)] = 36593,
  [SMALL_STATE(1558)] = 36621,
  [SMALL_STATE(1559)] = 36649,
  [SMALL_STATE(1560)] = 36677,
  [SMALL_STATE(1561)] = 36705,
  [SMALL_STATE(1562)] = 36733,
  [SMALL_STATE(1563)] = 36761,
  [SMALL_STATE(1564)] = 36789,
  [SMALL_STATE(1565)] = 36821,
  [SMALL_STATE(1566)] = 36849,
  [SMALL_STATE(1567)] = 36877,
  [SMALL_STATE(1568)] = 36905,
  [SMALL_STATE(1569)] = 36932,
  [SMALL_STATE(1570)] = 36959,
  [SMALL_STATE(1571)] = 36986,
  [SMALL_STATE(1572)] = 37013,
  [SMALL_STATE(1573)] = 37040,
  [SMALL_STATE(1574)] = 37067,
  [SMALL_STATE(1575)] = 37094,
  [SMALL_STATE(1576)] = 37121,
  [SMALL_STATE(1577)] = 37148,
  [SMALL_STATE(1578)] = 37175,
  [SMALL_STATE(1579)] = 37202,
  [SMALL_STATE(1580)] = 37243,
  [SMALL_STATE(1581)] = 37270,
  [SMALL_STATE(1582)] = 37297,
  [SMALL_STATE(1583)] = 37324,
  [SMALL_STATE(1584)] = 37351,
  [SMALL_STATE(1585)] = 37378,
  [SMALL_STATE(1586)] = 37405,
  [SMALL_STATE(1587)] = 37432,
  [SMALL_STATE(1588)] = 37459,
  [SMALL_STATE(1589)] = 37510,
  [SMALL_STATE(1590)] = 37537,
  [SMALL_STATE(1591)] = 37573,
  [SMALL_STATE(1592)] = 37617,
  [SMALL_STATE(1593)] = 37663,
  [SMALL_STATE(1594)] = 37695,
  [SMALL_STATE(1595)] = 37733,
  [SMALL_STATE(1596)] = 37764,
  [SMALL_STATE(1597)] = 37801,
  [SMALL_STATE(1598)] = 37846,
  [SMALL_STATE(1599)] = 37878,
  [SMALL_STATE(1600)] = 37905,
  [SMALL_STATE(1601)] = 37946,
  [SMALL_STATE(1602)] = 37985,
  [SMALL_STATE(1603)] = 38024,
  [SMALL_STATE(1604)] = 38049,
  [SMALL_STATE(1605)] = 38074,
  [SMALL_STATE(1606)] = 38113,
  [SMALL_STATE(1607)] = 38138,
  [SMALL_STATE(1608)] = 38177,
  [SMALL_STATE(1609)] = 38212,
  [SMALL_STATE(1610)] = 38247,
  [SMALL_STATE(1611)] = 38276,
  [SMALL_STATE(1612)] = 38301,
  [SMALL_STATE(1613)] = 38342,
  [SMALL_STATE(1614)] = 38377,
  [SMALL_STATE(1615)] = 38412,
  [SMALL_STATE(1616)] = 38448,
  [SMALL_STATE(1617)] = 38472,
  [SMALL_STATE(1618)] = 38496,
  [SMALL_STATE(1619)] = 38534,
  [SMALL_STATE(1620)] = 38572,
  [SMALL_STATE(1621)] = 38596,
  [SMALL_STATE(1622)] = 38632,
  [SMALL_STATE(1623)] = 38664,
  [SMALL_STATE(1624)] = 38688,
  [SMALL_STATE(1625)] = 38712,
  [SMALL_STATE(1626)] = 38736,
  [SMALL_STATE(1627)] = 38762,
  [SMALL_STATE(1628)] = 38786,
  [SMALL_STATE(1629)] = 38810,
  [SMALL_STATE(1630)] = 38846,
  [SMALL_STATE(1631)] = 38874,
  [SMALL_STATE(1632)] = 38898,
  [SMALL_STATE(1633)] = 38930,
  [SMALL_STATE(1634)] = 38966,
  [SMALL_STATE(1635)] = 38990,
  [SMALL_STATE(1636)] = 39026,
  [SMALL_STATE(1637)] = 39062,
  [SMALL_STATE(1638)] = 39098,
  [SMALL_STATE(1639)] = 39134,
  [SMALL_STATE(1640)] = 39166,
  [SMALL_STATE(1641)] = 39190,
  [SMALL_STATE(1642)] = 39214,
  [SMALL_STATE(1643)] = 39246,
  [SMALL_STATE(1644)] = 39270,
  [SMALL_STATE(1645)] = 39292,
  [SMALL_STATE(1646)] = 39328,
  [SMALL_STATE(1647)] = 39366,
  [SMALL_STATE(1648)] = 39390,
  [SMALL_STATE(1649)] = 39426,
  [SMALL_STATE(1650)] = 39450,
  [SMALL_STATE(1651)] = 39474,
  [SMALL_STATE(1652)] = 39503,
  [SMALL_STATE(1653)] = 39536,
  [SMALL_STATE(1654)] = 39571,
  [SMALL_STATE(1655)] = 39606,
  [SMALL_STATE(1656)] = 39641,
  [SMALL_STATE(1657)] = 39662,
  [SMALL_STATE(1658)] = 39685,
  [SMALL_STATE(1659)] = 39710,
  [SMALL_STATE(1660)] = 39731,
  [SMALL_STATE(1661)] = 39752,
  [SMALL_STATE(1662)] = 39787,
  [SMALL_STATE(1663)] = 39818,
  [SMALL_STATE(1664)] = 39839,
  [SMALL_STATE(1665)] = 39862,
  [SMALL_STATE(1666)] = 39897,
  [SMALL_STATE(1667)] = 39932,
  [SMALL_STATE(1668)] = 39953,
  [SMALL_STATE(1669)] = 39978,
  [SMALL_STATE(1670)] = 40011,
  [SMALL_STATE(1671)] = 40046,
  [SMALL_STATE(1672)] = 40067,
  [SMALL_STATE(1673)] = 40100,
  [SMALL_STATE(1674)] = 40135,
  [SMALL_STATE(1675)] = 40168,
  [SMALL_STATE(1676)] = 40201,
  [SMALL_STATE(1677)] = 40234,
  [SMALL_STATE(1678)] = 40267,
  [SMALL_STATE(1679)] = 40300,
  [SMALL_STATE(1680)] = 40333,
  [SMALL_STATE(1681)] = 40366,
  [SMALL_STATE(1682)] = 40399,
  [SMALL_STATE(1683)] = 40432,
  [SMALL_STATE(1684)] = 40465,
  [SMALL_STATE(1685)] = 40498,
  [SMALL_STATE(1686)] = 40531,
  [SMALL_STATE(1687)] = 40552,
  [SMALL_STATE(1688)] = 40573,
  [SMALL_STATE(1689)] = 40608,
  [SMALL_STATE(1690)] = 40641,
  [SMALL_STATE(1691)] = 40662,
  [SMALL_STATE(1692)] = 40697,
  [SMALL_STATE(1693)] = 40718,
  [SMALL_STATE(1694)] = 40753,
  [SMALL_STATE(1695)] = 40788,
  [SMALL_STATE(1696)] = 40820,
  [SMALL_STATE(1697)] = 40848,
  [SMALL_STATE(1698)] = 40880,
  [SMALL_STATE(1699)] = 40912,
  [SMALL_STATE(1700)] = 40944,
  [SMALL_STATE(1701)] = 40976,
  [SMALL_STATE(1702)] = 41000,
  [SMALL_STATE(1703)] = 41024,
  [SMALL_STATE(1704)] = 41044,
  [SMALL_STATE(1705)] = 41076,
  [SMALL_STATE(1706)] = 41096,
  [SMALL_STATE(1707)] = 41128,
  [SMALL_STATE(1708)] = 41160,
  [SMALL_STATE(1709)] = 41180,
  [SMALL_STATE(1710)] = 41212,
  [SMALL_STATE(1711)] = 41244,
  [SMALL_STATE(1712)] = 41268,
  [SMALL_STATE(1713)] = 41290,
  [SMALL_STATE(1714)] = 41310,
  [SMALL_STATE(1715)] = 41342,
  [SMALL_STATE(1716)] = 41374,
  [SMALL_STATE(1717)] = 41406,
  [SMALL_STATE(1718)] = 41426,
  [SMALL_STATE(1719)] = 41446,
  [SMALL_STATE(1720)] = 41478,
  [SMALL_STATE(1721)] = 41510,
  [SMALL_STATE(1722)] = 41542,
  [SMALL_STATE(1723)] = 41562,
  [SMALL_STATE(1724)] = 41594,
  [SMALL_STATE(1725)] = 41626,
  [SMALL_STATE(1726)] = 41658,
  [SMALL_STATE(1727)] = 41690,
  [SMALL_STATE(1728)] = 41722,
  [SMALL_STATE(1729)] = 41754,
  [SMALL_STATE(1730)] = 41786,
  [SMALL_STATE(1731)] = 41818,
  [SMALL_STATE(1732)] = 41850,
  [SMALL_STATE(1733)] = 41882,
  [SMALL_STATE(1734)] = 41914,
  [SMALL_STATE(1735)] = 41946,
  [SMALL_STATE(1736)] = 41966,
  [SMALL_STATE(1737)] = 41986,
  [SMALL_STATE(1738)] = 42018,
  [SMALL_STATE(1739)] = 42050,
  [SMALL_STATE(1740)] = 42082,
  [SMALL_STATE(1741)] = 42114,
  [SMALL_STATE(1742)] = 42146,
  [SMALL_STATE(1743)] = 42178,
  [SMALL_STATE(1744)] = 42210,
  [SMALL_STATE(1745)] = 42236,
  [SMALL_STATE(1746)] = 42268,
  [SMALL_STATE(1747)] = 42300,
  [SMALL_STATE(1748)] = 42332,
  [SMALL_STATE(1749)] = 42364,
  [SMALL_STATE(1750)] = 42396,
  [SMALL_STATE(1751)] = 42428,
  [SMALL_STATE(1752)] = 42458,
  [SMALL_STATE(1753)] = 42477,
  [SMALL_STATE(1754)] = 42496,
  [SMALL_STATE(1755)] = 42519,
  [SMALL_STATE(1756)] = 42548,
  [SMALL_STATE(1757)] = 42567,
  [SMALL_STATE(1758)] = 42586,
  [SMALL_STATE(1759)] = 42615,
  [SMALL_STATE(1760)] = 42638,
  [SMALL_STATE(1761)] = 42657,
  [SMALL_STATE(1762)] = 42684,
  [SMALL_STATE(1763)] = 42703,
  [SMALL_STATE(1764)] = 42724,
  [SMALL_STATE(1765)] = 42745,
  [SMALL_STATE(1766)] = 42764,
  [SMALL_STATE(1767)] = 42783,
  [SMALL_STATE(1768)] = 42802,
  [SMALL_STATE(1769)] = 42821,
  [SMALL_STATE(1770)] = 42840,
  [SMALL_STATE(1771)] = 42860,
  [SMALL_STATE(1772)] = 42886,
  [SMALL_STATE(1773)] = 42906,
  [SMALL_STATE(1774)] = 42928,
  [SMALL_STATE(1775)] = 42954,
  [SMALL_STATE(1776)] = 42974,
  [SMALL_STATE(1777)] = 42994,
  [SMALL_STATE(1778)] = 43020,
  [SMALL_STATE(1779)] = 43038,
  [SMALL_STATE(1780)] = 43058,
  [SMALL_STATE(1781)] = 43084,
  [SMALL_STATE(1782)] = 43104,
  [SMALL_STATE(1783)] = 43124,
  [SMALL_STATE(1784)] = 43144,
  [SMALL_STATE(1785)] = 43164,
  [SMALL_STATE(1786)] = 43186,
  [SMALL_STATE(1787)] = 43206,
  [SMALL_STATE(1788)] = 43226,
  [SMALL_STATE(1789)] = 43246,
  [SMALL_STATE(1790)] = 43272,
  [SMALL_STATE(1791)] = 43292,
  [SMALL_STATE(1792)] = 43312,
  [SMALL_STATE(1793)] = 43332,
  [SMALL_STATE(1794)] = 43351,
  [SMALL_STATE(1795)] = 43374,
  [SMALL_STATE(1796)] = 43397,
  [SMALL_STATE(1797)] = 43418,
  [SMALL_STATE(1798)] = 43441,
  [SMALL_STATE(1799)] = 43464,
  [SMALL_STATE(1800)] = 43483,
  [SMALL_STATE(1801)] = 43504,
  [SMALL_STATE(1802)] = 43527,
  [SMALL_STATE(1803)] = 43546,
  [SMALL_STATE(1804)] = 43567,
  [SMALL_STATE(1805)] = 43584,
  [SMALL_STATE(1806)] = 43607,
  [SMALL_STATE(1807)] = 43624,
  [SMALL_STATE(1808)] = 43645,
  [SMALL_STATE(1809)] = 43668,
  [SMALL_STATE(1810)] = 43685,
  [SMALL_STATE(1811)] = 43704,
  [SMALL_STATE(1812)] = 43723,
  [SMALL_STATE(1813)] = 43746,
  [SMALL_STATE(1814)] = 43769,
  [SMALL_STATE(1815)] = 43792,
  [SMALL_STATE(1816)] = 43809,
  [SMALL_STATE(1817)] = 43826,
  [SMALL_STATE(1818)] = 43843,
  [SMALL_STATE(1819)] = 43860,
  [SMALL_STATE(1820)] = 43881,
  [SMALL_STATE(1821)] = 43904,
  [SMALL_STATE(1822)] = 43921,
  [SMALL_STATE(1823)] = 43944,
  [SMALL_STATE(1824)] = 43967,
  [SMALL_STATE(1825)] = 43984,
  [SMALL_STATE(1826)] = 44003,
  [SMALL_STATE(1827)] = 44026,
  [SMALL_STATE(1828)] = 44049,
  [SMALL_STATE(1829)] = 44066,
  [SMALL_STATE(1830)] = 44083,
  [SMALL_STATE(1831)] = 44106,
  [SMALL_STATE(1832)] = 44123,
  [SMALL_STATE(1833)] = 44142,
  [SMALL_STATE(1834)] = 44165,
  [SMALL_STATE(1835)] = 44188,
  [SMALL_STATE(1836)] = 44211,
  [SMALL_STATE(1837)] = 44230,
  [SMALL_STATE(1838)] = 44253,
  [SMALL_STATE(1839)] = 44270,
  [SMALL_STATE(1840)] = 44287,
  [SMALL_STATE(1841)] = 44308,
  [SMALL_STATE(1842)] = 44329,
  [SMALL_STATE(1843)] = 44352,
  [SMALL_STATE(1844)] = 44369,
  [SMALL_STATE(1845)] = 44386,
  [SMALL_STATE(1846)] = 44405,
  [SMALL_STATE(1847)] = 44426,
  [SMALL_STATE(1848)] = 44449,
  [SMALL_STATE(1849)] = 44472,
  [SMALL_STATE(1850)] = 44489,
  [SMALL_STATE(1851)] = 44509,
  [SMALL_STATE(1852)] = 44525,
  [SMALL_STATE(1853)] = 44545,
  [SMALL_STATE(1854)] = 44565,
  [SMALL_STATE(1855)] = 44585,
  [SMALL_STATE(1856)] = 44605,
  [SMALL_STATE(1857)] = 44625,
  [SMALL_STATE(1858)] = 44645,
  [SMALL_STATE(1859)] = 44661,
  [SMALL_STATE(1860)] = 44677,
  [SMALL_STATE(1861)] = 44697,
  [SMALL_STATE(1862)] = 44717,
  [SMALL_STATE(1863)] = 44737,
  [SMALL_STATE(1864)] = 44755,
  [SMALL_STATE(1865)] = 44775,
  [SMALL_STATE(1866)] = 44795,
  [SMALL_STATE(1867)] = 44815,
  [SMALL_STATE(1868)] = 44835,
  [SMALL_STATE(1869)] = 44855,
  [SMALL_STATE(1870)] = 44873,
  [SMALL_STATE(1871)] = 44891,
  [SMALL_STATE(1872)] = 44911,
  [SMALL_STATE(1873)] = 44931,
  [SMALL_STATE(1874)] = 44947,
  [SMALL_STATE(1875)] = 44963,
  [SMALL_STATE(1876)] = 44983,
  [SMALL_STATE(1877)] = 45003,
  [SMALL_STATE(1878)] = 45023,
  [SMALL_STATE(1879)] = 45039,
  [SMALL_STATE(1880)] = 45055,
  [SMALL_STATE(1881)] = 45075,
  [SMALL_STATE(1882)] = 45091,
  [SMALL_STATE(1883)] = 45111,
  [SMALL_STATE(1884)] = 45131,
  [SMALL_STATE(1885)] = 45151,
  [SMALL_STATE(1886)] = 45171,
  [SMALL_STATE(1887)] = 45191,
  [SMALL_STATE(1888)] = 45211,
  [SMALL_STATE(1889)] = 45231,
  [SMALL_STATE(1890)] = 45247,
  [SMALL_STATE(1891)] = 45267,
  [SMALL_STATE(1892)] = 45287,
  [SMALL_STATE(1893)] = 45307,
  [SMALL_STATE(1894)] = 45327,
  [SMALL_STATE(1895)] = 45345,
  [SMALL_STATE(1896)] = 45365,
  [SMALL_STATE(1897)] = 45381,
  [SMALL_STATE(1898)] = 45401,
  [SMALL_STATE(1899)] = 45421,
  [SMALL_STATE(1900)] = 45441,
  [SMALL_STATE(1901)] = 45457,
  [SMALL_STATE(1902)] = 45477,
  [SMALL_STATE(1903)] = 45495,
  [SMALL_STATE(1904)] = 45515,
  [SMALL_STATE(1905)] = 45535,
  [SMALL_STATE(1906)] = 45555,
  [SMALL_STATE(1907)] = 45575,
  [SMALL_STATE(1908)] = 45595,
  [SMALL_STATE(1909)] = 45615,
  [SMALL_STATE(1910)] = 45635,
  [SMALL_STATE(1911)] = 45655,
  [SMALL_STATE(1912)] = 45675,
  [SMALL_STATE(1913)] = 45695,
  [SMALL_STATE(1914)] = 45715,
  [SMALL_STATE(1915)] = 45731,
  [SMALL_STATE(1916)] = 45751,
  [SMALL_STATE(1917)] = 45769,
  [SMALL_STATE(1918)] = 45789,
  [SMALL_STATE(1919)] = 45809,
  [SMALL_STATE(1920)] = 45829,
  [SMALL_STATE(1921)] = 45849,
  [SMALL_STATE(1922)] = 45869,
  [SMALL_STATE(1923)] = 45889,
  [SMALL_STATE(1924)] = 45909,
  [SMALL_STATE(1925)] = 45929,
  [SMALL_STATE(1926)] = 45945,
  [SMALL_STATE(1927)] = 45961,
  [SMALL_STATE(1928)] = 45977,
  [SMALL_STATE(1929)] = 45997,
  [SMALL_STATE(1930)] = 46017,
  [SMALL_STATE(1931)] = 46037,
  [SMALL_STATE(1932)] = 46057,
  [SMALL_STATE(1933)] = 46077,
  [SMALL_STATE(1934)] = 46093,
  [SMALL_STATE(1935)] = 46113,
  [SMALL_STATE(1936)] = 46133,
  [SMALL_STATE(1937)] = 46149,
  [SMALL_STATE(1938)] = 46165,
  [SMALL_STATE(1939)] = 46185,
  [SMALL_STATE(1940)] = 46201,
  [SMALL_STATE(1941)] = 46221,
  [SMALL_STATE(1942)] = 46241,
  [SMALL_STATE(1943)] = 46259,
  [SMALL_STATE(1944)] = 46279,
  [SMALL_STATE(1945)] = 46295,
  [SMALL_STATE(1946)] = 46311,
  [SMALL_STATE(1947)] = 46327,
  [SMALL_STATE(1948)] = 46347,
  [SMALL_STATE(1949)] = 46367,
  [SMALL_STATE(1950)] = 46385,
  [SMALL_STATE(1951)] = 46405,
  [SMALL_STATE(1952)] = 46421,
  [SMALL_STATE(1953)] = 46441,
  [SMALL_STATE(1954)] = 46457,
  [SMALL_STATE(1955)] = 46473,
  [SMALL_STATE(1956)] = 46491,
  [SMALL_STATE(1957)] = 46511,
  [SMALL_STATE(1958)] = 46531,
  [SMALL_STATE(1959)] = 46547,
  [SMALL_STATE(1960)] = 46567,
  [SMALL_STATE(1961)] = 46587,
  [SMALL_STATE(1962)] = 46605,
  [SMALL_STATE(1963)] = 46625,
  [SMALL_STATE(1964)] = 46643,
  [SMALL_STATE(1965)] = 46663,
  [SMALL_STATE(1966)] = 46681,
  [SMALL_STATE(1967)] = 46701,
  [SMALL_STATE(1968)] = 46719,
  [SMALL_STATE(1969)] = 46739,
  [SMALL_STATE(1970)] = 46759,
  [SMALL_STATE(1971)] = 46779,
  [SMALL_STATE(1972)] = 46795,
  [SMALL_STATE(1973)] = 46815,
  [SMALL_STATE(1974)] = 46833,
  [SMALL_STATE(1975)] = 46849,
  [SMALL_STATE(1976)] = 46869,
  [SMALL_STATE(1977)] = 46889,
  [SMALL_STATE(1978)] = 46905,
  [SMALL_STATE(1979)] = 46921,
  [SMALL_STATE(1980)] = 46937,
  [SMALL_STATE(1981)] = 46957,
  [SMALL_STATE(1982)] = 46973,
  [SMALL_STATE(1983)] = 46991,
  [SMALL_STATE(1984)] = 47011,
  [SMALL_STATE(1985)] = 47031,
  [SMALL_STATE(1986)] = 47051,
  [SMALL_STATE(1987)] = 47067,
  [SMALL_STATE(1988)] = 47087,
  [SMALL_STATE(1989)] = 47103,
  [SMALL_STATE(1990)] = 47119,
  [SMALL_STATE(1991)] = 47139,
  [SMALL_STATE(1992)] = 47159,
  [SMALL_STATE(1993)] = 47179,
  [SMALL_STATE(1994)] = 47199,
  [SMALL_STATE(1995)] = 47219,
  [SMALL_STATE(1996)] = 47239,
  [SMALL_STATE(1997)] = 47259,
  [SMALL_STATE(1998)] = 47279,
  [SMALL_STATE(1999)] = 47299,
  [SMALL_STATE(2000)] = 47319,
  [SMALL_STATE(2001)] = 47339,
  [SMALL_STATE(2002)] = 47355,
  [SMALL_STATE(2003)] = 47372,
  [SMALL_STATE(2004)] = 47389,
  [SMALL_STATE(2005)] = 47406,
  [SMALL_STATE(2006)] = 47423,
  [SMALL_STATE(2007)] = 47440,
  [SMALL_STATE(2008)] = 47457,
  [SMALL_STATE(2009)] = 47472,
  [SMALL_STATE(2010)] = 47489,
  [SMALL_STATE(2011)] = 47504,
  [SMALL_STATE(2012)] = 47521,
  [SMALL_STATE(2013)] = 47536,
  [SMALL_STATE(2014)] = 47551,
  [SMALL_STATE(2015)] = 47568,
  [SMALL_STATE(2016)] = 47585,
  [SMALL_STATE(2017)] = 47602,
  [SMALL_STATE(2018)] = 47619,
  [SMALL_STATE(2019)] = 47636,
  [SMALL_STATE(2020)] = 47651,
  [SMALL_STATE(2021)] = 47666,
  [SMALL_STATE(2022)] = 47681,
  [SMALL_STATE(2023)] = 47698,
  [SMALL_STATE(2024)] = 47713,
  [SMALL_STATE(2025)] = 47730,
  [SMALL_STATE(2026)] = 47747,
  [SMALL_STATE(2027)] = 47764,
  [SMALL_STATE(2028)] = 47781,
  [SMALL_STATE(2029)] = 47798,
  [SMALL_STATE(2030)] = 47815,
  [SMALL_STATE(2031)] = 47830,
  [SMALL_STATE(2032)] = 47847,
  [SMALL_STATE(2033)] = 47864,
  [SMALL_STATE(2034)] = 47879,
  [SMALL_STATE(2035)] = 47896,
  [SMALL_STATE(2036)] = 47911,
  [SMALL_STATE(2037)] = 47928,
  [SMALL_STATE(2038)] = 47945,
  [SMALL_STATE(2039)] = 47962,
  [SMALL_STATE(2040)] = 47979,
  [SMALL_STATE(2041)] = 47994,
  [SMALL_STATE(2042)] = 48011,
  [SMALL_STATE(2043)] = 48028,
  [SMALL_STATE(2044)] = 48045,
  [SMALL_STATE(2045)] = 48062,
  [SMALL_STATE(2046)] = 48079,
  [SMALL_STATE(2047)] = 48094,
  [SMALL_STATE(2048)] = 48111,
  [SMALL_STATE(2049)] = 48128,
  [SMALL_STATE(2050)] = 48145,
  [SMALL_STATE(2051)] = 48160,
  [SMALL_STATE(2052)] = 48177,
  [SMALL_STATE(2053)] = 48194,
  [SMALL_STATE(2054)] = 48211,
  [SMALL_STATE(2055)] = 48228,
  [SMALL_STATE(2056)] = 48243,
  [SMALL_STATE(2057)] = 48260,
  [SMALL_STATE(2058)] = 48277,
  [SMALL_STATE(2059)] = 48294,
  [SMALL_STATE(2060)] = 48311,
  [SMALL_STATE(2061)] = 48328,
  [SMALL_STATE(2062)] = 48345,
  [SMALL_STATE(2063)] = 48362,
  [SMALL_STATE(2064)] = 48379,
  [SMALL_STATE(2065)] = 48396,
  [SMALL_STATE(2066)] = 48413,
  [SMALL_STATE(2067)] = 48430,
  [SMALL_STATE(2068)] = 48447,
  [SMALL_STATE(2069)] = 48462,
  [SMALL_STATE(2070)] = 48479,
  [SMALL_STATE(2071)] = 48496,
  [SMALL_STATE(2072)] = 48511,
  [SMALL_STATE(2073)] = 48528,
  [SMALL_STATE(2074)] = 48545,
  [SMALL_STATE(2075)] = 48562,
  [SMALL_STATE(2076)] = 48579,
  [SMALL_STATE(2077)] = 48594,
  [SMALL_STATE(2078)] = 48611,
  [SMALL_STATE(2079)] = 48628,
  [SMALL_STATE(2080)] = 48645,
  [SMALL_STATE(2081)] = 48662,
  [SMALL_STATE(2082)] = 48679,
  [SMALL_STATE(2083)] = 48694,
  [SMALL_STATE(2084)] = 48711,
  [SMALL_STATE(2085)] = 48728,
  [SMALL_STATE(2086)] = 48745,
  [SMALL_STATE(2087)] = 48762,
  [SMALL_STATE(2088)] = 48779,
  [SMALL_STATE(2089)] = 48794,
  [SMALL_STATE(2090)] = 48811,
  [SMALL_STATE(2091)] = 48828,
  [SMALL_STATE(2092)] = 48845,
  [SMALL_STATE(2093)] = 48862,
  [SMALL_STATE(2094)] = 48879,
  [SMALL_STATE(2095)] = 48896,
  [SMALL_STATE(2096)] = 48913,
  [SMALL_STATE(2097)] = 48930,
  [SMALL_STATE(2098)] = 48947,
  [SMALL_STATE(2099)] = 48964,
  [SMALL_STATE(2100)] = 48981,
  [SMALL_STATE(2101)] = 48998,
  [SMALL_STATE(2102)] = 49015,
  [SMALL_STATE(2103)] = 49030,
  [SMALL_STATE(2104)] = 49047,
  [SMALL_STATE(2105)] = 49064,
  [SMALL_STATE(2106)] = 49081,
  [SMALL_STATE(2107)] = 49098,
  [SMALL_STATE(2108)] = 49115,
  [SMALL_STATE(2109)] = 49132,
  [SMALL_STATE(2110)] = 49147,
  [SMALL_STATE(2111)] = 49164,
  [SMALL_STATE(2112)] = 49181,
  [SMALL_STATE(2113)] = 49196,
  [SMALL_STATE(2114)] = 49213,
  [SMALL_STATE(2115)] = 49230,
  [SMALL_STATE(2116)] = 49247,
  [SMALL_STATE(2117)] = 49264,
  [SMALL_STATE(2118)] = 49281,
  [SMALL_STATE(2119)] = 49298,
  [SMALL_STATE(2120)] = 49315,
  [SMALL_STATE(2121)] = 49332,
  [SMALL_STATE(2122)] = 49349,
  [SMALL_STATE(2123)] = 49366,
  [SMALL_STATE(2124)] = 49383,
  [SMALL_STATE(2125)] = 49400,
  [SMALL_STATE(2126)] = 49417,
  [SMALL_STATE(2127)] = 49434,
  [SMALL_STATE(2128)] = 49449,
  [SMALL_STATE(2129)] = 49466,
  [SMALL_STATE(2130)] = 49483,
  [SMALL_STATE(2131)] = 49500,
  [SMALL_STATE(2132)] = 49517,
  [SMALL_STATE(2133)] = 49534,
  [SMALL_STATE(2134)] = 49549,
  [SMALL_STATE(2135)] = 49566,
  [SMALL_STATE(2136)] = 49581,
  [SMALL_STATE(2137)] = 49598,
  [SMALL_STATE(2138)] = 49615,
  [SMALL_STATE(2139)] = 49630,
  [SMALL_STATE(2140)] = 49647,
  [SMALL_STATE(2141)] = 49664,
  [SMALL_STATE(2142)] = 49679,
  [SMALL_STATE(2143)] = 49696,
  [SMALL_STATE(2144)] = 49713,
  [SMALL_STATE(2145)] = 49730,
  [SMALL_STATE(2146)] = 49747,
  [SMALL_STATE(2147)] = 49764,
  [SMALL_STATE(2148)] = 49779,
  [SMALL_STATE(2149)] = 49794,
  [SMALL_STATE(2150)] = 49811,
  [SMALL_STATE(2151)] = 49826,
  [SMALL_STATE(2152)] = 49843,
  [SMALL_STATE(2153)] = 49860,
  [SMALL_STATE(2154)] = 49875,
  [SMALL_STATE(2155)] = 49892,
  [SMALL_STATE(2156)] = 49909,
  [SMALL_STATE(2157)] = 49924,
  [SMALL_STATE(2158)] = 49941,
  [SMALL_STATE(2159)] = 49958,
  [SMALL_STATE(2160)] = 49975,
  [SMALL_STATE(2161)] = 49992,
  [SMALL_STATE(2162)] = 50009,
  [SMALL_STATE(2163)] = 50026,
  [SMALL_STATE(2164)] = 50043,
  [SMALL_STATE(2165)] = 50060,
  [SMALL_STATE(2166)] = 50077,
  [SMALL_STATE(2167)] = 50094,
  [SMALL_STATE(2168)] = 50111,
  [SMALL_STATE(2169)] = 50128,
  [SMALL_STATE(2170)] = 50145,
  [SMALL_STATE(2171)] = 50162,
  [SMALL_STATE(2172)] = 50176,
  [SMALL_STATE(2173)] = 50190,
  [SMALL_STATE(2174)] = 50204,
  [SMALL_STATE(2175)] = 50218,
  [SMALL_STATE(2176)] = 50232,
  [SMALL_STATE(2177)] = 50246,
  [SMALL_STATE(2178)] = 50260,
  [SMALL_STATE(2179)] = 50274,
  [SMALL_STATE(2180)] = 50288,
  [SMALL_STATE(2181)] = 50302,
  [SMALL_STATE(2182)] = 50316,
  [SMALL_STATE(2183)] = 50330,
  [SMALL_STATE(2184)] = 50344,
  [SMALL_STATE(2185)] = 50358,
  [SMALL_STATE(2186)] = 50372,
  [SMALL_STATE(2187)] = 50386,
  [SMALL_STATE(2188)] = 50400,
  [SMALL_STATE(2189)] = 50414,
  [SMALL_STATE(2190)] = 50428,
  [SMALL_STATE(2191)] = 50442,
  [SMALL_STATE(2192)] = 50456,
  [SMALL_STATE(2193)] = 50470,
  [SMALL_STATE(2194)] = 50484,
  [SMALL_STATE(2195)] = 50498,
  [SMALL_STATE(2196)] = 50512,
  [SMALL_STATE(2197)] = 50526,
  [SMALL_STATE(2198)] = 50540,
  [SMALL_STATE(2199)] = 50554,
  [SMALL_STATE(2200)] = 50568,
  [SMALL_STATE(2201)] = 50582,
  [SMALL_STATE(2202)] = 50596,
  [SMALL_STATE(2203)] = 50610,
  [SMALL_STATE(2204)] = 50624,
  [SMALL_STATE(2205)] = 50638,
  [SMALL_STATE(2206)] = 50652,
  [SMALL_STATE(2207)] = 50666,
  [SMALL_STATE(2208)] = 50680,
  [SMALL_STATE(2209)] = 50694,
  [SMALL_STATE(2210)] = 50708,
  [SMALL_STATE(2211)] = 50722,
  [SMALL_STATE(2212)] = 50736,
  [SMALL_STATE(2213)] = 50750,
  [SMALL_STATE(2214)] = 50764,
  [SMALL_STATE(2215)] = 50778,
  [SMALL_STATE(2216)] = 50792,
  [SMALL_STATE(2217)] = 50806,
  [SMALL_STATE(2218)] = 50820,
  [SMALL_STATE(2219)] = 50834,
  [SMALL_STATE(2220)] = 50848,
  [SMALL_STATE(2221)] = 50862,
  [SMALL_STATE(2222)] = 50876,
  [SMALL_STATE(2223)] = 50890,
  [SMALL_STATE(2224)] = 50904,
  [SMALL_STATE(2225)] = 50918,
  [SMALL_STATE(2226)] = 50932,
  [SMALL_STATE(2227)] = 50946,
  [SMALL_STATE(2228)] = 50960,
  [SMALL_STATE(2229)] = 50974,
  [SMALL_STATE(2230)] = 50988,
  [SMALL_STATE(2231)] = 51002,
  [SMALL_STATE(2232)] = 51016,
  [SMALL_STATE(2233)] = 51030,
  [SMALL_STATE(2234)] = 51044,
  [SMALL_STATE(2235)] = 51058,
  [SMALL_STATE(2236)] = 51072,
  [SMALL_STATE(2237)] = 51086,
  [SMALL_STATE(2238)] = 51100,
  [SMALL_STATE(2239)] = 51114,
  [SMALL_STATE(2240)] = 51128,
  [SMALL_STATE(2241)] = 51142,
  [SMALL_STATE(2242)] = 51156,
  [SMALL_STATE(2243)] = 51170,
  [SMALL_STATE(2244)] = 51184,
  [SMALL_STATE(2245)] = 51198,
  [SMALL_STATE(2246)] = 51212,
  [SMALL_STATE(2247)] = 51226,
  [SMALL_STATE(2248)] = 51240,
  [SMALL_STATE(2249)] = 51254,
  [SMALL_STATE(2250)] = 51268,
  [SMALL_STATE(2251)] = 51282,
  [SMALL_STATE(2252)] = 51296,
  [SMALL_STATE(2253)] = 51310,
  [SMALL_STATE(2254)] = 51324,
  [SMALL_STATE(2255)] = 51338,
  [SMALL_STATE(2256)] = 51352,
  [SMALL_STATE(2257)] = 51366,
  [SMALL_STATE(2258)] = 51380,
  [SMALL_STATE(2259)] = 51394,
  [SMALL_STATE(2260)] = 51408,
  [SMALL_STATE(2261)] = 51422,
  [SMALL_STATE(2262)] = 51436,
  [SMALL_STATE(2263)] = 51450,
  [SMALL_STATE(2264)] = 51464,
  [SMALL_STATE(2265)] = 51478,
  [SMALL_STATE(2266)] = 51492,
  [SMALL_STATE(2267)] = 51506,
  [SMALL_STATE(2268)] = 51520,
  [SMALL_STATE(2269)] = 51534,
  [SMALL_STATE(2270)] = 51548,
  [SMALL_STATE(2271)] = 51562,
  [SMALL_STATE(2272)] = 51576,
  [SMALL_STATE(2273)] = 51590,
  [SMALL_STATE(2274)] = 51604,
  [SMALL_STATE(2275)] = 51618,
  [SMALL_STATE(2276)] = 51632,
  [SMALL_STATE(2277)] = 51646,
  [SMALL_STATE(2278)] = 51660,
  [SMALL_STATE(2279)] = 51674,
  [SMALL_STATE(2280)] = 51688,
  [SMALL_STATE(2281)] = 51702,
  [SMALL_STATE(2282)] = 51716,
  [SMALL_STATE(2283)] = 51730,
  [SMALL_STATE(2284)] = 51744,
  [SMALL_STATE(2285)] = 51758,
  [SMALL_STATE(2286)] = 51772,
  [SMALL_STATE(2287)] = 51786,
  [SMALL_STATE(2288)] = 51800,
  [SMALL_STATE(2289)] = 51814,
  [SMALL_STATE(2290)] = 51828,
  [SMALL_STATE(2291)] = 51842,
  [SMALL_STATE(2292)] = 51856,
  [SMALL_STATE(2293)] = 51870,
  [SMALL_STATE(2294)] = 51884,
  [SMALL_STATE(2295)] = 51898,
  [SMALL_STATE(2296)] = 51912,
  [SMALL_STATE(2297)] = 51926,
  [SMALL_STATE(2298)] = 51940,
  [SMALL_STATE(2299)] = 51954,
  [SMALL_STATE(2300)] = 51968,
  [SMALL_STATE(2301)] = 51982,
  [SMALL_STATE(2302)] = 51996,
  [SMALL_STATE(2303)] = 52010,
  [SMALL_STATE(2304)] = 52024,
  [SMALL_STATE(2305)] = 52038,
  [SMALL_STATE(2306)] = 52052,
  [SMALL_STATE(2307)] = 52066,
  [SMALL_STATE(2308)] = 52080,
  [SMALL_STATE(2309)] = 52094,
  [SMALL_STATE(2310)] = 52108,
  [SMALL_STATE(2311)] = 52122,
  [SMALL_STATE(2312)] = 52136,
  [SMALL_STATE(2313)] = 52150,
  [SMALL_STATE(2314)] = 52164,
  [SMALL_STATE(2315)] = 52178,
  [SMALL_STATE(2316)] = 52192,
  [SMALL_STATE(2317)] = 52206,
  [SMALL_STATE(2318)] = 52220,
  [SMALL_STATE(2319)] = 52234,
  [SMALL_STATE(2320)] = 52248,
  [SMALL_STATE(2321)] = 52262,
  [SMALL_STATE(2322)] = 52276,
  [SMALL_STATE(2323)] = 52290,
  [SMALL_STATE(2324)] = 52304,
  [SMALL_STATE(2325)] = 52318,
  [SMALL_STATE(2326)] = 52332,
  [SMALL_STATE(2327)] = 52346,
  [SMALL_STATE(2328)] = 52360,
  [SMALL_STATE(2329)] = 52374,
  [SMALL_STATE(2330)] = 52388,
  [SMALL_STATE(2331)] = 52402,
  [SMALL_STATE(2332)] = 52416,
  [SMALL_STATE(2333)] = 52430,
  [SMALL_STATE(2334)] = 52444,
  [SMALL_STATE(2335)] = 52458,
  [SMALL_STATE(2336)] = 52472,
  [SMALL_STATE(2337)] = 52486,
  [SMALL_STATE(2338)] = 52500,
  [SMALL_STATE(2339)] = 52514,
  [SMALL_STATE(2340)] = 52528,
  [SMALL_STATE(2341)] = 52542,
  [SMALL_STATE(2342)] = 52556,
  [SMALL_STATE(2343)] = 52570,
  [SMALL_STATE(2344)] = 52584,
  [SMALL_STATE(2345)] = 52598,
  [SMALL_STATE(2346)] = 52612,
  [SMALL_STATE(2347)] = 52626,
  [SMALL_STATE(2348)] = 52640,
  [SMALL_STATE(2349)] = 52654,
  [SMALL_STATE(2350)] = 52668,
  [SMALL_STATE(2351)] = 52682,
  [SMALL_STATE(2352)] = 52696,
  [SMALL_STATE(2353)] = 52710,
  [SMALL_STATE(2354)] = 52724,
  [SMALL_STATE(2355)] = 52738,
  [SMALL_STATE(2356)] = 52752,
  [SMALL_STATE(2357)] = 52766,
  [SMALL_STATE(2358)] = 52780,
  [SMALL_STATE(2359)] = 52794,
  [SMALL_STATE(2360)] = 52808,
  [SMALL_STATE(2361)] = 52822,
  [SMALL_STATE(2362)] = 52836,
  [SMALL_STATE(2363)] = 52850,
  [SMALL_STATE(2364)] = 52864,
  [SMALL_STATE(2365)] = 52878,
  [SMALL_STATE(2366)] = 52892,
  [SMALL_STATE(2367)] = 52906,
  [SMALL_STATE(2368)] = 52920,
  [SMALL_STATE(2369)] = 52934,
  [SMALL_STATE(2370)] = 52948,
  [SMALL_STATE(2371)] = 52962,
  [SMALL_STATE(2372)] = 52976,
  [SMALL_STATE(2373)] = 52990,
  [SMALL_STATE(2374)] = 53004,
  [SMALL_STATE(2375)] = 53018,
  [SMALL_STATE(2376)] = 53032,
  [SMALL_STATE(2377)] = 53046,
  [SMALL_STATE(2378)] = 53060,
  [SMALL_STATE(2379)] = 53074,
  [SMALL_STATE(2380)] = 53088,
  [SMALL_STATE(2381)] = 53102,
  [SMALL_STATE(2382)] = 53116,
  [SMALL_STATE(2383)] = 53130,
  [SMALL_STATE(2384)] = 53144,
  [SMALL_STATE(2385)] = 53158,
  [SMALL_STATE(2386)] = 53172,
  [SMALL_STATE(2387)] = 53186,
  [SMALL_STATE(2388)] = 53200,
  [SMALL_STATE(2389)] = 53214,
  [SMALL_STATE(2390)] = 53228,
  [SMALL_STATE(2391)] = 53242,
  [SMALL_STATE(2392)] = 53256,
  [SMALL_STATE(2393)] = 53270,
  [SMALL_STATE(2394)] = 53284,
  [SMALL_STATE(2395)] = 53298,
  [SMALL_STATE(2396)] = 53312,
  [SMALL_STATE(2397)] = 53326,
  [SMALL_STATE(2398)] = 53340,
  [SMALL_STATE(2399)] = 53354,
  [SMALL_STATE(2400)] = 53368,
  [SMALL_STATE(2401)] = 53382,
  [SMALL_STATE(2402)] = 53396,
  [SMALL_STATE(2403)] = 53410,
  [SMALL_STATE(2404)] = 53424,
  [SMALL_STATE(2405)] = 53438,
  [SMALL_STATE(2406)] = 53452,
  [SMALL_STATE(2407)] = 53466,
  [SMALL_STATE(2408)] = 53480,
  [SMALL_STATE(2409)] = 53494,
  [SMALL_STATE(2410)] = 53508,
  [SMALL_STATE(2411)] = 53522,
  [SMALL_STATE(2412)] = 53536,
  [SMALL_STATE(2413)] = 53550,
  [SMALL_STATE(2414)] = 53564,
  [SMALL_STATE(2415)] = 53578,
  [SMALL_STATE(2416)] = 53592,
  [SMALL_STATE(2417)] = 53606,
  [SMALL_STATE(2418)] = 53620,
  [SMALL_STATE(2419)] = 53634,
  [SMALL_STATE(2420)] = 53648,
  [SMALL_STATE(2421)] = 53662,
  [SMALL_STATE(2422)] = 53676,
  [SMALL_STATE(2423)] = 53690,
  [SMALL_STATE(2424)] = 53704,
  [SMALL_STATE(2425)] = 53718,
  [SMALL_STATE(2426)] = 53732,
  [SMALL_STATE(2427)] = 53746,
  [SMALL_STATE(2428)] = 53760,
  [SMALL_STATE(2429)] = 53774,
  [SMALL_STATE(2430)] = 53788,
  [SMALL_STATE(2431)] = 53802,
  [SMALL_STATE(2432)] = 53816,
  [SMALL_STATE(2433)] = 53830,
  [SMALL_STATE(2434)] = 53844,
  [SMALL_STATE(2435)] = 53858,
  [SMALL_STATE(2436)] = 53872,
  [SMALL_STATE(2437)] = 53886,
  [SMALL_STATE(2438)] = 53900,
  [SMALL_STATE(2439)] = 53914,
  [SMALL_STATE(2440)] = 53928,
  [SMALL_STATE(2441)] = 53942,
  [SMALL_STATE(2442)] = 53956,
  [SMALL_STATE(2443)] = 53970,
  [SMALL_STATE(2444)] = 53984,
  [SMALL_STATE(2445)] = 53998,
  [SMALL_STATE(2446)] = 54012,
  [SMALL_STATE(2447)] = 54026,
  [SMALL_STATE(2448)] = 54040,
  [SMALL_STATE(2449)] = 54054,
  [SMALL_STATE(2450)] = 54068,
  [SMALL_STATE(2451)] = 54082,
  [SMALL_STATE(2452)] = 54096,
  [SMALL_STATE(2453)] = 54110,
  [SMALL_STATE(2454)] = 54124,
  [SMALL_STATE(2455)] = 54138,
  [SMALL_STATE(2456)] = 54152,
  [SMALL_STATE(2457)] = 54166,
  [SMALL_STATE(2458)] = 54180,
  [SMALL_STATE(2459)] = 54194,
  [SMALL_STATE(2460)] = 54198,
  [SMALL_STATE(2461)] = 54202,
  [SMALL_STATE(2462)] = 54206,
  [SMALL_STATE(2463)] = 54210,
};

static const TSParseActionEntry ts_parse_actions[] = {
  [0] = {.entry = {.count = 0, .reusable = false}},
  [1] = {.entry = {.count = 1, .reusable = false}}, RECOVER(),
  [3] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1934),
  [5] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1938),
  [7] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_source_file, 0, 0, 0),
  [9] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1655),
  [11] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2249),
  [13] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1775),
  [15] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1646),
  [17] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2314),
  [19] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1934),
  [21] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1938),
  [23] = {.entry = {.count = 1, .reusable = false}}, SHIFT(864),
  [25] = {.entry = {.count = 1, .reusable = false}}, SHIFT(862),
  [27] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1073),
  [29] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1052),
  [31] = {.entry = {.count = 1, .reusable = false}}, SHIFT(863),
  [33] = {.entry = {.count = 1, .reusable = false}}, SHIFT(859),
  [35] = {.entry = {.count = 1, .reusable = false}}, SHIFT(873),
  [37] = {.entry = {.count = 1, .reusable = false}}, SHIFT(875),
  [39] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1082),
  [41] = {.entry = {.count = 1, .reusable = false}}, SHIFT(861),
  [43] = {.entry = {.count = 1, .reusable = false}}, SHIFT(874),
  [45] = {.entry = {.count = 1, .reusable = false}}, SHIFT(881),
  [47] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2301),
  [49] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2171),
  [51] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2434),
  [53] = {.entry = {.count = 1, .reusable = false}}, SHIFT(927),
  [55] = {.entry = {.count = 1, .reusable = true}}, SHIFT(938),
  [57] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_return_expr, 1, 0, 0),
  [59] = {.entry = {.count = 1, .reusable = true}}, SHIFT(45),
  [61] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_return_expr, 1, 0, 0),
  [63] = {.entry = {.count = 1, .reusable = true}}, SHIFT(781),
  [65] = {.entry = {.count = 1, .reusable = true}}, SHIFT(23),
  [67] = {.entry = {.count = 1, .reusable = false}}, SHIFT(782),
  [69] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2432),
  [71] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2407),
  [73] = {.entry = {.count = 1, .reusable = false}}, SHIFT(908),
  [75] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2005),
  [77] = {.entry = {.count = 1, .reusable = false}}, SHIFT(146),
  [79] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2),
  [81] = {.entry = {.count = 1, .reusable = false}}, SHIFT(171),
  [83] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1665),
  [85] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1688),
  [87] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1055),
  [89] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1048),
  [91] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1074),
  [93] = {.entry = {.count = 1, .reusable = true}}, SHIFT(774),
  [95] = {.entry = {.count = 1, .reusable = false}}, SHIFT(775),
  [97] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2146),
  [99] = {.entry = {.count = 1, .reusable = false}}, SHIFT(65),
  [101] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3),
  [103] = {.entry = {.count = 1, .reusable = false}}, SHIFT(272),
  [105] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1070),
  [107] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1061),
  [109] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1059),
  [111] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1049),
  [113] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1160),
  [115] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1133),
  [117] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1185),
  [119] = {.entry = {.count = 1, .reusable = true}}, SHIFT(778),
  [121] = {.entry = {.count = 1, .reusable = false}}, SHIFT(779),
  [123] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2158),
  [125] = {.entry = {.count = 1, .reusable = false}}, SHIFT(125),
  [127] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6),
  [129] = {.entry = {.count = 1, .reusable = false}}, SHIFT(356),
  [131] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1670),
  [133] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1127),
  [135] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1123),
  [137] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1159),
  [139] = {.entry = {.count = 1, .reusable = true}}, SHIFT(790),
  [141] = {.entry = {.count = 1, .reusable = false}}, SHIFT(791),
  [143] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2056),
  [145] = {.entry = {.count = 1, .reusable = false}}, SHIFT(137),
  [147] = {.entry = {.count = 1, .reusable = false}}, SHIFT(7),
  [149] = {.entry = {.count = 1, .reusable = false}}, SHIFT(372),
  [151] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1161),
  [153] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1134),
  [155] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1200),
  [157] = {.entry = {.count = 1, .reusable = true}}, SHIFT(806),
  [159] = {.entry = {.count = 1, .reusable = false}}, SHIFT(807),
  [161] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2084),
  [163] = {.entry = {.count = 1, .reusable = false}}, SHIFT(97),
  [165] = {.entry = {.count = 1, .reusable = false}}, SHIFT(8),
  [167] = {.entry = {.count = 1, .reusable = false}}, SHIFT(229),
  [169] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1178),
  [171] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1156),
  [173] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1223),
  [175] = {.entry = {.count = 1, .reusable = true}}, SHIFT(786),
  [177] = {.entry = {.count = 1, .reusable = false}}, SHIFT(787),
  [179] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2025),
  [181] = {.entry = {.count = 1, .reusable = false}}, SHIFT(85),
  [183] = {.entry = {.count = 1, .reusable = false}}, SHIFT(9),
  [185] = {.entry = {.count = 1, .reusable = false}}, SHIFT(174),
  [187] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1126),
  [189] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1122),
  [191] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1162),
  [193] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1135),
  [195] = {.entry = {.count = 1, .reusable = true}}, SHIFT(830),
  [197] = {.entry = {.count = 1, .reusable = false}}, SHIFT(831),
  [199] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2159),
  [201] = {.entry = {.count = 1, .reusable = false}}, SHIFT(123),
  [203] = {.entry = {.count = 1, .reusable = false}}, SHIFT(12),
  [205] = {.entry = {.count = 1, .reusable = false}}, SHIFT(389),
  [207] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1201),
  [209] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1157),
  [211] = {.entry = {.count = 1, .reusable = true}}, SHIFT(837),
  [213] = {.entry = {.count = 1, .reusable = false}}, SHIFT(838),
  [215] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2072),
  [217] = {.entry = {.count = 1, .reusable = false}}, SHIFT(59),
  [219] = {.entry = {.count = 1, .reusable = false}}, SHIFT(14),
  [221] = {.entry = {.count = 1, .reusable = false}}, SHIFT(190),
  [223] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1152),
  [225] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1131),
  [227] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1203),
  [229] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1174),
  [231] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1237),
  [233] = {.entry = {.count = 1, .reusable = true}}, SHIFT(794),
  [235] = {.entry = {.count = 1, .reusable = false}}, SHIFT(795),
  [237] = {.entry = {.count = 1, .reusable = false}}, SHIFT(89),
  [239] = {.entry = {.count = 1, .reusable = false}}, SHIFT(16),
  [241] = {.entry = {.count = 1, .reusable = false}}, SHIFT(209),
  [243] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1196),
  [245] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1153),
  [247] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1224),
  [249] = {.entry = {.count = 1, .reusable = true}}, SHIFT(814),
  [251] = {.entry = {.count = 1, .reusable = false}}, SHIFT(815),
  [253] = {.entry = {.count = 1, .reusable = false}}, SHIFT(105),
  [255] = {.entry = {.count = 1, .reusable = false}}, SHIFT(17),
  [257] = {.entry = {.count = 1, .reusable = false}}, SHIFT(247),
  [259] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1183),
  [261] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1158),
  [263] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1222),
  [265] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1177),
  [267] = {.entry = {.count = 1, .reusable = true}}, SHIFT(770),
  [269] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1193),
  [271] = {.entry = {.count = 1, .reusable = false}}, SHIFT(780),
  [273] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1504),
  [275] = {.entry = {.count = 1, .reusable = true}}, SHIFT(782),
  [277] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1176),
  [279] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1619),
  [281] = {.entry = {.count = 1, .reusable = true}}, SHIFT(850),
  [283] = {.entry = {.count = 1, .reusable = true}}, SHIFT(848),
  [285] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1510),
  [287] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(864),
  [290] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(862),
  [293] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(1070),
  [296] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(1061),
  [299] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(863),
  [302] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(859),
  [305] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(873),
  [308] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(875),
  [311] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(1082),
  [314] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(861),
  [317] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(874),
  [320] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(881),
  [323] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2301),
  [326] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2171),
  [329] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2434),
  [332] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(927),
  [335] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(938),
  [338] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(770),
  [341] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(45),
  [344] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(1193),
  [347] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(780),
  [350] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(781),
  [353] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(23),
  [356] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0),
  [358] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(782),
  [361] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2432),
  [364] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2407),
  [367] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(908),
  [370] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2005),
  [373] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(146),
  [376] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2),
  [379] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(171),
  [382] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(1665),
  [385] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(1688),
  [388] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_block_repeat1, 2, 0, 0), SHIFT_REPEAT(1176),
  [391] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1512),
  [393] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1794),
  [395] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1375),
  [397] = {.entry = {.count = 1, .reusable = false}}, SHIFT(48),
  [399] = {.entry = {.count = 1, .reusable = true}}, SHIFT(852),
  [401] = {.entry = {.count = 1, .reusable = true}}, SHIFT(772),
  [403] = {.entry = {.count = 1, .reusable = false}}, SHIFT(773),
  [405] = {.entry = {.count = 1, .reusable = true}}, SHIFT(775),
  [407] = {.entry = {.count = 1, .reusable = false}}, SHIFT(47),
  [409] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2347),
  [411] = {.entry = {.count = 1, .reusable = true}}, SHIFT(942),
  [413] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2271),
  [415] = {.entry = {.count = 1, .reusable = true}}, SHIFT(940),
  [417] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2174),
  [419] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1889),
  [421] = {.entry = {.count = 1, .reusable = true}}, SHIFT(776),
  [423] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2411),
  [425] = {.entry = {.count = 1, .reusable = true}}, SHIFT(936),
  [427] = {.entry = {.count = 1, .reusable = false}}, SHIFT(777),
  [429] = {.entry = {.count = 1, .reusable = true}}, SHIFT(779),
  [431] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2321),
  [433] = {.entry = {.count = 1, .reusable = true}}, SHIFT(929),
  [435] = {.entry = {.count = 1, .reusable = true}}, SHIFT(788),
  [437] = {.entry = {.count = 1, .reusable = false}}, SHIFT(789),
  [439] = {.entry = {.count = 1, .reusable = true}}, SHIFT(791),
  [441] = {.entry = {.count = 1, .reusable = true}}, SHIFT(910),
  [443] = {.entry = {.count = 1, .reusable = true}}, SHIFT(912),
  [445] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1859),
  [447] = {.entry = {.count = 1, .reusable = true}}, SHIFT(919),
  [449] = {.entry = {.count = 1, .reusable = false}}, SHIFT(963),
  [451] = {.entry = {.count = 1, .reusable = false}}, SHIFT(964),
  [453] = {.entry = {.count = 1, .reusable = false}}, SHIFT(961),
  [455] = {.entry = {.count = 1, .reusable = true}}, SHIFT(835),
  [457] = {.entry = {.count = 1, .reusable = false}}, SHIFT(836),
  [459] = {.entry = {.count = 1, .reusable = true}}, SHIFT(838),
  [461] = {.entry = {.count = 1, .reusable = true}}, SHIFT(784),
  [463] = {.entry = {.count = 1, .reusable = false}}, SHIFT(785),
  [465] = {.entry = {.count = 1, .reusable = true}}, SHIFT(787),
  [467] = {.entry = {.count = 1, .reusable = false}}, SHIFT(975),
  [469] = {.entry = {.count = 1, .reusable = false}}, SHIFT(974),
  [471] = {.entry = {.count = 1, .reusable = false}}, SHIFT(972),
  [473] = {.entry = {.count = 1, .reusable = true}}, SHIFT(828),
  [475] = {.entry = {.count = 1, .reusable = false}}, SHIFT(829),
  [477] = {.entry = {.count = 1, .reusable = true}}, SHIFT(831),
  [479] = {.entry = {.count = 1, .reusable = false}}, SHIFT(962),
  [481] = {.entry = {.count = 1, .reusable = false}}, SHIFT(958),
  [483] = {.entry = {.count = 1, .reusable = true}}, SHIFT(792),
  [485] = {.entry = {.count = 1, .reusable = false}}, SHIFT(793),
  [487] = {.entry = {.count = 1, .reusable = true}}, SHIFT(795),
  [489] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2011),
  [491] = {.entry = {.count = 1, .reusable = true}}, SHIFT(804),
  [493] = {.entry = {.count = 1, .reusable = false}}, SHIFT(805),
  [495] = {.entry = {.count = 1, .reusable = true}}, SHIFT(807),
  [497] = {.entry = {.count = 1, .reusable = true}}, SHIFT(812),
  [499] = {.entry = {.count = 1, .reusable = false}}, SHIFT(813),
  [501] = {.entry = {.count = 1, .reusable = true}}, SHIFT(815),
  [503] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2144),
  [505] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1914),
  [507] = {.entry = {.count = 1, .reusable = false}}, SHIFT(970),
  [509] = {.entry = {.count = 1, .reusable = false}}, SHIFT(973),
  [511] = {.entry = {.count = 1, .reusable = true}}, SHIFT(946),
  [513] = {.entry = {.count = 1, .reusable = true}}, SHIFT(897),
  [515] = {.entry = {.count = 1, .reusable = true}}, SHIFT(900),
  [517] = {.entry = {.count = 1, .reusable = true}}, SHIFT(902),
  [519] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_func, 3, 0, 52),
  [521] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_func, 3, 0, 52),
  [523] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_func, 4, 0, 87),
  [525] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_func, 4, 0, 87),
  [527] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_func, 3, 0, 53),
  [529] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_func, 3, 0, 53),
  [531] = {.entry = {.count = 1, .reusable = false}}, SHIFT(978),
  [533] = {.entry = {.count = 1, .reusable = true}}, SHIFT(839),
  [535] = {.entry = {.count = 1, .reusable = false}}, SHIFT(840),
  [537] = {.entry = {.count = 1, .reusable = true}}, SHIFT(841),
  [539] = {.entry = {.count = 1, .reusable = true}}, SHIFT(842),
  [541] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2107),
  [543] = {.entry = {.count = 1, .reusable = false}}, SHIFT(69),
  [545] = {.entry = {.count = 1, .reusable = false}}, SHIFT(18),
  [547] = {.entry = {.count = 1, .reusable = false}}, SHIFT(294),
  [549] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1227),
  [551] = {.entry = {.count = 1, .reusable = true}}, SHIFT(800),
  [553] = {.entry = {.count = 1, .reusable = false}}, SHIFT(801),
  [555] = {.entry = {.count = 1, .reusable = true}}, SHIFT(802),
  [557] = {.entry = {.count = 1, .reusable = true}}, SHIFT(803),
  [559] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2114),
  [561] = {.entry = {.count = 1, .reusable = false}}, SHIFT(141),
  [563] = {.entry = {.count = 1, .reusable = false}}, SHIFT(11),
  [565] = {.entry = {.count = 1, .reusable = false}}, SHIFT(497),
  [567] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1106),
  [569] = {.entry = {.count = 1, .reusable = true}}, SHIFT(758),
  [571] = {.entry = {.count = 1, .reusable = false}}, SHIFT(759),
  [573] = {.entry = {.count = 1, .reusable = true}}, SHIFT(760),
  [575] = {.entry = {.count = 1, .reusable = true}}, SHIFT(761),
  [577] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2120),
  [579] = {.entry = {.count = 1, .reusable = false}}, SHIFT(75),
  [581] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4),
  [583] = {.entry = {.count = 1, .reusable = false}}, SHIFT(476),
  [585] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1235),
  [587] = {.entry = {.count = 1, .reusable = true}}, SHIFT(808),
  [589] = {.entry = {.count = 1, .reusable = false}}, SHIFT(809),
  [591] = {.entry = {.count = 1, .reusable = true}}, SHIFT(810),
  [593] = {.entry = {.count = 1, .reusable = true}}, SHIFT(811),
  [595] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2121),
  [597] = {.entry = {.count = 1, .reusable = false}}, SHIFT(101),
  [599] = {.entry = {.count = 1, .reusable = false}}, SHIFT(13),
  [601] = {.entry = {.count = 1, .reusable = false}}, SHIFT(392),
  [603] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1085),
  [605] = {.entry = {.count = 1, .reusable = true}}, SHIFT(824),
  [607] = {.entry = {.count = 1, .reusable = false}}, SHIFT(825),
  [609] = {.entry = {.count = 1, .reusable = true}}, SHIFT(826),
  [611] = {.entry = {.count = 1, .reusable = true}}, SHIFT(827),
  [613] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2067),
  [615] = {.entry = {.count = 1, .reusable = false}}, SHIFT(116),
  [617] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5),
  [619] = {.entry = {.count = 1, .reusable = false}}, SHIFT(478),
  [621] = {.entry = {.count = 1, .reusable = false}}, SHIFT(971),
  [623] = {.entry = {.count = 1, .reusable = true}}, SHIFT(754),
  [625] = {.entry = {.count = 1, .reusable = false}}, SHIFT(755),
  [627] = {.entry = {.count = 1, .reusable = true}}, SHIFT(756),
  [629] = {.entry = {.count = 1, .reusable = true}}, SHIFT(757),
  [631] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2006),
  [633] = {.entry = {.count = 1, .reusable = false}}, SHIFT(79),
  [635] = {.entry = {.count = 1, .reusable = false}}, SHIFT(19),
  [637] = {.entry = {.count = 1, .reusable = false}}, SHIFT(309),
  [639] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1197),
  [641] = {.entry = {.count = 1, .reusable = true}}, SHIFT(796),
  [643] = {.entry = {.count = 1, .reusable = false}}, SHIFT(846),
  [645] = {.entry = {.count = 1, .reusable = true}}, SHIFT(798),
  [647] = {.entry = {.count = 1, .reusable = true}}, SHIFT(799),
  [649] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2098),
  [651] = {.entry = {.count = 1, .reusable = false}}, SHIFT(145),
  [653] = {.entry = {.count = 1, .reusable = false}}, SHIFT(10),
  [655] = {.entry = {.count = 1, .reusable = false}}, SHIFT(499),
  [657] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1225),
  [659] = {.entry = {.count = 1, .reusable = true}}, SHIFT(832),
  [661] = {.entry = {.count = 1, .reusable = false}}, SHIFT(833),
  [663] = {.entry = {.count = 1, .reusable = true}}, SHIFT(834),
  [665] = {.entry = {.count = 1, .reusable = true}}, SHIFT(753),
  [667] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2048),
  [669] = {.entry = {.count = 1, .reusable = false}}, SHIFT(118),
  [671] = {.entry = {.count = 1, .reusable = false}}, SHIFT(15),
  [673] = {.entry = {.count = 1, .reusable = false}}, SHIFT(418),
  [675] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1239),
  [677] = {.entry = {.count = 1, .reusable = true}}, SHIFT(843),
  [679] = {.entry = {.count = 1, .reusable = false}}, SHIFT(844),
  [681] = {.entry = {.count = 1, .reusable = true}}, SHIFT(845),
  [683] = {.entry = {.count = 1, .reusable = true}}, SHIFT(797),
  [685] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2143),
  [687] = {.entry = {.count = 1, .reusable = false}}, SHIFT(120),
  [689] = {.entry = {.count = 1, .reusable = false}}, SHIFT(20),
  [691] = {.entry = {.count = 1, .reusable = false}}, SHIFT(431),
  [693] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1242),
  [695] = {.entry = {.count = 1, .reusable = true}}, SHIFT(816),
  [697] = {.entry = {.count = 1, .reusable = false}}, SHIFT(817),
  [699] = {.entry = {.count = 1, .reusable = true}}, SHIFT(818),
  [701] = {.entry = {.count = 1, .reusable = true}}, SHIFT(819),
  [703] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2164),
  [705] = {.entry = {.count = 1, .reusable = false}}, SHIFT(109),
  [707] = {.entry = {.count = 1, .reusable = false}}, SHIFT(21),
  [709] = {.entry = {.count = 1, .reusable = false}}, SHIFT(405),
  [711] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1626),
  [713] = {.entry = {.count = 1, .reusable = true}}, SHIFT(762),
  [715] = {.entry = {.count = 1, .reusable = false}}, SHIFT(763),
  [717] = {.entry = {.count = 1, .reusable = true}}, SHIFT(764),
  [719] = {.entry = {.count = 1, .reusable = true}}, SHIFT(765),
  [721] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2036),
  [723] = {.entry = {.count = 1, .reusable = false}}, SHIFT(52),
  [725] = {.entry = {.count = 1, .reusable = false}}, SHIFT(35),
  [727] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1630),
  [729] = {.entry = {.count = 1, .reusable = true}}, SHIFT(766),
  [731] = {.entry = {.count = 1, .reusable = false}}, SHIFT(767),
  [733] = {.entry = {.count = 1, .reusable = true}}, SHIFT(768),
  [735] = {.entry = {.count = 1, .reusable = true}}, SHIFT(769),
  [737] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2129),
  [739] = {.entry = {.count = 1, .reusable = false}}, SHIFT(93),
  [741] = {.entry = {.count = 1, .reusable = false}}, SHIFT(36),
  [743] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1599),
  [745] = {.entry = {.count = 1, .reusable = true}}, SHIFT(820),
  [747] = {.entry = {.count = 1, .reusable = false}}, SHIFT(821),
  [749] = {.entry = {.count = 1, .reusable = true}}, SHIFT(822),
  [751] = {.entry = {.count = 1, .reusable = true}}, SHIFT(823),
  [753] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2145),
  [755] = {.entry = {.count = 1, .reusable = false}}, SHIFT(113),
  [757] = {.entry = {.count = 1, .reusable = false}}, SHIFT(33),
  [759] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_block, 5, 0, 89),
  [761] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_block, 5, 0, 89),
  [763] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block, 3, 0, 0),
  [765] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block, 3, 0, 0),
  [767] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_block, 5, 0, 88),
  [769] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_block, 5, 0, 88),
  [771] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block, 2, 0, 0),
  [773] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block, 2, 0, 0),
  [775] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_block, 3, 0, 22),
  [777] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_block, 3, 0, 22),
  [779] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block, 4, 0, 0),
  [781] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block, 4, 0, 0),
  [783] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_block, 5, 0, 86),
  [785] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_block, 5, 0, 86),
  [787] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block, 5, 0, 0),
  [789] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block, 5, 0, 0),
  [791] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_block, 4, 0, 50),
  [793] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_block, 4, 0, 50),
  [795] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_block, 6, 0, 128),
  [797] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_block, 6, 0, 128),
  [799] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_block, 4, 0, 49),
  [801] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_block, 4, 0, 49),
  [803] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_block, 4, 0, 54),
  [805] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_block, 4, 0, 54),
  [807] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_primitive_type, 1, 0, 0),
  [809] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_primitive_type, 1, 0, 0),
  [811] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_discouraged_name, 1, 0, 0),
  [813] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_discouraged_name, 1, 0, 0),
  [815] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__reuseable_keywords, 1, 0, 0),
  [817] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__reuseable_keywords, 1, 0, 0),
  [819] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_number_type, 1, 0, 0),
  [821] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_number_type, 1, 0, 0),
  [823] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__quantifier_directive, 1, 0, 0),
  [825] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__quantifier_directive, 1, 0, 0),
  [827] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_name_access_chain, 1, 0, 1),
  [829] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_name_access_chain, 1, 0, 1),
  [831] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__leading_name_access, 1, 0, 0),
  [833] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_name_access_chain, 3, 0, 45),
  [835] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_name_access_chain, 3, 0, 45),
  [837] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2212),
  [839] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2193),
  [841] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_name_access_chain, 5, 0, 118),
  [843] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_name_access_chain, 5, 0, 118),
  [845] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2281),
  [847] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_name_access_chain, 7, 0, 168),
  [849] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_name_access_chain, 7, 0, 168),
  [851] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_args, 3, 0, 0),
  [853] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_args, 3, 0, 0),
  [855] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_args, 5, 0, 0),
  [857] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_args, 5, 0, 0),
  [859] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_args, 4, 0, 0),
  [861] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_args, 4, 0, 0),
  [863] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_args, 2, 0, 0),
  [865] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_args, 2, 0, 0),
  [867] = {.entry = {.count = 1, .reusable = true}}, SHIFT(37),
  [869] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1069),
  [871] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__reuseable_keywords, 1, 0, 0), SHIFT(169),
  [874] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__reuseable_keywords, 1, 0, 0), SHIFT(1433),
  [877] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_while_expr, 3, 0, 56),
  [879] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_while_expr, 3, 0, 56),
  [881] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_access_field, 3, 0, 69),
  [883] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_access_field, 3, 0, 69),
  [885] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2081),
  [887] = {.entry = {.count = 1, .reusable = true}}, SHIFT(46),
  [889] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__identifier_or_anon_field, 1, 0, 0),
  [891] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__identifier_or_anon_field, 1, 0, 0),
  [893] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_numerical_addr, 1, 0, 0),
  [895] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_numerical_addr, 1, 0, 0),
  [897] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_number, 1, 0, 0),
  [899] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_number, 1, 0, 0),
  [901] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_block, 2, 0, 0),
  [903] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_block, 2, 0, 0),
  [905] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__leading_name_access, 1, 0, 0),
  [907] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_number, 2, 0, 0),
  [909] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_number, 2, 0, 0),
  [911] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call_expr, 3, 0, 66),
  [913] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call_expr, 3, 0, 66),
  [915] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_bin_op_expr, 3, 0, 67),
  [917] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_bin_op_expr, 3, 0, 67),
  [919] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_assignment, 3, 0, 68),
  [921] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_assignment, 3, 0, 68),
  [923] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_copy_expr, 2, 0, 27),
  [925] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_copy_expr, 2, 0, 27),
  [927] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__control_body, 1, 0, 0),
  [929] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__control_body, 1, 0, 0),
  [931] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_loop_expr, 2, 0, 29),
  [933] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_loop_expr, 2, 0, 29),
  [935] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_quantifier, 5, 0, 130),
  [937] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_quantifier, 5, 0, 130),
  [939] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_quantifier, 5, 0, 131),
  [941] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_quantifier, 5, 0, 131),
  [943] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_return_expr, 2, 0, 30),
  [945] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_return_expr, 2, 0, 30),
  [947] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_abort_expr, 2, 0, 31),
  [949] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abort_expr, 2, 0, 31),
  [951] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_quantifier, 5, 0, 132),
  [953] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_quantifier, 5, 0, 132),
  [955] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_vector_value_expr, 5, 0, 0),
  [957] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_vector_value_expr, 5, 0, 0),
  [959] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_hint_expr, 5, 0, 0),
  [961] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_hint_expr, 5, 0, 0),
  [963] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_cast_expr, 5, 0, 0),
  [965] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_cast_expr, 5, 0, 0),
  [967] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple_expr, 5, 0, 0),
  [969] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_expr, 5, 0, 0),
  [971] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if_expr, 5, 0, 134),
  [973] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_expr, 5, 0, 134),
  [975] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call_args, 4, 0, 0),
  [977] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call_args, 4, 0, 0),
  [979] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_pack_expr, 5, 0, 64),
  [981] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pack_expr, 5, 0, 64),
  [983] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_pack_expr, 5, 0, 104),
  [985] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pack_expr, 5, 0, 104),
  [987] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call_expr, 2, 0, 32),
  [989] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call_expr, 2, 0, 32),
  [991] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__expr, 2, 0, 34),
  [993] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__expr, 2, 0, 34),
  [995] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_quantifier, 6, 0, 156),
  [997] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_quantifier, 6, 0, 156),
  [999] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_term, 1, 0, 0),
  [1001] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_term, 1, 0, 0),
  [1003] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_quantifier, 6, 0, 158),
  [1005] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_quantifier, 6, 0, 158),
  [1007] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_vector_value_expr, 6, 0, 0),
  [1009] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_vector_value_expr, 6, 0, 0),
  [1011] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_match_expr, 6, 0, 159),
  [1013] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_expr, 6, 0, 159),
  [1015] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call_args, 5, 0, 0),
  [1017] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call_args, 5, 0, 0),
  [1019] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_pack_expr, 6, 0, 64),
  [1021] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pack_expr, 6, 0, 64),
  [1023] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_pack_expr, 6, 0, 104),
  [1025] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pack_expr, 6, 0, 104),
  [1027] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_receiver_call, 6, 0, 161),
  [1029] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_receiver_call, 6, 0, 161),
  [1031] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_quantifier, 7, 0, 179),
  [1033] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_quantifier, 7, 0, 179),
  [1035] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_quantifier, 7, 0, 180),
  [1037] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_quantifier, 7, 0, 180),
  [1039] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__expr, 1, 0, 0),
  [1041] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__expr, 1, 0, 0),
  [1043] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_vector_value_expr, 7, 0, 0),
  [1045] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_vector_value_expr, 7, 0, 0),
  [1047] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__op_expr, 1, 0, 0),
  [1049] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__op_expr, 1, 0, 0),
  [1051] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_pack_expr, 7, 0, 104),
  [1053] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pack_expr, 7, 0, 104),
  [1055] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_quantifier, 8, 0, 194),
  [1057] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_quantifier, 8, 0, 194),
  [1059] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for_loop_expr, 9, 0, 207),
  [1061] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_loop_expr, 9, 0, 207),
  [1063] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for_loop_expr, 10, 0, 217),
  [1065] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_loop_expr, 10, 0, 217),
  [1067] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__unary_expr, 1, 0, 0),
  [1069] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__unary_expr, 1, 0, 0),
  [1071] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1949),
  [1073] = {.entry = {.count = 1, .reusable = true}}, SHIFT(274),
  [1075] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_bool_literal, 1, 0, 0),
  [1077] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_bool_literal, 1, 0, 0),
  [1079] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call_args, 2, 0, 0),
  [1081] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call_args, 2, 0, 0),
  [1083] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_value, 2, 0, 0),
  [1085] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_value, 2, 0, 0),
  [1087] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_pack_expr, 3, 0, 64),
  [1089] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pack_expr, 3, 0, 64),
  [1091] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_call_expr, 3, 0, 65),
  [1093] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_call_expr, 3, 0, 65),
  [1095] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_quantifier, 4, 0, 92),
  [1097] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_quantifier, 4, 0, 92),
  [1099] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_deref_expr, 2, 0, 0),
  [1101] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_deref_expr, 2, 0, 0),
  [1103] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple_expr, 2, 0, 0),
  [1105] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_expr, 2, 0, 0),
  [1107] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_typed_number, 2, 0, 0),
  [1109] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_typed_number, 2, 0, 0),
  [1111] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_byte_string, 1, 0, 0),
  [1113] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_byte_string, 1, 0, 0),
  [1115] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_quantifier, 4, 0, 94),
  [1117] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_quantifier, 4, 0, 94),
  [1119] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_vector_value_expr, 4, 0, 0),
  [1121] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_vector_value_expr, 4, 0, 0),
  [1123] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_ref_expr, 2, 0, 0),
  [1125] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ref_expr, 2, 0, 0),
  [1127] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_vector_value_expr, 3, 0, 0),
  [1129] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_vector_value_expr, 3, 0, 0),
  [1131] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_value, 1, 0, 0),
  [1133] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_value, 1, 0, 0),
  [1135] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_loop_invariant, 1, 0, 95),
  [1137] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_loop_invariant, 1, 0, 95),
  [1139] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_while_expr, 4, 0, 56),
  [1141] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_while_expr, 4, 0, 56),
  [1143] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple_expr, 4, 0, 0),
  [1145] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_expr, 4, 0, 0),
  [1147] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_ref_mut_expr, 2, 0, 0),
  [1149] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ref_mut_expr, 2, 0, 0),
  [1151] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple_expr, 3, 0, 0),
  [1153] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_expr, 3, 0, 0),
  [1155] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call_args, 3, 0, 0),
  [1157] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call_args, 3, 0, 0),
  [1159] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_not_expr, 2, 0, 0),
  [1161] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_not_expr, 2, 0, 0),
  [1163] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_move_expr, 2, 0, 27),
  [1165] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_move_expr, 2, 0, 27),
  [1167] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_pack_expr, 4, 0, 64),
  [1169] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pack_expr, 4, 0, 64),
  [1171] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_pack_expr, 4, 0, 104),
  [1173] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pack_expr, 4, 0, 104),
  [1175] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_receiver_call, 4, 0, 105),
  [1177] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_receiver_call, 4, 0, 105),
  [1179] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_mem_access, 4, 0, 106),
  [1181] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_mem_access, 4, 0, 106),
  [1183] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_match_expr, 7, 0, 159),
  [1185] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_expr, 7, 0, 159),
  [1187] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_use_decl, 3, 0, 18),
  [1189] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_decl, 3, 0, 18),
  [1191] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym__quantifier_directive, 1, 0, 0), SHIFT(2101),
  [1194] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym__quantifier_directive, 1, 0, 0), SHIFT(2004),
  [1197] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_var, 1, 0, 0),
  [1199] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_var, 1, 0, 0),
  [1201] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_var, 1, 0, 0), SHIFT(46),
  [1204] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1827),
  [1206] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2092),
  [1208] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_use_decl, 8, 0, 163),
  [1210] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_decl, 8, 0, 163),
  [1212] = {.entry = {.count = 1, .reusable = true}}, SHIFT(391),
  [1214] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym__quantifier_directive, 1, 0, 0), SHIFT(2038),
  [1217] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym__quantifier_directive, 1, 0, 0), SHIFT(2169),
  [1220] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_use_decl, 7, 0, 139),
  [1222] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_decl, 7, 0, 139),
  [1224] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_use_decl, 5, 0, 75),
  [1226] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_decl, 5, 0, 75),
  [1228] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_use_decl, 4, 0, 18),
  [1230] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_decl, 4, 0, 18),
  [1232] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_use_decl, 9, 0, 182),
  [1234] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_decl, 9, 0, 182),
  [1236] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_use_decl, 6, 0, 18),
  [1238] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_decl, 6, 0, 18),
  [1240] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym__quantifier_directive, 1, 0, 0), SHIFT(2166),
  [1243] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym__quantifier_directive, 1, 0, 0), SHIFT(2155),
  [1246] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1537),
  [1248] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1524),
  [1250] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1525),
  [1252] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1526),
  [1254] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1528),
  [1256] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1072),
  [1258] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_type, 3, 0, 112),
  [1260] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1080),
  [1262] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1137),
  [1264] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1148),
  [1266] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_closure_type, 3, 0, 112),
  [1268] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_type, 4, 0, 141),
  [1270] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_closure_type, 4, 0, 141),
  [1272] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2316),
  [1274] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_type, 5, 0, 165),
  [1276] = {.entry = {.count = 1, .reusable = true}}, SHIFT(981),
  [1278] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_closure_type, 5, 0, 165),
  [1280] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_type, 2, 0, 0),
  [1282] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_closure_type, 2, 0, 0),
  [1284] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_var, 2, 0, 33),
  [1286] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_var, 2, 0, 33),
  [1288] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_var, 2, 0, 33), SHIFT(46),
  [1291] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1798),
  [1293] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1084),
  [1295] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1154),
  [1297] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1155),
  [1299] = {.entry = {.count = 1, .reusable = true}}, SHIFT(568),
  [1301] = {.entry = {.count = 1, .reusable = false}}, SHIFT(569),
  [1303] = {.entry = {.count = 1, .reusable = false}}, SHIFT(581),
  [1305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(594),
  [1307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(595),
  [1309] = {.entry = {.count = 1, .reusable = false}}, SHIFT(596),
  [1311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(596),
  [1313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(597),
  [1315] = {.entry = {.count = 1, .reusable = true}}, SHIFT(598),
  [1317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(599),
  [1319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(600),
  [1321] = {.entry = {.count = 1, .reusable = false}}, SHIFT(568),
  [1323] = {.entry = {.count = 1, .reusable = false}}, SHIFT(193),
  [1325] = {.entry = {.count = 1, .reusable = false}}, SHIFT(311),
  [1327] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if_expr, 3, 0, 61),
  [1329] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_expr, 3, 0, 61),
  [1331] = {.entry = {.count = 1, .reusable = false}}, SHIFT(64),
  [1333] = {.entry = {.count = 1, .reusable = true}}, SHIFT(582),
  [1335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(546),
  [1337] = {.entry = {.count = 1, .reusable = false}}, SHIFT(546),
  [1339] = {.entry = {.count = 1, .reusable = false}}, SHIFT(296),
  [1341] = {.entry = {.count = 1, .reusable = false}}, SHIFT(547),
  [1343] = {.entry = {.count = 1, .reusable = false}}, SHIFT(548),
  [1345] = {.entry = {.count = 1, .reusable = true}}, SHIFT(549),
  [1347] = {.entry = {.count = 1, .reusable = true}}, SHIFT(550),
  [1349] = {.entry = {.count = 1, .reusable = true}}, SHIFT(554),
  [1351] = {.entry = {.count = 1, .reusable = false}}, SHIFT(566),
  [1353] = {.entry = {.count = 1, .reusable = true}}, SHIFT(566),
  [1355] = {.entry = {.count = 1, .reusable = true}}, SHIFT(567),
  [1357] = {.entry = {.count = 1, .reusable = true}}, SHIFT(604),
  [1359] = {.entry = {.count = 1, .reusable = true}}, SHIFT(605),
  [1361] = {.entry = {.count = 1, .reusable = true}}, SHIFT(606),
  [1363] = {.entry = {.count = 1, .reusable = true}}, SHIFT(667),
  [1365] = {.entry = {.count = 1, .reusable = false}}, SHIFT(669),
  [1367] = {.entry = {.count = 1, .reusable = true}}, SHIFT(720),
  [1369] = {.entry = {.count = 1, .reusable = true}}, SHIFT(721),
  [1371] = {.entry = {.count = 1, .reusable = true}}, SHIFT(722),
  [1373] = {.entry = {.count = 1, .reusable = false}}, SHIFT(667),
  [1375] = {.entry = {.count = 1, .reusable = false}}, SHIFT(668),
  [1377] = {.entry = {.count = 1, .reusable = true}}, SHIFT(716),
  [1379] = {.entry = {.count = 1, .reusable = true}}, SHIFT(717),
  [1381] = {.entry = {.count = 1, .reusable = false}}, SHIFT(718),
  [1383] = {.entry = {.count = 1, .reusable = true}}, SHIFT(718),
  [1385] = {.entry = {.count = 1, .reusable = true}}, SHIFT(719),
  [1387] = {.entry = {.count = 1, .reusable = true}}, SHIFT(715),
  [1389] = {.entry = {.count = 1, .reusable = false}}, SHIFT(84),
  [1391] = {.entry = {.count = 1, .reusable = false}}, SHIFT(136),
  [1393] = {.entry = {.count = 1, .reusable = false}}, SHIFT(475),
  [1395] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_params, 2, 0, 0),
  [1397] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_params, 2, 0, 0),
  [1399] = {.entry = {.count = 1, .reusable = true}}, SHIFT(607),
  [1401] = {.entry = {.count = 1, .reusable = false}}, SHIFT(609),
  [1403] = {.entry = {.count = 1, .reusable = true}}, SHIFT(654),
  [1405] = {.entry = {.count = 1, .reusable = true}}, SHIFT(655),
  [1407] = {.entry = {.count = 1, .reusable = true}}, SHIFT(686),
  [1409] = {.entry = {.count = 1, .reusable = false}}, SHIFT(607),
  [1411] = {.entry = {.count = 1, .reusable = false}}, SHIFT(608),
  [1413] = {.entry = {.count = 1, .reusable = true}}, SHIFT(624),
  [1415] = {.entry = {.count = 1, .reusable = true}}, SHIFT(625),
  [1417] = {.entry = {.count = 1, .reusable = false}}, SHIFT(532),
  [1419] = {.entry = {.count = 1, .reusable = true}}, SHIFT(532),
  [1421] = {.entry = {.count = 1, .reusable = true}}, SHIFT(653),
  [1423] = {.entry = {.count = 1, .reusable = true}}, SHIFT(610),
  [1425] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_params, 3, 0, 0),
  [1427] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_params, 3, 0, 0),
  [1429] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_params, 5, 0, 0),
  [1431] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_params, 5, 0, 0),
  [1433] = {.entry = {.count = 1, .reusable = false}}, SHIFT(74),
  [1435] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_params, 4, 0, 0),
  [1437] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_params, 4, 0, 0),
  [1439] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_spec_block_repeat1, 2, 0, 0),
  [1441] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_spec_block_repeat1, 2, 0, 0),
  [1443] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_spec_block_repeat1, 2, 0, 0), SHIFT_REPEAT(1619),
  [1446] = {.entry = {.count = 1, .reusable = true}}, SHIFT(928),
  [1448] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_spec_block_repeat1, 1, 0, 0),
  [1450] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_spec_block_repeat1, 1, 0, 0),
  [1452] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__sequence_item, 2, 0, 0),
  [1454] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__sequence_item, 2, 0, 0),
  [1456] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_block_repeat1, 1, 0, 0),
  [1458] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_block_repeat1, 1, 0, 0),
  [1460] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2101),
  [1462] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2044),
  [1464] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2140),
  [1466] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_lambda_bind_list, 3, 0, 0),
  [1468] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_lambda_bind_list, 3, 0, 0),
  [1470] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2160),
  [1472] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_condition_props, 5, 0, 133),
  [1474] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_condition_props, 5, 0, 133),
  [1476] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_condition_props, 3, 0, 59),
  [1478] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_condition_props, 3, 0, 59),
  [1480] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2038),
  [1482] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_lambda_bind_list, 2, 0, 0),
  [1484] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_lambda_bind_list, 2, 0, 0),
  [1486] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_condition_props, 2, 0, 0),
  [1488] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_condition_props, 2, 0, 0),
  [1490] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parenthesized_expr, 3, 0, 0),
  [1492] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parenthesized_expr, 3, 0, 0),
  [1494] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_condition_props, 3, 0, 60),
  [1496] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_condition_props, 3, 0, 60),
  [1498] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2066),
  [1500] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_lambda_bind_list, 4, 0, 58),
  [1502] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_lambda_bind_list, 4, 0, 58),
  [1504] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_lambda_bind_list, 4, 0, 97),
  [1506] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_lambda_bind_list, 4, 0, 97),
  [1508] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_condition_props, 4, 0, 100),
  [1510] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_condition_props, 4, 0, 100),
  [1512] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_condition_props, 4, 0, 102),
  [1514] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_condition_props, 4, 0, 102),
  [1516] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_lambda_bind_list, 5, 0, 97),
  [1518] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_lambda_bind_list, 5, 0, 97),
  [1520] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_lambda_bind_list, 3, 0, 58),
  [1522] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_lambda_bind_list, 3, 0, 58),
  [1524] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2373),
  [1526] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1838),
  [1528] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2430),
  [1530] = {.entry = {.count = 1, .reusable = true}}, SHIFT(872),
  [1532] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2278),
  [1534] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1546),
  [1536] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2404),
  [1538] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1559),
  [1540] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1551),
  [1542] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1806),
  [1544] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1558),
  [1546] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1550),
  [1548] = {.entry = {.count = 1, .reusable = true}}, SHIFT(871),
  [1550] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2449),
  [1552] = {.entry = {.count = 1, .reusable = true}}, SHIFT(870),
  [1554] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1829),
  [1556] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1563),
  [1558] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1931),
  [1560] = {.entry = {.count = 1, .reusable = true}}, SHIFT(857),
  [1562] = {.entry = {.count = 1, .reusable = false}}, SHIFT(30),
  [1564] = {.entry = {.count = 1, .reusable = false}}, SHIFT(752),
  [1566] = {.entry = {.count = 1, .reusable = false}}, SHIFT(41),
  [1568] = {.entry = {.count = 1, .reusable = false}}, SHIFT(42),
  [1570] = {.entry = {.count = 1, .reusable = false}}, SHIFT(49),
  [1572] = {.entry = {.count = 1, .reusable = false}}, SHIFT(39),
  [1574] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2384),
  [1576] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2017),
  [1578] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2286),
  [1580] = {.entry = {.count = 1, .reusable = false}}, SHIFT(40),
  [1582] = {.entry = {.count = 1, .reusable = false}}, SHIFT(225),
  [1584] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1399),
  [1586] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1430),
  [1588] = {.entry = {.count = 1, .reusable = false}}, SHIFT(32),
  [1590] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1618),
  [1592] = {.entry = {.count = 1, .reusable = true}}, SHIFT(849),
  [1594] = {.entry = {.count = 1, .reusable = true}}, SHIFT(979),
  [1596] = {.entry = {.count = 1, .reusable = true}}, SHIFT(858),
  [1598] = {.entry = {.count = 1, .reusable = true}}, SHIFT(851),
  [1600] = {.entry = {.count = 1, .reusable = true}}, SHIFT(980),
  [1602] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1603),
  [1604] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1603),
  [1606] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_access_specifier_list, 2, 0, 0),
  [1608] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_access_specifier_list, 2, 0, 0),
  [1610] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_access_specifier_list, 3, 0, 0),
  [1612] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_access_specifier_list, 3, 0, 0),
  [1614] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1593),
  [1616] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1175),
  [1618] = {.entry = {.count = 1, .reusable = true}}, SHIFT(911),
  [1620] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_expr_repeat1, 2, 0, 0), SHIFT_REPEAT(1593),
  [1623] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_expr_repeat1, 2, 0, 0), SHIFT_REPEAT(862),
  [1626] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_expr_repeat1, 2, 0, 0), SHIFT_REPEAT(863),
  [1629] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_expr_repeat1, 2, 0, 0), SHIFT_REPEAT(859),
  [1632] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_expr_repeat1, 2, 0, 0), SHIFT_REPEAT(861),
  [1635] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_expr_repeat1, 2, 0, 0), SHIFT_REPEAT(881),
  [1638] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_expr_repeat1, 2, 0, 0), SHIFT_REPEAT(2301),
  [1641] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_expr_repeat1, 2, 0, 0), SHIFT_REPEAT(2171),
  [1644] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_expr_repeat1, 2, 0, 0), SHIFT_REPEAT(2434),
  [1647] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_expr_repeat1, 2, 0, 0), SHIFT_REPEAT(1175),
  [1650] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_match_expr_repeat1, 2, 0, 0),
  [1652] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2154),
  [1654] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2103),
  [1656] = {.entry = {.count = 1, .reusable = true}}, SHIFT(956),
  [1658] = {.entry = {.count = 1, .reusable = true}}, SHIFT(853),
  [1660] = {.entry = {.count = 1, .reusable = true}}, SHIFT(855),
  [1662] = {.entry = {.count = 1, .reusable = true}}, SHIFT(847),
  [1664] = {.entry = {.count = 1, .reusable = true}}, SHIFT(856),
  [1666] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2123),
  [1668] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_spec_block_repeat2, 2, 0, 0), SHIFT_REPEAT(1931),
  [1671] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_spec_block_repeat2, 2, 0, 0),
  [1673] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_spec_block_repeat2, 2, 0, 0), SHIFT_REPEAT(30),
  [1676] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_spec_block_repeat2, 2, 0, 0), SHIFT_REPEAT(752),
  [1679] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_spec_block_repeat2, 2, 0, 0), SHIFT_REPEAT(41),
  [1682] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_spec_block_repeat2, 2, 0, 0), SHIFT_REPEAT(42),
  [1685] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_spec_block_repeat2, 2, 0, 0), SHIFT_REPEAT(49),
  [1688] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_spec_block_repeat2, 2, 0, 0), SHIFT_REPEAT(39),
  [1691] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_spec_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2384),
  [1694] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_spec_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2017),
  [1697] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_spec_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2286),
  [1700] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_spec_block_repeat2, 2, 0, 0), SHIFT_REPEAT(40),
  [1703] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_spec_block_repeat2, 2, 0, 0), SHIFT_REPEAT(225),
  [1706] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_spec_block_repeat2, 2, 0, 0), SHIFT_REPEAT(1399),
  [1709] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_spec_block_repeat2, 2, 0, 0), SHIFT_REPEAT(1430),
  [1712] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_spec_block_repeat2, 2, 0, 0), SHIFT_REPEAT(32),
  [1715] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2165),
  [1717] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2142),
  [1719] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2168),
  [1721] = {.entry = {.count = 1, .reusable = true}}, SHIFT(570),
  [1723] = {.entry = {.count = 1, .reusable = false}}, SHIFT(571),
  [1725] = {.entry = {.count = 1, .reusable = false}}, SHIFT(572),
  [1727] = {.entry = {.count = 1, .reusable = false}}, SHIFT(576),
  [1729] = {.entry = {.count = 1, .reusable = true}}, SHIFT(576),
  [1731] = {.entry = {.count = 1, .reusable = true}}, SHIFT(577),
  [1733] = {.entry = {.count = 1, .reusable = true}}, SHIFT(578),
  [1735] = {.entry = {.count = 1, .reusable = true}}, SHIFT(579),
  [1737] = {.entry = {.count = 1, .reusable = true}}, SHIFT(580),
  [1739] = {.entry = {.count = 1, .reusable = false}}, SHIFT(570),
  [1741] = {.entry = {.count = 1, .reusable = true}}, SHIFT(574),
  [1743] = {.entry = {.count = 1, .reusable = true}}, SHIFT(575),
  [1745] = {.entry = {.count = 1, .reusable = true}}, SHIFT(573),
  [1747] = {.entry = {.count = 1, .reusable = false}}, SHIFT(480),
  [1749] = {.entry = {.count = 1, .reusable = true}}, SHIFT(133),
  [1751] = {.entry = {.count = 1, .reusable = false}}, SHIFT(276),
  [1753] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2149),
  [1755] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2115),
  [1757] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2122),
  [1759] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2125),
  [1761] = {.entry = {.count = 1, .reusable = true}}, SHIFT(751),
  [1763] = {.entry = {.count = 1, .reusable = false}}, SHIFT(540),
  [1765] = {.entry = {.count = 1, .reusable = false}}, SHIFT(541),
  [1767] = {.entry = {.count = 1, .reusable = true}}, SHIFT(661),
  [1769] = {.entry = {.count = 1, .reusable = false}}, SHIFT(662),
  [1771] = {.entry = {.count = 1, .reusable = true}}, SHIFT(662),
  [1773] = {.entry = {.count = 1, .reusable = true}}, SHIFT(663),
  [1775] = {.entry = {.count = 1, .reusable = true}}, SHIFT(665),
  [1777] = {.entry = {.count = 1, .reusable = true}}, SHIFT(723),
  [1779] = {.entry = {.count = 1, .reusable = true}}, SHIFT(724),
  [1781] = {.entry = {.count = 1, .reusable = false}}, SHIFT(751),
  [1783] = {.entry = {.count = 1, .reusable = true}}, SHIFT(614),
  [1785] = {.entry = {.count = 1, .reusable = false}}, SHIFT(614),
  [1787] = {.entry = {.count = 1, .reusable = false}}, SHIFT(615),
  [1789] = {.entry = {.count = 1, .reusable = false}}, SHIFT(616),
  [1791] = {.entry = {.count = 1, .reusable = true}}, SHIFT(617),
  [1793] = {.entry = {.count = 1, .reusable = true}}, SHIFT(618),
  [1795] = {.entry = {.count = 1, .reusable = true}}, SHIFT(619),
  [1797] = {.entry = {.count = 1, .reusable = false}}, SHIFT(620),
  [1799] = {.entry = {.count = 1, .reusable = true}}, SHIFT(620),
  [1801] = {.entry = {.count = 1, .reusable = true}}, SHIFT(621),
  [1803] = {.entry = {.count = 1, .reusable = true}}, SHIFT(622),
  [1805] = {.entry = {.count = 1, .reusable = true}}, SHIFT(623),
  [1807] = {.entry = {.count = 1, .reusable = true}}, SHIFT(626),
  [1809] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_var, 1, 0, 0), SHIFT(1827),
  [1812] = {.entry = {.count = 1, .reusable = true}}, SHIFT(660),
  [1814] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2118),
  [1816] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2198),
  [1818] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1790),
  [1820] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2131),
  [1822] = {.entry = {.count = 1, .reusable = false}}, SHIFT(271),
  [1824] = {.entry = {.count = 1, .reusable = true}}, SHIFT(104),
  [1826] = {.entry = {.count = 1, .reusable = true}}, SHIFT(150),
  [1828] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2305),
  [1830] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1625),
  [1832] = {.entry = {.count = 1, .reusable = true}}, SHIFT(543),
  [1834] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2452),
  [1836] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1056),
  [1838] = {.entry = {.count = 1, .reusable = false}}, SHIFT(278),
  [1840] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1062),
  [1842] = {.entry = {.count = 1, .reusable = true}}, SHIFT(601),
  [1844] = {.entry = {.count = 1, .reusable = false}}, SHIFT(603),
  [1846] = {.entry = {.count = 1, .reusable = true}}, SHIFT(551),
  [1848] = {.entry = {.count = 1, .reusable = true}}, SHIFT(552),
  [1850] = {.entry = {.count = 1, .reusable = true}}, SHIFT(553),
  [1852] = {.entry = {.count = 1, .reusable = false}}, SHIFT(601),
  [1854] = {.entry = {.count = 1, .reusable = false}}, SHIFT(602),
  [1856] = {.entry = {.count = 1, .reusable = true}}, SHIFT(725),
  [1858] = {.entry = {.count = 1, .reusable = true}}, SHIFT(727),
  [1860] = {.entry = {.count = 1, .reusable = false}}, SHIFT(542),
  [1862] = {.entry = {.count = 1, .reusable = true}}, SHIFT(542),
  [1864] = {.entry = {.count = 1, .reusable = true}}, SHIFT(545),
  [1866] = {.entry = {.count = 1, .reusable = true}}, SHIFT(666),
  [1868] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1066),
  [1870] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1782),
  [1872] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1628),
  [1874] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1781),
  [1876] = {.entry = {.count = 1, .reusable = true}}, SHIFT(68),
  [1878] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1620),
  [1880] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1250),
  [1882] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1722),
  [1884] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1694),
  [1886] = {.entry = {.count = 1, .reusable = true}}, SHIFT(30),
  [1888] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2277),
  [1890] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1686),
  [1892] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1643),
  [1894] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1619),
  [1896] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2392),
  [1898] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2445),
  [1900] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2052),
  [1902] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2054),
  [1904] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_module_repeat1, 2, 0, 0), SHIFT_REPEAT(1250),
  [1907] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_module_repeat1, 2, 0, 0),
  [1909] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_module_repeat1, 2, 0, 0), SHIFT_REPEAT(2249),
  [1912] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_module_repeat1, 2, 0, 0), SHIFT_REPEAT(1694),
  [1915] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_module_repeat1, 2, 0, 0), SHIFT_REPEAT(30),
  [1918] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_module_repeat1, 2, 0, 0), SHIFT_REPEAT(2277),
  [1921] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_module_repeat1, 2, 0, 0), SHIFT_REPEAT(1686),
  [1924] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_module_repeat1, 2, 0, 0), SHIFT_REPEAT(1643),
  [1927] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_module_repeat1, 2, 0, 0), SHIFT_REPEAT(1619),
  [1930] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_module_repeat1, 2, 0, 0), SHIFT_REPEAT(2392),
  [1933] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_module_repeat1, 2, 0, 0), SHIFT_REPEAT(2445),
  [1936] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_module_repeat1, 2, 0, 0), SHIFT_REPEAT(2052),
  [1939] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_module_repeat1, 2, 0, 0), SHIFT_REPEAT(2054),
  [1942] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1708),
  [1944] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2458),
  [1946] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1670),
  [1948] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_var, 2, 0, 33), SHIFT(1798),
  [1951] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_invariant, 5, 0, 62),
  [1953] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_invariant, 5, 0, 62),
  [1955] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_invariant, 4, 0, 70),
  [1957] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_invariant, 4, 0, 70),
  [1959] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_invariant, 5, 0, 107),
  [1961] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_invariant, 5, 0, 107),
  [1963] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_invariant, 5, 0, 70),
  [1965] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_invariant, 5, 0, 70),
  [1967] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_invariant, 4, 0, 62),
  [1969] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_invariant, 4, 0, 62),
  [1971] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_invariant, 4, 0, 0),
  [1973] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_invariant, 4, 0, 0),
  [1975] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_invariant, 6, 0, 107),
  [1977] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_invariant, 6, 0, 107),
  [1979] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_invariant, 3, 0, 0),
  [1981] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_invariant, 3, 0, 0),
  [1983] = {.entry = {.count = 1, .reusable = false}}, SHIFT(375),
  [1985] = {.entry = {.count = 1, .reusable = true}}, SHIFT(82),
  [1987] = {.entry = {.count = 1, .reusable = true}}, SHIFT(628),
  [1989] = {.entry = {.count = 1, .reusable = false}}, SHIFT(629),
  [1991] = {.entry = {.count = 1, .reusable = false}}, SHIFT(630),
  [1993] = {.entry = {.count = 1, .reusable = true}}, SHIFT(635),
  [1995] = {.entry = {.count = 1, .reusable = true}}, SHIFT(636),
  [1997] = {.entry = {.count = 1, .reusable = true}}, SHIFT(637),
  [1999] = {.entry = {.count = 1, .reusable = true}}, SHIFT(638),
  [2001] = {.entry = {.count = 1, .reusable = false}}, SHIFT(628),
  [2003] = {.entry = {.count = 1, .reusable = true}}, SHIFT(632),
  [2005] = {.entry = {.count = 1, .reusable = true}}, SHIFT(633),
  [2007] = {.entry = {.count = 1, .reusable = false}}, SHIFT(634),
  [2009] = {.entry = {.count = 1, .reusable = true}}, SHIFT(634),
  [2011] = {.entry = {.count = 1, .reusable = true}}, SHIFT(555),
  [2013] = {.entry = {.count = 1, .reusable = false}}, SHIFT(557),
  [2015] = {.entry = {.count = 1, .reusable = true}}, SHIFT(563),
  [2017] = {.entry = {.count = 1, .reusable = true}}, SHIFT(564),
  [2019] = {.entry = {.count = 1, .reusable = true}}, SHIFT(565),
  [2021] = {.entry = {.count = 1, .reusable = false}}, SHIFT(555),
  [2023] = {.entry = {.count = 1, .reusable = false}}, SHIFT(556),
  [2025] = {.entry = {.count = 1, .reusable = true}}, SHIFT(559),
  [2027] = {.entry = {.count = 1, .reusable = true}}, SHIFT(560),
  [2029] = {.entry = {.count = 1, .reusable = false}}, SHIFT(561),
  [2031] = {.entry = {.count = 1, .reusable = true}}, SHIFT(561),
  [2033] = {.entry = {.count = 1, .reusable = true}}, SHIFT(562),
  [2035] = {.entry = {.count = 1, .reusable = true}}, SHIFT(558),
  [2037] = {.entry = {.count = 1, .reusable = true}}, SHIFT(131),
  [2039] = {.entry = {.count = 1, .reusable = false}}, SHIFT(501),
  [2041] = {.entry = {.count = 1, .reusable = false}}, SHIFT(420),
  [2043] = {.entry = {.count = 1, .reusable = true}}, SHIFT(631),
  [2045] = {.entry = {.count = 1, .reusable = false}}, SHIFT(233),
  [2047] = {.entry = {.count = 1, .reusable = true}}, SHIFT(693),
  [2049] = {.entry = {.count = 1, .reusable = false}}, SHIFT(694),
  [2051] = {.entry = {.count = 1, .reusable = false}}, SHIFT(695),
  [2053] = {.entry = {.count = 1, .reusable = true}}, SHIFT(698),
  [2055] = {.entry = {.count = 1, .reusable = false}}, SHIFT(699),
  [2057] = {.entry = {.count = 1, .reusable = true}}, SHIFT(699),
  [2059] = {.entry = {.count = 1, .reusable = true}}, SHIFT(700),
  [2061] = {.entry = {.count = 1, .reusable = true}}, SHIFT(701),
  [2063] = {.entry = {.count = 1, .reusable = true}}, SHIFT(702),
  [2065] = {.entry = {.count = 1, .reusable = true}}, SHIFT(703),
  [2067] = {.entry = {.count = 1, .reusable = false}}, SHIFT(693),
  [2069] = {.entry = {.count = 1, .reusable = true}}, SHIFT(108),
  [2071] = {.entry = {.count = 1, .reusable = true}}, SHIFT(642),
  [2073] = {.entry = {.count = 1, .reusable = false}}, SHIFT(644),
  [2075] = {.entry = {.count = 1, .reusable = true}}, SHIFT(650),
  [2077] = {.entry = {.count = 1, .reusable = true}}, SHIFT(651),
  [2079] = {.entry = {.count = 1, .reusable = true}}, SHIFT(656),
  [2081] = {.entry = {.count = 1, .reusable = false}}, SHIFT(642),
  [2083] = {.entry = {.count = 1, .reusable = false}}, SHIFT(643),
  [2085] = {.entry = {.count = 1, .reusable = true}}, SHIFT(646),
  [2087] = {.entry = {.count = 1, .reusable = true}}, SHIFT(647),
  [2089] = {.entry = {.count = 1, .reusable = false}}, SHIFT(648),
  [2091] = {.entry = {.count = 1, .reusable = true}}, SHIFT(648),
  [2093] = {.entry = {.count = 1, .reusable = true}}, SHIFT(649),
  [2095] = {.entry = {.count = 1, .reusable = true}}, SHIFT(645),
  [2097] = {.entry = {.count = 1, .reusable = true}}, SHIFT(697),
  [2099] = {.entry = {.count = 1, .reusable = true}}, SHIFT(129),
  [2101] = {.entry = {.count = 1, .reusable = true}}, SHIFT(696),
  [2103] = {.entry = {.count = 1, .reusable = false}}, SHIFT(360),
  [2105] = {.entry = {.count = 1, .reusable = true}}, SHIFT(583),
  [2107] = {.entry = {.count = 1, .reusable = false}}, SHIFT(585),
  [2109] = {.entry = {.count = 1, .reusable = true}}, SHIFT(591),
  [2111] = {.entry = {.count = 1, .reusable = true}}, SHIFT(592),
  [2113] = {.entry = {.count = 1, .reusable = true}}, SHIFT(593),
  [2115] = {.entry = {.count = 1, .reusable = false}}, SHIFT(583),
  [2117] = {.entry = {.count = 1, .reusable = false}}, SHIFT(584),
  [2119] = {.entry = {.count = 1, .reusable = true}}, SHIFT(587),
  [2121] = {.entry = {.count = 1, .reusable = true}}, SHIFT(588),
  [2123] = {.entry = {.count = 1, .reusable = false}}, SHIFT(589),
  [2125] = {.entry = {.count = 1, .reusable = true}}, SHIFT(589),
  [2127] = {.entry = {.count = 1, .reusable = true}}, SHIFT(590),
  [2129] = {.entry = {.count = 1, .reusable = false}}, SHIFT(394),
  [2131] = {.entry = {.count = 1, .reusable = true}}, SHIFT(144),
  [2133] = {.entry = {.count = 1, .reusable = true}}, SHIFT(586),
  [2135] = {.entry = {.count = 1, .reusable = true}}, SHIFT(671),
  [2137] = {.entry = {.count = 1, .reusable = false}}, SHIFT(673),
  [2139] = {.entry = {.count = 1, .reusable = true}}, SHIFT(679),
  [2141] = {.entry = {.count = 1, .reusable = true}}, SHIFT(680),
  [2143] = {.entry = {.count = 1, .reusable = true}}, SHIFT(681),
  [2145] = {.entry = {.count = 1, .reusable = false}}, SHIFT(671),
  [2147] = {.entry = {.count = 1, .reusable = false}}, SHIFT(672),
  [2149] = {.entry = {.count = 1, .reusable = true}}, SHIFT(675),
  [2151] = {.entry = {.count = 1, .reusable = true}}, SHIFT(676),
  [2153] = {.entry = {.count = 1, .reusable = false}}, SHIFT(677),
  [2155] = {.entry = {.count = 1, .reusable = true}}, SHIFT(677),
  [2157] = {.entry = {.count = 1, .reusable = true}}, SHIFT(678),
  [2159] = {.entry = {.count = 1, .reusable = true}}, SHIFT(674),
  [2161] = {.entry = {.count = 1, .reusable = true}}, SHIFT(62),
  [2163] = {.entry = {.count = 1, .reusable = false}}, SHIFT(444),
  [2165] = {.entry = {.count = 1, .reusable = false}}, SHIFT(178),
  [2167] = {.entry = {.count = 1, .reusable = true}}, SHIFT(652),
  [2169] = {.entry = {.count = 1, .reusable = true}}, SHIFT(538),
  [2171] = {.entry = {.count = 1, .reusable = true}}, SHIFT(539),
  [2173] = {.entry = {.count = 1, .reusable = false}}, SHIFT(652),
  [2175] = {.entry = {.count = 1, .reusable = true}}, SHIFT(544),
  [2177] = {.entry = {.count = 1, .reusable = false}}, SHIFT(612),
  [2179] = {.entry = {.count = 1, .reusable = true}}, SHIFT(657),
  [2181] = {.entry = {.count = 1, .reusable = true}}, SHIFT(658),
  [2183] = {.entry = {.count = 1, .reusable = true}}, SHIFT(659),
  [2185] = {.entry = {.count = 1, .reusable = false}}, SHIFT(544),
  [2187] = {.entry = {.count = 1, .reusable = false}}, SHIFT(611),
  [2189] = {.entry = {.count = 1, .reusable = true}}, SHIFT(627),
  [2191] = {.entry = {.count = 1, .reusable = true}}, SHIFT(639),
  [2193] = {.entry = {.count = 1, .reusable = false}}, SHIFT(640),
  [2195] = {.entry = {.count = 1, .reusable = true}}, SHIFT(640),
  [2197] = {.entry = {.count = 1, .reusable = true}}, SHIFT(641),
  [2199] = {.entry = {.count = 1, .reusable = false}}, SHIFT(726),
  [2201] = {.entry = {.count = 1, .reusable = true}}, SHIFT(537),
  [2203] = {.entry = {.count = 1, .reusable = false}}, SHIFT(664),
  [2205] = {.entry = {.count = 1, .reusable = true}}, SHIFT(533),
  [2207] = {.entry = {.count = 1, .reusable = true}}, SHIFT(534),
  [2209] = {.entry = {.count = 1, .reusable = false}}, SHIFT(535),
  [2211] = {.entry = {.count = 1, .reusable = true}}, SHIFT(535),
  [2213] = {.entry = {.count = 1, .reusable = true}}, SHIFT(536),
  [2215] = {.entry = {.count = 1, .reusable = true}}, SHIFT(739),
  [2217] = {.entry = {.count = 1, .reusable = true}}, SHIFT(613),
  [2219] = {.entry = {.count = 1, .reusable = true}}, SHIFT(728),
  [2221] = {.entry = {.count = 1, .reusable = false}}, SHIFT(730),
  [2223] = {.entry = {.count = 1, .reusable = true}}, SHIFT(736),
  [2225] = {.entry = {.count = 1, .reusable = true}}, SHIFT(737),
  [2227] = {.entry = {.count = 1, .reusable = true}}, SHIFT(738),
  [2229] = {.entry = {.count = 1, .reusable = false}}, SHIFT(728),
  [2231] = {.entry = {.count = 1, .reusable = false}}, SHIFT(729),
  [2233] = {.entry = {.count = 1, .reusable = true}}, SHIFT(732),
  [2235] = {.entry = {.count = 1, .reusable = true}}, SHIFT(733),
  [2237] = {.entry = {.count = 1, .reusable = false}}, SHIFT(734),
  [2239] = {.entry = {.count = 1, .reusable = true}}, SHIFT(734),
  [2241] = {.entry = {.count = 1, .reusable = true}}, SHIFT(735),
  [2243] = {.entry = {.count = 1, .reusable = false}}, SHIFT(433),
  [2245] = {.entry = {.count = 1, .reusable = true}}, SHIFT(140),
  [2247] = {.entry = {.count = 1, .reusable = true}}, SHIFT(731),
  [2249] = {.entry = {.count = 1, .reusable = false}}, SHIFT(249),
  [2251] = {.entry = {.count = 1, .reusable = true}}, SHIFT(88),
  [2253] = {.entry = {.count = 1, .reusable = true}}, SHIFT(92),
  [2255] = {.entry = {.count = 1, .reusable = true}}, SHIFT(704),
  [2257] = {.entry = {.count = 1, .reusable = false}}, SHIFT(705),
  [2259] = {.entry = {.count = 1, .reusable = false}}, SHIFT(706),
  [2261] = {.entry = {.count = 1, .reusable = true}}, SHIFT(709),
  [2263] = {.entry = {.count = 1, .reusable = false}}, SHIFT(710),
  [2265] = {.entry = {.count = 1, .reusable = true}}, SHIFT(710),
  [2267] = {.entry = {.count = 1, .reusable = true}}, SHIFT(711),
  [2269] = {.entry = {.count = 1, .reusable = true}}, SHIFT(712),
  [2271] = {.entry = {.count = 1, .reusable = true}}, SHIFT(713),
  [2273] = {.entry = {.count = 1, .reusable = true}}, SHIFT(714),
  [2275] = {.entry = {.count = 1, .reusable = false}}, SHIFT(704),
  [2277] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1713),
  [2279] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2202),
  [2281] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1057),
  [2283] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1673),
  [2285] = {.entry = {.count = 1, .reusable = true}}, SHIFT(670),
  [2287] = {.entry = {.count = 1, .reusable = false}}, SHIFT(683),
  [2289] = {.entry = {.count = 1, .reusable = true}}, SHIFT(690),
  [2291] = {.entry = {.count = 1, .reusable = true}}, SHIFT(691),
  [2293] = {.entry = {.count = 1, .reusable = true}}, SHIFT(692),
  [2295] = {.entry = {.count = 1, .reusable = false}}, SHIFT(670),
  [2297] = {.entry = {.count = 1, .reusable = false}}, SHIFT(682),
  [2299] = {.entry = {.count = 1, .reusable = true}}, SHIFT(685),
  [2301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(687),
  [2303] = {.entry = {.count = 1, .reusable = false}}, SHIFT(688),
  [2305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(688),
  [2307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(689),
  [2309] = {.entry = {.count = 1, .reusable = false}}, SHIFT(407),
  [2311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(707),
  [2313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(708),
  [2315] = {.entry = {.count = 1, .reusable = false}}, SHIFT(213),
  [2317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(684),
  [2319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(112),
  [2321] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2427),
  [2323] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1462),
  [2325] = {.entry = {.count = 1, .reusable = true}}, SHIFT(72),
  [2327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2308),
  [2329] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1634),
  [2331] = {.entry = {.count = 1, .reusable = true}}, SHIFT(740),
  [2333] = {.entry = {.count = 1, .reusable = false}}, SHIFT(741),
  [2335] = {.entry = {.count = 1, .reusable = false}}, SHIFT(742),
  [2337] = {.entry = {.count = 1, .reusable = true}}, SHIFT(743),
  [2339] = {.entry = {.count = 1, .reusable = true}}, SHIFT(744),
  [2341] = {.entry = {.count = 1, .reusable = true}}, SHIFT(745),
  [2343] = {.entry = {.count = 1, .reusable = false}}, SHIFT(746),
  [2345] = {.entry = {.count = 1, .reusable = true}}, SHIFT(746),
  [2347] = {.entry = {.count = 1, .reusable = true}}, SHIFT(747),
  [2349] = {.entry = {.count = 1, .reusable = true}}, SHIFT(748),
  [2351] = {.entry = {.count = 1, .reusable = true}}, SHIFT(749),
  [2353] = {.entry = {.count = 1, .reusable = true}}, SHIFT(750),
  [2355] = {.entry = {.count = 1, .reusable = false}}, SHIFT(740),
  [2357] = {.entry = {.count = 1, .reusable = true}}, SHIFT(96),
  [2359] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1649),
  [2361] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1506),
  [2363] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1053),
  [2365] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1623),
  [2367] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1447),
  [2369] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1064),
  [2371] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_match_arm, 5, 0, 218),
  [2373] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_arm, 5, 0, 218),
  [2375] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_match_arm, 3, 0, 195),
  [2377] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_arm, 3, 0, 195),
  [2379] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_match_expr_repeat1, 1, 0, 0),
  [2381] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_match_expr_repeat1, 1, 0, 0),
  [2383] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1431),
  [2385] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1432),
  [2387] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_match_expr_repeat1, 2, 0, 0),
  [2389] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_spec_block_repeat1, 2, 0, 0), SHIFT_REPEAT(1618),
  [2392] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 7, 0, 176),
  [2394] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 7, 0, 176),
  [2396] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 4, 0, 85),
  [2398] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 4, 0, 85),
  [2400] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_pragma, 4, 0, 123),
  [2402] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_pragma, 4, 0, 123),
  [2404] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_pragma, 4, 0, 124),
  [2406] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_pragma, 4, 0, 124),
  [2408] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_axiom, 4, 0, 125),
  [2410] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_axiom, 4, 0, 125),
  [2412] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_axiom, 4, 0, 126),
  [2414] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_axiom, 4, 0, 126),
  [2416] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_variable, 5, 0, 147),
  [2418] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_variable, 5, 0, 147),
  [2420] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_spec_block_repeat2, 1, 0, 0),
  [2422] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_spec_block_repeat2, 1, 0, 0),
  [2424] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__spec_block_member, 1, 0, 0),
  [2426] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__spec_block_member, 1, 0, 0),
  [2428] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_variable, 6, 0, 169),
  [2430] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_variable, 6, 0, 169),
  [2432] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_variable, 6, 0, 147),
  [2434] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_variable, 6, 0, 147),
  [2436] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 6, 0, 148),
  [2438] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 6, 0, 148),
  [2440] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 6, 0, 171),
  [2442] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 6, 0, 171),
  [2444] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 6, 0, 172),
  [2446] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 6, 0, 172),
  [2448] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 6, 0, 174),
  [2450] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 6, 0, 174),
  [2452] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_include, 3, 0, 0),
  [2454] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_include, 3, 0, 0),
  [2456] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_pragma, 3, 0, 83),
  [2458] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_pragma, 3, 0, 83),
  [2460] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_pragma, 2, 0, 0),
  [2462] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_pragma, 2, 0, 0),
  [2464] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_pragma, 3, 0, 84),
  [2466] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_pragma, 3, 0, 84),
  [2468] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_axiom, 3, 0, 85),
  [2470] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_axiom, 3, 0, 85),
  [2472] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 6, 0, 150),
  [2474] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 6, 0, 150),
  [2476] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 6, 0, 176),
  [2478] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 6, 0, 176),
  [2480] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_let, 6, 0, 178),
  [2482] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_let, 6, 0, 178),
  [2484] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_variable, 7, 0, 184),
  [2486] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_variable, 7, 0, 184),
  [2488] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_variable, 7, 0, 186),
  [2490] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_variable, 7, 0, 186),
  [2492] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 7, 0, 187),
  [2494] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 7, 0, 187),
  [2496] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 7, 0, 188),
  [2498] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 7, 0, 188),
  [2500] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 7, 0, 189),
  [2502] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 7, 0, 189),
  [2504] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 7, 0, 190),
  [2506] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 7, 0, 190),
  [2508] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 7, 0, 174),
  [2510] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 7, 0, 174),
  [2512] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 7, 0, 191),
  [2514] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 7, 0, 191),
  [2516] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 7, 0, 192),
  [2518] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 7, 0, 192),
  [2520] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 7, 0, 193),
  [2522] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 7, 0, 193),
  [2524] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 5, 0, 148),
  [2526] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 5, 0, 148),
  [2528] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_variable, 8, 0, 197),
  [2530] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_variable, 8, 0, 197),
  [2532] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 8, 0, 198),
  [2534] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 8, 0, 198),
  [2536] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 8, 0, 199),
  [2538] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 8, 0, 199),
  [2540] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 8, 0, 200),
  [2542] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 8, 0, 200),
  [2544] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 8, 0, 201),
  [2546] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 8, 0, 201),
  [2548] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_variable, 5, 0, 24),
  [2550] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_variable, 5, 0, 24),
  [2552] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 8, 0, 203),
  [2554] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 8, 0, 203),
  [2556] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 8, 0, 204),
  [2558] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 8, 0, 204),
  [2560] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 8, 0, 193),
  [2562] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 8, 0, 193),
  [2564] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 8, 0, 205),
  [2566] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 8, 0, 205),
  [2568] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 8, 0, 206),
  [2570] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 8, 0, 206),
  [2572] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 9, 0, 209),
  [2574] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 9, 0, 209),
  [2576] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 9, 0, 210),
  [2578] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 9, 0, 210),
  [2580] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 9, 0, 211),
  [2582] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 9, 0, 211),
  [2584] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 9, 0, 212),
  [2586] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 9, 0, 212),
  [2588] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 9, 0, 213),
  [2590] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 9, 0, 213),
  [2592] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 9, 0, 214),
  [2594] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 9, 0, 214),
  [2596] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 9, 0, 215),
  [2598] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 9, 0, 215),
  [2600] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 9, 0, 216),
  [2602] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 9, 0, 216),
  [2604] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 10, 0, 219),
  [2606] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 10, 0, 219),
  [2608] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 10, 0, 220),
  [2610] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 10, 0, 220),
  [2612] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 10, 0, 221),
  [2614] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 10, 0, 221),
  [2616] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 10, 0, 222),
  [2618] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 10, 0, 222),
  [2620] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 11, 0, 223),
  [2622] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 11, 0, 223),
  [2624] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 5, 0, 85),
  [2626] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 5, 0, 85),
  [2628] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 5, 0, 150),
  [2630] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 5, 0, 150),
  [2632] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_pragma, 5, 0, 152),
  [2634] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_pragma, 5, 0, 152),
  [2636] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_condition, 2, 0, 0),
  [2638] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_condition, 2, 0, 0),
  [2640] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_update, 3, 0, 0),
  [2642] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_update, 3, 0, 0),
  [2644] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_let, 5, 0, 153),
  [2646] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_let, 5, 0, 153),
  [2648] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_axiom, 5, 0, 154),
  [2650] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_axiom, 5, 0, 154),
  [2652] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_variable, 4, 0, 24),
  [2654] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_variable, 4, 0, 24),
  [2656] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_include, 4, 0, 0),
  [2658] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_include, 4, 0, 0),
  [2660] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_spec_apply, 8, 0, 202),
  [2662] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_apply, 8, 0, 202),
  [2664] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_body, 2, 0, 0),
  [2666] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_body, 2, 0, 0),
  [2668] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_body, 3, 0, 0),
  [2670] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_body, 3, 0, 0),
  [2672] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_discouraged_name, 1, 0, 0), REDUCE(sym__type, 1, 0, 0),
  [2675] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_discouraged_name, 1, 0, 0), REDUCE(sym__type, 1, 0, 0),
  [2678] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_discouraged_name, 1, 0, 0), SHIFT(1071),
  [2681] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_body, 5, 0, 0),
  [2683] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_body, 5, 0, 0),
  [2685] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2369),
  [2687] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_body, 4, 0, 0),
  [2689] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_body, 4, 0, 0),
  [2691] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2284),
  [2693] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type, 1, 0, 0),
  [2695] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1071),
  [2697] = {.entry = {.count = 1, .reusable = true}}, SHIFT(862),
  [2699] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_attributes_repeat2, 2, 0, 0),
  [2701] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributes_repeat2, 2, 0, 0), SHIFT_REPEAT(2249),
  [2704] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attributes, 1, 0, 0),
  [2706] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2239),
  [2708] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_attributes_repeat2, 3, 0, 0),
  [2710] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_attributes_repeat2, 4, 0, 0),
  [2712] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1117),
  [2714] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_type, 5, 0, 164),
  [2716] = {.entry = {.count = 1, .reusable = true}}, SHIFT(976),
  [2718] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_type, 6, 0, 183),
  [2720] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_type, 3, 0, 113),
  [2722] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_attributes_repeat2, 6, 0, 0),
  [2724] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_attributes_repeat2, 5, 0, 0),
  [2726] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_body, 2, 0, 0),
  [2728] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_body, 4, 0, 0),
  [2730] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_body, 3, 0, 0),
  [2732] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_discouraged_name, 1, 0, 0), SHIFT(1069),
  [2735] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_type, 4, 0, 140),
  [2737] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type, 1, 0, 41),
  [2739] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__ref_type, 2, 0, 77),
  [2741] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_type, 5, 0, 0),
  [2743] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_type, 2, 0, 0),
  [2745] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type, 2, 0, 0),
  [2747] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_type, 3, 0, 0),
  [2749] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_type, 4, 0, 0),
  [2751] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1069),
  [2753] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type, 1, 0, 0),
  [2755] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__ref_type, 2, 0, 76),
  [2757] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_decl, 4, 0, 14),
  [2759] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_decl, 2, 0, 9),
  [2761] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_module_repeat1, 1, 0, 0),
  [2763] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_declaration, 1, 0, 0),
  [2765] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_declaration, 3, 0, 0),
  [2767] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_decl, 4, 0, 14),
  [2769] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_decl, 5, 0, 14),
  [2771] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_decl, 2, 0, 16),
  [2773] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__struct_or_enum_decl, 1, 0, 0),
  [2775] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1187),
  [2777] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__bind, 1, 0, 25),
  [2779] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__bind, 1, 0, 25),
  [2781] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1401),
  [2783] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_decl, 5, 0, 14),
  [2785] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_friend_decl, 3, 0, 19),
  [2787] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_decl, 2, 0, 15),
  [2789] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_declaration, 2, 0, 0),
  [2791] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_decl, 2, 0, 15),
  [2793] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_decl, 2, 0, 8),
  [2795] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_constant_decl, 7, 0, 142),
  [2797] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__bind, 2, 0, 25),
  [2799] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__bind, 2, 0, 25),
  [2801] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0),
  [2803] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1655),
  [2806] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2249),
  [2809] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1775),
  [2812] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1646),
  [2815] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2314),
  [2818] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_source_file, 1, 0, 0),
  [2820] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_var_name, 1, 0, 0),
  [2822] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_var_name, 1, 0, 0),
  [2824] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_script_repeat1, 2, 0, 0), SHIFT_REPEAT(2249),
  [2827] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_script_repeat1, 2, 0, 0),
  [2829] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_script_repeat1, 2, 0, 0), SHIFT_REPEAT(1619),
  [2832] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_script_repeat2, 2, 0, 0), SHIFT_REPEAT(2249),
  [2835] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_script_repeat2, 2, 0, 0),
  [2837] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_script_repeat2, 2, 0, 0), SHIFT_REPEAT(2392),
  [2840] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_declaration_repeat1, 2, 0, 0),
  [2842] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_declaration_repeat1, 2, 0, 0), SHIFT_REPEAT(1686),
  [2845] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_declaration_repeat1, 2, 0, 0), SHIFT_REPEAT(1643),
  [2848] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1753),
  [2850] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1763),
  [2852] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1763),
  [2854] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2064),
  [2856] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1446),
  [2858] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1635),
  [2860] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1800),
  [2862] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__leading_name_access_wildcard, 1, 0, 0),
  [2864] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_name_access_chain_wildcard, 1, 0, 1),
  [2866] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2050),
  [2868] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2076),
  [2870] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_name_access_chain_wildcard, 5, 0, 118),
  [2872] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_address_block_repeat1, 2, 0, 0),
  [2874] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_address_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2249),
  [2877] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_address_block_repeat1, 2, 0, 0), SHIFT_REPEAT(1775),
  [2880] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_address_block_repeat1, 2, 0, 0), SHIFT_REPEAT(1646),
  [2883] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1100),
  [2885] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__function_signature, 4, 0, 38),
  [2887] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1978),
  [2889] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2147),
  [2891] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1228),
  [2893] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1113),
  [2895] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__function_signature, 3, 0, 17),
  [2897] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1232),
  [2899] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_access_specifier, 1, 0, 0),
  [2901] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2020),
  [2903] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_name_access_chain_wildcard, 3, 0, 45),
  [2905] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1768),
  [2907] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1115),
  [2909] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__function_signature, 5, 0, 79),
  [2911] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1109),
  [2913] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__function_signature, 4, 0, 42),
  [2915] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2272),
  [2917] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1465),
  [2919] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__bind_fields, 3, 0, 0),
  [2921] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__bind_fields, 3, 0, 0),
  [2923] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__bind_tuple, 3, 0, 58),
  [2925] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__bind_tuple, 3, 0, 58),
  [2927] = {.entry = {.count = 1, .reusable = true}}, SHIFT(883),
  [2929] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__bind_tuple, 5, 0, 97),
  [2931] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__bind_tuple, 5, 0, 97),
  [2933] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__spec_apply_pattern, 2, 0, 149),
  [2935] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__spec_apply_pattern, 2, 0, 149),
  [2937] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__function_signature, 6, 0, 114),
  [2939] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__bind_fields, 5, 0, 0),
  [2941] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__bind_fields, 5, 0, 0),
  [2943] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__bind, 2, 0, 57),
  [2945] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__bind, 2, 0, 57),
  [2947] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__bind_tuple, 2, 0, 0),
  [2949] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__bind_tuple, 2, 0, 0),
  [2951] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__bind, 3, 0, 96),
  [2953] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__bind, 3, 0, 96),
  [2955] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__bind_tuple, 4, 0, 58),
  [2957] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__bind_tuple, 4, 0, 58),
  [2959] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2188),
  [2961] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1445),
  [2963] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__bind_tuple, 3, 0, 0),
  [2965] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__bind_tuple, 3, 0, 0),
  [2967] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__function_signature, 7, 0, 144),
  [2969] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1459),
  [2971] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1645),
  [2973] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__bind_fields, 2, 0, 0),
  [2975] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__bind_fields, 2, 0, 0),
  [2977] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2348),
  [2979] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1503),
  [2981] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2233),
  [2983] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1487),
  [2985] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__spec_apply_pattern, 1, 0, 122),
  [2987] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__spec_apply_pattern, 1, 0, 122),
  [2989] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2211),
  [2991] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1456),
  [2993] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__function_signature, 5, 0, 72),
  [2995] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__bind, 3, 0, 25),
  [2997] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__bind, 3, 0, 25),
  [2999] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__bind, 1, 0, 24),
  [3001] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__bind, 1, 0, 24),
  [3003] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__function_signature, 6, 0, 111),
  [3005] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2001),
  [3007] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_visibility, 1, 0, 0),
  [3009] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_name_access_chain_wildcard, 7, 0, 168),
  [3011] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2450),
  [3013] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1474),
  [3015] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2073),
  [3017] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__bind_tuple, 4, 0, 97),
  [3019] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__bind_tuple, 4, 0, 97),
  [3021] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1477),
  [3023] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1636),
  [3025] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__bind_fields, 4, 0, 0),
  [3027] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__bind_fields, 4, 0, 0),
  [3029] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__spec_apply_pattern_repeat1, 2, 0, 0), SHIFT_REPEAT(1763),
  [3032] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__spec_apply_pattern_repeat1, 2, 0, 0), SHIFT_REPEAT(1763),
  [3035] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__spec_apply_pattern_repeat1, 2, 0, 0),
  [3037] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym__spec_apply_pattern_repeat1, 2, 0, 0),
  [3039] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_script_repeat3, 2, 0, 0),
  [3041] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_script_repeat3, 2, 0, 0), SHIFT_REPEAT(2249),
  [3044] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_script_repeat3, 2, 0, 0), SHIFT_REPEAT(1688),
  [3047] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1762),
  [3049] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1688),
  [3051] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 4, 0, 0),
  [3053] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1236),
  [3055] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_access_specifier, 2, 0, 0),
  [3057] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1119),
  [3059] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__script_use_decl, 1, 0, 0),
  [3061] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_script_repeat1, 1, 0, 0),
  [3063] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1757),
  [3065] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 5, 0, 0),
  [3067] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_access_specifier_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1238),
  [3070] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_access_specifier_list_repeat1, 2, 0, 0),
  [3072] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1765),
  [3074] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 3, 0, 0),
  [3076] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1118),
  [3078] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_access_specifier_list, 1, 0, 0),
  [3080] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1755),
  [3082] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1110),
  [3084] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2187),
  [3086] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2214),
  [3088] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2116),
  [3090] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2185),
  [3092] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_declaration_repeat1, 1, 0, 0),
  [3094] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1472),
  [3096] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2170),
  [3098] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1480),
  [3100] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1482),
  [3102] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1761),
  [3104] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1548),
  [3106] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1485),
  [3108] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1490),
  [3110] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1491),
  [3112] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1493),
  [3114] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1496),
  [3116] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1498),
  [3118] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1499),
  [3120] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1501),
  [3122] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1502),
  [3124] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_module_member_modifier, 1, 0, 0),
  [3126] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__script_use_decl, 2, 0, 0),
  [3128] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1553),
  [3130] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 2, 0, 0),
  [3132] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1760),
  [3134] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_visibility, 4, 0, 0),
  [3136] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2003),
  [3138] = {.entry = {.count = 1, .reusable = true}}, SHIFT(336),
  [3140] = {.entry = {.count = 1, .reusable = true}}, SHIFT(253),
  [3142] = {.entry = {.count = 1, .reusable = true}}, SHIFT(44),
  [3144] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_enum_body_repeat1, 2, 0, 0), SHIFT_REPEAT(1761),
  [3147] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_enum_body_repeat1, 2, 0, 0),
  [3149] = {.entry = {.count = 1, .reusable = true}}, SHIFT(342),
  [3151] = {.entry = {.count = 1, .reusable = true}}, SHIFT(297),
  [3153] = {.entry = {.count = 1, .reusable = true}}, SHIFT(344),
  [3155] = {.entry = {.count = 1, .reusable = true}}, SHIFT(300),
  [3157] = {.entry = {.count = 1, .reusable = true}}, SHIFT(350),
  [3159] = {.entry = {.count = 1, .reusable = true}}, SHIFT(312),
  [3161] = {.entry = {.count = 1, .reusable = true}}, SHIFT(352),
  [3163] = {.entry = {.count = 1, .reusable = true}}, SHIFT(315),
  [3165] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2250),
  [3167] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__attribute_name, 2, 0, 5),
  [3169] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__specifier_repeat1, 2, 0, 73),
  [3171] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__specifier_repeat1, 2, 0, 73), SHIFT_REPEAT(1978),
  [3174] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__specifier_repeat1, 2, 0, 73), SHIFT_REPEAT(1228),
  [3177] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__script_constant_decl, 1, 0, 0),
  [3179] = {.entry = {.count = 1, .reusable = true}}, SHIFT(517),
  [3181] = {.entry = {.count = 1, .reusable = true}}, SHIFT(280),
  [3183] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_access_specifier, 4, 0, 162),
  [3185] = {.entry = {.count = 1, .reusable = true}}, SHIFT(211),
  [3187] = {.entry = {.count = 1, .reusable = true}}, SHIFT(212),
  [3189] = {.entry = {.count = 1, .reusable = true}}, SHIFT(519),
  [3191] = {.entry = {.count = 1, .reusable = true}}, SHIFT(285),
  [3193] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_module, 4, 0, 7),
  [3195] = {.entry = {.count = 1, .reusable = true}}, SHIFT(326),
  [3197] = {.entry = {.count = 1, .reusable = true}}, SHIFT(214),
  [3199] = {.entry = {.count = 1, .reusable = true}}, SHIFT(328),
  [3201] = {.entry = {.count = 1, .reusable = true}}, SHIFT(217),
  [3203] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__attribute_name, 1, 0, 2),
  [3205] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__attribute_name_repeat1, 2, 0, 11), SHIFT_REPEAT(2250),
  [3208] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__attribute_name_repeat1, 2, 0, 11),
  [3210] = {.entry = {.count = 1, .reusable = true}}, SHIFT(445),
  [3212] = {.entry = {.count = 1, .reusable = true}}, SHIFT(376),
  [3214] = {.entry = {.count = 1, .reusable = true}}, SHIFT(450),
  [3216] = {.entry = {.count = 1, .reusable = true}}, SHIFT(379),
  [3218] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1068),
  [3220] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__struct_signature, 2, 0, 14), SHIFT(1814),
  [3223] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__struct_signature, 2, 0, 14),
  [3225] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1792),
  [3227] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_script_repeat2, 1, 0, 0),
  [3229] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_access_specifier, 5, 0, 181),
  [3231] = {.entry = {.count = 1, .reusable = true}}, SHIFT(279),
  [3233] = {.entry = {.count = 1, .reusable = true}}, SHIFT(293),
  [3235] = {.entry = {.count = 1, .reusable = true}}, SHIFT(459),
  [3237] = {.entry = {.count = 1, .reusable = true}}, SHIFT(395),
  [3239] = {.entry = {.count = 1, .reusable = true}}, SHIFT(461),
  [3241] = {.entry = {.count = 1, .reusable = true}}, SHIFT(398),
  [3243] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_module, 5, 0, 7),
  [3245] = {.entry = {.count = 1, .reusable = true}}, SHIFT(346),
  [3247] = {.entry = {.count = 1, .reusable = true}}, SHIFT(194),
  [3249] = {.entry = {.count = 1, .reusable = true}}, SHIFT(463),
  [3251] = {.entry = {.count = 1, .reusable = true}}, SHIFT(408),
  [3253] = {.entry = {.count = 1, .reusable = true}}, SHIFT(465),
  [3255] = {.entry = {.count = 1, .reusable = true}}, SHIFT(411),
  [3257] = {.entry = {.count = 1, .reusable = true}}, SHIFT(330),
  [3259] = {.entry = {.count = 1, .reusable = true}}, SHIFT(234),
  [3261] = {.entry = {.count = 1, .reusable = true}}, SHIFT(467),
  [3263] = {.entry = {.count = 1, .reusable = true}}, SHIFT(421),
  [3265] = {.entry = {.count = 1, .reusable = true}}, SHIFT(469),
  [3267] = {.entry = {.count = 1, .reusable = true}}, SHIFT(424),
  [3269] = {.entry = {.count = 1, .reusable = true}}, SHIFT(332),
  [3271] = {.entry = {.count = 1, .reusable = true}}, SHIFT(237),
  [3273] = {.entry = {.count = 1, .reusable = true}}, SHIFT(471),
  [3275] = {.entry = {.count = 1, .reusable = true}}, SHIFT(434),
  [3277] = {.entry = {.count = 1, .reusable = true}}, SHIFT(473),
  [3279] = {.entry = {.count = 1, .reusable = true}}, SHIFT(437),
  [3281] = {.entry = {.count = 1, .reusable = true}}, SHIFT(322),
  [3283] = {.entry = {.count = 1, .reusable = true}}, SHIFT(179),
  [3285] = {.entry = {.count = 1, .reusable = true}}, SHIFT(324),
  [3287] = {.entry = {.count = 1, .reusable = true}}, SHIFT(182),
  [3289] = {.entry = {.count = 1, .reusable = true}}, SHIFT(167),
  [3291] = {.entry = {.count = 1, .reusable = true}}, SHIFT(168),
  [3293] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__script_constant_decl, 2, 0, 0),
  [3295] = {.entry = {.count = 1, .reusable = true}}, SHIFT(334),
  [3297] = {.entry = {.count = 1, .reusable = true}}, SHIFT(250),
  [3299] = {.entry = {.count = 1, .reusable = true}}, SHIFT(521),
  [3301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(446),
  [3303] = {.entry = {.count = 1, .reusable = true}}, SHIFT(523),
  [3305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(451),
  [3307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(492),
  [3309] = {.entry = {.count = 1, .reusable = true}}, SHIFT(153),
  [3311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(494),
  [3313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(484),
  [3315] = {.entry = {.count = 1, .reusable = true}}, SHIFT(348),
  [3317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(197),
  [3319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(338),
  [3321] = {.entry = {.count = 1, .reusable = true}}, SHIFT(491),
  [3323] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__specifier, 1, 0, 37),
  [3325] = {.entry = {.count = 1, .reusable = true}}, SHIFT(513),
  [3327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(502),
  [3329] = {.entry = {.count = 1, .reusable = true}}, SHIFT(515),
  [3331] = {.entry = {.count = 1, .reusable = true}}, SHIFT(505),
  [3333] = {.entry = {.count = 1, .reusable = true}}, SHIFT(340),
  [3335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(317),
  [3337] = {.entry = {.count = 1, .reusable = true}}, SHIFT(525),
  [3339] = {.entry = {.count = 1, .reusable = true}}, SHIFT(361),
  [3341] = {.entry = {.count = 1, .reusable = true}}, SHIFT(527),
  [3343] = {.entry = {.count = 1, .reusable = true}}, SHIFT(364),
  [3345] = {.entry = {.count = 1, .reusable = true}}, SHIFT(284),
  [3347] = {.entry = {.count = 1, .reusable = true}}, SHIFT(354),
  [3349] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__specifier_repeat1, 3, 0, 110),
  [3351] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_address_block, 5, 0, 0),
  [3353] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__struct_def_name, 1, 0, 1),
  [3355] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1758),
  [3357] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1822),
  [3359] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_block_target, 1, 0, 13),
  [3361] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__attribute_name_repeat1, 2, 0, 10),
  [3363] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_script, 4, 0, 0),
  [3365] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1803),
  [3367] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2242),
  [3369] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1849),
  [3371] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2026),
  [3373] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_script, 5, 0, 0),
  [3375] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant, 1, 0, 0),
  [3377] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1814),
  [3379] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_script, 6, 0, 0),
  [3381] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__spec_apply_fragment, 1, 0, 0),
  [3383] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__spec_apply_fragment, 1, 0, 0),
  [3385] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym__spec_apply_pattern_repeat1, 1, 0, 0),
  [3387] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__spec_apply_pattern_repeat1, 1, 0, 0),
  [3389] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_script, 7, 0, 0),
  [3391] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 1, 0, 0),
  [3393] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_address_block, 4, 0, 0),
  [3395] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__specifier_repeat1, 2, 0, 0),
  [3397] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1778),
  [3399] = {.entry = {.count = 1, .reusable = true}}, SHIFT(147),
  [3401] = {.entry = {.count = 1, .reusable = true}}, SHIFT(948),
  [3403] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1111),
  [3405] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1102),
  [3407] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1780),
  [3409] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute, 1, 0, 3),
  [3411] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1075),
  [3413] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1711),
  [3415] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2225),
  [3417] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1540),
  [3419] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__module_keyword, 1, 0, 0),
  [3421] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__module_keyword, 1, 0, 0),
  [3423] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_vector_value_expr_repeat1, 2, 0, 0), SHIFT_REPEAT(192),
  [3426] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_vector_value_expr_repeat1, 2, 0, 0),
  [3428] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1828),
  [3430] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ability, 1, 0, 0),
  [3432] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_bind_list, 3, 0, 0),
  [3434] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_bind_list, 3, 0, 0),
  [3436] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2306),
  [3438] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1944),
  [3440] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_bind_list, 5, 0, 97),
  [3442] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_bind_list, 5, 0, 97),
  [3444] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_bind_list, 4, 0, 58),
  [3446] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_bind_list, 4, 0, 58),
  [3448] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_bind_list, 3, 0, 58),
  [3450] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_bind_list, 3, 0, 58),
  [3452] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_quantifier_repeat1, 2, 0, 0), SHIFT_REPEAT(2003),
  [3455] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_quantifier_repeat1, 2, 0, 0),
  [3457] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__use_member, 1, 0, 0),
  [3459] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2269),
  [3461] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_bind_list, 4, 0, 97),
  [3463] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_bind_list, 4, 0, 97),
  [3465] = {.entry = {.count = 1, .reusable = true}}, SHIFT(100),
  [3467] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1804),
  [3469] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_bind_list, 2, 0, 0),
  [3471] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_bind_list, 2, 0, 0),
  [3473] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_bind_list, 1, 0, 44),
  [3475] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_bind_list, 1, 0, 44),
  [3477] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_spec_apply_repeat1, 2, 0, 177), SHIFT_REPEAT(1751),
  [3480] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_spec_apply_repeat1, 2, 0, 177),
  [3482] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1803),
  [3484] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2400),
  [3486] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1026),
  [3488] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1785),
  [3490] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2341),
  [3492] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2289),
  [3494] = {.entry = {.count = 1, .reusable = true}}, SHIFT(78),
  [3496] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1648),
  [3498] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1466),
  [3500] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1629),
  [3502] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1982),
  [3504] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2330),
  [3506] = {.entry = {.count = 1, .reusable = true}}, SHIFT(953),
  [3508] = {.entry = {.count = 1, .reusable = true}}, SHIFT(58),
  [3510] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1633),
  [3512] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1505),
  [3514] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1615),
  [3516] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_constraints_repeat1, 2, 0, 0),
  [3518] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_constraints_repeat1, 2, 0, 0), SHIFT_REPEAT(1770),
  [3521] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_param, 1, 0, 26),
  [3523] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1772),
  [3525] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__struct_type_params, 5, 0, 0),
  [3527] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_anon_fields, 5, 0, 0),
  [3529] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_triggers, 1, 0, 55),
  [3531] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1981),
  [3533] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_tuple_type_repeat1, 2, 0, 0),
  [3535] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__spec_pragma_prop, 1, 0, 28),
  [3537] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1104),
  [3539] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_type_repeat1, 2, 0, 0), SHIFT_REPEAT(1114),
  [3542] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2074),
  [3544] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__enum_signature, 2, 0, 14), SHIFT(1676),
  [3547] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2360),
  [3549] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2361),
  [3551] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1519),
  [3553] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_anon_fields, 3, 0, 0),
  [3555] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_quantifier_bind, 3, 0, 90),
  [3557] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_quantifier_bind, 3, 0, 91),
  [3559] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_constraints, 3, 0, 0),
  [3561] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1770),
  [3563] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__address_member, 2, 0, 0),
  [3565] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2388),
  [3567] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2389),
  [3569] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1690),
  [3571] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_address_block_repeat1, 1, 0, 0),
  [3573] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_triggers_repeat2, 2, 0, 93),
  [3575] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_triggers_repeat2, 2, 0, 93), SHIFT_REPEAT(44),
  [3578] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1545),
  [3580] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2325),
  [3582] = {.entry = {.count = 1, .reusable = true}}, SHIFT(931),
  [3584] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__struct_type_params, 4, 0, 0),
  [3586] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_anon_fields, 4, 0, 0),
  [3588] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_abilities_repeat1, 2, 0, 0), SHIFT_REPEAT(1788),
  [3591] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_abilities_repeat1, 2, 0, 0),
  [3593] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1919),
  [3595] = {.entry = {.count = 1, .reusable = true}}, SHIFT(957),
  [3597] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2217),
  [3599] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__address_specifier, 1, 0, 137),
  [3601] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributes_repeat1, 2, 0, 0), SHIFT_REPEAT(1932),
  [3604] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_attributes_repeat1, 2, 0, 0),
  [3606] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1930),
  [3608] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1439),
  [3610] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_anon_fields, 2, 0, 0),
  [3612] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__struct_def_name, 2, 0, 1),
  [3614] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1788),
  [3616] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abilities, 2, 0, 0),
  [3618] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_constraints, 2, 0, 0),
  [3620] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1933),
  [3622] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__address_member, 1, 0, 0),
  [3624] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__struct_type_params, 3, 0, 0),
  [3626] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_condition_props_repeat1, 2, 0, 103), SHIFT_REPEAT(1427),
  [3629] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_condition_props_repeat1, 2, 0, 103),
  [3631] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abilities, 3, 0, 0),
  [3633] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1547),
  [3635] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2387),
  [3637] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2390),
  [3639] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__struct_type_params, 2, 0, 0),
  [3641] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1656),
  [3643] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_spec_apply_repeat1, 2, 0, 175),
  [3645] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1847),
  [3647] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1541),
  [3649] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1663),
  [3651] = {.entry = {.count = 1, .reusable = true}}, SHIFT(192),
  [3653] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_aborts_with_or_modifies, 3, 0, 48),
  [3655] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1901),
  [3657] = {.entry = {.count = 1, .reusable = true}}, SHIFT(903),
  [3659] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1677),
  [3661] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1476),
  [3663] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__script_spec_block, 2, 0, 0),
  [3665] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_triggers_repeat2, 5, 0, 157),
  [3667] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1672),
  [3669] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1458),
  [3671] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1424),
  [3673] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1060),
  [3675] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1076),
  [3677] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_lambda_bind_list_repeat1, 2, 0, 98), SHIFT_REPEAT(1240),
  [3680] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_lambda_bind_list_repeat1, 2, 0, 98),
  [3682] = {.entry = {.count = 1, .reusable = true}}, SHIFT(985),
  [3684] = {.entry = {.count = 1, .reusable = true}}, SHIFT(57),
  [3686] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1883),
  [3688] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2331),
  [3690] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1105),
  [3692] = {.entry = {.count = 1, .reusable = true}}, SHIFT(977),
  [3694] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1583),
  [3696] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_type_repeat1, 2, 0, 0), SHIFT_REPEAT(1091),
  [3699] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_parameters_repeat1, 2, 0, 0), SHIFT_REPEAT(2057),
  [3702] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_parameters_repeat1, 2, 0, 0),
  [3704] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1415),
  [3706] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1448),
  [3708] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1789),
  [3710] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__spec_apply_pattern, 2, 0, 151),
  [3712] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1226),
  [3714] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1647),
  [3716] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1204),
  [3718] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1067),
  [3720] = {.entry = {.count = 1, .reusable = true}}, SHIFT(148),
  [3722] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__use_member, 2, 0, 0),
  [3724] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__attribute_val, 1, 0, 0),
  [3726] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1678),
  [3728] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1481),
  [3730] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute, 3, 0, 12),
  [3732] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1422),
  [3734] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2352),
  [3736] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1417),
  [3738] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1065),
  [3740] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1679),
  [3742] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1516),
  [3744] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1928),
  [3746] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1218),
  [3748] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1783),
  [3750] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_triggers_repeat2, 2, 0, 0),
  [3752] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1680),
  [3754] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1486),
  [3756] = {.entry = {.count = 1, .reusable = true}}, SHIFT(102),
  [3758] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1953),
  [3760] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_block_target, 2, 0, 23),
  [3762] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1681),
  [3764] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1489),
  [3766] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__bind_fields_repeat1, 2, 0, 0), SHIFT_REPEAT(1425),
  [3769] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__bind_fields_repeat1, 2, 0, 0),
  [3771] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1083),
  [3773] = {.entry = {.count = 1, .reusable = true}}, SHIFT(869),
  [3775] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1915),
  [3777] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2189),
  [3779] = {.entry = {.count = 1, .reusable = true}}, SHIFT(529),
  [3781] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1522),
  [3783] = {.entry = {.count = 1, .reusable = true}}, SHIFT(913),
  [3785] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_pack_expr_repeat1, 2, 0, 0), SHIFT_REPEAT(2015),
  [3788] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_pack_expr_repeat1, 2, 0, 0),
  [3790] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1682),
  [3792] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1492),
  [3794] = {.entry = {.count = 1, .reusable = true}}, SHIFT(921),
  [3796] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1279),
  [3798] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1077),
  [3800] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1777),
  [3802] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1844),
  [3804] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1683),
  [3806] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1495),
  [3808] = {.entry = {.count = 1, .reusable = true}}, SHIFT(914),
  [3810] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1684),
  [3812] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1497),
  [3814] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1993),
  [3816] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1034),
  [3818] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1904),
  [3820] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1423),
  [3822] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1463),
  [3824] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_triggers_repeat2, 4, 0, 129),
  [3826] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2176),
  [3828] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__struct_type_params_repeat1, 2, 0, 0), SHIFT_REPEAT(1805),
  [3831] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__struct_type_params_repeat1, 2, 0, 0),
  [3833] = {.entry = {.count = 1, .reusable = true}}, SHIFT(51),
  [3835] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1685),
  [3837] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1500),
  [3839] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1848),
  [3841] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1088),
  [3843] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1562),
  [3845] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1086),
  [3847] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1964),
  [3849] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1826),
  [3851] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__script_func_decl, 3, 0, 0),
  [3853] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute, 5, 0, 3),
  [3855] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2396),
  [3857] = {.entry = {.count = 1, .reusable = true}}, SHIFT(22),
  [3859] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1795),
  [3861] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1101),
  [3863] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2433),
  [3865] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2446),
  [3867] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2463),
  [3869] = {.entry = {.count = 1, .reusable = true}}, SHIFT(149),
  [3871] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__script_func_decl, 1, 0, 0),
  [3873] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__script_func_decl, 2, 0, 0),
  [3875] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2459),
  [3877] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2097),
  [3879] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2333),
  [3881] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_triggers_repeat2, 4, 0, 157),
  [3883] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1219),
  [3885] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1617),
  [3887] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_aborts_with_or_modifies, 2, 0, 48),
  [3889] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_bind_field, 1, 0, 63),
  [3891] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1233),
  [3893] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1413),
  [3895] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1616),
  [3897] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute, 3, 0, 3),
  [3899] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_triggers_repeat2, 3, 0, 0),
  [3901] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_lambda_bind_list_repeat1, 2, 0, 58),
  [3903] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1842),
  [3905] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1977),
  [3907] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1587),
  [3909] = {.entry = {.count = 1, .reusable = true}}, SHIFT(878),
  [3911] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1087),
  [3913] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1815),
  [3915] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__script_spec_block, 1, 0, 0),
  [3917] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1078),
  [3919] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_triggers_repeat2, 3, 0, 129),
  [3921] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_script_repeat3, 1, 0, 0),
  [3923] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_lambda_bind_list_repeat1, 2, 0, 98), SHIFT_REPEAT(1234),
  [3926] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1108),
  [3928] = {.entry = {.count = 1, .reusable = true}}, SHIFT(54),
  [3930] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1939),
  [3932] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute, 4, 0, 3),
  [3934] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1580),
  [3936] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1568),
  [3938] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_struct_body_repeat1, 2, 0, 0), SHIFT_REPEAT(2053),
  [3941] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_struct_body_repeat1, 2, 0, 0),
  [3943] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1573),
  [3945] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_triggers_repeat1, 2, 0, 93), SHIFT_REPEAT(268),
  [3948] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_triggers_repeat1, 2, 0, 93),
  [3950] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2363),
  [3952] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_use_decl_repeat1, 2, 0, 0), SHIFT_REPEAT(2002),
  [3955] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_use_decl_repeat1, 2, 0, 0),
  [3957] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1969),
  [3959] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1520),
  [3961] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__variant, 1, 0, 0),
  [3963] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2055),
  [3965] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2085),
  [3967] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__variant, 1, 0, 0), REDUCE(sym__variant_last, 1, 0, 0),
  [3970] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1530),
  [3972] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1899),
  [3974] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant_struct, 2, 0, 71),
  [3976] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1035),
  [3978] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_type_params_repeat1, 2, 0, 0), SHIFT_REPEAT(2014),
  [3981] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_type_params_repeat1, 2, 0, 0),
  [3983] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__spec_pragma_prop, 3, 0, 99),
  [3985] = {.entry = {.count = 1, .reusable = true}}, SHIFT(983),
  [3987] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1850),
  [3989] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1667),
  [3991] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1215),
  [3993] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_condition_props_repeat1, 2, 0, 101),
  [3995] = {.entry = {.count = 1, .reusable = true}}, SHIFT(151),
  [3997] = {.entry = {.count = 1, .reusable = true}}, SHIFT(949),
  [3999] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute, 6, 0, 3),
  [4001] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expr_field, 1, 0, 63),
  [4003] = {.entry = {.count = 1, .reusable = true}}, SHIFT(226),
  [4005] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1674),
  [4007] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1471),
  [4009] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1909),
  [4011] = {.entry = {.count = 1, .reusable = true}}, SHIFT(904),
  [4013] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1220),
  [4015] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1786),
  [4017] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_module_ident, 3, 0, 39),
  [4019] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1675),
  [4021] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1473),
  [4023] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__spec_apply_pattern, 3, 0, 173),
  [4025] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__use_alias, 2, 0, 40),
  [4027] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1853),
  [4029] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1079),
  [4031] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1549),
  [4033] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1808),
  [4035] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1926),
  [4037] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1042),
  [4039] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1217),
  [4041] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1063),
  [4043] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1885),
  [4045] = {.entry = {.count = 1, .reusable = true}}, SHIFT(952),
  [4047] = {.entry = {.count = 1, .reusable = true}}, SHIFT(53),
  [4049] = {.entry = {.count = 1, .reusable = true}}, SHIFT(177),
  [4051] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1112),
  [4053] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_expr, 2, 0, 43),
  [4055] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1972),
  [4057] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_aborts_with_or_modifies, 4, 0, 48),
  [4059] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2218),
  [4061] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2038),
  [4063] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2101),
  [4065] = {.entry = {.count = 1, .reusable = true}}, SHIFT(270),
  [4067] = {.entry = {.count = 1, .reusable = true}}, SHIFT(115),
  [4069] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__function_signature, 5, 0, 78),
  [4071] = {.entry = {.count = 1, .reusable = true}}, SHIFT(255),
  [4073] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_expr, 4, 0, 117),
  [4075] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_bind_field, 3, 0, 160),
  [4077] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter, 3, 0, 24),
  [4079] = {.entry = {.count = 1, .reusable = true}}, SHIFT(207),
  [4081] = {.entry = {.count = 1, .reusable = true}}, SHIFT(208),
  [4083] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1823),
  [4085] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expr_field, 3, 0, 135),
  [4087] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1606),
  [4089] = {.entry = {.count = 1, .reusable = true}}, SHIFT(518),
  [4091] = {.entry = {.count = 1, .reusable = true}}, SHIFT(283),
  [4093] = {.entry = {.count = 1, .reusable = true}}, SHIFT(261),
  [4095] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1484),
  [4097] = {.entry = {.count = 1, .reusable = true}}, SHIFT(327),
  [4099] = {.entry = {.count = 1, .reusable = true}}, SHIFT(216),
  [4101] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1089),
  [4103] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__spec_target_signature_opt, 2, 0, 46),
  [4105] = {.entry = {.count = 1, .reusable = true}}, SHIFT(329),
  [4107] = {.entry = {.count = 1, .reusable = true}}, SHIFT(221),
  [4109] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__struct_type_parameter, 2, 0, 0),
  [4111] = {.entry = {.count = 1, .reusable = true}}, SHIFT(520),
  [4113] = {.entry = {.count = 1, .reusable = true}}, SHIFT(289),
  [4115] = {.entry = {.count = 1, .reusable = true}}, SHIFT(349),
  [4117] = {.entry = {.count = 1, .reusable = true}}, SHIFT(201),
  [4119] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__function_signature, 8, 0, 166),
  [4121] = {.entry = {.count = 1, .reusable = true}}, SHIFT(854),
  [4123] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1046),
  [4125] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__struct_type_parameter, 1, 0, 0),
  [4127] = {.entry = {.count = 1, .reusable = true}}, SHIFT(323),
  [4129] = {.entry = {.count = 1, .reusable = true}}, SHIFT(181),
  [4131] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1103),
  [4133] = {.entry = {.count = 1, .reusable = true}}, SHIFT(166),
  [4135] = {.entry = {.count = 1, .reusable = true}}, SHIFT(262),
  [4137] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_emits, 4, 0, 121),
  [4139] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__function_signature, 6, 0, 109),
  [4141] = {.entry = {.count = 1, .reusable = true}}, SHIFT(227),
  [4143] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1513),
  [4145] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1514),
  [4147] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_aborts_if, 3, 0, 82),
  [4149] = {.entry = {.count = 1, .reusable = true}}, SHIFT(228),
  [4151] = {.entry = {.count = 1, .reusable = true}}, SHIFT(263),
  [4153] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1451),
  [4155] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1611),
  [4157] = {.entry = {.count = 1, .reusable = true}}, SHIFT(288),
  [4159] = {.entry = {.count = 1, .reusable = true}}, SHIFT(366),
  [4161] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1754),
  [4163] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__variant, 2, 0, 0),
  [4165] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__address_specifier, 1, 0, 136),
  [4167] = {.entry = {.count = 1, .reusable = true}}, SHIFT(448),
  [4169] = {.entry = {.count = 1, .reusable = true}}, SHIFT(378),
  [4171] = {.entry = {.count = 1, .reusable = true}}, SHIFT(325),
  [4173] = {.entry = {.count = 1, .reusable = true}}, SHIFT(186),
  [4175] = {.entry = {.count = 1, .reusable = true}}, SHIFT(454),
  [4177] = {.entry = {.count = 1, .reusable = true}}, SHIFT(383),
  [4179] = {.entry = {.count = 1, .reusable = true}}, SHIFT(347),
  [4181] = {.entry = {.count = 1, .reusable = true}}, SHIFT(196),
  [4183] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1676),
  [4185] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1478),
  [4187] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1638),
  [4189] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__function_signature, 7, 0, 138),
  [4191] = {.entry = {.count = 1, .reusable = true}}, SHIFT(460),
  [4193] = {.entry = {.count = 1, .reusable = true}}, SHIFT(397),
  [4195] = {.entry = {.count = 1, .reusable = true}}, SHIFT(462),
  [4197] = {.entry = {.count = 1, .reusable = true}}, SHIFT(401),
  [4199] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__module_path, 1, 0, 1),
  [4201] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1644),
  [4203] = {.entry = {.count = 1, .reusable = true}}, SHIFT(245),
  [4205] = {.entry = {.count = 1, .reusable = true}}, SHIFT(246),
  [4207] = {.entry = {.count = 1, .reusable = true}}, SHIFT(464),
  [4209] = {.entry = {.count = 1, .reusable = true}}, SHIFT(410),
  [4211] = {.entry = {.count = 1, .reusable = true}}, SHIFT(466),
  [4213] = {.entry = {.count = 1, .reusable = true}}, SHIFT(414),
  [4215] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant_posit, 2, 0, 71),
  [4217] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__variant, 2, 0, 0), REDUCE(sym__variant_last, 2, 0, 0),
  [4220] = {.entry = {.count = 1, .reusable = true}}, SHIFT(468),
  [4222] = {.entry = {.count = 1, .reusable = true}}, SHIFT(423),
  [4224] = {.entry = {.count = 1, .reusable = true}}, SHIFT(470),
  [4226] = {.entry = {.count = 1, .reusable = true}}, SHIFT(427),
  [4228] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__function_signature, 7, 0, 143),
  [4230] = {.entry = {.count = 1, .reusable = true}}, SHIFT(331),
  [4232] = {.entry = {.count = 1, .reusable = true}}, SHIFT(236),
  [4234] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1097),
  [4236] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__spec_target_signature_opt, 1, 0, 21),
  [4238] = {.entry = {.count = 1, .reusable = true}}, SHIFT(472),
  [4240] = {.entry = {.count = 1, .reusable = true}}, SHIFT(436),
  [4242] = {.entry = {.count = 1, .reusable = true}}, SHIFT(474),
  [4244] = {.entry = {.count = 1, .reusable = true}}, SHIFT(440),
  [4246] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2460),
  [4248] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2402),
  [4250] = {.entry = {.count = 1, .reusable = true}}, SHIFT(333),
  [4252] = {.entry = {.count = 1, .reusable = true}}, SHIFT(241),
  [4254] = {.entry = {.count = 1, .reusable = true}}, SHIFT(282),
  [4256] = {.entry = {.count = 1, .reusable = true}}, SHIFT(308),
  [4258] = {.entry = {.count = 1, .reusable = true}}, SHIFT(277),
  [4260] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__spec_func_signatures, 4, 0, 127),
  [4262] = {.entry = {.count = 1, .reusable = true}}, SHIFT(343),
  [4264] = {.entry = {.count = 1, .reusable = true}}, SHIFT(299),
  [4266] = {.entry = {.count = 1, .reusable = true}}, SHIFT(345),
  [4268] = {.entry = {.count = 1, .reusable = true}}, SHIFT(303),
  [4270] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_param, 2, 0, 26),
  [4272] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_aborts_if, 2, 0, 31),
  [4274] = {.entry = {.count = 1, .reusable = true}}, SHIFT(184),
  [4276] = {.entry = {.count = 1, .reusable = true}}, SHIFT(351),
  [4278] = {.entry = {.count = 1, .reusable = true}}, SHIFT(314),
  [4280] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_annot, 3, 0, 108),
  [4282] = {.entry = {.count = 1, .reusable = true}}, SHIFT(353),
  [4284] = {.entry = {.count = 1, .reusable = true}}, SHIFT(318),
  [4286] = {.entry = {.count = 1, .reusable = true}}, SHIFT(266),
  [4288] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_emits, 5, 0, 146),
  [4290] = {.entry = {.count = 1, .reusable = true}}, SHIFT(267),
  [4292] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1455),
  [4294] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__struct_signature, 3, 0, 14),
  [4296] = {.entry = {.count = 1, .reusable = true}}, SHIFT(335),
  [4298] = {.entry = {.count = 1, .reusable = true}}, SHIFT(252),
  [4300] = {.entry = {.count = 1, .reusable = true}}, SHIFT(522),
  [4302] = {.entry = {.count = 1, .reusable = true}}, SHIFT(449),
  [4304] = {.entry = {.count = 1, .reusable = true}}, SHIFT(524),
  [4306] = {.entry = {.count = 1, .reusable = true}}, SHIFT(455),
  [4308] = {.entry = {.count = 1, .reusable = true}}, SHIFT(337),
  [4310] = {.entry = {.count = 1, .reusable = true}}, SHIFT(257),
  [4312] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_enum_body_repeat1, 1, 0, 0),
  [4314] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__variant_last, 1, 0, 0),
  [4316] = {.entry = {.count = 1, .reusable = true}}, SHIFT(493),
  [4318] = {.entry = {.count = 1, .reusable = true}}, SHIFT(483),
  [4320] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__function_signature, 5, 0, 74),
  [4322] = {.entry = {.count = 1, .reusable = true}}, SHIFT(495),
  [4324] = {.entry = {.count = 1, .reusable = true}}, SHIFT(487),
  [4326] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__specifier, 1, 0, 35),
  [4328] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__function_signature, 4, 0, 36),
  [4330] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__spec_func_signatures, 5, 0, 155),
  [4332] = {.entry = {.count = 1, .reusable = true}}, SHIFT(514),
  [4334] = {.entry = {.count = 1, .reusable = true}}, SHIFT(504),
  [4336] = {.entry = {.count = 1, .reusable = true}}, SHIFT(516),
  [4338] = {.entry = {.count = 1, .reusable = true}}, SHIFT(508),
  [4340] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__function_signature, 6, 0, 115),
  [4342] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_triggers_repeat1, 2, 0, 129),
  [4344] = {.entry = {.count = 1, .reusable = true}}, SHIFT(339),
  [4346] = {.entry = {.count = 1, .reusable = true}}, SHIFT(512),
  [4348] = {.entry = {.count = 1, .reusable = true}}, SHIFT(341),
  [4350] = {.entry = {.count = 1, .reusable = true}}, SHIFT(157),
  [4352] = {.entry = {.count = 1, .reusable = true}}, SHIFT(526),
  [4354] = {.entry = {.count = 1, .reusable = true}}, SHIFT(363),
  [4356] = {.entry = {.count = 1, .reusable = true}}, SHIFT(481),
  [4358] = {.entry = {.count = 1, .reusable = true}}, SHIFT(368),
  [4360] = {.entry = {.count = 1, .reusable = true}}, SHIFT(884),
  [4362] = {.entry = {.count = 1, .reusable = true}}, SHIFT(396),
  [4364] = {.entry = {.count = 1, .reusable = true}}, SHIFT(170),
  [4366] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1945),
  [4368] = {.entry = {.count = 1, .reusable = true}}, SHIFT(960),
  [4370] = {.entry = {.count = 1, .reusable = true}}, SHIFT(399),
  [4372] = {.entry = {.count = 1, .reusable = true}}, SHIFT(400),
  [4374] = {.entry = {.count = 1, .reusable = true}}, SHIFT(180),
  [4376] = {.entry = {.count = 1, .reusable = true}}, SHIFT(292),
  [4378] = {.entry = {.count = 1, .reusable = true}}, SHIFT(402),
  [4380] = {.entry = {.count = 1, .reusable = true}}, SHIFT(403),
  [4382] = {.entry = {.count = 1, .reusable = true}}, SHIFT(404),
  [4384] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_expr, 6, 0, 167),
  [4386] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2043),
  [4388] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1578),
  [4390] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_block_target, 1, 0, 0),
  [4392] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1488),
  [4394] = {.entry = {.count = 1, .reusable = true}}, SHIFT(965),
  [4396] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_block_target, 3, 0, 51),
  [4398] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1581),
  [4400] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1096),
  [4402] = {.entry = {.count = 1, .reusable = true}}, SHIFT(865),
  [4404] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2091),
  [4406] = {.entry = {.count = 1, .reusable = true}}, SHIFT(122),
  [4408] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1468),
  [4410] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_emits, 6, 0, 170),
  [4412] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1779),
  [4414] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1469),
  [4416] = {.entry = {.count = 1, .reusable = true}}, SHIFT(409),
  [4418] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__address_specifier, 4, 0, 196),
  [4420] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1054),
  [4422] = {.entry = {.count = 1, .reusable = true}}, SHIFT(195),
  [4424] = {.entry = {.count = 1, .reusable = true}}, SHIFT(412),
  [4426] = {.entry = {.count = 1, .reusable = true}}, SHIFT(413),
  [4428] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1246),
  [4430] = {.entry = {.count = 1, .reusable = true}}, SHIFT(415),
  [4432] = {.entry = {.count = 1, .reusable = true}}, SHIFT(416),
  [4434] = {.entry = {.count = 1, .reusable = true}}, SHIFT(417),
  [4436] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2244),
  [4438] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1470),
  [4440] = {.entry = {.count = 1, .reusable = true}}, SHIFT(867),
  [4442] = {.entry = {.count = 1, .reusable = true}}, SHIFT(205),
  [4444] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1892),
  [4446] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__module_path, 3, 0, 6),
  [4448] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__spec_target_signature_opt, 3, 0, 80),
  [4450] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2410),
  [4452] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1692),
  [4454] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1552),
  [4456] = {.entry = {.count = 1, .reusable = true}}, SHIFT(771),
  [4458] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1058),
  [4460] = {.entry = {.count = 1, .reusable = true}}, SHIFT(530),
  [4462] = {.entry = {.count = 1, .reusable = true}}, SHIFT(43),
  [4464] = {.entry = {.count = 1, .reusable = true}}, SHIFT(422),
  [4466] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1443),
  [4468] = {.entry = {.count = 1, .reusable = true}}, SHIFT(425),
  [4470] = {.entry = {.count = 1, .reusable = true}}, SHIFT(426),
  [4472] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2414),
  [4474] = {.entry = {.count = 1, .reusable = true}}, SHIFT(428),
  [4476] = {.entry = {.count = 1, .reusable = true}}, SHIFT(429),
  [4478] = {.entry = {.count = 1, .reusable = true}}, SHIFT(430),
  [4480] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1494),
  [4482] = {.entry = {.count = 1, .reusable = true}}, SHIFT(235),
  [4484] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1508),
  [4486] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1986),
  [4488] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1231),
  [4490] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1718),
  [4492] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1531),
  [4494] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1247),
  [4496] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1252),
  [4498] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_asserts, 3, 0, 81),
  [4500] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__address_specifier, 5, 0, 208),
  [4502] = {.entry = {.count = 1, .reusable = true}}, SHIFT(238),
  [4504] = {.entry = {.count = 1, .reusable = true}}, SHIFT(286),
  [4506] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1509),
  [4508] = {.entry = {.count = 1, .reusable = true}}, SHIFT(435),
  [4510] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1774),
  [4512] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1756),
  [4514] = {.entry = {.count = 1, .reusable = true}}, SHIFT(438),
  [4516] = {.entry = {.count = 1, .reusable = true}}, SHIFT(439),
  [4518] = {.entry = {.count = 1, .reusable = true}}, SHIFT(200),
  [4520] = {.entry = {.count = 1, .reusable = true}}, SHIFT(441),
  [4522] = {.entry = {.count = 1, .reusable = true}}, SHIFT(442),
  [4524] = {.entry = {.count = 1, .reusable = true}}, SHIFT(443),
  [4526] = {.entry = {.count = 1, .reusable = true}}, SHIFT(240),
  [4528] = {.entry = {.count = 1, .reusable = true}}, SHIFT(202),
  [4530] = {.entry = {.count = 1, .reusable = true}}, SHIFT(215),
  [4532] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2215),
  [4534] = {.entry = {.count = 1, .reusable = true}}, SHIFT(242),
  [4536] = {.entry = {.count = 1, .reusable = true}}, SHIFT(243),
  [4538] = {.entry = {.count = 1, .reusable = true}}, SHIFT(244),
  [4540] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1281),
  [4542] = {.entry = {.count = 1, .reusable = true}}, SHIFT(231),
  [4544] = {.entry = {.count = 1, .reusable = true}}, SHIFT(198),
  [4546] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_spec_block_target, 2, 0, 20),
  [4548] = {.entry = {.count = 1, .reusable = true}}, SHIFT(355),
  [4550] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1989),
  [4552] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1511),
  [4554] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1475),
  [4556] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2279),
  [4558] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1116),
  [4560] = {.entry = {.count = 1, .reusable = true}}, SHIFT(967),
  [4562] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__spec_target_signature_opt, 4, 0, 119),
  [4564] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1833),
  [4566] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1120),
  [4568] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1244),
  [4570] = {.entry = {.count = 1, .reusable = true}}, SHIFT(868),
  [4572] = {.entry = {.count = 1, .reusable = true}}, SHIFT(898),
  [4574] = {.entry = {.count = 1, .reusable = true}}, SHIFT(899),
  [4576] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1529),
  [4578] = {.entry = {.count = 1, .reusable = true}}, SHIFT(298),
  [4580] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2018),
  [4582] = {.entry = {.count = 1, .reusable = true}}, SHIFT(359),
  [4584] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1107),
  [4586] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1444),
  [4588] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2236),
  [4590] = {.entry = {.count = 1, .reusable = true}}, SHIFT(301),
  [4592] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1090),
  [4594] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1830),
  [4596] = {.entry = {.count = 1, .reusable = true}}, SHIFT(302),
  [4598] = {.entry = {.count = 1, .reusable = true}}, SHIFT(203),
  [4600] = {.entry = {.count = 1, .reusable = true}}, SHIFT(304),
  [4602] = {.entry = {.count = 1, .reusable = true}}, SHIFT(305),
  [4604] = {.entry = {.count = 1, .reusable = true}}, SHIFT(306),
  [4606] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1454),
  [4608] = {.entry = {.count = 1, .reusable = true}}, SHIFT(219),
  [4610] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_asserts, 2, 0, 47),
  [4612] = {.entry = {.count = 1, .reusable = true}}, SHIFT(313),
  [4614] = {.entry = {.count = 1, .reusable = true}}, SHIFT(183),
  [4616] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1631),
  [4618] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1958),
  [4620] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1095),
  [4622] = {.entry = {.count = 1, .reusable = true}}, SHIFT(316),
  [4624] = {.entry = {.count = 1, .reusable = true}}, SHIFT(528),
  [4626] = {.entry = {.count = 1, .reusable = true}}, SHIFT(281),
  [4628] = {.entry = {.count = 1, .reusable = true}}, SHIFT(319),
  [4630] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1441),
  [4632] = {.entry = {.count = 1, .reusable = true}}, SHIFT(218),
  [4634] = {.entry = {.count = 1, .reusable = true}}, SHIFT(199),
  [4636] = {.entry = {.count = 1, .reusable = true}}, SHIFT(320),
  [4638] = {.entry = {.count = 1, .reusable = true}}, SHIFT(321),
  [4640] = {.entry = {.count = 1, .reusable = true}}, SHIFT(220),
  [4642] = {.entry = {.count = 1, .reusable = true}}, SHIFT(271),
  [4644] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1249),
  [4646] = {.entry = {.count = 1, .reusable = true}}, SHIFT(204),
  [4648] = {.entry = {.count = 1, .reusable = true}}, SHIFT(185),
  [4650] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1245),
  [4652] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_aborts_if, 5, 0, 145),
  [4654] = {.entry = {.count = 1, .reusable = true}}, SHIFT(377),
  [4656] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1436),
  [4658] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_aborts_if, 4, 0, 120),
  [4660] = {.entry = {.count = 1, .reusable = true}}, SHIFT(374),
  [4662] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1460),
  [4664] = {.entry = {.count = 1, .reusable = true}}, SHIFT(264),
  [4666] = {.entry = {.count = 1, .reusable = true}}, SHIFT(251),
  [4668] = {.entry = {.count = 1, .reusable = true}}, SHIFT(381),
  [4670] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1612),
  [4672] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1515),
  [4674] = {.entry = {.count = 1, .reusable = true}}, SHIFT(265),
  [4676] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__enum_signature, 3, 0, 14),
  [4678] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1457),
  [4680] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1705),
  [4682] = {.entry = {.count = 1, .reusable = true}}, SHIFT(447),
  [4684] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1248),
  [4686] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1435),
  [4688] = {.entry = {.count = 1, .reusable = true}}, SHIFT(452),
  [4690] = {.entry = {.count = 1, .reusable = true}}, SHIFT(453),
  [4692] = {.entry = {.count = 1, .reusable = true}}, SHIFT(287),
  [4694] = {.entry = {.count = 1, .reusable = true}}, SHIFT(456),
  [4696] = {.entry = {.count = 1, .reusable = true}}, SHIFT(457),
  [4698] = {.entry = {.count = 1, .reusable = true}}, SHIFT(458),
  [4700] = {.entry = {.count = 1, .reusable = true}}, SHIFT(254),
  [4702] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1093),
  [4704] = {.entry = {.count = 1, .reusable = true}},  ACCEPT_INPUT(),
  [4706] = {.entry = {.count = 1, .reusable = true}}, SHIFT(968),
  [4708] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1589),
  [4710] = {.entry = {.count = 1, .reusable = true}}, SHIFT(256),
  [4712] = {.entry = {.count = 1, .reusable = true}}, SHIFT(380),
  [4714] = {.entry = {.count = 1, .reusable = true}}, SHIFT(258),
  [4716] = {.entry = {.count = 1, .reusable = true}}, SHIFT(259),
  [4718] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1538),
  [4720] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2260),
  [4722] = {.entry = {.count = 1, .reusable = true}}, SHIFT(260),
  [4724] = {.entry = {.count = 1, .reusable = true}}, SHIFT(482),
  [4726] = {.entry = {.count = 1, .reusable = true}}, SHIFT(232),
  [4728] = {.entry = {.count = 1, .reusable = true}}, SHIFT(187),
  [4730] = {.entry = {.count = 1, .reusable = true}}, SHIFT(485),
  [4732] = {.entry = {.count = 1, .reusable = true}}, SHIFT(486),
  [4734] = {.entry = {.count = 1, .reusable = true}}, SHIFT(387),
  [4736] = {.entry = {.count = 1, .reusable = true}}, SHIFT(488),
  [4738] = {.entry = {.count = 1, .reusable = true}}, SHIFT(489),
  [4740] = {.entry = {.count = 1, .reusable = true}}, SHIFT(490),
  [4742] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1602),
  [4744] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1956),
  [4746] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1449),
  [4748] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1450),
  [4750] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1099),
  [4752] = {.entry = {.count = 1, .reusable = true}}, SHIFT(969),
  [4754] = {.entry = {.count = 1, .reusable = true}}, SHIFT(188),
  [4756] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2292),
  [4758] = {.entry = {.count = 1, .reusable = true}}, SHIFT(307),
  [4760] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1464),
  [4762] = {.entry = {.count = 1, .reusable = true}}, SHIFT(189),
  [4764] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1438),
  [4766] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1094),
  [4768] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1467),
  [4770] = {.entry = {.count = 1, .reusable = true}}, SHIFT(382),
  [4772] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2462),
  [4774] = {.entry = {.count = 1, .reusable = true}}, SHIFT(384),
  [4776] = {.entry = {.count = 1, .reusable = true}}, SHIFT(496),
  [4778] = {.entry = {.count = 1, .reusable = true}}, SHIFT(385),
  [4780] = {.entry = {.count = 1, .reusable = true}}, SHIFT(888),
  [4782] = {.entry = {.count = 1, .reusable = true}}, SHIFT(386),
  [4784] = {.entry = {.count = 1, .reusable = true}}, SHIFT(503),
  [4786] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2201),
  [4788] = {.entry = {.count = 1, .reusable = true}}, SHIFT(506),
  [4790] = {.entry = {.count = 1, .reusable = true}}, SHIFT(507),
  [4792] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2210),
  [4794] = {.entry = {.count = 1, .reusable = true}}, SHIFT(509),
  [4796] = {.entry = {.count = 1, .reusable = true}}, SHIFT(510),
  [4798] = {.entry = {.count = 1, .reusable = true}}, SHIFT(511),
  [4800] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1582),
  [4802] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1251),
  [4804] = {.entry = {.count = 1, .reusable = true}}, SHIFT(222),
  [4806] = {.entry = {.count = 1, .reusable = true}}, SHIFT(966),
  [4808] = {.entry = {.count = 1, .reusable = true}}, SHIFT(223),
  [4810] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_emits, 7, 0, 185),
  [4812] = {.entry = {.count = 1, .reusable = true}}, SHIFT(783),
  [4814] = {.entry = {.count = 1, .reusable = true}}, SHIFT(154),
  [4816] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1442),
  [4818] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1461),
  [4820] = {.entry = {.count = 1, .reusable = true}}, SHIFT(156),
  [4822] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1479),
  [4824] = {.entry = {.count = 1, .reusable = true}}, SHIFT(362),
  [4826] = {.entry = {.count = 1, .reusable = true}}, SHIFT(951),
  [4828] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2461),
  [4830] = {.entry = {.count = 1, .reusable = true}}, SHIFT(365),
  [4832] = {.entry = {.count = 1, .reusable = true}}, SHIFT(158),
  [4834] = {.entry = {.count = 1, .reusable = true}}, SHIFT(367),
  [4836] = {.entry = {.count = 1, .reusable = true}}, SHIFT(159),
  [4838] = {.entry = {.count = 1, .reusable = true}}, SHIFT(160),
  [4840] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1574),
  [4842] = {.entry = {.count = 1, .reusable = true}}, SHIFT(369),
  [4844] = {.entry = {.count = 1, .reusable = true}}, SHIFT(370),
  [4846] = {.entry = {.count = 1, .reusable = true}}, SHIFT(176),
  [4848] = {.entry = {.count = 1, .reusable = true}}, SHIFT(371),
  [4850] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2293),
  [4852] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2461),
  [4854] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_expr, 4, 0, 116),
  [4856] = {.entry = {.count = 1, .reusable = true}}, SHIFT(290),
  [4858] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1483),
  [4860] = {.entry = {.count = 1, .reusable = true}}, SHIFT(269),
  [4862] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1051),
  [4864] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1507),
  [4866] = {.entry = {.count = 1, .reusable = true}}, SHIFT(239),
  [4868] = {.entry = {.count = 1, .reusable = true}}, SHIFT(291),
  [4870] = {.entry = {.count = 1, .reusable = true}}, SHIFT(224),
  [4872] = {.entry = {.count = 1, .reusable = true}}, SHIFT(955),
  [4874] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__address_specifier, 1, 0, 0),
  [4876] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_comment, 2, 0, 0),
  [4878] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_comment, 3, 0, 0),
  [4880] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_line_comment, 3, 0, 0),
  [4882] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_comment, 4, 0, 4),
  [4884] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_line_comment, 2, 0, 0),
};

enum ts_external_scanner_symbol_identifiers {
  ts_external_token__block_doc_comment_marker = 0,
  ts_external_token__block_comment_content = 1,
  ts_external_token__doc_line_comment = 2,
  ts_external_token__error_sentinel = 3,
};

static const TSSymbol ts_external_scanner_symbol_map[EXTERNAL_TOKEN_COUNT] = {
  [ts_external_token__block_doc_comment_marker] = sym__block_doc_comment_marker,
  [ts_external_token__block_comment_content] = sym__block_comment_content,
  [ts_external_token__doc_line_comment] = sym__doc_line_comment,
  [ts_external_token__error_sentinel] = sym__error_sentinel,
};

static const bool ts_external_scanner_states[5][EXTERNAL_TOKEN_COUNT] = {
  [1] = {
    [ts_external_token__block_doc_comment_marker] = true,
    [ts_external_token__block_comment_content] = true,
    [ts_external_token__doc_line_comment] = true,
    [ts_external_token__error_sentinel] = true,
  },
  [2] = {
    [ts_external_token__block_doc_comment_marker] = true,
    [ts_external_token__block_comment_content] = true,
  },
  [3] = {
    [ts_external_token__block_comment_content] = true,
  },
  [4] = {
    [ts_external_token__doc_line_comment] = true,
  },
};

#ifdef __cplusplus
extern "C" {
#endif
void *tree_sitter_move_on_aptos_external_scanner_create(void);
void tree_sitter_move_on_aptos_external_scanner_destroy(void *);
bool tree_sitter_move_on_aptos_external_scanner_scan(void *, TSLexer *, const bool *);
unsigned tree_sitter_move_on_aptos_external_scanner_serialize(void *, char *);
void tree_sitter_move_on_aptos_external_scanner_deserialize(void *, const char *, unsigned);

#ifdef TREE_SITTER_HIDE_SYMBOLS
#define TS_PUBLIC
#elif defined(_WIN32)
#define TS_PUBLIC __declspec(dllexport)
#else
#define TS_PUBLIC __attribute__((visibility("default")))
#endif

TS_PUBLIC const TSLanguage *tree_sitter_move_on_aptos(void) {
  static const TSLanguage language = {
    .abi_version = LANGUAGE_VERSION,
    .symbol_count = SYMBOL_COUNT,
    .alias_count = ALIAS_COUNT,
    .token_count = TOKEN_COUNT,
    .external_token_count = EXTERNAL_TOKEN_COUNT,
    .state_count = STATE_COUNT,
    .large_state_count = LARGE_STATE_COUNT,
    .production_id_count = PRODUCTION_ID_COUNT,
    .field_count = FIELD_COUNT,
    .max_alias_sequence_length = MAX_ALIAS_SEQUENCE_LENGTH,
    .parse_table = &ts_parse_table[0][0],
    .small_parse_table = ts_small_parse_table,
    .small_parse_table_map = ts_small_parse_table_map,
    .parse_actions = ts_parse_actions,
    .symbol_names = ts_symbol_names,
    .field_names = ts_field_names,
    .field_map_slices = ts_field_map_slices,
    .field_map_entries = ts_field_map_entries,
    .symbol_metadata = ts_symbol_metadata,
    .public_symbol_map = ts_symbol_map,
    .alias_map = ts_non_terminal_alias_map,
    .alias_sequences = &ts_alias_sequences[0][0],
    .lex_modes = (const void*)ts_lex_modes,
    .lex_fn = ts_lex,
    .keyword_lex_fn = ts_lex_keywords,
    .keyword_capture_token = sym_identifier,
    .external_scanner = {
      &ts_external_scanner_states[0][0],
      ts_external_scanner_symbol_map,
      tree_sitter_move_on_aptos_external_scanner_create,
      tree_sitter_move_on_aptos_external_scanner_destroy,
      tree_sitter_move_on_aptos_external_scanner_scan,
      tree_sitter_move_on_aptos_external_scanner_serialize,
      tree_sitter_move_on_aptos_external_scanner_deserialize,
    },
    .primary_state_ids = ts_primary_state_ids,
  };
  return &language;
}
#ifdef __cplusplus
}
#endif
